#!/usr/bin/perl

use Getopt::Std;
use Cwd;
use File::Basename;

getopt ('mfo');

my $usage="\n\nThis script is a wrapper for the currently complex interaction between CPN-AMI and spot-greatspn.\n"
  ."(c) 2006 Yann Thierry-Mieg, Laboratoire d'Informatique de Paris 6 \n"
  ."Usage:\n"
  ."SnowSpotGreatSPN -m [input_model = xxx.cami] -f [input_formula = xxx.prop]\n"
  ."Options : [-o output_file = default model_dir/out.txt] \n";
my $model = $opt_m or  die $usage;
my $formula = $opt_f or die $usage;


my $cwd =  getcwd;
my $workdir = $cwd."/".dirname($model);
my $modelff = basename($model);

my $output = $workdir."/out.txt";
if ($opt_o) { $output = $cwd."/".$opt_o ; }

use strict 'vars';   # After obtaining $opt_xxx, which is not a local variable

################ ZONE DE CONFIGURATION ##################
#print STDERR $0."n";
my $tooldir = dirname($0);
my $trans2prop = $tooldir."/trans2prop.pl";
my $snow_exe = $tooldir."/snow";
my $filterCami = $tooldir."/filterCamiTobs.pl";
my $trace2cami = $tooldir."/trace2cami.pl";
my $findap = $tooldir."/findAP_sources/findAP";
my $ltlspot = $tooldir."/findAP_sources/ltlgspn-srg";
my $splitCami =  $tooldir."/splitCamiEvents.pl";
#########################################################



# check existence of everybody
if (! -r $model) {
  die "ERROR : model file $model not found, please check existence and permissions. \n $usage";
} elsif (! -r $formula) {
  die "ERROR : formula file $formula not found, please check existence and permissions. \n $usage";
} elsif (!-w $workdir) {
  die "ERROR : work dir containing model file should have write access to store intermediate files.  \n $usage";
} elsif (!-x $trans2prop) {
  die "ERROR : $trans2prop script not found or permissions not executable. \n $usage";
} elsif (!-x $snow_exe ) {
  die "Could not execute snow tool : $snow_exe, check that it is on the path. \n $usage";
} elsif (!-x $filterCami) {
  die "Could not execute  $filterCami tool, check that it is on the path. \n $usage";
} elsif (!-x $findap) {
  die "Could not execute  $findap tool, check that it is on the path. \n $usage";
} elsif (!-x $ltlspot) {
  die "Could not execute  $ltlspot tool, check that it is on the path. \n $usage";
} elsif (!-x $trace2cami) {
  die "Could not execute  $trace2cami tool, check that it is on the path. \n $usage";
} elsif (!-x $splitCami) {
  die "Could not execute  $splitCami tool, check that it is on the path. \n $usage";
} 
# etc...


######### Pre treatments : load formula and atomic prop identifiers from files

#get the formula string
open (IN,$formula) or die "couldn't open formula file : $formula\nPlease check permissions and existence.";
my $formstring = <IN>;
chomp $formstring;
my $origformstring = $formstring;
close IN;

my $findDeadlock = 0 ;
$findDeadlock = 1 if  ($formstring =~ /^\s*G\s*!\s*DEAD\s*/i );
#print STDERR "find deadlock $findDeadlock \n";

### Support for guards on events
print "formula : $formstring \n";
system("\\cp $model $model.split");
while ( $formstring =~ s/(\w+)\s*\[([^\]]+)\]/$1/x ) {
    print "found event $1 with guard $2 \n";
    my $event = $1;
    my $guard = $2;
    my $callstr = "$splitCami -m $model.split -t $event -g \"$guard\" -o $model.split";
#    print "$callstr\n";
    system($callstr);
#    my $event_name = $1."_".$2;
#    $event_name =~ s/\s+$//;
#    $event_name =~ s/\s+/\s/;
#    $event_name =~ s/[^\w]+/\_/g;
#    print $event_name."\n";
}

#print "formula : $formstring \n";

my $apstring;
if ($findDeadlock == 0) {
#get the filter string
    system ("$findap \"$formstring\" >$output 2>$output.err");
    open (AP,$output);
    $apstring = <AP>;
    chomp $apstring;
    close(AP);
    if ($apstring =~ /^$/) {
	print STDERR "parse error in formula or empty AP list"; 
	system("\\cp $output.err $output");
	goto TRACE2CAMI;
    } 
} else {
    $apstring = "";
}
######### Ready to work

# set current dir (lots of temp are generated)
chdir $workdir;

#filter AP from cami file
if ($findDeadlock == 0) {
    system("$filterCami -m $modelff.split -f \"$apstring\" -o $modelff.filt") == 0
	or die "Cami filter $filterCami script failed, check stderr.";
} else {
    system("$filterCami -m $modelff.split -o $modelff.filt") == 0
	or die "Cami filter $filterCami script failed, check stderr.";
}
#run snow
print "$snow_exe -m $modelff.filt -d ./";
system("$snow_exe -m $modelff.filt -d ./") == 0
  or die "Snow exe : $snow_exe failed with an error, check stderr";

#should have everybody now, split tobs
system("$trans2prop ./model") ==0
  or die "Trans2prop script : $trans2prop failed. Check stderr";

#run spot
if (-r $output) {
    unlink $output;
}
$apstring =~ s/,/ /g;

my $call;
if ($findDeadlock == 0) {
    $call = "$ltlspot -e -c -f -P  model \"! $formstring\" $apstring > $output.spot 2> $output.spot";
} else {
    $call = "$ltlspot -e -c -f -d dead -P  model \"! $formstring\" dead > $output.spot 2> $output.spot";
}
# print STDERR $call;
system($call);

#if (! -r $output) {
#  die  "$ltlspot failed at execution. Command run was \n $call\n check stderr";
#}

#run trace to cami
TRACE2CAMI:
$call = "$trace2cami -m $modelff -g model.def -t $output.spot -o $output -f  \"$origformstring\"";
system($call) == 0
  or die "Trace to cami script : $trace2cami failed. Check stderr";


#$call = "\\cp  $output.cami $output";
#system($call);





 
