#!/usr/bin/perl -w


use Getopt::Std;

getopt ('mtgo');

my $usage="This script splits transiitons from a cami file, to allow observation of fixed color instances. \n (c) 2006 Yann Thierry-Mieg, Laboratoire d'Informatique de Paris 6 \n Usage:\n filterCamiTobs.pl -m [input_model = xxx.cami] -t [transition name] -g [observation guard] -o [output model file] \n";
my $model = $opt_m or  die "no option -m provided.\n".$usage;
my $tname = $opt_t or  die "no option -t provided.\n".$usage;
my $obsguard = $opt_g or  die "no option -g provided.\n".$usage;
my $outff = $opt_o or  die "no option -o provided.\n".$usage;

use strict 'vars';


# check existence of everybody
if (! -f $model) {
  die "ERROR : model file $model not found, please check existence and permissions. \n $usage";
}



open(IN,"< $model");

# one shortcircuit pass to find macao object number of transition
my $tnum; # t macao id
my $lastMac = 0; # last macao id
while (my $line = <IN>) {
  if ( $line =~ /^CT\(4:name,(\d+),\d+:\s*$tname\s*\)/ ) {
    # got it
    $tnum = $1;
  } elsif ( $line =~ /^CN\(\d+:.+,(\d+)\)\n$^/ ) {
    $lastMac = $1 if ($1 > $lastMac);
  } elsif ( $line =~ /^CA\(\d+:.+,(\d+),\d+,\d+\)\n$/ ) {
    $lastMac = $1 if ($1 > $lastMac);
  }
}  
unless (defined $tnum) {
    die "No transition named : $tname found in model $model \n";
}
close IN;

# output and transform


my @transitionLines;
open(IN,"< $model");
open(OUT,"> $outff.tmp") or die "Couldn't create output file $outff.tmp";

#one pass : substitute guard for t1 by g and ! tobsg
# store all lines that reference transition t1
# update macaonum
# add new transition g and tobsg

# the macao id for our new transition
my $newtnum = ++$lastMac;
# objects that touch the transition
my %todup;
$todup{$tnum} = $newtnum;
my @translines;
while (my $line = <IN>) {
  if ( $line =~ /^(CN\(\d+:[^,]+,)$tnum(\)\n)$/ ) {
    # duplicate create node command
    print OUT $1.$newtnum.")\n";

  } elsif ( $line =~ /^CT\(5:guard,$tnum,\d+:(.*)\)\n$/ ) {
    # find current guard of t1 and update
    my $g = $1;
    my $gtrue = $1." and (".$obsguard.")";
    my $gfalse = $1." and not(".$obsguard.")";
    print OUT "CT(5:guard,$tnum,".length($gtrue).":$gtrue)\n";
    print OUT "CT(5:guard,$newtnum,".length($gfalse).":$gfalse)\n";
    # dont print line (default behavior
    next;

  } elsif ( $line =~ /^CT\(4:name,$tnum,\d+:(.+)\)\n/ ){
    # update name for newt1
    my $newtname = $1."_falseobs";
    print OUT "CT(4:name,$newtnum,".length($newtname).":$newtname)\n";

  } elsif ( $line =~ /^(CA\(\d+:.+),(\d+),(\d+),(\d+)\)\n$/ ) {
    # duplicate connected arcs
    # captures : $1: CA(3:arc  $2: 31 (macaobj)   19 (src)  14 (dest)
    if ($4 == $tnum ) {
      $todup{$2} = ++$lastMac;
      print OUT "$1,".$todup{$2}.",$3,$newtnum)\n" ;
    } elsif ($3 == $tnum) {
      $todup{$2} = ++$lastMac;
      print OUT "$1,".$todup{$2}.",$newtnum,$4)\n" ;
    }
  } elsif ( $line =~ /^(CT\(\d+:[^,]+,)(\d+)(,\d+:.*\n)$/ ) {
    # duplicate tags on concerned objects
    if (exists $todup{$2}) {
      print OUT "$1$todup{$2}$3";
    }
  }
  # make a copy of every line
  print OUT $line;
}


close IN;
close OUT;

system("\\mv $outff.tmp $outff");

exit(0);

#system("\\mv $model.tmp $model");
