#!/usr/bin/perl 

use Getopt::Std;

getopt ('mtogf');
my $usage="\n\nThis script transforms illegible greatSPN output into nice macao traces.\n"
  ."(c) 2006 Yann Thierry-Mieg, Laboratoire d'Informatique de Paris 6 \n"
  ."Usage:\n"
  ."trace2cami -m [input_model = xxx.cami] -g [input gspn model.def file] -t [input_trace = xxx.out] -f [formula string]\n"
  ."Options : [-o output_file = default trace.cami]";
my $macaomodelff = $opt_m or  die $usage;
my $traceff = $opt_t or die $usage;
my $defff = $opt_g or die $usage;
my $formula = $opt_f or die $usage;
my $output = "trace.cami";
if ($opt_o) { $output = $opt_o ; }

use strict 'vars';




# load greatspn class definitions
my %elements;
my %statsubs;
open (IN, $defff) or die "could not open greatspn model file $defff to get subclass definitions. $usage" ;
while (my $line = <IN>) {
  if ( $line =~ /^\((.+) c .*\@c\n$/ ) {
    #    print "matched : $line ";
      my $cname = $1;
    $line = <IN>;
    chomp $line;
    unless ($line =~ /^[uo] /) {
      # not a static sub
      if ($line =~ /num\{(\d+)\-(\d+)\}/ ) {
	# integer range domains
	my @elts;
	foreach ($1..$2) {
	  push @elts , $_;
	}
	$elements{$cname}= \@elts ;
      } elsif ($line =~ /(\w+)\{(\d+)\-(\d+)\}/ ) {
	# prefixed named enumeration
	my @elts;
	foreach ($2..$3) {
	  push  @elts, $_.$1 ;
	}
	$elements{$cname}= \@elts ;	
      } else {
	my @elts = split (/[\{,\}]/,$line) ;
	shift @elts ; # = @elts[0..$#elts-1];
	$elements{$cname}= \@elts ;
      }
    } else {
	$line =~ s/^[uo]\s+// ;
	my @elts = split (/\s*,\s*/,$line) ;
	$statsubs{$cname}= \@elts ;
    }
  }
}
#while( my ($k, $v) = each %statsubs ) {
#  my $txt = "$k : ";
#  foreach (@$v) {
#    $txt .= $_;
#    my $r_arr = $elements{$_};
#    $txt .= "{";
#    $txt .= join(",",@$r_arr[0..@$r_arr-1]);
#    $txt .= "} ";
#  }
#  print STDERR "$txt\n";	  
#}
close IN;
######### end greatspn classes
#exit(0);

open (IN, "$traceff");
my $line = <IN> ;
chomp $line;
if ($line =~ /Prefix/) {
  print "\nCounter example found !\n";
} elsif ($line =~ /empty/) {
  open (OUT,"> $output");
  print OUT "DE(10:statistics,1)\n";
  while ($line = <IN>) {
    chomp $line;
    print OUT "RT(".length($line).":".$line.")\n" unless ( length($line) == 0);
  }
  print OUT "FE()\n";
  my $txt = "Property \"$formula\" is true !";
  print OUT "DE(".length($txt).":".$txt.",1)\n";
  $txt = "Graph built under equivalence relation :";
  print OUT "RT(".length($txt).":".$txt.")\n";
  
  while( my ($k, $v) = each %statsubs ) {
      $txt = "$k : ";
      foreach (@$v) {
	  my $r_arr = $elements{$_};
	  $txt .= "{";
	  $txt .= join(",",@$r_arr[0..@$r_arr-1]);
	  $txt .= "} ";
      }
	  print OUT "RT(".length($txt).":".$txt.")\n";	  
  }

  $txt = "no counter examples exist.";
  print OUT "RT(".length($txt).":".$txt.")\n" unless ( length($txt) == 0);
  print OUT "ME(1)\n";
  print OUT "FE()\n";
  close(OUT);
  exit (0);
} else {
    ############# ERRORS IN TOOL CHAIN 
    open (OUT,"> $output");
    print OUT "DE()\n";
    # Something went wrong with ltlgspn-srg
    if ($line =~ /prop\_index\((.+)\)/) {
	# an AP name was not found
	my $err = "ERROR : Atomic property $1 is not known. Check your LTL formula.";
	print OUT "RT(".length($err).":".$err.")\n";
	$err = "It should be the name of an observation transition (ie. for transition PROP_myap, use G(myap) in the formula for a state based property) or else the name of a transition of the net (for event based property).";
	print OUT "RT(".length($err).":".$err.")\n";
    } elsif ($line =~/^>>>/) {
	# probably a parse error of the formula
	my $err = "ERROR : Parse error while reading LTL formula.";
	print OUT "RT(".length($err).":".$err.")\n";
	print OUT "RT(".length($line).":".$line.")\n" unless ( length($line) == 0);
	while ($line = <IN>) {
	    chomp $line;
	    print OUT "RT(".length($line).":".$line.")\n" unless ( length($line) == 0);
	}
    } else {
	# another error ??
	my $err = "ERROR : An uncaught exception has occured. Please mail Yann.Thierry-Mieg\@lip6.fr with a report on this error (model+formula)";
	print OUT "RT(".length($err).":".$err.")\n";
	print OUT "RT(".length($line).":".$line.")\n" unless ( length($line) == 0);
	while ($line = <IN>) {
	    chomp $line;
	    print OUT "RT(".length($line).":".$line.")\n" unless ( length($line) == 0);
	}
    }
    print OUT "FE()\n";
    close(OUT);
    exit (0);
}
close(IN);


#if we get here, we have a counter example

# load cami object numbers
open (IN,$macaomodelff) or die "could not open macao model file $macaomodelff to get object numbers. $usage" ;
my %macaonum;
while (my $line = <IN>) {
  if ( $line =~ /^CT\(4:name,(\d+),\d+:(.*)\)$/ ) {
    $macaonum{$2}=$1;
  }
}
close IN;

open (OUT,"> $output");
# header of trace
#print OUT "DR()\n";
#print OUT "RQ(7:AMI-Net,9:LTL trace,1)\n";
#print OUT "TQ(7:AMI-Net,9:LTL trace,2,37:Launching service \"SnowSpotGreatSPN\".)\n";
print OUT "TD(7:AMI-Net)\n";
print OUT "OB(1,1,5:place)\n";
print OUT "AT(6:weight,2,1,2,1:0)\n";
print OUT "FA()\n";

open (IN,$traceff) or die "could not open trace file $traceff.$usage";
my $isprefix = 0;
my $iscycle = 0;

while (my $line1 = <IN>) {
  if ($line1 =~ /Prefix/) {
      #Preifx line
      $isprefix = 1;
  } elsif ($line1 =~ /Cycle/) { 
      # prefix and cycle lines
      $iscycle = 1;
#    print OUT "DE(5:State,1)\n";
#    chomp $line1;
#    print OUT "RT(".length($line1).":".$line1.")\n";
#    print OUT "FE()\n";
  } elsif ($line1 =~ /unique states/ ) {
    # end of spot output
    chomp $line1;
    print OUT "RT(".length($line1).":".$line1.")\n";
    while ($line1 = <IN>) {
      chomp $line1;
      print OUT "RT(".length($line1).":".$line1.")\n" unless ( length($line1) == 0);
    }
    print OUT "FE()\n";
      my $txt = "Property \"$formula\" is false !"; 
    print OUT "DE(".length($txt).":".$txt.",1)\n";
    $txt = "Graph built under equivalence relation :";
    print OUT "RT(".length($txt).":".$txt.")\n";
    $txt = "";
      
      while( my ($k, $v) = each %statsubs ) {
	  $txt = "$k : ";
	  foreach (@$v) {
	      my $r_arr = $elements{$_};
	      $txt .= "{";
	      $txt .= join(",",@$r_arr[0..@$r_arr-1]);
	      $txt .= "} ";
	  }
	  print OUT "RT(".length($txt).":".$txt.")\n";	  
      }
      
    $txt = "Elements in these sets may be arbitrarily permuted in counter example traces.";
    print OUT "RT(".length($txt).":".$txt.")\n";
    print OUT "FE()\n";
    last;
  } else {
    #main case : synchronized product state description
    # $line1 : state description
    my ($line2,$line3);
    unless ($line2 = <IN>) {last;} # dyn subs description
    unless ($line3 = <IN>) {last;} # LTL automata state
    chomp $line1;
    chomp $line2;
    chomp $line3;
    # check whether cycle is useful : if synchronizing with true, it's over
    if ($line2 =~ /\* 1$/ ) {
	next ;
    }

    if ($iscycle == 1) {
	my $txt = "Cycle start state";
	print OUT "DE(".length($txt).":".$txt.",1)\n";
	$iscycle = 0;
    } elsif ($isprefix == 1) {
	my $txt = "Prefix start state";
	print OUT "DE(".length($txt).":".$txt.",1)\n";
	$isprefix = 0;
    } else {
	print OUT "DE(5:State,1)\n";
    }
   # output dyn subs cardinality to popup tag
    my %text =() ;
    my %cons =();
    # construct correspondence table for dyn subs
    while ($line2 =~ /\|(\w+)\|=(\d+) /g ) {
      my $dynname = $1;
      my $dyncard = $2;
#      print "matched class : $dynname card : $dyncard \n";
      my $contclass = $dynname;
      #TODO : separate digits correctly
      $contclass =~ s/\d+$//;
      #chop $contclass;
      my $con = 0;
      $con = $cons{$contclass}  if (defined $cons{$contclass}) ;
      my $r_arr = $elements{$contclass};
#      print "con $con $dyncard, cont class : $contclass $r_arr @$r_arr \n";
      if ($dyncard == 1) {
	$text{$dynname} = "@$r_arr[$con]";
      } else {
	$text{$dynname} = join("+",  @$r_arr[$con..$con+$dyncard-1] );
      }
      $cons{$contclass} = $con+$dyncard;
    }
    while(my ($k, $v) = each %text ) {
#        print "key: $k, value: $v.\n";
	$line2 =~ s/$k/$v/g ;
	$line1 =~ s/$k/$v/g ;
    }
    $line2 =~ s/(?:.+) \* (.+)/$1/ ;
    $line2 = "Automaton state : ".$line2;
    print OUT "RT(".length($line2).":".$line2.")\n";
    $line3 =~ s/^\s*\|\s*// ;
    $line3 =~ s/\s+\{.+$// ;
    $line3 = "Atomic props :".$line3;
    print OUT "RT(".length($line3).":".$line3.")\n";

    #analyze line 1 :  C_Start(1<Client_01>1<Client_14>)S_Start(1<Server_00>1<Server_11>1<Server_22>)C_Comp(1<Client_00>)Send(1<Client_00>)
    $line1 =~ s/^\s+//;
    my @words = split (/[\(\)]/,$line1);
    # words : [C_Start ; 1<Client_01>1<Client_14> ; S_Start ; ...etc
    for (my $i = 0 ; $i < $#words ; $i+=2) {
#      print "place : $words[$i]; num : $macaonum{$words[$i]} \n";
      my $macnum = $macaonum{$words[$i]};
      my $markstring;
      while ( $words[$i+1] =~ /(\d+)(\<[^\>]+\>)/xg ) {
	if ($1 == 1) { $markstring .= " + $2";}
	else { $markstring .= " + $1*$2"; }
      }
      $markstring =~ s/^ \+ // ;
      unless ($macnum == 0) {
	print OUT "ME($macnum)\n";
	print OUT "XA($macnum,6:weight,".length($markstring).":".$markstring.")\n";
      } else {
	my $str = $words[$i]." ".$words[$i+1];
	print OUT "RT(".length($str).":".$str.")\n";
      }
    }
    print OUT "FE()\n";
  }
}
#print OUT "FR(1)\n";

close IN;
close OUT;

exit (0);
