#!/usr/bin/perl

use Getopt::Std;
use Cwd;
use File::Basename;

getopt ('mfo');

my $usage="\n\nThis script is a wrapper for the currently complex interaction between CPN-AMI and mcsog.\n"
  ."(c) 2006-2008 Kais Klai, Denis Poitrenaud, Yann Thierry-Mieg, LIPN & LIP6 \n"
  ."Usage:\n"
  ."mcsog.pl -m [input_model = xxx.cami] -f [input_formula = xxx.prop]\n"
  ."Options : [-o output_file = default model_dir/out.txt] \n";
my $model = $opt_m or  die $usage;
my $formula = $opt_f or die $usage;

my $cwd =  getcwd;
my $workdir = $cwd."/".dirname($model);
my $modelff = basename($model);

my $output = $workdir."/out.txt";
if ($opt_o) { $output = $cwd."/".$opt_o ; }

use strict 'vars';   # After obtaining $opt_xxx, which is not a local variable

################ ZONE DE CONFIGURATION ##################
#print STDERR $0."n";
my $tooldir = dirname($0);
my $mcsog_exe = $tooldir."/check-sog";
my $trace2cami = $tooldir."/trace2cami.pl";
my $cami2prod = $tooldir."/cami2prod";
my $algoff =  $tooldir."/algo.mcsog";
#########################################################



# check existence of everybody
if (! -r $formula) {
  die "ERROR : formula file $formula not found, please check existence and permissions. \n $usage";
} elsif (!-w $workdir) {
  die "ERROR : work dir containing model file should have write access to store intermediate files.  \n $usage";
} elsif (!-x $mcsog_exe ) {
  die "Could not execute snow tool : $mcsog_exe, check that it is on the path. \n $usage";
} elsif (!-x $trace2cami) {
  die "Could not execute  $trace2cami tool, check that it is on the path. \n $usage";
} elsif (!-x $cami2prod) {
  die "Could not execute  $cami2prod tool, check that it is on the path. \n $usage";
} elsif (!-r $algoff) {
  die "ERROR : algorithm selection file $algoff not found, please check existence and permissions. \n $usage";
} 
# etc...


######### Pre treatments : load formula and atomic prop identifiers from files

#get the formula string
open (IN,$formula) or die "couldn't open formula file : $formula\nPlease check permissions and existence.";
my $formstring = <IN>;
chomp $formstring;
my $origformstring = $formstring;
close IN;

my $findDeadlock = 0 ;
#print STDERR "find deadlock $findDeadlock \n";

print "formula : $formstring \n";

my $apstring = "";
######### Ready to work

# set current dir (lots of temp are generated)
chdir $workdir;
my $call;

# produce correct prod format file
my $prod_model = $modelff.".net";


$call = "$cami2prod -m $modelff";
print STDERR $call;

system($call);

if (! -r $prod_model) {
  die "ERROR : PROD model file generation failed. \n $usage";
}

#run spot
if (-r $output) {
    unlink $output;
}


$call = "$mcsog_exe -r -P $prod_model -A $algoff -i $formula $apstring > $output.spot 2> $output.spot";

print STDERR $call;
system($call);

#if (! -r $output) {
#  die  "$ltlspot failed at execution. Command run was \n $call\n check stderr";
#}

#run trace to cami
#$call = "$trace2cami -m $modelff -g model.def -t $output.spot -o $output -f  \"$origformstring\"";
$call = "$trace2cami -m $modelff -t $output.spot -o $output";
print STDERR $call;
system($call) == 0
  or die "Trace to cami script : $trace2cami failed. Check stderr";


#$call = "\\cp  $output.cami $output";
#system($call);





 
