#!/usr/bin/perl

use Getopt::Std;
use Cwd;
use File::Basename;

getopt ('mo');

my $usage="\n\nThis script is a wrapper for the currently complex interaction between CPN-AMI and modsog.\n"
  ."(c) 2006-2008 Laure Petrucci, Kais Klai integrated with Yann Thierry-Mieg  LIPN & LIP6 \n"
  ."Usage:\n"
  ."modsog.pl -m [input_file = xxx]\n"
  ."Options : [-o output_file = default model_dir/out.txt] \n";
my $model = $opt_m or die $usage;

my $cwd =  getcwd;
my $workdir = $cwd."/".dirname($model);
my $modelff = basename($model);

my $output = $workdir."/out.txt";
if ($opt_o) { $output = $cwd."/".$opt_o ; }

use strict 'vars';   # After obtaining $opt_xxx, which is not a local variable

################ ZONE DE CONFIGURATION ##################
#print STDERR $0."n";
my $tooldir = dirname($0);
my $commandff =  $tooldir."/commands";
my $modsog_exe = $tooldir."/ObsGraph";
my $trace2cami = $tooldir."/trace2cami.pl";
my $cami2prod = $tooldir."/cami2prod";
my $formulaff = $workdir."/modsog-formula";
#########################################################

# check existence of everybody
if (! -r $model) {
  die "ERROR : model configuration file $model not found, please check existence and permissions. \n $usage";
} elsif (!-w $workdir) {
  die "ERROR : work dir containing model file should have write access to store intermediate files.  \n $usage";
} elsif (!-x $modsog_exe ) {
  die "Could not execute ObsGraph tool : $modsog_exe, check that it is on the path. \n $usage";
} elsif (!-x $trace2cami) {
  die "Could not execute  $trace2cami tool, check that it is on the path. \n $usage";
} elsif (!-x $cami2prod) {
  die "Could not execute  $cami2prod tool, check that it is on the path. \n $usage";
}  elsif (!-r $commandff) {
  die "Could not find the command file $commandff supposed to contain : 3\nn\n0\n";
} 
# etc...

my $workspacedir = $workdir. "/../..";

########### pre treatments : parse the config file
my @subnets = ();

# parser states : 
# 0 => default intial state, looking for subnet decl or interface/formula keyword
# 1 => in an interface declaration
# 2 => in a formula declaration
my $parserstate = 0;

# the transitions of a transition list (interface or formula)
my @transitions = ();
# the last interface declaration block refers to this net
my $subnetref;

#print "reached parse\n workdir : $workdir , tooldir = $tooldir\n";

open (IN,$model) or die "couldn't open model config file : $model\nPlease check permissions and existence.";
while (my $line=<IN>) {
  chomp $line;
  if ($parserstate == 0) {
    if ( $line =~ /^---\s*interface\s*(.+)\s*\{\s*$/ ) {
      $subnetref = $1;
      $subnetref =~ s/\s//g;
      # check that the interface name matches a subnet name
      my $found = 0;
      foreach my $item (@subnets) {
	if ($subnetref =~ /$item/) {
	  $found = 1;
	  last;
	}
      }
      if ($found == 0) {
	die "The interface $subnetref is not a subnet defined in preamble part";
      }

      @transitions = ();
      $parserstate = 1;
    } elsif ( $line =~ /^---\s*formula\s*\{\s*$/ ) {
      @transitions = ();
      $parserstate = 2;
    } elsif ( $line =~ /^\s*(.+)\s*$/) {
#      print "## a priori this is a subnet declaration for $1 \n ";
      my $subnet = $1;
      $subnet =~ s/\s//g;
      push @subnets,$subnet;

    } else {
      print "Attempting to recover from parse error on line : $line \n." unless $line =~ /^\s*$/;
    }
  } else {
    # parser state 1 or 2 => looking for a list of transitions
    if ( $line =~ /^\s*\}\s*$/ ) {
      if ($parserstate == 1) {
	# an interface decl
#	print "parsed interface $subnetref: $#transitions  @transitions\n\n";
	open (OUT,"> $workdir/int$subnetref");
      } else {
	# a formula decl
#	print "parsed formula: @transitions \n\n";
	open (OUT,"> $formulaff");
      }
      print OUT ($#transitions+1)." \n";
      print OUT (join ("\t",@transitions));	
      print OUT "\n";
      close OUT;

      $parserstate = 0;
      next;
    }
    my @words = split /\s+/,$line;
    foreach my $item (@words) {
      push @transitions,$item;
    }
#    push @transitions,@words;
  }
}



######### Ready to work

# set current dir (lots of temp are generated)
chdir $workdir;
my $call;





# produce correct prod format file
my $prod_model = $modelff.".net";


for my $subnet (@subnets) {
  my $subnetff = $workspacedir."/$subnet/FK_MNGR/.model";
  if (! -r $subnetff) {
    die "Could not find model part $subnet. Was searching in $subnetff. Did you connect this model and run syntax check ?";
  }
  my $prod_model =  $subnet.".net";
  $call = "$cami2prod -m $subnetff -o $prod_model";
  print STDERR $call."\n";

  system($call);
  
  if (! -r $prod_model) {
    die "ERROR : PROD model file generation failed. \n $usage";
  }
}

# ready to call modsog tool : we have interface, formula and .net files ready

# Example :
# ../../ObsGraph BD10_1-2.net BD10_int1-2 BD10_3-4.net BD10_int3-4 BD10_5-6.net BD10_int5-6 BD10_7-8.net BD10_int7-8 BD10_9-10.net BD10_int9-10 formula_trans < command

$call = $modsog_exe ;
for my $item (@subnets) {
  if (! -r "int$item") {
    die "Input error : no interface definition provided for subnet $item. Please correct your input.\n";
  }
  $call .= " $item.net int$item ";
}
# add formula
my $directoutput = $output."modsog";
$call .= "$formulaff < $commandff > $directoutput";


print STDERR $call."\n";

system($call);

#run trace to cami
#$call = "$trace2cami -m $modelff -g model.def -t $output.spot -o $output -f  \"$origformstring\"";
$call = "$trace2cami -i $directoutput -o $output";
print STDERR $call;
system($call) == 0
  or die "Trace to cami script : $trace2cami failed. Check stderr";




