#! /usr/bin/env python
import sys
import re
import math
import commands
import copy

#################
#Read input file#
#################
f=open(sys.argv[1], 'r')

N=int(f.readline())
M=int(f.readline())
Sis=[[] for i in range(M)] # it is a column major matrix

matchLevel = re.compile('(\d+) :')
matchEvents = re.compile(' (\d+)')

#Read line by line
while 1 :
	line=f.readline()
	if not line: #End of file
		break
	level = matchLevel.match(line) # Find the number of the level
	events=matchEvents.findall(line)
	for event in events:
		Sis[int(event)-1].append(int(level.group(1))-1) 
f.close()

print Sis
###############################
#End of the input file reading#
###############################

#Write Sis, N and M in an FP.dat file
f=open("FP.dat", 'w')
f.write("param N := %i;\n"%N)
f.write("param M := %i;\n"%M)
for i in range(0,M):
	f.write("set S[%i] :=" % i)
	for j in Sis[i]:
		f.write(" %i" % j)
	
	f.write(";\n")		

f.write("end;\n")		
f.close()

#print "FP.dat file generated"
counter=M
index=0
for i in range(M):
	if len(Sis[index])<=1:
		del Sis[index]
	else:
		index+=1

M=index

#Run glpsol
commands.getoutput("glpk-4.9/glpsol --model FP.mod  --data FP.dat -o FP.out")

#########################
#Read the glpsol output:#
#########################
f=open("FP.out", 'r')
matchCost = re.compile('Objective:  cost = (\d*.\d*)')

#Read the cost
while 1:
	line = f.readline()
	if not line:
		break
	cost=matchCost.match(line)
	if cost:
		cost=float(cost.group(1))
		break
#Read the distances:
matchDf = re.compile(' *\d* df\[(\d*),(\d*)\] *\D* *(\d*.\d*)')

df=[[0.0 for i in range(N)] for i in range(N)]

while 1:
	line = f.readline()
	if not line:
		break
	dfm=matchDf.match(line)
	if dfm:
		df[int(dfm.group(1))][int(dfm.group(2))] = float(dfm.group(3))
f.close()

print df

#########################################################################################################################
# Rounding algorithm A:													#
#Rounding algorithm A takes a set of vertices H and a set of S_i and outputs the linear ordering of the vertices of H.	#
#We assume the set of distances is contained in 									#
#########################################################################################################################
def roundA(H,Sib):
#	print "Entrance:" + "H =" + str(H) +"Si = " +  str(Sib)

	#removing useless Si:
	Si=Sib
	for i in Si:
		for j in Sis[i]:
			contained = 0
			for k in H:
				if j==k:
					contained=1
			if contained==0:
				Si.remove(i)
				break
	
#if H contains 1 or less vertices, there is nothing to do, if there is no Si, it's the same
	
	if len(Si)<=1:  #XXX <= or < ?!
#		print "Exits:" + "sol ="  + str(H)
		return H
	if len(H)<=1:
#		print "Exits:" + "sol ="  + str(H)
		return H
#	print H
#	print Si
#Part 1: Select a vertex v such that sum(df(v,w)) is maximum

#	levels=[[] for i in range(len(H))] #XXX H may contain vertices with indices not from 0 to len(H)
	levels=[[] for i in range(0,N)] #XXX H may contain vertices with indices not from 0 to len(H)
	buckets=[[] for i in range(-1,int(math.ceil(math.log(len(Si)))))]
	v=0
	maxV=0
	
	for i in H:
		temp=0
		for j in H:
			temp+=df[i][j]
		if temp>maxV:
			v=i
			maxV=temp
#	print 'V:' +str(v) + ' sum =' + str(maxV)
	

#	for i in H:
#		temp=0
#		neighbours=[]
#		for j in Si:
#			for k in Sis[j]:
#				if not neighbours.__contains__(k):
#					neighbours.append(k)
#		for j in neighbours:	
#			temp+=df[i][j]
#		if temp>maxV:
#			v=i
#			maxV=temp


#	print 'V:' +str(v) + ' sum =' + str(maxV)

#Part 2: Assign S_i to levels : S_i is in level L(k) if k is contained in span(S_i)
	for i in Si: #for each Si, put it in the levels it belongs to
		minS=N+1
		maxS=0
		for j in Sis[i]:
			if df[v][j] < minS:
				minS=df[v][j]
			if df[v][j] > maxS:
				maxS=df[v][j]
		for j in range(int(math.ceil(minS)),int(math.floor(maxS))):
			levels[j].append(i)

	#Test if every level are empty:
	empty = 1
	for i in levels:
		if len(i)>0:
			empty = 0
	if empty:
		H_i=[[] for i in range(0,len(levels))]
		for i in range(0,len(levels)):
			for j in H:
				if (i<=df[v][j]) and (df[v][j]<i+1):
					H_i[i].append(j)
		
		for j in H:
			if len(levels)<=df[v][j]:
				H_i[len(levels)].append(j)
		sol=[]
		for i in H_i:
			sol+=roundA(i,Si)
#		print "Exits:" + "sol ="  + str(sol)
		return sol
		
		
		
	#print "Levels:" + str(levels)
#Part 3: regroup levels into buckets
	for i in range(-1, len(buckets)):
		for j in range(0, len(levels)):
		#	print len(levels[j])
			if (len(levels[j]) > math.pow(2,i)) and (len(levels[j])<=pow(2,i+1)):
				buckets[i].append(j)
	
	#print buckets
#Part 4: select the largest bucket and create the H_i
	maxbucket=0
	chosenbucket = 0
	for i in buckets:
		if len(i) > maxbucket:
			maxbucket = len(i)
			chosenbucket = i
	H_i=[[] for i in range(0,len(chosenbucket)+1)]
	
	
	chosenbucket.sort()
	for i in range(0,len(chosenbucket)-1):
		for j in H:
			if (chosenbucket[i]<=df[v][j]) and (df[v][j]<chosenbucket[i+1]):
				H_i[i+1].append(j)
	for j in H:
		if chosenbucket[0]>df[v][j]:
			H_i[0].append(j)
		if chosenbucket[len(chosenbucket)-1]<=df[v][j]:
			H_i[len(chosenbucket)].append(j)
	#print 'H_i'
	#print H_i
	
	#Create the new S_is:
	newSi=copy.deepcopy(Si)
	for i in chosenbucket:
		for j in levels[i]:
			if j in newSi:
				newSi.remove(j)
	#print 'Si:'
	#print newSi
	sol=[]
	for i in H_i:
		sol+=roundA(i,newSi)
	
	#print "Exits:" + "sol ="  + str(sol)
	
	return sol
	
sis=[i for i in range(0,M)]
#print sis
H=[i for i in range(0,N)]
#print H
sol=roundA(H,sis)
#print "solution:" + str(sol)

Sum=0
sis=[i for i in range(0,M)]

for i in sis: #for each Si, find the new span
	top=0
	bottom=N+1
	l=0
	for j in range(0,len(sol)):
		l+=1
		for k in Sis[i]:
			if k == sol[j]:
				if l<bottom:
					bottom=l
				if l>top:
					top=l
	if top<bottom:
		print "top", top, "bottom", bottom
	Sum+=top-bottom

print "Score:"
print Sum
print "Lower bound:"
print cost
finalorder=sol
####################
#Writes output file#
####################
f=open(sys.argv[2],'w')
f.write("%i\n"%len(finalorder))
for i in finalorder:
	f.write("%i\n"%(int(i)+1))


f.close()

############################
#End of output file writing#
############################




