#!/usr/bin/env python

# TODO:
# Get PaToH tool for your architecture:
# http://bmi.osu.edu/~umit/software.html
# Put binary in current directory.

from JSONWrite import *
from subprocess import *
from IGenerateOrder import *
from Ordering_IDD import *
import sys

# Ordering Generation Strategy 1:
# Here after we use the PATOH tools for generate a automatic ordering variable
# with hierarchy
class GenerateOrder_1(IGenerateOrder):
    
    # Constructor :
    # net        : The model
    # partitions : The number of places into one partitions
    # patoh      : Tools access to patoh
    # path_conf  : The directory where we generate the JSON File configuration
    def __init__(self, net, partitions, patoh, path_conf):
        IGenerateOrder.__init__(self, net)
        # Initialize from command line option
        self.partitions = 10
        self.patoh = patoh
        self.path_conf = path_conf
    
    # PatoH Tools
    #  - flat : produce a flat order (true.false)
    #  - pred : work by Predecessor/Successor OR Successor/Predecessor (true.false)
    #  - IDD  : Pre-ordering from IDD order (true.false)
    def PaToH(self,flat,pred,IDD):
        net = self.net
        if '--partition_size' in sys.argv:
            self.partitions = int(sys.argv[sys.argv.index('--partition_size')+1])
            
        partitions_number = self.places_count / self.partitions
        if partitions_number <= 1:
            partitions_number = 2
      
        # Mapping identification for PaToH Element
        patoh_id = 1
        if (IDD):
            ordering = Ordering_IDD(net)
            places = ordering.computeOrder()
        else:
            places = net.places()
        for p in places:
            p.patoh_id = patoh_id
            patoh_id += 1
        # Pins counter:
        pins = 0
        # Output graph prefix:
        graph_file = open(self.modelname + ".graph", "w")
        for t in net.transitions():
            places_set = set()
            if pred:
                for p, tmp in t.predecessors():
                  places_set.add(p)
                for p, tmp in t.successors():
                  places_set.add(p)
            else:
                for p, tmp in t.successors():
                  places_set.add(p)
                for p, tmp in t.predecessors():
                  places_set.add(p)
            pins += len(places_set)
        # Print graph information:
        graph_file.write("% Graph information:\n")
        graph_file.write("% [Start-Id #Vertices #Hyperedges #Pins]\n")
        graph_file.write("1 " + str(self.places_count) + " " + str(self.transitions_count) + " " + str(pins) + "\n")
        # Output graph hyperedges:
        graph_file.write("% Hyperedges:\n")
        graph_file.write("% [Vertice-Id ...]\n")
        for t in net.transitions():
            places_set = set()
            for p, tmp in t.predecessors():
                places_set.add(p)
            for p, tmp in t.successors():
                places_set.add(p)
            for p in places_set:
                graph_file.write(str(p.patoh_id) + " ")
            graph_file.write("\n")
        graph_file.close()
        
        # Execute PaToH
        p1 = Popen([self.patoh, self.modelname + ".graph", str(partitions_number)], stdout=PIPE)
        p1.wait()
        p1out, p1in = p1.communicate()
        
        # Parse the result
        partition_file = open(self.modelname + ".graph.part." + str(partitions_number))
        patoh_result = ""
        for line in partition_file:
            patoh_result += line
        partition_file.close()
        partitions = {}
        id = 1
        try:
            for s in patoh_result.split(" "):
                partitions[id] = int(s)
                id += 1
        except ValueError, e:
            pass
        # Give partition number to each place:
        for p in net.places():
            p.partition_id = partitions[p.patoh_id]
        
        # Create the 1-Hierarchy
        h = Hierarchy()
        for i in range(0, partitions_number):
            if flat:
                h1 = h
            else:
                h1 = Hierarchy()
                h.append(h1)
            for p in [ p for p in net.places() if p.partition_id == i ]:
                h1.append(p)
                
        return h
    
    def span_calculation():
        return 0
    
    # Help command
    # Print help about option of this generation order
    def help(self):
        str = "Generation order 1 : Hyper-graph partionning with PaToH \n"
        str += " --partition_size <integer> : give the size of one partitions\n"
        str += " --flat : desactivate the hierarchy (flat order)\n"
        return str
    
    # Print Information Order into XLS information
    def __str__(self):
        res = ''
        res += str(self.time) + '\t' 
        res += str(self.places_count) + '\t' 
        res += str(self.transitions_count) + '\t' 
        res += str(1) + '\t' 
        res += str(self.partitions) + '\t'
        res += 'PaToH 1-Hierarchy with PREDECESSOR-SUCCESSOR'
        return res
    
    # Strategy Operation for generate one Configuration
    def generate_order(self):
      # Execution of ordering place with PaToH : Activation of hierarchy + PRED and SUCC + No PreOrdering with IDD
      order = self.PaToH(False,True,False)
      # Write JSON File
      conf = []
      conf.append(self.path_conf)
      write_conf(self.path_conf,order)
      return conf

