#!/usr/bin/env python

from IGenerateOrder import *
from generate_order_9 import *
import sys




      



# Ordering Generation Strategy 10 :
# This heuristic is based on the Strategy 9, and it consist to use the default order
# generate by H9, but we want to build a hierarchy during the firing model
# Strategy build Hierarchy :
#  - At each firing transition :
#        - HX = Get the hierarchy linked with one of the transition's places
#        - Add PRED places in HX
#        - Create new Hierarchy HY and Put in SUCC places
class GenerateOrder_10(GenerateOrder_9):
    
    # Constructor :
    # net        : The model
    # path_conf  : The directory where we generate the JSON File configuration
    # api        : Cami API Communication
    def __init__(self,net,path_conf,api):
        GenerateOrder_9.__init__(self,net,path_conf,api)
        # Redefinition of Function
        self.function_fire = self.fire
        
    
    # Print Information Order into XLS information
    def __str__(self):
        res = ''
        res += str(self.time)   + '\t' 
        res += str(self.places_count) + '\t' 
        res += str(self.transitions_count)  + '\t' 
        res += str(self.X)  + '\t' 
        res += str(self.Y) + '\t'
        res += 'Pre-Fired Model (PF-2) with 1-Hierarchy build by Transition'
        return res
    
    # Help command
    # Print help about option of this generation order
    def help(self):
        str = "Generation order 10 : Transition hierarchy building (Variant) (based on PF-2) : " 
        str += "hierarchy is build by transition fired\n"
        str += " --flat : desactivate the hierarchy (flat order)\n"
        return str
    
    
    # This function search which Hierarchy that's the current transition is linked
    def build_hierarchy(self,transition,order):
        # First check a partition link
        partition = None
        min = None
        for place, arc in transition.predecessors():
            if all_p[place] != None and partition == None:
                partition = all_p[place]
                min = all_p[place].countFromRoot()
            elif all_p[place] != None and partition != None and all_p[place].countFromRoot() < min:
                partition = all_p[place]
                min = all_p[place].countFromRoot()
        for place, arc in transition.successors():
            if all_p[place] != None and partition == None:
                partition = all_p[place]
                min = all_p[place].countFromRoot()
            elif all_p[place] != None and partition != None and all_p[place].countFromRoot() < min:
                partition = all_p[place]
                min = all_p[place].countFromRoot()
        if partition == None:
            partition = Hierarchy()
            order.append(partition)
            partition.setParent(order)
        else:
            None
            
        
        return partition


    # This function remove one Hierarchy from the current order
    def __remove(self,order,to_remove):
        if to_remove in order:
            index = order.index(to_remove)
            order.remove(to_remove)
            return order,index
        for elt in order:
            if type(elt).__name__=='list' or type(elt).__name__=='Hierarchy':
                res,index = self.__remove(elt,to_remove)
                if res != None:
                    return res,index
        return None,None
        


    # Function fire :
    # Description :
    # Apply the firing transition, only the POST Firing (PRE are not apply).
    # This is a wrong firing transition, because we don't want to generate one possibility
    # of the space state system, but we want to ensure that we fire the most transition as much as possible.
    # And so cover most of the place's model, directly touch by the firing transitions.
    # Flat Ordering precision :
    # At each fire, we put :
    #  - All predecessors places of the current transition if not exist in ordering vector
    #  - All successors places of the current transition if not exist in ordering vector
    #
    # OPTIMIZATION :
    # 1) New Firing strategy (avoid loop of all transition for know which transition firing)
    # After firing one transition, from all successors places impacting by the current one,
    # we must recalculate the current TSS vector for know which transitions is now firing
    # 2) Avoiding repetition transition :
    # After one firing, we must check in the current vector TSS if there are any transitions
    # that its fired will no add new place in Ordering vector. So we avoid the firing of Transition where all place are already in vector
    #
    # GLOBAL :
    #  --> Fires : Count the total number of fire
    #  --> all_t : For switch off some transition of the firing strategy because :
    #    - One transition can be fired only one time
    #    - One transition must impact the ordering vector, else that's mean all places
    # are already in vector, so transition is not to be used
    #  --> all_p : For know and note that one place is in order vector
    #  --> tss :  - After firing current transition :
    #     - calculate the new transition which can be fired
    #     - remove all transition that will not impacted the ordering vector 
    #
    # Parameter :
    #  - order : Variable Flat oder vector
    #  - transition : the current transition to fire
    #  - torder : the Transition vector ordering
    def fire(self,order,transition,torder):
        # Global used
        global fires,all_t,all_p,tss,tps2,order_len
        # Increment firing counter
        fires += 1
        # Check of transition was not removed from a previous firing
        # It can be happened when algorithme have fired some transition in structural conflict
        if not transition in tss:
            # Avoid function
            return
        
        ## Before search the first partition link
        save = order
        order = self.build_hierarchy(transition,order)
        
        # Add the Predecessors places in vector
        if len(order) != 0:
            res = Hierarchy()
            res.append(order)
            order.setParent(res)
            parent,index = self.__remove(save,order)
            parent.insert(index,res)
            res.setParent(parent)
        else:
            res = order
        for place, arc in transition.predecessors():
            if all_p[place] == None:    
                res.append(place)
                all_p[place] = res

        # Add the Successors places in vector
        for place, arc in transition.successors():
            place.marking = 1 # Now we work only in True, False Value on place marking
            if all_p[place] == None:
                res.append(place)
                all_p[place] = res
                
        # Note that current transition fired, and remove it from TSS vector
        all_t[transition] = 1
        if transition in tss:
            tss.remove(transition)
        
        # After a fired, check if all transition in TSS vector will impact
        # the ordering vector, of not remove, removed it and note it
        ttttt = []
        for t in tss:
            if not self.function_checkFire(t):
                all_t[t] = 1
                ttttt.append(t)
        for t in ttttt:
            tss.remove(t)
                
            
        # After a fired, update the new transition will can be fire
        # Check only transition impacted by the current successors places 
        for place, arc in transition.successors():
            for T,arc in place.successors():
                # only if we can use the transition t
                if all_t[T] == None:
                    if self.function_canFire(T):
                        # Say that current transition is in vector
                        tss.append(T)
                        all_t[T] = 2
        order = save
    
    
    # Strategy Operation for generate one Configuration
    def generate_order(self):
      
      # Generate 1-Hierarchy order
      order,torder = self.function_pre_firing_order()
      if '--flat' in sys.argv:
          order =flating(order)

      # Write JSON File
      conf = []
      conf.append(self.path_conf)
      write_conf(self.path_conf,order)
    
      return conf
          

