#!/usr/bin/env python

from generate_order_11 import *
from JSONWrite import *
import sys



    

# Ordering Generation Strategy 12 :
class GenerateOrder_12(GenerateOrder_11):
    
    # Constructor :
    # net        : The model
    # path_conf  : The directory where we generate the JSON File configuration
    # api        : Cami API Communication
    def __init__(self,net,path_conf,api=None):
        GenerateOrder_11.__init__(self,net,path_conf,api)
    
    # Print Information Order into XLS information
    def __str__(self):
        res = ''
        res += str(self.time)   + '\t' 
        res += str(self.places_count) + '\t' 
        res += str(self.transitions_count)  + '\t' 
        res += str(self.X)  + '\t' 
        res += str(self.Y) + '\t'
        res += 'Flat FORCE Heuristics + XY-Hierarchy'
        return res
    
    # Help command
    # Print help about option of this generation order
    def help(self):
        str = "Generation order 12 : FORCE flat variable ordering + Hierarchy build with N-Cut-Naive \n"
        str += " --flat : desactivate the hierarchy (flat order)\n"
        str +="Parameters for N-Cut-Naive Hierarchy\n"
        str +="   --X <value> : Maximum height of structure\n"
        str +="   --Y <value> : Number of element per list (Y=2 by default)\n"
        str +="If X is not set, X is automatically calculated from X(Y)=Log[Y](count_places)"
        return str
    
    # Calculation of the places SET with recursive PRE and SUC
    def set_places(self,linked,t,recursive,list_p):
        recursive -= 1
        for p,a in t.predecessors():
            if p in list_p:
                linked.add(p) 
        for p,a in  t.successors():
            if p in list_p:
                linked.add(p)
                if recursive != 0:
                    for tt,a in p.predecessors():
                        self.set_places(linked,tt,recursive)
                    for tt,a in p.successors():
                        self.set_places(linked,tt,recursive)

                    
    # Calculation of the transitions SET with recursive PRE and SUC
    def set_transitions(self,linked,p,recursive):
        recursive -= 1
        for t,a in p.predecessors():
            linked.add(t)
        for t,a in p.successors():
            linked.add(t)
            if recursive != 0:
                for pp,a in t.predecessors():
                    self.set_transitions(linked,pp,recursive) 
                for pp,a in t.successors():
                    self.set_transitions(linked,pp,recursive)
    
    # FORCE ALGORITHM Heuristics
    # Consist into the calculation of the Center Gravity of one transition
    # The calculation of Gravity can be execute on X Pre and Suc Node
    # See function set_places and set_transitions
    def FORCE(self, list_p, max, rec):
      
      # Sorted the ordering places vector
      cpt = 1
      for p in list_p:
          p.order = cpt
          cpt += 1
      
      # For I iterations
      for i in range(0,max):
          stabilized = 0
          print 'Calculation COG rank ' , i
          # Calculate Transitions COG, from the place Order
          for t in self.net.transitions():
              linked = set()
              t.cog = 0.0
              self.set_places(linked, t, rec , list_p)
              for p in linked:
                  t.cog += p.order
              if len(linked) != 0:
                t.cog =  float(t.cog) / float(len(linked))
              else:
                t.cog = 0.0
              
          # Calculate Places COG, from the transition Order
          for p in list_p:
              linked = set()
              save = p.order
              p.order = 0.0
              self.set_transitions(linked, p, rec)
              for t in linked:
                  p.order += t.cog
              new = float(p.order) / float(len(linked))
              #if new != p.order:
              #   stabilized += 1
              p.order = new 
        
          
          # Sorted the Ordering Vector of place
          import operator
          cpt = 1
          for p in list_p:
              p.old = cpt
              cpt += 1
          list_p.sort(key=operator.attrgetter('order'))
          cpt = 1
          for p in list_p:
              p.order = cpt
              if p.order != p.old:
                  stabilized += 1
              cpt += 1
          print "Nb Modification : " , stabilized
          if stabilized == 0:
              break
    
      return list_p
    

    # Strategy Operation for generate one Configuration
    def generate_order(self):
      
      # Add all places into ordering vector
      order = []
      for p in self.net.places():
          order.append(p)
      
      # Parameters for User : How many iteration you want
      max = len(order)
      if '--i' in sys.argv:
          max = int(sys.argv[sys.argv.index('--i')+1])
      rec = 1
      if '--rec' in sys.argv:
          rec = int(sys.argv[sys.argv.index('--rec')+1])
      stabilized = 0
      if '--stabilized' in sys.argv:
          stabilized = int(sys.argv[sys.argv.index('--stabilized')+1])
      # FORCE Algorithm
      print 'Execution of FORCE Heuristics :'
      print ' - Maximum runs             : ' , max
      print ' - Calculation depth of COG : ' , rec
      print ' - Stabilization COG break  : ' , stabilized 
      
      
      order=self.FORCE(order, max, rec)
      t1 = time.clock()
      print 'Flat Time generation : ',  (t1 -  self.time), ' s'
      # Naive XY Build Hierarchy
      self.X , self.Y = self.calculate_XY(len(order))
      order = self.write_partition(order,self.X,self.Y)
      print 'Hierarchichalization Time generation : ',  time.clock() - t1, ' s'
      
      if '--flat' in sys.argv:
          order =flating(order)
          
      # Write JSON File
      conf = []
      conf.append(self.path_conf)
      write_conf(self.path_conf,order)
      
      
      return conf
          

