#!/usr/bin/env python

from IGenerateOrder import *
from generate_order_12 import *
from pycami.converter import *
import sys



test = []

# Ordering Generation Strategy 13 :
class GenerateOrder_13(GenerateOrder_12):
    
    # Constructor :
    # net        : The model
    # path_conf  : The directory where we generate the JSON File configuration
    # api        : Cami API Communication
    def __init__(self,net,path_conf,api):
        GenerateOrder_12.__init__(self,net,path_conf,api)
        self.function_fire = self.fire
        self.function_weight = self.weight
        self.function_chooseFire = self.chooseFire
    
    # Print Information Order into XLS information
    def __str__(self):
        res = ''
        res += str(self.time)   + '\t' 
        res += str(self.places_count) + '\t' 
        res += str(self.transitions_count)  + '\t' 
        res += str(self.X)  + '\t' 
        res += str(self.Y) + '\t'
        res += 'Pre-Firing in Path Width with a Partially Identical Sequence Hierarchy + naive XY-Hierarchy in each Sequence'
        return res
    
    # Help command
    # Print help about option of this generation order
    def help(self):
        str = "Generation order 13 : Partially Identical Sequence\n"
        str += " --flat : desactivate the hierarchy (flat order)\n"
        str += " '--reverse' : Reverse each sequence ordering"
        str += " '--force' : Apply FORCE Heuristic in each sequence"
        str +="Parameters for N-Cut-Naive Hierarchy\n"
        str +="   --X <value> : Maximum height of structure\n"
        str +="   --Y <value> : Number of element per list (Y=2 by default)\n"
        str +="If X is not set, X is automatically calculated from X(Y)=Log[Y](count_places)"
        str += "With Communication places"
        str += " --com <file> : Give the file which contain places in communication, for extract from sequence"
        str += "Stragegy of placing Communication places, by default with Calculation by Gravity Center"
        str += " --F1 : Put COM place after First Sequence Linked"
        str += " --F2 : Put COM place after Last Sequence Linked"
        str += " --F3 : Put COM place before Last Sequence Linked"
        return str
    
    
    # This function is used at each Firing Transition for build a default Hierarchy
    # Description :
    # At each transition firing, we search if one element is link with on partition
    # if YES :
    #   - We add the current element in the partition linked
    # if NO :
    #   - We create a new Hierarchy
    def build_hierarchy(self,transition,order):
        # First check a partition link
        partition = None
        min = None
        for place, arc in transition.predecessors(): 
            if all_p[place] != None and partition == None and place.ignored == False:
                partition = all_p[place]
                min = len(partition)
            elif all_p[place] != None and partition != None and len(all_p[place]) < min and place.ignored == False:
                partition = all_p[place]
                min = len(partition)
                
        if partition == None:
            partition = Hierarchy()
            order.append(partition)
        else:
            None
        
        return partition
    
        
    # Function weight :
    # Redefinition : No calculation Weight, we take all transition which
    # can be firing at one time and return it (path width) 
    def weight(self,order,ts):
        min_w = []
        # For each transition, calculate the Weight of the transition
        for t in ts:

           min_w.append(t)
           
        return min_w

    # Redefinition of ChooseFire for a path width Firing Model
    def chooseFire(self,order , ts , torder):
        min_w = self.function_weight(order,ts)
        # We choose all transition at one time
        if len(min_w) == 0:
            # No transition can be fired : Break ALGO
            transitionChoose = None
        else:
            for t in min_w:
                self.fire(order, t, torder)
            transitionChoose = min_w[0]
                                
        return transitionChoose
    
    # Function fire :
    # Redefinition of FIRE :
    # Same as H9 but we call a different build_hierarchy function
    def fire(self,order,transition,torder):
        # Global used
        global fires,all_t,all_p,tss,tps2,order_len,other
        # Increment firing counter
        fires += 1

        # Check of transition was not removed from a previous firing
        # It can be happened when algorithme have fired some transition in structural conflict
        if not transition in tss:
            # Avoid function
            return
        ## Before search the first partition link
        save = order
        order = self.build_hierarchy(transition,order)
        
        preds = set()
        # Add the Predecessors places in vector
        for place, arc in transition.predecessors():
            if all_p[place] == None and place.ignored == False:
                print ' - Append ' , place.name , ' --> ' , place.ignored
                preds.add(place)

        
        if len(preds) > 1 and '--optim1' in sys.argv:
            tab = Hierarchy()
            order.append(tab)
            for place in preds:
                tab.append(place)
                all_p[place] = order
        else:
            for place in preds:
                order.append(place)
                all_p[place] = order
                
                    
                    
                # Add the Successors places in vector
        
        succ = set()
        for place,a in transition.successors():
            place.marking = 1 # Now we work only in True, False Value on place marking
            if all_p[place] == None and place.ignored == False:
                succ.add(place)
        
        
        for place in succ:
            place.marking = 1 # Now we work only in True, False Value on place marking

            
        if len(succ) > 1 and '--optim2' in sys.argv:
            tab = Hierarchy()
            order.append(tab)
            for place in succ:
                tab.append(place)
                all_p[place] = order
                hie = True
        else:
            for place in succ:
                order.append(place)
                all_p[place] = order
                hie = True
        
                
        # Note that current transition fired, and remove it from TSS vector
        all_t[transition] = 1
        if transition in tss:
            tss.remove(transition)
        
        # After a fired, check if all transition in TSS vector will impact
        # the ordering vector, of not remove, removed it and note it
        ttttt = []
        for t in tss:
            if not self.function_checkFire(t):
                all_t[t] = 1
                ttttt.append(t)
        for t in ttttt:
            tss.remove(t)
                
            
        # After a fired, update the new transition will can be fire
        # Check only transition impacted by the current successors places 
        for place, arc in transition.successors():
            for T,arc in place.successors():
                # only if we can use the transition t
                if all_t[T] == None:
                    if self.function_canFire(T):
                        # Say that current transition is in vector
                        tss.append(T)
                        #tss.insert(0,T)
                        all_t[T] = 2
        order = save
    
    
    
    def extract_comPlaces(self):
        test = []
        # Default
        for p in self.net.places():
            p.ignored = False
        # If COM
        if '--com' in sys.argv:
            comfile = sys.argv[sys.argv.index('--com') + 1]
            com = []
            f = open(comfile,'r')
            for line in f:
                com.append(line.replace('\n',''))
            f.close()
            
            # Finding COM
            for the_name in com:
                for p in self.net.places():
                    if p.name.find(the_name) == 0:
                        p.ignored = True
                        test.append(p)
        
            print 'COM places extracted : '
            for p in test:
                print p.name , ' --> ', p.ignored

        
        return test
            


    # Build a basic 1-hierarchy communication
    # Order : Group of PS distance by COM Place in ROOT
    def build_hierarchy_by_communication(self,order):
        h1 = Hierarchy()
        new_order = Hierarchy()
        cpt = 0
        for h in order:
            if type(h).__name__=='Hierarchy':
                if cpt == 2:
                    new_order.append(h1)
                    h1 = Hierarchy()
                    cpt = 0
                h1.append(h)
                cpt += 1
            else:
                h1.append(h)
        # At the end : Check if there are some element in H1, if yes
        # we have not find 2 partitions for put it, so we up all in order
        if cpt == 2 :
            a = 0
            new_order.append(h1)
        else:
            a = 0
            for h in h1:
                a+=1
                new_order.append(h)
        
        if (len(new_order)-a) >= 5:
            new_order = self.build_hierarchy_by_communication(new_order)
        
        return new_order
   
    def searchLink(self,order,place):
        
        # Initialize places Link within
        setPlaces = set()
        for t,a in place.successors():
            for p,aa in t.predecessors():
                setPlaces.add(p)
        link = []
        #print ''
        #print 'Place ' , place.name , ' is linked with : '
        for ps in order:
            if type(ps).__name__=='Hierarchy' or type(ps).__name__=='list':
                for p in setPlaces:
                    if p in ps and not ps in link:
                        link.append(ps)
                        #print ' - ' , p.name

        
        return link
    
    # - Get all Leaf Node of Root
    # - Build graph dependance
    def step2_graph(self,order):
        print ' WORKING ON GRAPH COMMUNICATION PLACES !!!!'
        comPlaces = []
        paralleSequence = []
        mod = {}
        
        for h in order:
            if type(h).__name__=='Hierarchy' or type(h).__name__=='list':
                paralleSequence.append(h)
        
        print "Before SORT :"
        for ps in paralleSequence:
            print ' - [',len(ps), '] = ' , ps  
        
        if '--with-rev' in sys.argv:
            paralleSequence.sort(lambda x,y : cmp(len(x) , len(y)) , reverse=True)
            print "After SORT :"
            for ps in paralleSequence:
                print ' - [',len(ps), ']'
        # 1) Apply link PSi for each COM places
        max = -1
        for h in order:
            print h 
            if not (type(h).__name__=='Hierarchy' or type(h).__name__=='list'):
                # We work on COM Place
                print 'Work on ',h.name
                h.link = self.searchLink(order, h)
               
                
#                if len(h.link) <= 1:
#                    # Exist only one link, so remove COM place from order and re-put it it into PS
#                    px,ii = h.old_position
#                    #print "OLD Position Find for place ", h.name
#                    cog = -1
#                    res = 'NIL - '
#                    for ps in h.link:
#                         res += str(paralleSequence.index(ps)) + ' '
#                    px.insert(ii,h)
                if h.link >= 1:
                    res = ''
                    cog = 0.0
                    min = len(paralleSequence)
                    maxx = -1
                    for ps in h.link:
                        index = paralleSequence.index(ps)
                        if min > index:
                            min = index
                        if maxx < index:
                            maxx = index
                        cog += paralleSequence.index(ps)
                        res += str(paralleSequence.index(ps)) + ' '
                    # Strategy 1 : Calculation of COG (default)
                    cog = int ( cog / len(h.link)) + 1
                    if '--F1' in sys.argv:
                        # Strategy 2 PUT Place after the first Sequence
                        cog = min + 1
                    if '--F2' in sys.argv:
                        # Strategy 3 PUT Place after the last Sequence
                        cog = maxx + 1
                    if '--F3' in sys.argv:
                        # Strategy 4 PUT Place before the last Sequence
                        cog = maxx 
                
                # At the end, add COM place into MOD Offset
                if not mod.has_key(cog):
                    mod[cog] = []
                mod[cog].append(h)
                if cog > -1:
                    comPlaces.append(h)
                
                # PRINT Iformation
                print 'COM place ' , h.name , ' is linked with [' , res,'] = ' ,  cog , ' : ' , len(mod[cog])
                if max < cog:
                    max = cog
        
        # At the end we have all COM place range by COG Position
        # Inserting into Main Vector
        
        offset = 0
        for i in range(0,max+1):
            if mod.has_key(i):
                if len(mod[i]) > 1:
                    paralleSequence.insert(i+offset,mod[i])
                else:
                    paralleSequence.insert(i+offset,mod[i][0])
                offset+=1
        
        
        print ''
        print ''
        print ''
        print 'COM places after filtering  : ' , len(comPlaces)
        for p in  comPlaces:
            print ' - ' , p.name
                            
        return paralleSequence
                



    # Strategy Operation for generate one Configuration
    def generate_order(self):
      
      if '--safe' in sys.argv:  
          ToColor(self.net,'safe.converted')
      test = self.extract_comPlaces()
      
      # Generate Order
      order,torder = self.function_pre_firing_order()
      
      
      for p in test:
          print 'adding ' , p.name
          order.append(p)
          
      # Re-organize the COM places
      if '--com' in sys.argv:
          order = self.step2_graph(order)
      
      i = 0
      for h in order:
          if type(h).__name__=='Hierarchy' or type(h).__name__=='list':
              # Reversing if user want to reverse
              if '--reverse' in sys.argv and (i % 2 != 0):
                  h.reverse()
              if '--force' in sys.argv:
                  h = self.FORCE(h, 1, 1)
              # Naive XY Build Hierarchy
              #self.X , self.Y = self.calculate_XY(len(h))
              #h = self.write_partition(h,self.X,self.Y)
              self.X , self.Y = self.calculate_XY(len(h))
              newH = self.write_partition(h,self.X,self.Y)
              order.insert(order.index(h),newH)
              order.remove(h)
              #print 'Elements Number Before Partition [',i,'] = : ', len(h)
              #order2.append(h)
              i += 1
    
      while len(order) > 10:
          order = self.write_partition(order,1,self.Y)
          
      if '--flat' in sys.argv:
          order =flating(order)
      
      # Write JSON File
      conf = []
      conf.append(self.path_conf)
      write_conf(self.path_conf,order)
    
      return conf

          

