#!/usr/bin/env python

from IGenerateOrder import *
from generate_order_1 import *
from Ordering_IDD import *
import sys


# Ordering Generation Strategy 6:
# Use PaToH for build a default 1-Hierarchy order and then we use the naive XY-Hierarchy
# for build a multi-level Hierarchy 
class GenerateOrder_6(GenerateOrder_1):
    
    # Constructor :
    # net        : The model
    # patoh      : Tools access to patoh
    # path_conf  : The directory where we generate the JSON File configuration
    def __init__(self,net,partitions,patoh,path_conf):
        GenerateOrder_1.__init__(self,net,partitions,patoh,path_conf)
        self.X = 0
        self.Y = 0
    
    # Print Information Order into XLS information
    def __str__(self):
        res = ''
        res += str(self.time)   + '\t' 
        res += str(self.places_count) + '\t' 
        res += str(self.transitions_count)  + '\t' 
        res += str(self.X)  + '\t' 
        res += str(self.Y) + '\t'
        res += 'PaToH Flat +  naive XY-Hierarchy'
        return res

    # Help command
    # Print help about option of this generation order
    def help(self):
        str = "Generation order 6 : Flat variable ordering with PaToH + Hierarchy build with N-Cut-Naive \n"
        str += " --partition_size <integer> : give the size of one partitions\n"
        str += " --flat : desactivate the hierarchy (flat order)\n"
        str +="Parameters for N-Cut-Naive Hierarchy\n"
        str +="   --X <value> : Maximum height of structure\n"
        str +="   --Y <value> : Number of element per list (Y=2 by default)\n"
        str +="If X is not set, X is automatically calculated from X[Y]=Log_Y(count_places)"
        
        return str

    # Function write_new_partition :
    # Used by write_partition and it consist into the second algorithme phase
    # for cut the current order into many hierarchy of Y Elements into each one
    # Return new Order with hierarchy
    def write_new_partition(self,order,Y,X):
        h1 = []
        h2 = []
        i = 0
        for elt in order:
            if i == (Y):
                h1.append(h2)
                h2 = []
                i = 0
            h2.append(elt)
            i += 1
        if len(h2) != 0:
            h1.append(h2)
            
        return h1

    # Function write_partition
    # Used into the second algorithme phase for cut the Flat order into
    # many hierarchy, with a hierarchy proof of X Elements
    def write_partition(self,order,X,Y):
        for i in range(X):
            order = self.write_new_partition(order,Y,X)
        return order


    # Automatic definition of XY-Hierarchy value
    # User can define this value by the --X and --Y parameters into command line
    # P : Number of element into Flat Order Vector
    # Return the X,Y Value
    def calculate_XY(self,P):
        self.X = -1
        self.Y = 2
        if '--X' in sys.argv:
            self.X = int(sys.argv[sys.argv.index('--X')+1])
        if '--Y' in sys.argv:
            self.Y = int(sys.argv[sys.argv.index('--Y')+1])
        MaxX = 0
        while P > self.Y and self.Y != 0:
          MaxX += 1
          P = P / self.Y
        if self.X == -1:
            self.X = MaxX/2
        print 'We can divides into maximum ' , MaxX , ' number of proof hierarchy with ',self.Y, ' into one partition'
        print 'You have choosen X = ',self.X, ' Y = ',self.Y
        
        return self.X,self.Y

    # Strategy Operation for generate one Configuration
    def generate_order(self):
        # Execution of ordering place with PaToH : Desactivation of hierarchy + PRED and SUCC + PreOrdering with IDD
        order = self.PaToH(False,True,False)
        # Calculate Automatic or a User definition of XY-Hierarchy
        self.X , self.Y = self.calculate_XY(len(order))
        # Naive XY Build Hierarchy
        order = self.write_partition(order,self.X,self.Y)
        if '--flat' in sys.argv:
          order =flating(order)
        # Write JSON File
        conf = []
        conf.append(self.path_conf)
        write_conf(self.path_conf,order)
        
        return conf

