#!/usr/bin/env python

from IGenerateOrder import *
from generate_order_6 import *
from Ordering_IDD import *
import sys


# Global information of the module
#  - lastTab  ==> MAP List of Counter Transition fire, where key is transition and value
# the number of fired
#  - lastFire ==> The latest transition fired (for detect simple cycle and avoiding the
# infinite loop)
#  - fires ==> The total number of transition fired during the ordering variable 
lastTab = {}
lastFire = None
nb = 0
fires = 0

    
    

# Ordering Generation Strategy 7:
# Pre-Firing Model H1 :
# This heuristic consist into 2 phases :
#  - Produce a Flat Ordering by pre-fired the petri net
#  - Produce a naive XY-Hierarchy build from the flat ordering vector
# FIRING :
# Here after, the firing consist to check at each case which Transition can be fire
# , fire it and initialize the Flat Ordering Vector with the place touch by the transition fired 
class GenerateOrder_7(GenerateOrder_6):
    
    # Constructor :
    # net        : The model
    # X          : The number of hierarchy
    # Y          : The number of element in hierarchy
    # path_conf  : The directory where we generate the JSON File configuration
    def __init__(self,net,path_conf,api):
        GenerateOrder_6.__init__(self,net,0,None,path_conf)
        self.Y = 0
        self.api = api
        self.X = 0
    
    # Print Information Order into XLS information
    def __str__(self):
        res = ''
        res += str(self.time)   + '\t' 
        res += str(self.places_count) + '\t' 
        res += str(self.transitions_count)  + '\t' 
        res += str(self.X)  + '\t' 
        res += str(self.Y) + '\t'
        res += 'Pre-Fired (PF-1) Model Flat + naive XY-Hierarchy'
        return res
    
    # Help command
    # Print help about option of this generation order
    def help(self):
        str = "Generation order 7 : Pre-Firing (PF-1) Variable ordering + Hierarchy build with N-Cut-Naive \n"
        str += " --flat : desactivate the hierarchy (flat order)\n"
        str +="Parameters for N-Cut-Naive Hierarchy\n"
        str +="   --X <value> : Maximum height of structure\n"
        str +="   --Y <value> : Number of element per list (Y=2 by default)\n"
        str +="If X is not set, X is automatically calculated from X(Y)=Log[Y](count_places)"
        
        return str
    
    # Function chooseFire :
    # Description :
    # At each time of pre-firing transition, some cases can appears :
    #  - 0 transition can be fire : stop the algorithm
    #  - 1 transition can be fire : choose this one
    #  - N transitions can be fire
    #        - From N transitions, groups all which have the greatest Weight Calculation Transition (N[MAX_W])
    #        - From N[MAX_W] transitions and from the MAP List Transition Counter
    #            - Keep the one which have not recently fired, or the first one if thera are many transitions (TransitionChoose)
    #        - From the TransitionChoose, check if transition is not a indeterminism transition
    #            - If yes, fire the other and keep TransitionChoose
    #        - Then, check in no infinite loop are detected, if yes break the loop and return None
    #        - Return the TransitionChoose
    # Calculation of the weight :
    #  W ==> The number of place in PRED and SUCC already in the order vector
    #  if W ==  T(PRED) + T(SUCC) ==> All place are already in vector, W = -1
    #  W = -1 ==> All place of the transition are already in vector, so we prefere to fire
    #  another transition
    def chooseFire(self,order , ts , torder):
        min = -20
        min_w = []
        # For each transition, calculate the Weight of the transition
        for t in ts:
            # Calculate W(T)
            w = 0 
            places = set() 
            for p, tmp in t.predecessors():places.add(p)
            for p, tmp in t.successors():places.add(p)
            for p in places:
                if p in order:
                    w += 1
            # If W == T(PRED) + T(SUCC)
            if len(places) == w:
                w = -1
            # Build the list of transition with W = MIN
            if min < w:
                min_w = [] # Reseting
                min_w.append(t)
                min = w
            elif min == w:
                min_w.append(t)
        
        # Choose the transition
        if len(min_w) == 0:
            # No transition can be fired : Break ALGO
            transitionChoose = None
        elif len(min_w) == 1:
            # Only one transition can be choose : choose it
            transitionChoose = min_w[0]
        else:
            # We must choose one transition among N one
            # First initialize the indeterminism transition table
            links = {}
            for t in ts:
                for p, tmp in t.predecessors():
                    if not links.has_key(p): 
                        links[p] = []
                    if not t in links[p]:
                        links[p].append(t)        
            # Among the N transition
            transitionChoose = None
            for t in min_w:
                # By default, choose the first one
                if transitionChoose == None:
                    transitionChoose = t
                else:
                    # We must choose the one not recently fired
                    global lastTab
                    if not lastTab.has_key(transitionChoose):
                         lastTab[transitionChoose] = 0
                    if not lastTab.has_key(t):
                         lastTab[t] = 0
                    if lastTab[t] < lastTab[transitionChoose]:
                        transitionChoose = t
                        bestWeight = w
            
            # At the end, check if the transition choose is no indeterminism, if yes
            # We must do a pre-fired all one which not the current transition
            for pla,arc in transitionChoose.predecessors():
                if len(links[pla]) > 1:
                    # We have found one indeterminism transition
                    #print 'Found on transition indeterminism : ' , trans[0].name
                    for ttt in links[pla]:
                        if transitionChoose != ttt:
                            self.fire(order,ttt,torder)
            
        # Check the possible cycle of one transition fired
        # FIXME : For the moment, we detect only one transition cycle (every 20 fired following)
        # but we must work on another one for detect cycle about one to XX transition fired in repetition
        global lastFire
        global nb
        if transitionChoose == lastFire:
            nb += 1
        else:
            lastFire = transitionChoose
            nb = 0
        if nb == 20:
            transitionChoose = None
        
        return transitionChoose
    
    # Function canFire :
    # Description :
    # Check if on transition can be fire
    # Parameter :
    #  - transition : the current transition to check
    def canFire(self,transition):
        res = True
        for place, arc in transition.predecessors():
            if not (int(place.marking) >= int(arc.valuation)):
                res = False
        return res
    
    # Function fire :
    # Description :
    # Apply the firing transition, only the POST Firing (PRE are not apply).
    # This is a wrong firing transition, because we don't want to generate one possibility
    # of the space state system, but we want to ensure that we fire the most transition as much as possible.
    # And so cover most of the place's model, directly touch by the firing transitions.
    # Flat Ordering precision :
    # At each fire, we put :
    #  - All predecessors places of the current transition if not exist in ordering vector
    #  - All successors places of the current transition if not exist in ordering vector
    # Parameter :
    #  - order : Variable Flat oder vector
    #  - transition : the current transition to fire
    #  - torder : the Transition vector ordering
    def fire(self,order,transition,torder):
        # At each firing Transition, count the number of fire of the current transition
        # It is used later for choose the transition which have not been fired recently
        global lastTab
        # Count the total number of firing transitions for statistic
        global fires
        fires += 1
        if not lastTab.has_key(transition):
            lastTab[transition] = 0
        lastTab[transition] += 1
        # If transition does not exist in the current transition order, add it to the vector order
        if not transition in torder:
            torder.append(transition)
        # Add the Predecessors places in vector
        for place, arc in transition.predecessors():
            # This line is comment because it is a wrong firing transition
            #place.marking = int(place.marking) - int(arc.valuation)
            if not place in order:
                order.append(place)
        # Add the Successors places in vector
        for place, arc in transition.successors():
            place.marking = int(place.marking) + int(arc.valuation)
            if not place in order:
                order.append(place)

    
    def pre_firing_order(self):
      # Ordering Variable consist into Pre-firing Net
      # and list an order on the first Variable Firing
      # Test for stop the Pre-Firing Algorithm : we have used all places in model
      # List of transition which can be fired
      order = []
      torder = []
      empty = []
      ts = ['t']
      

      # Read and Parse CAMI file:
      net = self.net
      
      try:
          while len(ts) != 0 and len(order) != self.places_count:
              ts = []
              for t in net.transitions():
                # Check the possible firing
                if self.canFire(t):
                    ts.append(t)
              if len(ts) != 0:
                  # Choose one firing
                  t = self.chooseFire(order,ts,torder)
                  # Firing
                  self.fire(order,t,torder)
      except :
         pass
      
      # If missing some places
      diffP = self.places_count - len(order)        
      if len(order) != self.places_count:
          for p in net.places():
              if not p in order:
                for tra,arc in p.predecessors():
                    for pla,arc in tra.predecessors():
                        if pla in order and not (p in order):
                            #print ' - Place : ' , p.name ,' inserted after is first predecessor'
                            order.insert( order.index(pla),p)
              # At the end, if P not in the end, adding it to the end of vector
              if not p in order:
                  #print ' - PLace : ', p.name,' found no predecessor place, so it has been put at the end of vector'
                  order.append(p)
      
      
     
      # Print Stat about Pre-Firing
      global fires
      self.api.write('')
      self.api.write('')
      self.api.write( 'Statistic of the Pre-Fired Heuristic')
      self.api.write('Total of transition firing                         : '+ str(fires))
      self.api.write('Total of place touched by firing                   : ' + str(self.places_count-diffP) + '/' + str(self.places_count))
      self.api.write('Total of place replacing by first predecessor      : '+ str(diffP))
      self.api.write( 'Total time of the ordering                         : '+str(self.time) + ' s')
      self.api.write( '')
      self.api.write('')
      
      return order,torder
    
    
    
    
    # Strategy Operation for generate one Configuration
    def generate_order(self):

      # Generate Flat oder by pre-firing petri net
      order,torder = self.pre_firing_order()
      # Calculate Automatic or a User definition of XY-Hierarchy
      self.X , self.Y = self.calculate_XY(len(order))
      # Naive XY Build Hierarchy
      order = self.write_partition(order,self.X,self.Y)
      if '--flat' in sys.argv:
          order =flating(order)
      # Write JSON File
      conf = []
      conf.append(self.path_conf)
      write_conf(self.path_conf,order)
    
      return conf
          

