#!/usr/bin/env python

from IGenerateOrder import *
from generate_order_8 import *
import sys

# Ordering Generation Strategy 9 :
# This heuristic is based on the Strategy 8, and it consist to use the default order
# generate by H8, but we want to build a hierarchy during the firing model
# Strategy build Hierarchy :
#  - At each firing transition :
#        - HX = Get the hierarchy linked with one of the transition's places
#        - Add PRED places in HX
#        - Create new Hierarchy HY and Put in SUCC places
class GenerateOrder_9(GenerateOrder_8):
    
    # Constructor :
    # net        : The model
    # path_conf  : The directory where we generate the JSON File configuration
    # api        : Cami API Communication
    def __init__(self,net,path_conf,api):
        GenerateOrder_8.__init__(self,net,path_conf,api)
        self.function_fire = self.fire
    
    # Print Information Order into XLS information
    def __str__(self):
        res = ''
        res += str(self.time)   + '\t' 
        res += str(self.places_count) + '\t' 
        res += str(self.transitions_count)  + '\t' 
        res += str(self.X)  + '\t' 
        res += str(self.Y) + '\t'
        res += 'Pre-Fired Model (PF-2) with 1-Hierarchy build by Transition Fired'
        return res
    
    # Help command
    # Print help about option of this generation order
    def help(self):
        str = "Generation order 9 : Transition hierarchy building (based on PF-2) : " 
        str += "hierarchy is build by transition fired\n"
        str += " --flat : desactivate the hierarchy (flat order)\n"
        return str
    
    
    # This function is used at each Firing Transition for build a default Hierarchy
    # Principe :
    # At each transition firing, we search if one element is link with on partition
    # if YES :
    #   - We add the current element in the partition linked
    # if NO :
    #   - We create a new Hierarchy
    def build_hierarchy(self,transition,order):
        # First check a partition link
        partition = None
        min = None
        for place, arc in transition.predecessors():
            if all_p[place] != None and partition == None:
                partition = all_p[place]
                min = len(partition)
            elif all_p[place] != None and partition != None and len(all_p[place]) < min:
                partition = all_p[place]
                min = len(partition)
        for place, arc in transition.successors():
            if all_p[place] != None and partition == None:
                partition = all_p[place]
                min = len(partition)
            elif all_p[place] != None and partition != None and len(all_p[place]) < min:
                partition = all_p[place]
                min = len(partition)
        if partition == None:
            partition = Hierarchy()
            order.append(partition)
        else:
            #res = partition
            #partition.append(res)
            #partition = res
            None
        
        return partition
    
        


    # Function fire :
    def fire(self,order,transition,torder):
        # Global used
        global fires,all_t,all_p,tss,tps2,order_len
        # Increment firing counter
        fires += 1

        # Check of transition was not removed from a previous firing
        # It can be happened when algorithme have fired some transition in structural conflict
        if not transition in tss:
            # Avoid function
            return
        ## Before search the first partition link
        save = order
        order = self.build_hierarchy(transition,order)
        
        # Add the Predecessors places in vector
        for place, arc in transition.predecessors():
            if all_p[place] == None:
                order.append(place)
                all_p[place] = order

        # Add the Successors places in vector
        for place, arc in transition.successors():
            place.marking = 1 # Now we work only in True, False Value on place marking
            if all_p[place] == None:
                res = Hierarchy()
                res.append(place)
                order.append(res)
                all_p[place] = res
                
        # Note that current transition fired, and remove it from TSS vector
        all_t[transition] = 1
        if transition in tss:
            tss.remove(transition)
        
        # After a fired, check if all transition in TSS vector will impact
        # the ordering vector, of not remove, removed it and note it
        ttttt = []
        for t in tss:
            if not self.function_checkFire(t):
                all_t[t] = 1
                ttttt.append(t)
        for t in ttttt:
            tss.remove(t)
                
            
        # After a fired, update the new transition will can be fire
        # Check only transition impacted by the current successors places 
        for place, arc in transition.successors():
            for T,arc in place.successors():
                # only if we can use the transition t
                if all_t[T] == None:
                    if self.function_canFire(T):
                        # Say that current transition is in vector
                        tss.append(T)
                        all_t[T] = 2
        order = save
    
    
    # Strategy Operation for generate one Configuration
    def generate_order(self):
      # Generate Order
      order,torder = self.function_pre_firing_order()
      if '--flat' in sys.argv:
          order =flating(order)

      # Write JSON File
      conf = []
      conf.append(self.path_conf)
      write_conf(self.path_conf,order)
    
      return conf

          

