# Author : S. HONG
# Date   : 26/01/2009
# Description :
# Module Conversion of Ordinary Petri Net to :
#  - CAMI Format
#  - APNN Format

import pn
import sys
# Mapping for the unic ID of CAMI Element
map_id = {}
unic_id = 2

# Transform STR into cami Text Format
def text2cami(str):
    return len(str).__str__()+ ":" + str


# Get unic ID from a STR identification
def idFromText(str):
    id = 0
    global map_id,unic_id
    if map_id.has_key(str):
        return map_id[str]
    else:
        map_id[str] = unic_id
        unic_id += 1
        return map_id[str]


def ToColor(net,dest):
    arcTestPre = {}
    colors = {}
    variables = set()
    arcs = []
    id = 0
    maxB = 2
    if '--borne' in sys.argv:
        maxB = int(sys.argv[sys.argv.index('--borne')+1])
    
    # Before convert the model, we must check if there are arcs Test
    for t in net.transitions():
        for p,a in t.predecessors():
            a.test = None
            for pp,aa in t.successors():
                aa.test = None
                if p.id == pp.id and aa.valuation == a.valuation:
                    arcTestPre[a] = aa
    print ''
    print ''
    print 'There are ' , len(arcTestPre), ' arcs test'
    print ''
    print ''
    
    # Calculate borne
    for p in net.places():
        # Make something
        p.borne = maxB
    
    for p in net.places():
        colors[p.borne] = "c" + str(p.borne) + " is 0.."+str(p.borne)+";"
        p.domain = "c" + str(p.borne)
        p.cmarking = None
        if int(p.marking) >= 0:
            p.cmarking = '<'+str(p.marking)+'>'
        
    for t in net.transitions():
        nbPr = 0
        t.guard = '['
        for p,a in t.predecessors():
            if not arcTestPre.has_key(a):
                variable = ("pr"+ p.domain + "n" + str(nbPr),p.domain)
                variables.add(variable)
                a.valuation = "<"+variable[0] +">"
                arcs.append((t,p,"<"+variable[0] +"--1>"))
                if nbPr != 0:
                    t.guard += ' AND '
                t.guard += '(' + variable[0] + '>' + '0' + ')'
                nbPr += 1
            else:
                # Manage Arc test
                print "Managing arc test from Place " , p.name , " to transition " , t.name 
                variable = ("pr"+ p.domain + "n" + str(nbPr),p.domain)
                arcTestPre[a].test = variable
                variables.add(variable)
                a.valuation = "<"+variable[0] +">"
                if nbPr != 0:
                    t.guard += ' AND '
                t.guard += '(' + variable[0] + '>' + '0' + ')'
                nbPr += 1
        nbPr = 0
        for p,a in t.successors():
            if a.test == None:
                variable = ("po"+ p.domain + "n" + str(nbPr),p.domain)
                variables.add(variable)
                a.valuation = "<"+ variable[0] +"++1>"
                arcs.append((p,t,"<"+variable[0] +">"))
                nbPr += 1
            else:
                print "Managing arc test from  transition " , t.name," to Place " , p.name
                a.valuation = "<"+ a.test[0] +">"
                
        t.guard += ']'
   
    # At the end, add new arcs
    id = 10000
    for f,t,v in arcs:
        net.create_arc('arc',id,f,t)
        arc = net.get(id)
        arc.valuation = v
        id += 1
   
    atpn = open(dest,"w")
    # The Name net
    atpn.write('DB()\n')
    # NET NODE, and then print declaration
    atpn.write('CN('+text2cami('net')+','+str(1)+')\n')
    line = 1
    atpn.write('CM(11:declaration,1,'+str(line)+',1,'+text2cami('CLASS')+')\n')
    line +=1
    for k,c in colors.items():
        atpn.write('CM(11:declaration,1,'+str(line)+',1,'+text2cami(c)+')\n')
        line +=1
    atpn.write('CM(11:declaration,1,'+str(line)+',1,'+text2cami('VAR')+')\n')
    line +=1
    for v in variables:
        atpn.write('CM(11:declaration,1,'+str(line)+',1,'+text2cami("  " + v[0] + " in " + v[1] + ";")+')\n')
        line +=1
    # Conversion of the place
    for p in net.places():
        atpn.write('CN('+text2cami('place')+','+str(idFromText(p.id))+')\n')
        atpn.write('CT('+text2cami('name')+','+str(idFromText(p.id))+','+text2cami(p.name)+')\n')
        atpn.write('CT('+text2cami('domain')+','+str(idFromText(p.id))+','+text2cami(p.domain)+')\n')
        if p.cmarking != None:
            atpn.write('CT('+text2cami('marking')+','+str(idFromText(p.id))+','+text2cami(p.cmarking)+')\n')
    for t in net.transitions():
        atpn.write('CN('+text2cami('transition')+','+str(idFromText(t.id))+')\n')
        atpn.write('CT('+text2cami('name')+','+str(idFromText(t.id))+','+text2cami(t.name)+')\n')
        atpn.write('CT('+text2cami('guard')+','+str(idFromText(t.id))+','+text2cami(t.guard)+')\n')
    for id,arc in net.all_arcs().items():
        atpn.write('CA('+text2cami('arc')+','+str(idFromText(arc.id))+','+str(idFromText(arc.source.id))+','+str(idFromText(arc.destination.id))+')\n')
        atpn.write('CT('+text2cami('valuation')+','+str(idFromText(arc.id))+','+text2cami(arc.valuation)+')\n')
    atpn.write('FB()\n')
    atpn.close()
# Conversion of a net into ATPN Format
# net  : The model to transform
# dest : Name file of the model converted
def ToAtpn(net,dest):
    atpn = open(dest,"w")
    # The Name net
    atpn.write('\\beginnet{model}\n')
    # Conversion of the place
    for p in net.places():
        atpn.write('\\place{E_'+str(p.id)+'}{\\name{'+p.name+'}\\capacity{1}\\init{'+str(p.marking)+'}}\n')
    atpn.write('\n\n\n')
    for t in net.transitions():
        atpn.write('\\transition{E_'+str(t.id)+'}{\\name{'+t.name+'}}\n')
    atpn.write('\n\n\n')
    for id,arc in net.all_arcs().items():
        atpn.write('\\arc{E_'+str(arc.id)+'}{\\from{E_'+str(arc.source.id)+'} \\to{E_'+str(arc.destination.id)+'} \\weight{'+str(arc.valuation)+'} \\type{ordinary}}\n')
    
    atpn.write('\\endnet')
    atpn.close()

# Conversion of a net into Matrix Format and add Index information in NET place and NET transition
# net    : The model to transform
# dest   : Name file of the model converted
def ToMatrix(net,dest):
    p_map = {}
    t_map = {}
    col = 0
    lig = 0
    for p in net.places():
        p.index = col
        p_map[p.index]=p
        col += 1
    for t in net.transitions():
        t.index = lig
        t_map[t.index]=t
        lig += 1
    
    # Then Create the Empty matrix
    matrix = []
    for i in range(0,lig):
        matrix.append([]) 
        for j in range(0,col):
            matrix[i].append(0)
    
    # For each Transition
    for t in net.transitions():
        for p,a in t.predecessors():
            matrix[t.index][p.index] = 1
        for p,a in t.successors():
            matrix[t.index][p.index] = 1    
    
    # Write File
    convert = open(dest, 'w')
    convert.write (str(lig)+'\n')
    convert.write (str(col)+'\n')
    for i in range(0,len(matrix)):
        convert.write (str(i) + ' :')
        for j in range(0,len(matrix[i])):
            if matrix[i][j] != 0:
                convert.write (' ' + str(j))
        convert.write ('\n')
    convert.close()
    
    return p_map,t_map

# Conversion of a net into CAMI Format
# net  : The model to transform
# dest : Name file of the model converted
def ToCami(net,dest):
    atpn = open(dest,"w")
    # The Name net
    atpn.write('DB()\n')
    # Conversion of the place
    for p in net.places():
        atpn.write('CN('+text2cami('place')+','+str(idFromText(p.id))+')\n')
        atpn.write('CT('+text2cami('name')+','+str(idFromText(p.id))+','+text2cami(p.name)+')\n')
        atpn.write('CT('+text2cami('marking')+','+str(idFromText(p.id))+','+text2cami(p.marking)+')\n')
    for t in net.transitions():
        atpn.write('CN('+text2cami('transition')+','+str(idFromText(t.id))+')\n')
        atpn.write('CT('+text2cami('name')+','+str(idFromText(t.id))+','+text2cami(t.name)+')\n')
    for id,arc in net.all_arcs().items():
        atpn.write('CA('+text2cami('arc')+','+str(idFromText(arc.id))+','+str(idFromText(arc.source.id))+','+str(idFromText(arc.destination.id))+')\n')
        atpn.write('CT('+text2cami('valuation')+','+str(idFromText(arc.id))+','+text2cami(arc.valuation)+')\n')
    atpn.write('FB()\n')
    atpn.close()