import pn
import types

##############################################################################

CN = """CN(%(node_type)s,%(node_id)d)"""
CA = """CA(%(arc_type)s,%(arc_id)d,%(starting_id)d,%(end_id)d)"""
CT = """CT(%(attr_name)s,%(node_id)d,%(attr_value)s)"""

##############################################################################

def to_cami_text(string):

  return str(len(string)) + ":" + string

##############################################################################

def to_cami(net):

  cami = ""

  # all nodes
  for node_id , node in net.all_nodes().items():
    cami += CN % { 'node_type'   : to_cami_text(pn.cami_mapping.pn_as_cami(node))
                 , 'node_id'     : node_id }
    cami += "\n"

  # all arcs
  for arc_id , arc in net.all_arcs().items():
    cami += CA % { 'arc_type'    : to_cami_text(pn.cami_mapping.pn_as_cami(arc))
                 , 'arc_id'      : arc_id
                 , 'starting_id' : arc.source.id
                 , 'end_id'      : arc.destination.id}
    cami += "\n"

  # all attributes
  for node_id , node in net.all_nodes().items():

    for attribute , default_value in pn.attributes_mapping[type(node)].items():

      value = getattr( node , attribute )
      if value == default_value:
        continue

      cami += CT % { 'attr_name'   : to_cami_text(attribute)
                   , 'node_id'     : node_id
                   , 'attr_value'  : to_cami_text(str( value ))}
      cami += "\n"

  return cami

##############################################################################
