# Author : A. HAMEZ
# Date   : --------
# Description :
# CAMI Parser written into Python Code
# NOTE : The NET model is a dynamic Class
# Modification S. HONG 26/01/10
# Adding the ATPN Parser Format (can now parse the SNOOP Editor file)


import pn
#import antlr.declaration
##############################################################################

def read_cami_text(string):
  
  colon_pos = string.find(':')
  size = int(string[0:colon_pos])
  text = string[ colon_pos + 1 : colon_pos + size + 1]
  return text , string[ colon_pos + size + 1 : ]

##############################################################################

def read_integer(string):
  
  comma_pos = string.find(',')
  if comma_pos == -1:
    comma_pos = string.find(')')
  value = int( string[ 0 : comma_pos ] )
  return value , string[ comma_pos : ]

##############################################################################


def read_parameter(string):
  colon_begin = string.find('{')
  colon_end = string.find('}')
  text = string[ colon_begin + 1 : colon_end]
  return text , string[ colon_end+1 : ]

##############################################################################

    
# Parse model into ATPN Format
# namefile : Path access of the model to parse
def parse_atpn(namefile):
  
  file = open(namefile,"rU")
  unprocessed = set()
  
  net        = pn.Net()
  arcs       = {}
  attributes = {}
  
  for line in file:
  
    # Remove RC
    line = line.replace('\n','')
    
    # Place
    if line.startswith('\\place{'):
      node_type  = 'place'
      node_id   , rem = read_parameter(line)
      net.create_node( node_type, node_id)
      if rem.find('name{') != -1:
          rem = rem[rem.find('name{'):]
          name , rem = read_parameter(rem)
          attributes[ (node_id , 'name') ] =  name
      if rem.find('capacity{') != -1:
          rem = rem[rem.find('capacity{'):]
          capacity , rem = read_parameter(rem)
      if rem.find('init{') != -1:
          rem = rem[rem.find('init{'):]
          marking , rem = read_parameter(rem)
          attributes[ (node_id , 'marking') ] =  marking
      # Goto Next Line
      continue
        
    # Transition
    elif line.startswith('\\transition{'):
      node_type  = 'transition'
      node_id   , rem = read_parameter(line)
      net.create_node( node_type, node_id)
      if rem.find('name{') != -1:
          rem = rem[rem.find('name{'):]
          name , rem = read_parameter(rem)
          attributes[ (node_id , 'name') ] =  name
      # Goto Next Line
      continue
    
     # Arcs
    elif line.startswith('\\arc{'):
      arc_type  = 'arc'
      arc_id   , rem = read_parameter(line)
      if rem.find('from{') != -1:
          rem = rem[rem.find('from{'):]
          starting_id , rem = read_parameter(rem)
      if rem.find('to{') != -1:
          rem = rem[rem.find('to{'):]
          end_id , rem = read_parameter(rem)
      arcs[arc_id] = ( arc_type , starting_id , end_id )
      if rem.find('weight{') != -1:
          rem = rem[rem.find('weight{'):]
          valuation , rem = read_parameter(rem)
          attributes[ (arc_id , 'valuation') ] =  valuation
      # Goto Next Line
      continue
      
    
          
    # Ignored ATPN commands
    elif line.startswith('\beginnet') or line.startswith('\endnet'):
        # Goto Next Line
        continue
    
    # Unprocessed ATPN commands
    elif len(line) != 0:    
        unprocessed.add(line)

  # Process arcs
  for identifier , ( arc_type , start_id , end_id ) in arcs.items():
    net.create_arc( arc_type , identifier , net.get(start_id) , net.get(end_id) )

  # Process attributes
  for ( node_id , name ) , value in attributes.items():
    if node_id == 1:
        #print 'Add attribute to Net ' , name , ' - ' , value
        setattr( net, name , value )
        #print net.declaration
    else:
        setattr( net.get(node_id) , name , value )

  file.close()
  net.pathmodel = namefile
  #link_declaration(net)
  return net

# Parse model into CAMI Format
# namefile : Path access of the model to parse
def parse_cami(namefile):
    
  file = open(namefile,"rU")
  unprocessed = set()
  
  net        = pn.Net()
  arcs       = {}
  attributes = {}
  
  for line in file:
  
    # Nodes
    if line.startswith('CN('):
      node_type , rem = read_cami_text(line[3:])
      node_id   , rem = read_integer(rem[1:])
    
      net.create_node( node_type, node_id)
    
    # Arcs
    elif line.startswith('CA('):
      arc_type    , rem = read_cami_text(line[3:])
      arc_id      , rem = read_integer(rem[1:])
      starting_id , rem = read_integer(rem[1:])
      end_id      , rem = read_integer(rem[1:])
      
      arcs[arc_id] = ( arc_type , starting_id , end_id )
            
    # Attributes
    elif line.startswith('CT('):
      attribute_name  , rem = read_cami_text(line[3:])
      associated_node , rem = read_integer(rem[1:])
      attribute_value , rem = read_cami_text(rem[1:])
      
      attributes[ (associated_node , attribute_name) ] = attribute_value
    
    # Attributes
    elif line.startswith('CM('):
      attribute_name  , rem = read_cami_text(line[3:])
      associated_node , rem = read_integer(rem[1:])
      ign_1 , rem = read_integer(rem[1:])
      ign_2 , rem = read_integer(rem[1:])
      
      attribute_value , rem = read_cami_text(rem[1:])
      
      if attributes.has_key((associated_node , attribute_name)):
        attributes[ (associated_node , attribute_name) ] = attributes[ (associated_node , attribute_name) ] + '\n' +  attribute_value
      else:
        attributes[ (associated_node , attribute_name) ] =  attribute_value
          
    # Unprocessed CAMI commands
    else:
      unprocessed.add( line[0:2] )

  # Process arcs
  for identifier , ( arc_type , start_id , end_id ) in arcs.items():
    net.create_arc( arc_type , identifier , net.get(start_id) , net.get(end_id) )

  # Process attributes
  for ( node_id , name ) , value in attributes.items():
    if node_id == 1:
        #print 'Add attribute to Net ' , name , ' - ' , value
        setattr( net, name , value )
        #print net.declaration
    else:
        setattr( net.get(node_id) , name , value )

  link_declaration(net)
  file.close()
  net.pathmodel = namefile 
  return net


def link_declaration(net):
    print 'Parsing Colored Domain'
    # Check if there is some declaration into Model
    #if net.declaration != None:

#            print 'Parsing Declaration : '
#            print net.declaration
#            print ''
#            net.declaration = net.declaration.replace('\n','\t\n')
#            net.declaration = antlr.declaration.parse_declaration(net.declaration)
#            # For each places of the net, check if there is some domain to redefine
#            for p in net.places():
#                # Make a good instance of domain
#                p.domain = net.declaration['domain'][p.domain]
#                print 'Place ' , p.name , ' with domain ' , p.domain.name , ' : '
#                for color in p.domain.colors:
#                    print ' Colored Class ', color.name , ' = ' , color.the_val
