#!/usr/bin/env python

'''
Created on 16 sept. 2009

@author: hong
'''


from CAMI.DialogBox import *
from PROCESS.Process import *
from HEURISTICS.IGenerateOrder import *
from HEURISTICS.pycami.converter import *
from HEURISTICS.pycami.parser import *
from PNITS import *
from PNSDD import *
from IDDMC import *
from ZBDDMC import *
from MEMORY.MemoryChecker import *
from UTILS.Utils import * 
import time

# Instanciation Tool
def createTool(input,mem,conf):
    if input.appname == 'pnsdd':
        return PNSDD(input.api
              ,input.host
              ,mem
              ,input.path_pnsdd
              ,input.path_lib
              ,input.path_model_ord
              ,conf
              ,input.library
              ,input.path_formula
              ,input.input_model)
    if input.appname == 'its-ctl':
        return PNITS(input.api
              ,input.host
              ,mem
              ,input.path_pnsdd
              ,input.path_lib
              ,input.path_model_ord
              ,conf
              ,input.library
              ,input.path_formula
              ,input.input_model)
    if input.appname == 'zbddmc':
        return ZBDDMC(input.api
              ,input.host
              ,mem
              ,input.path_pnsdd
              ,input.path_lib
              ,input.path_model_ord
              ,conf + '.idd'
              ,input.library
              ,input.path_formula
              ,input.input_model)
    if input.appname == 'iddmc':
        return IDDMC(input.api
              ,input.host
              ,mem
              ,input.path_pnsdd
              ,input.path_lib
              ,input.path_model_ord
              ,conf + '.idd'
              ,input.library
              ,input.path_formula
              ,input.input_model)

def application(standalone):
    # Initialize the API Communication with FrameKit
    tool = None
    input = InputArgument(standalone,'pnsdd')
    if '--IDDMC' in sys.argv:
        input = InputArgument(standalone,'iddmc')
    elif '--PNITS' in sys.argv:
        input = InputArgument(standalone,'its-ctl')
    elif '--ZBDDMC' in sys.argv:
        input = InputArgument(standalone,'zbddmc')
    
    
    # Print help and Exit program
    if input.help:
        input.print_help()
        json = GenerateOrder(None,input)
        print json.help()
        exit()

    try:
        
        # Inform FK the Start application
        input.api.start_tool("Driver for use " + input.appname,"1.1","S. HONG, E. PAVIOT-ADET, F. KORDON - LIP6 Copyright")
        
        # Check the correct unix end-line of model file
        # Before unfolding, check the end-line of the model,
        # If EL is Mac Type, we must convert it into Unix EL
        macunix_endline_conversion(input.path_model,input.path_model+'.converted',input.input_model)
        input.path_model += '.converted'
        
        # if mode interactif, asking to user what order he wants to execute
        if input.ask:
            # Create the Dialog Box and send it
            map = {}
            map[0]=1;map[1]=2;map[2]=6;map[3]=8
            map[4]=11;map[5]=12;map[6]=13;map[7]=21
            tab=["PaToH"
                ,"NOA99 + PaToH"
                ,"PaToH + N-Cut-Naive"
                ,"PF1 + N-Cut-Naive"
                ,"PF2 + N-Cut-Naive"
                ,"FORCE + N-Cut-Naive"
                ,"PISH"
                ,"NOA99 + N-Cut-Naive"]
            db = ComboBox("Hierarchical Strategy Order"
                          ,"Select the hierarchical order"
                          ,tab)
            print "Wait answer for order"
            input.api.send_DB(db)
            # Get answer : which is the selected order
            answer = input.api.wait_ADB()
            input.order = 11
            if answer.button == DBAnswer.BUTTON_OK:
                index = tab.index(answer.value)
                if index != -1:
                    input.order = map[index]
                    print "Order chosen : " , input.order
                
                
        # Bounded the memory
        mem = MemoryChecker(input.host,input.appname)
        mem.percents = input.percents
        # Print information about memory
        input.api.send_Trace_Bloc(str(mem))
        
        # If unfolding is necessary :
        if input.to_unfold == 1 and input.input_model == 'CAMI':
           input.api.send_State("Removing old unfolding model")
           p1 = Popen(["rm","-f" ,input.path_model_ord])
           p1.wait()
           input.api.send_State("Unfolding the Colored Petri Net with the A. LINARD Unfolder")
           p1 = Popen([input.path_unfolder, "-s", "-ac" , "-ao",input.path_model , input.path_model_ord],stdin=PIPE,stdout=PIPE,stderr=PIPE)
           p1.communicate() # Becarefull PIPE is not infinite, so we must call communicate for free the buffer PIPE
           p1.wait()
           # Then parse the colored model if user want to work within
           colored_net = parse_cami(input.path_model)
        else:
           input.path_model_ord = input.path_model
        
        if input.input_model == 'CAMI':
            net = parse_cami(input.path_model_ord)
        elif input.input_model == 'ATPN':
            net = parse_atpn(input.path_model_ord)
       
        # Waiting answer
        if input.ask:
            # Create the Dialog Box and send it
            db = TextBox("TOOL CTL Verification","Give the CTL Formula to check",["!EX(TRUE);"])
            input.api.send_DB(db)
            input.answer = input.api.wait_ADB()

        # Then From the HEURISTIC Program
        if input.porder != None:
            input.api.send_Trace('Strategy Order User : user provide his variable configuration : ' + input.porder)
            ccc = []
            ccc.append(input.porder)
            input.order = 0
            json = GenerateOrder(net,input)
            ccc = json.generate_order()
            #json = None
        else:
            # Waiting answer
            json = GenerateOrder(net,input)
            ccc = json.generate_order()

        
        if input.answer.button == DBAnswer.BUTTON_OK:
            # Get the value and print it into the directory works
            formula = open(input.path_formula,"w")
            formula.write(input.answer.value)
            formula.close()
        else:
            formula = open(input.path_formula,"w")
            formula.close()
 
        # Execute the tool
        for conf in ccc:
            # Instanciation of tools
            tool = createTool(input,mem,conf)
            tool.execute()
            # If we must print Stats into file
            print tool , json
            if input.stats != None:
                res = ''
                res += input.path_model + '\t'
                res += input.input_model + '\t'
                if input.to_unfold == 1:
                    res += 'colored' + '\t'
                else:
                    res += 'ordinary' + '\t'
                if input.library == 1:
                    res += 'libDDD' + '\t'
                else:
                    res += 'libSDD' + '\t'
                # Add PNSDD Information
                res += str(tool) + '\t'
                res += str(json)  + '\t'
                res += str(tool.mem_check.checker.Mprocess)
                file = open(input.stats,'a')
                file.write(res+'\n')
                file.close()
        
        # Clean
        if json != None and not '--keep' in sys.argv:
            json.clean()
                        
    except Exception, ee:
        # Exception Raised during the execution python script
        # We must send a wrong ending to Framekit
        input.api.send_Trace("***************************************************************")
        input.api.send_Trace("***************************************************************")
        input.api.send_Trace("Error during the execution script, error message is : "+str(ee))
        input.api.send_Trace("***************************************************************")
        input.api.send_Trace("***************************************************************")
        if pnsdd != None:
            pnsdd.stop()
        # Ending tools with error
        input.api.end_tool(1)
        exit(0)
    

    # End of Application
    input.api.end_tool(0)
    # At the end : Clean File
    clean_file(input.path_formula)
    clean_file(input.path_model+'*')
    clean_file(input.path_model_ord+'*')
    
    


if __name__ == '__main__':
    standalone = False
    if '-kit_name' in sys.argv:
        print "PNXDD"
        exit(0)
    if '-version' in sys.argv:
        print "DataMngr 2.1 EnvCom 2.6 UsrInterf 2.3 Tools 3.5"
        exit(0)
    if '-s' in sys.argv:
        print "StandAlone Execution Command"
        standalone = True
    
    # Execute the Application
    application(standalone)


    
