#!/usr/bin/env python

'''
Created on 16 sept. 2009

@author: hong
'''

from PROCESS.Process import *
from HEURISTICS.pycami.converter import *
from HEURISTICS.pycami.parser import *
from MEMORY.MemoryChecker import *
from UTILS.Utils import *


class PNITS(Process):
    # Constructor of the PNSDD Application tools
    def __init__(self,
                api,
                host,
                mem_check,
                path_pnsdd,
                path_lib,
                path_model_ord,
                path_conf,
                library = 0,
                path_formula = None,
                format = 'CAMI'):
        self.temp_dir = ''
        if host != None:
            path_pnsdd = 'its-ctl'
        if mem_check.os_name == 'Darwin':
            ldpath = LDPATH_Darwin(host,path_lib)
        if mem_check.os_name == 'Linux':
            ldpath = LDPATH_Linux(host,path_lib)
        if format == 'ATPN':
            net = parse_atpn(path_model_ord)
            ToCami(net,path_model_ord+'.cami')
            path_model_ord = path_model_ord + '.cami'
        elif format == 'CAMI':
            net = parse_cami(path_model_ord)
            ToCami(net,path_model_ord+'.cami')
            path_model_ord = path_model_ord + '.cami'
        # Build the argument command
        argument = []
        argument.append("-c")               # 0
        argument.append(path_model_ord)     # 1
        argument.append('-j')               # 2
        argument.append(path_conf)          # 3
        
        if path_formula != None:
            argument.append("-ctl")         # 4
            argument.append(path_formula)   # 5
        else:
            argument.append("")             # 4
            argument.append("")             # 5
        # Constructor of Base 
        Process.__init__(self,host,'its-ctl',path_pnsdd,argument,ldpath,mem_check,api)
        
    # Redefinition for add comments
    def start(self):
        # Execute PNSDD on local host
        Process.start(self)
        self.api.send_State('Execution of ITS-CTL Tools')

    # Operation POST SSH necessary if you want to clean the execution on remote host
    # After execution
    def post_ssh(self):
        # Remove the tempory directory
        p1 = Process(self.host,'rm','rm',['-rf' , self.temp_dir])
        p1.execute()
        
    # Operation PRE SSH necessary for prepare the execution on remote host
    def pre_ssh(self):
        # Create a unique tempory directory
        self.api.send_State('Create new directory on remote host')
        p1 = Process(self.host,'mktemp','mktemp',['-d -t pnsdd'])
        p1.execute()
        self.temp_dir,ii = p1.process.communicate()
        self.temp_dir = self.temp_dir.replace('\n','')
        
        # Then copy the model
        self.api.send_State('Copy the petri net file on remote host')
        model_ord = None
        for word in self.argument[1].split('/'):
            model_ord = word
        p1 = Process(None,'scp','scp',[self.argument[1] + ' ' + self.host + ':' + self.temp_dir])
        p1.execute()
        self.argument[1] = self.temp_dir + '/' + model_ord
            
        # Then copy the JSON configuration file
        conf = None
        self.api.send_State('Copy the JSON Configuration file on remote host')
        for word in self.argument[3].split('/'):
            conf = word
        p1 = Process(None,'scp','scp',[self.argument[3] + ' ' + self.host + ':' + self.temp_dir])
        p1.execute()
        self.argument[3] = self.temp_dir + '/' + conf
        
        # Then copy the formula file if exists
        if self.argument[5] != "":
            formula = None
            self.api.send_State('Copy the CTL Formula file on remote host')
            for word in self.argument[5].split('/'):
                formula = word
            p1 = Process(None,'scp','scp',[self.argument[5] + ' ' + self.host + ':' + self.temp_dir])
            p1.execute()
            self.argument[5] = self.temp_dir + '/' + formula

    # Parse the result command of PNSDD
    def parseResult(self):
        line_to_parse = False
        self.states = '0'
        self.sdd_final = '0'
        self.unicity_sdd_peak = '0'
        self.cache_sdd_peak = '0'
        self.cache_hom_peak = '0'
        self.unicity_hom_peak = '0'
        self.time_generation = '0'
        for line in self.buf.split('\n'):
            line = line.replace('\n','')
            if line.lower().find('reachable') != -1:
                line_to_parse = True
            if line_to_parse:
                words = line.split(',')
                self.states = words[1]
                self.time_generation = words[2]
                self.sdd_final = words[4] + ' + ' + words[5]
                self.unicity_sdd_peak = words[6]+ ' + ' + words[7]
                self.cache_sdd_peak = words[9]+ ' + ' + words[11]
                self.cache_hom_peak = words[12]
                self.unicity_hom_peak = words[8]+ ' + ' + words[10]
                
                line_to_parse = False
                
    def __str__(self):
        res = ''
        res += self.states + '\t'
        res += self.sdd_final + '\t'
        res += self.unicity_sdd_peak + '\t'
        res += self.cache_sdd_peak + '\t'
        res += self.cache_hom_peak + '\t'
        res += self.unicity_hom_peak + '\t'
        res += self.time_generation
        return res
    
    def end(self):
        Process.end(self)
        try:
            self.parseResult()
        except:
            pass
        # At the end check the status code
        if self.status_code != 0:
            self.api.send_Trace("Error during the execution of ITS-CTL Tools, maybe PNSDD have been reached the maximum bounded memory. Error CODE : " +str(self.status_code))
            self.api.send_Trace(" ")
        else:
            self.api.send_Trace("Maximum Memory Used (%)  : " + str((self.mem_check.checker.Mprocess*100)/(self.mem_check.checker.MaxMemory / (1024 * 1024))) + " %")
            self.api.send_Trace("Maximum Memory Used (MB) : " + str(self.mem_check.checker.Mprocess) + " MB")
            self.api.send_Trace("PNITS is successfully terminated")
    
