#!/usr/bin/env python
import time
import sys
from subprocess import *
from MEMORY.MemoryChecker import *
import threading

def affiche(process):
    status_code = None
    while(status_code == None):
        time.sleep(1)
        o,e = process.process.communicate()
        process.buf += o
        process.buf += "\n"+ e
        status_code = process.process.poll()
         

# Specialization Command of PNSDD for DARWIN System
class LDPATH_Darwin:
    # Constructor
    #  - host    : Remote host execution (None is localhost)
    #  - the_lib : The directory to add into DYLD_LIBRARY_PATH 
    def __init__(self,host,path_lib):
        self.path_lib = path_lib
        self.host = host
    
    # Return the Initialization Command of the LD PATH initialization SHELL
    def command(self):
        command = 'export DYLD_LIBRARY_PATH='+self.path_lib+':$DYLD_LIBRARY_PATH && '
        return command

# Specialization Command of PNSDD for LINUX System
class LDPATH_Linux:
    # Constructor
    #  - host    : Remote host execution (None is localhost)
    #  - the_lib : The directory to add into LD_LIBRARY_PATH
    def __init__(self,host,path_lib):
        self.path_lib = path_lib
        self.host = host
    
    # Return the Initialization Command of the LD PATH initialization SHELL
    def command(self):
        command = 'export LD_LIBRARY_PATH='+self.path_lib+':$LD_LIBRARY_PATH && '
        return command
    



# Class definition for execution a PROCESS
# A process is composed by :
#  - self.procpath  = the process path execution
#  - self.procname  = the process name identification
#  - self.api       = Optional (CAMI API Communication), use for print TRACE Message
#  - self.host      = On which host you want to execute the Process (None is localhost)
#  - self.ldpath    = The required LDPATH initialization variable
#  - self.arguments = Vector of argument composed the command
#  - self.mem_check = The memory checker (Optional if you want to bound memory)
#  - self.status_code = The END Status Code of execution process
#  - self.process = The process Object execution
class Process:
    def __init__(self,host,procname,procpath,arguments,ldpath=None,mem_check=None,api=None):
        self.procpath = procpath
        self.procname = procname
        self.api = api
        self.host = host
        self.ldpath = ldpath
        self.argument = arguments
        self.mem_check = mem_check
        self.status_code = None
        self.buf = ""
        self.process = None # The process ID to execute
        # Affectation of name procees of Memory Check Module if not null
        if self.mem_check != None:
            self.mem_check.checker.process_name =  self.procname
    
    def start(self):
        # Initialization of the command
        command = self.procpath
        if self.ldpath != None:
            command = self.ldpath.command() + ' ' + command
        if self.host != None:
            # Must be define by User, for instance for transfering some file to remote host
            self.pre_ssh()
            command = 'ssh ' + self.host + ' ' + self.procname
        command = command + ' '
        for word in self.argument:
            command += ' ' + word
        # Execute of the process
        if self.mem_check != None:
            self.mem_check.reset()
        if '--parse' in sys.argv:
            self.process = Popen(command,shell=True,stdout=PIPE,stdin=PIPE,stderr=PIPE)
        else:
            self.process = Popen(command,shell=True,stdout=PIPE,stdin=PIPE,stderr=PIPE)
    
    # Execution Command
    def execute(self):
        # Begin execution
        self.start()
        # If we must check the memory
        if self.mem_check != None and self.api != None:
            self.wait_and_check()
        else:
            self.wait()
        # End execution
        self.end()
    
    # Stop the execution of the process
    def stop(self):
        p1 = Process(self.host,'killall','killall',[self.procname])
        p1.execute()
    
    # Function wait
    # Default wait of the end application
    def wait(self):
        self.process.wait()
    
    # Function end
    # End of the execution process
    def end(self):
        if self.host != None:
            self.post_ssh()
    
    # Default PRE SHH
    def pre_ssh(self):
        None
    
    # Default POST SSH
    def post_ssh(self):
        None
    
    # Function :
    # Wait the end of memory and check every second the current memory usage of the process
    def wait_and_check(self):
        a = threading.Thread(None, affiche, None,(self,))
        a.start()
        self.status_code = None
        while(self.status_code == None):
            # Wait 1 second
            time.sleep(1)
            # Check memory Bound, if yes stop the process and send trace to user
            self.mem_check.verify()
            #self.api.send_State( 'Maximum memory ' + str(self.mem_check.percents) + ' % - CURRENT ' + str(self.mem_check.current) + ' %')
            # Check if son P1 is terminated
            self.status_code = self.process.poll()
    
        # Wait ending of thread
        a.join(10)
        # Process is terminated, show information result
        try:            
            self.api.send_Trace("Result of Command :\n")
            self.api.send_cami_file(self.buf)
            
        except:
            pass
        