from CAMI.APICami import *
from PROCESS.Process import *

# Class initialization if the Driver Configuration file 
class FileINI:
    # Default constructor with a initialization file
    def __init__(self,filename):
        self.host = []
        self.remote = 'localhost'
        try:
            file = open(filename)
            for line in file:
                line = line.replace('\n','')
                try:
                    if(line.find('remote') != -1):
                        index = line.find('=')
                        self.remote = line[index+1:]
                    if(line.find('host') != -1):
                        index = line.find('=')
                        self.host.append(line[index+1:])
                    env.append(line.replace('\n',''))
                except:
                    pass
            file.close()
        except:
            pass


# Class definition for parse the command line in
#  - Standalone execution
#  - Framekit execution
# After parsing command line, it create this argument :
#  api            : The CAMI API Library, for communication with FrameKit
#  path           : Directory where ROOT application are (directory groups command pnsdd,patoh ...) 
#  path_model     : The directory path of the petri net model
#  path_model_ord : The directory path of the unfold petri net model
#  path_lib       : The directory path of the library required by PNSDD
#  path_unfolder  : The directory path of the unfolder tools (A. LINARD)  
#  path_patoh     : The directory path of the patoh tools
#  path_conf      : The directory path of the JSON SDD Variables Configuration file
#  path_formula   : The directory path of the CTL Formula to verify
#  library        : Kind of DD library to use (SDD or DDD) 
#  to_unfold      : Is the petri net model is to unfold
#  partitions     : Number of partitions for PATOH tools
#  ask            : Standalone Execution ?
#  percents       : Maximum percents of memory to use
#  host           : Execution on localhost or a remote host
class InputArgument:
    # Initialization from ARGV
    def __init__(self,standalone,appname):
        self.appname = appname
        self.__parse_arg()
    
    def __parse_arg(self):
        # Default API Cami
        self.api = APICami()
        # Default path execution
        if True:
            # Default parameters
            self.partitions = 2
            self.ask = False
            if len(sys.argv) < 8:
                raise Exception('Missing some argument')
            if '-fifo_in_out' in sys.argv:
                self.api = APICami(sys.argv[sys.argv.index('-fifo_in_out')+1],sys.argv[sys.argv.index('-fifo_in_out')+2])
                self.ask = True
            if '--root' in sys.argv:
                self.path = sys.argv[sys.argv.index('--root')+1]
            else:
                if not '-end_env_p' in sys.argv:
                    raise Exception('Missing --root argument')
                index = sys.argv.index('-end_env_p') + 1
                # Here we must have in order : the model name, the library Mode
                file = open("fk_basic_info")
                env = []
                for line in file:
                    env.append(line.replace('\n',''))
                self.path = env[5] + "/" + "AMI-Net" + "/" + env[0] + "/PNSDD/"
                
            if '--model' in sys.argv:
                self.path_model = sys.argv[sys.argv.index('--model')+1]
                self.path_model_ord = self.path_model + '.unfold.ord'
            else:
                raise Exception('Missing --model argument')
            if '--library' in sys.argv:
                self.library = int(sys.argv[sys.argv.index('--library')+1])
            else:
                raise Exception('Missing --library argument')
            if '--unfold' in sys.argv:
                self.to_unfold = int(sys.argv[sys.argv.index('--unfold')+1])
            else:
                self.to_unfold = 0
            if '--porder' in sys.argv:
                self.porder = sys.argv[sys.argv.index('--porder')+1]
            else:
                self.porder = None
            if '--memory' in sys.argv:
                self.percents = int(sys.argv[sys.argv.index('--memory')+1])
            else:
                self.percents = 90
            if '--host' in sys.argv:
                self.host = sys.argv[sys.argv.index('--host')+1]
            else:
                self.host = None
            if '--statistic' in sys.argv:
                self.stats = sys.argv[sys.argv.index('--statistic')+1]
            else:
                self.stats = None
            # Default mode in CAMI
            self.input_model = 'CAMI'
            if '--cami' in sys.argv:
                self.input_model = 'CAMI' # CAMI
            if '--atpn' in sys.argv:
                self.input_model = 'ATPN' # ATPN
            self.answer = DBAnswer()
            if '--ctl' in sys.argv:
                self.path_formula = sys.argv[sys.argv.index('--ctl')+1]
                self.answer.button = self.answer.BUTTON_OK
                f = open(self.path_formula,"r")
                for line in f:
                    self.answer.value = self.answer.value + line
                f.close()
            else:
                self.path_formula = None
            if '--order' in sys.argv:
                self.order = int(sys.argv[sys.argv.index('--order')+1])
            else:
                self.order = 11
            self.help = False
            if '--help' in sys.argv:
                self.help = True
            self.path_patoh    = self.path + "patoh"
            self.path_lib      = self.path + "./lib"
            self.path_conf     = self.path_model + ".conf"
            self.path_formula  = self.path_model + ".ctl"
            self.path_pnsdd    = self.path + self.appname
            self.path_unfolder = self.path + "col2ordequivDDD_reference_FrameKit"
            self.fileini       = self.path + 'remote_host.ini'


    # Help Command Line
    def print_help(self):
        print './Main.py --root <root-dir> --model <path-to-model> --library <DDD or SDD> --unfold <True or False> --ctl <path-to-formula> --memory <percents>'
        print '--root    : Specify the root directory where binary are (pnsdd, patoh ...)'
        print '--model   : Specify the model directory'
        print '--library : Specify the DD Library to use'
        print '          : 0 is libSDD (No valid option for IDDMC)'
        print '          : 1 is libDDD (No valid option for IDDMC)'
        print '--unfold  : Specify if we must unfold the model'
        print '          : 0 is No'
        print '          : 1 is Yes'
        print '--ctl     : Specify the CTL Formula directory file'
        print '--memory  : Specify the maximum memory to use (in percents), memory is calculated from the percents of the maximum memory of the current computer'
        print '--host    : Which host you want to execute PNSDD by SSH Command (give the username@hostname)'
        print '--order   : Give the Strategy of automatic generation of SDD order'
        print '          : 0  : Special Flags Order for use others heuristics tools (valid only for IDDMC and ZBDDMC)'
        print '          : 1  : PaToH 1-Hierarchy Order'
        print '          : 2  : IDD Pre-ordering + PaToH 1-Hierarchy'
        print '          : 3  : IDD Pre-ordering in FLAT'
        print '          : 4  : FLAT PaToH Order'
        print '          : 5  : PaToH 1-Hierarchy with SUCCESSOR-PREDECESSOR'
        print '          : 6  : PaToH XY-Hierarchy'
        print '          : 7  : Pre-Fired (PF-1) Model Flat + naive XY-Hierarchy'
        print '          : 8  : Pre-Fired (PF-2 Optimum Fired) Model Flat + naive XY-Hierarchy'
        print '          : 9  : Pre-Fired (PF-2 Optimum Fired) Model Flat + 1-Hierarchy build by Transition Variant 1'
        print '          : 10 : Pre-Fired (PF-2 Optimum Fired) Model Flat + 1-Hierarchy build by Transition Variant 2'
        print '          : 11 : Pre-Fired (PF-2 Optimum Fired + SPAN Weight) Model Flat + naive XY-Hierarchy'
        print '          : 12 : FORCE Algorithm Heuristic Flat order + naive XY-Hierarchy'
        print '          : 13 : Partially Identical Sequence Heuristic !!!'
        print '--porder  : Give a JSON Configuration SDD Order'
        print '--cami    : Input model CAMI Format'
        print '--atpn    : Input model ATPN Format'


def clean_file(filename):
    p1 = Process(None,'rm','rm',['-f',filename])
    p1.execute()

# Conversion of EndLine of MAC into Unix Mode
# Source : Source File
# Dest   : Destination File
def macunix_endline_conversion(source,dest,format):
    cami_file = open(source,"rU")
    cami_file_converted = open (dest,'w')
    for line in cami_file:
        if line.find('PT') == -1 and format == 'CAMI':
            cami_file_converted.write(line)
        elif format == 'ATPN':
            cami_file_converted.write(line)
    cami_file.close()
    cami_file_converted.close()