#!/usr/bin/env python

'''
Created on 16 sept. 2009

@author: hong
'''


from PROCESS.Process import *
from HEURISTICS.pycami.converter import *
from HEURISTICS.pycami.parser import *
from MEMORY.MemoryChecker import *
from UTILS.Utils import *



class ZBDDMC(Process):
    # Constructor of the PNSDD Application tools
    def __init__(self,
                api,
                host,
                mem_check,
                path_pnsdd,
                path_lib,
                path_model_ord,
                path_conf,
                library = 0,
                path_formula = None,
                format = 'CAMI'):
        self.temp_dir = ''
        if host != None:
            path_pnsdd = 'zbddmc'
        if mem_check.os_name == 'Darwin':
            ldpath = LDPATH_Darwin(host,path_lib)
        if mem_check.os_name == 'Linux':
            ldpath = LDPATH_Linux(host,path_lib)
        if format == 'CAMI':
            net = parse_cami(path_model_ord)
            ToAtpn(net,path_model_ord+'.idd')
            path_model_ord = path_model_ord + '.idd'
        # Build the argument command
        argument = []
        argument.append("--net-file="+path_model_ord)
        if library == 0:
            argument.append('--porder-read='+path_conf)
            argument.append('--place-order=7')
        else:
            argument.append('--place-order='+str(library))
        if path_formula != None:
            import os
            if os.path.getsize(path_formula) != 0:
                argument.append("--Ctl-file="+path_formula)
        
        # Constructor of Base
        Process.__init__(self,host,'zbddmc',path_pnsdd,argument,ldpath,mem_check,api)
        
    # Redefinition for add comments
    def start(self):
        # Execute PNSDD on local host
        Process.start(self)
        self.api.send_State('Execution of ZBDDMC Tools')

    # Operation POST SSH necessary if you want to clean the execution on remote host
    # After execution
    def post_ssh(self):
        # Remove the tempory directory
        p1 = Process(self.host,'rm','rm',['-rf' , self.temp_dir])
        p1.execute()
        
    # Operation PRE SSH necessary for prepare the execution on remote host
    def pre_ssh(self):
        # Create a unique tempory directory
        self.api.send_State('Create new directory on remote host')
        p1 = Process(self.host,'mktemp','mktemp',['-d -t iddmc'])
        p1.execute()
        self.temp_dir,ii = p1.process.communicate()
        self.temp_dir = self.temp_dir.replace('\n','')
        
        # Then copy the model
        self.api.send_State('Copy the petri net file on remote host')
        model_ord = None
        for word in self.argument[1].split('/'):
            model_ord = word
        p1 = Process(None,'scp','scp',[self.argument[1] + ' ' + self.host + ':' + self.temp_dir])
        p1.execute()
        self.argument[1] = self.temp_dir + '/' + model_ord
            
        # Then copy the JSON configuration file
        conf = None
        self.api.send_State('Copy the JSON Configuration file on remote host')
        for word in self.argument[3].split('/'):
            conf = word
        p1 = Process(None,'scp','scp',[self.argument[3] + ' ' + self.host + ':' + self.temp_dir])
        p1.execute()
        self.argument[3] = self.temp_dir + '/' + conf
        
        # Then copy the formula file if exists
        if self.argument[9] != "":
            formula = None
            self.api.send_State('Copy the CTL Formula file on remote host')
            for word in self.argument[9].split('/'):
                formula = word
            p1 = Process(None,'scp','scp',[self.argument[9] + ' ' + self.host + ':' + self.temp_dir])
            p1.execute()
            self.argument[9] = self.temp_dir + '/' + formula

    def __str__(self):
        res = ''
        res += self.states + '\t'
        res += self.sdd_final + '\t'
        res += self.unicity_sdd_peak + '\t'
        res += self.cache_sdd_peak + '\t'
        res += self.cache_hom_peak + '\t'
        res += self.unicity_hom_peak + '\t'
        res += self.time_generation
        return res


    def parseLineWithKey(self,line,key,end):
        i1 = line.find(key)
        if i1 != -1:
            if end != None:
                i2 = line.find(end,i1)
                if i2 == -1:
                    i2 = len(line)
            else:
                i2 = len(line)
            if i2 != -1:
                return line[i1+len(key):i2]
        return '0'
    # Parse the result command of IDDMC
    def parseResult(self):
        line_to_parse = False
        self.states = '0'
        self.sdd_final = '0'
        self.unicity_sdd_peak = '0'
        self.cache_sdd_peak = '0'
        self.cache_hom_peak = '0'
        self.unicity_hom_peak = '0'
        self.time_generation = '0'
        for line in self.buf.split('\n'):
            if line.find('#nodes') != -1:
                self.sdd_final = self.parseLineWithKey(line,'#nodes','(')
            if line.find('#states') != -1:
                self.states = self.parseLineWithKey(line,'#states','(')
                self.states = self.states.replace(',','')
            if line.find('nodes used:') != -1:
                self.unicity_sdd_peak = self.parseLineWithKey(line,'nodes used:','(')
            if line.find('total processing time:') != -1:
                self.time_generation = self.parseLineWithKey(line,'total processing time:',None)
    
    def end(self):
        Process.end(self)
        try:
            self.parseResult()
        except:
            pass
        # At the end check the status code
        if self.status_code != 0:
            self.api.send_Trace("Error during the execution of ZBDDMC Tools, maybe ZBDDMC have been reached the maximum bounded memory. Error CODE : " +str(self.status_code))
            self.api.send_Trace(" ")
        else:
            self.api.send_Trace("Maximum Memory Used (%)  : " + str((self.mem_check.checker.Mprocess*100)/(self.mem_check.checker.MaxMemory / (1024 * 1024))) + " %")
            self.api.send_Trace("Maximum Memory Used (MB) : " + str(self.mem_check.checker.Mprocess) + " MB")
            self.api.send_Trace("PNSDD is successfully terminated")
  