/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  absolute.c
*
* Part of: PROD's program absolute
*
* Author: Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     The "absolute" program produces "absolute.h". "absolute.h"
*     is needed for function declarations. If the PROTOZOA macro
*     is defined, then the prototype style is used. Otherwise the
*     old non-ANSI style is used.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>

#ifdef PROTOZOA
#define MAIN2() int main(int argc, char *argv[])
#else
#define MAIN2() int main(argc, argv) int argc; char *argv[];
#endif

MAIN2()
{
    unsigned long a = 0L;
    unsigned long n;
    unsigned long i;
    FILE *_out;
    FILE *_err;

    _out = (FILE *) stdout;
    _err = (FILE *) stderr;
    if ((argc != 2) || (sscanf(argv[1], "%lu", &a) != 1))
	{
	    (void) fprintf(_err, "\nUsage: %s number\n",
#ifdef ARGV0
			   argv[0]
#else
			   "absolute"
#endif
			   );
	    (void) fflush(_err);
	    return(1);
	}
    (void) fprintf(_out, "#ifdef PROTOZOA\n#define MAIN0() int main(void)\n#define MAIN2() int main(int _argc_, char *_argv_[])\n");
    n = 0L;
    while (1)
	{
	    (void) fprintf(_out, "#define D%lu(k0,n0", n);
	    if (n != 0L)
		{
		    i = 1L;
		    while (1)
			{
			    (void) fprintf(_out, ",k%lu,n%lu", i, i);
			    if (i == n)
				break;
			    i++;
			}
		}
	    (void) fprintf(_out, ") k0 n0(");
	    if (n != 0L)
		{
		    i = 1L;
		    while (1)
			{
			    (void) fprintf(_out, "k%lu n%lu", i, i);
			    if (i == n)
				break;
			    (void) fprintf(_out, ",");
			    i++;
			}
		}
	    else
		(void) fprintf(_out, "void");
	    (void) fprintf(_out, ")\n#define FE%lu(k0,n0", n);
	    if (n != 0L)
		{
		    i = 1L;
		    while (1)
			{
			    (void) fprintf(_out, ",k%lu,n%lu", i, i);
			    if (i == n)
				break;
			    i++;
			}
		}
	    (void) fprintf(_out, ") D%lu(k0,n0", n);
	    if (n != 0L)
		{
		    i = 1L;
		    while (1)
			{
			    (void) fprintf(_out, ",k%lu,n%lu", i, i);
			    if (i == n)
				break;
			    i++;
			}
		}
	    (void) fprintf(_out, ")\n");
	    if (n == a)
		break;
	    n++;
	}
    (void) fprintf(_out, "#else\n#define MAIN0() int main()\n#define MAIN2() int main(_argc_, _argv_) int _argc_; char *_argv_[];\n");
    n = 0L;
    while (1)
	{
	    (void) fprintf(_out, "#define D%lu(k0,n0", n);
	    if (n != 0L)
		{
		    i = 1L;
		    while (1)
			{
			    (void) fprintf(_out, ",k%lu,n%lu", i, i);
			    if (i == n)
				break;
			    i++;
			}
		}
	    (void) fprintf(_out, ") k0 n0()\n#define FE%lu(k0,n0", n);
	    if (n != 0L)
		{
		    i = 1L;
		    while (1)
			{
			    (void) fprintf(_out, ",k%lu,n%lu", i, i);
			    if (i == n)
				break;
			    i++;
			}
		}
	    (void) fprintf(_out, ") k0 n0(");
	    if (n != 0L)
		{
		    i = 1L;
		    while (1)
			{
			    (void) fprintf(_out, "n%lu", i);
			    if (i == n)
				break;
			    (void) fprintf(_out, ",");
			    i++;
			}
		}
	    (void) fprintf(_out, ")");
	    if (n != 0L)
		{
		    (void) fprintf(_out, " ");
		    i = 1L;
		    while (1)
			{
			    (void) fprintf(_out, "k%lu n%lu;", i, i);
			    if (i == n)
				break;
			    i++;
			}
		}
	    (void) fprintf(_out, "\n");
	    if (n == a)
		break;
	    n++;
	}
    (void) fprintf(_out, "#endif\n");
    n = 0L;
    while (1)
	{
	    (void) fprintf(_out, "#define DE%lu(k0,n0", n);
	    if (n != 0L)
		{
		    i = 1L;
		    while (1)
			{
			    (void) fprintf(_out, ",k%lu,n%lu", i, i);
			    if (i == n)
				break;
			    i++;
			}
		}
	    (void) fprintf(_out, ") extern D%lu(k0,n0", n);
	    if (n != 0L)
		{
		    i = 1L;
		    while (1)
			{
			    (void) fprintf(_out, ",k%lu,n%lu", i, i);
			    if (i == n)
				break;
			    i++;
			}
		}
	    (void) fprintf(_out, ");\n#define DS%lu(k0,n0", n);
	    if (n != 0L)
		{
		    i = 1L;
		    while (1)
			{
			    (void) fprintf(_out, ",k%lu,n%lu", i, i);
			    if (i == n)
				break;
			    i++;
			}
		}
	    (void) fprintf(_out, ") static D%lu(k0,n0", n);
	    if (n != 0L)
		{
		    i = 1L;
		    while (1)
			{
			    (void) fprintf(_out, ",k%lu,n%lu", i, i);
			    if (i == n)
				break;
			    i++;
			}
		}
	    (void) fprintf(_out, ");\n#define FS%lu(k0,n0", n);
	    if (n != 0L)
		{
		    i = 1L;
		    while (1)
			{
			    (void) fprintf(_out, ",k%lu,n%lu", i, i);
			    if (i == n)
				break;
			    i++;
			}
		}
	    (void) fprintf(_out, ") static FE%lu(k0,n0", n);
	    if (n != 0L)
		{
		    i = 1L;
		    while (1)
			{
			    (void) fprintf(_out, ",k%lu,n%lu", i, i);
			    if (i == n)
				break;
			    i++;
			}
		}
	    (void) fprintf(_out, ")\n");
	    if (n == a)
		break;
	    n++;
	}
    return(0);
}
