/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  alloc.c
*
* Part of: PROD's programs araprod, "gphgen", probe, prod, prpp,
*          strong, and t
*
* Author:  Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     The routines in this module take care of the allocation and
*     deallocation of memory.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include <string.h>
#ifdef UNIX
#include <sys/types.h>
#else
#ifdef TNIL
#include <sys/types.h>
#else
#include <sys\types.h>
#endif
#endif
#include <stdlib.h>
#include <malloc.h>
#include <errno.h>
#ifndef UNIX
#include <time.h>
#endif
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "error.h"

#ifdef exit
#undef exit
#endif
#define exit(x) Exit(x)

FILE *_in;  /* For default input, PROD uses _in which is
             * typically equal to stdin. */
FILE *_out; /* For typical non-error messages, PROD uses _out which is
             * typically equal to stdout. */
FILE *_err; /* For typical error messages, PROD uses _err which is
             * typically equal to stderr. */

#ifdef abort
#undef abort
#endif
FE0(void, Abort)
{
#ifdef UNIX
    abort();
#else
    Exit(1);
#endif
}
#define abort() Abort()

/***************************************************************************
*
* Function:  Pvoid
* Purpose:   conversion from a char pointer to a VOID pointer
* Arguments: pointer - a char pointer
* Returns:   the corresponding VOID pointer
* See also:
*
****************************************************************************
*/
FE1(VOID *, Pvoid,
    char *, pointer)
{
    return((VOID *) pointer);
}

/***************************************************************************
*
* Function:  FileNotFoundErrno
* Purpose:   This routine checks if the system error indicator indicates
*            that a requested file is not found.
* Arguments:
* Returns:   TRUE if the indicator indicates a "file not found" error,
*            and FALSE otherwise
* See also:
*
****************************************************************************
*/
FE0(BOOL, FileNotFoundErrno)
{
    return(((int) ENOENT) == ErrnoVal());
}

/***************************************************************************
*
* Function:  ErrnoOutput
* Purpose:   This routine displays the message associated with the
*            system error indicator.
* Arguments: file - the output file
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, ErrnoOutput,
    FILE *, file)
{
    int onrre;

    onrre = ErrnoVal();
    (void) fprintf(file, "Errno %d: %s\n", onrre, SysErrStr(onrre));
    (void) fflush(file);
}
 
/***************************************************************************
*
* Function:  ResetErr
* Purpose:   sets _err to stderr
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, ResetErr)
{
    _err = (FILE *) stderr;
}

/***************************************************************************
*
* Function:  SetIO
* Purpose:   sets _in, _out and _err to their default values
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, SetIO)
{
    _in = (FILE *) stdin;
    _out = (FILE *) stdout;
    ResetErr();
}

FE2(int, ScanLdu,
    char *, buffer,
    unsigned long *, ulPointer)
{
    register long *pointer;

    pointer = (long *) ulPointer;
    return((int) sscanf(buffer, "%ld", pointer));
}

FE3(BOOL, BruteReadln,
    char **, buffer,
    unsigned *, n,
    FILE *, file)
{
    char *s;
    unsigned i;
    int ch;
    BOOL rdStat;

    s = *buffer;
    i = 0;
    rdStat = TRUE;
    while (TRUE)
	{
	    if (i == *n)
		{
		    *n = ((*n) >> 1) + (*n) + 1;
		    *buffer = (char *) Reallocate(*buffer, *n);
		}
	    ch = fgetc(file);
	    if (((int) EOF) == ch)
		{
		    rdStat = FALSE;
		    if (((unsigned) 0) == i)
			break;
		    *s = '\n';
		}
	    else
		*s = (char) ch;
	    s++;
	    i++;
	    if ((!rdStat) || (((int) '\n') == ch))
		{
		    if (i == *n)
			{
			    *n = ((*n) >> 1) + (*n) + 1;
			    *buffer = (char *) Reallocate(*buffer, *n);
			}
		    break;
		}
	}
    *s = '\0';
    return(rdStat);
}

/***************************************************************************
*
* Function:  Etacolla
* Purpose:   This routine reserves a block of memory for a single element
*            or for an array of one-byte elements. The actual reservation
*            is done by malloc. If malloc fails, an error message is given
*            and the program is aborted. This is done because PROD has no
*            recovery strategy for unsuccessful reservations.
* Arguments: size - the number of bytes to be reserved
* Returns:   a pointer to the beginning of the reserved block
* See also:  malloc, YarraColla, Retacolla, Deallocate
*
****************************************************************************
*/
FE1(VOID *, Etacolla,
    unsigned, size)
{
    register VOID *pointer;

    if (((unsigned) 0) == size)
	return((VOID *) NULL);
    if (!(pointer = (VOID *) malloc((size_t) size)))
	{
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err, "\nmalloc(%u) failed\n", size);
	    (void) fflush(_err);
	    abort();
	}
    return(pointer);
}

/***************************************************************************
*
* Function:  YarraColla
* Purpose:   This routine reserves a block of memory for an array of
*            equally sized elements. The actual reservation is done by
*            calloc. If calloc fails, an error message is given and the
*            program is aborted. This is done because PROD has no recovery
*            strategy for unsuccessful reservations.
* Arguments: elCount - the number of elements in the array
*            elSize - the number of bytes in a single element
* Returns:   a pointer to the beginning of the reserved block
* See also:  calloc, Etacolla, Retacolla, Deallocate
*
****************************************************************************
*/
FE2(VOID *, YarraColla,
    unsigned, elCount,
    unsigned, elSize)
{
    register VOID *pointer;

    if ((((unsigned) 0) == elCount) || (((unsigned) 0) == elSize))
	return((VOID *) NULL);
    if (!(pointer = (VOID *) calloc((size_t) elCount, (size_t) elSize)))
	{
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err, "\ncalloc(%u, %u) failed\n", elCount, elSize);
	    (void) fflush(_err);
	    abort();
	}
    return(pointer);
}

/***************************************************************************
*
* Function:  Retacolla
* Purpose:   This routine reserves a block of memory, then copies the
*            contents of the given old block to the new block and finally
*            frees the old block. The actual work is done by realloc.
*            If realloc fails, an error message is given and the program
*            is aborted. This is done because PROD has no recovery strategy
*            for unsuccessful reservations.
* Arguments: oldPtr - a pointer to the beginning of the old block
*            size - the number of bytes in the new block
* Returns:   a pointer to the beginning of the new block
* See also:  realloc, Etacolla, YarraColla, Deallocate
*
****************************************************************************
*/
FE2(VOID *, Retacolla,
    char *, oldPtr,
    unsigned, size)
{
    register VOID *pointer;

    if (((unsigned) 0) == size)
	{
	    Deallocate(oldPtr);
	    return((VOID *) NULL);
	}
    if (!(pointer = (VOID *) realloc(oldPtr, (size_t) size)))
	{
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err, "\nrealloc(0x%lx, %u) failed\n",
			   (unsigned long) oldPtr, size);
	    (void) fflush(_err);
	    abort();
	}
    return(pointer);
}

/***************************************************************************
*
* Function:  Deallocate
* Purpose:   This routine frees the given block of memory. The actual
*            deallocation is done by "free". However, nothing is done
*            if the block starts at address 0. Note that a block reserved
*            by Etacolla, YarraColla or Retacolla is empty if it starts
*            at address 0.
* Arguments: oldPtr - a pointer to the beginning of the block
* Returns:
* See also:  free, Etacolla, YarraColla, Retacolla
*
****************************************************************************
*/
FE1(void, Deallocate,
    char *, oldPtr)
{
    if (oldPtr)
        free(oldPtr);
}

#ifndef UNIX
#ifdef HALLOC
FE2(VOID *, HyarraColla,
    unsigned long, elCount,
    unsigned, elSize)
{
    register VOID *pointer;

    if ((((unsigned long) 0L) == elCount) || (((unsigned) 0) == elSize))
	return((VOID *) NULL);
    if (!(pointer = (VOID *) _halloc((long) elCount, (size_t) elSize)))
	{
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err,
			   "\n_halloc(%lu, %u) failed\n", elCount, elSize);
	    (void) fflush(_err);
	    abort();
	}
    return(pointer);
}

FE1(void, Dehallocate,
    char *, oldPtr)
{
    if (oldPtr)
        _hfree(oldPtr);
}
#endif
#endif

/***************************************************************************
*
* Function:  FileName
* Purpose:   This routine forms a name for a file by adding a suffix
*            to a base name.
* Arguments: name - the base name
*            suffix - the suffix (without the preceding dot)
* Returns:   the formed name
* See also:
*
****************************************************************************
*/
FE2(char *, FileName,
    char *, name,
    char *, suffix)
{
    char *concatenation;
    
    concatenation = (char *) Allocate(strlen(name) + strlen(suffix) + 2);
    (void) sprintf(concatenation, "%s.%s", name, suffix);
    return(concatenation);
}

#ifndef UNIX
static BOOL clockOn = FALSE;
static time_t timeStart;
static int dupArgc = 0;
static char **dupArgv;

FE0(void, ClockOn)
{
    (void) time(&timeStart);
    clockOn = TRUE;
}

FE3(void, StoreCmd,
    int, argc,
    char *, cmdName,
    char **, argv)
{
    int i;

    if (0 >= argc)
        return;
    dupArgv = (char **) AllocArray(argc, sizeof(char *));
    dupArgv[0] = cmdName;
    for (i = 1; i < argc; i++)
        dupArgv[i] = StrCopy(argv[i]);
    dupArgc = argc;
}
#endif

#ifdef exit
#undef exit
#endif
/***************************************************************************
*
* Function:  Exit
* Purpose:   to exit from the program
* Arguments: status - the exit status
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, Exit,
    int, status)
{
#ifndef UNIX
    clock_t clockExit;
    time_t timeExit;
    char *d;
    double timeDiff;
    int i;

    if (clockOn)
        {
            clockExit = clock();
            (void) time(&timeExit);
            timeDiff = difftime(timeExit, timeStart);
            if (0 < dupArgc)
                {
                    (void) fprintf(_err, "\nWe ran \"%s", dupArgv[0]);
                    for (i = 1; i < dupArgc; i++)
                        (void) fprintf(_err, " %s", dupArgv[i]);
                    (void) fprintf(_err, "\"");
                }
            d = ctime(&timeStart);
            (void) fprintf(_err, "\nStarted %s", d);
            if ('\n' != d[strlen(d) - 1])
                (void) fprintf(_err, "\n");
            d = ctime(&timeExit);
            (void) fprintf(_err, "Ended   %s", d);
            if ('\n' != d[strlen(d) - 1])
                (void) fprintf(_err, "\n");
            (void) fprintf(_err, "Difference: %lu seconds\n",
                           (unsigned long) (timeDiff + (double) 0.5));
            (void) fprintf(_err, "Processor time used: %.3f seconds\n",
                           ((double) clockExit) / (double)
#ifdef CLOCKS_PER_SEC
                           CLOCKS_PER_SEC
#else
#ifdef CLK_TCK
                           CLK_TCK
#else
			   -1.0
#endif
#endif
                          );
            (void) fflush(_err);
        }
#endif
    exit(status);
}
#define exit(x) Exit(x)

FE0(int, O_n_e)
{
    return(1);
}

FE1(int, Z_e_r_o,
    int, n)
{
    return(n - n);
}
