/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  araprod.h
*
* Part of: PROD's programs araprod, "gphgen", probe, and strong
*
* Author:  Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     See araprod.c.
*
* Version history:
*     $Log$
***************************************************************************/

/* An action consists of a number, a name, a hiding flag and a list
 * of references to parts of LTS's associated with the action. */
typedef struct ACTION
{
    unsigned number;
    char *name;
    BOOL hidden;
    LIST *used;
} ACTION;

#define ActAlloc() ((ACTION *) Allocate(sizeof(ACTION)))
#define ActGNumber(action) ((action)->number)
#define ActANumber(action) (&((action)->number))
#define ActSNumber(action,_number) ((action)->number = (_number))
#define ActGName(action) ((action)->name)
#define ActAName(action) (&((action)->name))
#define ActSName(action,_name) ((action)->name = (_name))
#define ActHidden(action) ((action)->hidden)
#define ActHide(action) ((action)->hidden = TRUE)
#define ActDisplay(action) ((action)->hidden = FALSE)
#define ActGUsed(action) ((action)->used)
#define ActSUsed(action) ((action)->used = LstCreate(&lacTools))
#define ActDUsed(action) LstDestroy((action)->used, TRUE)

extern ACTION *_iAction; /* This is an invisible action for all LTS's.
                          * For compilation reasons, the actual
                          * declaration of this variable is in graph.c. */
#define IAction() _iAction
#define SetIAction(action) (_iAction = (action))

/* Elements of TRANS are called LTS transitions. Such transition
 * is associated with a certain action and a certain LTS and consists
 * of a so called source node and a set of target nodes, i.e. nodes that
 * are reachable from the source node by a single execution of the action.
 * Target nodes are kept in a tree if transB is TRUE, and otherwise
 * in a list. */
typedef struct TRANS
{
    DOMAIN source;
    union
	{
	    BALTREE *balTree;
	    LIST *list;
	} targets;
} TRANS;

#define TransAlloc() ((TRANS *) Allocate(sizeof(TRANS)))
#define TransGSource(trans) ((trans)->source)
#define TransASource(trans) (&((trans)->source))
#define TransSSource(trans,_source) ((trans)->source = (_source))
#define TransGBalTargets(trans) ((trans)->targets.balTree)
#define TransSBalTargets(trans) \
((trans)->targets.balTree = BalTrCreate(&domBalTools))
#define TransDBalTargets(trans) BalTrDestroy((trans)->targets.balTree, TRUE)
#define TransGLisTargets(trans) ((trans)->targets.list)
#define TransSLisTargets(trans,_list) ((trans)->targets.list = (_list))
#define TransDLisTargets(trans) LstDestroy((trans)->targets.list, TRUE)

/* An element of LACTION consists of an LTS number and a set of
 * LTS transitions associated with a certain action and the LTS.
 * An element of LACTION thus corresponds to that part of the LTS which is
 * associated with the action. LTS transitions are kept in a tree if
 * lacB is TRUE, and otherwise in a list. */
typedef struct LACTION
{
    DOMAIN system;
    union
	{
	    BALTREE *balTree;
	    LIST *list;
	} transitions;
} LACTION;

#define LacAlloc() ((LACTION *) Allocate(sizeof(LACTION)))
#define LacGSystem(laction) ((laction)->system)
#define LacASystem(laction) (&((laction)->system))
#define LacSSystem(laction,_system) ((laction)->system = (_system))
#define LacGBalTransitions(laction) ((laction)->transitions.balTree)
#define LacSBalTransitions(laction) \
((laction)->transitions.balTree = BalTrCreate(&transBalTools))
#define LacDBalTransitions(laction) \
BalTrDestroy((laction)->transitions.balTree, TRUE)
#define LacGLisTransitions(laction) ((laction)->transitions.list)
#define LacSLisTransitions(laction,_list) \
((laction)->transitions.list = (_list))
#define LacDLisTransitions(laction) \
LstDestroy((laction)->transitions.list, TRUE)

/* The elements of SYSOT are called flow components. A flow component
 * consists of an LTS number, an LTS transition and a target node of the
 * transition. */
typedef struct SYSOT
{
    DOMAIN system;
    TRANS *trans;
    DOMAIN target;
} SYSOT;

#define SysotAlloc() ((SYSOT *) Allocate(sizeof(SYSOT)))
#define SysotGSystem(sysot) ((sysot)->system)
#define SysotASystem(sysot) (&((sysot)->system))
#define SysotSSystem(sysot,_system) ((sysot)->system = (_system))
#define SysotGTrans(sysot) ((sysot)->trans)
#define SysotATrans(sysot) (&((sysot)->trans))
#define SysotSTrans(sysot,_trans) ((sysot)->trans = (_trans))
#define SysotGTarget(sysot) ((sysot)->target)
#define SysotATarget(sysot) (&((sysot)->target))
#define SysotSTarget(sysot,_target) ((sysot)->target = (_target))

extern BOOL lacB;   /* LTS transitions associated with an action
                     * are kept in a tree
                     * if lacB is TRUE, and otherwise in a list. */
extern BOOL transB; /* Target nodes of an LTS transition are kept in
                     * a tree if transB is TRUE, and otherwise in a list. */
extern BALTREE *actTree;     /* A tree of actions ordered by name. */
extern BALTREE *actNumTree;  /* A tree of actions ordered by number. */
extern TUPLE *arpDouble;     /* A binary tuple. */
extern TUPLE *arpSingle;     /* A unary tuple. */
extern PLC *arpPlc;          /* A place in a low level net. */
extern PLC *arp2Plc;         /* A place in a low level net. */
extern TRN *arpTrn;          /* A transition in a low level net. */
extern LIST *domList;        /* A list of elements of DOMAIN. */
extern LIST *transList;      /* A list of LTS transitions. */
extern ACTION **actTbl;      /* For each transition in a high level net,
                              * this table tells the corresponding action. */
extern DATATYPE *arpPointer; /* A pointer to an encoded instance of
                              * a transition in a high level net. */
extern BALTREE *flowTree;    /* A tree of flows. */
extern ACTION *arpAction;    /* An action. */
extern unsigned long arpCounter; /* A counter used, among other purposes,
                                  * in the generation of the description
                                  * of a high level net. */
extern BOOL goEnd;           /* A flag used in the generation of
                              * the description of a high level net. */
extern LIST *goList;         /* A list of numbers used in the generation of
                                the description of a high level net. */
extern DOMAIN arpMin;        /* The least node number (testers excluded).*/
extern DOMAIN arpMax;        /* The greatest node number (testers excluded). */
extern DOMAIN tstrSys;       /* The number of a tester LTS. */
extern DOMAIN tstrMin;       /* The least node number in the tester. */
extern DOMAIN tstrMax;       /* The greatest node number in the tester. */
extern LACTION *arpLaction;  /* A reference to the part of an LTS associated
                                with a certain action .*/
extern LIST *sysotList;      /* A flow, i.e. a list of flow components. */
extern LIST *sysot2List;     /* A flow. */
extern SYSOT *arpSysot;      /* A flow component, consisting of an LTS
                              * number, an LTS transition and a target node
                              * of the transition. */
extern unsigned arpHiTr;     /* The number of a transition in a high level
                              * net. */
extern unsigned arpIndex;    /* An index used for referring to variables
                              * of a transition in a high level net. */
extern TRANS *arpTrans;      /* An LTS transition. */
extern DOMAIN *arpTarget;    /* A pointer to a target node of an LTS
                              * transition. */
extern DOMAIN *arpVars;      /* An array of values of variables of
                              * a transition in a high level net. */
extern FILE *netFile;        /* A file for the description of a high level
                              * net. */
extern TOOLS transTools;
extern BALTOOLS transBalTools;
extern TOOLS lacTools;
extern TOOLS actTools;
extern BALTOOLS actBalTools;
extern BALTOOLS actNumBalTools;
extern TOOLS sysotTools;
extern BALTOOLS flowBalTools;

DE1(TRANS *, TransCons,
    DOMAIN, source)
DE1(void, TransDiscard,
    TRANS *, trans)
#ifdef DEBUG
DE2(void, TransDump,
    TRANS *, trans,
    unsigned, depth)
#endif
DE2(int, TransKeyCmp,
    TRANS *, trans,
    DOMAIN *, key)
DE1(BOOL, DomTrLst,
    DOMAIN *, n)
DE1(BOOL, TransTrLst,
    TRANS *, trans)
DE1(LACTION *, LacCons,
    DOMAIN, system)
DE1(void, LacDiscard,
    LACTION *, laction)
#ifdef DEBUG
DE2(void, LacDump,
    LACTION *, laction,
    unsigned, depth)
#endif
DE3(ACTION *, ActCons,
    unsigned, number,
    char *, name,
    BOOL, hidden)
DE1(void, ActDiscard,
    ACTION *, action)
#ifdef DEBUG
DE2(void, ActDump,
    ACTION *, action,
    unsigned, depth)
#endif
DE2(int, ActKeyCmp,
    ACTION *, action,
    char *, key)
DE2(int, ActNumKeyCmp,
    ACTION *, action,
    unsigned *, key)
DE1(BOOL, ActTrLst,
    ACTION *, action)
DE3(SYSOT *, SysotCons,
    DOMAIN, system,
    TRANS *, trans,
    DOMAIN, target)
DE1(SYSOT *, SysotCopy,
    SYSOT *, sysot)
#ifdef DEBUG
DE2(void, SysotDump,
    SYSOT *, sysot,
    unsigned, depth)
#endif
DE2(int, SysotCompare,
    SYSOT *, sysot1,
    SYSOT *, sysot2)
DE2(int, FlowCompare,
    LIST *, flow1,
    LIST *, flow2)
DE2(void, HiTrNmOutput,
    char *, name,
    unsigned, ind)
DE3(void, HiTrNmWrite,
    char *, name,
    unsigned, ind,
    BINARYFILE, bFile)
DE3(void, HiVarWrite,
    char *, name,
    unsigned long, ind,
    BINARYFILE, bFile)
DE1(BOOL, PlcMon,
    PLC *, plc)
DE1(void, DatOutput,
    BINARYFILE, bFile)
DE0(void, TauInstances)
DE0(void, TstrTauInstances)
DE1(void, Accepted,
    VOID *, sysotCursor)
DE1(void, Instances,
    VOID *, lacCursor)
DE4(void, ReadLTS,
    char *, name,
    DOMAIN, number,
    FILE *, file,
    BOOL, tstr)
DE2(void, ReadMonitor,
    char *, name,
    FILE *, file)
DE2(void, ReadHidden,
    char *, name,
    FILE *, file)
DE1(BOOL, TrnFold,
    TRN *, trn)
DE0(void, TreeTrnFold)

/* The NetOutput macro is used for outputting the description of a high
 * level net. However, output is produced only if netFile is not NULL. */
#define NetOutput if (!netFile) ; else (void) fprintf
