/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  cmdnm.c
*
* Part of: PROD's programs araprod, "gphgen", probe, prod, prpp,
*          and strong
*
* Authors: Lasse Anderson, Robert Jurgens and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     The motivation of this module is as follows. In an error message,
*     it is often good to mention the name of the complaining program.
*     Moreover, it is often best to show the name without any directory
*     part, drive identifier or ".exe" suffix.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "error.h"
#include "cmdnm.h"

char *myName;   /* the name of the program that is being run */
char *baseName; /* like myName but fixed in the code */

/***************************************************************************
*
* Function:  CmdNm
* Purpose:   To get a suitable myName, this routine is needed. (See the
*            "Description:" in the beginning of this file.)
* Arguments: name - the name of the program in the form that was used
*                   in the command line
* Returns:   the essential part of the name of the program
* See also:
*
****************************************************************************
*/
FE1(char *, CmdNm,
    char *, name)
{
    unsigned i, j, nullInd;
    char *cmdName;

    i = 0;
    while (name[i] != '\0')
	i++;
    nullInd = i;
#ifdef UNIX
    while ((i != 0) && (name[i - 1] != DIRCHAR))
	i--;
#else
    while ((i != 0) && (name[i - 1] != DIRCHAR) && (name[i - 1] != ':'))
	i--;
    j = nullInd;
    while ((j != i) && (name[j - 1] != '.'))
	j--;
    if (j != i)
	nullInd = j - 1;
#endif
    cmdName = (char *) Allocate(nullInd - i + 1);
    j = i;
    while (j != nullInd)
	{
#ifdef UNIX
	    cmdName[j - i] = name[j];
#else
	    if (isalpha((UCHAR) (name[j])) && isupper((UCHAR) (name[j])))
		cmdName[j - i] = tolower((UCHAR) (name[j]));
	    else
		cmdName[j - i] = name[j];
#endif
	    j++;
	}
    cmdName[nullInd - i] = '\0';
    return(cmdName);
}

/***************************************************************************
*
* Function:  OpenText
* Purpose:   This routine opens an ordinary text file.
* Arguments: subdir - a potential subdirectory containing the file
*            name - the name of the file
*            skip - when TRUE, the current working directory is skipped
*            pseudo - when TRUE, the file is binary!!!
* Returns:   the file
* See also:
*
****************************************************************************
*/
FE4(FILE *, OpenText,
    char *, subdir,
    char *, name,
    BOOL, skip,
    BOOL, pseudo)
{
    FILE *file = NULL;
    char *primName = NULL, *directory, *scndName = NULL;
    unsigned len = 0;
    char opmode[3];

    if (pseudo)
	(void) strcpy(opmode, "rb");
    else
	(void) strcpy(opmode, "r");
    if (!name)
	{
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err, "\nNo name in OpenText\n");
	    (void) fflush(_err);
	    abort();
	}
    directory = subdir;
    if (directory && *directory)
	{
	    len = (unsigned) strlen(directory);
	    if ((DIRCHAR == directory[len - 1]) && (DIRCHAR == *name))
		{
		    if (1 == len)
			primName = name;
		    else
			{
			    primName = (char *) Allocate(len + (unsigned)
							 strlen(name));
			    (void) sprintf(primName, "%s%s",
					   directory, name + 1);
			}
		}
	    else if ((DIRCHAR == directory[len - 1]) || (DIRCHAR == *name))
		{
		    primName = (char *) Allocate(len + (unsigned)
						 (strlen(name) + 1));
		    (void) sprintf(primName, "%s%s", directory, name);
		}
	    else
		{
		    primName = (char *) Allocate(len + (unsigned)
						 (strlen(name) + 2));
		    (void) sprintf(primName, "%s%c%s", 
				   directory, DIRCHAR, name);
		}
	}
    else
	primName = name;
    directory = NULL;
    if (skip || !(file = fopen(primName, opmode)))
	{
	    if (skip || FileNotFoundErrno())
		{
		    if ((directory = getenv("PRODPATH")) && !skip)
			ClearErrno();
		}
	    if (directory && *directory)
		{
		    len = (unsigned) strlen(directory);
		    if ((DIRCHAR == directory[len - 1]) &&
			(DIRCHAR == *primName))
			{
			    if (1 == len)
				scndName = primName;
			    else
				{
				    scndName= (char *)
					Allocate(len + (unsigned)
						 strlen(primName));
				    (void) sprintf(scndName, "%s%s",
						   directory, primName + 1);
				}
			}
		    else if ((DIRCHAR == directory[len - 1]) ||
			     (DIRCHAR == *primName))
			{
			    scndName = (char *)
				Allocate(len + (unsigned)
					 (strlen(primName) + 1));
			    (void) sprintf(scndName, "%s%s",
					   directory, primName);
			}
		    else
			{
			    scndName = (char *)
				Allocate(len + (unsigned)
					 (strlen(primName) + 2));
			    (void) sprintf(scndName, "%s%c%s", 
					   directory, DIRCHAR, primName);
			}
		}
	    else
		scndName = primName;
	    if (((scndName == primName) && !skip) ||
		 !(file = fopen(scndName, opmode)))
		{
		    (void) fprintf(_err, 
				   "\nCannot open file %s for reading\n",
				   scndName);
		    ErrnoOutput(_err);
		    ClearErrno();
		}
	    if (scndName != primName)
		Deallocate(scndName);
	}
    if (primName != name)
	Deallocate(primName);
    return(file);
}

FE4(void, ExpArgs,
    int, _argc_,
    char **, _argv_,
    int *, argc,
    char ***, argv)
{
    LIST **arguments;
    int i, k;
    unsigned n;
    char *s, *u, *p, *q, *r, c;
    FILE *fpi;
    BOOL rdStat;

    *argc = _argc_;
    *argv = _argv_;
    if (2 > _argc_)
	return;
    s = FileName(baseName, "cfg");
    fpi = OpenText("cfg", s, TRUE, FALSE);
    Deallocate(s);
    if (!fpi)
	return;
    arguments = NULL;
    n = (unsigned) BUFSIZ;
    s = (char *) Allocate(n);
    while (TRUE)
	{
	    rdStat = BruteReadln(&s, &n, fpi);
	    if ((!rdStat) && !*s)
		break;
	    s[n - 1] = '\0';
	    for (p = s; (*p) && ('#' != *p) && isspace((UCHAR) *p); p++)
		;
	    if ((!*p) || ('#' == *p))
		{
		    if (!rdStat)
			break;
		    continue;
		}
	    u = p;
	    for (q = p++; (*p) && ('#' != *p); p++)
		if (!isspace((UCHAR) *p))
		    q = p;
	    *(q + 1) = '\0';
	    for (p = u; (*p) && !isspace((UCHAR) *p); p++)
		;
	    if (*p)
		for(*(p++) = '\0'; (*p) && isspace((UCHAR) *p); p++)
			;
	    for (i = 1; i < _argc_; i++) 
		if (((!arguments) || !arguments[i - 1]) &&
		    ('-' == _argv_[i][0]) && ('-' == _argv_[i][1]) &&
		    !strcmp(_argv_[i] + 2, u))
		    {
			if (!arguments)
			    {
				arguments = (LIST **)
				    AllocArray(_argc_ - 1, sizeof(LIST *));
				for (k = 1; k < _argc_; k++)
				    arguments[k - 1] = NULL;
			    }
			arguments[i - 1] = LstCreate(&strTools);
			q = p;
			do
			    {
				for (r = q++; (*q) && !isspace((UCHAR) *q);
				     q++)
				    ;
				c = *q;
				*q = '\0';
				if (*r)
				    (void) LstInsLast(arguments[i - 1],
						      StrCopy(r));
				*q = c;
				if (c)
				    for (q++; (*q) && isspace((UCHAR) *q); q++)
					;
			    } while (c);
		    }
	    if (!rdStat)
		break;
	}
    Deallocate(s);
    (void) fclose(fpi);
    if (!arguments)
	return;
    for (i = 1, k = 1; i < _argc_; i++)
	if (arguments[i - 1])
	    k += LstSize(arguments[i - 1]);
	else
	    k++;
    *argc = k;
    *argv = (char **) AllocArray(k, sizeof(char *));
    (*argv)[0] = StrCopy(_argv_[0]);
    for (i = 1, k = 1; i < _argc_; i++)
	{
	    if (arguments[i - 1])
		{
		    LSTFOR (arguments[i - 1], s, char)
			(*argv)[k++] = s;
		    LstDestroy(arguments[i - 1], FALSE);
		}
	    else
		(*argv)[k++] = StrCopy(_argv_[i]);
	}
    Deallocate((char *) arguments);
}

FE2(void, MsgExit,
    char *, escOptions,
    int, status)
{
    char *s;
    unsigned n;
    FILE *fpi;
    BOOL rdStat;

    s = FileName(baseName, "msg");
    fpi = OpenText("cfg", s, TRUE, FALSE);
    Deallocate(s);
    (void) fprintf(_err, "\nUsage:  %s ", myName);
    if (fpi)
	{
	    n = (unsigned) BUFSIZ;
	    s = (char *) Allocate(n);
	    while (TRUE)
		{
		    rdStat = BruteReadln(&s, &n, fpi);
		    if ((!rdStat) && !*s)
			break;
		    (void) fputs(s, _err);
		    if (!rdStat)
			break;
		}
	    Deallocate(s);
	    (void) fclose(fpi);
	}
    else
	(void) fprintf(_err, "%s\n", escOptions);
    (void) fflush(_err);
    Exit(status);
}
