/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  comline.h
*
* Part of: PROD's program probe
*
* Authors: Keijo Heljanko, Tomi Janhunen, Kimmo Valkealahti,
*          and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     See comline.c.
*
* Version history:
*     $Log$
***************************************************************************/

#ifndef COMLINE_H
#define COMLINE_H

typedef struct _setOper 
{
    unsigned operator;
    union 
	{
	    EXPRESSION *expression;
	    struct 
		{
		    struct _setOper *sub1s, *sub2s;
		} os;
	} us;
} SETOPER;

typedef struct _comline 
{
    unsigned operator, options;
    union 
	{
	    EXPRESSION *expression;
	    MARKEXPR *markexpr;
	    FORMULA *formula;
	    SETOPER *setOper;
	} ul;
    EXPRESSION *expr2;
} COMLINE;

# define LI_PREV 0
# define LI_CALC 1
# define LI_CLEAR 2
# define LI_CLEARALL 3
# define LI_BUILD 4
# define LI_GOTO 6
# define LI_LOOK 7
# define LI_PRED 8
# define LI_QUERY 9
# define LI_QUIT 10
# define LI_SETS 11
# define LI_SHRINK 12
# define LI_STATISTICS 13
# define LI_NEXT 14
# define LI_SUCC 15
# define LI_VERBOSITY 16
# define LI_SGSUCC 17
# define LI_SGPRED 18
# define LI_TERMCOMPS 19
# define LI_ALLCOMPS 20
# define LI_SHOWENDS 21
# define LI_CHECK 22

# define TG_NONE 0
# define TG_MUTE 1
# define TG_VERBOSE 2
# define TG_SUPERVERBOSE 4
# define TG_VOLATILE 8
# define TG_PREDS 16
# define TG_SUCCS 32
# define TG_MARKING 1

#define SE_SET 1
#define SE_COMP 2
#define SE_UNION 3
#define SE_CUT 4
#define SE_MINUS 5

#define CmlAlloc() ((COMLINE *) Allocate(sizeof(COMLINE)))
#define SopAlloc() ((SETOPER *) Allocate(sizeof(SETOPER)))

typedef struct _redirect
{
    unsigned type;
    char *name;
    char *args;
} REDIRECT;

#define REDIRECT_NONE 0
#define REDIRECT_FILE 1
#define REDIRECT_FILE_APPEND 2
#define REDIRECT_FILTER 3

extern unsigned symCount;
extern char **plaNames, **traNames, ***traVarNames, **symNames, **priNames;
extern BALTREE *resTree, *plaTree, *traTree, **traVarTrees, *symTree, *priTree;
extern MBUF *ioBuf;
extern MBUF *evalBuf;
extern BOOL evalStuff;
extern BOOL transform;
extern unsigned long lineLength;

DE0(void, MacpParse)
DE0(void, MacroEval)
DE1(COMLINE *, CmlParse,
    REDIRECT *, redirect)
DE2(void, PrtCaret,
    unsigned, count,
    unsigned, textLen)
DE1(void, CLineInit,
    BINARYFILE, datFile)
#ifdef USE_READLINE
DE2(void, CLine,
    FILE *, inFile,
    char *, prompt)
#else
DE1(void, CLine,
    FILE *, inFile)
#endif
DE1(char *, LngToPth,
    unsigned long *, lng)
DE1(BALTREE *, SopEval,
    SETOPER *, setOper)
DE2(BOOL, SopIn,
    SETOPER *, setOper,
    unsigned long, world)
DE3(SETOPER *, SopCons,
    unsigned, operator,
    SETOPER *, sop1,
    SETOPER *, sop2)
DE3(SETOPER *, SopAux,
    unsigned, operator,
    SETOPER *, sop1,
    SETOPER *, sop2)
DE1(SETOPER *, SopCopy,
    SETOPER *, setOper)
DE1(void, SopDiscard,
    SETOPER *, setOper)
#ifdef DEBUG
DE2(void, SopDump,
    SETOPER *, setOper,
    unsigned, depth)
#endif
DE4(COMLINE *, CmlCons,
    unsigned, operator,
    unsigned, options,
    EXPRESSION *, ep1,
    EXPRESSION *, ep2)
DE4(COMLINE *, CmlAux,
    unsigned, operator,
    unsigned, options,
    EXPRESSION *, ep1,
    EXPRESSION *, ep2)
DE1(void, CmlDiscard,
    COMLINE *, comline)
#ifdef DEBUG
DE2(void, CmlDump,
    COMLINE *, comline,
    unsigned, depth)
#endif
DE2(void, PthOutput,
    PATH *, path,
    FILE *, f)
DE2(void, NdOutput,
    unsigned long, nd,
    FILE *, f)
DE6(void, AwSgOutput,
    unsigned long, ordfa,
    unsigned long, compfa,
    unsigned long, aw,
    unsigned long, ordson,
    unsigned long, compson,
    FILE *, f)
DE2(void, NamSgOutput,
    unsigned long, sg,
    FILE *, f)
DE2(void, DomOutput,
    DOMAIN, n,
    FILE *, f)
DE2(void, PriOutput,
    unsigned, n,
    FILE *, f)
DE2(void, MarOutput,
    MARKING *, marking,
    FILE *, f)
DE2(void, WldVerbose,
    unsigned long, world,
    FILE *, f)
DE3(void, ArwVerbose,
    unsigned long, awNr,
    unsigned long, awAddr,
    FILE *, f)
DE1(void, CutVerbose,
    FILE *, f)
DE2(void, PthVerbose,
    PATH *, path,
    FILE *, f)
DE2(void, BStaDecode,
    DATATYPE *, buffer,
    STATE *, state)
DE4(DOMAIN *, BInsDecode,
    unsigned *, trIndex,
    unsigned *, priority,
    BOOL *, faithful,
    DATATYPE **, argPointer)

#endif
