/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  encode.c
*
* Part of: PROD's programs araprod, "gphgen", probe, and prpp
*
* Authors: Marko Rauhamaa and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     This module contains routines for encoding of markings (and such)
*     and decoding of them.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#ifdef DEBUG
#include <string.h>
#endif
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ptrbl.h"
#include "baltree.h"
#include "gphutil.h"
#include "mark.h"
#include "encode.h"

unsigned _ingNofClasses;     /* the number of precedence classes */
unsigned _ingNofTransitions; /* the number of transitions */
unsigned *_ingProfile;       /* _ingProfile[i] is the number of variables
                              * of the (i + 1)th transition */
unsigned *_ingFdProfile;     /* _ingFdProfile[i] is the number of so called
                              * folding variables of the (i + 1)th
                              * transition */
BALTREE **_ingRange;          /* _ingRange[i] contains the range of the 
                              * "let" variables of the (i + 1)th
                              * transition */
BOOL _elimination;           /* "intermediate states" may be eliminated if
                              * this is TRUE */
BOOL _depthFirst;            /* reachability graph generation is performed
                              * in a depth first order if this is TRUE,
                              * and in a breadth first order otherwise */
BOOL _freeExpansion;         /* this is TRUE if and only if _depthFirst is
                              * TRUE and the successor information of a node
                              * can be updated arbitrarily many times */
unsigned long _onTheFly;     /* this indicates if or how on-the-fly
                              * verification is performed */
DOMAIN **_x_;                /* _x_[i][j] is the (j + 1)th successor state
                              * of the (i + 1)th state of a Buchi automaton
                              * (a NULL pointer closes the _x_[i] array) */
unsigned long *_z_;          /* _z_[i] is the label of the (i + 1)th state
                              * of a Buchi automaton (all outgoing arcs
                              * have this label) */
BOOL _pep;                   /* a PEP low-level net file is produced
                              * if this is true */
FILE *_pepFile;              /* a file for the PEP low-level net */
BOOL _ara;                   /* an ARA LTS is produced if this is TRUE */
FILE *_araFile;              /* a file for the ARA LTS */
FILE *_araArcFile;           /* a temporary arc file for the ARA LTS */
char **_araTraNames = NULL;  /* _araTraNames[i] is the name of the (i + 1)th
                              * transition */
char ***_araVarNames = NULL; /* _araVarNames[i][j] is the name of the
                              * (j + 1)th variable of the (i + 1)th
                              * transition */
unsigned **_araIndices = NULL;  /* if _araVarNames[i][j] is alphabetically
                                 * the (k + 1)th variable of the (i + 1)th
                                 * transition, then _araIndices[i][k] == j */
unsigned _araSymCount;       /* the number of enum symbols */
char **_araSymNames = NULL;  /* _araSymNames[i] is the name of the (i + 1)th
                              * enum symbol */
char **_araPriNames = NULL;  /* _araPriNames[i] is the name of the precedence
                              * class corresponding to the number i + 1 */
BALTREE *_araTree;           /* a tree of transition instances and the
                              * corresponding action numbers for the
                              * ARA LTS */
char **_plaNames = NULL;     /* _plaNames[i] is the name of the (i + 1)th
                              * place */
PATH *_path[4];              /* paths in a reachability graph */
BALTREE *_nodeTree;          /* a tree of nodes, used for detecting
                              * a loop in a search path */
STATE *_state;               /* the current marking of the net */
STATE *_folState;            /* a computed successor marking of _state */
STATE *_evalState;           /* a marking of the net to be compared to
                              * the label of a state of a Buchi automaton */
BOOL _new;                   /* used for indicating whether an element
                              * that was to be inserted to a structure
                              * was new */
unsigned long _back;
unsigned _engBfSize;         /* the size of _engBuffer */
DATATYPE *_engBuffer;        /* an encoding buffer */
BOOL _dots;                  /* if this is TRUE, dots are printed along
                              * the generation of a reachability graph */
int _inner;                  /* a row counter in dot output */
int _outer;                  /* a column counter in dot output */
unsigned _monPlInd;          /* the index of the tester place (-1 if there
                              * is no tester) */

/***************************************************************************
*
* Function:  BaseInit
* Purpose:   This routine reads net information from a binary file.
* Arguments: file - the file
*            readRanges - indicates if the ranges of "let" variables
*                         are to be read
* Returns:
* See also:  pplst_dat (pplst.c), BaseLTSInit, InitialState,
*            PtInit (ptelem.c), AraNames, RdPlaNames
*
****************************************************************************
*/
FE2(void, BaseInit,
    BINARYFILE, file,
    BOOL, readRanges)
{
    unsigned p;
    unsigned t;
    unsigned len = 0;
    unsigned i;
    DATATYPE *buffer, *pointer;

    ReadUns(&_monPlInd, 1, file);
    if (_monPlInd != (unsigned) -1)
	SetOnTheFly();
    ReadUns(&_ingNofClasses, 1, file);
    ReadUns(&len, 1, file);
    buffer = (DATATYPE *) AllocArray(len + 1, sizeof(DATATYPE));
    ReadData(buffer, len, file);
    pointer = buffer;
    GetValue(unsigned, i, pointer);
    if ((_magNofPlaces = i))
	_magProfile = (unsigned *) AllocArray(i, sizeof(unsigned));
    else
	_magProfile = (unsigned *) NULL;
    for (p = 0; p < MagGNofPlaces(); p++)
	{
	    GetValue(unsigned, i, pointer);
	    MagIProfile(p, i);
	}
    Deallocate((char *) buffer);
    ReadUns(&len, 1, file);
    buffer = (DATATYPE *) AllocArray(len + 1, sizeof(DATATYPE));
    ReadData(buffer, len, file);
    pointer = buffer;
    GetValue(unsigned, i, pointer);
    if ((_ingNofTransitions = i))
	{
	    _ingProfile = (unsigned *) AllocArray(i, sizeof(unsigned));
	    _ingFdProfile = (unsigned *) AllocArray(i, sizeof(unsigned));
	}
    else
	{
	    _ingProfile = (unsigned *) NULL;
	    _ingFdProfile = (unsigned *) NULL;
	}
    for (t = 0; t < IngGNofTransitions(); t++)
	{
	    GetValue(unsigned, i, pointer);
	    IngIProfile(t, i);
	}
    Deallocate((char *) buffer);
    ReadUns(&len, 1, file);
    buffer = (DATATYPE *) AllocArray(len + 1, sizeof(DATATYPE));
    ReadData(buffer, len, file);
    pointer = buffer;
    for (t = 0; t < IngGNofTransitions(); t++)
	{
	    GetValue(unsigned, i, pointer);
	    IngIFdProfile(t, i);
	}
    Deallocate((char *) buffer);
    if (!readRanges)
        return;
    if (IngGNofTransitions())
	_ingRange = (BALTREE **)
	    AllocArray(IngGNofTransitions(), sizeof(BALTREE *));
    else
	_ingRange = (BALTREE **) NULL;
    for (t = 0; t < IngGNofTransitions(); t++)
        {
            ReadUns(&len, 1, file);
            if (len == 0)
                IngIRange(t, (BALTREE *) NULL);
            else
                {
                    IngIRange(t, BalTrCreate(&dataBalTools));
                    do
                        {
                            buffer = (DATATYPE *)
                                AllocArray(len + 1, sizeof(DATATYPE));
                            ReadData(buffer, len, file);
                            (void) BalTrInsert(IngGRange(t),
                                               (char *) buffer,
                                               (char *) buffer, TRUE);
                            ReadUns(&len, 1, file);
                        } while (len != 0);
                }
        }
}

/***************************************************************************
*
* Function:  BaseLTSInit
* Purpose:   This routine replaces BaseInit when the araprod program
*            is run.
* Arguments: nHiPlc - the number of places in the net
*            nHiTr - the number of transitions
* Returns:
* See also:  BaseInit, main (araprod.c)
*
****************************************************************************
*/
FE2(void, BaseLTSInit,
    unsigned, nHiPlc,
    unsigned, nHiTr)
{
    _magNofPlaces = nHiPlc;
    if (nHiPlc)
	_magProfile = (unsigned *) AllocArray(nHiPlc, sizeof(unsigned));
    else
	_magProfile = (unsigned *) NULL;
    _ingNofClasses = 1;
    if ((_ingNofTransitions = nHiTr))
	_ingProfile = (unsigned *) AllocArray(nHiTr, sizeof(unsigned));
    else
	_ingProfile = (unsigned *) NULL;
}

/***************************************************************************
*
* Function:  InitialState
* Purpose:   This routine reads the initial marking of a net from
*            a binary file. Possible information about a tester
*            or a Buchi automaton is read, too.
* Arguments: file -  the file
* Returns:   the initial marking
* See also:  pplst_dat (pplst.c), BaseInit, PtInit (ptelem.c), AraNames,
*            RdPlaNames
*
****************************************************************************
*/
FE1(STATE *, InitialState,
    BINARYFILE, file)
{
    STATE *state;
    unsigned p;
    unsigned q;
    unsigned r;
    unsigned len = 0;
    TUPLE *tuple;
    DATATYPE *buffer;
    register DATATYPE *pointer;
    register unsigned i;
    register unsigned long count;
    register UDOMAIN value;
    DOMAIN monState;
    MON *mon;

    state = StaAlloc();
    for (p = 0; p < MagGNofPlaces(); p++)
	StaIMarking(state, p, MarCreate());
    while (TRUE)
	{
	    ReadUns(&len, 1, file);
	    if (len == (unsigned) -1)
		break;
	    buffer = (DATATYPE *) AllocArray(len + 1, sizeof(DATATYPE));
	    ReadData(buffer, len, file);
	    pointer = buffer;
	    GetValue(unsigned, i, pointer);
	    p = i;
	    if (AriAt(pointer))
		{
		    AriPass(pointer);
		    GetValue(unsigned, i, pointer);
		    tuple = TupCons(i);
		}
	    else
		tuple = TupCons(MagGProfile(p));
	    if (TupGArity(tuple))
		{
		    for (i = 0; i < TupGArity(tuple); i++)
			{
			    GetValue(UDOMAIN, value, pointer);
			    TupSField(tuple, i, Decode((DOMAIN) value));
			}
		    if (CntAt(pointer))
			{
			    CntPass(pointer);
			    GetValue(unsigned long, count, pointer);
			    MarInsLast(StaGMarking(state, p),
				       MultupCons(count + 2L, tuple));
			}
		    else
			MarInsLast(StaGMarking(state, p),
				   MultupCons((unsigned long) 1L, tuple));
		}
	    else
		{
		    GetValue(unsigned long, count, pointer);
		    MarInsLast(StaGMarking(state, p),
			       MultupCons(count + 1L, tuple));
		}
	    Deallocate((char *) buffer);
	}
    monitors = NULL;
    if (!OnTheFly())
	return(state);
    while (TRUE)
	{
	    ReadUns(&len, 1, file);
	    if (len == (unsigned) -1)
		break;
	    if (!monitors)
		monitors = BalTrCreate(&monBalTools);
	    buffer = (DATATYPE *) AllocArray(len + 1, sizeof(DATATYPE));
	    ReadData(buffer, len, file);
	    pointer = buffer;
	    GetValue(UDOMAIN, value, pointer);
	    monState = Decode((DOMAIN) value);
	    mon = (MON *) BalTrElement(monitors, (char *) &monState);
	    if (!mon)
		{
		    mon = MonCons(monState);
		    (void) BalTrInsert(monitors, (char *) mon,
				       (char *) &monState, TRUE);
		}
	    GetValue(unsigned, i, pointer);
	    MonSFlag(mon, i);
	    if (MonReject(mon))
		SetReject();
	    if (MonDeadlock(mon))
		SetStabFail();
	    if (MonLivelock(mon))
		SetDivergence();
	    if (MonInfinite(mon))
		SetInfinite();
	    Deallocate((char *) buffer);
	}
    ReadUns(&p, 1, file);
    if (!p)
	return(state);
    SetBuchi();
    _x_ = (DOMAIN **) AllocArray(p, sizeof(DOMAIN *));
    for (q = 0; q < p; q++)
	{
	    ReadUns(&len, 1, file);
	    buffer = (DATATYPE *) AllocArray(len + 1, sizeof(DATATYPE));
	    ReadData(buffer, len, file);
	    pointer = buffer;
	    GetValue(unsigned, i, pointer);
	    r = i;
	    _x_[q] = (DOMAIN *) AllocArray(r + 1, sizeof(DOMAIN));
	    for (i = 0; i < r; i++)
		{
		    GetValue(UDOMAIN, value, pointer);
		    _x_[q][i] = Decode((DOMAIN) value);
		}
	    _x_[q][r] = (DOMAIN) 0;
	    Deallocate((char *) buffer);
	}
    p--;
    if (p)
	_z_ = (unsigned long *) AllocArray(p, sizeof(unsigned long));
    else
	_z_ = (unsigned long *) NULL;
    for (q = 0; q < p; q++)
	ReadLng(_z_ + q, 1, file);
    return(state);
}

/***************************************************************************
*
* Function:  InRange
* Purpose:   This routine checks if the "let" variables of a transition
*            are in the specified range.
* Arguments: t - the transition
*            n - the number of "let" variables
*            a - a value combination of the "let" variables
*            buffer - a buffer (extended if needed)
*            size - the size of the buffer
* Returns:   TRUE if the variables are in the range, and otherwise FALSE
* See also:  TravRange
*
****************************************************************************
*/
FE5(BOOL, InRange,
    unsigned, t,
    unsigned, n,
    DOMAIN *, a,
    DATATYPE **, buffer,
    unsigned *, size)
{
    register DATATYPE *pointer;
    register unsigned i, len;
    register UDOMAIN value;

    if ((!n) || !IngGRange(t))
        {
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
            (void) fprintf(_err, "\nInRange: internal error\n");
	    (void) fflush(_err);
	    abort();
        }
    len = 0;
    for (i = 0; i < n; i++)
	{
            value = a[i];
	    value = Encode((DOMAIN) value);
	    IncLen(value, len);
	}
    if (len >= *size)
	{
	    *size = (len >> 1) + len + 1;
	    Deallocate((char *) *buffer);
	    *buffer = (DATATYPE *) AllocArray(*size, sizeof(DATATYPE));
	}
    pointer = *buffer;
    for (i = 0; i < n; i++)
	{
            value = a[i];
	    value = Encode((DOMAIN) value);
	    PutValue(value, pointer);
	}
    *pointer = (DATATYPE) 0;
    return(BalTrElement(IngGRange(t), (char *) *buffer) != NULL);
}

static unsigned nVisComb;
static DOMAIN *aVisComb;
static VOF fVisComb;

/***************************************************************************
*
* Function:  VisComb
* Purpose:   This routine visits a combination of values of the "let"
*            variables of a transition.
* Arguments: buffer - an encoding of the combination
* Returns:
* See also:  TravRange
*
****************************************************************************
*/
FS1(BOOL, VisComb,
    DATATYPE *, buffer)
{
    register DATATYPE *pointer;
    register DOMAIN *a;
    register UDOMAIN value;

    pointer = buffer;
    a = aVisComb;
    while (*pointer)
	{
	    GetValue(UDOMAIN, value, pointer);
            *a++ = Decode((DOMAIN) value);
	}
    if (a != aVisComb + nVisComb)
        {
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
            (void) fprintf(_err,
                           "\nVisComb: nVisComb == %u, a - aVisComb == %u\n",
                           nVisComb, (unsigned) (a - aVisComb));
	    (void) fflush(_err);
	    abort();
        }
    (*fVisComb)();
    return(FALSE);
}

/***************************************************************************
*
* Function:  TravRange
* Purpose:   This routine traverses the range of the "let" variables of
*            a transition.
* Arguments: t - the transition
*            n - the number of "let" variables
*            a - a value combination of the "let" variables can be put here
*            f - a function to be called at each combination
* Returns:
* See also:  InRange, VisComb
*
****************************************************************************
*/
FE4(void, TravRange,
    unsigned, t,
    unsigned, n,
    DOMAIN *, a,
    VOF, f)
{
    if ((!n) || !IngGRange(t))
        {
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
            (void) fprintf(_err, "\nTravRange: internal error\n");
	    (void) fflush(_err);
	    abort();
        }
    nVisComb = n;
    aVisComb = a;
    fVisComb = f;
    (void) BalTrApply(IngGRange(t), VisComb);
}

/***************************************************************************
*
* Function:  TupEncode
* Purpose:   This routine encodes a pair of a place and a tuple into a
*            buffer.
* Arguments: place - the place
*            tuple - the tuple
*            buffer - the buffer (extended if needed)
*            size - the size of the buffer
* Returns:   the length of the produced DATATYPE string
* See also:  TupDecode, StaEncode, StaDecode, InsEncode, InsDecode
*
****************************************************************************
*/
FE4(unsigned, TupEncode,
    unsigned, place,
    TUPLE *, tuple,
    DATATYPE **, buffer,
    unsigned *, size)
{
    register DATATYPE *pointer;
    register unsigned i, len;
    register UDOMAIN value;

    len = 0;
    i = place;
    IncLen(i, len);
    if (TupGArity(tuple) != MagGProfile(place))
	{
	    len++;
	    i = TupGArity(tuple);
	    IncLen(i, len);
	}
    for (i = 0; i < TupGArity(tuple); i++)
	{
	    value = TupGField(tuple, i);
	    value = Encode((DOMAIN) value);
	    IncLen(value, len);
	}
    if (len >= *size)
	{
	    *size = (len >> 1) + len + 1;
	    Deallocate((char *) *buffer);
	    *buffer = (DATATYPE *) AllocArray(*size, sizeof(DATATYPE));
	}
    pointer = *buffer;
    i = place;
    PutValue(i, pointer);
    if (TupGArity(tuple) != MagGProfile(place))
	{
	    AriCode(pointer);
	    i = TupGArity(tuple);
	    PutValue(i, pointer);
	}
    for (i = 0; i < TupGArity(tuple); i++)
	{
	    value = TupGField(tuple, i);
	    value = Encode((DOMAIN) value);
	    PutValue(value, pointer);
	}
    *pointer = (DATATYPE) 0;
    return(len);
}

/***************************************************************************
*
* Function:  TupDecode
* Purpose:   This routine decodes an encoded pair of a place and a tuple
* Arguments: place - the place (will be set here)
*            buffer - the encoding buffer
*            tuple - the result is put here
* Returns:
* See also:  TupEncode, StaEncode, StaDecode, InsEncode, InsDecode
*
****************************************************************************
*/
FE3(void, TupDecode,
    unsigned *, place,
    DATATYPE *, buffer,
    TUPLE *, tuple)
{
    register DATATYPE *pointer;
    register unsigned i;
    register UDOMAIN value;

    pointer = buffer;
    GetValue(unsigned, i, pointer);
    *place = i;
    if (AriAt(pointer))
	{
	    AriPass(pointer);
	    GetValue(unsigned, i, pointer);
	    TupSArity(tuple, i);
	}
    else
	TupSArity(tuple, MagGProfile(*place));
    if (TupGArity(tuple))
	TupSArray(tuple, (DOMAIN *)
		  AllocArray(TupGArity(tuple), sizeof(DOMAIN)));
    else
	TupSArray(tuple, (DOMAIN *) NULL);
    for (i = 0; i < TupGArity(tuple); i++)
	{
	    GetValue(UDOMAIN, value, pointer);
	    TupSField(tuple, i, Decode((DOMAIN) value));
	}
}

/***************************************************************************
*
* Function:  PlKeyOut
* Purpose:   This routine displays a pair of a place and a tuple.
* Arguments: key - an encoding of the place and the tuple
*            file - the output file
*            pepInTurn - if FALSE, display "as usual",
*                        and otherwise write to a PEP low-level
*                        net file
* Returns:
* See also:
*
****************************************************************************
*/
FE3(void, PlKeyOut,
    DATATYPE *, key,
    FILE *, file,
    BOOL, pepInTurn)
{
    unsigned place = 0;
    unsigned i;
    TUPLE t_u_p_l_e;
    TUPLE *tuple;

    tuple = &t_u_p_l_e;
    TupSArity(tuple, (unsigned) 0);
    TupSArray(tuple, (DOMAIN *) NULL);
    TupDecode(&place, key, tuple);
    (void) fputs(PlaNames(place), file);
    if (TupGArity(tuple))
	{
	    (void) fputs(pepInTurn ? "." : " <.", file);
	    for (i = 0; i < TupGArity(tuple); i++)
		{
		    if (i)
			(void) fputs(pepInTurn ? "." : ", ", file);
		    if (AraSymCount() > (unsigned)
			(((DOMAIN) MAXDOMAIN) - TupGField(tuple, i)))
			(void) fputs(AraSymNames((unsigned)
						 (((DOMAIN) MAXDOMAIN) -
						  TupGField(tuple, i))),
				     file);
		    else if (pepInTurn)
			(void) fprintf(file, "%lu",
				       (unsigned long) TupGField(tuple, i));
		    else
			(void) fprintf(file, DOMFORMAT, TupGField(tuple, i));
		}
	    Deallocate((char *) TupGArray(tuple));
	    if (!pepInTurn)
		(void) fputs(".>", file);
	}
}

#ifdef DEBUG
FE2(void, PlKeySp,
    DATATYPE *, key,
    char *, buf)
{
    unsigned place = 0;
    unsigned i;
    TUPLE t_u_p_l_e;
    TUPLE *tuple;

    if ((!key) || !_plaNames)
	return;
    tuple = &t_u_p_l_e;
    TupSArity(tuple, (unsigned) 0);
    TupSArray(tuple, (DOMAIN *) NULL);
    TupDecode(&place, key, tuple);
    (void) strcat(buf, PlaNames(place));
    if (TupGArity(tuple))
	{
	    (void) strcat(buf, "__");
	    for (i = 0; i < TupGArity(tuple); i++)
		{
		    if (i)
			(void) strcat(buf, "__");
		    if (_araSymNames &&
			(AraSymCount() > (unsigned)
			 (((DOMAIN) MAXDOMAIN) - TupGField(tuple, i))))
			(void) strcat(buf,
				      AraSymNames((unsigned)
						 (((DOMAIN) MAXDOMAIN) -
						  TupGField(tuple, i))));
		    else
			(void) sprintf(buf + strlen(buf), "%lu",
				       (unsigned long) TupGField(tuple, i));
		}
	    Deallocate((char *) TupGArray(tuple));
	}
}
#endif

/***************************************************************************
*
* Function:  StaEncode
* Purpose:   This routine encodes a marking of a net to a buffer.
* Arguments: state - the marking
*            buffer - the buffer (extended if needed)
*            size - the size of the buffer
* Returns:   the length of the produced DATATYPE string
* See also:  TupEncode, TupDecode, StaDecode, InsEncode, InsDecode
*
****************************************************************************
*/
FE3(unsigned, StaEncode,
    STATE *, state,
    DATATYPE **, buffer,
    unsigned *, size)
{
    register DATATYPE *pointer;
    unsigned place;
    register unsigned i, len;
    MARKING *marking;
    MULTUP *multup;
    register UDOMAIN value;
    register unsigned long count;

    len = 0;
    for (place = 0; place < MagGNofPlaces(); place++)
	{
	    marking = StaGMarking(state, place);
	    MARFOR (marking,multup)
		{
		    if (MultupGArity(multup) != MagGProfile(place))
			{
			    len++;
			    i = MultupGArity(multup);
			    IncLen(i, len);
			}
		    if (MultupGArity(multup))
			{
			    for (i = 0; i < MultupGArity(multup); i++)
				{
				    value = MultupGField(multup, i);
				    value = Encode((DOMAIN) value);
				    IncLen(value, len);
				}
			    if (MultupGCount(multup) != 1L)
				{
				    len++;
				    count = MultupGCount(multup) - 2L;
				    IncLen(count, len);
				}
			}
		    else
			{
			    count = MultupGCount(multup) - 1L;
			    IncLen(count, len);
			}
		}
	    len++;
	}
    if (len >= *size)
	{
	    *size = (len >> 1) + len + 1;
	    Deallocate((char *) *buffer);
	    *buffer = (DATATYPE *) AllocArray(*size, sizeof(DATATYPE));
	}
    pointer = *buffer;
    for (place = 0; place < MagGNofPlaces(); place++)
	{
	    marking = StaGMarking(state, place);
	    MARFOR (marking,multup)
		{
		    if (MultupGArity(multup) != MagGProfile(place))
			{
			    AriCode(pointer);
			    i = MultupGArity(multup);
			    PutValue(i, pointer);
			}
		    if (MultupGArity(multup))
			{
			    for (i = 0; i < MultupGArity(multup); i++)
				{
				    value = MultupGField(multup, i);
				    value = Encode((DOMAIN) value);
				    PutValue(value, pointer);
				}
			    if (MultupGCount(multup) != 1L)
				{
				    CntCode(pointer);
				    count = MultupGCount(multup) - 2L;
				    PutValue(count, pointer);
				}
			}
		    else
			{
			    count = MultupGCount(multup) - 1L;
			    PutValue(count, pointer);
			}
		}
	    SepCode(pointer);
	}
    *pointer = (DATATYPE) 0;
    return(len);
}

/***************************************************************************
*
* Function:  StaDecode
* Purpose:   This routine decodes an encoded marking of the net.
* Arguments: buffer - a buffer containing the encoding
*            state - the state (will be set here)
* Returns:
* See also:  TupEncode, TupDecode, StaEncode, InsEncode, InsDecode
*
****************************************************************************
*/
FE2(void, StaDecode,
    DATATYPE *, buffer,
    STATE *, state)
{
    register DATATYPE *pointer;
    unsigned place;
    register unsigned i;
    MARKING *marking;
    TUPLE *tuple;
    register UDOMAIN value;
    register unsigned long count;

    pointer = buffer;
    for (place = 0; place < MagGNofPlaces(); place++)
	{
	    marking = MarCreate();
	    while (!SepAt(pointer))
		{
		    if (AriAt(pointer))
			{
			    AriPass(pointer);
			    GetValue(unsigned, i, pointer);
			    tuple = TupCons(i);
			}
		    else
			tuple = TupCons(MagGProfile(place));
		    if (TupGArity(tuple))
			{
			    for (i = 0; i < TupGArity(tuple); i++)
				{
				    GetValue(UDOMAIN, value, pointer);
				    TupSField(tuple, i,
					      Decode((DOMAIN) value));
				}
			    if (CntAt(pointer))
				{
				    CntPass(pointer);
				    GetValue(unsigned long, count, pointer);
				    MarInsLast(marking, MultupCons(count + 2L,
								tuple));
				}
			    else
				MarInsLast(marking,
					   MultupCons((unsigned long) 1L,
						      tuple));
			}
		    else
			{
			    GetValue(unsigned long, count, pointer);
			    MarInsLast(marking, MultupCons(count + 1L, tuple));
			}
		}
	    SepPass(pointer);
	    StaIMarking(state, place, marking);
	}
}

/***************************************************************************
*
* Function:  InsEncode
* Purpose:   This routine encodes a transition instance to a buffer.
* Arguments: trIndex - the transition
*            priority - the precedence class of the instance
*            instance - the values of the variables of the transition
*            buffer - the buffer (extended if needed)
*            size - the size of the buffer
* Returns:   the length of the produced DATATYPE string
* See also:  TupEncode, TupDecode, StaEncode, StaDecode, InsDecode
*
****************************************************************************
*/
FE5(unsigned, InsEncode,
    unsigned, trIndex,
    unsigned, priority,
    DOMAIN *, instance,
    DATATYPE **, buffer,
    unsigned *, size)
{
    register DATATYPE *pointer;
    register unsigned i, dimension, len;
    register UDOMAIN value;

    len = 0;
    i = trIndex;
    dimension = IngGProfile(i);
    IncLen(i, len);
    for (i = 0; i < dimension; i++)
	{
	    value = instance[i];
	    value = Encode((DOMAIN) value);
	    IncLen(value, len);
	}
    if (IngGNofClasses() > 1)
	{
	    i = priority;
	    IncLen(i, len);
	}
    if (len >= *size)
	{
	    *size = (len >> 1) + len + 1;
	    Deallocate((char *) *buffer);
	    *buffer = (DATATYPE *) AllocArray(*size, sizeof(DATATYPE));
	}
    pointer = *buffer;
    i = trIndex;
    dimension = IngGProfile(i);
    PutValue(i, pointer);
    for (i = 0; i < dimension; i++)
	{
	    value = instance[i];
	    value = Encode((DOMAIN) value);
	    PutValue(value, pointer);
	}
    if (IngGNofClasses() > 1)
	{
	    i = priority;
	    PutValue(i, pointer);
	}
    *pointer = (DATATYPE) 0;
    return(len);
}

/***************************************************************************
*
* Function:  InsDecode
* Purpose:   This routine decodes an encoded transition instance.
* Arguments: pTrIndex - the transition (will be set here)
*            priority - the precedence class of the instance
*                       (will be set here)
*            buffer - the buffer
* Returns:   the values of the variables of the transition
* See also:  TupEncode, TupDecode, StaEncode, StaDecode, InsEncode
*
****************************************************************************
*/
FE3(DOMAIN *, InsDecode,
    unsigned *, pTrIndex,
    unsigned *, priority,
    DATATYPE *, buffer)
{
    register DATATYPE *pointer;
    register unsigned i, dimension;
    register UDOMAIN value;
    DOMAIN *instance;

    pointer = buffer;
    GetValue(unsigned, i, pointer);
    *pTrIndex = i;
    if ((dimension = IngGProfile(i)))
        instance = (DOMAIN *) AllocArray(dimension, sizeof(DOMAIN));
    else
	instance = (DOMAIN *) NULL;
    for (i = 0; i < dimension; i++)
	{
	    GetValue(UDOMAIN, value, pointer);
	    instance[i] = Decode((DOMAIN) value);
	}
    if (IngGNofClasses() > 1)
	{
	    GetValue(unsigned, i, pointer);
	    *priority = i;
	}
    else
	*priority = 0;
    return(instance);
}

/***************************************************************************
*
* Function:  BTupComp
* Purpose:   This routine compares two encoded place-tuple pairs to
*            each other.
* Arguments: buffer1 - the first encoding
*            buffer2 - the second encoding
* Returns:   -1 if the first pair is less than the second,
*             1 if the first pair is greater than the second, and
*             0 otherwise
* See also:  InsComp, FdEqual
*
****************************************************************************
*/
FE2(int, BTupComp,
    DATATYPE *, buffer1,
    DATATYPE *, buffer2)
{
    register DATATYPE *pointer1, *pointer2;
    register unsigned i1, i2;
    register UDOMAIN value1, value2;

    pointer1 = buffer1;
    pointer2 = buffer2;
    GetValue(unsigned, i1, pointer1);
    GetValue(unsigned, i2, pointer2);
    if (i1 < i2)
	return(-1);
    else if (i1 > i2)
	return(1);
    if (AriAt(pointer1))
	{
	    AriPass(pointer1);
	    GetValue(unsigned, i1, pointer1);
	}
    else
	i1 = MagGProfile(i1);
    if (AriAt(pointer2))
	{
	    AriPass(pointer2);
	    GetValue(unsigned, i2, pointer2);
	}
    else
	i2 = MagGProfile(i2);
    if (i1 < i2)
	return(-1);
    else if (i1 > i2)
	return(1);
    while (i1--)
	{
	    GetValue(UDOMAIN, value1, pointer1);
	    value1 = Decode((DOMAIN) value1);
	    GetValue(UDOMAIN, value2, pointer2);
	    value2 = Decode((DOMAIN) value2);
	    if (((DOMAIN) value1) < ((DOMAIN) value2))
		return(-1);
	    else if (((DOMAIN) value1) > ((DOMAIN) value2))
		return(1);
	}
    return(0);
}

/***************************************************************************
*
* Function:  InsComp
* Purpose:   This routine compares two encoded transition instances to
*            each other.
* Arguments: buffer1 - the first encoding
*            buffer2 - the second encoding
* Returns:   -1 if the first instance is less than the second,
*             1 if the first instance is greater than the second, and
*             0 otherwise
* See also:  BTupComp, FdEqual
*
****************************************************************************
*/
FE2(int, InsComp,
    DATATYPE *, buffer1,
    DATATYPE *, buffer2)
{
    register DATATYPE *pointer1, *pointer2;
    register unsigned i1, i2;
    register UDOMAIN value1, value2;

    pointer1 = buffer1;
    pointer2 = buffer2;
    GetValue(unsigned, i1, pointer1);
    GetValue(unsigned, i2, pointer2);
    if (i1 < i2)
	return(-1);
    else if (i1 > i2)
	return(1);
    i1 = IngGProfile(i1);
    while (i1--)
	{
	    GetValue(UDOMAIN, value1, pointer1);
	    value1 = Decode((DOMAIN) value1);
	    GetValue(UDOMAIN, value2, pointer2);
	    value2 = Decode((DOMAIN) value2);
	    if (((DOMAIN) value1) < ((DOMAIN) value2))
		return(-1);
	    else if (((DOMAIN) value1) > ((DOMAIN) value2))
		return(1);
	}
    return(0);
}

/***************************************************************************
*
* Function:  FdEqual
* Purpose:   This routine checks whether two encoded transition instances
*            have, so to say, the same folding.
* Arguments: buffer1 - the first encoding
*            buffer2 - the second encoding
* Returns:   TRUE if and only if the instances have the same folding
* See also:  BTupComp, InsComp
*
****************************************************************************
*/
FE2(BOOL, FdEqual,
    DATATYPE *, buffer1,
    DATATYPE *, buffer2)
{
    register DATATYPE *pointer1, *pointer2;
    register unsigned i, width;
    register UDOMAIN value1, value2;

    pointer1 = buffer1;
    pointer2 = buffer2;
    GetValue(unsigned, i, pointer1);
    width = IngGFdProfile(i);
    GetValue(unsigned, i, pointer2);
    if (width != IngGFdProfile(i))
	return(FALSE);
    while (width--)
	{
	    GetValue(UDOMAIN, value1, pointer1);
	    GetValue(UDOMAIN, value2, pointer2);
	    if (Decode((DOMAIN) value1) != Decode((DOMAIN) value2))
		return(FALSE);
	}
    return(TRUE);
}


/***************************************************************************
*
* Function:  DotOutput
* Purpose:   This routine prints dots corresponding to the progress
*            made in reachability graph generation.
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, DotOutput)
{
    if (theInner == 9)
	{
	    theInner = 0;
	    (void) fputc((int) '.', _out);
	    if (theOuter == 24)
		{
		    theOuter = 0;
		    (void) fputc((int) '\n', _out);
		}
	    else
		theOuter++;    
	    (void) fflush(_out);
	}
    else
	theInner++;
}

static unsigned *araVarInds;  /* needed in the setting of _araIndices */

/***************************************************************************
*
* Function:  AraVarMap
* Purpose:   This routine is needed for the setting of _araIndices.
* Arguments: ref - a pair of a transition variable number and name.
* Returns:
* See also:  AraNames
*
****************************************************************************
*/
FS1(BOOL, AraVarMap,
    REF *, ref)
{
    *araVarInds++ = (unsigned) RefGSlot(ref);
    return(FALSE);
}

/***************************************************************************
*
* Function:  AraNames
* Purpose:   This routine reads name information about a net from a
*            binary file. The information is needed in the construction of
*            an ARA LTS.
* Arguments: datFile - the binary file
* Returns:
* See also:  pplst_dat (pplst.c), BaseInit, InitialState,
*            PtInit (ptelem.c), AraVarMap, DelAraNames, RdPlaNames
*
****************************************************************************
*/
FE1(void, AraNames,
    BINARYFILE, datFile)
{
    unsigned i;
    unsigned j;
    unsigned len = 0;
    BALTREE *varTree;

    if (IngGNofTransitions())
	{
	    _araTraNames = (char **)
		AllocArray(IngGNofTransitions(), sizeof(char *));
	    _araVarNames = (char ***)
		AllocArray(IngGNofTransitions(), sizeof(char **));
	    _araIndices = (unsigned **) AllocArray(IngGNofTransitions(),
						   sizeof(unsigned *));
	}
    else
	{
	    _araTraNames = (char **) NULL;
	    _araVarNames = (char ***) NULL;
	    _araIndices = (unsigned **) NULL;
	}
    for (i = 0; i < IngGNofTransitions(); i++)
	{
	    ReadUns(&len, 1, datFile);
	    _araTraNames[i] = (char *) Allocate(len + 1);
	    ReadChr(_araTraNames[i], len, datFile);
	    if (IngGProfile(i) != 0)
		{
		    _araVarNames[i] = (char **)
			AllocArray(IngGProfile(i), sizeof(char *));
		    for (j = 0; j < IngGProfile(i); j++)
			{
			    ReadUns(&len, 1, datFile);
			    _araVarNames[i][j] = (char *) Allocate(len + 1);
			    ReadChr(_araVarNames[i][j], len, datFile);
			}
		    if (IngGProfile(i) > 1)
			{
			    varTree = BalTrCreate(&refBalTools);
			    for (j = 0; j < IngGProfile(i); j++)
				(void) BalTrInsert(varTree, (char *)
						   RefCons(_araVarNames[i][j],
							   (unsigned long) j),
						   (char *)
						   (_araVarNames[i][j]), TRUE);
			    araVarInds = _araIndices[i] =
				(unsigned *) AllocArray(IngGProfile(i),
							sizeof(unsigned));
			    (void) BalTrApply(varTree, AraVarMap);
			    BalTrDestroy(varTree, TRUE);
			}
		}
	}
    ReadUns(&_araSymCount, 1, datFile);
    if (_araSymCount != 0)
	{
	    _araSymNames = (char **) AllocArray(_araSymCount, sizeof(char *));
	    for (i = 0; i < _araSymCount; i++)
		{
		    ReadUns(&len, 1, datFile);
		    _araSymNames[i] = (char *) Allocate(len + 1);
		    ReadChr(_araSymNames[i], len, datFile);
		}
	}
    if (IngGNofClasses() != 1)
	{
	    _araPriNames = (char **)
		AllocArray(IngGNofClasses() - 1, sizeof(char *));
	    for (i = 0; i < IngGNofClasses() - 1; i++)
		{
		    ReadUns(&len, 1, datFile);
		    _araPriNames[i] = (char *) Allocate(len + 1);
		    ReadChr(_araPriNames[i], len, datFile);
		}
	}
}

/***************************************************************************
*
* Function:  DelAraNames
* Purpose:   This routine deletes the arrays created by AraNames.
* Arguments: delSymNames - _araSymNames deleted iff this is TRUE
* Returns:
* See also:  AraNames
*
****************************************************************************
*/
FE1(void, DelAraNames,
    BOOL, delSymNames)
{
    unsigned i, j;
    
    for (i = 0; i < IngGNofTransitions(); i++)
	{
	    Deallocate((char *) _araTraNames[i]);
	    if (IngGProfile(i) != 0)
		{
		    for (j = 0; j < IngGProfile(i); j++)
			Deallocate((char *) _araVarNames[i][j]);
		    Deallocate((char *) (_araVarNames[i]));
		    if (IngGProfile(i) > 1)
			Deallocate((char *) _araIndices[i]);
		}
	}
    Deallocate((char *) _araTraNames);
    Deallocate((char *) _araVarNames);
    Deallocate((char *) _araIndices);
    if (delSymNames && (_araSymCount != 0))
	{
	    for (i = 0; i < _araSymCount; i++)
		Deallocate((char *) _araSymNames[i]);
	    Deallocate((char *) _araSymNames);
	}
    if (IngGNofClasses() != 1)
	{
	    for (i = 0; i < IngGNofClasses() - 1; i++)
		Deallocate((char *) _araPriNames[i]);
	    Deallocate((char *) _araPriNames);
	}
}

/***************************************************************************
*
* Function:  RdPlaNames
* Purpose:   This routine reads place and enum symbol information
*            from a binary file. The information is needed in the
*            displaying of place symmetries computed in the symmetry
*            method.
* Arguments: datFile - the file
* Returns:
* See also:  pplst_dat (pplst.c), BaseInit, InitialState,
*            PtInit (ptelem.c), AraNames, DelPlaNames
*
****************************************************************************
*/
FE1(void, RdPlaNames,
    BINARYFILE, datFile)
{
    unsigned i;
    unsigned len = 0;
#ifndef DEBUG
    unsigned j;
    char *s;

    if (Ara() || Pep())
	goto CONT;
    for (i = 0; i < IngGNofTransitions(); i++)
	{
	    ReadUns(&len, 1, datFile);
	    s = (char *) Allocate(len + 1);
	    ReadChr(s, len, datFile);
	    Deallocate((char *) s);
	    if (IngGProfile(i) != 0)
		{
		    for (j = 0; j < IngGProfile(i); j++)
			{
			    ReadUns(&len, 1, datFile);
			    s = (char *) Allocate(len + 1);
			    ReadChr(s, len, datFile);
			    Deallocate((char *) s);
			}
		}
	}
    ReadUns(&_araSymCount, 1, datFile);
    if (_araSymCount != 0)
	{
	    _araSymNames = (char **) AllocArray(_araSymCount, sizeof(char *));
	    for (i = 0; i < _araSymCount; i++)
		{
		    ReadUns(&len, 1, datFile);
		    _araSymNames[i] = (char *) Allocate(len + 1);
		    ReadChr(_araSymNames[i], len, datFile);
		}
	}
    if (IngGNofClasses() != 1)
	{
	    for (i = 0; i < IngGNofClasses() - 1; i++)
		{
		    ReadUns(&len, 1, datFile);
		    s = (char *) Allocate(len + 1);
		    ReadChr(s, len, datFile);
		    Deallocate((char *) s);
		}
	}
 CONT: ;
#endif
    if (MagGNofPlaces())
	_plaNames = (char **) AllocArray(MagGNofPlaces(), sizeof(char *));
    else
	_plaNames = (char **) NULL;
    for (i = 0; i < MagGNofPlaces(); i++)
	{
	    ReadUns(&len, 1, datFile);
	    _plaNames[i] = (char *) Allocate(len + 1);
	    ReadChr(_plaNames[i], len, datFile);
	}
}

/***************************************************************************
*
* Function:  DelPlaNames
* Purpose:   This routine deletes the arrays created by RdPlaNames.
* Arguments:
* Returns:
* See also:  RdPlaNames
*
****************************************************************************
*/
FE0(void, DelPlaNames)
{
    unsigned i;
    
    if (_araSymCount != 0)
	{
	    for (i = 0; i < _araSymCount; i++)
		Deallocate((char *) _araSymNames[i]);
	    Deallocate((char *) _araSymNames);
	}
    for (i = 0; i < MagGNofPlaces(); i++)
	Deallocate((char *) _plaNames[i]);
    Deallocate((char *) _plaNames);
}

/***************************************************************************
*
* Function:  InsOutput
* Purpose:   This routine writes to an ARA LTS file an action name
*            corresponding to a transition instance.
*            Alternatively, a name of a low-level transition
*            is written to a PEP low-level net file.
* Arguments: number - the number of the action
*            key - an encoding of the instance
*            file - the output file
*            pepInTurn - if FALSE, write to an ARA LTS file,
*                        and otherwise write to a PEP low-level
*                        net file
* Returns:
* See also:  LTSBegin (ptelem.c), PtAraOutput (ptelem.c), LTSEnd (ptelem.c),
*            AraStep (inst.c), TrnFire (ptnet.c)
*
****************************************************************************
*/
FE4(void, InsOutput,
    unsigned long, number,
    DATATYPE *, key,
    FILE *, file,
    BOOL, pepInTurn)
{
    unsigned trIndex = 0;
    unsigned trPriority = 0;
    unsigned j;
    unsigned trVarIndex;
    DOMAIN *trInstance;
    int delimiter;

    delimiter = pepInTurn ? ((int) '.') : ((int) '$');
    trInstance = InsDecode(&trIndex, &trPriority, key);
    (void) fprintf(file, "%lu", number);
    (void) fputc(pepInTurn ? ((int) '\"') : ((int) ' '), file);
    (void) fputs(AraTraNames(trIndex), file);
    for (j = 0; j < IngGProfile(trIndex); j++)
	{
	    if (IngGProfile(trIndex) > 1)
		trVarIndex = AraIndices(trIndex, j);
	    else
		trVarIndex = j;
	    (void) fputc(delimiter, file);
	    (void) fputs(AraVarNames(trIndex, trVarIndex), file);
	    (void) fputc(delimiter, file);
	    if (AraSymCount() > (unsigned)
		(((DOMAIN) MAXDOMAIN) - trInstance[trVarIndex]))
		(void) fputs(AraSymNames((unsigned)
					 (((DOMAIN) MAXDOMAIN) -
					  trInstance[trVarIndex])),
			     file);
	    else
		(void) fprintf(file, "%lu", (unsigned long)
			       (trInstance[trVarIndex]));
	}
    if (trInstance)
	Deallocate((char *) trInstance);
    if (IngGNofClasses() != 1)
	{
	    (void) fputc(delimiter, file);
	    (void) fputs("prec", file);
	    (void) fputc(delimiter, file);
	    (void) fputs((trPriority != 0)
			 ? AraPriNames(trPriority - 1) : "0", file);
	}
    (void) fputs(pepInTurn ? "\"0@0\n" : "\n", file);
}

#ifdef DEBUG
FE2(void, InsSp,
    DATATYPE *, key,
    char *, buf)
{
    unsigned trIndex = 0;
    unsigned trPriority = 0;
    unsigned j;
    unsigned trVarIndex;
    DOMAIN *trInstance;

    if ((!key) || !_araTraNames)
	return;
    trInstance = InsDecode(&trIndex, &trPriority, key);
    (void) strcat(buf, AraTraNames(trIndex));
    for (j = 0; j < IngGProfile(trIndex); j++)
	{
	    if (IngGProfile(trIndex) > 1)
		{
		    if (_araIndices)
			trVarIndex = AraIndices(trIndex, j);
		    else
			trVarIndex = (unsigned) -1;
		}
	    else
		trVarIndex = j;
	    (void) strcat(buf, "__");
	    if ((trVarIndex == (unsigned) -1) || !_araVarNames)
		(void) sprintf(buf + strlen(buf), "__%u", j);
	    else
		(void) strcat(buf, AraVarNames(trIndex, trVarIndex));
	    (void) strcat(buf, "__");
	    if (_araSymNames &&
		(AraSymCount() > (unsigned)
		 (((DOMAIN) MAXDOMAIN) - trInstance[trVarIndex])))
		(void) strcat(buf,
			      AraSymNames((unsigned)
					  (((DOMAIN) MAXDOMAIN) -
					   trInstance[trVarIndex])));
	    else
		(void) sprintf(buf + strlen(buf), "%lu", (unsigned long)
			       (trInstance[trVarIndex]));
	}
    if (trInstance)
	Deallocate((char *) trInstance);
    if (IngGNofClasses() != 1)
	{
	    (void) strcat(buf, "__prec__");
	    if ((!trPriority) || !_araPriNames)
		(void) sprintf(buf + strlen(buf), "%u", trPriority);
	    else
		(void) strcat(buf, AraPriNames(trPriority - 1));
	}
}
#endif
