/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  gen.tpl
*
* Part of: PROD's program prpp
*
* Authors: Marko Rauhamaa and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     This module makes C functions from high level transitions.
*     The made functions are needed in the generation of a reachability
*     graph.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include "gen.h"

FILE *f;
unsigned long fileCount;
NET *net;
TRANSITION *transition;
unsigned t;
PLACE *place;
unsigned p;
ARC *arc, *arc2;
unsigned a, a2;
unsigned ca, ca2;
SYMMULTUP *symmultup, *syt2;
unsigned s, s2;
unsigned cs, cs2;
char *ls, *ls2;
VARIABLE *variable, *var2;
DOMAIN x, y;
TUPLE *low, *high, *xlow, *xhigh, *ylow, *yhigh;
unsigned i, j, iVarCount, dimension, maxDim;
SYMBOL *symbol, *sym2, *cffSym, *csm2, *gate;
char *str;
VOID *acursor, *scursor;
PRECEDENCE *precedence, *priority;
LIST *cffOrdVariables;
LIST *cffOrdInputs;
LIST *cffOrdSymsum;
BOOL found, pre, post, outa, comp, cvars, priNonTriv;
unsigned thisDepth, maxDepth;
FORMULA *atom;
unsigned long mask;
unsigned letVarCount;

/***************************************************************************
*
* Function:  DefsDecls
* Purpose:   This routine produces the "include" and "define" directives
*            and variable introductions that are needed in the
*            functions corresponding to high level transitions.
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, DefsDecls)
{
    ((void) fprintf(f, "#include <stdio.h>\n"), nOutpLines++);
    ((void) fprintf(f, "#include \"absolute.h\"\n"), nOutpLines++);
    ((void) fprintf(f, "#include \"prod.h\"\n"), nOutpLines++);
    ((void) fprintf(f, "#include \"alloc.h\"\n"), nOutpLines++);
    ((void) fprintf(f, "#include \"dynam.h\"\n"), nOutpLines++);
    ((void) fprintf(f, "#include \"ptrbl.h\"\n"), nOutpLines++);
    ((void) fprintf(f, "#include \"baltree.h\"\n"), nOutpLines++);
    ((void) fprintf(f, "#include \"bdd.h\"\n"), nOutpLines++);
    ((void) fprintf(f, "#include \"gphutil.h\"\n"), nOutpLines++);
    ((void) fprintf(f, "#include \"graph.h\"\n"), nOutpLines++);
    ((void) fprintf(f, "#include \"mark.h\"\n"), nOutpLines++);
    ((void) fprintf(f, "#include \"inst.h\"\n"), nOutpLines++);
    ((void) fprintf(f, "#include \"encode.h\"\n"), nOutpLines++);
    ((void) fprintf(f, "#include \"ptnet.h\"\n"), nOutpLines++);
    ((void) fprintf(f, "#include \"ptsymm.h\"\n"), nOutpLines++);
    ((void) fprintf(f, "\n"), nOutpLines++);
    if (!LstEmpty(NetGTransitions(net)))
	{
	    ((void) fprintf(f, "extern TRN *_trn;\n"), nOutpLines++);
	    ((void) fprintf(f, "extern unsigned _visible, _len;\n"), nOutpLines++);
	}
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	if ((!LstEmpty(TraGInputs(transition))) ||
	    !LstEmpty(TraGOutputs(transition)))
	{
	    ((void) fprintf(f, "extern MARKING *_marking;\n"), nOutpLines++);
	    break;
	}
    if (NetGAtoms(net))
	{
	    ((void) fprintf(f, "extern DOMAIN *_y_;\n"), nOutpLines++);
	    ((void) fprintf(f, "extern BOOL _someEnb;\n"), nOutpLines++);
	}
    pre = FALSE;
    post = FALSE;
    maxDepth = 0;
    if (NetGAtoms(net))
	{
	    LSTFOR (NetGAtoms(net), atom, FORMULA)
		{
		    thisDepth = XprDepth(atom->uf.expression);
		    if (thisDepth > maxDepth)
			maxDepth = thisDepth;
		}
	    if (maxDepth != 0)
		{
		    ((void) fprintf(f, "\n"), nOutpLines++);
		    ((void) fprintf(f, "typedef union\n"), nOutpLines++);
		    ((void) fprintf(f, "{\n"), nOutpLines++);
		    ((void) fprintf(f, "    DOMAIN d;\n"), nOutpLines++);
		    ((void) fprintf(f, "    struct\n"), nOutpLines++);
		    ((void) fprintf(f, "    {\n"), nOutpLines++);
		    ((void) fprintf(f, "        char *c;\n"), nOutpLines++);
		    ((void) fprintf(f, "        MARKING *ma;\n"), nOutpLines++);
		    ((void) fprintf(f, "        TUPLE *t;\n"), nOutpLines++);
		    ((void) fprintf(f, "    } a;\n"), nOutpLines++);
		    ((void) fprintf(f, "    struct\n"), nOutpLines++);
		    ((void) fprintf(f, "    {\n"), nOutpLines++);
		    ((void) fprintf(f, "        MARKING *mb;\n"), nOutpLines++);
		    ((void) fprintf(f, "        TUPLE *t0;\n"), nOutpLines++);
		    ((void) fprintf(f, "        TUPLE *t1;\n"), nOutpLines++);
		    ((void) fprintf(f, "        TUPLE *t2;\n"), nOutpLines++);
		    ((void) fprintf(f, "    } b;\n"), nOutpLines++);
		    ((void) fprintf(f, "} XPRVAL;\n"), nOutpLines++);
		    ((void) fprintf(f, "\n"), nOutpLines++);
		    ((void) fprintf(f, "extern XPRVAL _xprVal[%s", UnsStr(maxDepth)), (void) fprintf(f, "];\n"), nOutpLines++);
		}
	    ((void) fprintf(f, "extern TUPLE *_xprTuple;\n"), nOutpLines++);
	    ((void) fprintf(f, "extern DOMAIN _xprDom;\n"), nOutpLines++);
	    ((void) fprintf(f, "extern MARKING *_xprMk, *_tmpMk;\n"), nOutpLines++);
	    ((void) fprintf(f, "extern unsigned _i_;\n"), nOutpLines++);
	}
    maxDim = 0;
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	{
	    dimension = 0;
	    LSTFOR (TraGInputs(transition), arc, ARC)
		dimension = dimension + LstSize(ArcGSymsum(arc));
	    if (dimension > maxDim)
		maxDim = dimension;
	}
    if (maxDim != 0)
	((void) fprintf(f, "extern MULTUP *_t[%s", UnsStr(maxDim)), (void) fprintf(f, "];\n"), nOutpLines++);
    maxDim = 0;
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	{
	    dimension = 0;
	    LSTFOR (TraGInputs(transition), arc, ARC)
		{
		    if (!LstEmpty(ArcGSymsum(arc)))
			dimension = dimension + LstSize(ArcGSymsum(arc)) - 1;
		}
	    if (dimension > maxDim)
		maxDim = dimension;
	}
    if (maxDim != 0)
	((void) fprintf(f, "extern VOID *_u[%s", UnsStr(maxDim)), (void) fprintf(f, "];\n"), nOutpLines++);
    maxDim = 0;
    outa = FALSE;
    comp = FALSE;
    cvars = FALSE;
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	{
	    dimension = LstSize(TraGVariables(transition));
	    if (dimension > maxDim)
		maxDim = dimension;
	    if (!LstEmpty(TraGOutputs(transition)))
		outa = TRUE;
	    if (TraGComputation(transition))
		{
		    comp = TRUE;
		    if (!cvars)
			LSTFOR (TraGVariables(transition), variable, VARIABLE)
			    if (!VarIsIn(variable))
				{
				    cvars = TRUE;
				    break;
				}
		}
	}
    if (maxDim != 0)
	((void) fprintf(f, "extern DOMAIN _v[%s", UnsStr(maxDim)), (void) fprintf(f, "];\n"), nOutpLines++);
    if (outa)
	((void) fprintf(f, "extern TUPLE *_tuple, *_w;\n"), nOutpLines++);
    else
	((void) fprintf(f, "extern TUPLE *_w;\n"), nOutpLines++);
    if (comp)
	{
	    ((void) fprintf(f, "extern LIST *_list;\n"), nOutpLines++);
	    if (cvars)
		((void) fprintf(f, "extern VOID *_cursor;\n"), nOutpLines++);
	}
    priNonTriv = FALSE;
    if (NetGNofClasses(net) != 1)
	LSTFOR (NetGPrecedences(net), precedence, PRECEDENCE)
	    if (PreGSucc(precedence) && (!LstEmpty(PreGSucc(precedence)))
		&& (!PreGEasy(precedence)) && !PreGSelfLoop(precedence))
		{
		    priNonTriv = TRUE;
		    break;
		}
    if (cvars || (NetGNofClasses(net) != 1))
	{
	    ((void) fprintf(f, "extern DOMAIN *_instance;\n"), nOutpLines++);
	    ((void) fprintf(f, "extern PEER *_peer;\n"), nOutpLines++);
	}
    if (priNonTriv)
	{
	    ((void) fprintf(f, "extern PEER *_hpeer;\n"), nOutpLines++);
	    ((void) fprintf(f, "extern unsigned _lwidth, _field;\n"), nOutpLines++);
	    ((void) fprintf(f, "extern DOMAIN *_hfolding;\n"), nOutpLines++);
	}
    if (NetGNofClasses(net) != 1)
	{
	    ((void) fprintf(f, "extern LIST *_bundle[%s", UnsStr(NetGNofClasses(net))), (void) fprintf(f, "];\n"), nOutpLines++);
	    ((void) fprintf(f, "extern LIST *_tmpBundle[%s", UnsStr(NetGNofClasses(net))), (void) fprintf(f, "];\n"), nOutpLines++);
            ((void) fprintf(f, "#define _CL_INDEPENDENT 0\n"), nOutpLines++);
            ((void) fprintf(f, "#define _CL_MAXIMUM 1\n"), nOutpLines++);
            ((void) fprintf(f, "#define _CL_MEDIUM 2\n"), nOutpLines++);
            ((void) fprintf(f, "#define _CL_INHIBITOR 3\n"), nOutpLines++);
            ((void) fprintf(f, "#define _CL_NOTHING 4\n"), nOutpLines++);
	    ((void) fprintf(f, "extern unsigned _kind[%s", UnsStr(NetGNofClasses(net))), (void) fprintf(f, "];\n"), nOutpLines++);
	    ((void) fprintf(f, "extern unsigned _class;\n"), nOutpLines++);
	}
}

/***************************************************************************
*
* Function:  ActualDecls
* Purpose:   This routine produces the actual variable declarations
*            that are needed in the functions corresponding to high
*            level transitions.
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, ActualDecls)
{
    if (!LstEmpty(NetGTransitions(net)))
	{
	    ((void) fprintf(f, "TRN *_trn;\n"), nOutpLines++);
	    ((void) fprintf(f, "unsigned _visible, _len;\n"), nOutpLines++);
	}
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	if ((!LstEmpty(TraGInputs(transition))) ||
	    !LstEmpty(TraGOutputs(transition)))
	{
	    ((void) fprintf(f, "MARKING *_marking;\n"), nOutpLines++);
	    break;
	}
    if (NetGAtoms(net))
	{
	    ((void) fprintf(f, "DOMAIN *_y_;\n"), nOutpLines++);
	    ((void) fprintf(f, "BOOL _someEnb;\n"), nOutpLines++);
	}
    pre = FALSE;
    post = FALSE;
    maxDepth = 0;
    if (NetGAtoms(net))
	{
	    LSTFOR (NetGAtoms(net), atom, FORMULA)
		{
		    thisDepth = XprDepth(atom->uf.expression);
		    if (thisDepth > maxDepth)
			maxDepth = thisDepth;
		}
	    if (maxDepth != 0)
		((void) fprintf(f, "XPRVAL _xprVal[%s", UnsStr(maxDepth)), (void) fprintf(f, "];\n"), nOutpLines++);
	    ((void) fprintf(f, "TUPLE *_xprTuple;\n"), nOutpLines++);
	    ((void) fprintf(f, "DOMAIN _xprDom;\n"), nOutpLines++);
	    ((void) fprintf(f, "MARKING *_xprMk, *_tmpMk;\n"), nOutpLines++);
	    ((void) fprintf(f, "unsigned _i_;\n"), nOutpLines++);
	}
    maxDim = 0;
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	{
	    dimension = 0;
	    LSTFOR (TraGInputs(transition), arc, ARC)
		dimension = dimension + LstSize(ArcGSymsum(arc));
	    if (dimension > maxDim)
		maxDim = dimension;
	}
    if (maxDim != 0)
	((void) fprintf(f, "MULTUP *_t[%s", UnsStr(maxDim)), (void) fprintf(f, "];\n"), nOutpLines++);
    maxDim = 0;
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	{
	    dimension = 0;
	    LSTFOR (TraGInputs(transition), arc, ARC)
		{
		    if (!LstEmpty(ArcGSymsum(arc)))
			dimension = dimension + LstSize(ArcGSymsum(arc)) - 1;
		}
	    if (dimension > maxDim)
		maxDim = dimension;
	}
    if (maxDim != 0)
	((void) fprintf(f, "VOID *_u[%s", UnsStr(maxDim)), (void) fprintf(f, "];\n"), nOutpLines++);
    maxDim = 0;
    outa = FALSE;
    comp = FALSE;
    cvars = FALSE;
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	{
	    dimension = LstSize(TraGVariables(transition));
	    if (dimension > maxDim)
		maxDim = dimension;
	    if (!LstEmpty(TraGOutputs(transition)))
		outa = TRUE;
	    if (TraGComputation(transition))
		{
		    comp = TRUE;
		    if (!cvars)
			LSTFOR (TraGVariables(transition), variable, VARIABLE)
			    if (!VarIsIn(variable))
				{
				    cvars = TRUE;
				    break;
				}
		}
	}
    if (maxDim != 0)
	((void) fprintf(f, "DOMAIN _v[%s", UnsStr(maxDim)), (void) fprintf(f, "];\n"), nOutpLines++);
    if (outa)
	((void) fprintf(f, "TUPLE *_tuple, *_w;\n"), nOutpLines++);
    else
	((void) fprintf(f, "TUPLE *_w;\n"), nOutpLines++);
    if (comp)
	{
	    ((void) fprintf(f, "LIST *_list;\n"), nOutpLines++);
	    if (cvars)
		((void) fprintf(f, "VOID *_cursor;\n"), nOutpLines++);
	}
    priNonTriv = FALSE;
    if (NetGNofClasses(net) != 1)
	LSTFOR (NetGPrecedences(net), precedence, PRECEDENCE)
	    if (PreGSucc(precedence) && (!LstEmpty(PreGSucc(precedence)))
		&& (!PreGEasy(precedence)) && !PreGSelfLoop(precedence))
		{
		    priNonTriv = TRUE;
		    break;
		}
    if (cvars || (NetGNofClasses(net) != 1))
	{
	    ((void) fprintf(f, "DOMAIN *_instance;\n"), nOutpLines++);
	    ((void) fprintf(f, "PEER *_peer;\n"), nOutpLines++);
	}
    if (priNonTriv)
	{
	    ((void) fprintf(f, "PEER *_hpeer;\n"), nOutpLines++);
	    ((void) fprintf(f, "unsigned _lwidth, _field;\n"), nOutpLines++);
	    ((void) fprintf(f, "DOMAIN *_hfolding;\n"), nOutpLines++);
	}
    if (NetGNofClasses(net) != 1)
	{
	    ((void) fprintf(f, "LIST *_bundle[%s", UnsStr(NetGNofClasses(net))), (void) fprintf(f, "];\n"), nOutpLines++);
	    ((void) fprintf(f, "LIST *_tmpBundle[%s", UnsStr(NetGNofClasses(net))), (void) fprintf(f, "];\n"), nOutpLines++);
	    ((void) fprintf(f, "unsigned _kind[%s", UnsStr(NetGNofClasses(net))), (void) fprintf(f, "] = {0"));
	    LSTFOR (NetGPrecedences(net), precedence, PRECEDENCE)
		{
		    if (PreGEasy(precedence))
			((void) fprintf(f, ",4"));
		    else if (PreGSelfLoop(precedence))
			((void) fprintf(f, ",3"));
		    else if (PreGSucc(precedence) &&
			     !LstEmpty(PreGSucc(precedence)))
			((void) fprintf(f, ",2"));
		    else if (PreGPred(precedence) &&
			     !LstEmpty(PreGPred(precedence)))
			((void) fprintf(f, ",1"));
		    else
			((void) fprintf(f, ",0"));
		}
	    ((void) fprintf(f, "};\n"), nOutpLines++);
	    ((void) fprintf(f, "unsigned _class;\n"), nOutpLines++);
	}
}

/***************************************************************************
*
* Function:  ADLimits
* Purpose:   This routine produces those checks that are made in Accept().
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, ADLimits)
{
    LSTFOR (TraGVariables(transition), variable, VARIABLE)
	{
	    if (VarIsIn(variable))
		VarSet(variable);
	    if (VarIsIn(variable) || !VarHard(variable))
		continue;
	    VarReset(variable);
	    if (VarGLow(variable) > (DOMAIN) MINDOMAIN)
		{
		    ((void) fprintf(f, "    if (%s <", VarGLabel(variable)));
                    ((void) fprintf(f, " (DOMAIN) %s", DomStr(VarGLow(variable))), (void) fprintf(f, ")\n"), nOutpLines++);
		    ((void) fprintf(f, "        return;\n"), nOutpLines++);
		}
	    if (VarGHigh(variable) < (DOMAIN) MAXDOMAIN)
		{
		    ((void) fprintf(f, "    if (%s >", VarGLabel(variable)));
                    ((void) fprintf(f, " (DOMAIN) %s", DomStr(VarGHigh(variable))), (void) fprintf(f, ")\n"), nOutpLines++);
		    ((void) fprintf(f, "        return;\n"), nOutpLines++);
		}
	}
    LSTFOR (TraGOutputs(transition), arc, ARC)
	{
	    place = ArcGPlace(arc);
	    LSTFOR (ArcGSymsum(arc), symmultup, SYMMULTUP)
		{
		    low = NULL;
		    high = NULL;
		    if (PlaGLow(place))
			LSTFOR (PlaGLow(place), low, TUPLE)
			    {
				if (TupGArity(low) == SytGArity(symmultup))
				    break;
				else if (TupGArity(low) > SytGArity(symmultup))
				    {
					low = NULL;
					break;
				    }
			    }
		    if (PlaGHigh(place))
			LSTFOR (PlaGHigh(place), high, TUPLE)
			    {
				if (TupGArity(high) == SytGArity(symmultup))
				    break;
				else if (TupGArity(high) >
					 SytGArity(symmultup))
				    {
					high = NULL;
					break;
				    }
			    }
		    if ((!low) && !high)
			continue;
		    if (SytGCffSym(symmultup))
			{
			    found = FALSE;
			    for (i = 0; i < SytGArity(symmultup); i++)
				{
				    symbol = SytGSymbol(symmultup, i);
				    if (SymSatisfied(symbol))
					continue;
				    if (SymIsVariable(symbol)
					&& VarHard(variable =
						   SymGVariable(symbol)))
					{
					    variable = SymGVariable(symbol);
					    x = VarGLow(variable);
					}
				    else
					{
					    variable = NULL;
					    x = (DOMAIN) MINDOMAIN;
					}
				    if (low &&
					(TupGField(low, i) > x))
					{
					    if (!found)
						{
						    found = TRUE;
						    ((void) fprintf(f, "    if (0L != (unsigned long) ("));
						    PrintSymbol(f,
								SytGCffSym(symmultup));
						    ((void) fprintf(f, "))\n"), nOutpLines++);
						    ((void) fprintf(f, "        {\n"), nOutpLines++);
						}
					    ((void) fprintf(f, "            if (((DOMAIN)"));
                                            ((void) fprintf(f, " %s", DomStr(TupGField(low, i))));
					    ((void) fprintf(f, ") > (DOMAIN) ("));
					    PrintSymbol(f, symbol);
					    ((void) fprintf(f, "))\n"), nOutpLines++);
					    ((void) fprintf(f, "                return;\n"), nOutpLines++);
					}
				    if (variable)
					x = VarGHigh(variable);
				    else
					x = (DOMAIN) MAXDOMAIN;
				    if (high &&
					(TupGField(high, i) < x))
					{
					    if (!found)
						{
						    found = TRUE;
						    ((void) fprintf(f, "    if (0L != (unsigned long) ("));
						    PrintSymbol(f,
								SytGCffSym(symmultup));
						    ((void) fprintf(f, "))\n"), nOutpLines++);
						    ((void) fprintf(f, "        {\n"), nOutpLines++);
						}
					    ((void) fprintf(f, "            if (((DOMAIN)"));
                                            ((void) fprintf(f, " %s", DomStr(TupGField(high, i))));
					    ((void) fprintf(f, ") < (DOMAIN) ("));
					    PrintSymbol(f, symbol);
					    ((void) fprintf(f, "))\n"), nOutpLines++);
					    ((void) fprintf(f, "                return;\n"), nOutpLines++);
					}
				}
			    if (found)
				((void) fprintf(f, "        }\n"), nOutpLines++);
			}
		    else
			{
			    for (i = 0; i < SytGArity(symmultup); i++)
				{
				    symbol = SytGSymbol(symmultup, i);
				    if (SymIsVariable(symbol) ||
					SymSatisfied(symbol))
					continue;
				    if (low &&
					(TupGField(low, i)
					 > (DOMAIN) MINDOMAIN))
					{
					    ((void) fprintf(f, "    if (((DOMAIN)"));
                                            ((void) fprintf(f, " %s", DomStr(TupGField(low, i))));
					    ((void) fprintf(f, ") > (DOMAIN) ("));
					    PrintSymbol(f, symbol);
					    ((void) fprintf(f, "))\n"), nOutpLines++);
					    ((void) fprintf(f, "        return;\n"), nOutpLines++);
					}
				    if (high &&
					(TupGField(high, i)
					 < (DOMAIN) MAXDOMAIN))
					{
					    ((void) fprintf(f, "    if (((DOMAIN)"));
                                            ((void) fprintf(f, " %s", DomStr(TupGField(high, i))));
					    ((void) fprintf(f, ") < (DOMAIN) ("));
					    PrintSymbol(f, symbol);
					    ((void) fprintf(f, "))\n"), nOutpLines++);
					    ((void) fprintf(f, "        return;\n"), nOutpLines++);
					}
				}
			}
		}
	}
    if (NetGNofClasses(net) != 1)
	{
	    ((void) fprintf(f, "    if (_class >= %s", UnsStr(NetGNofClasses(net))), (void) fprintf(f, ")\n"), nOutpLines++);
	    ((void) fprintf(f, "        {\n"), nOutpLines++);
	    ((void) fprintf(f, "            (void) fprintf(_out, \"\\n\");\n"), nOutpLines++);
	    ((void) fprintf(f, "            (void) fflush(_out);\n"), nOutpLines++);
	    ((void) fprintf(f, "            (void) fprintf(_err,\n"), nOutpLines++);
	    ((void) fprintf(f, "                           \"\\nTransition \\\"%s\\\": precedence class %%u\\nis meaningless. Meaningful classes have a number >= 0 and < %s", TraGName(transition), UnsStr(NetGNofClasses(net))), (void) fprintf(f, "\\n\",\n"), nOutpLines++);
	    ((void) fprintf(f, "                           _class);\n"), nOutpLines++);
	    ((void) fprintf(f, "            (void) fflush(_err);\n"), nOutpLines++);
	    ((void) fprintf(f, "            abort();\n"), nOutpLines++);
	    ((void) fprintf(f, "        }\n"), nOutpLines++);
	    ((void) fprintf(f, "    if (_kind[_class] == _CL_NOTHING)\n"), nOutpLines++);
	    ((void) fprintf(f, "        return;\n"), nOutpLines++);
	}
    if (NetGMonPlace(net) && !NetGAtoms(net))
	{
	    LSTFOR (TraGInputs(transition), arc, ARC)
		{
		    if (ArcGPlace(arc) == NetGMonPlace(net))
			{
			    symmultup = (SYMMULTUP *)
				LstFirst(ArcGSymsum(arc));
			    break;
			}
		}
	    if (arc && SytGCffSym(symmultup))
		{
		    LSTFOR (TraGOutputs(transition), arc, ARC)
			{
			    if (ArcGPlace(arc) == NetGMonPlace(net))
				{
				    syt2 = (SYMMULTUP *)
					LstFirst(ArcGSymsum(arc));
				    break;
				}
			}
		    ((void) fprintf(f, "    if (((unsigned long) ("));
		    PrintSymbol(f, SytGCffSym(symmultup));
		    ((void) fprintf(f, ")) != (unsigned long) ("));
		    PrintSymbol(f, SytGCffSym(syt2));
		    ((void) fprintf(f, "))\n"), nOutpLines++);
		    ((void) fprintf(f, "        {\n"), nOutpLines++);
		    ((void) fprintf(f, "            (void) fprintf(_out, \"\\n\");\n"), nOutpLines++);
		    ((void) fprintf(f, "            (void) fflush(_out);\n"), nOutpLines++);
		    ((void) fprintf(f, "            (void) fprintf(_err,\n"), nOutpLines++);
		    ((void) fprintf(f, "                           \"\\nAttempt to change the multiplicity of the tester \\\"%s\\\":\\ninput weight %%lu, output weight %%lu\\n\",\n", PlaGName(NetGMonPlace(net))), nOutpLines++);
		    ((void) fprintf(f, "                           (unsigned long) ("));
		    PrintSymbol(f, SytGCffSym(symmultup));
		    ((void) fprintf(f, "),\n"), nOutpLines++);
		    ((void) fprintf(f, "                           (unsigned long) ("));
		    PrintSymbol(f, SytGCffSym(syt2));
		    ((void) fprintf(f, "));\n"), nOutpLines++);
		    ((void) fprintf(f, "            (void) fflush(_err);\n"), nOutpLines++);
		    ((void) fprintf(f, "            abort();\n"), nOutpLines++);
		    ((void) fprintf(f, "        }\n"), nOutpLines++);
		    ((void) fprintf(f, "    _visible = (unsigned) (0L != (unsigned long) ("));
		    PrintSymbol(f, SytGCffSym(symmultup));
		    ((void) fprintf(f, "));\n"), nOutpLines++);
		}
	    else if (arc)
		((void) fprintf(f, "    _visible = 1;\n"), nOutpLines++);
	    else
		((void) fprintf(f, "    _visible = 0;\n"), nOutpLines++);
	}
}

/***************************************************************************
*
* Function:  WhatInitial
* Purpose:   This routine produces those limit and gate checks that are
*            made before any variable is set.
* Arguments: checkArcs - the limit checks are limited to these arcs
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, WhatInitial,
    LIST *, checkArcs)
{
    LSTFOR (TraGVariables(transition), var2, VARIABLE)
	VarReset(var2);
    LSTFOR (TraGArcs(transition), arc2, ARC)
	LSTFOR (ArcGSymsum(arc2), syt2, SYMMULTUP)
	    for (j = 0; j < SytGArity(syt2); j++)
		SymReset(SytGSymbol(syt2, j));
    LSTFOR (TraGGates(transition), gate, SYMBOL)
	{
	    SymReset(gate);
	    if (!SymSatisfied(gate))
		continue;
	    SymSet(gate);
	    ((void) fprintf(f, "    if (!("));
	    PrintSymbol(f, gate);
	    ((void) fprintf(f, "))\n"), nOutpLines++);
	    ((void) fprintf(f, "        return;\n"), nOutpLines++);
	}
    LSTFOR (checkArcs, arc2, ARC)
	{
	    LSTFOR (ArcGSymsum(arc2), syt2, SYMMULTUP)
		{
		    ylow = NULL;
		    yhigh = NULL;
		    if (PlaGLow(ArcGPlace(arc2)))
			LSTFOR (PlaGLow(ArcGPlace(arc2)), ylow, TUPLE)
			    {
				if (TupGArity(ylow) == SytGArity(syt2))
				    break;
				else if (TupGArity(ylow) > SytGArity(syt2))
				    {
					ylow = NULL;
					break;
				    }
			    }
		    if (PlaGHigh(ArcGPlace(arc2)))
			LSTFOR (PlaGHigh(ArcGPlace(arc2)), yhigh, TUPLE)
			    {
				if (TupGArity(yhigh) == SytGArity(syt2))
				    break;
				else if (TupGArity(yhigh) > SytGArity(syt2))
				    {
					yhigh = NULL;
					break;
				    }
			    }
		    if ((!(csm2 = SytGCffSym(syt2))) ||
			!SymSatisfied(csm2))
			continue;
		    found = FALSE;
		    for (j = 0; j < SytGArity(syt2); j++)
			{
			    if (!SymSatisfied(sym2 = SytGSymbol(syt2, j)))
				continue;
			    SymSet(sym2);
			    if (ylow &&
				((y = TupGField(ylow, j))
				 > (DOMAIN) MINDOMAIN))
				{
				    if (!found)
					{
					    found = TRUE;
					    ((void) fprintf(f, "    if (0L !="));
					    ((void) fprintf(f, " (unsigned long) ("));
					    PrintSymbol(f, csm2);
					    ((void) fprintf(f, "))\n"), nOutpLines++);
					    ((void) fprintf(f, "        {\n"), nOutpLines++);
					}
                                    ((void) fprintf(f, "            if (((DOMAIN) %s", DomStr(y)), (void) fprintf(f, ")"));
				    ((void) fprintf(f, " > (DOMAIN) ("));
				    PrintSymbol(f, sym2);
				    ((void) fprintf(f, "))\n"), nOutpLines++);
				    ((void) fprintf(f, "                return;\n"), nOutpLines++);
				}
			    if (yhigh &&
				((y = TupGField(yhigh, j))
				 < (DOMAIN) MAXDOMAIN))
				{
				    if (!found)
					{
					    found = TRUE;
					    ((void) fprintf(f, "    if (0L !="));
					    ((void) fprintf(f, " (unsigned long) ("));
					    PrintSymbol(f, csm2);
					    ((void) fprintf(f, "))\n"), nOutpLines++);
					    ((void) fprintf(f, "        {\n"), nOutpLines++);
					}
                                    ((void) fprintf(f, "            if (((DOMAIN) %s", DomStr(y)), (void) fprintf(f, ")"));
				    ((void) fprintf(f, " < (DOMAIN) ("));
				    PrintSymbol(f, sym2);
				    ((void) fprintf(f, "))\n"), nOutpLines++);
				    ((void) fprintf(f, "                return;\n"), nOutpLines++);
				}
			}
		    if (found)
			((void) fprintf(f, "        }\n"), nOutpLines++);
		}
	    LSTFOR (ArcGSymsum(arc2), syt2, SYMMULTUP)
		{
		    if (SytGCffSym(syt2))
			continue;
		    for (j = 0; j < SytGArity(syt2); j++)
			{
			    if (!SymSatisfied(sym2 = SytGSymbol(syt2, j)))
				continue;
			    SymSet(sym2);
			    if (ylow &&
				((y = TupGField(ylow, j))
				 > (DOMAIN) MINDOMAIN))
				{
                                    ((void) fprintf(f, "    if (((DOMAIN) %s", DomStr(y)), (void) fprintf(f, ")"));
				    ((void) fprintf(f, " > (DOMAIN) ("));
				    PrintSymbol(f, sym2);
				    ((void) fprintf(f, "))\n"), nOutpLines++);
				    ((void) fprintf(f, "        return;\n"), nOutpLines++);
				}
			    if (yhigh &&
				((y = TupGField(yhigh, j))
				 < (DOMAIN) MAXDOMAIN))
				{
                                    ((void) fprintf(f, "    if (((DOMAIN) %s", DomStr(y)), (void) fprintf(f, ")"));
				    ((void) fprintf(f, " < (DOMAIN) ("));
				    PrintSymbol(f, sym2);
				    ((void) fprintf(f, "))\n"), nOutpLines++);
				    ((void) fprintf(f, "        return;\n"), nOutpLines++);
				}
			}
		}
	    LSTFOR (ArcGSymsum(arc2), syt2, SYMMULTUP)
		if (!SytGCffSym(syt2))
		    for (j = 0; j < SytGArity(syt2); j++)
			if (SymIsVariable(sym2 = SytGSymbol(syt2, j)))
			    SymSet(sym2);
	}
}
