/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  gen.tpl
*
* Part of: PROD's program prpp
*
* Authors: Marko Rauhamaa and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     This module makes C functions from high level transitions.
*     The made functions are needed in the generation of a reachability
*     graph.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include "gen.h"

FILE *f;
unsigned long fileCount;
NET *net;
TRANSITION *transition;
unsigned t;
PLACE *place;
unsigned p;
ARC *arc, *arc2;
unsigned a, a2;
unsigned ca, ca2;
SYMMULTUP *symmultup, *syt2;
unsigned s, s2;
unsigned cs, cs2;
char *ls, *ls2;
VARIABLE *variable, *var2;
DOMAIN x, y;
TUPLE *low, *high, *xlow, *xhigh, *ylow, *yhigh;
unsigned i, j, iVarCount, dimension, maxDim;
SYMBOL *symbol, *sym2, *cffSym, *csm2, *gate;
char *str;
VOID *acursor, *scursor;
PRECEDENCE *precedence, *priority;
LIST *cffOrdVariables;
LIST *cffOrdInputs;
LIST *cffOrdSymsum;
BOOL found, pre, post, outa, comp, cvars, priNonTriv;
unsigned thisDepth, maxDepth;
FORMULA *atom;
unsigned long mask;
unsigned letVarCount;

/***************************************************************************
*
* Function:  DefsDecls
* Purpose:   This routine produces the "include" and "define" directives
*            and variable introductions that are needed in the
*            functions corresponding to high level transitions.
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, DefsDecls)
{
    @@##include <stdio.h>
    @@##include "absolute.h"
    @@##include "prod.h"
    @@##include "alloc.h"
    @@##include "dynam.h"
    @@##include "ptrbl.h"
    @@##include "baltree.h"
    @@##include "bdd.h"
    @@##include "gphutil.h"
    @@##include "graph.h"
    @@##include "mark.h"
    @@##include "inst.h"
    @@##include "encode.h"
    @@##include "ptnet.h"
    @@##include "ptsymm.h"
    @@
    if (!LstEmpty(NetGTransitions(net)))
	{
	    @@extern TRN *_trn;
	    @@extern unsigned _visible, _len;
	}
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	if ((!LstEmpty(TraGInputs(transition))) ||
	    !LstEmpty(TraGOutputs(transition)))
	{
	    @@extern MARKING *_marking;
	    break;
	}
    if (NetGAtoms(net))
	{
	    @@extern DOMAIN *_y_;
	    @@extern BOOL _someEnb;
	}
    pre = FALSE;
    post = FALSE;
    maxDepth = 0;
    if (NetGAtoms(net))
	{
	    LSTFOR (NetGAtoms(net), atom, FORMULA)
		{
		    thisDepth = XprDepth(atom->uf.expression);
		    if (thisDepth > maxDepth)
			maxDepth = thisDepth;
		}
	    if (maxDepth != 0)
		{
		    @@
		    @@typedef union
		    @@{
		    @@    DOMAIN d;
		    @@    struct
		    @@    {
		    @@        char *c;
		    @@        MARKING *ma;
		    @@        TUPLE *t;
		    @@    } a;
		    @@    struct
		    @@    {
		    @@        MARKING *mb;
		    @@        TUPLE *t0;
		    @@        TUPLE *t1;
		    @@        TUPLE *t2;
		    @@    } b;
		    @@} XPRVAL;
		    @@
		    @@extern XPRVAL _xprVal[%maxDepth%];
		}
	    @@extern TUPLE *_xprTuple;
	    @@extern DOMAIN _xprDom;
	    @@extern MARKING *_xprMk, *_tmpMk;
	    @@extern unsigned _i_;
	}
    maxDim = 0;
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	{
	    dimension = 0;
	    LSTFOR (TraGInputs(transition), arc, ARC)
		dimension = dimension + LstSize(ArcGSymsum(arc));
	    if (dimension > maxDim)
		maxDim = dimension;
	}
    if (maxDim != 0)
	@@extern MULTUP *_t[%maxDim%];
    maxDim = 0;
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	{
	    dimension = 0;
	    LSTFOR (TraGInputs(transition), arc, ARC)
		{
		    if (!LstEmpty(ArcGSymsum(arc)))
			dimension = dimension + LstSize(ArcGSymsum(arc)) - 1;
		}
	    if (dimension > maxDim)
		maxDim = dimension;
	}
    if (maxDim != 0)
	@@extern VOID *_u[%maxDim%];
    maxDim = 0;
    outa = FALSE;
    comp = FALSE;
    cvars = FALSE;
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	{
	    dimension = LstSize(TraGVariables(transition));
	    if (dimension > maxDim)
		maxDim = dimension;
	    if (!LstEmpty(TraGOutputs(transition)))
		outa = TRUE;
	    if (TraGComputation(transition))
		{
		    comp = TRUE;
		    if (!cvars)
			LSTFOR (TraGVariables(transition), variable, VARIABLE)
			    if (!VarIsIn(variable))
				{
				    cvars = TRUE;
				    break;
				}
		}
	}
    if (maxDim != 0)
	@@extern DOMAIN _v[%maxDim%];
    if (outa)
	@@extern TUPLE *_tuple, *_w;
    else
	@@extern TUPLE *_w;
    if (comp)
	{
	    @@extern LIST *_list;
	    if (cvars)
		@@extern VOID *_cursor;
	}
    priNonTriv = FALSE;
    if (NetGNofClasses(net) != 1)
	LSTFOR (NetGPrecedences(net), precedence, PRECEDENCE)
	    if (PreGSucc(precedence) && (!LstEmpty(PreGSucc(precedence)))
		&& (!PreGEasy(precedence)) && !PreGSelfLoop(precedence))
		{
		    priNonTriv = TRUE;
		    break;
		}
    if (cvars || (NetGNofClasses(net) != 1))
	{
	    @@extern DOMAIN *_instance;
	    @@extern PEER *_peer;
	}
    if (priNonTriv)
	{
	    @@extern PEER *_hpeer;
	    @@extern unsigned _lwidth, _field;
	    @@extern DOMAIN *_hfolding;
	}
    if (NetGNofClasses(net) != 1)
	{
	    @@extern LIST *_bundle[%NetGNofClasses(net)%];
	    @@extern LIST *_tmpBundle[%NetGNofClasses(net)%];
            @@##define _CL_INDEPENDENT 0
            @@##define _CL_MAXIMUM 1
            @@##define _CL_MEDIUM 2
            @@##define _CL_INHIBITOR 3
            @@##define _CL_NOTHING 4
	    @@extern unsigned _kind[%NetGNofClasses(net)%];
	    @@extern unsigned _class;
	}
}

/***************************************************************************
*
* Function:  ActualDecls
* Purpose:   This routine produces the actual variable declarations
*            that are needed in the functions corresponding to high
*            level transitions.
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, ActualDecls)
{
    if (!LstEmpty(NetGTransitions(net)))
	{
	    @@TRN *_trn;
	    @@unsigned _visible, _len;
	}
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	if ((!LstEmpty(TraGInputs(transition))) ||
	    !LstEmpty(TraGOutputs(transition)))
	{
	    @@MARKING *_marking;
	    break;
	}
    if (NetGAtoms(net))
	{
	    @@DOMAIN *_y_;
	    @@BOOL _someEnb;
	}
    pre = FALSE;
    post = FALSE;
    maxDepth = 0;
    if (NetGAtoms(net))
	{
	    LSTFOR (NetGAtoms(net), atom, FORMULA)
		{
		    thisDepth = XprDepth(atom->uf.expression);
		    if (thisDepth > maxDepth)
			maxDepth = thisDepth;
		}
	    if (maxDepth != 0)
		@@XPRVAL _xprVal[%maxDepth%];
	    @@TUPLE *_xprTuple;
	    @@DOMAIN _xprDom;
	    @@MARKING *_xprMk, *_tmpMk;
	    @@unsigned _i_;
	}
    maxDim = 0;
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	{
	    dimension = 0;
	    LSTFOR (TraGInputs(transition), arc, ARC)
		dimension = dimension + LstSize(ArcGSymsum(arc));
	    if (dimension > maxDim)
		maxDim = dimension;
	}
    if (maxDim != 0)
	@@MULTUP *_t[%maxDim%];
    maxDim = 0;
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	{
	    dimension = 0;
	    LSTFOR (TraGInputs(transition), arc, ARC)
		{
		    if (!LstEmpty(ArcGSymsum(arc)))
			dimension = dimension + LstSize(ArcGSymsum(arc)) - 1;
		}
	    if (dimension > maxDim)
		maxDim = dimension;
	}
    if (maxDim != 0)
	@@VOID *_u[%maxDim%];
    maxDim = 0;
    outa = FALSE;
    comp = FALSE;
    cvars = FALSE;
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	{
	    dimension = LstSize(TraGVariables(transition));
	    if (dimension > maxDim)
		maxDim = dimension;
	    if (!LstEmpty(TraGOutputs(transition)))
		outa = TRUE;
	    if (TraGComputation(transition))
		{
		    comp = TRUE;
		    if (!cvars)
			LSTFOR (TraGVariables(transition), variable, VARIABLE)
			    if (!VarIsIn(variable))
				{
				    cvars = TRUE;
				    break;
				}
		}
	}
    if (maxDim != 0)
	@@DOMAIN _v[%maxDim%];
    if (outa)
	@@TUPLE *_tuple, *_w;
    else
	@@TUPLE *_w;
    if (comp)
	{
	    @@LIST *_list;
	    if (cvars)
		@@VOID *_cursor;
	}
    priNonTriv = FALSE;
    if (NetGNofClasses(net) != 1)
	LSTFOR (NetGPrecedences(net), precedence, PRECEDENCE)
	    if (PreGSucc(precedence) && (!LstEmpty(PreGSucc(precedence)))
		&& (!PreGEasy(precedence)) && !PreGSelfLoop(precedence))
		{
		    priNonTriv = TRUE;
		    break;
		}
    if (cvars || (NetGNofClasses(net) != 1))
	{
	    @@DOMAIN *_instance;
	    @@PEER *_peer;
	}
    if (priNonTriv)
	{
	    @@PEER *_hpeer;
	    @@unsigned _lwidth, _field;
	    @@DOMAIN *_hfolding;
	}
    if (NetGNofClasses(net) != 1)
	{
	    @@LIST *_bundle[%NetGNofClasses(net)%];
	    @@LIST *_tmpBundle[%NetGNofClasses(net)%];
	    @+unsigned _kind[%NetGNofClasses(net)%] = {0
	    LSTFOR (NetGPrecedences(net), precedence, PRECEDENCE)
		{
		    if (PreGEasy(precedence))
			@+,4
		    else if (PreGSelfLoop(precedence))
			@+,3
		    else if (PreGSucc(precedence) &&
			     !LstEmpty(PreGSucc(precedence)))
			@+,2
		    else if (PreGPred(precedence) &&
			     !LstEmpty(PreGPred(precedence)))
			@+,1
		    else
			@+,0
		}
	    @@};
	    @@unsigned _class;
	}
}

/***************************************************************************
*
* Function:  ADLimits
* Purpose:   This routine produces those checks that are made in Accept().
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, ADLimits)
{
    LSTFOR (TraGVariables(transition), variable, VARIABLE)
	{
	    if (VarIsIn(variable))
		VarSet(variable);
	    if (VarIsIn(variable) || !VarHard(variable))
		continue;
	    VarReset(variable);
	    if (VarGLow(variable) > (DOMAIN) MINDOMAIN)
		{
		    @+    if ($VarGLabel(variable)$ <
                    @@ (DOMAIN) @VarGLow(variable)@)
		    @@        return;
		}
	    if (VarGHigh(variable) < (DOMAIN) MAXDOMAIN)
		{
		    @+    if ($VarGLabel(variable)$ >
                    @@ (DOMAIN) @VarGHigh(variable)@)
		    @@        return;
		}
	}
    LSTFOR (TraGOutputs(transition), arc, ARC)
	{
	    place = ArcGPlace(arc);
	    LSTFOR (ArcGSymsum(arc), symmultup, SYMMULTUP)
		{
		    low = NULL;
		    high = NULL;
		    if (PlaGLow(place))
			LSTFOR (PlaGLow(place), low, TUPLE)
			    {
				if (TupGArity(low) == SytGArity(symmultup))
				    break;
				else if (TupGArity(low) > SytGArity(symmultup))
				    {
					low = NULL;
					break;
				    }
			    }
		    if (PlaGHigh(place))
			LSTFOR (PlaGHigh(place), high, TUPLE)
			    {
				if (TupGArity(high) == SytGArity(symmultup))
				    break;
				else if (TupGArity(high) >
					 SytGArity(symmultup))
				    {
					high = NULL;
					break;
				    }
			    }
		    if ((!low) && !high)
			continue;
		    if (SytGCffSym(symmultup))
			{
			    found = FALSE;
			    for (i = 0; i < SytGArity(symmultup); i++)
				{
				    symbol = SytGSymbol(symmultup, i);
				    if (SymSatisfied(symbol))
					continue;
				    if (SymIsVariable(symbol)
					&& VarHard(variable =
						   SymGVariable(symbol)))
					{
					    variable = SymGVariable(symbol);
					    x = VarGLow(variable);
					}
				    else
					{
					    variable = NULL;
					    x = (DOMAIN) MINDOMAIN;
					}
				    if (low &&
					(TupGField(low, i) > x))
					{
					    if (!found)
						{
						    found = TRUE;
						    @+    if (0L != (unsigned long) (
						    PrintSymbol(f,
								SytGCffSym(symmultup));
						    @@))
						    @@        {
						}
					    @+            if (((DOMAIN)
                                            @+ @TupGField(low, i)@
					    @+) > (DOMAIN) (
					    PrintSymbol(f, symbol);
					    @@))
					    @@                return;
					}
				    if (variable)
					x = VarGHigh(variable);
				    else
					x = (DOMAIN) MAXDOMAIN;
				    if (high &&
					(TupGField(high, i) < x))
					{
					    if (!found)
						{
						    found = TRUE;
						    @+    if (0L != (unsigned long) (
						    PrintSymbol(f,
								SytGCffSym(symmultup));
						    @@))
						    @@        {
						}
					    @+            if (((DOMAIN)
                                            @+ @TupGField(high, i)@
					    @+) < (DOMAIN) (
					    PrintSymbol(f, symbol);
					    @@))
					    @@                return;
					}
				}
			    if (found)
				@@        }
			}
		    else
			{
			    for (i = 0; i < SytGArity(symmultup); i++)
				{
				    symbol = SytGSymbol(symmultup, i);
				    if (SymIsVariable(symbol) ||
					SymSatisfied(symbol))
					continue;
				    if (low &&
					(TupGField(low, i)
					 > (DOMAIN) MINDOMAIN))
					{
					    @+    if (((DOMAIN)
                                            @+ @TupGField(low, i)@
					    @+) > (DOMAIN) (
					    PrintSymbol(f, symbol);
					    @@))
					    @@        return;
					}
				    if (high &&
					(TupGField(high, i)
					 < (DOMAIN) MAXDOMAIN))
					{
					    @+    if (((DOMAIN)
                                            @+ @TupGField(high, i)@
					    @+) < (DOMAIN) (
					    PrintSymbol(f, symbol);
					    @@))
					    @@        return;
					}
				}
			}
		}
	}
    if (NetGNofClasses(net) != 1)
	{
	    @@    if (_class >= %NetGNofClasses(net)%)
	    @@        {
	    @@            (void) fprintf(_out, "\n");
	    @@            (void) fflush(_out);
	    @@            (void) fprintf(_err,
	    @@                           "\nTransition \"$TraGName(transition)$\": precedence class %%%%u\nis meaningless. Meaningful classes have a number >= 0 and < %NetGNofClasses(net)%\n",
	    @@                           _class);
	    @@            (void) fflush(_err);
	    @@            abort();
	    @@        }
	    @@    if (_kind[_class] == _CL_NOTHING)
	    @@        return;
	}
    if (NetGMonPlace(net) && !NetGAtoms(net))
	{
	    LSTFOR (TraGInputs(transition), arc, ARC)
		{
		    if (ArcGPlace(arc) == NetGMonPlace(net))
			{
			    symmultup = (SYMMULTUP *)
				LstFirst(ArcGSymsum(arc));
			    break;
			}
		}
	    if (arc && SytGCffSym(symmultup))
		{
		    LSTFOR (TraGOutputs(transition), arc, ARC)
			{
			    if (ArcGPlace(arc) == NetGMonPlace(net))
				{
				    syt2 = (SYMMULTUP *)
					LstFirst(ArcGSymsum(arc));
				    break;
				}
			}
		    @+    if (((unsigned long) (
		    PrintSymbol(f, SytGCffSym(symmultup));
		    @+)) != (unsigned long) (
		    PrintSymbol(f, SytGCffSym(syt2));
		    @@))
		    @@        {
		    @@            (void) fprintf(_out, "\n");
		    @@            (void) fflush(_out);
		    @@            (void) fprintf(_err,
		    @@                           "\nAttempt to change the multiplicity of the tester \"$PlaGName(NetGMonPlace(net))$\":\ninput weight %%%%lu, output weight %%%%lu\n",
		    @+                           (unsigned long) (
		    PrintSymbol(f, SytGCffSym(symmultup));
		    @@),
		    @+                           (unsigned long) (
		    PrintSymbol(f, SytGCffSym(syt2));
		    @@));
		    @@            (void) fflush(_err);
		    @@            abort();
		    @@        }
		    @+    _visible = (unsigned) (0L != (unsigned long) (
		    PrintSymbol(f, SytGCffSym(symmultup));
		    @@));
		}
	    else if (arc)
		@@    _visible = 1;
	    else
		@@    _visible = 0;
	}
}

/***************************************************************************
*
* Function:  WhatInitial
* Purpose:   This routine produces those limit and gate checks that are
*            made before any variable is set.
* Arguments: checkArcs - the limit checks are limited to these arcs
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, WhatInitial,
    LIST *, checkArcs)
{
    LSTFOR (TraGVariables(transition), var2, VARIABLE)
	VarReset(var2);
    LSTFOR (TraGArcs(transition), arc2, ARC)
	LSTFOR (ArcGSymsum(arc2), syt2, SYMMULTUP)
	    for (j = 0; j < SytGArity(syt2); j++)
		SymReset(SytGSymbol(syt2, j));
    LSTFOR (TraGGates(transition), gate, SYMBOL)
	{
	    SymReset(gate);
	    if (!SymSatisfied(gate))
		continue;
	    SymSet(gate);
	    @+    if (!(
	    PrintSymbol(f, gate);
	    @@))
	    @@        return;
	}
    LSTFOR (checkArcs, arc2, ARC)
	{
	    LSTFOR (ArcGSymsum(arc2), syt2, SYMMULTUP)
		{
		    ylow = NULL;
		    yhigh = NULL;
		    if (PlaGLow(ArcGPlace(arc2)))
			LSTFOR (PlaGLow(ArcGPlace(arc2)), ylow, TUPLE)
			    {
				if (TupGArity(ylow) == SytGArity(syt2))
				    break;
				else if (TupGArity(ylow) > SytGArity(syt2))
				    {
					ylow = NULL;
					break;
				    }
			    }
		    if (PlaGHigh(ArcGPlace(arc2)))
			LSTFOR (PlaGHigh(ArcGPlace(arc2)), yhigh, TUPLE)
			    {
				if (TupGArity(yhigh) == SytGArity(syt2))
				    break;
				else if (TupGArity(yhigh) > SytGArity(syt2))
				    {
					yhigh = NULL;
					break;
				    }
			    }
		    if ((!(csm2 = SytGCffSym(syt2))) ||
			!SymSatisfied(csm2))
			continue;
		    found = FALSE;
		    for (j = 0; j < SytGArity(syt2); j++)
			{
			    if (!SymSatisfied(sym2 = SytGSymbol(syt2, j)))
				continue;
			    SymSet(sym2);
			    if (ylow &&
				((y = TupGField(ylow, j))
				 > (DOMAIN) MINDOMAIN))
				{
				    if (!found)
					{
					    found = TRUE;
					    @+    if (0L !=
					    @+ (unsigned long) (
					    PrintSymbol(f, csm2);
					    @@))
					    @@        {
					}
                                    @+            if (((DOMAIN) @y@)
				    @+ > (DOMAIN) (
				    PrintSymbol(f, sym2);
				    @@))
				    @@                return;
				}
			    if (yhigh &&
				((y = TupGField(yhigh, j))
				 < (DOMAIN) MAXDOMAIN))
				{
				    if (!found)
					{
					    found = TRUE;
					    @+    if (0L !=
					    @+ (unsigned long) (
					    PrintSymbol(f, csm2);
					    @@))
					    @@        {
					}
                                    @+            if (((DOMAIN) @y@)
				    @+ < (DOMAIN) (
				    PrintSymbol(f, sym2);
				    @@))
				    @@                return;
				}
			}
		    if (found)
			@@        }
		}
	    LSTFOR (ArcGSymsum(arc2), syt2, SYMMULTUP)
		{
		    if (SytGCffSym(syt2))
			continue;
		    for (j = 0; j < SytGArity(syt2); j++)
			{
			    if (!SymSatisfied(sym2 = SytGSymbol(syt2, j)))
				continue;
			    SymSet(sym2);
			    if (ylow &&
				((y = TupGField(ylow, j))
				 > (DOMAIN) MINDOMAIN))
				{
                                    @+    if (((DOMAIN) @y@)
				    @+ > (DOMAIN) (
				    PrintSymbol(f, sym2);
				    @@))
				    @@        return;
				}
			    if (yhigh &&
				((y = TupGField(yhigh, j))
				 < (DOMAIN) MAXDOMAIN))
				{
                                    @+    if (((DOMAIN) @y@)
				    @+ < (DOMAIN) (
				    PrintSymbol(f, sym2);
				    @@))
				    @@        return;
				}
			}
		}
	    LSTFOR (ArcGSymsum(arc2), syt2, SYMMULTUP)
		if (!SytGCffSym(syt2))
		    for (j = 0; j < SytGArity(syt2); j++)
			if (SymIsVariable(sym2 = SytGSymbol(syt2, j)))
			    SymSet(sym2);
	}
}
