/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  gen2.tpl
*
* Part of: PROD's program prpp
*
* Authors: Marko Rauhamaa and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     See gen.tpl.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include "gen.h"
    
/***************************************************************************
*
* Function:  WhatNew
* Purpose:   This routine produces checks that are made after the setting
*            of some input variable.
* Arguments: checkArcs - the limit checks are limited to these arcs
*            arcContext - FALSE if and only if the produce piece of code
*                         is for a function that is called when the net
*                         is unfolded
* Returns:
* See also:
*
****************************************************************************
*/
FE2(void, WhatNew,
    LIST *, checkArcs,
    BOOL, arcContext)
{
    LSTFOR (TraGGates(transition), gate, SYMBOL)
	{
	    if (SymMarked(gate) || !SymSatisfied(gate))
		continue;
	    SymSet(gate);
	    ((void) fprintf(f, "    if (!("));
	    PrintSymbol(f, gate);
	    ((void) fprintf(f, "))\n"), nOutpLines++);
	    if (arcContext)
		((void) fprintf(f, "        goto _c%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
	    else if (placeLimits)
		{
		    VarSEscape(variable);
		    ((void) fprintf(f, "        goto _q%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
		}
	    else
		((void) fprintf(f, "        return;\n"), nOutpLines++);
	}
    if (!placeLimits)
	return;
    LSTFOR (checkArcs, arc2, ARC)
	{
	    LSTFOR (ArcGSymsum(arc2), syt2, SYMMULTUP)
		{
		    ylow = NULL;
		    yhigh = NULL;
		    if (PlaGLow(ArcGPlace(arc2)))
			LSTFOR (PlaGLow(ArcGPlace(arc2)), ylow, TUPLE)
			    {
				if (TupGArity(ylow) == SytGArity(syt2))
				    break;
				else if (TupGArity(ylow) > SytGArity(syt2))
				    {
					ylow = NULL;
					break;
				    }
			    }
		    if (PlaGHigh(ArcGPlace(arc2)))
			LSTFOR (PlaGHigh(ArcGPlace(arc2)), yhigh, TUPLE)
			    {
				if (TupGArity(yhigh) == SytGArity(syt2))
				    break;
				else if (TupGArity(yhigh) > SytGArity(syt2))
				    {
					yhigh = NULL;
					break;
				    }
			    }
		    if ((!(csm2 = SytGCffSym(syt2)))
			|| !SymSatisfied(csm2))
			continue;
		    found = FALSE;
		    for (j = 0; j < SytGArity(syt2); j++)
			{
			    if (SymMarked(sym2 = SytGSymbol(syt2, j))
				|| !SymSatisfied(sym2))
				continue;
			    SymSet(sym2);
			    if (SymIsVariable(sym2) &&
				VarHard(var2 = SymGVariable(sym2)))
				{
				    x = VarGLow(var2);
				    if (x < VarGAuxLow(var2))
					x = (DOMAIN) MAXDOMAIN;
				}
			    else
				{
				    var2 = NULL;
				    x = (DOMAIN) MINDOMAIN;
				}
			    if (ylow &&
				((y = TupGField(ylow, j)) > x))
				{
				    if (!found)
					{
					    found = TRUE;
					    ((void) fprintf(f, "    if (0L !="));
					    ((void) fprintf(f, " (unsigned long) ("));
					    PrintSymbol(f, csm2);
					    ((void) fprintf(f, "))\n"), nOutpLines++);
					    ((void) fprintf(f, "        {\n"), nOutpLines++);
					}
                                    ((void) fprintf(f, "            if (((DOMAIN) %s", DomStr(y)), (void) fprintf(f, ")"));
				    ((void) fprintf(f, " > (DOMAIN) ("));
				    PrintSymbol(f, sym2);
				    ((void) fprintf(f, "))\n"), nOutpLines++);
				    if (arcContext)
					((void) fprintf(f, "                goto _c%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
				    else
					{
					    VarSEscape(variable);
					    ((void) fprintf(f, "                goto _q%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
					}
				}
			    if (var2)
				{
				    x = VarGHigh(var2);
				    if (x > VarGAuxHigh(var2))
					x = (DOMAIN) MINDOMAIN;
				}
			    else
				x = (DOMAIN) MAXDOMAIN;
			    if (yhigh &&
				((y = TupGField(yhigh, j)) < x))
				{
				    if (!found)
					{
					    found = TRUE;
					    ((void) fprintf(f, "    if (0L !="));
					    ((void) fprintf(f, " (unsigned long) ("));
					    PrintSymbol(f, csm2);
					    ((void) fprintf(f, "))\n"), nOutpLines++);
					    ((void) fprintf(f, "        {\n"), nOutpLines++);
					}
                                    ((void) fprintf(f, "            if (((DOMAIN) %s", DomStr(y)), (void) fprintf(f, ")"));
				    ((void) fprintf(f, " < (DOMAIN) ("));
				    PrintSymbol(f, sym2);
				    ((void) fprintf(f, "))\n"), nOutpLines++);
				    if (arcContext)
					((void) fprintf(f, "                goto _c%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
				    else
					{
					    VarSEscape(variable);
					    ((void) fprintf(f, "                goto _q%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
					}
				}
			}
		    if (found)
			((void) fprintf(f, "        }\n"), nOutpLines++);
		}
	    LSTFOR (ArcGSymsum(arc2), syt2, SYMMULTUP)
		{
		    if (SytGCffSym(syt2))
			continue;
		    for (j = 0; j < SytGArity(syt2); j++)
			{
			    if (SymMarked(sym2 = SytGSymbol(syt2, j))
				|| !SymSatisfied(sym2))
				continue;
			    SymSet(sym2);
			    if (ylow &&
				((y = TupGField(ylow, j))
				 > (DOMAIN) MINDOMAIN))
				{
                                    ((void) fprintf(f, "    if (((DOMAIN) %s", DomStr(y)), (void) fprintf(f, ")"));
				    ((void) fprintf(f, " > (DOMAIN) ("));
				    PrintSymbol(f, sym2);
				    ((void) fprintf(f, "))\n"), nOutpLines++);
				    if (arcContext)
					((void) fprintf(f, "        goto _c%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
				    else
					{
					    VarSEscape(variable);
					    ((void) fprintf(f, "        goto _q%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
					}
				}
			    if (yhigh &&
				((y = TupGField(yhigh, j))
				 < (DOMAIN) MAXDOMAIN))
				{
                                    ((void) fprintf(f, "    if (((DOMAIN) %s", DomStr(y)), (void) fprintf(f, ")"));
				    ((void) fprintf(f, " < (DOMAIN) ("));
				    PrintSymbol(f, sym2);
				    ((void) fprintf(f, "))\n"), nOutpLines++);
				    if (arcContext)
					((void) fprintf(f, "        goto _c%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
				    else
					{
					    VarSEscape(variable);
					    ((void) fprintf(f, "        goto _q%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
					}
				}
			}
		}
	}
}

/***************************************************************************
*
* Function:  GenFollowers
* Purpose:   Together with ARoutine and FRoutine, this routine produces
*            the computation and firing of enabled transition instances
*            in the case that the net is not unfolded.
* Arguments:
* Returns:
* See also:  ARoutine, FRoutine
*
****************************************************************************
*/
FE0(void, GenFollowers)
{
    ((void) fprintf(f, "FE0(void, _Followers)\n"), nOutpLines++);
    ((void) fprintf(f, "{\n"), nOutpLines++);
    ((void) fprintf(f, "    ClBlocked();\n"), nOutpLines++);
    if (NetGNofClasses(net) != 1)
	{
	    ((void) fprintf(f, "    for (_class = 0; _class < %s", UnsStr(NetGNofClasses(net))), (void) fprintf(f, "; _class++)\n"), nOutpLines++);
	    ((void) fprintf(f, "        {\n"), nOutpLines++);
	    ((void) fprintf(f, "            _bundle[_class] = LstCreate(&peeTools);\n"), nOutpLines++);
	    ((void) fprintf(f, "            _tmpBundle[_class] = LstCreate(&peeTools);\n"), nOutpLines++);
	    ((void) fprintf(f, "        }\n"), nOutpLines++);
	}
    if (NetGAtoms(net))
	{
	    LSTNUM (NetGTransitions(net), transition, TRANSITION, t)
		if (TraGKind(transition) == TK_INITIAL)
		    break;
	    ((void) fprintf(f, "    if (MultupGField(MarFirst(StaGMarking(_state, /*%s*/MonPlInd())),", PlaGName(NetGMonPlace(net))));
	    ((void) fprintf(f, " (unsigned) 0)\n"), nOutpLines++);
	    ((void) fprintf(f, "        == (DOMAIN)"));
            ((void) fprintf(f, " %s", DomStr(MultupGField(MarFirst(PlaGMarking(NetGMonPlace(net))), (unsigned) 0))), (void) fprintf(f, ")\n"), nOutpLines++);
	    ((void) fprintf(f, "        {\n"), nOutpLines++);
	    ((void) fprintf(f, "            _F%s", UnsStr(t)), (void) fprintf(f, "();\n"), nOutpLines++);
	    ((void) fprintf(f, "            _someEnb = TRUE;\n"), nOutpLines++);
            ((void) fprintf(f, "        }\n"), nOutpLines++);
	    ((void) fprintf(f, "    else\n"), nOutpLines++);
	    ((void) fprintf(f, "        {\n"), nOutpLines++);
	    ((void) fprintf(f, "            _someEnb = FALSE;\n"), nOutpLines++);
	    LSTNUM (NetGTransitions(net), transition, TRANSITION, t)
		{
		    if (TraGKind(transition) == TK_INITIAL)
			continue;
		    if (TraGKind(transition) == TK_DUMMY)
			{
			    if (NetGNofClasses(net) == 1)
				{
				    ((void) fprintf(f, "            if (!_someEnb)\n"), nOutpLines++);
				    ((void) fprintf(f, "                _F%s", UnsStr(t)), (void) fprintf(f, "();\n"), nOutpLines++);
				}
			}
		    else
			((void) fprintf(f, "            _F%s", UnsStr(t)), (void) fprintf(f, "();\n"), nOutpLines++);
		}
	    ((void) fprintf(f, "        }\n"), nOutpLines++);
	}
    else
	{
	    LSTNUM (NetGTransitions(net), transition, TRANSITION, t)
		((void) fprintf(f, "    _F%s", UnsStr(t)), (void) fprintf(f, "();\n"), nOutpLines++);
	}
    if (NetGNofClasses(net) == 1)
	{
	    ((void) fprintf(f, "}\n"), nOutpLines++);
	    return;
	}
    ((void) fprintf(f, "    for (_class = 0; _class < %s", UnsStr(NetGNofClasses(net))), (void) fprintf(f, "; _class++)\n"), nOutpLines++);
    ((void) fprintf(f, "        LstDestroy(_tmpBundle[_class], TRUE);\n"), nOutpLines++);
    LSTNUM (NetGPrecedences(net), precedence, PRECEDENCE, p)
	{
	    if ((!PreGSucc(precedence)) || LstEmpty(PreGSucc(precedence))
		|| PreGEasy(precedence) || PreGSelfLoop(precedence))
		continue;
	    ((void) fprintf(f, "    LSTFOR (_bundle[%s", UnsStr(p + 1)), (void) fprintf(f, "], _peer, PEER)\n"), nOutpLines++);
	    ((void) fprintf(f, "        {\n"), nOutpLines++);
	    ((void) fprintf(f, "            _lwidth = IngGFdProfile(PeeGTrIndex(_peer));\n"), nOutpLines++);
	    ((void) fprintf(f, "            _instance = PeeGInstance(_peer);\n"), nOutpLines++);
	    LSTFOR (PreGSucc(precedence), priority, PRECEDENCE)
		{
		    ((void) fprintf(f, "            LSTFOR (_bundle[%s", UnsStr(PreGValue(priority))), (void) fprintf(f, "],"));
		    ((void) fprintf(f, " _hpeer, PEER)\n"), nOutpLines++);
		    ((void) fprintf(f, "                if (_lwidth =="));
		    ((void) fprintf(f, " IngGFdProfile(PeeGTrIndex(_hpeer)))\n"), nOutpLines++);
		    ((void) fprintf(f, "                    {\n"), nOutpLines++);
		    ((void) fprintf(f, "                        _hfolding = PeeGInstance(_hpeer);\n"), nOutpLines++);
		    ((void) fprintf(f, "                        for (_field = 0;"));
		    ((void) fprintf(f, " _field < _lwidth; _field++)\n"), nOutpLines++);
		    ((void) fprintf(f, "                            if (_instance[_field] !="));
		    ((void) fprintf(f, " _hfolding[_field])\n"), nOutpLines++);
		    ((void) fprintf(f, "                                break;\n"), nOutpLines++);
		    ((void) fprintf(f, "                        if (_field == _lwidth)\n"), nOutpLines++);
		    ((void) fprintf(f, "                            {\n"), nOutpLines++);
		    ((void) fprintf(f, "                                PeeSet(_peer);\n"), nOutpLines++);
		    ((void) fprintf(f, "                                break;\n"), nOutpLines++);
		    ((void) fprintf(f, "                            }\n"), nOutpLines++);
		    ((void) fprintf(f, "                    }\n"), nOutpLines++);
		    if (priority != (PRECEDENCE *)
			LstLast(PreGSucc(precedence)))
			{
			    ((void) fprintf(f, "            if (PeeOn(_peer))\n"), nOutpLines++);
			    ((void) fprintf(f, "                continue;\n"), nOutpLines++);
			}
		}
	    ((void) fprintf(f, "        }\n"), nOutpLines++);
	}
    ((void) fprintf(f, "    for (_class = 0; _class < %s", UnsStr(NetGNofClasses(net))), (void) fprintf(f, "; _class++)\n"), nOutpLines++);
    ((void) fprintf(f, "        {\n"), nOutpLines++);
    ((void) fprintf(f, "            if (_kind[_class] != _CL_MEDIUM)\n"), nOutpLines++);
    ((void) fprintf(f, "                continue;\n"), nOutpLines++);
    ((void) fprintf(f, "            LSTFOR (_bundle[_class], _peer, PEER)\n"), nOutpLines++);
    ((void) fprintf(f, "                {\n"), nOutpLines++);
    ((void) fprintf(f, "                    if (PeeOn(_peer))\n"), nOutpLines++);
    ((void) fprintf(f, "                        continue;\n"), nOutpLines++);
    if (NetGAtoms(net))
	{
	    ((void) fprintf(f, "                    _someEnb = TRUE;\n"), nOutpLines++);
	}
    ((void) fprintf(f, "                    if (((DATATYPE *) NULL) == PeeGBuffer(_peer))\n"), nOutpLines++);
    ((void) fprintf(f, "                        continue;\n"), nOutpLines++);
    ((void) fprintf(f, "                    _ndAddr = GphPutNode(_graph[0], PeeGBuffer(_peer),\n"), nOutpLines++);
    ((void) fprintf(f, "                                         DataLen(PeeGBuffer(_peer)),\n"), nOutpLines++);
    ((void) fprintf(f, "                                         &_back);\n"), nOutpLines++);
    ((void) fprintf(f, "                    _len = InsEncode(PeeGTrIndex(_peer), _class,\n"), nOutpLines++);
    ((void) fprintf(f, "                                     PeeGInstance(_peer), EngABuffer(),\n"), nOutpLines++);
    ((void) fprintf(f, "                                     EngABufSize());\n"), nOutpLines++);
    ((void) fprintf(f, "                    _awAddr = GphPutArrow(_graph[0], EngBuffer(), _len,\n"), nOutpLines++);
    ((void) fprintf(f, "                                          _awCount, PthEnd(nowPath),\n"), nOutpLines++);
    ((void) fprintf(f, "                                          _ndAddr, _awFirst, _back,"));
    ((void) fprintf(f, " TRUE);\n"), nOutpLines++);
    ((void) fprintf(f, "                    if (Ara())\n"), nOutpLines++);
    ((void) fprintf(f, "                        AraStep(PeeVisible(_peer));\n"), nOutpLines++);
    ((void) fprintf(f, "                    if ((_awCount == 0L) || FreeExpansion())\n"), nOutpLines++);
    ((void) fprintf(f, "                        {\n"), nOutpLines++);
    ((void) fprintf(f, "                            _awFirst = _awAddr;\n"), nOutpLines++);
    ((void) fprintf(f, "                            _ndFirst = _ndAddr;\n"), nOutpLines++);
    ((void) fprintf(f, "                        }\n"), nOutpLines++);
    ((void) fprintf(f, "                    _awCount++;\n"), nOutpLines++);
    if (NetGMonPlace(net))
	{
            ((void) fprintf(f, "                    if (((unsigned long) -1L) != _back)\n"), nOutpLines++);
            ((void) fprintf(f, "                        {\n"), nOutpLines++);
            ((void) fprintf(f, "                            if (InfNow())\n"), nOutpLines++);
            ((void) fprintf(f, "                                IllInfTrace(TRUE);\n"), nOutpLines++);
            ((void) fprintf(f, "                            else if"));
            ((void) fprintf(f, " (DivNow() && !PeeVisible(_peer))\n"), nOutpLines++);
            ((void) fprintf(f, "                                IllDivTrace(TRUE);\n"), nOutpLines++);
            ((void) fprintf(f, "                        }\n"), nOutpLines++);
            ((void) fprintf(f, "                    else if (Reject() && PeeVisible(_peer))\n"), nOutpLines++);
            ((void) fprintf(f, "                        {\n"), nOutpLines++);
            ((void) fprintf(f, "                            _folState = StaAlloc();\n"), nOutpLines++);
            ((void) fprintf(f, "                            StaDecode"));
            ((void) fprintf(f, "(PeeGBuffer(_peer), _folState);\n"), nOutpLines++);
            ((void) fprintf(f, "                            IllTrace();\n"), nOutpLines++);
            ((void) fprintf(f, "                            StaDiscard(_folState);\n"), nOutpLines++);
            ((void) fprintf(f, "                        }\n"), nOutpLines++);
	}
    ((void) fprintf(f, "                }\n"), nOutpLines++);
    ((void) fprintf(f, "        }\n"), nOutpLines++);
    if (NetGAtoms(net))
	{
	    LSTNUM (NetGTransitions(net), transition, TRANSITION, t)
		{
		    if (TraGKind(transition) == TK_DUMMY)
			{
			    ((void) fprintf(f, "    if (!_someEnb)\n"), nOutpLines++);
			    ((void) fprintf(f, "        _F%s", UnsStr(t)), (void) fprintf(f, "();\n"), nOutpLines++);
			}
		}
	}
    ((void) fprintf(f, "    for (_class = 0; _class < %s", UnsStr(NetGNofClasses(net))), (void) fprintf(f, "; _class++)\n"), nOutpLines++);
    ((void) fprintf(f, "        LstDestroy(_bundle[_class], TRUE);\n"), nOutpLines++);
    ((void) fprintf(f, "}\n"), nOutpLines++);
}

FE0(void, GenAux)
{
    if ((fileCount != 0L) && (nOutpLines < (unsigned long) 800))
	return;
    (void) fclose(f);
    (void) sprintf(cName, "%s_%07lx.c", srcPrefix, fileCount);
    prpOutput = fopen(cName, "w");
    if (!prpOutput)
	{
	    (void) sprintf(errStr, "$Cannot open empty file %s for writing",
			   cName);
	    panic(errStr);
	}
    pxFile = f = prpOutput;
    fileCount++;
    nOutpLines = 0L;
    ((void) fprintf(f, "#include \"_.h\"\n"), nOutpLines++);
    ((void) fprintf(f, "\n"), nOutpLines++);
}

/***************************************************************************
*
* Function:  Generate
* Purpose:   This routine generates the code corresponding to high level
*            transitions. The _CompInst function is the main routine
*            in unfolding. The _CompPrec function is needed in the unfolding
*            if there are more than one precedence class. The _BuchiOK
*            function returs TRUE if and only if in the considered marking,
*            the atomic subformulas of the specification formula have
*            the truth values mentioned in the label of the considered
*            state of the Buchi automaton. (The Buchi automaton corresponds
*            to the negation of the specification formula.)
* Arguments: genrF - the produced code is put here
*            genrNet - the input net
* Returns:
* See also:
*
****************************************************************************
*/
FE2(void, Generate,
    FILE *, genrF,
    NET *, genrNet)
{
    unsigned long fileInd;

    pxFile = f = genrF;
    net = genrNet;
    ((void) fprintf(f, "\n"), nOutpLines++);
    DefsDecls();
    LSTNUM (NetGTransitions(net), transition, TRANSITION, t)
	{
	    ((void) fprintf(f, "DE0(void, _A%s", UnsStr(t)), (void) fprintf(f, ")/*%s*/\n", TraGName(transition)), nOutpLines++);
	    ((void) fprintf(f, "DE0(void, _D%s", UnsStr(t)), (void) fprintf(f, ")/*%s*/\n", TraGName(transition)), nOutpLines++);
	    ((void) fprintf(f, "DE0(void, _E%s", UnsStr(t)), (void) fprintf(f, ")/*%s*/\n", TraGName(transition)), nOutpLines++);
	    ((void) fprintf(f, "DE0(void, _I%s", UnsStr(t)), (void) fprintf(f, ")/*%s*/\n", TraGName(transition)), nOutpLines++);
	    ((void) fprintf(f, "DE0(void, _F%s", UnsStr(t)), (void) fprintf(f, ")/*%s*/\n", TraGName(transition)), nOutpLines++);
	}
    LSTNUM (NetGPrecedences(net), precedence, PRECEDENCE, p)
	{
	    if ((!PreGSucc(precedence)) || LstEmpty(PreGSucc(precedence))
		|| PreGEasy(precedence) || PreGSelfLoop(precedence))
		continue;
	    ((void) fprintf(f, "DE1(BOOL, _P%s", UnsStr(p + 1)), (void) fprintf(f, ",\n"), nOutpLines++);
	    ((void) fprintf(f, "    TRN *, trn)\n"), nOutpLines++);
	}
    fileCount = 0L;
    GenAux();
    ActualDecls();
    LSTNUM (NetGTransitions(net), transition, TRANSITION, t)
	{
	    dimension = 0;
	    iVarCount = 0;
	    letVarCount = 0;
	    LSTFOR (TraGVariables(transition), variable, VARIABLE)
		{
		    dimension++;
		    if (VarIsIn(variable))
			iVarCount++;
		    if (VarIsLet(variable))
			letVarCount++;
		}
	    if (letVarCount)
		{
		    LSTNUM (TraGVariables(transition), variable, VARIABLE, i)
			if (!VarIsLet(variable))
			    break;
		    letVarCount =
			(letVarCount << 1) | ((i == letVarCount) ? 1 : 0);
		}
	    ((void) fprintf(f, "\n"), nOutpLines++);
	    ARoutine();
	    GenAux();
	    ((void) fprintf(f, "\n"), nOutpLines++);
	    DRoutine();
	    GenAux();
	    ((void) fprintf(f, "\n"), nOutpLines++);
	    ERoutine();
	    GenAux();
	    ((void) fprintf(f, "\n"), nOutpLines++);
	    IRoutine();
	    GenAux();
	    cffOrdInputs = LstCreate(&arcTools);
	    LSTFOR (TraGInputs(transition), arc, ARC)
		{
		    cffOrdSymsum = NULL;
		    i = 0;
		    LSTFOR (ArcGSymsum(arc), symmultup, SYMMULTUP)
			{
			    if (!SytGCffSym(symmultup))
				{
				    if (!cffOrdSymsum)
					cffOrdSymsum = LstCreate(&sytTools);
				    (void) LstInsLast(cffOrdSymsum,
						      (char *) symmultup);
				}
			    else
				i = 1;
			}
		    if (cffOrdSymsum)
			{
			    arc2 = ArcCons(ArcGPlace(arc), cffOrdSymsum);
			    ArcSNumber(arc2, ArcGNumber(arc));
			    ArcSFirst(arc2);
			    if (i == 0)
				ArcSLast(arc2);
			    (void) LstInsLast(cffOrdInputs, (char *) arc2);
			}
		}
	    LSTFOR (TraGInputs(transition), arc, ARC)
		{
		    cffOrdSymsum = NULL;
		    i = 0;
		    LSTFOR (ArcGSymsum(arc), symmultup, SYMMULTUP)
			{
			    if (SytGCffSym(symmultup))
				{
				    if (!cffOrdSymsum)
					cffOrdSymsum = LstCreate(&sytTools);
				    (void) LstInsLast(cffOrdSymsum,
						      (char *) symmultup);
				}
			    else
				i = 1;
			}
		    if (cffOrdSymsum)
			{
			    arc2 = ArcCons(ArcGPlace(arc), cffOrdSymsum);
			    ArcSNumber(arc2, ArcGNumber(arc));
			    ArcSLast(arc2);
			    if (i == 0)
				ArcSFirst(arc2);
			    (void) LstInsLast(cffOrdInputs, (char *) arc2);
			}
		}
	    found = FALSE;
	    i = 0;
	    LSTFOR (cffOrdInputs, arc, ARC)
		{
		    a = ArcGNumber(arc);
		    LSTFOR (ArcGSymsum(arc), symmultup, SYMMULTUP)
			{
			    s = SytGNumber(symmultup);
			    if (!found)
				((void) fprintf(f, "\n"), nOutpLines++);
                            ((void) fprintf(f, "#define _t%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, " (_t[%s", UnsStr(i)), (void) fprintf(f, "])\n"), nOutpLines++);
			    i++;
			    found = TRUE;
			}
		}
	    i = 0;
	    LSTFOR (cffOrdInputs, arc, ARC)
		{
		    a = ArcGNumber(arc);
		    LSTFOR (ArcGSymsum(arc), symmultup, SYMMULTUP)
			{
			    if ((!ArcLast(arc)) ||
				(symmultup != (SYMMULTUP *)
				 LstLast(ArcGSymsum(arc))))
				{
				    s = SytGNumber(symmultup);
                                    ((void) fprintf(f, "#define _u%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, " (_u[%s", UnsStr(i)), (void) fprintf(f, "])\n"), nOutpLines++);
				    i++;
				}
			}
		}
	    ((void) fprintf(f, "\n"), nOutpLines++);
	    FRoutine();
	    GenAux();
	    found = FALSE;
	    LSTFOR (cffOrdInputs, arc, ARC)
		{
		    a = ArcGNumber(arc);
		    LSTFOR (ArcGSymsum(arc), symmultup, SYMMULTUP)
			{
			    s = SytGNumber(symmultup);
			    if (!found)
				((void) fprintf(f, "\n"), nOutpLines++);
                            ((void) fprintf(f, "#undef _t%s", UnsStr(a)), (void) fprintf(f, "_%s\n", UnsStr(s)), nOutpLines++);
			    found = TRUE;
			}
		}
	    LSTFOR (cffOrdInputs, arc, ARC)
		{
		    a = ArcGNumber(arc);
		    LSTFOR (ArcGSymsum(arc), symmultup, SYMMULTUP)
			{
			    if ((!ArcLast(arc)) ||
				(symmultup != (SYMMULTUP *)
				 LstLast(ArcGSymsum(arc))))
				{
				    s = SytGNumber(symmultup);
                                    ((void) fprintf(f, "#undef _u%s", UnsStr(a)), (void) fprintf(f, "_%s\n", UnsStr(s)), nOutpLines++);
				}
			}
		}
	    LSTFOR (cffOrdInputs, arc, ARC)
		{
		    LstDestroy(ArcGSymsum(arc), FALSE);
		    Deallocate((char *) arc);
		}
	    LstDestroy(cffOrdInputs, FALSE);
	}
    ((void) fprintf(f, "\n"), nOutpLines++);
    GenFollowers();
    GenAux();
    LSTNUM (NetGPrecedences(net), precedence, PRECEDENCE, p)
	{
	    if ((!PreGSucc(precedence)) || LstEmpty(PreGSucc(precedence))
		|| PreGEasy(precedence) || PreGSelfLoop(precedence))
		continue;
	    ((void) fprintf(f, "\n"), nOutpLines++);
	    ((void) fprintf(f, "FE1(BOOL, _P%s", UnsStr(p + 1)), (void) fprintf(f, ",\n"), nOutpLines++);
	    ((void) fprintf(f, "    TRN *, trn)\n"), nOutpLines++);
	    ((void) fprintf(f, "{\n"), nOutpLines++);
	    ((void) fprintf(f, "    SetLowTrn(trn);\n"), nOutpLines++);
	    LSTFOR(PreGSucc(precedence), priority, PRECEDENCE)
		{
		    ((void) fprintf(f, "    SetHiClass(%s", UnsStr(PreGValue(priority))), (void) fprintf(f, ");\n"), nOutpLines++);
		    ((void) fprintf(f, "    (void) BalTrApply(TrnBalTree(%s", UnsStr(PreGValue(priority))), (void) fprintf(f, "),"));
		    ((void) fprintf(f, " TrnPrec);\n"), nOutpLines++);
		    ((void) fprintf(f, "    if (Lower(%s", UnsStr(PreGValue(priority))), (void) fprintf(f, "))\n"), nOutpLines++);
		    ((void) fprintf(f, "        SetHigher(%s", UnsStr(p + 1)), (void) fprintf(f, ");\n"), nOutpLines++);
		}
	    ((void) fprintf(f, "    return(FALSE);\n"), nOutpLines++);
	    ((void) fprintf(f, "}\n"), nOutpLines++);
	    GenAux();
	}
    maxDim = 0;
    LSTFOR (NetGTransitions(net), transition, TRANSITION)
	LSTFOR (TraGArcs(transition), arc, ARC)
	    LSTFOR (ArcGSymsum(arc), symmultup, SYMMULTUP)
		if (SytGArity(symmultup) > maxDim)
		    maxDim = SytGArity(symmultup);
    ((void) fprintf(f, "\n"), nOutpLines++);
    ((void) fprintf(f, "FE0(void, _CompInst)\n"), nOutpLines++);
    ((void) fprintf(f, "{\n"), nOutpLines++);
    if (newBlock != 0L)
	{
	    ((void) fprintf(f, "    unsigned _iBlock;\n"), nOutpLines++);
	    ((void) fprintf(f, "\n"), nOutpLines++);
	    ((void) fprintf(f, "    if (Blocked() && PtStub() && PtDelAlg())\n"), nOutpLines++);
	    ((void) fprintf(f, "        {\n"), nOutpLines++);
            ((void) fprintf(f, "            ABlocks((unsigned) %s", UnsStr((unsigned) (newBlock + 1L))), (void) fprintf(f, ");\n"), nOutpLines++);
            ((void) fprintf(f, "            for (_iBlock = 0; _iBlock <"));
            ((void) fprintf(f, " (unsigned) %s", UnsStr((unsigned) newBlock)), (void) fprintf(f, ";"));
	    ((void) fprintf(f, " _iBlock++)\n"), nOutpLines++);
	    ((void) fprintf(f, "                SBlock(_iBlock);\n"), nOutpLines++);
	    ((void) fprintf(f, "            CBlock(_iBlock);\n"), nOutpLines++);
	    ((void) fprintf(f, "        }\n"), nOutpLines++);
	    ((void) fprintf(f, "    else\n"), nOutpLines++);
	    ((void) fprintf(f, "        ClBlocked();\n"), nOutpLines++);
	}
    else
	((void) fprintf(f, "    ClBlocked();\n"), nOutpLines++);
    ((void) fprintf(f, "    _w = TupCons((unsigned) %s", UnsStr(maxDim)), (void) fprintf(f, ");\n"), nOutpLines++);
    LSTNUM (NetGTransitions(net), transition, TRANSITION, t)
	((void) fprintf(f, "    _I%s", UnsStr(t)), (void) fprintf(f, "();\n"), nOutpLines++);
    ((void) fprintf(f, "    TupSArity(_w, (unsigned) %s", UnsStr(maxDim)), (void) fprintf(f, ");\n"), nOutpLines++);
    ((void) fprintf(f, "    TupDiscard(_w);\n"), nOutpLines++);
    ((void) fprintf(f, "}\n"), nOutpLines++);
    GenAux();
    ((void) fprintf(f, "\n"), nOutpLines++);
    ((void) fprintf(f, "FE0(void, _CompPrec)\n"), nOutpLines++);
    ((void) fprintf(f, "{\n"), nOutpLines++);
    LSTNUM (NetGPrecedences(net), precedence, PRECEDENCE, p)
	{
	    if ((!PreGSucc(precedence)) || LstEmpty(PreGSucc(precedence))
		|| PreGEasy(precedence))
		continue;
	    if (PreGSelfLoop(precedence))
		((void) fprintf(f, "    SetSelfHigher(%s", UnsStr(p + 1)), (void) fprintf(f, ");\n"), nOutpLines++);
	    else
		((void) fprintf(f, "    (void) BalTrApply(TrnBalTree(%s", UnsStr(p + 1)), (void) fprintf(f, "), _P%s", UnsStr(p + 1)), (void) fprintf(f, ");\n"), nOutpLines++);
	}
    ((void) fprintf(f, "}\n"), nOutpLines++);
    ((void) fprintf(f, "\n"), nOutpLines++);
    ((void) fprintf(f, "FE0(BOOL, _BuchiOK)\n"), nOutpLines++);
    ((void) fprintf(f, "{\n"), nOutpLines++);
    if (NetGAtoms(net))
	{
	    ((void) fprintf(f, "    _i_ = (unsigned) (MultupGField"));
	    ((void) fprintf(f, "(MarFirst(StaGMarking(_evalState, /*%s*/MonPlInd())),\n", PlaGName(NetGMonPlace(net))), nOutpLines++);
	    ((void) fprintf(f, "                                (unsigned) 0) - 1);\n"), nOutpLines++);
	    pxGotoNumber = 0L;
	    for (atom = (FORMULA *) LstBegin(NetGAtoms(net)), mask = 2L; atom;
		 atom = (FORMULA *) LstStep(NetGAtoms(net)), mask <<= 2)
		{
                    ((void) fprintf(f, "    if ((_z_[_i_] & (unsigned long) %s", UlongStr(mask)), (void) fprintf(f, ") != 0L) {\n"), nOutpLines++);
		    ((void) fprintf(f, "        _xprTuple = NULL;\n"), nOutpLines++);
		    XprOut(atom->uf.expression, 0);
                    ((void) fprintf(f, "    if (((_z_[_i_] & (unsigned long) %s", UlongStr(mask >> 1)), (void) fprintf(f, ") == 0L)"));
		    ((void) fprintf(f, " != (_xprDom == (DOMAIN) 0L))\n"), nOutpLines++);
		    ((void) fprintf(f, "        return(FALSE);\n"), nOutpLines++);
		    ((void) fprintf(f, "    }\n"), nOutpLines++);
		}
	}
    ((void) fprintf(f, "    return(TRUE);\n"), nOutpLines++);
    ((void) fprintf(f, "}\n"), nOutpLines++);
    (void) fclose(f);
#ifdef UNIX
    (void) sprintf(cName, "%s_.sh", srcPrefix);
    f = fopen(cName, "a");
    if (!f)
	{
            (void) sprintf(errStr, "$Cannot open file %s for appending",
			   cName);
	    panic(errStr);
	}
    for (fileInd = 0L; fileInd < fileCount; fileInd++)
	(void)
	    fprintf(f,
		    "${COMPILE} _%07lx.c || exit\n${%s} ${LIBRARY} _%07lx${O} || exit\n${DELETE} _%07lx${O} || exit\n",
		    fileInd, (0L == fileInd) ? "LIBFIRST" : "LIBNEXT",
		    fileInd, fileInd);
#else
    (void) sprintf(cName, "%s__.bat", srcPrefix);
    f = fopen(cName, "w");
    if (!f)
	{
	    (void) sprintf(errStr, "$Cannot open empty file %s for writing",
			   cName);
	    panic(errStr);
	}
    (void) fprintf(f, "@echo on\n@if errorlevel 1 clerr.exe\n");
    for (fileInd = 0L; fileInd < fileCount; fileInd++)
	{
            (void) fprintf(f, "call .%c_.bat _%07lx.c\n", DIRCHAR, fileInd);
	    if (fileInd + 1L != fileCount)
		(void) fprintf(f, "@if errorlevel 1 goto end\n");
	}
    if (1L < fileCount)
	(void) fprintf(f, ":end\n");
    (void) fclose(f);
    (void) sprintf(cName, "%s_.rsp", srcPrefix);
    f = fopen(cName, "w");
    if (!f)
	{
	    (void) sprintf(errStr, "$Cannot open empty file %s for writing",
			   cName);
	    panic(errStr);
	}
    for (fileInd = 0L; fileInd < fileCount; fileInd++)
	if (fileInd + 1L == fileCount)
	    (void) fprintf(f, "+_%07lx.obj;\n", fileInd);
	else
	    (void) fprintf(f, "+_%07lx.obj &\n", fileInd);
    (void) fclose(f);
    (void) sprintf(cName, "%s_.r32", srcPrefix);
    f = fopen(cName, "w");
    if (!f)
	{
	    (void) sprintf(errStr, "$Cannot open empty file %s for writing",
			   cName);
	    panic(errStr);
	}
    for (fileInd = 0L; fileInd < fileCount; fileInd++)
	(void) fprintf(f, "_%07lx.obj\n", fileInd);
#endif
    (void) fclose(f);
}
