/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  gen3.tpl
*
* Part of: PROD's program prpp
*
* Authors: Marko Rauhamaa and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     See gen.tpl.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include "gen.h"

/***************************************************************************
*
* Function:  IRoutine
* Purpose:   Together with DRoutine and ERoutine, this routine produces
*            the unfolding of a high level transition.
* Arguments:
* Returns:
* See also:  DRoutine, ERoutine, FRoutine
*
****************************************************************************
*/
FE0(void, IRoutine)
{
    ((void) fprintf(f, "FE0(void, _I%s", UnsStr(t)), (void) fprintf(f, ")/*%s*/\n", TraGName(transition)), nOutpLines++);
    ((void) fprintf(f, "{\n"), nOutpLines++);
    WhatInitial(TraGArcs(transition));
    if (letVarCount)
        {
            ((void) fprintf(f, "    TravRange((unsigned) %s", UnsStr(t)), (void) fprintf(f, ", (unsigned) %s", UnsStr(letVarCount >> 1)), (void) fprintf(f, ","));
            if (letVarCount & 1)
                ((void) fprintf(f, " _v"));
            else
                ((void) fprintf(f, " TupGArray(_w)"));
            ((void) fprintf(f, ", _E%s", UnsStr(t)), (void) fprintf(f, ");\n"), nOutpLines++);
        }
    else
        {
            cffOrdVariables = LstCreate(&varTools);
            LSTFOR (TraGVariables(transition), variable, VARIABLE)
            if (VarIsIn(variable) && (!VarUgly(variable)) &&
                VarGName(variable))
                (void) LstInsLast(cffOrdVariables, (char *) variable);
            LSTFOR (TraGVariables(transition), variable, VARIABLE)
                if (VarIsIn(variable) && VarUgly(variable) &&
                    VarGName(variable))
                (void) LstInsLast(cffOrdVariables, (char *) variable);
            LSTROLL (cffOrdVariables, variable, VARIABLE, s)
                {
                    ((void) fprintf(f, "    %s =", VarGLabel(variable)));
                    ((void) fprintf(f, " (DOMAIN) %s", DomStr(VarGLow(variable))), (void) fprintf(f, ";\n"), nOutpLines++);
                    VarSet(variable);
                    VarCEscape(variable);
                    ((void) fprintf(f, "_p%s", UnsStr(s)), (void) fprintf(f, ": ;\n"), nOutpLines++);
                    WhatNew(TraGArcs(transition), FALSE);
                }
            ((void) fprintf(f, "    _E%s", UnsStr(t)), (void) fprintf(f, "();\n"), nOutpLines++);
            LSTUNROLL (cffOrdVariables, variable, VARIABLE, s)
                {
                    if (VarEscape(variable))
                        ((void) fprintf(f, "_q%s", UnsStr(s)), (void) fprintf(f, ": ;\n"), nOutpLines++);
                    ((void) fprintf(f, "    if (%s !=", VarGLabel(variable)));
                    ((void) fprintf(f, " (DOMAIN) %s", DomStr(VarGHigh(variable))), (void) fprintf(f, ")\n"), nOutpLines++);
                    ((void) fprintf(f, "        {\n"), nOutpLines++);
                    ((void) fprintf(f, "            %s++;\n", VarGLabel(variable)), nOutpLines++);
                    ((void) fprintf(f, "            goto _p%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
                    ((void) fprintf(f, "        }\n"), nOutpLines++);
                }
            LstDestroy(cffOrdVariables, FALSE);
        }
    ((void) fprintf(f, "}\n"), nOutpLines++);
}

/***************************************************************************
*
* Function:  FRoutine
* Purpose:   Together with ARoutine, this routine produces the computation
*            (and possible firing) of enabled instances of a high level
*            transition in the case that the net is not unfolded.
* Arguments:
* Returns:
* See also:  ARoutine, IRoutine
*
****************************************************************************
*/
FE0(void, FRoutine)
{
    ((void) fprintf(f, "FE0(void, _F%s", UnsStr(t)), (void) fprintf(f, ")/*%s*/\n", TraGName(transition)), nOutpLines++);
    ((void) fprintf(f, "{\n"), nOutpLines++);
    WhatInitial(TraGOutputs(transition));
    LSTFOR (TraGInputs(transition), arc, ARC)
	{
	    LSTFOR (ArcGSymsum(arc), symmultup, SYMMULTUP)
		if (!SytGCffSym(symmultup))
		    break;
	    if (symmultup)
		{
		    p = LstWhich(NetGPlaces(net),
				 (char *) ArcGPlace(arc)) - 1;
                    ((void) fprintf(f, "    if (MarEmpty(StaGMarking(_state,"));
                    ((void) fprintf(f, " (unsigned) /*%s*/%s", PlaGName(ArcGPlace(arc)), UnsStr(p)), (void) fprintf(f, ")))\n"), nOutpLines++);
		    ((void) fprintf(f, "        return;\n"), nOutpLines++);
		}
	}
    LSTFOR (TraGVariables(transition), variable, VARIABLE)
	{
	    if ((!VarIsIn(variable)) || (!VarHard(variable)))
		continue;
	    VarSLow(variable, (DOMAIN) MINDOMAIN);
	    VarSHigh(variable, (DOMAIN) MAXDOMAIN);
	    VarSAuxLow(variable, (DOMAIN) MINDOMAIN);
	    VarSAuxHigh(variable, (DOMAIN) MAXDOMAIN);
	}
    LSTFOR (TraGInputs(transition), arc, ARC)
	LSTFOR (ArcGSymsum(arc), symmultup, SYMMULTUP)
	    {
		xlow = NULL;
		xhigh = NULL;
		if (PlaGLow(ArcGPlace(arc)))
		    LSTFOR (PlaGLow(ArcGPlace(arc)), xlow, TUPLE)
			{
			    if (TupGArity(xlow) == SytGArity(symmultup))
				break;
			    else if (TupGArity(xlow) > SytGArity(symmultup))
				{
				    xlow = NULL;
				    break;
				}
			}
		if (PlaGHigh(ArcGPlace(arc)))
		    LSTFOR (PlaGHigh(ArcGPlace(arc)), xhigh, TUPLE)
			{
			    if (TupGArity(xhigh) == SytGArity(symmultup))
				break;
			    else if (TupGArity(xhigh) > SytGArity(symmultup))
				{
				    xhigh = NULL;
				    break;
				}
			}
		if (!SytGCffSym(symmultup))
		    for (i = 0; i < SytGArity(symmultup); i++)
			if (SymIsVariable(symbol =
					  SytGSymbol(symmultup, i)))
			    {
				variable = SymGVariable(symbol);
				if (xlow &&
				    ((x = TupGField(xlow, i))
				     > VarGAuxLow(variable)))
				    VarSAuxLow(variable, x);
				if (xhigh &&
				    ((x = TupGField(xhigh, i))
				     < VarGAuxHigh(variable)))
				    VarSAuxHigh(variable, x);
			    }
	    }
    LSTFOR (TraGOutputs(transition), arc, ARC)
	LSTFOR (ArcGSymsum(arc), symmultup, SYMMULTUP)
	    {
		xlow = NULL;
		xhigh = NULL;
		if (PlaGLow(ArcGPlace(arc)))
		    LSTFOR (PlaGLow(ArcGPlace(arc)), xlow, TUPLE)
			{
			    if (TupGArity(xlow) == SytGArity(symmultup))
				break;
			    else if (TupGArity(xlow) > SytGArity(symmultup))
				{
				    xlow = NULL;
				    break;
				}
			}
		if (PlaGHigh(ArcGPlace(arc)))
		    LSTFOR (PlaGHigh(ArcGPlace(arc)), xhigh, TUPLE)
			{
			    if (TupGArity(xhigh) == SytGArity(symmultup))
				break;
			    else if (TupGArity(xhigh) > SytGArity(symmultup))
				{
				    xhigh = NULL;
				    break;
				}
			}
		for (i = 0; i < SytGArity(symmultup); i++)
		    if (SymIsVariable(symbol =
				      SytGSymbol(symmultup, i)) &&
			VarIsIn(variable = SymGVariable(symbol)))
			{
			    if (SytGCffSym(symmultup))
				{
				    VarSAuxLow(variable, (DOMAIN) MINDOMAIN);
				    VarSAuxHigh(variable, (DOMAIN) MAXDOMAIN);
				}
			    else
				{
				    if (xlow &&
					((x = TupGField(xlow, i))
					 > VarGLow(variable)))
					VarSLow(variable, x);
				    if (xhigh &&
					((x = TupGField(xhigh, i))
					 < VarGHigh(variable)))
					VarSHigh(variable, x);
				}
			}
	    }
    LSTROLL (cffOrdInputs, arc, ARC, ca)
	{
	    a = ArcGNumber(arc);
	    p = LstWhich(NetGPlaces(net), (char *) ArcGPlace(arc)) - 1;
	    ((void) fprintf(f, "    _marking = StaGMarking(_state, (unsigned) /*%s*/%s", PlaGName(ArcGPlace(arc)), UnsStr(p)), (void) fprintf(f, ");\n"), nOutpLines++);
	    if (ArcFirst(arc))
		{
		    s = SytGNumber((SYMMULTUP *) LstFirst(ArcGSymsum(arc)));
		    ls = SytGLabel((SYMMULTUP *) LstFirst(ArcGSymsum(arc)));
		    ((void) fprintf(f, "    MARFOR (_marking, %s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ")\n"), nOutpLines++);
		    ((void) fprintf(f, "        MultupSFreeCount(%s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ", MultupGCount(%s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, "));\n"), nOutpLines++);
		}
	    LstStart(ArcGSymsum(arc));
	    LstStore(ArcGSymsum(arc));
	    symmultup = (SYMMULTUP *) LstStep(ArcGSymsum(arc));
	    cs = 0;
	    while (symmultup)
		{
		    s = SytGNumber(symmultup);
		    ls = SytGLabel(symmultup);
		    if (SytGCffSym(symmultup))
			{
                            ((void) fprintf(f, "    if (0L == (unsigned long) ("));
                            PrintSymbol(f, SytGCffSym(symmultup));
                            ((void) fprintf(f, "))\n"), nOutpLines++);
			    ((void) fprintf(f, "        goto _e%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
                        }
                    ((void) fprintf(f, "    %s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, " = MarBegin(_marking);\n"), nOutpLines++);
                    if (SytGArity(symmultup))
                        {
                            ((void) fprintf(f, "    while (%s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, " &&"));
                            ((void) fprintf(f, " (MultupGArity(%s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ") < (unsigned)"));
                            ((void) fprintf(f, " %s", UnsStr(SytGArity(symmultup))), (void) fprintf(f, "))\n"), nOutpLines++);
                            ((void) fprintf(f, "        %s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, " = MarStep(_marking);\n"), nOutpLines++);
                        }
                    ((void) fprintf(f, "    if ((!%s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ") ||"));
                    ((void) fprintf(f, " (MultupGArity(%s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ") > (unsigned)"));
                    ((void) fprintf(f, " %s", UnsStr(SytGArity(symmultup))), (void) fprintf(f, "))\n"), nOutpLines++);
                    ((void) fprintf(f, "        goto _d%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
                    ((void) fprintf(f, "_b%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ": ;\n"), nOutpLines++);
                    ((void) fprintf(f, "    if (MultupGFreeCount(%s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ")"));
                    ((void) fprintf(f, " < (unsigned long)"));
                    if (SytGCffSym(symmultup))
                        {
                            ((void) fprintf(f, " ("));
                            PrintSymbol(f, SytGCffSym(symmultup));
                            ((void) fprintf(f, ")"));
                        }
                    else
                        ((void) fprintf(f, " %s", UlongStr(SytGMulti(symmultup))));
                    ((void) fprintf(f, ")\n"), nOutpLines++);
		    ((void) fprintf(f, "        goto _c%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
	            for (i = 0; i < SytGArity(symmultup); i++)
			{
			    symbol = SytGSymbol(symmultup, i);
			    if (!SymSatisfied(symbol))
				continue;
			    SymSet(symbol);
			    ((void) fprintf(f, "    if (MultupGField(%s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ", (unsigned) %s", UnsStr(i)), (void) fprintf(f, ") != (DOMAIN) ("));
			    PrintSymbol(f, symbol);
			    ((void) fprintf(f, "))\n"), nOutpLines++);
			    ((void) fprintf(f, "        goto _c%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
			}
	            for (i = 0; i < SytGArity(symmultup); i++)
			{
			    symbol = SytGSymbol(symmultup, i);
			    if (!SymIsVariable(symbol))
				continue;
			    variable = SymGVariable(symbol);
			    if (VarUnified(variable))
				continue;
			    VarSet(variable);
			    SymSet(symbol);
			    ((void) fprintf(f, "    %s", VarGLabel(variable)));
			    ((void) fprintf(f, " = MultupGField(%s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ", (unsigned) %s", UnsStr(i)), (void) fprintf(f, ");\n"), nOutpLines++);
			    if (VarHard(variable))
				{
				    if (VarGLow(variable) >
					VarGAuxLow(variable))
					{
					    ((void) fprintf(f, "    if (%s <", VarGLabel(variable)));
                                            ((void) fprintf(f, " (DOMAIN) %s", DomStr(VarGLow(variable))), (void) fprintf(f, ")\n"), nOutpLines++);
				            ((void) fprintf(f, "        goto _c%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
					}
				    if (VarGHigh(variable) <
					VarGAuxHigh(variable))
					{
					    ((void) fprintf(f, "    if (%s >", VarGLabel(variable)));
                                            ((void) fprintf(f, " (DOMAIN) %s", DomStr(VarGHigh(variable))), (void) fprintf(f, ")\n"), nOutpLines++);
				            ((void) fprintf(f, "        goto _c%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
					}
				}
			    acursor = LstGet(cffOrdInputs);
			    arc2 = (ARC *) LstBegin(cffOrdInputs);
			    ca2 = 0;
			    while (ca2 <= ca)
				{
				    a2 = ArcGNumber(arc2);
				    scursor = LstGet(ArcGSymsum(arc2));
				    syt2 = (SYMMULTUP *)
					LstBegin(ArcGSymsum(arc2));
				    cs2 = 0;
				    while (syt2 &&
					   ((ca2 != ca) || (cs2 <= cs)))
					{
					    s2 = SytGNumber(syt2);
					    ls2 = SytGLabel(syt2);
					    for (j = 0; j < SytGArity(syt2);
						 j++)
						{
						    symbol =
							SytGSymbol(syt2, j);
						    if (SymMarked(symbol) ||
							!SymSatisfied(symbol))
							continue;
						    SymSet(symbol);
						    ((void) fprintf(f, "    if (MultupGField"));
						    ((void) fprintf(f, "(%s_t%s", ls2, UnsStr(a2)), (void) fprintf(f, "_%s", UnsStr(s2)), (void) fprintf(f, ", (unsigned) %s", UnsStr(j)), (void) fprintf(f, ")"));
						    ((void) fprintf(f, " != (DOMAIN) ("));
						    PrintSymbol(f, symbol);
						    ((void) fprintf(f, "))\n"), nOutpLines++);
						    ((void) fprintf(f, "        goto _c%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
						}
					    syt2 = (SYMMULTUP *)
						LstStep(ArcGSymsum(arc2));
					    cs2++;
					}
				    LstSet(ArcGSymsum(arc2), scursor);
				    arc2 = (ARC *)
					LstStep(cffOrdInputs);
				    ca2++;
				}
			    LstSet(cffOrdInputs, acursor);
			    WhatNew(TraGOutputs(transition), TRUE);
			}
                    if (SytGCffSym(symmultup))
			{
			    ((void) fprintf(f, "    MultupDFreeCount(%s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ", (unsigned long) ("));
                            PrintSymbol(f, SytGCffSym(symmultup));
                            ((void) fprintf(f, "));\n"), nOutpLines++);
			}
	            else
			{
			    ((void) fprintf(f, "    MultupDFreeCount(%s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ", (unsigned long)"));
                            ((void) fprintf(f, " %s", UlongStr(SytGMulti(symmultup))), (void) fprintf(f, ");\n"), nOutpLines++);
			}
		    if ((!ArcLast(arc)) ||
			(symmultup != (SYMMULTUP *) LstLast(ArcGSymsum(arc))))
			((void) fprintf(f, "    _u%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, " = MarGet(_marking);\n"), nOutpLines++);
                    if (SytGCffSym(symmultup))
                        ((void) fprintf(f, "_e%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ": ;\n"), nOutpLines++);
		    LstStore(ArcGSymsum(arc));
	            symmultup = (SYMMULTUP *) LstStep(ArcGSymsum(arc));
	            cs++;
	        }
	}
    LSTFOR (TraGVariables(transition), variable, VARIABLE)
	if (VarUgly(variable) || !VarIsIn(variable))
	    VarReset(variable);
    LSTFOR (TraGOutputs(transition), arc2, ARC)
	LSTFOR (ArcGSymsum(arc2), syt2, SYMMULTUP)
	    for (j = 0; j < SytGArity(syt2); j++)
		if (!SymSatisfied(sym2 = SytGSymbol(syt2, j)))
		    SymReset(sym2);
    LSTFOR (TraGGates(transition), gate, SYMBOL)
	if (!SymSatisfied(gate))
	    SymReset(gate);
    LSTROLL (TraGVariables(transition), variable, VARIABLE, s)
	{
	    if ((!VarUgly(variable)) || (!VarIsIn(variable)))
		continue;
	    LSTFOR (VarGCoeffs(variable), cffSym, SYMBOL)
		{
		    ((void) fprintf(f, "    if (0L != (unsigned long) ("));
                    PrintSymbol(f, cffSym);
                    ((void) fprintf(f, "))\n"), nOutpLines++);
                    ((void) fprintf(f, "        goto _r%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
		}
            ((void) fprintf(f, "    %s = (DOMAIN) %s", VarGLabel(variable), DomStr(VarGLow(variable))), (void) fprintf(f, ";\n"), nOutpLines++);
	    VarSet(variable);
	    VarCEscape(variable);
            ((void) fprintf(f, "_p%s", UnsStr(s)), (void) fprintf(f, ": ;\n"), nOutpLines++);
	    WhatNew(TraGOutputs(transition), FALSE);
	    if (!LstEmpty(VarGCoeffs(variable)))
		((void) fprintf(f, "_r%s", UnsStr(s)), (void) fprintf(f, ": ;\n"), nOutpLines++);
	}
    if (letVarCount)
        {
            if (!(letVarCount & 1))
                {
                    i = 0;
                    LSTFOR (TraGVariables(transition), variable, VARIABLE)
                        if (VarIsLet(variable))
                            {
                                ((void) fprintf(f, "    TupSField(_w, (unsigned) %s", UnsStr(i)), (void) fprintf(f, ","));
                                ((void) fprintf(f, " %s);\n", VarGLabel(variable)), nOutpLines++);
                                i++;
                            }
                }
            ((void) fprintf(f, "    if (!InRange((unsigned) %s", UnsStr(t)), (void) fprintf(f, ", (unsigned) %s", UnsStr(letVarCount >> 1)), (void) fprintf(f, ","));
            if (letVarCount & 1)
                ((void) fprintf(f, " _v,\n"), nOutpLines++);
            else
                ((void) fprintf(f, " TupGArray(_w),\n"), nOutpLines++);
            ((void) fprintf(f, "                 EngABuffer(), EngABufSize()))\n"), nOutpLines++);
            ((void) fprintf(f, "        goto _CONT;\n"), nOutpLines++);
        }
    if (NetGNofClasses(net) != 1)
	((void) fprintf(f, "    _class = 0;\n"), nOutpLines++);
    else if (TraGComputation(transition))
	((void) fprintf(f, "    _list = LstCreate(&peeTools);\n"), nOutpLines++);
    if (!NetGMonPlace(net))
	((void) fprintf(f, "    _visible = PtDefVis() ? 1 : 0;\n"), nOutpLines++);
    else if (NetGAtoms(net))
	{
	    ((void) fprintf(f, "    for (_y_ = _x_["));
	    if (TraGKind(transition) == TK_INITIAL)
                ((void) fprintf(f, "%s", UnsStr(((unsigned) MultupGField(MarFirst(PlaGMarking(NetGMonPlace(net))), (unsigned) 0)) - 1)));
	    else
		((void) fprintf(f, "((unsigned) %s) - 1", VarGLabel((VARIABLE *) LstNth(TraGVariables(transition), LstSize(TraGVariables(transition)) - 1))));
	    ((void) fprintf(f, "]; (%s = *_y_); _y_++)\n", VarGLabel((VARIABLE *) LstLast(TraGVariables(transition)))), nOutpLines++);
	}
    if (!TraGComputation(transition))
	((void) fprintf(f, "    _A%s", UnsStr(t)), (void) fprintf(f, "();\n"), nOutpLines++);
    else
	{
	    if (NetGNofClasses(net) != 1)
                ((void) fprintf(f, "#define Precedence(class) (_class = (class))\n"), nOutpLines++);
	    if (!NetGMonPlace(net))
		{
                    ((void) fprintf(f, "#define Visible() (_visible = 1)\n"), nOutpLines++);
                    ((void) fprintf(f, "#define Invisible() (_visible = 0)\n"), nOutpLines++);
		}
            ((void) fprintf(f, "#define Accept() _A%s", UnsStr(t)), (void) fprintf(f, "()\n"), nOutpLines++);
	    ((void) fprintf(f, "    {"));
	    LSTFOR (TraGComputation(transition), str, char)
		((void) fprintf(f, "%s", str));
	    ((void) fprintf(f, "}\n"), nOutpLines++);
	    if (NetGNofClasses(net) != 1)
                ((void) fprintf(f, "#undef Precedence\n"), nOutpLines++);
	    if (!NetGMonPlace(net))
		{
                    ((void) fprintf(f, "#undef Visible\n"), nOutpLines++);
                    ((void) fprintf(f, "#undef Invisible\n"), nOutpLines++);
		}
            ((void) fprintf(f, "#undef Accept\n"), nOutpLines++);
	    if (NetGNofClasses(net) != 1)
		{
		    ((void) fprintf(f, "    for"));
                    ((void) fprintf(f, " (_class = 0; _class < %s", UnsStr(NetGNofClasses(net))), (void) fprintf(f, "; _class++)\n"), nOutpLines++);
		    ((void) fprintf(f, "        {\n"), nOutpLines++);
		    ((void) fprintf(f, "            if (LstEmpty(_tmpBundle[_class]))\n"), nOutpLines++);
		    ((void) fprintf(f, "                continue;\n"), nOutpLines++);
		    ((void) fprintf(f, "            if (_kind[_class] == _CL_INDEPENDENT)\n"), nOutpLines++);
	            ((void) fprintf(f, "                {\n"), nOutpLines++);
	            ((void) fprintf(f, "                    LstDestroy"));
	            ((void) fprintf(f, "(_tmpBundle[_class], TRUE);\n"), nOutpLines++);
	            ((void) fprintf(f, "                    _tmpBundle[_class] ="));
	            ((void) fprintf(f, " LstCreate(&peeTools);\n"), nOutpLines++);
	            ((void) fprintf(f, "                }\n"), nOutpLines++);
	            ((void) fprintf(f, "            else\n"), nOutpLines++);
	            ((void) fprintf(f, "                (void) LstCatenate"));
	            ((void) fprintf(f, "(_bundle[_class], _tmpBundle[_class]);\n"), nOutpLines++);
	            ((void) fprintf(f, "        }\n"), nOutpLines++);
		}
	    else
		((void) fprintf(f, "    LstDestroy(_list, TRUE);\n"), nOutpLines++);
	}
    if (letVarCount)
        ((void) fprintf(f, "_CONT: ;\n"), nOutpLines++);
    LSTUNROLL (TraGVariables(transition), variable, VARIABLE, s)
	{
	    if ((!VarUgly(variable)) || (!VarIsIn(variable)))
		continue;
	    LSTFOR (VarGCoeffs(variable), cffSym, SYMBOL)
		{
		    ((void) fprintf(f, "    if (0L != (unsigned long) ("));
                    PrintSymbol(f, cffSym);
                    ((void) fprintf(f, "))\n"), nOutpLines++);
                    ((void) fprintf(f, "        goto _s%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
		}
	    if (VarEscape(variable))
		((void) fprintf(f, "_q%s", UnsStr(s)), (void) fprintf(f, ": ;\n"), nOutpLines++);
            ((void) fprintf(f, "    if (%s != (DOMAIN) %s", VarGLabel(variable), DomStr(VarGHigh(variable))), (void) fprintf(f, ")\n"), nOutpLines++);
            ((void) fprintf(f, "        {\n"), nOutpLines++);
            ((void) fprintf(f, "            %s++;\n", VarGLabel(variable)), nOutpLines++);
            ((void) fprintf(f, "            goto _p%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
            ((void) fprintf(f, "        }\n"), nOutpLines++);
	    if (!LstEmpty(VarGCoeffs(variable)))
		((void) fprintf(f, "_s%s", UnsStr(s)), (void) fprintf(f, ": ;\n"), nOutpLines++);
	}
    LSTUNNEST (cffOrdInputs, arc, ARC)
	{
	    a = ArcGNumber(arc);
	    p = LstWhich(NetGPlaces(net), (char *) ArcGPlace(arc)) - 1;
	    ((void) fprintf(f, "    _marking = StaGMarking(_state, (unsigned) /*%s*/%s", PlaGName(ArcGPlace(arc)), UnsStr(p)), (void) fprintf(f, ");\n"), nOutpLines++);
	    LSTUNNEST (ArcGSymsum(arc), symmultup, SYMMULTUP)
		{
		    s = SytGNumber(symmultup);
		    ls = SytGLabel(symmultup);
                    if (SytGCffSym(symmultup))
			{
                            ((void) fprintf(f, "    if (0L == (unsigned long) ("));
                            PrintSymbol(f, SytGCffSym(symmultup));
                            ((void) fprintf(f, "))\n"), nOutpLines++);
			    ((void) fprintf(f, "        goto _d%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
			}
		    if ((!ArcLast(arc)) ||
			(symmultup != (SYMMULTUP *) LstLast(ArcGSymsum(arc))))
			((void) fprintf(f, "    MarSet(_marking, _u%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ");\n"), nOutpLines++);
                    if (SytGCffSym(symmultup))
			{
			    ((void) fprintf(f, "    MultupIFreeCount(%s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ", (unsigned long) ("));
                            PrintSymbol(f, SytGCffSym(symmultup));
                            ((void) fprintf(f, "));\n"), nOutpLines++);
			}
	            else
			{
			    ((void) fprintf(f, "    MultupIFreeCount(%s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ", (unsigned long)"));
                            ((void) fprintf(f, " %s", UlongStr(SytGMulti(symmultup))), (void) fprintf(f, ");\n"), nOutpLines++);
			}
		    ((void) fprintf(f, "_c%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ": ;\n"), nOutpLines++);
                    ((void) fprintf(f, "    if ((%s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, " = MarStep(_marking)) &&\n"), nOutpLines++);
                    ((void) fprintf(f, "        (MultupGArity(%s_t%s", ls, UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ") == (unsigned)"));
                    ((void) fprintf(f, " %s", UnsStr(SytGArity(symmultup))), (void) fprintf(f, "))\n"), nOutpLines++);
		    ((void) fprintf(f, "        goto _b%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ";\n"), nOutpLines++);
                    ((void) fprintf(f, "_d%s", UnsStr(a)), (void) fprintf(f, "_%s", UnsStr(s)), (void) fprintf(f, ": ;\n"), nOutpLines++);
		}
	}
    ((void) fprintf(f, "}\n"), nOutpLines++);
}
