/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  genesig.c
*
* Part of: PROD's programs araprod and "gphgen"
*
* Author:  Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     This module takes care of signal handling during reachability
*     graph generation.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#ifdef UNIX
#include <sys/types.h>
#include <unistd.h>
#endif
#include <signal.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "genesig.h"

BOOL continueGeneration;  /* if this becomes FALSE, the generation
                           * of the reachability graph is interrupted
                           * as soon as possible (an immediate interrupt
                           * would leave the graph inconsistent) */
BOOL stopFlag;            /* if this becomes TRUE, the generation
                           * of the reachability graph is stopped
                           * as soon as possible, but the generator
                           * process is suspended, thus not killed */

/***************************************************************************
*
* Function:  IgnoreSignals
* Purpose:   This routine forces certain signals to be ignored.
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, IgnoreSignals)
{
#ifdef UNIX
    (void) signal(SIGHUP, SIG_IGN);
    (void) signal(SIGQUIT, SIG_IGN);
    (void) signal(SIGTERM, SIG_IGN);
#endif
}

#if (defined(UNIX) || !defined(_WINDOWS))
/***************************************************************************
*
* Function:  KSGeneration
* Purpose:   This routine handles the SIGINT and SIGTSTP signals.
* Arguments: sig - the number of the signal
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, KSGeneration,
    int, sig)
{
    if (((int) SIGINT) == sig)
	{
#ifdef UNIX
	    (void) signal(SIGTSTP, SIG_IGN);
#endif
	    (void) signal(SIGINT, SIG_IGN);
	    continueGeneration = FALSE;
	    stopFlag = FALSE;
	}
#ifdef UNIX
    else if (((int) SIGTSTP) == sig)
	{
	    (void) signal(SIGINT, SIG_IGN);
	    (void) signal(SIGTSTP, SIG_IGN);
	    stopFlag = TRUE;
	    continueGeneration = TRUE;
	}
#endif
}
#endif

/***************************************************************************
*
* Function:  SetSigHandlers
* Purpose:   This routine sets the signal handlers.
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, SetSigHandlers)
{
#ifndef _WINDOWS
    (void) signal(SIGINT, KSGeneration);
#endif
#ifdef UNIX
    (void) signal(SIGTSTP, KSGeneration);
#endif
}

/***************************************************************************
*
* Function:  IgnSigHandlers
* Purpose:   This routine disables the signal handlers. The corresponding
*            signals are then ignored.
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, IgnSigHandlers)
{
#ifndef _WINDOWS
    (void) signal(SIGINT, SIG_IGN);
#endif
#ifdef UNIX
    (void) signal(SIGTSTP, SIG_IGN);
#endif
}

/***************************************************************************
*
* Function:  Suspend
* Purpose:   The reachability graph generator process can suspend itself
*            simply by calling this routine.
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, Suspend)
{
#ifdef UNIX
    (void) kill(getpid(), SIGSTOP);
#endif
}
