/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  lexer.c
*
* Part of: PROD's program prpp
*
* Authors: Heikki Suonsivu and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     This module performs most of the lexical analysis of the
*     net description.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ppio.h"
#include "token.h"
#include "util.h"
#include "prpp.h"
#include "dynamic.h"
#include "lexer.h"

#define TEXT_STATE int

#define INQUOTE 1
#define INNAME 2
#define INNUMBER 3
#define INSPACE 4
#define INCOMMENT 5
#define NONE 6

PP_TOKEN_TYPE closing_bracket_pair[4] =
{
    CLOSING_BRACKET, CLOSING_SQUARE_BRACKET, CLOSING_PARENTHESIS,
    TUPLE_RIGHT
};

/***************************************************************************
*
* Function:  finish_token
* Purpose:
* Arguments: buffer - 
*            state - 
* Returns:
* See also:
*
****************************************************************************
*/
FS2(PP_TOKEN *, finish_token,
    char *, buffer,
    TEXT_STATE, state)
{
    register PP_TOKEN *token;
    register int c;
    
    token = PpTokCons(buffer, NOTHING);
    Deallocate((char *) buffer);
    switch (state)
	{
	case INQUOTE:
	    token->type = QUOTED_EXPRESSION;
	    break;
      
	case INNAME:
	    token->type = NAME;
	    break;

	case INNUMBER:
	    token->type = NUMBER;
	    break;

	case INSPACE:
	    token->type = SPACE;
	    break;

	case INCOMMENT:
	    token->type = COMMENT;
	    break;

	case NONE:
	    switch (*(token->token))
		{
		case (int) '#':
		    token->type = DV;
		    break;
	  
		case (int) '\n':
		    token->type = NEWLINE;
		    break;
	  
		case (int) '(':
		    token->type = OPENING_PARENTHESIS;
		    break;
	  
		case (int) '[':
		    token->type = OPENING_SQUARE_BRACKET;
		    break;
	  
		case (int) '{':
		    token->type = OPENING_BRACKET;
		    break;
	  
		case (int) ')':
		    token->type = CLOSING_PARENTHESIS;
		    break;
	  
		case (int) ']':
		    token->type = CLOSING_SQUARE_BRACKET;
		    break;
	  
		case (int) '}':
		    token->type = CLOSING_BRACKET;
		    break;
	  
		case (int) ',':
		    token->type = COMMA;
		    break;
	  
		case (int) '+':
		    token->type = PLUS;
		    break;

		case (int) '-':
		    token->type = MINUS;
		    break;

		case (int) '*':
		    token->type = MULTIPLY;
		    break;

		case (int) '/':
		    token->type = DIVIDE;
		    break;

		case (int) '%':
		    token->type = REMAINDER;
		    break;
	  
		case (int) '!':
		    c = ppio_getc();
		    if (c != EOF)
			{
			    if (c == (int) '=')
				{
				    Deallocate((char *) token->token);
				    token->token = StrCopy("!=");
				    token->type = NOT_EQUAL;
				    break;
				}
			    c_ppio_unget(c);
			}
	  
		    token->type = NOT;
		    break;
	  
		case (int) '^':
		    token->type = BINARY_XOR;
		    break;
	  
		case (int) '~':
		    token->type = BINARY_NOT;
		    break;

		case (int) '?':
		    token->type = CONDITIONAL;
		    break;

		case (int) ':':
		    token->type = COLON;
		    break;

		case (int) ';':		
		    token->type = SEMICOLON;
		    break;

		case (int) '<':
	  
		    c = ppio_getc();
		    if (c != EOF)
			{
			    if (c == (int) '=')
				{
				    Deallocate((char *) token->token);
				    token->token = StrCopy("<=");
				    token->type = LESS_THAN_OR_EQUAL;
				    break;
				}
			    else if (c == (int) '<')
				{
				    Deallocate((char *) token->token);
				    token->token = StrCopy("<<");
				    token->type = SHIFT_LEFT;
				    break;
				}
			    else if (c == (int) '.')
				{
				    Deallocate((char *) token->token);
				    token->token = StrCopy("<.");
				    token->type = TUPLE_LEFT;
				    break;
				}
			    c_ppio_unget(c);
			}
		    token->type = LESS_THAN;
		    break;
	  
		case (int) '>':
	  
		    c = ppio_getc();
		    if (c != EOF)
			{
			    if (c == (int) '=')
				{
				    Deallocate((char *) token->token);
				    token->token = StrCopy(">=");
				    token->type = MORE_THAN_OR_EQUAL;
				    break;
				}
			    else if (c == (int) '>')
				{
				    Deallocate((char *) token->token);
				    token->token = StrCopy(">>");
				    token->type = SHIFT_RIGHT;
				    break;
				}
			    c_ppio_unget(c);
			}
		    token->type = MORE_THAN;
		    break;

		case (int) '.':
	  
		    c = ppio_getc();
		    if (c != EOF)
			{
			    if (c == (int) '>')
				{
				    Deallocate((char *) token->token);
				    token->token = StrCopy(".>");
				    token->type = TUPLE_RIGHT;
				    break;
				}
			    else if (c == (int) '.')
				{
				    Deallocate((char *) token->token);
				    token->token = StrCopy("..");
				    token->type = TUPLE_RANGE;
				    break;
				}
			    c_ppio_unget(c);
			}
		    token->type = SPECIAL; 
		    break;
		case (int) '=':
		    c = ppio_getc();
		    if (c != EOF)
			{
			    if (c == (int) '=')
				{
				    Deallocate((char *) token->token);
				    token->token = StrCopy("==");
				    token->type = EQUAL;
				    break;
				}
			    c_ppio_unget(c);
			}

		    token->type = SPECIAL; 
		    break;
	  
		case (int) '&':
		    c = ppio_getc();
		    if (c != EOF)
			{
			    if (c == (int) '&')
				{
				    Deallocate((char *) token->token);
				    token->token = StrCopy("&&");
				    token->type = AND;
				    break;
				}
			    c_ppio_unget(c);
			}

		    token->type = BINARY_AND;
		    break;
	  
		case (int) '|':
		    c = ppio_getc();
		    if (c != EOF)
			{
			    if (c == (int) '|')
				{
				    Deallocate((char *) token->token);
				    token->token = StrCopy("||");
				    token->type = OR;
				    break;
				}
	      
			    c_ppio_unget(c);
			}

		    token->type = BINARY_OR;
		    break;
	      
		default:
		    token->type = SPECIAL;
		}
	    break;
      
	default:
	    (void) sprintf(errStr, "Unknown state %d in get_token", state);
	    panic(errStr);
      
	}
    return token;
}

/***************************************************************************
*
* Function:  get_token
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(PP_TOKEN *, get_token)
{
  register int c, quotechar;
  TEXT_STATE state = NONE; 
  BUFFER_DEFINITION(register);
  
  INITIALIZE_BUFFER(8);

  quotechar = EOF;
  for (c = ppio_getc(); c != EOF; c = ppio_getc())
    {
      switch (state)
	{
	 case INQUOTE:
	  if (c == quotechar)
	    {
	      
	      ADD_BUFFER(c);
	      ADD_BUFFER('\0');
	      return finish_token(buffer, state);
	    }
	  else if (c == (int) '\\')
	    {
	      
	      c = ppio_getc();
	      if (c == EOF)
		{
		  panic("Premature end inside a string constant.");
		  
		}
	      ADD_BUFFER('\\');
	    }

	  ADD_BUFFER(c);
	  break;
	  
	 case INNAME:
	  
	  if ((!isalnum((UCHAR) c)) && (c !=  '_') && (c != '$'))
	      {
		  c_ppio_unget(c); 
		  ADD_BUFFER('\0');
		  return finish_token(buffer, state);
	      }
	  ADD_BUFFER(c);
	  break;

	 case INNUMBER:
	  
	  if (!isdigit((UCHAR) c))
	      {
		  c_ppio_unget(c); 
		  ADD_BUFFER('\0');
		  return finish_token(buffer, state);
	      }
	  ADD_BUFFER(c);
	  break;

	 case INSPACE:
	  
	  if (c != ' ' && c != '\t')
	      {
		  c_ppio_unget(c);
		  ADD_BUFFER('\0');
		  return finish_token(buffer, state);
	      }
	  ADD_BUFFER(c);
	  break;

	 case INCOMMENT:
	  {
            register unsigned long line = 0L;
	    register char *filename = NULL;
	    
	    
	    may_have_lost_line = TRUE;
	    if (curFile)
	      {
		filename = curFile->name;
		line = curFile->line;
	      }
	    while (c != EOF)
	      {
		ADD_BUFFER(c);
		if (c == (int) '*')
		  {
		    c = ppio_getc();
		    
		    if (c == (int) '/')
		      {
			ADD_BUFFER(c);
			break; 
		      }
		  }
		else
		  c = ppio_getc();
	      }
	    if (c == EOF)
	      {
		if (filename)
		    {
			(void) sprintf(errStr,
                                       "Unterminated comment starts in %s at line %lu",
				       filename, line);
			error(errStr);
		    }
		else
		    error("Unterminated comment, I have lost start information?");
	      }
	    ADD_BUFFER('\0');
	    if (preserve_comments)
	      return finish_token(buffer, state);
	    else if (ansi)
		{
		    Deallocate((char *) buffer);
		    return finish_token(StrCopy(" "), INSPACE);
		}
	    p = buffer;
	    state = NONE;
	    break;
	  }

	 case NONE:
	  
	  if (isdigit((UCHAR) c))
	    {
	      ADD_BUFFER(c);
	      state = INNUMBER;
	      continue;
	    }

	  if (isalpha((UCHAR) c) || (c == (int) '_') || (c == (int) '$'))
	    {
	      ADD_BUFFER(c);
	      state = INNAME;
	      continue;
	    }
	  
	  if (c == (int) ' ' || c == (int) '\t')
	    {
	      ADD_BUFFER(c);
	      state = INSPACE;
	      continue;
	    }

	  switch (c)
	    {
	     case (int) '\'':
	     case (int) '"':
	      ADD_BUFFER(c);
	      state = INQUOTE;
	      quotechar = c;
	      continue;

	     case (int) '\\':
	      
	      if (parsing_control)
		{
		  c = ppio_getc();

		  
		  if (c == EOF)
		    {
		      c = (int) '\\';
		      break;
		    }
		  
		  if (c == (int) '\n')
		    {
		      
		      p = buffer;
		      continue;
		    }
		  c_ppio_unget(c);
		  c = (int) '\\';
		}
	      break;
	      
	     case (int) '/':
	      
	      c = ppio_getc();
	      if (c == (int) '*')
		  {
		      ADD_BUFFER('/');
		      ADD_BUFFER('*');
		      state = INCOMMENT;
		      continue;
		  }
	      else
		  {
		      c_ppio_unget(c);
		      c = (int) '/';
		  }
	      break;
	    }

	  
	  ADD_BUFFER(c);
	  ADD_BUFFER('\0');
	  return finish_token(buffer, state);

	 default:
	  (void) sprintf(errStr, "Unknown state %d in get_token", state);
	  panic(errStr);
	  
	}
    }

  if (p != buffer)
    {
      ADD_BUFFER('\0');
      return finish_token(buffer, state);
    }
  Deallocate((char *) buffer);
  return NULL;
}

/***************************************************************************
*
* Function:  PpTokCons
* Purpose:
* Arguments: s - 
*            type - 
* Returns:
* See also:
*
****************************************************************************
*/
FE2(PP_TOKEN *, PpTokCons,
    char *, s,
    PP_TOKEN_TYPE, type)
{
    PP_TOKEN *token;
    
    token = (PP_TOKEN *) Allocate(sizeof(PP_TOKEN));
    token->token = StrCopy(s);
    token->type = type; 
    return token;
}

/***************************************************************************
*
* Function:  PpTokDracsid
* Purpose:
* Arguments: token - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, PpTokDracsid,
    PP_TOKEN *, token)
{
    if (token)
	{
	    Deallocate((char *) token->token);
	    Deallocate((char *) token);
	}
}

/* A collection of routines for the PP_TOKEN type to be used by
 * list routines of dynam.c and table routines of ptrbl.c. */
TOOLS ppTokTools = { (CFUNC) NULL, PpTokDracsid };

/***************************************************************************
*
* Function:  unget_token
* Purpose:
* Arguments: token - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, unget_token,
    PP_TOKEN *, token)
{
    if (token)
	s_ppio_unget(token->token);
}
