/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  macptree.c
*
* Part of: PROD's program probe
*
* Authors: Tomi Janhunen and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     .
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#ifdef UNIX
#include <sys/types.h>
#else
#ifdef TNIL
#include <sys/types.h>
#else
#include <sys\types.h>
#endif
#endif
#include <string.h>
#include <ctype.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ptrbl.h"
#include "baltree.h"
#include "bdd.h"
#include "gphutil.h"
#include "graph.h"
#include "mark.h"
#include "expr.h"
#include "formula.h"
#include "mbufs.h"
#include "comline.h"
#include "probe.h"
#include "cmdnm.h"
#include "macptree.h"

BALTREE *probeMacros, *probeHelps;
unsigned mEvalTimes, mEvalLimit;

/***************************************************************************
*
* Function:  MacpCons
* Purpose:
* Arguments: type - 
*            cid - 
*            arity - 
*            args - 
*            expCount - 
*            exps - 
* Returns:
* See also:
*
****************************************************************************
*/
FE6(MACPELEM *, MacpCons,
    unsigned, type,
    char *, cid,
    unsigned, arity,
    char **, args,
    unsigned, expCount,
    MACPELEM **, exps)
{
    MACPELEM *macpElem;

    macpElem = MacpAlloc();
    macpElem->type = type;
    if (macpElem->type == ML_SLOT)
	{
	    if (cid)
		Deallocate((char *) cid);
	    macpElem->sid = NULL;
	}
    else
	macpElem->sid = cid;
    switch(macpElem->type)
	{
	case ML_SLOT:
	    macpElem->ua.slot = arity;
	    break;
	case ML_CONST:
	case ML_MACRO:
	    macpElem->ua.arity = arity;
	    macpElem->args = args;
	    macpElem->expCount = expCount;
	    macpElem->exps = exps;
	    break;
	}
    return(macpElem);
}

/***************************************************************************
*
* Function:  MacpCopy
* Purpose:
* Arguments: macpElem - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(MACPELEM *, MacpCopy,
    MACPELEM *, macpElem)
{
    MACPELEM *copy;
    register unsigned i;

    copy = MacpAlloc();
    copy->type = macpElem->type;
    if (copy->type == ML_SLOT)
	copy->sid = NULL;
    else
	copy->sid = StrCopy(macpElem->sid);
    switch(copy->type)
	{
	case ML_SLOT:
	    copy->ua.slot = macpElem->ua.slot;
	    break;
	case ML_CONST:
	case ML_MACRO:
	    if ((i = (copy->ua.arity = macpElem->ua.arity)))
		{
		    copy->args = (char **) AllocArray(i, sizeof(char *));
		    while (i--)
			copy->args[i] = StrCopy(macpElem->args[i]);
		}
	    else
		copy->args = NULL;
	    if ((i = (copy->expCount = macpElem->expCount)))
		{
		    copy->exps = (MACPELEM **)
			AllocArray(i, sizeof(MACPELEM *));
		    while (i--)
			copy->exps[i] = MacpCopy(macpElem->exps[i]);
		}
	    else
		copy->exps = NULL;
	    break;
	}
    return(copy);
}

/***************************************************************************
*
* Function:  MacpDiscard
* Purpose:
* Arguments: macpElem - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, MacpDiscard,
    MACPELEM *, macpElem)
{
    register unsigned i;

    if (macpElem->type != ML_SLOT)
	Deallocate((char *) macpElem->sid);
    if ((macpElem->type == ML_CONST) || (macpElem->type == ML_MACRO))
	{
	    if ((i = macpElem->ua.arity))
		{
		    while (i--)
			Deallocate((char *) macpElem->args[i]);
		    Deallocate((char *) (macpElem->args));
		}
	    if ((i = macpElem->expCount))
		{
		    while (i--)
			MacpDiscard(macpElem->exps[i]);
		    Deallocate((char *) (macpElem->exps));
		}
	}
    Deallocate((char *) macpElem);
}

#ifdef DEBUG
/***************************************************************************
*
* Function:  MacpDump
* Purpose:
* Arguments: macpElem - 
*            depth - 
* Returns:
* See also:
*
****************************************************************************
*/
FE2(void, MacpDump,
    MACPELEM *, macpElem,
    unsigned, depth)
{
    unsigned i;

    DumpIndent(depth);
    (void) fprintf(_err, "MACPELEM ");
    switch(macpElem->type)
	{
	case ML_SLOT:
	    (void) fprintf(_err, "slot %u\n", macpElem->ua.slot);
	    break;
	case ML_CONST:
	    (void) fprintf(_err, "const");
	    break;
	case ML_MACRO:
	    (void) fprintf(_err, "macro");
	    break;
	case ML_STRING:
	    (void) fprintf(_err, "string");
            /* intentional abuse of switch */
	case ML_PARM:
	    (void) fprintf(_err, "parameter");
	    break;
	}
    if (macpElem->type != ML_SLOT)
	(void) fprintf(_err, " %s\n", macpElem->sid);
    if (macpElem->type == ML_MACRO)
	{
	    DumpIndent(depth + 1);
	    (void) fprintf(_err, "args\n");
	    for (i = 0; i < macpElem->ua.arity; i++)
		{
		    DumpIndent(depth + 2);
		    (void) fprintf(_err, "%s\n", macpElem->args[i]);
		}
	}
    if ((macpElem->type == ML_CONST) || (macpElem->type == ML_MACRO))
	{
	    DumpIndent(depth + 1);
	    (void) fprintf(_err, "expansions\n");
	    for (i = 0; i < macpElem->expCount; i++)
		MacpDump(macpElem->exps[i], depth + 2);
	}
    DumpIndent(depth);
    (void) fprintf(_err, "End of MACPELEM\n");
    (void) fflush(_err);
}
#endif

TOOLS macpTools = 
{ (CFUNC) MacpCopy, MacpDiscard
#ifdef DEBUG
, MacpDump
#endif
 };

/***************************************************************************
*
* Function:  MacpKeyCmp
* Purpose:
* Arguments: macpElem - 
*            key - 
* Returns:
* See also:
*
****************************************************************************
*/
FE2(int, MacpKeyCmp,
    MACPELEM *, macpElem,
    char *, key)
{
    return((macpElem->type == ML_SLOT) ?
	   ((int) (macpElem->ua.slot - *((unsigned *) Pvoid(key)))) :
	   strcmp(macpElem->sid, key));
}

/***************************************************************************
*
* Function:  MacpPrefix
* Purpose:
* Arguments: macpElem - 
*            prefix - 
* Returns:
* See also:
*
****************************************************************************
*/
FE2(BOOL, MacpPrefix,
    MACPELEM *, macpElem,
    char *, prefix)
{
    return((macpElem->type == ML_SLOT) ?
	   (macpElem->ua.slot == *((unsigned *) Pvoid(prefix))) :
	   StrPrefix(macpElem->sid, prefix));
}

BALTOOLS macpBalTools =
{ (CFUNC) MacpCopy, MacpDiscard
#ifdef DEBUG
, MacpDump
#endif
, MacpKeyCmp, MacpPrefix };

FILE *shmFile;

/***************************************************************************
*
* Function:  ShowMacro
* Purpose:
* Arguments: macpElem - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(BOOL, ShowMacro,
    MACPELEM *, macpElem)
{
    unsigned i;

    if (macpElem->type == ML_MACRO)
	{
	    (void) fprintf(shmFile, "#define %s(", macpElem->sid);
	    if (macpElem->ua.arity)
		{
		    (void) fprintf(shmFile, "%s", macpElem->args[0]);
		    for (i = 1; i < macpElem->ua.arity; i++)
			(void) fprintf(shmFile, ",%s", macpElem->args[i]);
		}
	    (void) fprintf(shmFile, ") ");
	}
    else
	(void) fprintf(shmFile, "#define %s ", macpElem->sid);
    for (i = 0; i < macpElem->expCount; i++)
	{
	    if (macpElem->exps[i]->type == ML_SLOT)
		(void) fprintf(shmFile, "%s",
			       macpElem->args[macpElem->exps[i]->ua.slot]);
	    else
		(void) fprintf(shmFile, "%s", macpElem->exps[i]->sid);
	}
    (void) fprintf(shmFile, "\n");
    (void) fflush(shmFile);
    return(FALSE);
}

char *shdPrefix;

/***************************************************************************
*
* Function:  ShowDef
* Purpose:
* Arguments: str - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(BOOL, ShowDef,
    char *, str)
{
    if (!(shdPrefix && StrPrefix(str, shdPrefix)))
	{
	    MACPELEM *macpElem;

	    shdPrefix = str;
	    if ((macpElem = (MACPELEM *)
		 BalTrGrEqPrefix(probeMacros, (char *) shdPrefix, TRUE)))
		{
		    do
			{
			    (void) ShowMacro(macpElem);
			} while ((macpElem = (MACPELEM *)
				  BalTrNext(probeMacros, (char *) macpElem,
					    (char *) (macpElem->sid),
					    (char *) shdPrefix)));
		}
	    else
		{
		    (void) fprintf(_err, "%s: no such definition\n",
				   shdPrefix);
		    (void) fflush(_err);
		    errorFlag = MAC_ERROR;
		}
	}
    return(FALSE);
}

/***************************************************************************
*
* Function:  Undefine
* Purpose:
* Arguments: str - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(BOOL, Undefine,
    char *, str)
{
    if (!BalTrDelete(probeMacros, (char *) NULL, (char *) str, TRUE))
	{
	    (void) fprintf(_err, "%s: no such definition\n", str);
	    (void) fflush(_err);
	    errorFlag = MAC_ERROR;
	}
    return(FALSE);
}

/***************************************************************************
*
* Function:  Define
* Purpose:
* Arguments: type - 
*            name - 
*            argList - 
*            defList - 
* Returns:
* See also:
*
****************************************************************************
*/
FE4(void, Define,
    unsigned, type,
    char *, name,
    LIST *, argList,
    LIST *, defList)
{
    MACPELEM *macpElem, *dupMac;
    char *str;
    REF *arg, *dupArg;
    BALTREE *argTree = NULL;
    unsigned i, arity, expCount;
    char **args;
    MACPELEM **exps;
    
    if (BalTrElement(resTree, (char *) name))
	{
	    (void) fprintf(_err, "%s: reserved word\n", name);
	    (void) fflush(_err);
	    errorFlag = MAC_ERROR;
	    Deallocate((char *) name);
	    if (argList)
		LstDestroy(argList, TRUE);
	    if (defList)
		LstDestroy(defList, TRUE);
	    return;
	}
    if (argList)
	{
	    argTree = BalTrCreate(&refBalTools);
	    LSTNUM (argList, str, char, i)
		{
		    arg = RefCons(str, (unsigned long) i);
		    if ((dupArg = (REF *)
			 BalTrInsert(argTree, (char *) arg, (char *) str,
				     TRUE)))
			{
			    (void) fprintf(_err,
					   "Duplicate parameter %s (%u, %u) for %s\n",
					   str, (unsigned) RefGSlot(dupArg),
					   i, name);
			    (void) fflush(_err);
			    RefDiscard(arg);
			    errorFlag = MAC_ERROR;
			}
		}
	    if (errorFlag != NO_ERROR)
		{
		    BalTrDestroy(argTree, TRUE);
		    Deallocate((char *) name);
		    LstDestroy(argList, TRUE);
		    if (defList)
			LstDestroy(defList, TRUE);
		    return;
		}
	}
    if (defList)
	{
	    LSTFOR (defList, macpElem, MACPELEM)
		{
		    if (macpElem->type == ML_PARM)
			{
			    if (argList &&
				(arg = (REF *)
				 BalTrElement(argTree, (char *)
					      (macpElem->sid))))
				{
				    macpElem->type = ML_SLOT;
				    Deallocate((char *) macpElem->sid);
				    macpElem->sid = NULL;
				    macpElem->ua.slot =
					(unsigned) RefGSlot(arg);
				}
			    else
				macpElem->type = ML_STRING;
			}
		}
	}
    if (argList)
	BalTrDestroy(argTree, TRUE);
    if (argList && !LstEmpty(argList))
	{
	    args = (char **)
		AllocArray(arity = LstSize(argList), sizeof(char *));
	    LSTNUM (argList, str, char, i)
		args[i] = str;
	}
    else
	{
	    arity = 0;
	    args = NULL;
	}
    if (argList)
	LstDestroy(argList, FALSE);
    if (defList && !LstEmpty(defList))
	{
	    exps = (MACPELEM **) 
		AllocArray(expCount = LstSize(defList), sizeof(MACPELEM *));
	    LSTNUM (defList, macpElem, MACPELEM, i)
		exps[i] = macpElem;
	}
    else
	{
	    expCount = 0;
	    exps = NULL;
	}
    if (defList)
	LstDestroy(defList, FALSE);
    macpElem = MacpCons(type, name, arity, args, expCount, exps);
    if ((dupMac = (MACPELEM *)
	 BalTrInsert(probeMacros, (char *) macpElem, (char *) name, TRUE)))
	{
	    (void) fprintf(_err, "%s: definition already exists\n",
			   name);
	    shmFile = _err;
	    (void) ShowMacro(dupMac);
	    MacpDiscard(macpElem);
	    errorFlag = MAC_ERROR;
	}
}

/***************************************************************************
*
* Function:  Evaluate
* Purpose:
* Arguments: macpElem - 
*            plist - 
* Returns:
* See also:
*
****************************************************************************
*/
FE2(LIST *, Evaluate,
    MACPELEM *, macpElem,
    LIST *, plist)
{
    LIST *param, **args = NULL, *result;
    char *str;
    unsigned i;

    i = plist ? LstSize(plist) : 0;
    if (i != macpElem->ua.arity)
	{
	    (void) fprintf(_err,
			   "%s should have %u instead of %u arguments\n",
			   macpElem->sid, macpElem->ua.arity, i);
	    errorFlag = MAC_ERROR;
	    if (plist)
		LstDestroy(plist, TRUE);
	    return(NULL);
	}
    if (plist)
	{
	    LSTNUM (plist, param, LIST, i)
		{
		    if (LstEmpty(param))
			{
			    (void) fprintf(_err,
					   "Empty argument (number %u) for %s\n",
					   i, macpElem->sid);
			    (void) fflush(_err);
			    errorFlag = MAC_ERROR;
			    LstDestroy(plist, TRUE);
			    return(NULL);
			}
		}
	}
    if (mEvalTimes + 1 >= mEvalLimit)
	{
	    (void) fprintf(_err, "Too many macro evaluations needed\n");
	    (void) fflush(_err);
	    errorFlag = MAC_ERROR;
	    if (plist)
		LstDestroy(plist, TRUE);
	    return(NULL);
	}
    mEvalTimes++;
    if (plist && !LstEmpty(plist))
	{
	    args = (LIST **) AllocArray(macpElem->ua.arity, sizeof(LIST *));
	    LSTNUM (plist, param, LIST, i)
		args[i] = param;
	}
    if (plist)
	LstDestroy(plist, FALSE);
    result = LstCreate(&strTools);
    for (i = 0; i < macpElem->expCount; i++)
	{
	    if (macpElem->exps[i]->type == ML_SLOT)
		{
		    param = args[macpElem->exps[i]->ua.slot];
		    LSTFOR (param, str, char)
			(void) LstInsLast(result, (char *) StrCopy(str));
		}
	    else
		(void) LstInsLast(result, (char *)
				  StrCopy(macpElem->exps[i]->sid));
	}
    return(result);
}

/***************************************************************************
*
* Function:  HelpCons
* Purpose:
* Arguments: key - 
*            header - 
*            data - 
* Returns:
* See also:
*
****************************************************************************
*/
FE3(HELPITEM *, HelpCons,
    LIST *, key,
    LIST *, header,
    LIST *, data)
{
    HELPITEM *helpItem;

    helpItem = HelpAlloc();
    helpItem->key = key;
    helpItem->header = header;
    helpItem->data = data;
    return(helpItem);
}

/***************************************************************************
*
* Function:  HelpCopy
* Purpose:
* Arguments: helpItem - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(HELPITEM *, HelpCopy,
    HELPITEM *, helpItem)
{
    HELPITEM *copy;

    copy = HelpAlloc();
    copy->key = LstCopy(helpItem->key, FALSE);
    copy->header = LstCopy(helpItem->header, TRUE);
    copy->data = LstCopy(helpItem->data, TRUE);
    return(copy);
}

/***************************************************************************
*
* Function:  HelpDiscard
* Purpose:
* Arguments: helpItem - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, HelpDiscard,
    HELPITEM *, helpItem)
{
    LstDestroy(helpItem->key, FALSE);
    LstDestroy(helpItem->header, TRUE);
    LstDestroy(helpItem->data, TRUE);
    Deallocate((char *) helpItem);
}

#ifdef DEBUG
/***************************************************************************
*
* Function:  HelpDump
* Purpose:
* Arguments: helpItem - 
*            depth - 
* Returns:
* See also:
*
****************************************************************************
*/
FE2(void, HelpDump,
    HELPITEM *, helpItem,
    unsigned, depth)
{
    DumpIndent(depth);
    (void) fprintf(_err, "HELPITEM\n");
    DumpIndent(depth + 1);
    (void) fprintf(_err, "key\n");
    LstDump(helpItem->key, depth + 2);
    DumpIndent(depth + 1);
    (void) fprintf(_err, "header\n");
    LstDump(helpItem->header, depth + 2);
    DumpIndent(depth + 1);
    (void) fprintf(_err, "data\n");
    LstDump(helpItem->data, depth + 2);
    (void) fflush(_err);
}
#endif

/***************************************************************************
*
* Function:  HelpKeyCmp
* Purpose:
* Arguments: helpItem - 
*            key - 
* Returns:
* See also:
*
****************************************************************************
*/
FE2(int, HelpKeyCmp,
    HELPITEM *, helpItem,
    LIST *, key)
{
    return(StrLstCompare(helpItem->key, key));
}

/***************************************************************************
*
* Function:  HelpPrefix
* Purpose:
* Arguments: helpItem - 
*            prefix - 
* Returns:
* See also:
*
****************************************************************************
*/
FE2(BOOL, HelpPrefix,
    HELPITEM *, helpItem,
    LIST *, prefix)
{
    return(StrLstPrefix(helpItem->key, prefix));
}

BALTOOLS helpBalTools =
{ (CFUNC) HelpCopy, HelpDiscard
#ifdef DEBUG
, HelpDump
#endif
, HelpKeyCmp, HelpPrefix };

/***************************************************************************
*
* Function:  HelpHdrOutput
* Purpose:
* Arguments: helpItem - 
* Returns:
* See also:
*
****************************************************************************
*/
FS1(BOOL, HelpHdrOutput,
    HELPITEM *, helpItem)
{
    char *str;

    LSTFOR(helpItem->header, str, char)
	(void) fprintf(shmFile, "%s", str);
    (void) fprintf(shmFile, "\n");
    (void) fflush(shmFile);
    return(FALSE);
}

/***************************************************************************
*
* Function:  ReadHelp
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, ReadHelp)
{
    HELPITEM *newItem;
    char *str;
    int ch, ch2;
    unsigned len;
    FILE *file;

    (void) fprintf(_out, "Loading help file ...");
    (void) fflush(_out);
    if (!(file = OpenText((char *) NULL, probHelp, FALSE, FALSE)))
	return;
    probeHelps = BalTrCreate(&helpBalTools);
    ch = fgetc(file);
    while (ch != EOF)
	{
	    if (ch != (int) '@')
		{
		    while ((ch != EOF) && (ch != (int) '\n'))
			ch = fgetc(file);
		    if (ch != EOF)
			ch = fgetc(file);
		    continue;
		}
	    newItem = HelpCons(LstCreate(&strTools),
			       LstCreate(&strTools),
			       LstCreate(&strTools));
	    ch = fgetc(file);
	    while ((ch != EOF) && (ch != (int) '\n'))
		{
		    if (isalpha((UCHAR) ch) || (ch == (int) '_') ||
			(ch == (int) '$'))
			{
			    MbufInit(evalBuf);
			    MbufPut(evalBuf, ch);
			    ch = fgetc(file);
			    while ((ch != EOF) &&
				   (isalnum((UCHAR) ch) || (ch == (int) '_') ||
				    (ch == (int) '$')))
				{
				    MbufPut(evalBuf, ch);
				    ch = fgetc(file);
				}
			    len = MbufSize(evalBuf);
			    str = strncpy((char *) Allocate(len + 1),
					  evalBuf->_buf, (size_t) len);
			    str[len] = '\0';
			    (void) LstInsLast(newItem->key, (char *) str);
			    (void) LstInsLast(newItem->header, (char *) str);
			}
		    else if (isdigit((UCHAR) ch))
			{
			    MbufInit(evalBuf);
			    MbufPut(evalBuf, ch);
			    ch = fgetc(file);
			    while ((ch != EOF) && isdigit((UCHAR) ch))
				{
				    MbufPut(evalBuf, ch);
				    ch = fgetc(file);
				}
			    len = MbufSize(evalBuf);
			    str = strncpy((char *) Allocate(len + 1),
					  evalBuf->_buf, (size_t) len);
			    str[len] = '\0';
			    (void) LstInsLast(newItem->key, (char *) str);
			    (void) LstInsLast(newItem->header, (char *) str);
			}
		    else if ((ch == (int) ' ') || (ch == (int) '\t'))
			{
			    MbufInit(evalBuf);
			    MbufPut(evalBuf, ch);
			    ch = fgetc(file);
			    while ((ch != EOF) && 
				   ((ch == (int) ' ') || (ch == (int) '\t')))
				{
				    MbufPut(evalBuf, ch);
				    ch = fgetc(file);
				}
			    len = MbufSize(evalBuf);
			    str = strncpy((char *) Allocate(len + 1),
					  evalBuf->_buf, (size_t) len);
			    str[len] = '\0';
			    (void) LstInsLast(newItem->header, (char *) str);
			}
		    else
			{ 
			    ch2 = fgetc(file);
			    if ((ch2 != EOF) && 
				(((ch == (int) '<') && 
				  ((ch2 == (int) '.') || (ch2 == (int) '<') ||
				   (ch2 == (int) '=')))
				 ||
				 ((ch == (int) '.') &&
				  ((ch2 == (int) '.') || (ch2 == (int) '>')))
				 ||
				 ((ch == (int) '>') &&
				  ((ch2 == (int) '>') || (ch2 == (int) '=')))
				 ||
				 (((ch == (int) '!') || (ch == (int) '=')) &&
				  (ch2 == (int) '='))
				 ||
				 (((ch == (int) '&') || (ch == (int) '|') ||
				   (ch == (int) '%')) && (ch2 == ch))))
				{
				    str = (char *) Allocate(3);
				    str[0] = (char) ch;
				    str[1] = (char) ch2;
				    str[2] = '\0';
				    ch = fgetc(file);
				}
			    else
				{
				    str = (char *) Allocate(2);
				    str[0] = (char) ch;
				    str[1] = '\0';
				    ch = ch2;
				}
			    (void) LstInsLast(newItem->key, (char *) str);
			    (void) LstInsLast(newItem->header, (char *) str);
			}
		}
	    if (ch != EOF)
		ch = fgetc(file);
	    if (LstEmpty(newItem->key))
		{
		    HelpDiscard(newItem);
		    continue;
		}
	    if (BalTrInsert(probeHelps, (char *) newItem,
			    (char *) (newItem->key), TRUE))
		{
		    (void) fprintf(_err,
				   "\nDuplicate help subject (skipped):\n");
		    shmFile = _err;
		    (void) HelpHdrOutput(newItem);
		    HelpDiscard(newItem);
		    continue;
		}
	    MbufInit(evalBuf);
	    while ((ch != EOF) && (ch != (int) '@'))
		{
		    while ((ch != EOF) && (ch != (int) '\n'))
			{
			    if (evalBuf->_written == evalBuf->_max)
				{
				    len = MbufSize(evalBuf);
				    str = strncpy((char *) Allocate(len + 1),
						  evalBuf->_buf, (size_t) len);
				    str[len] = '\0';
				    (void) LstInsLast(newItem->data,
						      (char *) str);
				    MbufInit(evalBuf);
				}
			    MbufPut(evalBuf, ch);
			    ch = fgetc(file);
			}
		    if (evalBuf->_written == evalBuf->_max)
			{
			    len = MbufSize(evalBuf);
			    str = strncpy((char *) Allocate(len + 1),
					  evalBuf->_buf, (size_t) len);
			    str[len] = '\0';
			    (void) LstInsLast(newItem->data, (char *) str);
			    MbufInit(evalBuf);
			}
		    MbufPut(evalBuf, (int) '\n');
		    if (ch != EOF)
			ch = fgetc(file);
		}
	    if ((len = MbufSize(evalBuf)) != 0)
		{
		    str = strncpy((char *) Allocate(len + 1),
				  evalBuf->_buf, (size_t) len);
		    str[len] = '\0';
		    (void) LstInsLast(newItem->data, (char *) str);
		}
	}
    (void) fclose(file);
    (void) fprintf(_out, "\nHelp file loaded\n");
    (void) fflush(_out);
    CutVerbose(_out);
}

/***************************************************************************
*
* Function:  HelpOutput
* Purpose:
* Arguments: strList - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, HelpOutput,
    LIST *, strList)
{
    HELPITEM *helpItem, *nextItem;
    char *str;

    if (probeHelps == NULL)
	ReadHelp();
    if (probeHelps == NULL)
	return;
    shmFile = _out;
    if ((strList == NULL) || LstEmpty(strList))
	{
	    if (BalTrGElCount(probeHelps) == 0L)
		(void) fprintf(_out, "Sorry, no help subject\n");
	    else
		{
		    (void) fprintf(_out, "Help subjects (one per line):\n");
		    CutVerbose(_out);
		    (void) BalTrApply(probeHelps, HelpHdrOutput);
		    CutVerbose(_out);
		    (void) fprintf(_out, "Type help followed by a subject for more information\n");
		    (void) fprintf(_out, "Abbreviations are allowed\n");
		}
	    (void) fflush(_out);
	}
    else if ((helpItem = (HELPITEM *)
	      BalTrGrEqPrefix(probeHelps, (char *) strList, TRUE)))
	{
	    if ((nextItem = (HELPITEM *)
		 BalTrNext(probeHelps, (char *) helpItem,
			   (char *) (helpItem->key), (char *) strList)))
		{
		    if (StrLstCompare(helpItem->key, strList))
			(void) fprintf(_out, "Ambiguous abbreviation\n");
		    else
			{
			    (void) HelpHdrOutput(helpItem);
			    LSTFOR (helpItem->data, str, char)
				(void) fprintf(_out, "%s", str);
			    CutVerbose(_out);
			    (void) fprintf(_out, "Complete but not unique\n");
			}
		    (void) fprintf(_out, "Possible completions (one per line):\n");
		    CutVerbose(_out);
		    (void) HelpHdrOutput(helpItem);
		    do
			{
			    (void) HelpHdrOutput(nextItem);
			} while ((nextItem = (HELPITEM *)
				  BalTrNext(probeHelps, (char *) nextItem,
					    (char *) (nextItem->key),
					    (char *) strList)));
		}
	    else
		{
		    (void) HelpHdrOutput(helpItem);
		    LSTFOR (helpItem->data, str, char)
			(void) fprintf(_out, "%s", str);
		}
	    (void) fflush(_out);
	}
    else
	{
	    (void) fprintf(_err, "No such help subject\n");
	    (void) fflush(_err);
	}
}
