/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  macro.c
*
* Part of: PROD's program prpp
*
* Authors: Heikki Suonsivu and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     This module handles the definitions, expansions and undefinitions
*     of macros in the net description.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include <string.h>
#ifdef UNIX
#include <sys/types.h>
#else
#ifdef TNIL
#include <sys/types.h>
#else
#include <sys\types.h>
#endif
#endif
#include <ctype.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"

#include "ppio.h"
#include "token.h"
#include "macro.h"
#include "util.h"
#include "ptrbl.h"
#include "baltree.h"
#include "gphutil.h"

#define EP_ARGUMENT 0
#define EP_NAME 1
#define EP_STRING 2

typedef struct MACRO_EXPAND_ENTRY
{
    int method;
    union
	{
	    char *string;
	    int varnum;
	} data;
} MACRO_EXPAND_ENTRY;

/***************************************************************************
*
* Function:  free_macro
* Purpose:
* Arguments: macrop - 
* Returns:
* See also:
*
****************************************************************************
*/
FS1(void, free_macro,
    MACRO *, macrop)
{
    MACRO_EXPAND_ENTRY *ep;

    Deallocate((char *) macrop->name);
    if (macrop->expand_scheme)
	{
	    LSTFOR (macrop->expand_scheme, ep, MACRO_EXPAND_ENTRY)
		{
		    if (ep->method != EP_ARGUMENT)
			Deallocate((char *) ep->data.string);
		    Deallocate((char *) ep);
		}
	    LstDestroy(macrop->expand_scheme, FALSE);
	}
    Deallocate((char *) macrop);
}

#ifdef DEBUG
/***************************************************************************
*
* Function:  display_macro
* Purpose:
* Arguments: macrop - 
*            depth - 
* Returns:
* See also:
*
****************************************************************************
*/
FS2(void, display_macro,
    MACRO *, macrop,
    unsigned, depth)
{
    MACRO_EXPAND_ENTRY *ep;

    DumpIndent(depth);
    (void) fprintf(_err, "Macro name %s\n", macrop->name);
    DumpIndent(depth);
    (void) fprintf(_err, "Macro expansion scheme (note reverse order)\n");
    if (macrop->expand_scheme)
	LSTFOR (macrop->expand_scheme, ep, MACRO_EXPAND_ENTRY)
	    {
		DumpIndent(depth + 1);
		if (ep->method != EP_ARGUMENT)
		    (void) fprintf(_err, "String <%s>\n",
				   ep->data.string);
		else
		    (void) fprintf(_err, "Argument %d\n",
				   ep->data.varnum);
	    }
    (void) fflush(_err);
}
#endif

/***************************************************************************
*
* Function:  macro_key_cmp
* Purpose:
* Arguments: macrop - 
*            key - 
* Returns:
* See also:
*
****************************************************************************
*/
FS2(int, macro_key_cmp,
    MACRO *, macrop,
    char *, key)
{
    return(strcmp(macrop->name, key));
}

static BALTOOLS ppmcBalTools =
{ (CFUNC) NULL, free_macro
#ifdef DEBUG
, display_macro
#endif
, macro_key_cmp, (FOOL) NULL };

static BALTREE *macro_tree;
static BALTREE *directives;

/***************************************************************************
*
* Function:  macro_init
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, macro_init)
{
    REF *ref;

    macro_tree = BalTrCreate(&ppmcBalTools);
    directives = BalTrCreate(&refBalTools);
    ref = RefCons(StrCopy("define"), (unsigned long) DV_DEFINE);
    (void) BalTrInsert(directives, (char *) ref, (char *) RefGKey(ref), TRUE);
    ref = RefCons(StrCopy("undef"), (unsigned long) DV_UNDEF);
    (void) BalTrInsert(directives, (char *) ref, (char *) RefGKey(ref), TRUE);
    ref = RefCons(StrCopy("if"), (unsigned long) DV_IF);
    (void) BalTrInsert(directives, (char *) ref, (char *) RefGKey(ref), TRUE);
    ref = RefCons(StrCopy("ifdef"), (unsigned long) DV_IFDEF);
    (void) BalTrInsert(directives, (char *) ref, (char *) RefGKey(ref), TRUE);
    ref = RefCons(StrCopy("ifndef"), (unsigned long) DV_IFNDEF);
    (void) BalTrInsert(directives, (char *) ref, (char *) RefGKey(ref), TRUE);
    ref = RefCons(StrCopy("else"), (unsigned long) DV_ELSE);
    (void) BalTrInsert(directives, (char *) ref, (char *) RefGKey(ref), TRUE);
    ref = RefCons(StrCopy("endif"), (unsigned long) DV_ENDIF);
    (void) BalTrInsert(directives, (char *) ref, (char *) RefGKey(ref), TRUE);
    ref = RefCons(StrCopy("include"), (unsigned long) DV_INCLUDE);
    (void) BalTrInsert(directives, (char *) ref, (char *) RefGKey(ref), TRUE);
    ref = RefCons(StrCopy("enum"), (unsigned long) DV_ENUM);
    (void) BalTrInsert(directives, (char *) ref, (char *) RefGKey(ref), TRUE);
    ref = RefCons(StrCopy("prec"), (unsigned long) DV_PREC);
    (void) BalTrInsert(directives, (char *) ref, (char *) RefGKey(ref), TRUE);
    ref = RefCons(StrCopy("place"), (unsigned long) DV_PLACE);
    (void) BalTrInsert(directives, (char *) ref, (char *) RefGKey(ref), TRUE);
    ref = RefCons(StrCopy("trans"), (unsigned long) DV_TRANS);
    (void) BalTrInsert(directives, (char *) ref, (char *) RefGKey(ref), TRUE);
    ref = RefCons(StrCopy("endtr"), (unsigned long) DV_ENDTR);
    (void) BalTrInsert(directives, (char *) ref, (char *) RefGKey(ref), TRUE);
    ref = RefCons(StrCopy("tester"), (unsigned long) DV_TESTER);
    (void) BalTrInsert(directives, (char *) ref, (char *) RefGKey(ref), TRUE);
    ref = RefCons(StrCopy("verify"), (unsigned long) DV_VERIFY);
    (void) BalTrInsert(directives, (char *) ref, (char *) RefGKey(ref), TRUE);
}

/***************************************************************************
*
* Function:  ismacro
* Purpose:
* Arguments: name - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(MACRO *, ismacro,
    char *, name)
{
    return((MACRO *) BalTrElement(macro_tree, (char *) name));
}

/***************************************************************************
 *
* Function:  isdirective
* Purpose:
* Arguments: name - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(int, isdirective,
    char *, name)
{
    REF *ref;

    ref = (REF *) BalTrElement(directives, (char *) name);
    if (ref)
	return((int) RefGSlot(ref));
    return(0);
}

/***************************************************************************
*
* Function:  macro_clear
* Purpose:
* Arguments: macrop - 
* Returns:
* See also:
*
****************************************************************************
*/
FS1(BOOL, macro_clear,
    MACRO *, macrop)
{
    if (mc_refers(macrop))
	{
	    mc_unvisit(macrop);
	    mc_pop(macrop);
	}
    return(FALSE);
}

static LIST *visStack;
static MACRO *porcam;
static MACRO_EXPAND_ENTRY *epvis;
static BOOL cyclicDef;

/***************************************************************************
*
* Function:  macro_visit
* Purpose:
* Arguments: macrop - 
* Returns:
* See also:
*
****************************************************************************
*/
FS1(void, macro_visit,
    MACRO *, macrop)
{
    if (cyclicDef || (!mc_refers(macrop)) || mc_visited(macrop))
	return;
BEGIN: ;
    mc_enter(macrop);
    epvis = (MACRO_EXPAND_ENTRY *) LstBegin(macrop->expand_scheme);
ARROW: ;
    if (epvis->method != EP_NAME)
	goto PROCEED;
    porcam = ismacro(epvis->data.string);
    if ((!porcam) || !mc_refers(porcam))
	goto PROCEED;
    if (!mc_visited(porcam))
	{
	    (void) LstInsFirst(visStack, (char *) macrop);
	    mc_push(macrop);
	    macrop = porcam;
	    goto BEGIN;
	}
    if ((porcam == macrop) || mc_instack(porcam))
	{
	    (void) sprintf(errStr, "Cyclic definition: %s", porcam->name);
	    error(errStr);
	    (void) fprintf(_err, "is referred by %s\n", macrop->name);
	    if (porcam != macrop)
		LSTFOR (visStack, macrop, MACRO)
		    {
			(void) fprintf(_err, "which is referred by %s\n",
				       macrop->name);
			if (macrop == porcam)
			    break;
		    }
	    (void) fflush(_err);
	    cyclicDef = TRUE;
	    return;
	}
PROCEED: ;
    epvis = (MACRO_EXPAND_ENTRY *) LstStep(macrop->expand_scheme);
    if (epvis)
	goto ARROW;
    if (!LstEmpty(visStack))
	{
	    macrop = (MACRO *) LstDelFirst(visStack, FALSE);
	    mc_pop(macrop);
	    goto PROCEED;
	}
}

/***************************************************************************
*
* Function:  undefine_macro
* Purpose:
* Arguments: macrop - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, undefine_macro,
    MACRO *, macrop)
{
    (void) BalTrDelete(macro_tree, (char *) NULL, (char *) (macrop->name),
		       TRUE);
}

/***************************************************************************
*
* Function:  memorize_macro
* Purpose:
* Arguments: name - 
*            args - 
*            body - 
* Returns:
* See also:
*
****************************************************************************
*/
FE3(MACRO *, memorize_macro,
    char *, name,
    LIST *, args,
    LIST *, body)
{
    MACRO *macrop;
    int argcount;
    MACRO_EXPAND_ENTRY *ep;
    char *str;
    PP_TOKEN *token;
  
    macrop = (MACRO *) Allocate(sizeof(MACRO));
    macrop->name = StrCopy(name);
    macrop->expand_scheme = NULL;
    macrop->args = args ? ((int) LstSize(args)) : (int) -1;
    macrop->flag = 0L;
    (void) BalTrInsert(macro_tree, (char *) macrop, (char *) name, TRUE);
    for (token = (PP_TOKEN *) LstThis(body); token && (token->type != NEWLINE);
	 token = (PP_TOKEN *) LstStep(body))
	{
	    ep = (MACRO_EXPAND_ENTRY *) Allocate(sizeof(MACRO_EXPAND_ENTRY));
	    if (token->type == NAME)
		ep->method = EP_NAME;
	    else
		ep->method = EP_STRING;
	    if ((token->type == NAME) && args)
		{
		    LSTNUM (args, str, char, argcount)
			if (!strcmp(token->token, str))
			    break;
		    if (str)
			{
			    ep->method = EP_ARGUMENT;
			    ep->data.varnum = argcount;
			}
		    else
			ep->data.string = StrCopy(token->token);
		}
	    else
		ep->data.string = StrCopy(token->token);
	    if (!(macrop->expand_scheme))
		macrop->expand_scheme = LstCreate((TOOLS *) NULL);
	    (void) LstInsFirst(macrop->expand_scheme, (char *) ep);
	    if (ep->method == EP_NAME)
		mc_cite(macrop);
	}
    if (!mc_refers(macrop))
	return(macrop);
    (void) BalTrApply(macro_tree, macro_clear);
    visStack = LstCreate((TOOLS *) NULL);
    cyclicDef = FALSE;
    macro_visit(macrop);
    LstDestroy(visStack, FALSE);
    if (cyclicDef)
	{
	    undefine_macro(macrop);
	    return(NULL);
	}
    return(macrop);
}

/***************************************************************************
*
* Function:  expand_macro
* Purpose:
* Arguments: macrop - 
*            args - 
* Returns:
* See also:
*
****************************************************************************
*/
FE2(void, expand_macro,
    MACRO *, macrop,
    LIST **, args)
{
    MACRO_EXPAND_ENTRY *ep;
    PP_TOKEN *token;

    if (macrop->expand_scheme)
	LSTFOR (macrop->expand_scheme, ep, MACRO_EXPAND_ENTRY)
	    {
		if (ep->method == EP_ARGUMENT)
		    LSTFOR (args[ep->data.varnum], token, PP_TOKEN)
			s_ppio_unget(token->token);
		else
		    s_ppio_unget(ep->data.string);
	    }
}

#ifdef TRAIN
FE1(char *, simple_expansion,
    MACRO *, macrop)
{
    MACRO_EXPAND_ENTRY *ep;
    unsigned length, phase;
    char *str, *ret, *str2;

    if (!(macrop->expand_scheme))
	return(NULL);
    length = 1;
    LSTFOR (macrop->expand_scheme, ep, MACRO_EXPAND_ENTRY)
	{
	    if (ep->method == EP_ARGUMENT)
		return(NULL);
	    length += (unsigned) strlen(ep->data.string);
	}
    ret = (char *) Allocate(length);
    *ret = '\0';
    str2 = NULL;
    phase = 0;
    LSTFOR (macrop->expand_scheme, ep, MACRO_EXPAND_ENTRY)
	{
	    str = ep->data.string;
	    if (0 == phase)
		{
		    while ((*str) && isspace((UCHAR) *str))
			str++;
		    if (*str)
			{
			    if (!isdigit((UCHAR) *str))
				{
				    Deallocate(ret);
				    return(NULL);
				}
			    phase = 1;
			    str2 = ret;
			}
		    else
			continue;
		}
	    if (1 == phase)
		{
		    while ((*str) && isdigit((UCHAR) *str))
			*str2++ = *str++;
		    if (*str)
			{
			    if (!isspace((UCHAR) *str))
				{
				    Deallocate(ret);
				    return(NULL);
				}
			    phase = 2;
			    *str2 = '\0';
			}
		    else
			continue;
		}
	    if (2 == phase)
		{
		    while ((*str) && isspace((UCHAR) *str))
			str++;
		    if (*str)
			{
			    Deallocate(ret);
			    return(NULL);
			}
		    else
			continue;
		}
	}
    return(ret);
}
#endif
