/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  mbufs.c
*
* Part of: PROD's program probe
*
* Authors: Keijo Heljanko, Tomi Janhunen and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     .
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ptrbl.h"
#include "baltree.h"
#include "bdd.h"
#include "gphutil.h"
#include "graph.h"
#include "mark.h"
#include "expr.h"
#include "formula.h"
#include "mbufs.h"
#include "macptree.h"
#include "comline.h"
#include "probe.h"
#ifdef USE_READLINE
#include "readline/readline.h"
#include "readline/history.h"
#endif

#define MB_ASTERISK  ((int) '*')
#define MB_BACKSLASH ((int) '\\')
#define MB_EOL       ((int) '\n')
#define MB_SLASH     ((int) '/')
#define MB_SPACE     ((int) ' ')

#define WAIT_NONE   0
#define WAIT_AST   1
#define WAIT_SLASH 2

/***************************************************************************
*
* Function:  MbufCreate
* Purpose:
* Arguments: size - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(MBUF *, MbufCreate,
    unsigned, size)
{
    MBUF *new;

    new = (MBUF *) Allocate(sizeof(MBUF));
    new->_buf = (char *) Allocate(size);
    new->_written = new->_buf;
    new->_read = new->_buf;
    new->_max = new->_buf + size - 1;
    return(new);
}

/***************************************************************************
*
* Function:  MbufPut
* Purpose:
* Arguments: mbuf - 
*            ch - 
* Returns:
* See also:
*
****************************************************************************
*/
FE2(void, MbufPut,
    MBUF *, mbuf,
    int, ch)
{
    if ((ch == (int) '\r') || (ch == /* Ctrl-Z */ (int) '\032'))
	return;
    if (mbuf->_written < mbuf->_max)
	*(mbuf->_written++) = (char) ch;
    else 
	{
	    if (errorFlag == NO_ERROR)
		{
		    (void) fprintf(_err, "Line buffer overflow\n");
		    (void) fflush(_err);
		    errorFlag = BUF_ERROR;
		}
	}
}

/***************************************************************************
*
* Function:  MbufStr
* Purpose:
* Arguments: mbuf - 
*            str - 
* Returns:
* See also:
*
****************************************************************************
*/
FE2(void, MbufStr,
    MBUF *, mbuf,
    char *, str)
{
    while (*str != 0)
	MbufPut(mbuf, *(str++));
}

/***************************************************************************
*
* Function:  MbufReadln
* Purpose:
* Arguments: mbuf - 
*            file - 
* Returns:
* See also:
*
****************************************************************************
*/
#ifdef USE_READLINE
FE4(void, MbufReadln,
    MBUF *, mbuf,
    FILE *, file,
    char *, prompt,
    BOOL, use_readline)
#else
FE2(void, MbufReadln,
    MBUF *, mbuf,
    FILE *, file)
#endif
{
    int ch, cmtDepth, cmtScan;
    BOOL skipEol, endOfLine;
#ifdef USE_READLINE
    BOOL getline = TRUE;
#endif

    cmtDepth = 0;
    cmtScan = WAIT_NONE;
    skipEol = FALSE;
    endOfLine = FALSE;
    do 
	{
#ifdef USE_READLINE
	    ch = MbufFgetc(file, prompt, use_readline, getline);
	    getline = FALSE;
#else
	    ch = fgetc(file);
#endif
	    if (echoInput)
		{
		    if (ch == EOF)
			(void) fputc((int) MB_EOL, _out);
		    else
			(void) fputc((int) ch, _out);
		}
	    if (skipEol)
		{
		    if (ch == MB_EOL)
			{
			    if (MbufSize(mbuf) != 0)
				MbufPut(mbuf, MB_SPACE);
			}
		    else
			{
			    MbufPut(mbuf, MB_BACKSLASH);
			    skipEol = FALSE;
			}
		}
	    if (skipEol)
		skipEol = FALSE;
	    else
		{
		    switch(ch)
			{
			case EOF:
			    switch(cmtScan)
				{
				case WAIT_NONE:
				    break;
				case WAIT_AST:
				    if (cmtDepth == 0)
					MbufPut(mbuf, MB_SLASH);
				    cmtScan = WAIT_NONE;
				    break;
				case WAIT_SLASH:
				    cmtScan = WAIT_NONE;
				    break;
				}
			    if (MbufSize(mbuf) != 0)
				MbufPut(mbuf, MB_EOL);
			    endOfLine = TRUE;
			    if (cmtDepth != 0)
				{
				    (void) fprintf(_err, 
					    "Incomplete comment at the end of input\n");
				    (void) fflush(_err);
				}
			    break;
			case MB_EOL:
			    switch(cmtScan)
				{
				case WAIT_NONE:
				    break;
				case WAIT_AST:
				    if (cmtDepth == 0)
					MbufPut(mbuf, MB_SLASH);
				    cmtScan = WAIT_NONE;
				    break;
				case WAIT_SLASH:
				    cmtScan = WAIT_NONE;
				    break;
				}
			    if (cmtDepth == 0)
				{
				    if (MbufSize(mbuf) != 0)
					MbufPut(mbuf, MB_EOL);
				    endOfLine = TRUE;
				}
			    break;
			case MB_BACKSLASH:
			    switch(cmtScan)
				{
				case WAIT_NONE:
				    break;
				case WAIT_AST:
				    if (cmtDepth == 0)
					MbufPut(mbuf, MB_SLASH);
				    cmtScan = WAIT_NONE;
				    break;
				case WAIT_SLASH:
				    cmtScan = WAIT_NONE;
				    break;
				}
			    if (cmtDepth == 0)
				skipEol = TRUE;
			    break;
			case MB_SLASH:
			    switch(cmtScan)
				{
				case WAIT_NONE:
				    cmtScan = WAIT_AST;
				    break;
				case WAIT_AST:
				    if (cmtDepth == 0)
					MbufPut(mbuf, MB_SLASH);
				    break;
				case WAIT_SLASH:
				    cmtDepth--;
				    if ((cmtDepth == 0) &&
					(MbufSize(mbuf) != 0))
					MbufPut(mbuf, MB_SPACE);
				    cmtScan = WAIT_NONE;
				    break;
				}
			    break;
			case MB_ASTERISK:
			    switch(cmtScan)
				{
				case WAIT_NONE:
				    if (cmtDepth == 0)
					MbufPut(mbuf, MB_ASTERISK);
				    else
					cmtScan = WAIT_SLASH;
				    break;
				case WAIT_AST:
				    cmtDepth++;
				    cmtScan = WAIT_NONE;
				    break;
				case WAIT_SLASH:
				    break;
				}
			    break;
			default:
			    switch(cmtScan)
				{
				case WAIT_NONE:
				    break;
				case WAIT_AST:
				    if (cmtDepth == 0)
					MbufPut(mbuf, MB_SLASH);
				    cmtScan = WAIT_NONE;
				    break;
				case WAIT_SLASH:
				    cmtScan = WAIT_NONE;
				    break;
				}
			    if (cmtDepth == 0)
				MbufPut(mbuf, ch);
			    break;
			}
		}
	} while (!endOfLine);
    if (echoInput)
	(void) fflush(_out);
}

#ifdef USE_READLINE
static UCHAR *readline_buffer=NULL;
static UCHAR *read_pointer=NULL;

/***************************************************************************
*
* Function:  MbufFgetc
* Purpose:
* Arguments: file - a file pointer
*            use_readline - use readline or not
*            getline - get a new line of commands from the user
* Returns:
* See also:
*
****************************************************************************
*/
FE4(int, MbufFgetc,
    FILE *, file,
    char *, prompt,
    BOOL, use_readline,
    BOOL, getline)
{
    if(use_readline == FALSE) {
	return fgetc(file);
    } else {

	if(getline != FALSE) {

	    if(readline_buffer != NULL) {
		Deallocate((char *)readline_buffer);
	    }

	    readline_buffer = (UCHAR *)readline(prompt);

	    if(readline_buffer != NULL &&
	       *readline_buffer != '\0') {

		add_history((char *)readline_buffer);
	    }

	    read_pointer = readline_buffer;
	}

	if(read_pointer == NULL) {
	    return ((int)EOF);
	}

	if(read_pointer != NULL &&
	   *read_pointer != '\0') {

	    return (int)(*read_pointer++);

	} else {
	    read_pointer = NULL;

	    return (int)'\n';
	}

	
    }
}
#endif
