/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  net.h
*
* Part of: PROD's program prpp
*
* Authors: Marko Rauhamaa and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     See net.c.
*
* Version history:
*     $Log$
***************************************************************************/

/* Elements of PLACE are high level places. */
typedef struct PLACE
{
    char *_name;
    unsigned _arity;
    struct {
        LIST *_low,*_high;
    } _range;
    MARKING *_marking;
#ifdef TRAIN
    unsigned long _down;
    unsigned long _up;
    unsigned long _capacity;
    struct PLACE *_complement;
#endif
} PLACE;

/* An element of NET is a net. */
typedef struct NET
{
    LIST *_places; /* of PLACE */
    LIST *_transes; /* of TRANSITION */
    LIST *_precs; /* of PRECEDENCE */
    unsigned _nofClasses;
    PLACE *_monPlace;
    LIST *_atoms;
} NET;

/* An element of VARIABLE is a transition variable (a variable associated
 * with a transition). */
typedef struct VARIABLE
{
    char *_name,*_label;
    DOMAIN _low,_high,_auxLow,_auxHigh;
    unsigned _flag;
    LIST *_coeffs;
} VARIABLE;

extern BOOL placeLimits;

#define TK_NORM 0
#define TK_INITIAL 1
#define TK_DUMMY 2

/* Elements of TRANSITION are high level transitions. */
typedef struct {
  char *_name;
  unsigned _kind;
  unsigned _fdWidth;
  LIST *_inputs,*_outputs,*_arcs; /* of ARC */
  LIST *_vars; /* of VARIABLE */
  LIST *_computation; /* of char * */
  LIST *_gates; /* of SYMBOL */
  LIST *_range;
  LIST *_blocks;
} TRANSITION;

#define SK_CONSTANT  0
#define SK_VARIABLE  1
#define SK_OPERATION 2

/* Elements of SYMBOL are integer-valued C expressions. */
typedef struct SYMBOL
{
    unsigned _flag;
    union
	{
            char *_constant;
            VARIABLE *_var;
            LIST *_operation; /* of SYMBOL */
        } _uy;
} SYMBOL;

/* An element of SYMMULTUP is a single-tuple expression together with
 * a possible multiplier. */
typedef struct SYMMULTUP
{
    SYMBOL *_cffSym;
    unsigned long _multi;
    unsigned _arity;
    SYMBOL **_symtup;
    char *_label;
    unsigned _number;
} SYMMULTUP;

/* Elements of ARC are high level arcs. */
typedef struct ARC
{
    PLACE *_place;
    LIST *_symsum;
    unsigned _number;
    unsigned _flag;
} ARC;

/* Elements of PRECEDENCE are precedence classes. */
typedef struct PRECEDENCE
{
    unsigned _value;
    unsigned _flag;
    LIST *_succ;
    LIST *_pred;
} PRECEDENCE;

#define NetGPlaces(net) ((net)->_places)
#define NetGTransitions(net) ((net)->_transes)
#define NetGPrecedences(net) ((net)->_precs)
#define NetGNofClasses(net) ((net)->_nofClasses)
#define NetGMonPlace(net) ((net)->_monPlace)
#define NetGAtoms(net) ((net)->_atoms)
#define NetAlloc() (NET *) Allocate(sizeof(NET))

#define PlaGName(place) ((place)->_name)
#define PlaGArity(place) ((place)->_arity)
#define PlaSArity(place,arity) ((place)->_arity = (arity))
#define PlaGLow(place) ((place)->_range._low)
#define PlaGHigh(place) ((place)->_range._high)
#define PlaGMarking(place) ((place)->_marking)
#define PlaAlloc() (PLACE *) Allocate(sizeof(PLACE))
extern TOOLS plaTools;
#ifdef TRAIN
#define PlaGDown(place) ((place)->_down)
#define PlaCDown(place) ((place)->_down = (unsigned long) 0L)
#define PlaIDown(place,down) ((place)->_down += (down))
#define PlaGUp(place) ((place)->_up)
#define PlaCUp(place) ((place)->_up = (unsigned long) 0L)
#define PlaIUp(place,up) ((place)->_up += (up))
#define PlaGCapacity(place) ((place)->_capacity)
#define PlaCCapacity(place) ((place)->_capacity = ~((unsigned long) 0L))
#define PlaSCapacity(place,capacity) ((place)->_capacity = (capacity))
#define PlaGComplement(place) ((place)->_complement)
#define PlaCComplement(place) ((place)->_complement = (PLACE *) NULL)
#define PlaSComplement(place,complement) ((place)->_complement = (complement))
#endif

#define VarGName(variable) ((variable)->_name)
#define VarGLabel(variable) ((variable)->_label)
#define VarSLabel(variable,label) ((variable)->_label = (label))
#define VarGLow(variable) ((variable)->_low)
#define VarSLow(variable,low) ((variable)->_low = (low))
#define VarGHigh(variable) ((variable)->_high)
#define VarSHigh(variable,high) ((variable)->_high = (high))
#define VarGAuxLow(variable) ((variable)->_auxLow)
#define VarSAuxLow(variable,low) ((variable)->_auxLow = (low))
#define VarGAuxHigh(variable) ((variable)->_auxHigh)
#define VarSAuxHigh(variable,high) ((variable)->_auxHigh = (high))
#define VarUnified(variable) ((variable)->_flag & 1)
#define VarSet(variable) ((variable)->_flag |= 1)
#define VarReset(variable) ((variable)->_flag &= ~1)
#define VarHard(variable) ((variable)->_flag & 2)
#define VarSHard(variable) ((variable)->_flag |= 2)
#define VarCHard(variable) ((variable)->_flag &= ~2)
#define VarIsIn(variable) ((variable)->_flag & 4)
#define VarSIsIn(variable) ((variable)->_flag |= 4, (variable)->_flag &= ~8)
#define VarIsOut(variable) ((variable)->_flag & 8)
#define VarSIsOut(variable) ((variable)->_flag |= 8, (variable)->_flag &= ~4)
#define VarIsFold(variable) ((variable)->_flag & 16)
#define VarSIsFold(variable) ((variable)->_flag |= 16)
#define VarIsGat(variable) ((variable)->_flag & 32)
#define VarSIsGat(variable) ((variable)->_flag |= 32)
#define VarIsCoeff(variable) ((variable)->_flag & 64)
#define VarSIsCoeff(variable) ((variable)->_flag |= 64)
#define VarBad(variable) ((variable)->_flag & 128)
#define VarSBad(variable) ((variable)->_flag |= 128)
#define VarCBad(variable) ((variable)->_flag &= ~128)
#define VarUgly(variable) ((variable)->_flag & 256)
#define VarSUgly(variable) ((variable)->_flag |= 256)
#define VarCUgly(variable) ((variable)->_flag &= ~256)
#define VarEscape(variable) ((variable)->_flag & 512)
#define VarSEscape(variable) ((variable)->_flag |= 512)
#define VarCEscape(variable) ((variable)->_flag &= ~512)
#define VarIsLet(variable) ((variable)->_flag & 1024)
#define VarSIsLet(variable) ((variable)->_flag |= 1024)
#define VarGCoeffs(variable) ((variable)->_coeffs)
#define VarSCoeffs(variable) ((variable)->_coeffs = LstCreate(&symTools))
#define VarAlloc() (VARIABLE *) Allocate(sizeof(VARIABLE))
extern TOOLS varTools;

#define TraGName(transition) ((transition)->_name)
#define TraGKind(transition) ((transition)->_kind)
#define TraSKind(transition,kind) ((transition)->_kind = (kind))
#define TraGFdWidth(transition) ((transition)->_fdWidth)
#define TraIFdWidth(transition) (((transition)->_fdWidth)++)
#define TraGInputs(transition) ((transition)->_inputs)
#define TraGOutputs(transition) ((transition)->_outputs)
#define TraGArcs(transition) ((transition)->_arcs)
#define TraGVariables(transition) ((transition)->_vars)
#define TraGComputation(transition) ((transition)->_computation)
#define TraSComputation(transition) \
((transition)->_computation = LstCreate(&strTools))
#define TraDComputation(transition) \
(LstDestroy((transition)->_computation, TRUE), (transition)->_computation = NULL)
#define TraGGates(transition) ((transition)->_gates)
#define TraGRange(transition) ((transition)->_range)
#define TraGBlocks(transition) ((transition)->_blocks)
#define TraAlloc() (TRANSITION *) Allocate(sizeof(TRANSITION))
extern TOOLS traTools;

#define SytGArity(symmultup) ((symmultup)->_arity)
#define SytSArity(symmultup,arity) ((symmultup)->_arity = (arity))
#define SytGSymbol(symmultup,pos) ((symmultup)->_symtup[pos])
#define SytRSymbol(symmultup,pos,symbol) ((symmultup)->_symtup[pos] = (symbol))
#define SytGCffSym(symmultup) ((symmultup)->_cffSym)
#define SytSCffSym(symmultup,cffSym) ((symmultup)->_cffSym = (cffSym))
#define SytGMulti(symmultup) ((symmultup)->_multi)
#define SytSMulti(symmultup,multi) ((symmultup)->_multi = (multi))
#define SytIMulti(symmultup,multi) ((symmultup)->_multi += (multi))
#define SytGLabel(symmultup) ((symmultup)->_label)
#define SytSLabel(symmultup,label) ((symmultup)->_label = (label))
#define SytGNumber(symmultup) ((symmultup)->_number)
#define SytSNumber(symmultup,number) ((symmultup)->_number = (number))
#define SytAlloc() (SYMMULTUP *) Allocate(sizeof(SYMMULTUP))
extern TOOLS sytTools;

#define SymMarked(symbol) ((symbol)->_flag & 1)
#define SymSet(symbol) ((symbol)->_flag |= 1)
#define SymReset(symbol) ((symbol)->_flag &= ~1)
#define SymIsConstant(symbol) ((symbol)->_flag & 2)
#define SymSIsConstant(symbol) ((symbol)->_flag |= 2)
#define SymIsVariable(symbol) ((symbol)->_flag & 4)
#define SymSIsVariable(symbol) ((symbol)->_flag |= 4)
#define SymIsOperation(symbol) ((symbol)->_flag & 8)
#define SymSIsOperation(symbol) ((symbol)->_flag |= 8)
#define SymGConstant(symbol) ((symbol)->_uy._constant)
#define SymGVariable(symbol) ((symbol)->_uy._var)
#define SymGOperation(symbol) ((symbol)->_uy._operation)
#define SymAlloc() (SYMBOL *) Allocate(sizeof(SYMBOL))
extern TOOLS symTools;

#define ArcGPlace(arc) ((arc)->_place)
#define ArcGSymsum(arc) ((arc)->_symsum)
#define ArcIPlace(arc,place) ((arc)->_place = (place))
#define ArcGNumber(arc) ((arc)->_number)
#define ArcSNumber(arc,number) ((arc)->_number = (number))
#define ArcFirst(arc) ((arc)->_flag & 1)
#define ArcSFirst(arc) ((arc)->_flag |= 1)
#define ArcLast(arc) ((arc)->_flag & 2)
#define ArcSLast(arc) ((arc)->_flag |= 2)
#define ArcAlloc() (ARC *) Allocate(sizeof(ARC))
extern TOOLS arcTools;

#define PreAlloc() (PRECEDENCE *) Allocate(sizeof(PRECEDENCE))
extern TOOLS preTools;
#define PreGValue(precedence) ((precedence)->_value)
#define PreMarked(precedence) ((precedence)->_flag & 1)
#define PreGSelfLoop(precedence) ((precedence)->_flag & 2)
#define PreGEasy(precedence) ((precedence)->_flag & 4)
#define PreGSucc(precedence) ((precedence)->_succ)
#define PreGPred(precedence) ((precedence)->_pred)
#define PreSValue(precedence,value) ((precedence)->_value = (value))
#define PreSet(precedence) ((precedence)->_flag |= 1)
#define PreReset(precedence) ((precedence)->_flag &= ~1)
#define PreSSelfLoop(precedence) ((precedence)->_flag |= 2)
#define PreSEasy(precedence) ((precedence)->_flag |= 4)
#define PreSSucc(precedence) ((precedence)->_succ = LstCreate(&preTools))
#define PreSPred(precedence) ((precedence)->_pred = LstCreate(&preTools))

DE2(void, Generate,
    FILE *, genrF,
    NET *, genrNet)
DE6(NET *, NetCons,
    LIST *, places,
    LIST *, transes,
    LIST *, precs,
    unsigned, nofClasses,
    PLACE *, monPl,
    LIST *, atoms)
DE1(void, NetDiscard,
    NET *, net)
#ifdef DEBUG
DE2(void, NetDump,
    NET *, net,
    unsigned, depth)
#endif
DE4(PLACE *, PlaCons,
    char *, name,
    LIST *, low,
    LIST *, high,
    MARKING *, marking)
DE1(PLACE *, PlaCopy,
    PLACE *, place)
DE1(void, PlaDiscard,
    PLACE *, place)
#ifdef DEBUG
DE2(void, PlaDump,
    PLACE *, place,
    unsigned, depth)
#endif
DE1(VARIABLE *, VarCons,
    char *, name)
DE1(VARIABLE *, VarCopy,
    VARIABLE *, variable)
DE1(void, VarDiscard,
    VARIABLE *, variable)
#ifdef DEBUG
DE2(void, VarDump,
    VARIABLE *, variable,
    unsigned, depth)
#endif
DE1(TRANSITION *, TraCons,
    char *, name)
DE1(void, TraDiscard,
    TRANSITION *, transition)
#ifdef DEBUG
DE2(void, TraDump,
    TRANSITION *, transition,
    unsigned, depth)
#endif
DE3(ARC *, TraAttInput,
    TRANSITION *, transition,
    PLACE *, place,
    LIST *, symsum)
DE3(ARC *, TraAttOutput,
    TRANSITION *, transition,
    PLACE *, place,
    LIST *, symsum)
DE1(SYMMULTUP *, SytCons,
    unsigned, arity)
DE1(SYMMULTUP *, SytCopy,
    SYMMULTUP *, symmultup)
DE1(void, SytDiscard,
    SYMMULTUP *, symmultup)
#ifdef DEBUG
DE2(void, SytDump,
    SYMMULTUP *, symmultup,
    unsigned, depth)
#endif
DE2(SYMBOL *, SymCons,
    unsigned, kind,
    char *, any)
DE1(SYMBOL *, SymCopy,
    SYMBOL *, symbol)
DE1(void, SymDiscard,
    SYMBOL *, symbol)
#ifdef DEBUG
DE2(void, SymDump,
    SYMBOL *, symbol,
    unsigned, depth)
#endif
DE1(BOOL, SymSatisfied,
    SYMBOL *, symbol)
DE2(void, PrintSymbol,
    FILE *, f,
    SYMBOL *, symbol)
DE2(ARC *, ArcCons,
    PLACE *, place,
    LIST *, symsum)
DE1(ARC *, ArcCopy,
    ARC *, arc)
DE1(void, ArcDiscard,
    ARC *, arc)
#ifdef DEBUG
DE2(void, ArcDump,
    ARC *, arc,
    unsigned, depth)
#endif
DE1(PRECEDENCE *, PreCons,
    unsigned, value)
DE1(PRECEDENCE *, PreCopy,
    PRECEDENCE *, precedence)
DE1(void, PreDiscard,
    PRECEDENCE *, precedence)
#ifdef DEBUG
DE2(void, PreDump,
    PRECEDENCE *, precedence,
    unsigned, depth)
#endif
DE1(char *, DomStr,
    DOMAIN, value)
DE1(char *, UnsStr,
    unsigned, value)
DE1(char *, UlongStr,
    unsigned long, value)
