/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  output.c
*
* Part of: PROD's program prpp
*
* Authors: Heikki Suonsivu and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     This module passes plain C code from the net description
*     to the target ".c" file.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include <string.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ppio.h"
#include "token.h"
#include "util.h"
#include "prpp.h"
#include "output.h"

#define TAG_MINIMUM 40

static PPIOFILE *output_file = NULL;

/***************************************************************************
*
* Function:  check_output_file
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FS0(void, check_output_file)
{
    if (!output_file)
	output_file = (PPIOFILE *) Allocate(sizeof(PPIOFILE));
    output_file->fp = NULL;
    output_file->name = NULL;
    output_file->line = 0;
    output_file->newlines_pushed = 0;
}

/***************************************************************************
*
* Function:  insert_line_tag
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, insert_line_tag)
{
    register char *s;

    if (line_directives)
	{
            (void) fprintf(prpOutput, "#line %lu \"",
			   curFile->line + 1
			   - curFile->newlines_pushed);
	    for (s = curFile->name; *s; s++)
		{
		    (void) fputc((int) *s, prpOutput);
		    if (*s == '\\')
			(void) fputc((int) *s, prpOutput);
		}
	    (void) fprintf(prpOutput, "\"\n");
	    nOutpLines++;
	}
    check_output_file();
    output_file->name = curFile->name;
    output_file->line = curFile->line - curFile->newlines_pushed;
}

/***************************************************************************
*
* Function:  outputc
* Purpose:
* Arguments: c - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, outputc,
    int, c)
{
    register BOOL do_tag = FALSE;
    static BOOL was_nl = TRUE;

    if (c == (int) '\n')
	{
	    check_output_file();
	    output_file->line++;
	}
    if (was_nl)
	{
	    if (may_have_lost_line)
		{
		    check_output_file();
		    if ((!output_file->name)
			|| strcmp(output_file->name, curFile->name))
			do_tag = TRUE;
		    else
			{
			    if (output_file->line
				< curFile->line
				- curFile->newlines_pushed)
				{
				    register unsigned long distance;
				    
				    if (curFile->newlines_pushed +
					output_file->line
					< curFile->line)
					distance = curFile->line -
					    curFile->newlines_pushed
						- output_file->line;
				    else
					distance = (unsigned long) 0L;
                                    if (distance > (unsigned long)
                                        (strlen(curFile->name) + TAG_MINIMUM))
					do_tag = TRUE;
				    else
					{
					    while (((unsigned long) 0L)
						   != distance)
						{
						    (void) fputc((int) '\n',
								 prpOutput);
						    nOutpLines++;
						    distance--;
						}
					}
				}
			    else if (output_file->line
				     > curFile->line
				     - curFile->newlines_pushed)
				do_tag = TRUE;
			}
		    if (do_tag)
			insert_line_tag();
		    else
			{
			    output_file->line = curFile->line
				- curFile->newlines_pushed ;
			}
#ifdef NOT_DEBUGGING_THIS
		    may_have_lost_line = FALSE; 
#endif
		} 
	} 
    (void) fputc((int) c, prpOutput);
    if (c == (int) '\n')
	nOutpLines++;
    was_nl = (c == (int) '\n');
}

/***************************************************************************
*
* Function:  output_token
* Purpose:
* Arguments: token - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, output_token,
    PP_TOKEN *, token)
{
    char *p;
  
    if (token)
	{
	    if (may_have_lost_line && (token->type == COMMENT))
		{
		    may_have_lost_line = FALSE;
		    for (p = token->token; *p; p++)
			outputc(*p);
		    may_have_lost_line = TRUE;
		}
	    else
		{
		    for (p = token->token; *p; p++)
			outputc(*p);
		}
	}
}
