/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  parse.c
*
* Part of: PROD's program prpp
*
* Authors: Heikki Suonsivu and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     Most of the parsing of the net description is done by the
*     routines of this module.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include <string.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ppio.h"
#include "token.h"
#include "lexer.h"
#include "parse.h"
#include "macro.h"
#include "util.h"
#include "ptrbl.h"
#include "baltree.h"
#include "gphutil.h"
#include "prpp.h"
#include "output.h"
#include "dynamic.h"
#include "place.h"
#include "mark.h"
#include "expr.h"
#include "net.h"
#include "prec.h"
#include "pplst.h"
#include "prpexpr.h"
#include "trans.h"

LIST *ppLsLs;

#define NEXT_PP_TOKEN(token, line) \
if (!(token = (PP_TOKEN *) LstThis(line))) ; else (void) LstStep(line)

#define SKIP_PP_TOKEN_AND_SPACE(token, line) \
do { NEXT_PP_TOKEN(token, line); } \
while (token && ((token->type == SPACE) || (token->type == COMMENT)))

#define SKIP_SPACE(token, line) \
for (token = (PP_TOKEN *) LstBegin(line); \
     token && ((token->type == SPACE) || (token->type == COMMENT));) \
     NEXT_PP_TOKEN(token, line)

static BOOL skipping = FALSE;
static int iflevel;

/***************************************************************************
*
* Function:  macro_definition_parse
* Purpose:
* Arguments: line - 
* Returns:
* See also:
*
****************************************************************************
*/
FS1(void, macro_definition_parse,
    LIST *, line)
{
    char *name;
    PP_TOKEN *token;
    LIST *arguments;
    VOID *cursor;
    char *str;

    arguments = NULL;
    SKIP_SPACE(token, line);
    if (!token)
	{
	    error("#define: expected name, got nothing");
	    return;
	}
    if (token->type != NAME)
	{
	    (void) sprintf(errStr, 
			   "#define: expected name, got \"%s\"", token->token);
	    error(errStr);
	    return;
	}
    name = token->token;
    if (ismacro(name))
	{
	    (void) sprintf(errStr, "%s already defined", name);
	    error(errStr);
	    return;
	}
    NEXT_PP_TOKEN(token, line);
    if (token)
	{
	    if (token->type == OPENING_PARENTHESIS)
		{
		    arguments = LstCreate(&strTools);
		    SKIP_PP_TOKEN_AND_SPACE(token, line);
		    while (token && token->type != CLOSING_PARENTHESIS)
			{
			    if (token->type != NAME)
				{
				    (void) sprintf(errStr,
						   "Definition of %s: parameter %s is no name",
						   name, token->token);
				    error(errStr);
				    LstDestroy(arguments, FALSE);
				    return;
				}
			    LSTFOR (arguments, str, char)
				if (!strcmp(str, token->token))
				    break;
			    if (str)
				{
				    (void) sprintf(errStr,
						   "Definition of %s: parameter %s is ambiguous",
						   name, token->token);
				    error(errStr);
				    LstDestroy(arguments, FALSE);
				    return;
				}
			    (void) LstInsLast(arguments, (char *)
					      (token->token));
			    SKIP_PP_TOKEN_AND_SPACE(token, line);
			    if (!token)
				{
				    error("Macro argument list: Premature eof.");
				    LstDestroy(arguments, FALSE);
				    return;
				}
			    if (token->type != COMMA)
				{
				    if (token->type == NAME)
					{
					    (void) sprintf(errStr,
							   "Macro argument list: Missing comma %s %s",
							   "inserted before",
							   token->token);
					    warning(errStr);
					}
				    else if (token->type == CLOSING_PARENTHESIS)
					break;
				    else
					{
					    (void) sprintf(errStr,
							   "Macro argument list: Expected \",\", got \"%s\"",
							   token->token);
					    error(errStr);
					    LstDestroy(arguments, FALSE);
					    return;
					}
				}
			    else
				SKIP_PP_TOKEN_AND_SPACE(token, line);
			} 
		    NEXT_PP_TOKEN(token, line);
		} 
	    else if ((token->type == SPACE) || (token->type == COMMENT))
		{
		    register char *source, *destination;
		    
		    if (*token->token)
			{
			    source = token->token + 1;
			    destination = token->token;
			    while ((*destination++ = *source++))
				;
			}
		}
	    else
		{
		    (void) sprintf(errStr,
				   "Expected space or comment or opening parenthesis to follow, got\n\"%s\"",
				   token ? token->token : "nothing");
		    error(errStr);
		    return;
		}
	} 
    cursor = LstGet(line);
    if (memorize_macro(name, arguments, line))
	{
	    LstSet(line, cursor);
	    (void) fprintf(defsfile, "#define %s", name);
	    if (!arguments)
		(void) fprintf(defsfile, " ");
	    else if (LstEmpty(arguments))
		(void) fprintf(defsfile, "()");
	    else
		{
		    (void) fprintf(defsfile, "(%s",
				   (char *) LstBegin(arguments));
		    while ((str = (char *) LstStep(arguments)))
			(void) fprintf(defsfile, ",%s", str);
		    (void) fprintf(defsfile, ")");
		}
	    while (TRUE)
		{
		    NEXT_PP_TOKEN(token, line);
		    if ((!token) || (token->type == NEWLINE))
			break;
		    (void) fprintf(defsfile, "%s", token->token);
		}
	    (void) fprintf(defsfile, "\n");
	}
    if (arguments)
	LstDestroy(arguments, FALSE);
}

/***************************************************************************
*
* Function:  macro_undefine_parse
* Purpose:
* Arguments: name - 
* Returns:
* See also:
*
****************************************************************************
*/
FS1(void, macro_undefine_parse,
    LIST *, name)
{
    PP_TOKEN *token;
    MACRO *macrop;

    SKIP_SPACE(token, name);
    if (!token)
	return;
    if (token->type != NAME)
	{
	    (void) sprintf(errStr, "#undef: expected name, got \"%s\"", token->token);
	    error(errStr);
	    return;
	}
    macrop = ismacro(token->token);
    if (!macrop)
	{
	    (void) sprintf(errStr, "%s: no such definition", token->token);
	    error(errStr);
	}
    else if (mc_locked(macrop))
	{
	    (void) sprintf(errStr,
			   "%s is under expansion\nand thus cannot be undefined",
			   token->token);
	    error(errStr);
	}
    else
	{
	    undefine_macro(macrop);
	    (void) fprintf(defsfile, "#undef %s\n", token->token);
	}
    for (;;)
	{
	    NEXT_PP_TOKEN(token, name);
	    if (!token)
		break;

	    if (token->type == NEWLINE)
		break;

	    if ((token->type != SPACE) && (token->type != COMMENT))
		{
		    (void) sprintf(errStr, "#undef: Ignoring %s",
				   token->token);
		    warning(errStr);
		}
	}
}

/***************************************************************************
*
* Function:  IsEnum
* Purpose:
* Arguments: name - 
*            enumVal - 
* Returns:
* See also:
*
****************************************************************************
*/
FS2(BOOL, IsEnum,
    char *, name,
    DOMAIN *, enumVal)
{
    REF *ref;

    ref = (REF *) BalTrElement(enumTree, (char *) name);
    if (!ref)
	return(FALSE);
    *enumVal = (DOMAIN) RefGSlot(ref);
    return(TRUE);
}

static DOMAIN minEnum = (DOMAIN) MAXDOMAIN;
static int enumOverflow = 0;

/***************************************************************************
*
* Function:  enum_parse
* Purpose:
* Arguments: line - 
* Returns:
* See also:
*
****************************************************************************
*/
FS1(void, enum_parse,
    LIST *, line)
{
    PP_TOKEN *token;
    DOMAIN enumVal;
    UDOMAIN uval;
    REF *ref;

    SKIP_SPACE(token, line);
    while (TRUE)
	{
	    if ((!token) || (token->type != NAME))
		{
		    (void) sprintf(errStr, "Expected name, got \"%s\".",
				   token ? token->token : "nothing");
		    error(errStr);
		    return;
		}
	    if (enumOverflow == 2)
		;
	    else if (IsEnum(token->token, &enumVal))
		{
		    (void) sprintf(errStr,
				   "#enum: %s redefined", token->token);
		    error(errStr);
		}
	    else if (enumOverflow == 1)
		{
		    error("#enum: overflow, too many");
		    enumOverflow = 2;
		}
	    else
		{
		    uval = (UDOMAIN) minEnum;
		    (void) sprintf(numStr,
				   (uval > (UDOMAIN) ~uval)
				   ? DOMXFORMAT : UDOMFORMAT, uval);
		    (void) fprintf(enmFile, "#define %s %s\n",
				   token->token, numStr);
		    (void) LstInsLast(EnumLst, (char *) StrCopy(token->token));
		    ref = RefCons((char *) LstLast(EnumLst),
				  (unsigned long) minEnum);
		    (void) BalTrInsert(enumTree, (char *) ref,
				       (char *) RefGKey(ref), TRUE);
		    if (minEnum == (DOMAIN) MINDOMAIN)
			enumOverflow = 1;
		    else
			minEnum--;
		}
	    SKIP_PP_TOKEN_AND_SPACE(token, line);
	    if (!token)
		break;
	    if (token->type != COMMA)
		{
		    (void) sprintf(errStr, "Expected \",\", got \"%s\".",
				   token->token);
		    error(errStr);
		    return;
		}
	    SKIP_PP_TOKEN_AND_SPACE(token, line);
	}
}

#ifdef IF_TRUE
#undef IF_TRUE
#endif
#define IF_TRUE (invert ? TRUE : FALSE)
#ifdef IF_FALSE
#undef IF_FALSE
#endif
#define IF_FALSE (invert ? FALSE : TRUE) 
/***************************************************************************
*
* Function:  prsIfDefCondition
* Purpose:
* Arguments: line - 
*            type - 
* Returns:
* See also:
*
****************************************************************************
*/
FS2(BOOL, prsIfDefCondition,
    LIST *, line,
    int, type)
{
    register PP_TOKEN *name;
    register BOOL invert;

    invert = (type == DV_IFNDEF);
    SKIP_SPACE(name, line);
    if (!name)
	return(IF_FALSE);
    if (name->type != NAME)
	{
	    (void) sprintf(errStr,
			   "#ifdef: Expected name, got \"%s\"", name->token);
	    error(errStr);
	    return(IF_FALSE);
	}
    return(ismacro(name->token) ? IF_TRUE : IF_FALSE);
}

/***************************************************************************
*
* Function:  macro_expansion_parse
* Purpose:
* Arguments: macrop - 
*            strict - 
* Returns:
* See also:
*
****************************************************************************
*/
FS2(void, macro_expansion_parse,
    MACRO *, macrop,
    BOOL, strict)
{
    PP_TOKEN *token;
    LIST **arguments, *list = NULL;
    int count, i;

    may_have_lost_line = TRUE;

    if (macrop->args == -1)
	{
	    expand_macro(macrop, (LIST **) NULL);
	    return;
	}
    token = gTSkEolAndSpace();
    if (!token)
	{
	    (void) sprintf(errStr,
			   "macro %s:\nend of file when expecting \"(\"",
			   macrop->name);
	    panic(errStr);
	}
    if (token->type != OPENING_PARENTHESIS)
	{
	    (void) sprintf(errStr, "macro %s:\nexpected \"(\", got \"%s\"",
			   macrop->name, token->token);
	    error(errStr);
	    PpTokDiscard(token);
	    return;
	}
    PpTokDiscard(token);
    if (macrop->args)
	{
	    arguments = (LIST **)
		AllocArray((unsigned) macrop->args, sizeof(LIST *));
	    for (i = 0; i < macrop->args; i++)
		arguments[i] = NULL;
	}
    else
	arguments = NULL;
    count = 0;
    mc_inclock(macrop);
    while (TRUE)
	{
	    parsing_control++;
	    list = LstCreate(&ppTokTools);
	    (void) LstInsFirst(ppLsLs, (char *) list);
	    token = cons_parse(CLOSING_PARENTHESIS + 1 + ((COMMA + 1) << 8),
			       strict);
	    (void) LstDelFirst(ppLsLs, FALSE);
	    parsing_control--;
	    if (!token)
		{
		    (void) sprintf(errStr,
				   "macro %s:\nend of file when expecting \"%s\"",
				   macrop->name,
				   ((macrop->args > 1) &&
				    (count + 1 < macrop->args))
				   ? "," : ")");
		    panic(errStr);
		}
	    else if (token->type == CLOSING_PARENTHESIS)
		{
		    PpTokDiscard(token);
		    break;
		}
	    else if (token->type == COMMA)
		{
		    PpTokDiscard(token);
		    count++;
		    if (count < macrop->args)
			arguments[count - 1] = list;
		    else
			{
			    LstDestroy(list, TRUE);
			    if (arguments)
				{
				    for (i = 0;
					 (i < macrop->args) &&
					 arguments[i];
					 i++)
					LstDestroy(arguments[i], TRUE);
				    Deallocate((char *) arguments);
				    arguments = NULL;
				}
			}
		}
	    else
		{
		    (void) sprintf(errStr,
				   "macro %s:\nexpected \"%s\", got \"%s\"",
				   macrop->name,
				   ((macrop->args > 1) &&
				    (count + 1 < macrop->args))
				   ? "," : ")",
				   token->token);
		    error(errStr);
		    PpTokDiscard(token);
		    if (arguments)
			{
			    for (i = 0;
				 (i < macrop->args) && arguments[i];
				 i++)
				LstDestroy(arguments[i], TRUE);
			    Deallocate((char *) arguments);
			}
		    mc_declock(macrop);
		    return;
		}
	}
    if (count)
	count++;
    else
	{
	    LSTFOR (list, token, PP_TOKEN)
		{
		    if ((token->type != SPACE) && (token->type != NEWLINE))
			{
			    count = 1;
			    break;
			}
		}
	}
    if (count != macrop->args)
	{
	    (void) sprintf(errStr, "macro %s:\nexpected %d arguments, got %d",
			   macrop->name, macrop->args, count);
	    error(errStr);
	    LstDestroy(list, TRUE);
	}
    else if (macrop->args)
	{
	    if (arguments)
		{
		    arguments[count - 1] = list;
		    expand_macro(macrop, arguments);
		}
	    else
		LstDestroy(list, TRUE);
	}
    else
	{
	    LstDestroy(list, TRUE);
	    expand_macro(macrop, (LIST **) NULL);
	}
    if (arguments)
	{
	    for (i = 0; (i < macrop->args) && arguments[i]; i++)
		LstDestroy(arguments[i], TRUE);
	    Deallocate((char *) arguments);
	}
    mc_declock(macrop);
}

#ifdef IF_TRUE
#undef IF_TRUE
#endif
#define IF_TRUE FALSE
#ifdef IF_FALSE
#undef IF_FALSE
#endif
#define IF_FALSE TRUE
/***************************************************************************
*
* Function:  prsIfCondition
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FS0(BOOL, prsIfCondition)
{
    PP_TOKEN *token;
    LIST *list;
    DOMAIN value;

    value = MINDOMAIN;
    list = LstCreate(&ppTokTools);
    (void) LstInsFirst(ppLsLs, (char *) list);
    token = app_parse(NEWLINE, FALSE); 
    (void) LstDelFirst(ppLsLs, FALSE);
    TOKEN_MATCH(token, NEWLINE, "End of line in #if");
    unget_token(token);
    PpTokDiscard(token);
    PrpExpr((MARKING **) NULL, (char *) NULL, (unsigned) 0, list, &value);
    if (errorFlag != NO_ERROR)
	return(IF_FALSE);
    return(value ? IF_TRUE : IF_FALSE);
 recover: ;
    if (token)
	PpTokDiscard(token);
    LstDestroy(list, TRUE);
    return(IF_FALSE);
}

/***************************************************************************
*
* Function:  parse_include
* Purpose:
* Arguments: file_specification - 
* Returns:
* See also:
*
****************************************************************************
*/
FS1(void, parse_include,
    LIST *, file_specification)
{
    PP_TOKEN *token;
    char *filename = NULL, *pointer;
    BOOL angle_brackets = FALSE;
    BUFFER_DEFINITION(static);

    INITIALIZE_BUFFER(32);
    SKIP_SPACE(token, file_specification);
    if (!token)
	{
	    error("#include requires a file.");
	    return;
	}
    if (token->type == LESS_THAN)
	{
	    angle_brackets = TRUE;

	    for (;;)
		{
		    NEXT_PP_TOKEN(token, file_specification);
		    if (!token)
			{
			    error("#include: unexpected end-of-file.");
			    return;
			}

		    if (token->type == MORE_THAN)
			break; 

		    for (pointer = token->token; *pointer; pointer++)
			{
			    ADD_BUFFER(*pointer);
			}
		}

	    ADD_BUFFER('\0');
	    filename = buffer;
	}
    else if (token->type == QUOTED_EXPRESSION)
	{

	    filename = token->token + 1;
	    if (*filename)
		filename[strlen(filename) - 1] = '\0';
	}
    else
	{
	    (void) sprintf(errStr,
			   "#include: Expected <file> or \"file\", got \"%s\"", 
			   token ? token->token : "nothing");
	    error(errStr);
	    return;
	}
    ppio_push_file(filename, angle_brackets);
    insert_line_tag();
}

/***************************************************************************
*
* Function:  read_line
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FS0(LIST *, read_line)
{
    PP_TOKEN *token;
    LIST *list;

    list = LstCreate(&ppTokTools);
    (void) LstInsFirst(ppLsLs, (char *) list);
    token = app_parse(NEWLINE, FALSE);
    (void) LstDelFirst(ppLsLs, FALSE);
    if (!token)
	warning("Preprocessor statement without newline.");
    else
	{
	    if (NEWLINE == token->type)
		unget_token(token);
	    else
		error("Possible internal problem: read_line terminates, but not in newline.");
	    PpTokDiscard(token);
	}
    return(list);
}

/***************************************************************************
*
* Function:  raw_read_line
* Purpose:
* Arguments: inclNewLine
* Returns:
* See also:
*
****************************************************************************
*/
FS1(LIST *, raw_read_line,
    BOOL, inclNewLine)
{
    PP_TOKEN *token;
    LIST *list;

    list = LstCreate(&ppTokTools);
    while ((token = get_token()) && (token->type != NEWLINE))
	(void) LstInsLast(list, (char *) token);
    if (token)
	{
	    if (inclNewLine)
		(void) LstInsLast(list, (char *) token);
	    else
		{
		    unget_token(token); 
		    PpTokDiscard(token);
		}
	}
    return(list);
}

typedef struct STACK_LEVEL
{
    char *filename;
    char *declarator;
    unsigned long line;
} STACK_LEVEL;

static LIST *if_stack = NULL;

/***************************************************************************
*
* Function:  push_stack
* Purpose:
* Arguments: declarator - 
* Returns:
* See also:
*
****************************************************************************
*/
FS1(void, push_stack,
    char *, declarator)
{
    STACK_LEVEL *stack_level;

    stack_level = (STACK_LEVEL *) Allocate(sizeof(*stack_level));
    stack_level->filename = StrCopy(curFile->name);
    stack_level->declarator = StrCopy(declarator);
    stack_level->line = curFile->line - curFile->newlines_pushed;
    if (!if_stack)
	if_stack = LstCreate((TOOLS *) NULL);
    (void) LstInsFirst(if_stack, (char *) stack_level);
}

/***************************************************************************
*
* Function:  pop_stack
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FS0(void, pop_stack)
{
    STACK_LEVEL *stack_level;

    stack_level = (STACK_LEVEL *) LstDelFirst(if_stack, FALSE);
    Deallocate((char *) stack_level->filename);
    Deallocate((char *) stack_level->declarator);
    Deallocate((char *) stack_level);
}

/***************************************************************************
*
* Function:  check_stack
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FS0(void, check_stack)
{
    STACK_LEVEL *stack_level;

    if (if_stack)
	LSTFOR (if_stack, stack_level, STACK_LEVEL)
	    {
                (void) sprintf(errStr, "Unpaired %s starts in %s at line %lu",
			       stack_level->declarator,
			       stack_level->filename, stack_level->line + 1);
		error(errStr);
	    }
}

/***************************************************************************
*
* Function:  do_skip
* Purpose:
* Arguments: skip_everything - 
* Returns:
* See also:
*
****************************************************************************
*/
FS1(void, do_skip,
    BOOL, skip_everything)
{
    register PP_TOKEN *token;
    register LIST *token_line;
    register unsigned long line;
    register int dv;
    register BOOL local_skipping;
    register char *filename = NULL;

    local_skipping = TRUE;
    if (curFile)
	{
	    filename = StrCopy(curFile->name);
	    line = curFile->line;
	}
    else
	{
	    filename = StrCopy("<Unknown?>");
            line = 0L;
	}
    (void) sprintf(errStr,
		   "Start skip %s.",
		   skip_everything ? "everything" : "until control");
    while (local_skipping)
	{
	    token_line = raw_read_line(TRUE);
	    (void) LstBegin(token_line);
	    NEXT_PP_TOKEN(token, token_line);
	    if (!token)
		{
		    (void) sprintf(errStr,
                                   "Endless #if, starts in %s at line %lu",
				   filename, line + 1);
		    error(errStr);
		    iflevel--;
		    pop_stack(); 
		    LstDestroy(token_line, TRUE);
		    break;
		}
	    if (token->type != DV)
		{
		    LstDestroy(token_line, TRUE);
		    continue;
		}
	    SKIP_PP_TOKEN_AND_SPACE(token, token_line);
	    if ((!token) || (token->type == NEWLINE))
		{
		    LstDestroy(token_line, TRUE);
		    continue;
		}
	    dv = isdirective(token->token);
	    if ((dv == DV_ELSE) || (dv == DV_ENDIF))
		{
		    SKIP_PP_TOKEN_AND_SPACE(token, token_line);
		    if (token && (token->type != NEWLINE))
			{
			    (void) sprintf(errStr, 
					   "Ignoring rest of line while looking at token \"%s\"", 
					   token->token);
			    warning(errStr);
			}
		}
	    LstDestroy(token_line, TRUE);
	    switch (dv)
		{
		case DV_IFDEF:
		case DV_IFNDEF:
		case DV_IF:
		    iflevel++;
		    push_stack((dv == DV_IFDEF) ? "ifdef"
			       : ((dv == DV_IFNDEF) ? "ifndef" : "if"));
		    do_skip(TRUE);
		    break;
		case DV_ELSE:
		    if (!skip_everything)
			{
			    local_skipping = FALSE;
			    skipping = FALSE;
			    Deallocate(filename);
			    return;
			}
		    break;
		case DV_ENDIF:
		    if (iflevel)
			{
			    iflevel--;
			    pop_stack();
			}
		    else
			error("Internal: Orphan #endif");
		    local_skipping = FALSE;
		    skipping = FALSE;
		    Deallocate(filename);
		    return;
		default:
		    break;
		}
	}
    Deallocate(filename);
}

static BOOL inTrans = FALSE;

/***************************************************************************
*
* Function:  control_parse
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FS0(void, control_parse)
{
    register PP_TOKEN *token;
    register LIST *ctrlLine;
    register int dv;

    parsing_control++;
    may_have_lost_line = TRUE;
    while ((token = get_token()))
	{
	    if ((token->type != SPACE) && (token->type != COMMENT))
		break;
	    PpTokDiscard(token);
	}
    if ((!token) || (token->type == NEWLINE))
	{
	    parsing_control--;
	    if (token)
		PpTokDiscard(token);
	    return;
	}
    dv = isdirective(token->token);
    switch (dv)
	{
	case DV_DEFINE:
	    ctrlLine = raw_read_line(FALSE);
	    macro_definition_parse(ctrlLine);
	    LstDestroy(ctrlLine, TRUE);
	    break;
	case DV_UNDEF:
	    ctrlLine = raw_read_line(FALSE);
	    macro_undefine_parse(ctrlLine);
	    LstDestroy(ctrlLine, TRUE);
	    break;
	case DV_IFDEF:
	case DV_IFNDEF:
	    ctrlLine = raw_read_line(FALSE);
	    iflevel++;
	    push_stack((dv == DV_IFDEF) ? "ifdef" : "ifndef");
	    skipping = prsIfDefCondition(ctrlLine, dv);
	    LstDestroy(ctrlLine, TRUE);
	    break;
	case DV_IF:
	    iflevel++;
	    push_stack("if");
	    skipping = prsIfCondition();
	    break;
	case DV_ELSE:
	    if (!iflevel)
		error("Orphan #else?");
	    else
		skipping = !skipping;
	    ctrlLine = raw_read_line(FALSE);
	    LstDestroy(ctrlLine, TRUE);
	    break;
	case DV_ENDIF:
	    if (!iflevel)
		error("Orphan #endif?");
	    else
		{
		    iflevel--;
		    pop_stack();
		}
	    ctrlLine = raw_read_line(FALSE);
	    LstDestroy(ctrlLine, TRUE);
	    break;
	case DV_INCLUDE:
	    ctrlLine = read_line();
	    parse_include(ctrlLine);
	    LstDestroy(ctrlLine, TRUE);
	    break;
	case DV_ENUM:
	    ctrlLine = raw_read_line(FALSE);
	    enum_parse(ctrlLine);
	    LstDestroy(ctrlLine, TRUE);
	    break;
	case DV_PLACE:
	    pla();
	    break;
	case DV_TRANS:
	    if (inTrans)
		{
		    error("#trans follows #trans without #endtr between them");
		    warning("skipping to next #endtr");
		    PpTokDiscard(token);
		    for(token = get_token(); token; )
			{
			    while(token && (token->type != DV))
				{
				    PpTokDiscard(token);
				    token = get_token();
				}
			    if (token)
				{
				    PpTokDiscard(token);
				    token = gTSkSpace();
				    if(token && (token->type == NAME) &&
				       !strcmp(token->token, "endtr"))
					break;
				}
			}
		    if (!token)
			panic("#trans failed, could not find #endtr when trying to recover");
		}
	    else
		{
		    inTrans = TRUE;
		    trans();
		    inTrans = FALSE;
		}
	    break;
	case DV_PREC:
	    prec();
	    break;
	case DV_ENDTR:
	    error("Orphan #endtr");
	    break;
	case DV_TESTER:
	    tester();
	    break;
	case DV_VERIFY:
	    ReadVerify();
	    break;
	default:
	    ctrlLine = raw_read_line(FALSE);
	    (void) sprintf(errStr, 
			   "Ignoring unknown control %s",
			   token ? token->token : "nothing");
	    error(errStr);
	    LstDestroy(ctrlLine, TRUE);
	    break;
	}
    if (token)
	PpTokDiscard(token);
    read_tokens_match_newline();
    parsing_control--;
    return;
}

static DOMAIN ppEnumVal;
static unsigned ppPriVal;
static PP_TOKEN *ppToken;
static MACRO *ppMacrop;

/***************************************************************************
*
* Function:  app_parse
* Purpose:
* Arguments: exit_trigger - 
*            strict - 
* Returns:
* See also:
*
****************************************************************************
*/
FE2(PP_TOKEN *, app_parse,
    PP_TOKEN_TYPE, exit_trigger,
    BOOL, strict)
{
    for (ppToken = get_token(); ppToken; ppToken = get_token())
	{
	    if (ppToken->type == exit_trigger)
		return(ppToken); 
	    switch (ppToken->type)
		{
		case OPENING_BRACKET:
		case OPENING_SQUARE_BRACKET:
		case OPENING_PARENTHESIS:
		case TUPLE_LEFT:
		    if (parsing_control)
			{
			    (void) LstInsLast((LIST *) LstFirst(ppLsLs),
					      (char *) ppToken);
			    ppToken =
				app_parse(closing_bracket_pair[ppToken->type],
					  strict);
			    (void) LstInsLast((LIST *) LstFirst(ppLsLs),
					      (char *) ppToken);
			    continue;
			}
		    break;
		case CLOSING_PARENTHESIS:
		case CLOSING_BRACKET:
		case CLOSING_SQUARE_BRACKET:
		case TUPLE_RIGHT:
		    break;
		case DV:
		    PpTokDiscard(ppToken);
		    control_parse();
		    if (skipping)
			do_skip(FALSE);
		    continue;
		case NAME:
		    ppMacrop = ismacro(ppToken->token);
		    if (ppMacrop)
			{
			    PpTokDiscard(ppToken);
			    macro_expansion_parse(ppMacrop, strict);
			    continue;
			}
		    else if (IsEnum(ppToken->token, &ppEnumVal))
			{
			    PpTokDiscard(ppToken);
			    d_ppio_unget(ppEnumVal, strict);
			    continue;
			}
		    else if (IsPri(ppToken->token, &ppPriVal))
			{
			    PpTokDiscard(ppToken);
			    u_ppio_unget(ppPriVal);
			    continue;
			}
		} 
	    if (parsing_control)
		(void) LstInsLast((LIST *) LstFirst(ppLsLs),
				  (char *) ppToken);
	    else
		PpTokDiscard(ppToken);
	}
    check_stack();
    return((PP_TOKEN *) NULL);
}

/***************************************************************************
*
* Function:  cons_parse
* Purpose:
* Arguments: exit_trigger - 
*            strict - 
* Returns:
* See also:
*
****************************************************************************
*/
FE2(PP_TOKEN *, cons_parse,
    PP_TOKEN_TYPE, exit_trigger,
    BOOL, strict)
{
    for (ppToken = get_token(); ppToken; ppToken = get_token())
	{
	    if ((ppToken->type + 1 == (exit_trigger & 255)) ||
		(ppToken->type + 1 == (exit_trigger >> 8)))
		return(ppToken); 
	    switch (ppToken->type)
		{
		case OPENING_BRACKET:
		case OPENING_SQUARE_BRACKET:
		case OPENING_PARENTHESIS:
		case TUPLE_LEFT:
		    if (parsing_control)
			{
			    (void) LstInsFirst((LIST *) LstFirst(ppLsLs),
					       (char *) ppToken);
			    ppToken =
				cons_parse(closing_bracket_pair[ppToken->type]
					   + 1, strict);
			    (void) LstInsFirst((LIST *) LstFirst(ppLsLs),
					       (char *) ppToken);
			    continue;
			}
		    break;
		case CLOSING_PARENTHESIS:
		case CLOSING_BRACKET:
		case CLOSING_SQUARE_BRACKET:
		case TUPLE_RIGHT:
		    break;
		case DV:
		    PpTokDiscard(ppToken);
		    control_parse();
		    if (skipping)
			do_skip(FALSE);
		    continue;
		case NAME:
		    ppMacrop = ismacro(ppToken->token);
		    if (ppMacrop)
			{
			    PpTokDiscard(ppToken);
			    macro_expansion_parse(ppMacrop, strict);
			    continue;
			}
		    else if (IsEnum(ppToken->token, &ppEnumVal))
			{
			    PpTokDiscard(ppToken);
			    d_ppio_unget(ppEnumVal, strict);
			    continue;
			}
		    else if (IsPri(ppToken->token, &ppPriVal))
			{
			    PpTokDiscard(ppToken);
			    u_ppio_unget(ppPriVal);
			    continue;
			}
		}
	    if (parsing_control)
		(void) LstInsFirst((LIST *) LstFirst(ppLsLs),
				   (char *) ppToken);
	    else
		PpTokDiscard(ppToken);
	}
    check_stack();
    return((PP_TOKEN *) NULL);
}

/***************************************************************************
*
* Function:  out_parse
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, out_parse)
{
    for (ppToken = get_token(); ppToken; ppToken = get_token())
	{
	    switch (ppToken->type)
		{
		case OPENING_BRACKET:
		case OPENING_SQUARE_BRACKET:
		case OPENING_PARENTHESIS:
		case TUPLE_LEFT:
		    if (parsing_control)
			{
			    output_token(ppToken);
			    PpTokDiscard(ppToken);
			    out_parse();
			    continue;
			}
		    break;
		case CLOSING_PARENTHESIS:
		case CLOSING_BRACKET:
		case CLOSING_SQUARE_BRACKET:
		case TUPLE_RIGHT:
		    break;
		case DV:
		    PpTokDiscard(ppToken);
		    control_parse();
		    if (skipping)
			do_skip(FALSE);
		    continue;
		case NAME:
		    ppMacrop = ismacro(ppToken->token);
		    if (ppMacrop)
			{
			    PpTokDiscard(ppToken);
			    macro_expansion_parse(ppMacrop, TRUE);
			    continue;
			}
		    else if (IsEnum(ppToken->token, &ppEnumVal))
			{
			    PpTokDiscard(ppToken);
			    d_ppio_unget(ppEnumVal, TRUE);
			    continue;
			}
		    else if (IsPri(ppToken->token, &ppPriVal))
			{
			    PpTokDiscard(ppToken);
			    u_ppio_unget(ppPriVal);
			    continue;
			}
		}
	    output_token(ppToken);
	    PpTokDiscard(ppToken);
	}
    check_stack();
}
