/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  place.c
*
* Part of: PROD's program prpp
*
* Authors: Johannes Helander and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     This module handles place and tester declarations.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include <string.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ppio.h"
#include "token.h"
#include "parse.h"
#include "place.h"
#include "prpp.h"
#include "lexer.h"
#include "macro.h"
#include "util.h"
#include "ptrbl.h"
#include "baltree.h"
#include "gphutil.h"
#include "mark.h"
#include "expr.h"
#include "net.h"
#include "pplst.h"
#include "cmdnm.h"
#include "prpexpr.h"

/***************************************************************************
*
* Function:  read_tokens_match_newline
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, read_tokens_match_newline)
{
    PP_TOKEN *token;
  
    token = gTSkSpace();
    if (!token)
	panic("Unexpected end of file"); 
    else
	{
	    if (token->type != NEWLINE) 
		{
		    (void) sprintf(errStr, 
				   "Ignoring rest of line while looking at token \"%s\"", 
				   token->token);
		    warning(errStr);
		    while(token && (token->type != NEWLINE))
			{
			    PpTokDiscard(token);
			    token = get_token();
			}
		}
	    if (token)
		PpTokDiscard(token);
	}
}


/***************************************************************************
*
* Function:  gTSkSpace
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(PP_TOKEN *, gTSkSpace)
{
  PP_TOKEN *token;

  token = get_token();
  while (token && ((token->type == SPACE) || (token->type == COMMENT)))
      {
	  PpTokDiscard(token);
	  token = get_token();
      }
  return token;
}

/***************************************************************************
*
* Function:  gTSkEolAndSpace
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(PP_TOKEN *, gTSkEolAndSpace)
{
  PP_TOKEN *token;

  token = get_token();
  while (token && ((token->type == SPACE) || (token->type == NEWLINE) ||
		   (token->type == COMMENT))) 
      {
	  PpTokDiscard(token);
	  token = get_token();
      }
  return token;
}

/***************************************************************************
*
* Function:  pla
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, pla)
{
    PP_TOKEN *token;
    char *name;
    LIST *low, *high;
    MARKING *marking, *limMk;
    PLACE *pl;
    unsigned i = 0;
    BOOL monFlag;
    MULTUP *multup;
    TUPLE *loTuple, *hiTuple;

    name = NULL;
    marking = NULL;
    limMk = NULL;
    low = high = NULL;
    monFlag = FALSE;
    token = gTSkSpace();
    TOKEN_MATCH(token, NAME, "name after \"#place\"");
    name = StrCopy(token->token);
    LSTFOR (PlaceLst, pl, PLACE)
	if (!strcmp(name, PlaGName(pl)))
	    break;
    if (pl)
	{
	    (void) sprintf(errStr, "Place %s redefined", name);
	    error(errStr);
	    goto recover;
	}
    if (verify && !strcmp(name, "t_e_s_t_e_r"))
	{
	    error("Place t_e_s_t_e_r is possible\nonly when there is no #verify");
	    goto recover;
	}
    if (monName && !strcmp(name, monName))
	monFlag = TRUE;
    for (;;)
	{
	    PpTokDiscard(token);
	    token = gTSkSpace();
	    if(token && token->type == NEWLINE) 
		{
		    if (monFlag &&
			((!marking) || (!MarFirst(marking)) ||
			 (MarFirst(marking) != MarLast(marking)) ||
			 (MultupGArity(MarFirst(marking)) != 1) ||
			 (MultupGCount(MarFirst(marking)) != 1L)))
			{
			    (void) sprintf(errStr,
					   "place %s: tester must have exactly",
					   name);
			    error(errStr);
			    error("one unary tuple in the initial marking");
			    goto recover;
			}
		    if (low && high)
			for (loTuple = (TUPLE *) LstBegin(low),
			     hiTuple = (TUPLE *) LstBegin(high);
			     loTuple && hiTuple;)
			    {
				if (TupGArity(loTuple) < TupGArity(hiTuple))
				    loTuple = (TUPLE *) LstStep(low);
				else if (TupGArity(loTuple) >
					 TupGArity(hiTuple))
				    hiTuple = (TUPLE *) LstStep(high);
				else
				    {
					for (i = 0; i < TupGArity(loTuple);
					     i++)
					    if (TupGField(loTuple, i) >
						TupGField(hiTuple, i))
						{
						    (void) sprintf(errStr,
								   "Place %s, contradictory limits",
								   name);
						    error(errStr);
						    (void) fprintf(_err,
								   "low limit:\n");
						    TupDump(loTuple, (unsigned) 0);
						    (void) fprintf(_err,
								   "high limit:\n");
						    TupDump(hiTuple,
							    (unsigned) 0);
						    goto recover;
						}
					loTuple = (TUPLE *) LstStep(low);
					hiTuple = (TUPLE *) LstStep(high);
				    }
			    }
		    if (!marking)
			marking = MarCreate();
		    if (low)
			for (multup = MarBegin(marking),
			     loTuple = (TUPLE *) LstBegin(low);
			     multup && loTuple;)
			    {
				if (MultupGArity(multup) < TupGArity(loTuple))
				    multup = MarStep(marking);
				else if (MultupGArity(multup) >
					 TupGArity(loTuple))
				    loTuple = (TUPLE *) LstStep(low);
				else
				    {
					for (i = 0; i < MultupGArity(multup);
					     i++)
					    if (MultupGField(multup, i) <
						TupGField(loTuple, i))
						{
						    (void) sprintf(errStr,
								   "place %s: impossible initial marking",
								   name);
						    error(errStr);
						    TupDump(MultupGTuple(multup),
							    (unsigned) 0);
						    (void) fprintf(_err,
								   "low limit:\n");
						    TupDump(loTuple,
							    (unsigned) 0);
						    goto recover;
						}
					multup = MarStep(marking);
					if (multup && (MultupGArity(multup) !=
						       TupGArity(loTuple)))
					    loTuple = (TUPLE *) LstStep(low);
				    }
			    }
		    if (high)
			for (multup = MarBegin(marking),
			     hiTuple = (TUPLE *) LstBegin(high);
			     multup && hiTuple;)
			    {
				if (MultupGArity(multup) < TupGArity(hiTuple))
				    multup = MarStep(marking);
				else if (MultupGArity(multup) >
					 TupGArity(hiTuple))
				    hiTuple = (TUPLE *) LstStep(high);
				else
				    {
					for (i = 0; i < MultupGArity(multup);
					     i++)
					    if (MultupGField(multup, i) >
						TupGField(hiTuple, i))
						{
						    (void) sprintf(errStr,
								   "place %s: impossible initial marking",
								   name);
						    error(errStr);
						    TupDump(MultupGTuple(multup),
							    (unsigned) 0);
						    (void) fprintf(_err,
								   "high limit:\n");
						    TupDump(hiTuple,
							    (unsigned) 0);
						    goto recover;
						}
					multup = MarStep(marking);
					if (multup && (MultupGArity(multup) !=
						       TupGArity(hiTuple)))
					    hiTuple = (TUPLE *) LstStep(high);
				    }
			    }
		    if (arity_complaints)
			{
			    i = (unsigned) -1;
			    if (low)
				LSTFOR (low, loTuple, TUPLE)
				    {
					if (i == (unsigned) -1)
					    i = TupGArity(loTuple);
					else if (TupGArity(loTuple) != i)
					    {
						(void) sprintf(errStr,
							       "Varying arity in %s (%u and %u)\nIf you want several arities in a place, run %s without the -a option",
							       name, i,
							       TupGArity(loTuple),
							       myName);
						error(errStr);
						goto recover;
					    }
				    }
			    if (high)
				LSTFOR (high, hiTuple, TUPLE)
				    {
					if (i == (unsigned) -1)
					    i = TupGArity(hiTuple);
					else if (TupGArity(hiTuple) != i)
					    {
						(void) sprintf(errStr,
							       "Varying arity in %s (%u and %u)\nIf you want several arities in a place, run %s without the -a option",
							       name, i,
							       TupGArity(hiTuple),
							       myName);
						error(errStr);
						goto recover;
					    }
				    }
			    MARFOR (marking, multup)
				{
				    if (i == (unsigned) -1)
					i = MultupGArity(multup);
				    else if (MultupGArity(multup) != i)
					{
					    (void) sprintf(errStr,
							   "Varying arity in %s (%u and %u)\nIf you want several arities in a place, run %s without the -a option",
							   name, i,
							   MultupGArity(multup),
							   myName);
					    error(errStr);
					    goto recover;
					}
				}

			}
		    pl = PlaCons(name, low, high, marking);
		    if (arity_complaints)
			PlaSArity(pl, i);
		    else if (!MarEmpty(marking))
			PlaSArity(pl, MultupGArity(MarFirst(marking)));
		    (void) LstInsLast(PlaceLst, (char *) pl);
		    if (monFlag)
			monPlace = pl;
		    unget_token(token);	
		    PpTokDiscard(token);
		    return;
		}
	    TOKEN_MATCH(token, NAME,
                        ((void) sprintf(errStr, "mk%s in place %s",
                                        placeLimits ? ", lo, or hi" : "",
					name), errStr));
	    if (!strcmp(token->token, "mk"))
		{
		    if (marking)
			{
			    (void) sprintf(errStr, "mk redefined in place %s",
					   name);
			    error(errStr);
			    goto recover;
			}
		    PpTokDiscard(token);
		    token = gTSkSpace();
		    TOKEN_MATCH(token, OPENING_PARENTHESIS,
				((void)
				 sprintf(errStr,
					 "\"(\" after \"mk\" in place %s",
					 name), errStr));
		    PrpExpr(&marking, name, (unsigned) 3,
			    (LIST *) NULL, (DOMAIN *) NULL);
		    if (errorFlag != NO_ERROR)
			goto recover;
		}
	    else if (!strcmp(token->token, "lo"))
		{
                    if (!placeLimits)
                        {
                            (void) sprintf(errStr,
                                           "lo defined in place %s\nIf you want a limit in a place, run %s without the -R option",
                                           name, myName);
			    error(errStr);
			    goto recover;
                        }
                    if (low)
			{
			    (void) sprintf(errStr, "lo redefined in place %s",
					   name);
			    error(errStr);
			    goto recover;
			}
		    PpTokDiscard(token);
		    token = gTSkSpace();
		    TOKEN_MATCH(token, OPENING_PARENTHESIS,
				((void)
				 sprintf(errStr,
					 "\"(\" after \"lo\" in place %s",
					 name), errStr));
		    PrpExpr(&limMk, name, (unsigned) 5,
			    (LIST *) NULL, (DOMAIN *) NULL);
		    if (errorFlag != NO_ERROR)
			goto recover;
		    low = LstCreate(&tupTools);
		    loTuple = NULL;
		    if (limMk)
			{
			    MARFOR (limMk, multup)
				if (!MultupGArity(multup))
				    ;
				else if ((!loTuple) ||
					 (MultupGArity(multup) !=
					  TupGArity(loTuple)))
				    {
					loTuple = TupCopy(MultupGTuple(multup));
					(void) LstInsLast(low, (char *)
							  loTuple);
				    }
				else
				    for (i = 0; i < MultupGArity(multup); i++)
					if (MultupGField(multup, i) >
					    TupGField(loTuple, i))
					    TupSField(loTuple, i,
						      MultupGField(multup, i));
			    MarDiscard(limMk);
			    limMk = NULL;
			}
		}
	    else if (!strcmp(token->token, "hi"))
		{
                    if (!placeLimits)
                        {
                            (void) sprintf(errStr,
                                           "hi defined in place %s\nIf you want a limit in a place, run %s without the -R option",
                                           name, myName);
			    error(errStr);
			    goto recover;
                        }
		    if (high)
			{
			    (void) sprintf(errStr, "hi redefined in place %s", name);
			    error(errStr);
			    goto recover;
			}
		    PpTokDiscard(token);
		    token = gTSkSpace();
		    TOKEN_MATCH(token, OPENING_PARENTHESIS,
				((void)
				 sprintf(errStr,
					 "\"(\" after \"hi\" in place %s",
					 name), errStr));
		    PrpExpr(&limMk, name, (unsigned) 1,
			    (LIST *) NULL, (DOMAIN *) NULL);
		    if (errorFlag != NO_ERROR)
			goto recover;
		    high = LstCreate(&tupTools);
		    hiTuple = NULL;
		    if (limMk)
			{
			    MARFOR (limMk, multup)
				if (!MultupGArity(multup))
				    ;
				else if ((!hiTuple) ||
					 (MultupGArity(multup) !=
					  TupGArity(hiTuple)))
				    {
					hiTuple = TupCopy(MultupGTuple(multup));
					(void) LstInsLast(high, (char *)
							  hiTuple);
				    }
				else
				    for (i = 0; i < MultupGArity(multup); i++)
					if (MultupGField(multup, i) <
					    TupGField(hiTuple, i))
					    TupSField(hiTuple, i,
						      MultupGField(multup, i));
			    MarDiscard(limMk);
			    limMk = NULL;
			}
		}
	    else
		TOKEN_MATCH(token, NEVERMATCH,
			    ((void) sprintf(errStr,
                                            "mk%s in place %s",
                                            placeLimits ? ", lo, or hi" : "",
                                            name), errStr));
	}
 recover: ;
    error_flag = 1;
    if (name)
	Deallocate((char *) name);
    if (marking)
	MarDiscard(marking);
    if (limMk)
	MarDiscard(limMk);
    if (low)
	LstDestroy(low, TRUE);
    if (high)
	LstDestroy(high, TRUE);
    while (token && (token->type != NEWLINE))
	{
	    PpTokDiscard(token);
	    token = get_token();
	}
    if (token)
	{
	    unget_token(token);	
	    PpTokDiscard(token);
	}
}

/***************************************************************************
*
* Function:  tester
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(void, tester)
{
    PP_TOKEN *token;
    char *name;
    MARKING *marking;
    MULTUP *multup;
    unsigned reject, deadlock, livelock, infinite;
    DOMAIN state;
    MON *mon;
    PLACE *pl;
    char *format;

    name = NULL;
    marking = NULL;
    token = gTSkSpace();
    TOKEN_MATCH(token, NAME, "name after \"#tester\"");
    if (monName)
	{
	    error("#tester is not allowed more than once");
	    goto recover;
	}
    if (verify)
	{
	    error("#tester is possible\nonly when there is no #verify");
	    goto recover;
	}
#if 0
    if (!LstEmpty(PrecLst))
	{
	    (void) sprintf(errStr, "tester \"%s\": #prec declarations",
			   token->token);
	    error(errStr);
	    error("make tester declarations illegal");
	    goto recover;
	}
#endif
    LSTFOR (PlaceLst, pl, PLACE)
	if (!strcmp(PlaGName(pl), token->token))
	    break;
    if (pl && ((!MarFirst(PlaGMarking(pl))) ||
	       (MarFirst(PlaGMarking(pl)) != MarLast(PlaGMarking(pl))) ||
	       (MultupGArity(MarFirst(PlaGMarking(pl))) != 1) ||
	       (MultupGCount(MarFirst(PlaGMarking(pl))) != 1L)))
	{
	    (void) sprintf(errStr,
			   "tester \"%s\": tester must have exactly",
			   name);
	    error(errStr);
	    error("one unary tuple in the initial marking");
	    goto recover;
	}
    name = StrCopy(token->token);
    reject = 0;
    deadlock = 0;
    livelock = 0;
    infinite = 0;
    for (;;)
	{
	    PpTokDiscard(token);
	    token = gTSkSpace();
	    if(token && (NEWLINE == token->type)) 
		{
		    monName = name;
		    if (pl)
			monPlace = pl;
		    unget_token(token);	
		    PpTokDiscard(token);
		    return;
		}
	    TOKEN_MATCH(token, NAME,
			((void) sprintf(errStr,
					"reject, deadlock, livelock, or infinite\nin tester \"%s\"",
					name), errStr));
	    if (!strcmp(token->token, "reject"))
		reject += 2;
	    else if (!strcmp(token->token, "deadlock"))
		deadlock += 2;
	    else if (!strcmp(token->token, "livelock"))
		livelock += 2;
	    else if (!strcmp(token->token, "infinite"))
		infinite += 2;
	    if ((reject > 2) || (deadlock > 2) ||
		(livelock > 2) || (infinite > 2))
		{
		    (void) sprintf(errStr,
				   "%s redefined in tester \"%s\"",
				   token->token, name);
		    error(errStr);
		    goto recover;
		}
	    if ((reject == 2) || (deadlock == 2) ||
		(livelock == 2) || (infinite == 2))
		{
		    PpTokDiscard(token);
		    token = gTSkSpace();
		    TOKEN_MATCH(token, OPENING_PARENTHESIS,
				((void)
				 sprintf(errStr,
					 "\"(\" after \"%s\" in tester \"%s\"",
					 (reject == 2) ? "reject"
					 : ((deadlock == 2) ? "deadlock"
					    : ((livelock == 2) ? "livelock"
					       : "infinite")),
					 name), errStr));
		    PrpExpr(&marking, name, (unsigned)
			    ((reject == 2) ? 2
			     : ((deadlock == 2)	? 4
				: ((livelock == 2) ? 8 : 0))),
			    (LIST *) NULL, (DOMAIN *) NULL);
		    if (errorFlag != NO_ERROR)
			goto recover;
    		    MARFOR (marking, multup)
			{
			    if (MultupGArity(multup) != 1)
				{
				    (void) sprintf(errStr,
						   "tester \"%s\": tuples should be unary",
						   name);
				    error(errStr);
				    goto recover;
				}
			    state = MultupGField(multup, (unsigned) 0);
			    if (pl && PlaGLow(pl) && LstFirst(PlaGLow(pl)) &&
				(TupGArity((TUPLE *)
					   LstFirst(PlaGLow(pl))) == 1) &&
				(state < TupGField((TUPLE *)
						   LstFirst(PlaGLow(pl)), 0)))
				{
				    (void) sprintf(errStr,
						   "tester %%s: %s is below low limit %s",
						   DOMFORMAT, DOMFORMAT);
				    format = StrCopy(errStr);
				    (void) sprintf(errStr, format,
						   monName, state,
						   TupGField((TUPLE *)
							     LstFirst(PlaGLow(pl)),
							     0));
				    Deallocate((char *) format);
				    error(errStr);
				    goto recover;
				}
			    if (pl && PlaGHigh(pl) && LstFirst(PlaGHigh(pl)) &&
				(TupGArity((TUPLE *)
					   LstFirst(PlaGHigh(pl))) == 1) &&
				(state > TupGField((TUPLE *)
						   LstFirst(PlaGHigh(pl)), 0)))
				{
				    (void) sprintf(errStr,
						   "tester %%s: %s is above high limit %s",
						   DOMFORMAT, DOMFORMAT);
				    format = StrCopy(errStr);
				    (void) sprintf(errStr, format,
						   monName, state,
						   TupGField((TUPLE *)
							     LstFirst(PlaGHigh(pl)),
							     0));
				    Deallocate((char *) format);
				    error(errStr);
				    goto recover;
				}
			    mon = (MON *)
				BalTrElement(monitors, (char *) &state);
			    if (!mon)
				{	
				    mon = MonCons(state);
				    (void) BalTrInsert(monitors, (char *) mon,
						       (char *) &state, TRUE);
				}
			    if (reject == 2)
				MonSReject(mon);
			    else if (deadlock == 2)
				MonSDeadlock(mon);
			    else if (livelock == 2)
				MonSLivelock(mon);
			    else
				MonSInfinite(mon);
			}
		    MarDiscard(marking);
		    marking = NULL;
		    if (reject == 2)
			reject = 1;
		    else if (deadlock == 2)
			deadlock = 1;
		    else if (livelock == 2)
			livelock = 1;
		    else
			infinite = 1;
		}
	    else
		TOKEN_MATCH(token, NEVERMATCH,
			    ((void) sprintf(errStr,
					    "reject, deadlock, livelock, or infinite\nin tester \"%s\"",
					    name), errStr));
	}
 recover: ;
    error_flag = 1;
    if (name)
	Deallocate((char *) name);
    if (marking)
	MarDiscard(marking);
    if (!monName)
	{
	    BalTrDestroy(monitors, TRUE);
	    monitors = BalTrCreate(&monBalTools);
	}
    while (token && (token->type != NEWLINE))
	{
	    PpTokDiscard(token);
	    token = get_token();
	}
    if (token)
	{
	    unget_token(token);	
	    PpTokDiscard(token);
	}
}
