/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  prpexpr.h
*
* Part of: PROD's program prpp
*
* Author:  Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     See prpexpr.c.
*
* Version history:
*     $Log$
***************************************************************************/

typedef struct 
{
    VOF out;
#ifdef DEBUG
    VOF print;
#endif
    FOOL identic;
} MEXOUT;

extern MEXOUT mexOutTable[SZ_MO];

typedef struct 
{
    VOF out;
#ifdef DEBUG
    VOF print;
#endif
    FOOL identic;
} XPROUT;

extern XPROUT xprOutTable[SZ_EO - 3];

#define MexOut(markexpr,depth) \
    ((*mexOutTable[(markexpr)->operator].out)(markexpr,depth))
#ifdef DEBUG
#define MexPrint(markexpr) \
    ((*mexOutTable[(markexpr)->operator].print)(markexpr))
#endif
#define MexIdentic(mep1,mep2) \
    (((mep1) == (mep2)) || \
     (((mep1)->operator == (mep2)->operator) && \
      (*mexOutTable[(mep1)->operator].identic)(mep1,mep2)))
#define XprOut(expression,depth) \
    ((*xprOutTable[(expression)->operator].out)(expression,depth))
#ifdef DEBUG
#define XprPrint(expression) \
    ((*xprOutTable[(expression)->operator].print)(expression))
#endif
#define XprIdentic(ep1,ep2) \
    (((ep1) == (ep2)) || \
     (((ep1)->operator == (ep2)->operator) && \
      (*xprOutTable[(ep1)->operator].identic)(ep1,ep2)))

extern FILE *pxFile;
extern unsigned long pxGotoNumber;

typedef struct FORMULA 
{
    unsigned operator;
    union
	{
	    EXPRESSION *expression;
	    struct 
		{
		    struct FORMULA *sub1f;
		    struct FORMULA *sub2f;
		} of;
	} uf;
} FORMULA;

typedef FORMULA *(*FORMF)();

#define ForAlloc() ((FORMULA *) Allocate(sizeof(FORMULA)))
extern TOOLS forSloot;

#define FO_UNTIL      0
#define FO_IMPLIES    1
#define FO_EXPR       2
#define SZ_FO 3
extern FORMF forPoTable[SZ_FO];

#define ForReduce(formula) ((*(forPoTable[(formula)->operator]))(formula))


typedef struct BCHST
{
    union
	{
	    struct
		{
		    unsigned long _flag;
		    unsigned long _val;
		} _fv;
	    struct
		{
		    struct BCHST *_left;
		    struct BCHST *_right;
		} _lr;
	} _ub;
    LIST *_succ;
} BCHST;

#define BchstAlloc() ((BCHST *) Allocate(sizeof(BCHST)))
#define BchstCFlag(bchst) ((bchst)->_ub._fv._flag = (unsigned long) 0L)
#define BchstInitial(bchst) ((bchst)->_ub._fv._flag & 1L)
#define BchstSInitial(bchst) ((bchst)->_ub._fv._flag |= 1L)
#define BchstCInitial(bchst) ((bchst)->_ub._fv._flag &= ~1L)
#define BchstAccepting(bchst) ((bchst)->_ub._fv._flag & 2L)
#define BchstSAccepting(bchst) ((bchst)->_ub._fv._flag |= 2L)
#define BchstCAccepting(bchst) ((bchst)->_ub._fv._flag &= ~2L)
#define BchstEventual(bchst) ((bchst)->_ub._fv._flag & 4L)
#define BchstSEventual(bchst) ((bchst)->_ub._fv._flag |= 4L)
#define BchstCEventual(bchst) ((bchst)->_ub._fv._flag &= ~4L)
#define BchstMarked(bchst) ((bchst)->_ub._fv._flag & 8L)
#define BchstSMarked(bchst) ((bchst)->_ub._fv._flag |= 8L)
#define BchstCMarked(bchst) ((bchst)->_ub._fv._flag &= ~8L)
#define BchstGVis(bchst) ((bchst)->_ub._fv._flag >> 4)
#define BchstSVis(bchst,vis) \
    ((bchst)->_ub._fv._flag = \
     ((bchst)->_ub._fv._flag & (unsigned long) 15) | ((vis) << 4))
#define BchstGVal(bchst) ((bchst)->_ub._fv._val)
#define BchstSVal(bchst,val) ((bchst)->_ub._fv._val = (val))
#define BchstAVal(bchst) (&((bchst)->_ub._fv._val))
#define BchstGLeft(bchst) ((bchst)->_ub._lr._left)
#define BchstSLeft(bchst,left) ((bchst)->_ub._lr._left = (left))
#define BchstGRight(bchst) ((bchst)->_ub._lr._right)
#define BchstSRight(bchst,right) ((bchst)->_ub._lr._right = (right))
#define BchstGSucc(bchst) ((bchst)->_succ)
#define BchstSSucc(bchst,succ) ((bchst)->_succ = (succ))

#ifdef DEBUG
#endif
extern TOOLS bchstTools;

typedef struct BCHTR
{
    unsigned long _val;
    BCHST *_bchst;
} BCHTR;

#define BchtrAlloc() ((BCHTR *) Allocate(sizeof(BCHTR)))
#define BchtrGVal(bchtr) ((bchtr)->_val)
#define BchtrSVal(bchtr,val) ((bchtr)->_val = (val))
#define BchtrGBchst(bchtr) ((bchtr)->_bchst)
#define BchtrSBchst(bchtr,bchst) ((bchtr)->_bchst = (bchst))
extern TOOLS bchtrTools;

extern LIST *prpExpList;
extern LIST *prpVerList;

DE3(void, LtlParse,
    FORMULA **, formula,
    DOMAIN *, value,
    MARKING **, marking)
DE1(unsigned, MexDepth,
    MARKEXPR *, markexpr)
DE1(unsigned, XprDepth,
    EXPRESSION *, expression)
DE2(void, PrpCaret,
    unsigned, count,
    unsigned, textLen)
DE0(int, PrpExpInput)
DE5(void, PrpExpr,
    MARKING **, marking,
    char *, placeName,
    unsigned, placeFlag,
    LIST *, list,
    DOMAIN *, value)
DE3(FORMULA *, ForSnoc,
    unsigned, operator,
    FORMULA *, fp1,
    FORMULA *, fp2)
DE3(FORMULA *, ForXua,
    unsigned, operator,
    FORMULA *, fp1,
    FORMULA *, fp2)
DE1(FORMULA *, ForYpoc,
    FORMULA *, formula)
DE1(void, ForDracsid,
    FORMULA *, formula)
#ifdef DEBUG
DE2(void, ForPmud,
    FORMULA *, formula,
    unsigned, depth)
DE1(void, ForPrint,
    FORMULA *, formula)
#endif
DE2(BOOL, ForIdentic,
    FORMULA *, fp1,
    FORMULA *, fp2)
DE1(BCHST *, BchstCons,
    BOOL, eventual)
DE1(void, BchstDiscard,
    BCHST *, bchst)
#ifdef DEBUG
DE2(void, BchstDump,
    BCHST *, bchst,
    unsigned, depth)
#endif
DE2(BCHTR *, BchtrCons,
    unsigned long, val,
    BCHST *, bchst)
#ifdef DEBUG
DE2(void, BchtrDump,
    BCHTR *, bchtr,
    unsigned, depth)
#endif
DE0(LIST *, BuTra)
DE1(void, OutBuchi,
    unsigned long *, pos)
DE0(void, ReadVerify)
DE2(BOOL, MexAffected,
    MARKEXPR *, markexpr,
    TRANSITION *, transition)
DE2(BOOL, XprAffected,
    EXPRESSION *, expression,
    TRANSITION *, transition)
DE2(BOOL, ForAffected,
    FORMULA *, formula,
    TRANSITION *, transition)
DE0(void, SetExMex)
DE3(void, ForClosure,
    FORMULA *, formula,
    LIST *, forList,
    LIST *, atomList)
DE1(void, BuchiSimplify,
    LIST *, buchi)
DE3(LIST *, ForBuchi,
    FORMULA *, formula,
    LIST *, atomList,
    unsigned long, atMask)
DE2(void, BuchiExt,
    LIST *, buchi,
    LIST *, atomList)

extern PP_TOKEN *prpExpNow;
extern char *prpExpPtr;
extern BOOL binaryMode;
extern unsigned long newBlock;

#define P (void) fprintf
