/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  prpexpr2.c
*
* Part of: PROD's program prpp
*
* Author:  Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     See prpexpr.c.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include <string.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ppio.h"
#include "token.h"
#include "output.h"
#include "place.h"
#include "prpp.h"
#include "lexer.h"
#include "macro.h"
#include "parse.h"
#include "util.h"
#include "ptrbl.h"
#include "baltree.h"
#include "gphutil.h"
#include "mark.h"
#include "encode.h"
#include "expr.h"
#include "net.h"
#include "prec.h"
#include "pplst.h"
#include "prpexpr.h"

#define F pxFile

unsigned long newBlock = 0L;

/* All these Cons, Copy, Discard, Dump and Tools are trivial and
 * similar to those explained in detail in modules such as gphutil.c. */

FE1(BCHST *, BchstCons,
    BOOL, eventual)
{
    BCHST *bchst;

    bchst = BchstAlloc();
    BchstCFlag(bchst);
    BchstSVal(bchst, (unsigned long) 0L);
    if (eventual)
	{
	    BchstSEventual(bchst);
	    BchstSSucc(bchst, LstCreate(&bchtrTools));
	}
    else
	BchstSSucc(bchst, LstCreate(&bchstTools));
    return(bchst);
}

FE1(void, BchstDiscard,
    BCHST *, bchst)
{
    LstDestroy(BchstGSucc(bchst), BchstEventual(bchst) != 0L);
    Deallocate((char *) bchst);
}

#ifdef DEBUG
FS1(void, ValDump,
    unsigned long, val)
{
    if (val)
	for (; val; val >>= 1)
	    P(_err, "%lu", val & 1L);
    else
	P(_err, "0");
    (void) fflush(_err);
}

static BOOL normBchst;

FS2(void, BchstPlDump,
    BCHST *, bchst,
    unsigned, depth)
{
    DumpIndent(depth);
    P(_err, "BCHST ");
    if (normBchst)
	{
	    P(_err, "%4lu", BchstGVis(bchst));
	    if (BchstInitial(bchst))
		P(_err, "i");
	    else
		P(_err, " ");
	    if (BchstAccepting(bchst))
		P(_err, "a");
	    else
		P(_err, " ");
	    if (BchstMarked(bchst))
		P(_err, "m");
	    else
		P(_err, " ");
	    ValDump(BchstGVal(bchst));
	}
    else
	{
	    P(_err, "%4lu", BchstGVis(BchstGLeft(bchst)));
	    if (BchstInitial(BchstGLeft(bchst)))
		P(_err, "i");
	    else
		P(_err, " ");
	    if (BchstAccepting(BchstGLeft(bchst)))
		P(_err, "a");
	    else
		P(_err, " ");
	    if (BchstMarked(BchstGLeft(bchst)))
		P(_err, "m");
	    else
		P(_err, " ");
	    ValDump(BchstGVal(BchstGLeft(bchst)));
	    P(_err, ", %4lu", BchstGVis(BchstGRight(bchst)));
	    if (BchstInitial(BchstGRight(bchst)))
		P(_err, "i");
	    else
		P(_err, " ");
	    if (BchstAccepting(BchstGRight(bchst)))
		P(_err, "a");
	    else
		P(_err, " ");
	    if (BchstMarked(BchstGRight(bchst)))
		P(_err, "m");
	    else
		P(_err, " ");
	    ValDump(BchstGVal(BchstGRight(bchst)));
	}
    P(_err, "\n");
    (void) fflush(_err);
}

FE2(void, BchstDump,
    BCHST *, bchst,
    unsigned, depth)
{
    BCHST *tshcb;
    BCHTR *bchtr;

    BchstPlDump(bchst, depth);
    if (normBchst && BchstEventual(bchst))
	LSTFOR (BchstGSucc(bchst), bchtr, BCHTR)
	    BchtrDump(bchtr, depth + 2);
    else
	LSTFOR (BchstGSucc(bchst), tshcb, BCHST)
	    BchstPlDump(tshcb, depth + 2);
}
#endif

TOOLS bchstTools =
{ (CFUNC) NULL, BchstDiscard
#ifdef DEBUG
, BchstDump
#endif
 };

FE2(BCHTR *, BchtrCons,
    unsigned long, val,
    BCHST *, bchst)
{
    BCHTR *bchtr;

    bchtr = BchtrAlloc();
    BchtrSVal(bchtr, val);
    BchtrSBchst(bchtr, bchst);
    return(bchtr);
}

#ifdef DEBUG
FE2(void, BchtrDump,
    BCHTR *, bchtr,
    unsigned, depth)
{
    DumpIndent(depth);
    P(_err, "BCHTR ");
    ValDump(BchtrGVal(bchtr));
    P(_err, " -> ");
    if (normBchst)
	{
	    P(_err, "%4lu", BchstGVis(BchtrGBchst(bchtr)));
	    if (BchstInitial(BchtrGBchst(bchtr)))
		P(_err, "i");
	    else
		P(_err, " ");
	    if (BchstAccepting(BchtrGBchst(bchtr)))
		P(_err, "a");
	    else
		P(_err, " ");
	    if (BchstMarked(BchtrGBchst(bchtr)))
		P(_err, "m");
	    else
		P(_err, " ");
	    ValDump(BchstGVal(BchtrGBchst(bchtr)));
	}
    else
	{
	    P(_err, "%4lu", BchstGVis(BchstGLeft(BchtrGBchst(bchtr))));
	    if (BchstInitial(BchstGLeft(BchtrGBchst(bchtr))))
		P(_err, "i");
	    else
		P(_err, " ");
	    if (BchstAccepting(BchstGLeft(BchtrGBchst(bchtr))))
		P(_err, "a");
	    else
		P(_err, " ");
	    if (BchstMarked(BchstGLeft(BchtrGBchst(bchtr))))
		P(_err, "m");
	    else
		P(_err, " ");
	    ValDump(BchstGVal(BchstGLeft(BchtrGBchst(bchtr))));
	    P(_err, ", %4lu", BchstGVis(BchstGRight(BchtrGBchst(bchtr))));
	    if (BchstInitial(BchstGRight(BchtrGBchst(bchtr))))
		P(_err, "i");
	    else
		P(_err, " ");
	    if (BchstAccepting(BchstGRight(BchtrGBchst(bchtr))))
		P(_err, "a");
	    else
		P(_err, " ");
	    if (BchstMarked(BchstGRight(BchtrGBchst(bchtr))))
		P(_err, "m");
	    else
		P(_err, " ");
	    ValDump(BchstGVal(BchstGRight(BchtrGBchst(bchtr))));
	}
    P(_err, "\n");
    (void) fflush(_err);
}
#endif

TOOLS bchtrTools =
{ (CFUNC) NULL, Deallocate
#ifdef DEBUG
, BchtrDump
#endif
 };

/***************************************************************************
*
* Function:  BchstVisit
* Purpose:   This routine is needed in the simplification of a Buchi
*            automaton. The strongly connected components of the
*            automaton are computed, using Tarjan's algorithm.
*            Components from which no accepting state is reachable
*            can simply be removed. Also, states that are reachable
*            from no initial state can be removed.
* Arguments: bchst - the current state
*            now - the visit counter
*            compStack - the "Tarjan" stack
*            visStack - the depth first search stack
* Returns:
* See also:  BchstSimplify
*
****************************************************************************
*/
FS4(void, BchstVisit,
    BCHST *, bchst,
    unsigned long *, now,
    LIST *, compStack,
    LIST *, visStack)
{
    unsigned long val;
    unsigned long min;
    BCHST *tshcb = NULL;
    BCHTR *bchtr = NULL;

BEGIN: ;
    (void) LstInsFirst(compStack, (char *) bchst);
    (*now)++;
    BchstSVis(bchst, *now);
    min = *now;
    if (BchstEventual(bchst))
	{
	    bchtr = (BCHTR *) LstBegin(BchstGSucc(bchst));
	    if (!bchtr)
		goto CONCLUDE;
	}
    else
	{
	    tshcb = (BCHST *) LstBegin(BchstGSucc(bchst));
	    if (!tshcb)
		goto CONCLUDE;
	}
ARROW: ;
    if (BchstEventual(bchst))
	val = BchstGVis(BchtrGBchst(bchtr));
    else
	val = BchstGVis(tshcb);
    if (!val)
	{
	    (void) LstInsFirst(visStack, (char *) BchtrCons(min, bchst));
	    if (BchstEventual(bchst))
		bchst = BchtrGBchst(bchtr);
	    else
		bchst = tshcb;
	    goto BEGIN;
	}
CONT: ;
    if (val < min)
	min = val;
    if (BchstEventual(bchst))
	{
	    if (BchstMarked(BchtrGBchst(bchtr)))
		BchstSMarked(bchst);
	    bchtr = (BCHTR *) LstStep(BchstGSucc(bchst));
	    if (bchtr)
		goto ARROW;
	}
    else
	{
	    if (BchstMarked(tshcb))
		BchstSMarked(bchst);
	    tshcb = (BCHST *) LstStep(BchstGSucc(bchst));
	    if (tshcb)
		goto ARROW;
	}
CONCLUDE: ;
    if (min == BchstGVis(bchst))
	{
	    if (BchstMarked(bchst))
		{
		    if ((bchst == (BCHST *) LstFirst(compStack)) &&
			BchstAccepting(bchst))
			{
			    if (BchstEventual(bchst))
				{
				    LSTFOR (BchstGSucc(bchst), bchtr, BCHTR)
					if (BchtrGBchst(bchtr) == bchst)
					    break;
				    if (!bchtr)
					BchstCAccepting(bchst);
				}
			    else if (!LstWhich(BchstGSucc(bchst),
					       (char *) bchst))
				BchstCAccepting(bchst);
			}
		}
	    else
		{
		    tshcb = (BCHST *) LstBegin(compStack);
		    if (tshcb != bchst)
			while (TRUE)
			    {
				if (BchstAccepting(tshcb))
				    {
					BchstSMarked(bchst);
					break;
				    }
				if (tshcb == bchst)
				    break;
				tshcb = (BCHST *) LstStep(compStack);
			    }
		    else if (BchstAccepting(bchst))
			{
			    if (BchstEventual(bchst))
				{
				    LSTFOR (BchstGSucc(bchst), bchtr, BCHTR)
					if (BchtrGBchst(bchtr) == bchst)
					    break;
				    if (bchtr)
					BchstSMarked(bchst);
				    else
					BchstCAccepting(bchst);
				}
			    else if (LstWhich(BchstGSucc(bchst),
					      (char *) bchst))
				BchstSMarked(bchst);
			    else
				BchstCAccepting(bchst);
			}
		}
	    do
		{
		    tshcb = (BCHST *) LstDelFirst(compStack, FALSE);
		    if (BchstMarked(bchst))
			BchstSMarked(tshcb);
		    BchstSVis(tshcb, (unsigned long) -1L);
		} while (tshcb != bchst);
	}
    if (!LstEmpty(visStack))
	{
	    val = min;
	    min = BchtrGVal((BCHTR *) LstFirst(visStack));
	    bchst = BchtrGBchst((BCHTR *) LstFirst(visStack));
	    (void) LstDelFirst(visStack, TRUE);
	    if (BchstEventual(bchst))
		bchtr = (BCHTR *) LstThis(BchstGSucc(bchst));
	    else
		tshcb = (BCHST *) LstThis(BchstGSucc(bchst));
	    goto CONT;
	}
}

/***************************************************************************
*
* Function:  BuchiSimplify
* Purpose:   This routine simplifies a Buchi automaton as explained in
*            the description of BchstVisit.
* Arguments: buchi - the automaton
* Returns:
* See also:  BchstVisit
*
****************************************************************************
*/
FE1(void, BuchiSimplify,
    LIST *, buchi)
{
    VOID *cursor;
    BCHST *bchst;
    BCHST *tshcb;
    BCHTR *bchtr;
    unsigned long now;
    LIST *compStack;
    LIST *visStack;

    LSTFOR (buchi, bchst, BCHST)
	{
	    BchstCMarked(bchst);
	    BchstSVis(bchst, (unsigned long) 0L);
	}
    now = 0L;
    compStack = LstCreate(&bchstTools);
    visStack = LstCreate(&bchtrTools);
    LSTFOR (buchi, bchst, BCHST)
	if (BchstInitial(bchst) && !BchstGVis(bchst))
	    BchstVisit(bchst, &now, compStack, visStack);
    LstDestroy(compStack, FALSE);
    LstDestroy(visStack, TRUE);
    LSTFOR (buchi, bchst, BCHST)
	{
	    if (!BchstMarked(bchst))
		continue;
	    cursor = NULL;
	    if (BchstEventual(bchst))
		{
		    LSTFOR (BchstGSucc(bchst), bchtr, BCHTR)
			if (BchstMarked(BchtrGBchst(bchtr)))
			    cursor = LstGet(BchstGSucc(bchst));
			else
			    {
				LstSet(BchstGSucc(bchst), cursor);
				(void) LstDelNext(BchstGSucc(bchst), TRUE);
			    }
		}
	    else
		{
		    LSTFOR (BchstGSucc(bchst), tshcb, BCHST)
			if (BchstMarked(tshcb))
			    cursor = LstGet(BchstGSucc(bchst));
			else
			    {
				LstSet(BchstGSucc(bchst), cursor);
				(void) LstDelNext(BchstGSucc(bchst), FALSE);
			    }
		}
	}
    now = 0L;
    cursor = NULL;
    LSTFOR (buchi, bchst, BCHST)
	if (BchstMarked(bchst))
	    {
		BchstCMarked(bchst);
		now++;
		BchstSVis(bchst, now);
		cursor = LstGet(buchi);
	    }
	else
	    {
		LstSet(buchi, cursor);
		(void) LstDelNext(buchi, TRUE);
	    }
}

/***************************************************************************
*
* Function:  BuchiOr
* Purpose:   This routine computes the union of two Buchi automata.
* Arguments: bp0 - an automaton
*            bp1 - an automaton
* Returns:   the union
* See also:  BuchiAnd
*
****************************************************************************
*/
FS2(LIST *, BuchiOr,
    LIST *, bp0,
    LIST *, bp1)
{
    BCHST *bchst;
    unsigned long now;

    now = (unsigned long) LstSize(bp0);
    LSTFOR (bp1, bchst, BCHST)
	{
	    now++;
	    BchstSVis(bchst, now);
	}
    (void) LstCatenate(bp0, bp1);
    LstDestroy(bp1, FALSE);
#ifdef DEBUG
    P(F, "OR\n");
    normBchst = TRUE;
    LSTFOR (bp0, bchst, BCHST)
	BchstDump(bchst, (unsigned) 0);
#endif
    return(bp0);
}

/***************************************************************************
*
* Function:  BuchiAnd
* Purpose:   This routine computes the intersection of two Buchi automata.
*            The intersection should be thought with respect to
*            (0,1)-vectors. In other words, you should imagine a set
*            of (0,1)-vectors in the place of a (0,1,x)-vector where
*            x is the "don't care" symbol.
* Arguments: bp0 - an automaton
*            bp1 - an automaton
*            atMask - indicates how long vectors we consider, i.e.
*                     how many atomic formulas we must take into account
* Returns:   the intersection
* See also:  BuchiOr
*
****************************************************************************
*/
FS3(LIST *, BuchiAnd,
    LIST *, bp0,
    LIST *, bp1,
    unsigned long, atMask)
{
    LIST *buchi;
    VOID *cursor;
    unsigned long flag;
    unsigned long mask;
    BCHST *bchst0;
    BCHST *bchst1;
    BCHST *bchst2;
    BCHST *bchst3;
    BCHST *bchst4;

    buchi = LstCreate(&bchstTools);
    LSTFOR (bp0, bchst0, BCHST)
	LSTFOR (bp1, bchst1, BCHST)
	    {
		for (mask = atMask;
		     mask && (((BchstGVal(bchst0) & mask) ==
			       (BchstGVal(bchst1) & mask)) ||
			      !(BchstGVal(bchst0) & (mask << 1) &
				BchstGVal(bchst1)));
		     mask >>= 2)
		    ;
		if (mask)
		    continue;
		(void) LstInsLast(buchi, (char *) BchstCons(FALSE));
		BchstSLeft((BCHST *) LstLast(buchi), bchst0);
		BchstSRight((BCHST *) LstLast(buchi), bchst1);
		(void) LstInsLast(buchi, (char *) BchstCons(FALSE));
		BchstSLeft((BCHST *) LstLast(buchi), bchst0);
		BchstSRight((BCHST *) LstLast(buchi), bchst1);
	    }
    LSTFOR (buchi, bchst0, BCHST)
	{
	    bchst1 = (BCHST *) LstStep(buchi);
	    LSTFOR (BchstGSucc(BchstGLeft(bchst0)), bchst3, BCHST)
		LSTFOR (BchstGSucc(BchstGRight(bchst0)), bchst4, BCHST)
		    {
			cursor = LstGet(buchi);
			LSTFOR (buchi, bchst2, BCHST)
			    if ((BchstGLeft(bchst2) == bchst3) &&
				(BchstGRight(bchst2) == bchst4))
				break;
			    else
				(void) LstStep(buchi);
			if (!bchst2)
			    {
				LstSet(buchi, cursor);
				continue;
			    }
			(void)
			    LstInsLast(BchstGSucc(bchst0),
				       BchstAccepting(BchstGLeft(bchst0))
				       ? ((char *) LstNext(buchi))
				       : (char *) bchst2);
			(void)
			    LstInsLast(BchstGSucc(bchst1),
				       BchstAccepting(BchstGRight(bchst0))
				       ? ((char *) bchst2) :
				       (char *) LstNext(buchi));
			LstSet(buchi, cursor);
		    }
	}
#ifdef DEBUG
    P(F, "AND\n");
    normBchst = FALSE;
    LSTFOR (buchi, bchst0, BCHST)
	BchstDump(bchst0, (unsigned) 0);
#endif
    LSTFOR (buchi, bchst0, BCHST)
	{
	    flag = BchstGVal(BchstGLeft(bchst0));
	    for (mask = atMask << 1; mask; mask >>= 2)
		if ((BchstGVal(BchstGRight(bchst0)) & mask) && !(flag & mask))
		    flag |=
			BchstGVal(BchstGRight(bchst0)) & (mask | (mask >> 1));
	    bchst1 = BchstGLeft(bchst0);
	    if (BchstInitial(bchst1) &&	BchstInitial(BchstGRight(bchst0)))
		BchstSInitial(bchst0);
	    else
		BchstCInitial(bchst0);
	    if (BchstAccepting(bchst1))
		BchstSAccepting(bchst0);
	    else
		BchstCAccepting(bchst0);
            BchstCEventual(bchst0);
	    BchstSVal(bchst0, flag);
	    bchst1 = (BCHST *) LstStep(buchi);
	    BchstCInitial(bchst1);
	    BchstCAccepting(bchst1);
            BchstCEventual(bchst1);
	    BchstSVal(bchst1, flag);
	}
    LstDestroy(bp0, TRUE);
    LstDestroy(bp1, TRUE);
    BuchiSimplify(buchi);
#ifdef DEBUG
    P(F, "AND simplified\n");
    normBchst = TRUE;
    LSTFOR (buchi, bchst0, BCHST)
	BchstDump(bchst0, (unsigned) 0);
#endif
    return(buchi);
}

/***************************************************************************
*
* Function:  ForOn
* Purpose:   This routine checks if a formula is equivalent to any
*            formula that is TRUE as specified by a bit vector.
* Arguments: formula - the formula
*            flag - the vector
*            forList - a sufficient set of reference formulas
* Returns:   TRUE if there is such equivalent formula, and FALSE otherwise
* See also:
*
****************************************************************************
*/
FS3(BOOL, ForOn,
    FORMULA *, formula,
    unsigned long, flag,
    LIST *, forList)
{
    VOID *cursor;
    FORMULA *fp;

    if (!formula)
	return(FALSE);
    if (formula->operator == FO_IMPLIES)
	{
	    if (!(formula->uf.of.sub1f))
		return(TRUE);
	    if (!(formula->uf.of.sub2f))
		{
		    cursor = LstGet(forList);
		    for (fp = (FORMULA *) LstBegin(forList); fp;
			 fp = (FORMULA *) LstStep(forList), flag >>= 1)
			if (ForIdentic(formula->uf.of.sub1f, fp))
			    break;
		    LstSet(forList, cursor);
		    if (!fp)
			{
			    P(_out, "\n");
			    (void) fflush(_out);
			    P(_err, "\nInternal error in ForOn\n");
			    (void) fflush(_err);
			    abort();
			}
		    return(!(flag & 1L));
		}
	}
    cursor = LstGet(forList);
    for (fp = (FORMULA *) LstBegin(forList); fp;
	 fp = (FORMULA *) LstStep(forList), flag >>= 1)
	if (ForIdentic(formula, fp))
	    break;
    LstSet(forList, cursor);
    if (!fp)
	{
	    P(_out, "\n");
	    (void) fflush(_out);
	    P(_err, "\nInternal error in ForOn\n");
	    (void) fflush(_err);
	    abort();
	}
    return(((unsigned long) 0L) != (flag & (unsigned long) 1L));
}

/***************************************************************************
*
* Function:  BuchiLocal
* Purpose:   This routine constructs a so called local automaton
*            corresponding to a formula. Consult from the LICS '86
*            paper of Vardi and Wolper and from the Gribomont & Wolper
*            mentioned in the reference manual.
* Arguments: buNeg - TRUE if and only if the formula is of the form
*                    "A implies false"
*            forList - a list of subformulas or their negations
*                      (the main formula or its negation is in the head)
* Returns:   the local automaton
* See also:  BuchiEventual, BuchiProd, BuchiRelabel
*
****************************************************************************
*/
FS2(LIST *, BuchiLocal,
    BOOL, buNeg,
    LIST *, forList)
{
    LIST *buchi;
    LIST *queue;
    FORMULA *fp;
    unsigned long flag;
    unsigned long mask;
    unsigned long now;
    BCHST *bchst0;
    BCHST *bchst1;

    buchi = LstCreate(&bchstTools);
    flag = (1L << LstSize(forList)) - 1L;
    while (TRUE)
	{
	    for (fp = (FORMULA *) LstBegin(forList), mask = flag;
		 fp && (fp->operator == FO_UNTIL);
		 fp = (FORMULA *) LstStep(forList), mask >>= 1)
		if (mask & 1L)
		    {
			if ((!ForOn(fp->uf.of.sub1f, flag, forList)) &&
			    !ForOn(fp->uf.of.sub2f, flag, forList))
			    break;
		    }
		else if (ForOn(fp->uf.of.sub2f, flag, forList))
		    break;
	    if (fp && (fp->operator == FO_UNTIL))
		{
		    if (!flag)
			break;
		    flag--;
		    continue;
		}
	    for (; fp && (fp->operator != FO_IMPLIES);
		 fp = (FORMULA *) LstStep(forList), mask >>= 1)
		;
	    for (; fp; fp = (FORMULA *) LstStep(forList), mask >>= 1)
		if (mask & 1L)
		    {
			if (ForOn(fp->uf.of.sub1f, flag, forList) &&
			    !ForOn(fp->uf.of.sub2f, flag, forList))
			    break;
		    }
		else if ((!ForOn(fp->uf.of.sub1f, flag, forList)) ||
			 ForOn(fp->uf.of.sub2f, flag, forList))
		    break;
	    if (!fp)
		{
		    (void) LstInsFirst(buchi, (char *) BchstCons(FALSE));
		    if (((flag & 1L) == 0L) == buNeg)
			BchstSInitial((BCHST *) LstFirst(buchi));
		    BchstSAccepting((BCHST *) LstFirst(buchi));
		    BchstSVal((BCHST *) LstFirst(buchi), flag);
		}
	    if (!flag)
		break;
	    flag--;
	}
    queue = LstCreate(&bchstTools);
    now = 0L;
    LSTFOR (buchi, bchst0, BCHST)
	{
	    now++;
	    BchstSVis(bchst0, now);
	    if (BchstInitial(bchst0))
		{
		    BchstSMarked(bchst0);
		    (void) LstInsLast(queue, (char *) (bchst0));
		}
	}
    while ((bchst0 = (BCHST *) LstDelFirst(queue, FALSE)))
	LSTFOR (buchi, bchst1, BCHST)
	    {
		for (fp = (FORMULA *) LstBegin(forList),
		     mask = 1L;
		     fp && (fp->operator == FO_UNTIL);
		     fp = (FORMULA *) LstStep(forList),
		     mask <<= 1)
		    {
			if (BchstGVal(bchst0) & mask)
			    {
				if ((!(BchstGVal(bchst1) & mask)) &&
				    !ForOn(fp->uf.of.sub2f, BchstGVal(bchst0),
					   forList))
				    break;
			    }
			else if ((BchstGVal(bchst1) & mask) &&
				 ForOn(fp->uf.of.sub1f, BchstGVal(bchst0),
				       forList))
			    break;
		    }
		if (fp && (fp->operator == FO_UNTIL))
		    continue;
		(void) LstInsLast(BchstGSucc(bchst0), (char *) bchst1);
		if (BchstMarked(bchst1))
		    continue;
		BchstSMarked(bchst1);
		(void) LstInsLast(queue, (char *) bchst1);
	    }
    LstDestroy(queue, FALSE);
#ifdef DEBUG
    P(F, "LOCAL\n");
    normBchst = TRUE;
    LSTFOR (buchi, bchst0, BCHST)
	BchstDump(bchst0, (unsigned) 0);
#endif
    BuchiSimplify(buchi);
#ifdef DEBUG
    P(F, "LOCAL SIMPLIFIED\n");
    normBchst = TRUE;
    LSTFOR (buchi, bchst0, BCHST)
	BchstDump(bchst0, (unsigned) 0);
#endif
    return(buchi);
}

/***************************************************************************
*
* Function:  BuchiEventual
* Purpose:   This routine constructs a so called eventuality automaton
*            corresponding to a formula. Consult from the LICS '86
*            paper of Vardi and Wolper and from the Gribomont & Wolper
*            mentioned in the reference manual.
* Arguments: local - a local automaton corresponding to the formula
*            forList - as in BuchiLocal
* Returns:   the eventuality automaton
* See also:  BuchiLocal, BuchiProd, BuchiRelabel
*
****************************************************************************
*/
FS2(LIST *, BuchiEventual,
    LIST *, local,
    LIST *, forList)
{
    LIST *buchi;
    LIST *queue;
    FORMULA *fp;
    unsigned long flag;
    unsigned long mask;
    unsigned long now;
    BCHST *bchst0;
    BCHST *bchst1;
    BCHST *bchst2;

    buchi = LstCreate(&bchstTools);
    for (fp = (FORMULA *) LstBegin(forList), flag = 1L;
	 fp && (fp->operator == FO_UNTIL);
	 fp = (FORMULA *) LstStep(forList), flag <<= 1)
	;
    while (--flag)
	{
	    (void) LstInsFirst(buchi, (char *) BchstCons(TRUE));
	    BchstSVal((BCHST *) LstFirst(buchi), flag);
	}
    bchst2 = BchstCons(TRUE);
    BchstSInitial(bchst2);
    BchstSAccepting(bchst2);
    BchstSMarked(bchst2);
    (void) LstInsFirst(buchi, (char *) bchst2);
    queue = LstCreate(&bchstTools);
    now = 0L;
    LSTFOR (buchi, bchst0, BCHST)
	{
	    now++;
	    BchstSVis(bchst0, now);
	}
    LSTFOR (buchi, bchst0, BCHST)
	LSTFOR (local, bchst1, BCHST)
	    {
		for (fp = (FORMULA *) LstBegin(forList), mask = 1L;
		     fp && (fp->operator == FO_UNTIL);
		     fp = (FORMULA *) LstStep(forList), mask <<= 1)
		    {
			if (!(BchstGVal(bchst1) & mask))
			    continue;
			if (ForOn(fp->uf.of.sub2f, BchstGVal(bchst1), forList)
			    == (0L != (BchstGVal(bchst0) & mask)))
			    break;
		    }
		if (fp && (fp->operator == FO_UNTIL))
		    continue;
		(void) LstInsLast(BchstGSucc(bchst2), (char *)
				  BchtrCons(BchstGVal(bchst1), bchst0));
		if (BchstMarked(bchst0))
		    continue;
		BchstSMarked(bchst0);
		(void) LstInsLast(queue, (char *) (bchst0));
	    }
    while ((bchst2 = (BCHST *) LstDelFirst(queue, FALSE)))
	LSTFOR (buchi, bchst0, BCHST)
	    LSTFOR (local, bchst1, BCHST)
		{
		    for (fp = (FORMULA *) LstBegin(forList),
			 mask = 1L;
			 fp && (fp->operator == FO_UNTIL);
			 fp = (FORMULA *) LstStep(forList),
			 mask <<= 1)
			{
			    if (!(BchstGVal(bchst2) & mask))
				continue;
			    if (ForOn(fp->uf.of.sub2f, BchstGVal(bchst1), forList)
				== (0L != (BchstGVal(bchst0) & mask)))
				break;
			}
		    if (fp && (fp->operator == FO_UNTIL))
			continue;
		    (void) LstInsLast(BchstGSucc(bchst2), (char *)
				      BchtrCons(BchstGVal(bchst1), bchst0));
		    if (BchstMarked(bchst0))
			continue;
		    BchstSMarked(bchst0);
		    (void) LstInsLast(queue, (char *) bchst0);
		}
    LstDestroy(queue, FALSE);
#ifdef DEBUG
    P(F, "EVENTUAL\n");
    normBchst = TRUE;
    LSTFOR (buchi, bchst0, BCHST)
	BchstDump(bchst0, (unsigned) 0);
#endif
    BuchiSimplify(buchi);
#ifdef DEBUG
    P(F, "EVENTUAL SIMPLIFIED\n");
    normBchst = TRUE;
    LSTFOR (buchi, bchst0, BCHST)
	BchstDump(bchst0, (unsigned) 0);
#endif
    return(buchi);
}

/***************************************************************************
*
* Function:  BuchiProd
* Purpose:   This routine constructs the product of a local automaton
*            and an eventuality automaton corresponding to a formula.
*            Consult from the LICS '86 paper of Vardi and Wolper and from
*            the Gribomont & Wolper mentioned in the reference manual.
* Arguments: local - the local automaton
*            eventual - the eventuality automaton
* Returns:   the product
* See also:  BuchiLocal, BuchiEventual, BuchiRelabel
*
****************************************************************************
*/
FS2(LIST *, BuchiProd,
    LIST *, local,
    LIST *, eventual)
{
    LIST *buchi;
    VOID *cursor;
    unsigned long flag;
    BCHST *bchst0;
    BCHST *bchst1;
    BCHST *bchst2;
    BCHTR *bchtr;

    buchi = LstCreate(&bchstTools);
    LSTFOR (local, bchst0, BCHST)
	LSTFOR (eventual, bchst1, BCHST)
	    {
		LSTFOR (BchstGSucc(bchst1), bchtr, BCHTR)
		    if (BchtrGVal(bchtr) == BchstGVal(bchst0))
			break;
		if (!bchtr)
		    continue;
		(void) LstInsLast(buchi, (char *) BchstCons(FALSE));
		BchstSLeft((BCHST *) LstLast(buchi), bchst0);
		BchstSRight((BCHST *) LstLast(buchi), bchst1);
	    }
    LSTFOR (buchi, bchst0, BCHST)
	LSTFOR (BchstGSucc(BchstGLeft(bchst0)), bchst2, BCHST)
	    LSTFOR (BchstGSucc(BchstGRight(bchst0)), bchtr, BCHTR)
		{
		    if (BchtrGVal(bchtr) != BchstGVal(BchstGLeft(bchst0)))
			continue;
		    cursor = LstGet(buchi);
		    LSTFOR (buchi, bchst1, BCHST)
			if ((BchstGLeft(bchst1) == bchst2) &&
			    (BchstGRight(bchst1) == BchtrGBchst(bchtr)))
			    break;
		    if (bchst1)
			(void)
			    LstInsLast(BchstGSucc(bchst0), (char *) bchst1);
		    LstSet(buchi, cursor);
		}
#ifdef DEBUG
    P(F, "PRODUCT\n");
    normBchst = FALSE;
    LSTFOR (buchi, bchst0, BCHST)
	BchstDump(bchst0, (unsigned) 0);
#endif
    LSTFOR (buchi, bchst0, BCHST)
	{
	    flag = BchstGVal(BchstGLeft(bchst0));
	    if (BchstGRight(bchst0) == (BCHST *) LstFirst(eventual))
		{
		    if (BchstInitial(BchstGLeft(bchst0)))
			BchstSInitial(bchst0);
		    else
			BchstCInitial(bchst0);
		    BchstSAccepting(bchst0);
		}
	    else
		{
		    BchstCInitial(bchst0);
		    BchstCAccepting(bchst0);
		}
            BchstCEventual(bchst0);
	    BchstSVal(bchst0, flag);
	}
    LstDestroy(local, TRUE);
    LstDestroy(eventual, TRUE);
    BuchiSimplify(buchi);
#ifdef DEBUG
    P(F, "PRODUCT SIMPLIFIED\n");
    normBchst = TRUE;
    LSTFOR (buchi, bchst0, BCHST)
	BchstDump(bchst0, (unsigned) 0);
#endif
    return(buchi);
}

/***************************************************************************
*
* Function:  BuchiRelabel
* Purpose:   This routine relabels the product of a local automaton
*            and an eventuality automaton corresponding to a formula.
*            Consult from the LICS '86 paper of Vardi and Wolper and from
*            the Gribomont & Wolper mentioned in the reference manual.
* Arguments: buchi - the product
*            forList - as in BuchiLocal
*            atomList - all atomic formulas in the user's formula
* Returns:
* See also:  BuchiLocal, BuchiEventual, BuchiProd
*
****************************************************************************
*/
FS3(void, BuchiRelabel,
    LIST *, buchi,
    LIST *, forList,
    LIST *, atomList)
{
    VOID *cursor;
    FORMULA *fp;
    BCHST *bchst;
    unsigned i;
    unsigned long flag;
    unsigned long mask0;
    unsigned long mask1;
    unsigned long exprStart;

    for (fp = (FORMULA *) LstBegin(forList), exprStart = 1L;
	 fp && (fp->operator != FO_EXPR);
	 fp = (FORMULA *) LstStep(forList), exprStart <<= 1)
	;
    cursor = LstGet(forList);
    LSTFOR (buchi, bchst, BCHST)
	{
	    flag = 0L;
	    for (LstSet(forList, cursor),
		 fp = (FORMULA *) LstThis(forList),
		 mask0 = exprStart;
		 fp && (fp->operator == FO_EXPR);
		 fp = (FORMULA *) LstStep(forList), mask0 <<= 1)
		{
		    i = LstWhich(atomList, (char *) fp);
		    if (!i)
			{
			    P(_out, "\n");
			    (void) fflush(_out);
			    P(_err, "\nInternal error in BuchiRelabel\n");
			    (void) fflush(_err);
			    abort();
			}
		    mask1 = ((unsigned long) 2L) << (2 * (i - 1));
		    if (BchstGVal(bchst) & mask0)
			flag |= mask1 | (mask1 >> 1);
		    else
			flag |= mask1;
		}
	    BchstSVal(bchst, flag);
	}
#ifdef DEBUG
    P(F, "RELABELLED\n");
    normBchst = TRUE;
    LSTFOR (buchi, bchst, BCHST)
	BchstDump(bchst, (unsigned) 0);
#endif
}

/***************************************************************************
*
* Function:  ForClosure
* Purpose:   This routine computes the closure of a formula as defined
*            in the LICS '86 paper of Vardi and Wolper.
* Arguments: formula - the formula (must be in a reduced form)
*            forList - subformulas or their negations are put here
*            atomList - all atomic formulas in the user's formula
* Returns:
* See also:
*
****************************************************************************
*/
FE3(void, ForClosure,
    FORMULA *, formula,
    LIST *, forList,
    LIST *, atomList)
{
    VOID *cursor;
    FORMULA *fp[2];

    if ((!formula) ||
	((formula->operator == FO_IMPLIES) && !(formula->uf.of.sub1f)))
	return;
    cursor = LstGet(forList);
    LSTFOR (forList, fp[0], FORMULA)
	if (ForIdentic(fp[0], formula))
	    {
		LstSet(forList, cursor);
		return;
	    }
    LstSet(forList, cursor);
    if (formula->operator == FO_UNTIL)
	{
	    if (atomList)
		{
		    if (LstEmpty(forList) ||
			(((FORMULA *) LstFirst(forList))->operator
			 != FO_UNTIL))
			(void) LstInsFirst(forList, (char *) formula);
		    else if (((FORMULA *) LstLast(forList))->operator
			     == FO_UNTIL)
			(void) LstInsLast(forList, (char *) formula);
		    else
			{
			    cursor = LstGet(forList);
			    LSTFOR (forList, fp[0], FORMULA)
				if ((fp[0]->operator == FO_UNTIL) &&
				    (((FORMULA *) LstNext(forList))->operator
				     != FO_UNTIL))
				    break;
			    (void) LstInsNext(forList, (char *) formula);
			    LstSet(forList, cursor);
			}
		}
	}
    else if (formula->operator == FO_EXPR)
	{
	    if (atomList)
		{
		    cursor = LstGet(atomList);
		    LSTFOR (atomList, fp[1], FORMULA)
			if (ForIdentic(fp[1], formula))
			    break;
		    LstSet(atomList, cursor);
		    if (!fp[1])
			{
			    P(_out, "\n");
			    (void) fflush(_out);
			    P(_err, "\nInternal error in ForClosure\n");
			    (void) fflush(_err);
			    abort();
			}
		}
	    else
		fp[1] = formula;
	    if (LstEmpty(forList) ||
		(((FORMULA *) LstFirst(forList))->operator != FO_UNTIL))
		{
		    (void) LstInsFirst(forList, (char *) fp[1]);
		    return;
		}
	    if (((FORMULA *) LstLast(forList))->operator != FO_IMPLIES)
		{
		    (void) LstInsLast(forList, (char *) fp[1]);
		    return;
		}
	    cursor = LstGet(forList);
	    LSTFOR (forList, fp[0], FORMULA)
		if ((fp[0]->operator == FO_UNTIL) &&
		    (((FORMULA *) LstNext(forList))->operator != FO_UNTIL))
		    break;
	    (void) LstInsNext(forList, (char *) fp[1]);
	    LstSet(forList, cursor);
	    return;
	}
    else if (formula->uf.of.sub2f)
	{
	    if (atomList)
		(void) LstInsLast(forList, (char *) formula);
	}
    ForClosure(formula->uf.of.sub1f, forList, atomList);
    if (formula->uf.of.sub2f)
	ForClosure(formula->uf.of.sub2f, forList, atomList);
}

/***************************************************************************
*
* Function:  ForBuchi
* Purpose:   This routine constructs a Buchi automaton corresponding to
*            a formula. In other words, the set of infinite words accepted
*            by the automaton consists of exactly those sequences that
*            satisfy the formula. The construction is compositional as
*            far as we can proceed by computing unions and intersections.
*            To get a full advantage of this approach, the formula
*            has earlier been reduced by using the rules of the CAV '93
*            paper of Peled. (This compositional approach seemed nice
*            until a much more efficient algorithm was published
*            in PSTV '95 by Gerth, Peled, Vardi and Wolper.)
* Arguments: formula - the formula (must be in a reduced form)
*            atomList - all atomic formulas in the user's formula
*            atMask - a mask determined by the number of atomic formulas
* Returns:   the automaton
* See also:
*
****************************************************************************
*/
FE3(LIST *, ForBuchi,
    FORMULA *, formula,
    LIST *, atomList,
    unsigned long, atMask)
{
    LIST *buchi;
    LIST *forList;
    TRANSITION *tran;
    unsigned long *lng;
#ifdef DEBUG
    FORMULA *fp;

    pxFile = _err;
    ForPrint(formula);
    P(F, "\n");
#endif
    if (!formula)
	return(LstCreate(&bchstTools));
    if ((formula->operator == FO_IMPLIES) && formula->uf.of.sub1f &&
	(formula->uf.of.sub2f || (formula->uf.of.sub1f->operator == FO_IMPLIES)))
	{
	    if (formula->uf.of.sub2f)
		{
		    buchi = ForBuchi(formula->uf.of.sub2f, atomList, atMask);
		    ForDracsid(formula->uf.of.sub2f);
		    if ((formula->uf.of.sub1f->operator == FO_IMPLIES) &&
			!(formula->uf.of.sub1f->uf.of.sub2f))
			{
			    formula->uf.of.sub2f = formula->uf.of.sub1f->uf.of.sub1f;
			    formula->uf.of.sub1f->uf.of.sub1f = NULL;
			    ForDracsid(formula->uf.of.sub1f);
			    buchi =
				BuchiOr(buchi,
					ForBuchi(formula->uf.of.sub2f,
						 atomList, atMask));
			    ForDracsid(formula->uf.of.sub2f);
			}
		    else
			{
			    formula->uf.of.sub1f =
				ForSnoc(FO_IMPLIES, formula->uf.of.sub1f,
					(FORMULA *) NULL);
			    buchi =
				BuchiOr(buchi,
					ForBuchi(formula->uf.of.sub1f,
						 atomList, atMask));
			    ForDracsid(formula->uf.of.sub1f);
			}
		    formula->uf.of.sub1f = NULL;
		    formula->uf.of.sub2f = NULL;
		    return(buchi);
		}
	    if ((formula->uf.of.sub1f->uf.of.sub2f->operator == FO_IMPLIES) &&
		!(formula->uf.of.sub1f->uf.of.sub2f->uf.of.sub2f))
		{
		    formula->uf.of.sub2f = formula->uf.of.sub1f->uf.of.sub2f->uf.of.sub1f;
		    formula->uf.of.sub1f->uf.of.sub2f->uf.of.sub1f = NULL;
		    ForDracsid(formula->uf.of.sub1f->uf.of.sub2f);
		    buchi = ForBuchi(formula->uf.of.sub2f, atomList, atMask);
		    ForDracsid(formula->uf.of.sub2f);
		    formula->uf.of.sub2f = NULL;
		    buchi = BuchiAnd(buchi,
				     ForBuchi(formula->uf.of.sub1f->uf.of.sub1f,
					      atomList, atMask), atMask);
		    ForDracsid(formula->uf.of.sub1f->uf.of.sub1f);
		}
	    else
		{
		    buchi = ForBuchi(formula->uf.of.sub1f->uf.of.sub1f,
				     atomList, atMask);
		    ForDracsid(formula->uf.of.sub1f->uf.of.sub1f);
		    formula->uf.of.sub1f->uf.of.sub2f =
			ForSnoc(FO_IMPLIES, formula->uf.of.sub1f->uf.of.sub2f,
				(FORMULA *) NULL);
		    buchi = BuchiAnd(buchi,
				     ForBuchi(formula->uf.of.sub1f->uf.of.sub2f,
					      atomList, atMask), atMask);
		    ForDracsid(formula->uf.of.sub1f->uf.of.sub2f);
		}
	    formula->uf.of.sub1f->uf.of.sub1f = NULL;
	    formula->uf.of.sub1f->uf.of.sub2f = NULL;
	    ForDracsid(formula->uf.of.sub1f);
	    formula->uf.of.sub1f = NULL;
	    return(buchi);
	}
    lng = LngCons(newBlock);
    LSTFOR (TransLst, tran, TRANSITION)
        if (ForAffected(formula, tran))
            (void) LstInsLast(TraGBlocks(tran), (char *) lng);
    newBlock++;
    forList = LstCreate(&forSloot);
    ForClosure(formula, forList, atomList);
#ifdef DEBUG
    LSTFOR (forList, fp, FORMULA)
	{
	    ForPrint(fp);
	    P(F, "\n");
	}
#endif
    buchi = BuchiLocal((formula->operator == FO_IMPLIES) &&
		       !(formula->uf.of.sub2f), forList);
    buchi = BuchiProd(buchi, BuchiEventual(buchi, forList));
    BuchiRelabel(buchi, forList, atomList);
    LstDestroy(forList, FALSE);
    return(buchi);
}

FE2(BOOL, ForAffected,
    FORMULA *, formula,
    TRANSITION *, transition)
{
    if (!formula)
        return(FALSE);
    if (formula->operator == FO_EXPR)
        return(XprAffected(formula->uf.expression, transition));
    return(ForAffected(formula->uf.of.sub1f, transition) ||
           ForAffected(formula->uf.of.sub2f, transition));
}
