/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  ptandor.c
*
* Part of: PROD's programs araprod and "gphgen"
*
* Author:  Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     The routines in this module are needed for constructing
*     an and/or-graph in the deletion algorithm of the stubborn set method.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ptrbl.h"
#include "baltree.h"
#include "bdd.h"
#include "gphutil.h"
#include "graph.h"
#include "mark.h"
#include "encode.h"
#include "ptnet.h"
#include "ptstable.h"

/***************************************************************************
*
* Function:  TrnOrPrIn
* Purpose:   This is an auxiliary routine of TrnAndOr.
* Arguments: trn - a transition
* Returns:
* See also:  TrnAndOr
*
****************************************************************************
*/
FE1(void, TrnOrPrIn,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        }
    LSTFOR (TrnGPurIn(trn), ptTrArc, TRARC)
	{
	    ptPlc = TrArcGPlc(ptTrArc);
	    if (
#ifdef SMODELS
		(!PtLongPlay()) &&
		(
#endif
		 PlcGCount(ptPlc) >= TrArcGWeight(ptTrArc)
#ifdef SMODELS
		 )
#endif
		)
		continue;
#ifdef SMODELS
	    if (PtSmodels())
		{
		    if (PtLongPlay())
			{
			    ptIneq = MkPlcIneq(ptPlc, TrArcGWeight(ptTrArc));
			    StartRule(TrArcGGamma(ptTrArc));
			    ContRule(IneqGLt(ptIneq));
			    ContRule(PlcGGamma(ptPlc));
			    EndRule();
			    StartRule(TrArcGGamma(ptTrArc));
			    ContRule(IneqGGe(ptIneq));
			    NegContRule(TrnGActive(trn));
			    EndRule();
			}
		    if (PlcSatisfied(ptPlc))
			continue;
		    PlcFeed(ptPlc);
		}
	    else
		{
#endif
		    if (PlcGOrPredAnd(ptPlc))
			{
			    (void) LstInsLast(PlcGOrPredAnd(ptPlc),
					      (char *) trn);
			    continue;
			}
		    PlcSOrPredAnd(ptPlc);
		    (void) LstInsLast(PlcGOrPredAnd(ptPlc), (char *) trn);
#ifdef SMODELS
		}
#endif
	    if (PlcGPurIn(ptPlc))
		LSTFOR (PlcGPurIn(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGPurIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return;
L1: ;
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
                                LstSet(PlcGPurIn(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGPurIn(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(PlcGGamma(ptPlc));
				if (PtLongPlay())
				    ContRule(IneqGLt(MkPlcIneq(ptPlc,
							       PlcGOutMax(ptPlc)
							       )));
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
			    {
#endif
				if (!TrnGPlcPredAO(ptTrn))
				    TrnSPlcPredAO(ptTrn);
				(void) LstInsLast(TrnGPlcPredAO(ptTrn),
						  (char *) ptPlc);
#ifdef SMODELS
			    }
#endif
		    }
	    ptLimit = PlcGCount(ptPlc);
	    if (PlcGMosIn(ptPlc))
		LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
		    {
			if (
#ifdef SMODELS
			    PtLongPlay() ?
			    (PlDrcGOut(ptPlDrc) == PlcGOutMax(ptPlc)) :
			    (
#endif
			     PlDrcGOut(ptPlDrc) > ptLimit
#ifdef SMODELS
			     )
#endif
			    )
			    continue;
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGMosIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return;
L2: ;
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
				ptLimit = PlcGCount(ptPlc);
                                LstSet(PlcGMosIn(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosIn(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(PlcGGamma(ptPlc));
				if (PtLongPlay())
				    {
					ContRule(IneqGLt(MkPlcIneq(ptPlc,
								   PlcGOutMax(ptPlc)
								   )));
					ContRule(IneqGGe(MkPlcIneq(ptPlc,
								   PlDrcGOut(ptPlDrc)
								   )));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
			    {
#endif
				if (!TrnGPlcPredAO(ptTrn))
				    TrnSPlcPredAO(ptTrn);
				(void) LstInsLast(TrnGPlcPredAO(ptTrn),
						  (char *) ptPlc);
#ifdef SMODELS
			    }
#endif
		    }
	}
}

/***************************************************************************
*
* Function:  TrnOrNeutral
* Purpose:   This is an auxiliary routine of TrnAndOr.
* Arguments: trn - a transition
* Returns:
* See also:  TrnAndOr
*
****************************************************************************
*/
FE1(void, TrnOrNeutral,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        }
    LSTFOR (TrnGNeutral(trn), ptTrArc, TRARC)
	{
	    ptPlc = TrArcGPlc(ptTrArc);
	    if (
#ifdef SMODELS
		(!PtLongPlay()) &&
		(
#endif
		 PlcGCount(ptPlc) >= TrArcGWeight(ptTrArc)
#ifdef SMODELS
		 )
#endif
		)
		continue;
#ifdef SMODELS
	    if (PtSmodels())
		{
		    if (PtLongPlay())
			{
			    ptIneq = MkPlcIneq(ptPlc, TrArcGWeight(ptTrArc));
			    StartRule(TrArcGGamma(ptTrArc));
			    ContRule(IneqGLt(ptIneq));
			    ContRule(PlcGGamma(ptPlc));
			    EndRule();
			    StartRule(TrArcGGamma(ptTrArc));
			    ContRule(IneqGGe(ptIneq));
			    NegContRule(TrnGActive(trn));
			    EndRule();
			}
		    if (PlcSatisfied(ptPlc))
			continue;
		    PlcFeed(ptPlc);
		}
	    else
		{
#endif
		    if (PlcGOrPredAnd(ptPlc))
			{
			    (void) LstInsLast(PlcGOrPredAnd(ptPlc),
					      (char *) trn);
			    continue;
			}
		    PlcSOrPredAnd(ptPlc);
		    (void) LstInsLast(PlcGOrPredAnd(ptPlc), (char *) trn);
#ifdef SMODELS
		}
#endif
	    if (PlcGPurIn(ptPlc))
		LSTFOR (PlcGPurIn(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGPurIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return;
L1: ;
				ptTrArc = (TRARC *)
				    LstThis(TrnGNeutral(trn));
				ptPlc = TrArcGPlc(ptTrArc);
                                LstSet(PlcGPurIn(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGPurIn(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(PlcGGamma(ptPlc));
				if (PtLongPlay())
				    ContRule(IneqGLt(MkPlcIneq(ptPlc,
							       PlcGOutMax(ptPlc)
							       )));
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
			    {
#endif
				if (!TrnGPlcPredAO(ptTrn))
				    TrnSPlcPredAO(ptTrn);
				(void) LstInsLast(TrnGPlcPredAO(ptTrn),
						  (char *) ptPlc);
#ifdef SMODELS
			    }
#endif
		    }
	    ptLimit = PlcGCount(ptPlc);
	    if (PlcGMosIn(ptPlc))
		LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
		    {
			if (
#ifdef SMODELS
			    PtLongPlay() ?
			    (PlDrcGOut(ptPlDrc) == PlcGOutMax(ptPlc)) :
			    (
#endif
			     PlDrcGOut(ptPlDrc) > ptLimit
#ifdef SMODELS
			     )
#endif
			    )
			    continue;
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGMosIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return;
L2: ;
				ptTrArc = (TRARC *)
				    LstThis(TrnGNeutral(trn));
				ptPlc = TrArcGPlc(ptTrArc);
				ptLimit = PlcGCount(ptPlc);
                                LstSet(PlcGMosIn(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosIn(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(PlcGGamma(ptPlc));
				if (PtLongPlay())
				    {
					ContRule(IneqGLt(MkPlcIneq(ptPlc,
								   PlcGOutMax(ptPlc)
								   )));
					ContRule(IneqGGe(MkPlcIneq(ptPlc,
								   PlDrcGOut(ptPlDrc)
								   )));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
			    {
#endif
				if (!TrnGPlcPredAO(ptTrn))
				    TrnSPlcPredAO(ptTrn);
				(void) LstInsLast(TrnGPlcPredAO(ptTrn),
						  (char *) ptPlc);
#ifdef SMODELS
			    }
#endif
		    }
	}
}

/***************************************************************************
*
* Function:  TrnOrMsIn
* Purpose:   This is an auxiliary routine of TrnAndOr.
* Arguments: trn - a transition
* Returns:
* See also:  TrnAndOr
*
****************************************************************************
*/
FE1(void, TrnOrMsIn,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        }
    LSTFOR (TrnGMosIn(trn), ptTrDrc, TRDRC)
	{
	    ptPlc = TrDrcGPlc(ptTrDrc);
	    if (
#ifdef SMODELS
		(!PtLongPlay()) &&
		(
#endif
		 PlcGCount(ptPlc) >= TrDrcGIn(ptTrDrc)
#ifdef SMODELS
		 )
#endif
		)
		continue;
#ifdef SMODELS
	    if (PtSmodels())
		{
		    if (PtLongPlay())
			{
			    ptIneq = MkPlcIneq(ptPlc, TrDrcGIn(ptTrDrc));
			    StartRule(TrDrcGGamma(ptTrDrc));
			    ContRule(IneqGLt(ptIneq));
			    ContRule(PlcGGamma(ptPlc));
			    EndRule();
			    StartRule(TrDrcGGamma(ptTrDrc));
			    ContRule(IneqGGe(ptIneq));
			    NegContRule(TrnGActive(trn));
			    EndRule();
			}
		    if (PlcSatisfied(ptPlc))
			continue;
		    PlcFeed(ptPlc);
		}
	    else
		{
#endif
		    if (PlcGOrPredAnd(ptPlc))
			{
			    (void) LstInsLast(PlcGOrPredAnd(ptPlc),
					      (char *) trn);
			    continue;
			}
		    PlcSOrPredAnd(ptPlc);
		    (void) LstInsLast(PlcGOrPredAnd(ptPlc), (char *) trn);
#ifdef SMODELS
		}
#endif
	    if (PlcGPurIn(ptPlc))
		LSTFOR (PlcGPurIn(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGPurIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return;
L1: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosIn(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
                                LstSet(PlcGPurIn(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGPurIn(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(PlcGGamma(ptPlc));
				if (PtLongPlay())
				    ContRule(IneqGLt(MkPlcIneq(ptPlc,
							       PlcGOutMax(ptPlc)
							       )));
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
			    {
#endif
				if (!TrnGPlcPredAO(ptTrn))
				    TrnSPlcPredAO(ptTrn);
				(void) LstInsLast(TrnGPlcPredAO(ptTrn),
						  (char *) ptPlc);
#ifdef SMODELS
			    }
#endif
		    }
	    ptLimit = PlcGCount(ptPlc);
	    if (PlcGMosIn(ptPlc))
		LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
		    {
			if (
#ifdef SMODELS
			    PtLongPlay() ?
			    (PlDrcGOut(ptPlDrc) == PlcGOutMax(ptPlc)) :
			    (
#endif
			     PlDrcGOut(ptPlDrc) > ptLimit
#ifdef SMODELS
			     )
#endif
			    )
			    continue;
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGMosIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return;
L2: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosIn(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
				ptLimit = PlcGCount(ptPlc);
                                LstSet(PlcGMosIn(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosIn(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(PlcGGamma(ptPlc));
				if (PtLongPlay())
				    {
					ContRule(IneqGLt(MkPlcIneq(ptPlc,
								   PlcGOutMax(ptPlc)
								   )));
					ContRule(IneqGGe(MkPlcIneq(ptPlc,
								   PlDrcGOut(ptPlDrc)
								   )));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
			    {
#endif
				if (!TrnGPlcPredAO(ptTrn))
				    TrnSPlcPredAO(ptTrn);
				(void) LstInsLast(TrnGPlcPredAO(ptTrn),
						  (char *) ptPlc);
#ifdef SMODELS
			    }
#endif
		    }
	}
}

/***************************************************************************
*
* Function:  TrnOrMsOut
* Purpose:   This is an auxiliary routine of TrnAndOr.
* Arguments: trn - a transition
* Returns:
* See also:  TrnAndOr
*
****************************************************************************
*/
FE1(void, TrnOrMsOut,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        }
    LSTFOR (TrnGMosOut(trn), ptTrDrc, TRDRC)
	{
	    ptPlc = TrDrcGPlc(ptTrDrc);
	    if (
#ifdef SMODELS
		(!PtLongPlay()) &&
		(
#endif
		 PlcGCount(ptPlc) >= TrDrcGIn(ptTrDrc)
#ifdef SMODELS
		 )
#endif
		)
		continue;
#ifdef SMODELS
	    if (PtSmodels())
		{
		    if (PtLongPlay())
			{
			    ptIneq = MkPlcIneq(ptPlc, TrDrcGIn(ptTrDrc));
			    StartRule(TrDrcGGamma(ptTrDrc));
			    ContRule(IneqGLt(ptIneq));
			    ContRule(PlcGGamma(ptPlc));
			    EndRule();
			    StartRule(TrDrcGGamma(ptTrDrc));
			    ContRule(IneqGGe(ptIneq));
			    NegContRule(TrnGActive(trn));
			    EndRule();
			}
		    if (PlcSatisfied(ptPlc))
			continue;
		    PlcFeed(ptPlc);
		}
	    else
		{
#endif
		    if (PlcGOrPredAnd(ptPlc))
			{
			    (void) LstInsLast(PlcGOrPredAnd(ptPlc),
					      (char *) trn);
			    continue;
			}
		    PlcSOrPredAnd(ptPlc);
		    (void) LstInsLast(PlcGOrPredAnd(ptPlc), (char *) trn);
#ifdef SMODELS
		}
#endif
	    if (PlcGPurIn(ptPlc))
		LSTFOR (PlcGPurIn(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGPurIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return;
L1: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosOut(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
                                LstSet(PlcGPurIn(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGPurIn(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(PlcGGamma(ptPlc));
				if (PtLongPlay())
				    ContRule(IneqGLt(MkPlcIneq(ptPlc,
							       PlcGOutMax(ptPlc)
							       )));
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
			    {
#endif
				if (!TrnGPlcPredAO(ptTrn))
				    TrnSPlcPredAO(ptTrn);
				(void) LstInsLast(TrnGPlcPredAO(ptTrn),
						  (char *) ptPlc);
#ifdef SMODELS
			    }
#endif
		    }
	    ptLimit = PlcGCount(ptPlc);
	    if (PlcGMosIn(ptPlc))
		LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
		    {
			if (
#ifdef SMODELS
			    PtLongPlay() ?
			    (PlDrcGOut(ptPlDrc) == PlcGOutMax(ptPlc)) :
			    (
#endif
			     PlDrcGOut(ptPlDrc) > ptLimit
#ifdef SMODELS
			     )
#endif
			    )
			    continue;
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGMosIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return;
L2: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosOut(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
				ptLimit = PlcGCount(ptPlc);
                                LstSet(PlcGMosIn(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosIn(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(PlcGGamma(ptPlc));
				if (PtLongPlay())
				    {
					ContRule(IneqGLt(MkPlcIneq(ptPlc,
								   PlcGOutMax(ptPlc)
								   )));
					ContRule(IneqGGe(MkPlcIneq(ptPlc,
								   PlDrcGOut(ptPlDrc)
								   )));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
			    {
#endif
				if (!TrnGPlcPredAO(ptTrn))
				    TrnSPlcPredAO(ptTrn);
				(void) LstInsLast(TrnGPlcPredAO(ptTrn),
						  (char *) ptPlc);
#ifdef SMODELS
			    }
#endif
		    }
	}
}

/***************************************************************************
*
* Function:  TrnOA
* Purpose:   This is an auxiliary routine of TrnAndOr.
* Arguments: trn - a transition
* Returns:
* See also:  TrnAndOr
*
****************************************************************************
*/
FE1(void, TrnOA,
    TRN *, trn)
{
    switch (TrnGWhere(trn) >> 3)
        {
        case 1:
                TrnSWhere(trn, TrnGWhere(trn) & 7);
                goto L1;
        case 2:
                TrnSWhere(trn, TrnGWhere(trn) & 7);
                goto L2;
        case 3:
                TrnSWhere(trn, TrnGWhere(trn) & 7);
                goto L3;
        case 4:
                TrnSWhere(trn, TrnGWhere(trn) & 7);
                goto L4;
        case 5:
                TrnSWhere(trn, TrnGWhere(trn) & 7);
                goto L5;
        case 6:
                TrnSWhere(trn, TrnGWhere(trn) & 7);
                goto L6;
        }
    TrnEnter(trn);
#ifdef SMODELS
    if (PtSmodels())
	{
	    if (TrnInitial(trn) || TrnDummy(trn))
		return;
	    if (!TrnGGamma(trn))
		{
		    MkTrnGamma(trn);
		    if (PtLongPlay() || TrnEnabled(trn))
			{
			    MkTrnPlain(trn);
			    if (PtLongPlay() &&
				((TrnGPurIn(trn) &&
				  !LstEmpty(TrnGPurIn(trn))) ||
				 (TrnGNeutral(trn) &&
				  !LstEmpty(TrnGNeutral(trn))) ||
				 (TrnGMosIn(trn) &&
				  !LstEmpty(TrnGMosIn(trn))) ||
				 (TrnGMosOut(trn) &&
				  !LstEmpty(TrnGMosOut(trn)))))
				{
				    MkTrnActive(trn);
				    StartRule(TrnGPlain(trn));
				    ContRule(TrnGActive(trn));
				    NegContRule(TrnGGamma(trn));
				    EndRule();
				    StartRule(TrnGGamma(trn));
				    ContRule(TrnGActive(trn));
				    NegContRule(TrnGPlain(trn));
				    EndRule();
				}
			    else
				ReverseRules(TrnGPlain(trn), TrnGGamma(trn));
			    if (((!OnTheFly()) && !PtCFFD()) ||
				!TrnVisible(trn))
				MkTrnKappa(trn);
			    if ((OnTheFly() || PtCFFD()) &&
				(Blocked() || !TrnVisible(trn)))
				{
				    if (PtLongPlay())
					{
					    if ((TrnGPurIn(trn) &&
						 !LstEmpty(TrnGPurIn(trn))) ||
						(TrnGPurOut(trn) &&
						 !LstEmpty(TrnGPurOut(trn))) ||
						(TrnGMosIn(trn) &&
						 !LstEmpty(TrnGMosIn(trn))) ||
						(TrnGMosOut(trn) &&
						 !LstEmpty(TrnGMosOut(trn))))
						MkTrnToStack(trn);
					}
				    else
					{
					    SetPtRecurs();
					    TrnLookStack(trn);
					    if (!PtRecurs())
						TrnSalt(trn);
					}
				}
			}
		}
	}
#endif
    if (
#ifdef SMODELS
	PtLongPlay() ||
#endif
	TrnEnabled(trn)
	)
	{
	    if (TrnGPurIn(trn) && !LstEmpty(TrnGPurIn(trn)))
                {
L1: ;
                    TrnNdPrIn(trn);
                    if (TrnGWhere(trn))
                        {
                            TrnSWhere(trn, TrnGWhere(trn) | (1 << 3));
                            return;
                        }
                }
	    if (TrnGMosIn(trn) && !LstEmpty(TrnGMosIn(trn)))
                {
L2: ;
                    TrnNdMsIn(trn);
                    if (TrnGWhere(trn))
                        {
                            TrnSWhere(trn, TrnGWhere(trn) | (2 << 3));
                            return;
                        }
                }
#ifdef SMODELS
	    if (!PtLongPlay())
#endif
		return;
	}
#ifdef SMODELS
    if (PtLongPlay())
	ptM = (unsigned long) 0L;
 START: ;
    if (PtSmodels())
	{
	    if ((!PtLongPlay()) ||
		((((unsigned long) 0L) == ptM) && TrnGActive(trn)))
		StartRule(TrnGGamma(trn));
	}
    else
#endif
	TrnCNOrSucc(trn);
    if (TrnGPurIn(trn))
	LSTFOR (TrnGPurIn(trn), ptTrArc, TRARC)
	    {
		ptPlc = TrArcGPlc(ptTrArc);
		if (
#ifdef SMODELS
		    (!PtLongPlay()) &&
		    (
#endif
		     PlcGCount(ptPlc) >= TrArcGWeight(ptTrArc)
#ifdef SMODELS
		     )
#endif
		    )
		    continue;
#ifdef SMODELS
		if (PtSmodels())
		    {
			if (!PlcGGamma(ptPlc))
			    MkPlcGamma(ptPlc);
			if (PtLongPlay())
			    {
				if (((unsigned long) 0L) == ptM)
				    {
					MkTAGamma(trn, ptTrArc);
					ContRule(TrArcGGamma(ptTrArc));
				    }
				else
				    ContRule(IneqGGe(MkPlcIneq(ptPlc,
							       TrArcGWeight(ptTrArc)
							       )));
			    }
			else
			    ContRule(PlcGGamma(ptPlc));
		    }
		else
		    {
#endif
			TrnINOrSucc(trn);
			if ((!PlcGPurIn(ptPlc)) || LstEmpty(PlcGPurIn(ptPlc)))
			    {
				if (!PlcGMosIn(ptPlc))
				    return;
				ptLimit = PlcGCount(ptPlc);
				LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
				    {
					if (PlDrcGOut(ptPlDrc) <= ptLimit)
					    break;
				    }
				if (!ptPlDrc)
				    return;
			    }
#ifdef SMODELS
		    }
#endif
	    }
    if (TrnGNeutral(trn))
	LSTFOR (TrnGNeutral(trn), ptTrArc, TRARC)
	    {
		ptPlc = TrArcGPlc(ptTrArc);
		if (
#ifdef SMODELS
		    (!PtLongPlay()) &&
		    (
#endif
		     PlcGCount(ptPlc) >= TrArcGWeight(ptTrArc)
#ifdef SMODELS
		     )
#endif
		    )
		    continue;
#ifdef SMODELS
		if (PtSmodels())
		    {
			if (!PlcGGamma(ptPlc))
			    MkPlcGamma(ptPlc);
			if (PtLongPlay())
			    {
				if (((unsigned long) 0L) == ptM)
				    {
					MkTAGamma(trn, ptTrArc);
					ContRule(TrArcGGamma(ptTrArc));
				    }
				else
				    ContRule(IneqGGe(MkPlcIneq(ptPlc,
							       TrArcGWeight(ptTrArc)
							       )));
			    }
			else
			    ContRule(PlcGGamma(ptPlc));
		    }
		else
		    {
#endif
			TrnINOrSucc(trn);
			if ((!PlcGPurIn(ptPlc)) || LstEmpty(PlcGPurIn(ptPlc)))
			    {
				if (!PlcGMosIn(ptPlc))
				    return;
				ptLimit = PlcGCount(ptPlc);
				LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
				    {
					if (PlDrcGOut(ptPlDrc) <= ptLimit)
					    break;
				    }
				if (!ptPlDrc)
				    return;
			    }
#ifdef SMODELS
		    }
#endif
	    }
    if (TrnGMosIn(trn))
	LSTFOR (TrnGMosIn(trn), ptTrDrc, TRDRC)
	    {
		ptPlc = TrDrcGPlc(ptTrDrc);
		if (
#ifdef SMODELS
		    (!PtLongPlay()) &&
		    (
#endif
		     PlcGCount(ptPlc) >= TrDrcGIn(ptTrDrc)
#ifdef SMODELS
		     )
#endif
		    )
		    continue;
#ifdef SMODELS
		if (PtSmodels())
		    {
			if (!PlcGGamma(ptPlc))
			    MkPlcGamma(ptPlc);
			if (PtLongPlay())
			    {
				if (((unsigned long) 0L) == ptM)
				    {
					MkTDGamma(trn, ptTrDrc);
					ContRule(TrDrcGGamma(ptTrDrc));
				    }
				else
				    ContRule(IneqGGe(MkPlcIneq(ptPlc,
							       TrDrcGIn(ptTrDrc)
							       )));
			    }
			else
			    ContRule(PlcGGamma(ptPlc));
		    }
		else
		    {
#endif
			TrnINOrSucc(trn);
			if ((!PlcGPurIn(ptPlc)) || LstEmpty(PlcGPurIn(ptPlc)))
			    {
				if (!PlcGMosIn(ptPlc))
				    return;
				ptLimit = PlcGCount(ptPlc);
				LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
				    {
					if (PlDrcGOut(ptPlDrc) <= ptLimit)
					    break;
				    }
				if (!ptPlDrc)
				    return;
			    }
#ifdef SMODELS
		    }
#endif
	    }
    if (TrnGMosOut(trn))
	LSTFOR (TrnGMosOut(trn), ptTrDrc, TRDRC)
	    {
		ptPlc = TrDrcGPlc(ptTrDrc);
		if (
#ifdef SMODELS
		    (!PtLongPlay()) &&
		    (
#endif
		     PlcGCount(ptPlc) >= TrDrcGIn(ptTrDrc)
#ifdef SMODELS
		     )
#endif
		    )
		    continue;
#ifdef SMODELS
		if (PtSmodels())
		    {
			if (!PlcGGamma(ptPlc))
			    MkPlcGamma(ptPlc);
			if (PtLongPlay())
			    {
				if (((unsigned long) 0L) == ptM)
				    {
					MkTDGamma(trn, ptTrDrc);
					ContRule(TrDrcGGamma(ptTrDrc));
				    }
				else
				    ContRule(IneqGGe(MkPlcIneq(ptPlc,
							       TrDrcGIn(ptTrDrc)
							       )));
			    }
			else
			    ContRule(PlcGGamma(ptPlc));
		    }
		else
		    {
#endif
			TrnINOrSucc(trn);
			if ((!PlcGPurIn(ptPlc)) || LstEmpty(PlcGPurIn(ptPlc)))
			    {
				if (!PlcGMosIn(ptPlc))
				    return;
				ptLimit = PlcGCount(ptPlc);
				LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
				    {
					if (PlDrcGOut(ptPlDrc) <= ptLimit)
					    break;
				    }
				if (!ptPlDrc)
				    return;
			    }
#ifdef SMODELS
		    }
#endif
	    }
#ifdef SMODELS
    if (PtSmodels())
	{
	    if ((!PtLongPlay()) || TrnGActive(trn))
	        EndRule();
	    if (PtLongPlay() && TrnGActive(trn) &&
		(((unsigned long) 0L) == ptM))
		{
		    ptM = (unsigned long) 1L;
		    StartRule(TrnGActive(trn));
		    goto START;
		}
	}
#endif
    if (TrnGPurIn(trn) && !LstEmpty(TrnGPurIn(trn)))
        {
L3: ;
            TrnOrPrIn(trn);
            if (TrnGWhere(trn))
                {
                    TrnSWhere(trn, TrnGWhere(trn) | (3 << 3));
                    return;
                }
        }
    if (TrnGNeutral(trn) && !LstEmpty(TrnGNeutral(trn)))
        {
L4: ;
            TrnOrNeutral(trn);
            if (TrnGWhere(trn))
                {
                    TrnSWhere(trn, TrnGWhere(trn) | (4 << 3));
                    return;
                }
        }
    if (TrnGMosIn(trn) && !LstEmpty(TrnGMosIn(trn)))
        {
L5: ;
            TrnOrMsIn(trn);
            if (TrnGWhere(trn))
                {
                    TrnSWhere(trn, TrnGWhere(trn) | (5 << 3));
                    return;
                }
        }
    if (TrnGMosOut(trn) && !LstEmpty(TrnGMosOut(trn)))
        {
L6: ;
            TrnOrMsOut(trn);
            if (TrnGWhere(trn))
                {
                    TrnSWhere(trn, TrnGWhere(trn) | (6 << 3));
                    return;
                }
        }
}

/***************************************************************************
*
* Function:  TrnAO
* Purpose:   This is an auxiliary routine of TrnAndOr.
* Arguments: trn - a transition
* Returns:
* See also:  TrnAndOr
*
****************************************************************************
*/
FE1(void, TrnAO,
    TRN *, trn)
{
    while (TRUE)
        {
            TrnOA(trn);
            if (TrnGWhere(trn))
                {
                    TrnSDown(ptTrn, trn);
                    trn = ptTrn;
                }
            else if (TrnGDown(trn))
                {
                    ptTrn = TrnGDown(trn);
                    TrnCDown(trn);
                    trn = ptTrn;
                }
            else
                return;
        }
}

/***************************************************************************
*
* Function:  TrnAndOr
* Purpose:   This is the main routine of this module.
* Arguments: trn - a transition
* Returns:
* See also:
*
****************************************************************************
*/
FE1(BOOL, TrnAndOr,
    TRN *, trn)
{
    if (!TrnOld(trn))
	TrnAO(trn);
    return(FALSE);
}
