/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  ptandor2.c
*
* Part of: PROD's programs araprod and "gphgen"
*
* Author:  Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     See ptandor.c.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ptrbl.h"
#include "baltree.h"
#include "bdd.h"
#include "gphutil.h"
#include "graph.h"
#include "mark.h"
#include "encode.h"
#include "ptnet.h"
#include "ptstable.h"

static unsigned long ptTokCount;
static unsigned long ptArcWeight;

/***************************************************************************
*
* Function:  TrnNdPrIn
* Purpose:   This is an auxiliary routine of TrnAndOr.
* Arguments: trn - a transition
* Returns:
* See also:  TrnAndOr
*
****************************************************************************
*/
FE1(void, TrnNdPrIn,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        case 3:
            TrnSWhere(trn, (unsigned) 0);
            goto L3;
        case 4:
            TrnSWhere(trn, (unsigned) 0);
            goto L4;
        case 5:
            TrnSWhere(trn, (unsigned) 0);
            goto L5;
        }
    LSTFOR (TrnGPurIn(trn), ptTrArc, TRARC)
	{
	    ptPlc = TrArcGPlc(ptTrArc);
#ifdef SMODELS
	    if (PtSmodels())
		MkTP(trn, ptPlc);
	    else
#endif
		TrnCBinOr(trn);
	    if (PlcGPurOut(ptPlc))
		LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGPurOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return;
L1: ;
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
                                LstSet(PlcGPurOut(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGPurOut(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
                        if (
#ifdef SMODELS
                            (!PtSmodels()) &&
#endif
                            !TrnGBinOr(trn))
			    {
                                TrnSBinOr(trn, BinOrCons(trn));
				(void) LstInsLast(BinOrList(),
                                                  (char *) TrnGBinOr(trn));
			    }
			if (
#ifdef SMODELS
			    (!PtSmodels()) &&
#endif
			    !TrnGOrStrongPredAO(ptTrn))
			    TrnSOrStrongPredAO(ptTrn);
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(TrnGStrong(trn));
				if (PtLongPlay())
				    {
				        ContRule(TrnGActive(trn));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
#endif
			    (void) LstInsLast(TrnGOrStrongPredAO(ptTrn),
					      (char *) TrnGBinOr(trn));
		    }
	    ptTokCount = PlcGCount(ptPlc);
	    if (PlcGMosOut(ptPlc))
		LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
		    {
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGMosOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return;
L2: ;
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
				ptTokCount = PlcGCount(ptPlc);
                                LstSet(PlcGMosOut(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosOut(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
                        if (
#ifdef SMODELS
                            (!PtSmodels()) &&
#endif
                            !TrnGBinOr(trn))
			    {
                                TrnSBinOr(trn, BinOrCons(trn));
				(void) LstInsLast(BinOrList(),
                                                  (char *) TrnGBinOr(trn));
			    }
			if (
#ifdef SMODELS
			    (!PtSmodels()) &&
#endif
			    !TrnGOrStrongPredAO(ptTrn))
			    TrnSOrStrongPredAO(ptTrn);
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(TrnGStrong(trn));
				if (PtLongPlay())
				    {
				        ContRule(TrnGActive(trn));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
#endif
			    (void) LstInsLast(TrnGOrStrongPredAO(ptTrn),
					      (char *) TrnGBinOr(trn));
			if (
#ifdef SMODELS
			    (!PtLongPlay()) &&
			    (
#endif
			     PlDrcGOut(ptPlDrc) > ptTokCount
#ifdef SMODELS
			     )
#endif
			    )
			    continue;
			if (
#ifdef SMODELS
			    (!PtSmodels()) &&
#endif
			    !TrnGOrWeakPredAO(ptTrn))
			    TrnSOrWeakPredAO(ptTrn);
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(TrnGWeak(trn));
				if (PtLongPlay())
				    {
				        ContRule(TrnGActive(trn));
					ContRule(IneqGGe(MkPlcIneq(ptPlc,
								   PlDrcGOut(ptPlDrc)
								   )));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
#endif
			    (void) LstInsLast(TrnGOrWeakPredAO(ptTrn),
					      (char *) TrnGBinOr(trn));
		    }
	    ptLimit = PlcGCount(ptPlc) - TrArcGWeight(ptTrArc);
	    if (PlcGNeutral(ptPlc))
		LSTFOR (PlcGNeutral(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGNeutral(ptPlc)));
                                TrnSWhere(trn, (unsigned) 3);
                                return;
L3: ;
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
				ptTokCount = PlcGCount(ptPlc);
				ptLimit = PlcGCount(ptPlc)
				    - TrArcGWeight(ptTrArc);
                                LstSet(PlcGNeutral(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGNeutral(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
			if (
#ifdef SMODELS
			    PtLongPlay() ||
			    (
#endif
			     PlArcGWeight(ptPlArc) > ptLimit
#ifdef SMODELS
			     )
#endif
			    )
			    {
                                if (
#ifdef SMODELS
                                    (!PtSmodels()) &&
#endif
                                    !TrnGBinOr(trn))
				    {
                                        TrnSBinOr(trn, BinOrCons(trn));
					(void) LstInsLast(BinOrList(),
							  (char *)
                                                          TrnGBinOr(trn));
				    }
				if (
#ifdef SMODELS
				    (!PtSmodels()) &&
#endif
				    !TrnGOrStrongPredAO(ptTrn))
				    TrnSOrStrongPredAO(ptTrn);
#ifdef SMODELS
				if (PtSmodels())
				    {
					StartRule(TrnGStrong(trn));
					if (PtLongPlay())
					    {
					        ContRule(TrnGActive(trn));
						ContRule(IneqGLt(MkPlcIneq(ptPlc,
									   PlArcGWeight(ptPlArc)
									   +
									   TrArcGWeight(ptTrArc)
									   )));
					    }
					ContRule(TrnGGamma(ptTrn));
					EndRule();
				    }
				else
#endif
				    (void)
					LstInsLast(TrnGOrStrongPredAO(ptTrn),
						   (char *) TrnGBinOr(trn));
			    }
			if (
#ifdef SMODELS
			    (!PtLongPlay()) &&
			    (
#endif
			     PlArcGWeight(ptPlArc) > ptTokCount
#ifdef SMODELS
			     )
#endif
			    )
			    continue;
                        if (
#ifdef SMODELS
                            (!PtSmodels()) &&
#endif
                            !TrnGBinOr(trn))
			    {
                                TrnSBinOr(trn, BinOrCons(trn));
				(void) LstInsLast(BinOrList(),
                                                  (char *) TrnGBinOr(trn));
			    }
			if (
#ifdef SMODELS
			    (!PtSmodels()) &&
#endif
			    !TrnGOrWeakPredAO(ptTrn))
			    TrnSOrWeakPredAO(ptTrn);
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(TrnGWeak(trn));
				if (PtLongPlay())
				    {
				        ContRule(TrnGActive(trn));
					ContRule(IneqGGe(MkPlcIneq(ptPlc,
								   PlArcGWeight(ptPlArc)
								   )));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
#endif
			    (void) LstInsLast(TrnGOrWeakPredAO(ptTrn),
					      (char *) TrnGBinOr(trn));
		    }
	    if (PlcGMosIn(ptPlc))
		LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
		    {
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGMosIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 4);
                                return;
L4: ;
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
				ptTokCount = PlcGCount(ptPlc);
				ptLimit = PlcGCount(ptPlc)
				    - TrArcGWeight(ptTrArc);
                                LstSet(PlcGMosIn(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosIn(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
			if (
#ifdef SMODELS
			    PtLongPlay() ||
			    (
#endif
			     PlDrcGOut(ptPlDrc) > ptLimit
#ifdef SMODELS
			     )
#endif
			    )
			    {
                                if (
#ifdef SMODELS
                                    (!PtSmodels()) &&
#endif
                                    !TrnGBinOr(trn))
				    {
                                        TrnSBinOr(trn, BinOrCons(trn));
					(void) LstInsLast(BinOrList(),
							  (char *)
                                                          TrnGBinOr(trn));
				    }
				if (
#ifdef SMODELS
				    (!PtSmodels()) &&
#endif
				    !TrnGOrStrongPredAO(ptTrn))
				    TrnSOrStrongPredAO(ptTrn);
#ifdef SMODELS
				if (PtSmodels())
				    {
					StartRule(TrnGStrong(trn));
					if (PtLongPlay())
					    {
					        ContRule(TrnGActive(trn));
						ContRule(IneqGLt(MkPlcIneq(ptPlc,
									   PlDrcGOut(ptPlDrc)
									   +
									   TrArcGWeight(ptTrArc)
									   )));
					    }
					ContRule(TrnGGamma(ptTrn));
					EndRule();
				    }
				else
#endif
				    (void)
					LstInsLast(TrnGOrStrongPredAO(ptTrn),
						   (char *) TrnGBinOr(trn));
			    }
			if (
#ifdef SMODELS
			    (!PtLongPlay()) &&
			    (
#endif
			     PlDrcGOut(ptPlDrc) > ptTokCount
#ifdef SMODELS
			     )
#endif
			    )
			    continue;
                        if (
#ifdef SMODELS
                            (!PtSmodels()) &&
#endif
                            !TrnGBinOr(trn))
			    {
                                TrnSBinOr(trn, BinOrCons(trn));
				(void) LstInsLast(BinOrList(),
                                                  (char *) TrnGBinOr(trn));
			    }
			if (
#ifdef SMODELS
			    (!PtSmodels()) &&
#endif
			    !TrnGOrWeakPredAO(ptTrn))
			    TrnSOrWeakPredAO(ptTrn);
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(TrnGWeak(trn));
				if (PtLongPlay())
				    {
				        ContRule(TrnGActive(trn));
					ContRule(IneqGGe(MkPlcIneq(ptPlc,
								   PlDrcGOut(ptPlDrc)
								   )));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
#endif
			    (void) LstInsLast(TrnGOrWeakPredAO(ptTrn),
					      (char *) TrnGBinOr(trn));
		    }
	    if (PlcGPurIn(ptPlc))
		LSTFOR (PlcGPurIn(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGPurIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 5);
                                return;
L5: ;
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
                                LstSet(PlcGPurIn(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGPurIn(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
                        if (
#ifdef SMODELS
                            (!PtSmodels()) &&
#endif
                            !TrnGBinOr(trn))
			    {
                                TrnSBinOr(trn, BinOrCons(trn));
				(void) LstInsLast(BinOrList(),
                                                  (char *) TrnGBinOr(trn));
			    }
			if (
#ifdef SMODELS
			    (!PtSmodels()) &&
#endif
			    !TrnGOrWeakPredAO(ptTrn))
			    TrnSOrWeakPredAO(ptTrn);
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(TrnGWeak(trn));
				if (PtLongPlay())
				    {
				        ContRule(TrnGActive(trn));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
#endif
			    (void) LstInsLast(TrnGOrWeakPredAO(ptTrn),
					      (char *) TrnGBinOr(trn));
		    }
	}
}

/***************************************************************************
*
* Function:  TrnNdMsIn
* Purpose:   This is an auxiliary routine of TrnAndOr.
* Arguments: trn - a transition
* Returns:
* See also:  TrnAndOr
*
****************************************************************************
*/
FE1(void, TrnNdMsIn,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        case 3:
            TrnSWhere(trn, (unsigned) 0);
            goto L3;
        case 4:
            TrnSWhere(trn, (unsigned) 0);
            goto L4;
        case 5:
            TrnSWhere(trn, (unsigned) 0);
            goto L5;
        }
    LSTFOR (TrnGMosIn(trn), ptTrDrc, TRDRC)
	{
	    ptPlc = TrDrcGPlc(ptTrDrc);
#ifdef SMODELS
	    if (PtSmodels())
		MkTP(trn, ptPlc);
	    else
#endif
		TrnCBinOr(trn);
	    if (PlcGPurOut(ptPlc))
		LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGPurOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return;
L1: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosIn(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
                                LstSet(PlcGPurOut(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGPurOut(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
                        if (
#ifdef SMODELS
                            (!PtSmodels()) &&
#endif
                            !TrnGBinOr(trn))
			    {
                                TrnSBinOr(trn, BinOrCons(trn));
				(void) LstInsLast(BinOrList(),
                                                  (char *) TrnGBinOr(trn));
			    }
			if (
#ifdef SMODELS
			    (!PtSmodels()) &&
#endif
			    !TrnGOrStrongPredAO(ptTrn))
			    TrnSOrStrongPredAO(ptTrn);
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(TrnGStrong(trn));
				if (PtLongPlay())
				    {
				        ContRule(TrnGActive(trn));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
#endif
			    (void) LstInsLast(TrnGOrStrongPredAO(ptTrn),
					      (char *) TrnGBinOr(trn));
		    }
	    ptTokCount = PlcGCount(ptPlc);
	    ptArcWeight = TrDrcGOut(ptTrDrc);
	    if (PlcGMosOut(ptPlc))
		LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
		    {
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGMosOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return;
L2: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosIn(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
				ptTokCount = PlcGCount(ptPlc);
				ptArcWeight = TrDrcGOut(ptTrDrc);
                                LstSet(PlcGMosOut(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosOut(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
                        if (
#ifdef SMODELS
                            (!PtSmodels()) &&
#endif
                            !TrnGBinOr(trn))
			    {
                                TrnSBinOr(trn, BinOrCons(trn));
				(void) LstInsLast(BinOrList(),
                                                  (char *) TrnGBinOr(trn));
			    }
			if (
#ifdef SMODELS
			    (!PtSmodels()) &&
#endif
			    !TrnGOrStrongPredAO(ptTrn))
			    TrnSOrStrongPredAO(ptTrn);
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(TrnGStrong(trn));
				if (PtLongPlay())
				    {
				        ContRule(TrnGActive(trn));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
#endif
			    (void) LstInsLast(TrnGOrStrongPredAO(ptTrn),
					      (char *) TrnGBinOr(trn));
			if ((PlDrcGIn(ptPlDrc) <= ptArcWeight) ||
#ifdef SMODELS
			     ((!PtLongPlay()) &&
#endif
			      (PlDrcGOut(ptPlDrc) > ptTokCount)
#ifdef SMODELS
			      )
#endif
			    )
			    continue;
			if (
#ifdef SMODELS
			    (!PtSmodels()) &&
#endif
			    !TrnGOrWeakPredAO(ptTrn))
			    TrnSOrWeakPredAO(ptTrn);
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(TrnGWeak(trn));
				if (PtLongPlay())
				    {
				        ContRule(TrnGActive(trn));
					ContRule(IneqGGe(MkPlcIneq(ptPlc,
								   PlDrcGOut(ptPlDrc)
								   )));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
#endif
			    (void) LstInsLast(TrnGOrWeakPredAO(ptTrn),
					      (char *) TrnGBinOr(trn));
		    }
	    ptLimit = PlcGCount(ptPlc)
		- TrDrcGIn(ptTrDrc) + TrDrcGOut(ptTrDrc);
	    if (PlcGNeutral(ptPlc))
		LSTFOR (PlcGNeutral(ptPlc), ptPlArc, PLARC)
		    {
			if (
#ifdef SMODELS
			    (!PtLongPlay()) &&
			    (
#endif
			     (PlArcGWeight(ptPlArc) <= ptLimit) &&
			     (PlArcGWeight(ptPlArc) <= ptArcWeight)
#ifdef SMODELS
			     )
#endif
			    )
			    continue;
			ptTrn = PlArcGTrn(ptPlArc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGNeutral(ptPlc)));
                                TrnSWhere(trn, (unsigned) 3);
                                return;
L3: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosIn(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
				ptTokCount = PlcGCount(ptPlc);
				ptArcWeight = TrDrcGOut(ptTrDrc);
				ptLimit = PlcGCount(ptPlc)
				    - TrDrcGIn(ptTrDrc)
					+ TrDrcGOut(ptTrDrc);
                                LstSet(PlcGNeutral(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGNeutral(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
			if (
#ifdef SMODELS
			    PtLongPlay() ||
			    (
#endif
			     PlArcGWeight(ptPlArc) > ptLimit
#ifdef SMODELS
			     )
#endif
			    )
			    {
                                if (
#ifdef SMODELS
                                    (!PtSmodels()) &&
#endif
                                    !TrnGBinOr(trn))
				    {
                                        TrnSBinOr(trn, BinOrCons(trn));
					(void) LstInsLast(BinOrList(),
							  (char *)
                                                          TrnGBinOr(trn));
				    }
				if (
#ifdef SMODELS
				    (!PtSmodels()) &&
#endif
				    !TrnGOrStrongPredAO(ptTrn))
				    TrnSOrStrongPredAO(ptTrn);
#ifdef SMODELS
				if (PtSmodels())
				    {
					StartRule(TrnGStrong(trn));
					if (PtLongPlay())
					    {
					        ContRule(TrnGActive(trn));
						ContRule(IneqGLt(MkPlcIneq(ptPlc,
									   PlArcGWeight(ptPlArc)
									   +
									   TrDrcGIn(ptTrDrc)
									   -
									   TrDrcGOut(ptTrDrc)
									   )));
					    }
					ContRule(TrnGGamma(ptTrn));
					EndRule();
				    }
				else
#endif
				    (void)
					LstInsLast(TrnGOrStrongPredAO(ptTrn),
						   (char *) TrnGBinOr(trn));
			    }
			if ((PlArcGWeight(ptPlArc) <= ptArcWeight) ||
#ifdef SMODELS
			    ((!PtLongPlay()) &&
#endif
			     (PlArcGWeight(ptPlArc) > ptTokCount)
#ifdef SMODELS
			     )
#endif
			    )
			    continue;
                        if (
#ifdef SMODELS
                            (!PtSmodels()) &&
#endif
                            !TrnGBinOr(trn))
			    {
                                TrnSBinOr(trn, BinOrCons(trn));
				(void) LstInsLast(BinOrList(),
                                                  (char *) TrnGBinOr(trn));
			    }
			if (
#ifdef SMODELS
			    (!PtSmodels()) &&
#endif
			    !TrnGOrWeakPredAO(ptTrn))
			    TrnSOrWeakPredAO(ptTrn);
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(TrnGWeak(trn));
				if (PtLongPlay())
				    {
				        ContRule(TrnGActive(trn));
					ContRule(IneqGGe(MkPlcIneq(ptPlc,
								   PlArcGWeight(ptPlArc)
								   )));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
#endif
			    (void) LstInsLast(TrnGOrWeakPredAO(ptTrn),
					      (char *) TrnGBinOr(trn));
		    }
	    if (PlcGMosIn(ptPlc))
		LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
		    {
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGMosIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 4);
                                return;
L4: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosIn(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
				ptTokCount = PlcGCount(ptPlc);
				ptArcWeight = TrDrcGOut(ptTrDrc);
				ptLimit = PlcGCount(ptPlc)
				    - TrDrcGIn(ptTrDrc)
					+ TrDrcGOut(ptTrDrc);
                                LstSet(PlcGMosIn(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosIn(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
			if (
#ifdef SMODELS
			    PtLongPlay() ||
			    (
#endif
			     PlDrcGOut(ptPlDrc) > ptLimit
#ifdef SMODELS
			     )
#endif
			    )
			    {
                                if (
#ifdef SMODELS
                                    (!PtSmodels()) &&
#endif
                                    !TrnGBinOr(trn))
				    {
                                        TrnSBinOr(trn, BinOrCons(trn));
					(void) LstInsLast(BinOrList(),
							  (char *)
                                                          TrnGBinOr(trn));
				    }
				if (
#ifdef SMODELS
				    (!PtSmodels()) &&
#endif
				    !TrnGOrStrongPredAO(ptTrn))
				    TrnSOrStrongPredAO(ptTrn);
#ifdef SMODELS
				if (PtSmodels())
				    {
					StartRule(TrnGStrong(trn));
					if (PtLongPlay())
					    {
					        ContRule(TrnGActive(trn));
						ContRule(IneqGLt(MkPlcIneq(ptPlc,
									   PlDrcGOut(ptPlDrc)
									   +
									   TrDrcGIn(ptTrDrc)
									   -
									   TrDrcGOut(ptTrDrc)
									   )));
					    }
					ContRule(TrnGGamma(ptTrn));
					EndRule();
				    }
				else
#endif
				    (void)
					LstInsLast(TrnGOrStrongPredAO(ptTrn),
						   (char *) TrnGBinOr(trn));
			    }
			if (
#ifdef SMODELS
			    (!PtLongPlay()) &&
			    (
#endif
			     PlDrcGOut(ptPlDrc) > ptTokCount
#ifdef SMODELS
			     )
#endif
			    )
			    continue;
                        if (
#ifdef SMODELS
                            (!PtSmodels()) &&
#endif
                            !TrnGBinOr(trn))
			    {
                                TrnSBinOr(trn, BinOrCons(trn));
				(void) LstInsLast(BinOrList(),
                                                  (char *) TrnGBinOr(trn));
			    }
			if (
#ifdef SMODELS
			    (!PtSmodels()) &&
#endif
			    !TrnGOrWeakPredAO(ptTrn))
			    TrnSOrWeakPredAO(ptTrn);
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(TrnGWeak(trn));
				if (PtLongPlay())
				    {
				        ContRule(TrnGActive(trn));
					ContRule(IneqGGe(MkPlcIneq(ptPlc,
								   PlDrcGOut(ptPlDrc)
								   )));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
#endif
			    (void) LstInsLast(TrnGOrWeakPredAO(ptTrn),
					      (char *) TrnGBinOr(trn));
		    }
	    if (PlcGPurIn(ptPlc))
		LSTFOR (PlcGPurIn(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			if (!TrnOld(ptTrn))
			    {
                                TrnSCursor(trn, LstGet(PlcGPurIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 5);
                                return;
L5: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosIn(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
                                LstSet(PlcGPurIn(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGPurIn(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
                        if (
#ifdef SMODELS
                            (!PtSmodels()) &&
#endif
                            !TrnGBinOr(trn))
			    {
                                TrnSBinOr(trn, BinOrCons(trn));
				(void) LstInsLast(BinOrList(),
                                                  (char *) TrnGBinOr(trn));
			    }
			if (
#ifdef SMODELS
			    (!PtSmodels()) &&
#endif
			    !TrnGOrWeakPredAO(ptTrn))
			    TrnSOrWeakPredAO(ptTrn);
#ifdef SMODELS
			if (PtSmodels())
			    {
				StartRule(TrnGWeak(trn));
				if (PtLongPlay())
				    {
				        ContRule(TrnGActive(trn));
				    }
				ContRule(TrnGGamma(ptTrn));
				EndRule();
			    }
			else
#endif
			    (void) LstInsLast(TrnGOrWeakPredAO(ptTrn),
					      (char *) TrnGBinOr(trn));
		    }
	}
}
