/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  ptelem.c
*
* Part of: PROD's programs araprod and "gphgen"
*
* Author:  Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     This is an auxiliary module of ptnet.c.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#if (defined(UNIX) || defined(TNIL))
#include <unistd.h>
#endif
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ptrbl.h"
#include "baltree.h"
#include "bdd.h"
#include "gphutil.h"
#include "graph.h"
#include "mark.h"
#include "encode.h"
#include "ptnet.h"
#include "ptsymm.h"
#include "error.h"
#include "cmdnm.h"

unsigned long trNumber;
static char *araArcName;

/***************************************************************************
*
* Function:  LTSBegin
* Purpose:   This routine starts the creation of an ARA LTS.
* Arguments: name - the name of the lts
*            separate - a distinct arc file is used if and only if
*                       this is TRUE (actually exactly when the net
*                       is not unfolded)
* Returns:
* See also:  PtAraOutput, LTSEnd, TrnFire (ptnet.c), InsOutput (encode.c),
*            AraStep (inst.c)
*
****************************************************************************
*/
FE2(void, LTSBegin,
    char *, name,
    BOOL, separate)
{
    char *str;
    unsigned i, j;

    str = FileName(name, "lts");
    if (!SetAraFile(fopen(str, "w")))
	{
	    (void) fprintf(_err, 
			   "\nCannot open empty file %s for writing\n",
			   str);
	    ErrnoOutput(_err);
	    ClearErrno();
	    Exit(11);
	}
    Deallocate((char *) str);
    if (separate)
	{
	    araArcName = FileName(name, "arc");
	    if (!AraSetArcFile(fopen(araArcName, "w+")))
		{
		    (void) fprintf(_err, 
				   "\nCannot open empty file %s for reading and writing\n",
				   araArcName);
		    ErrnoOutput(_err);
		    ClearErrno();
		    Exit(11);
		}
	}
    else
	AraSetArcFile(AraFile());
    for (j = 0; j < 2; j++)
	{
	    for (i = 0; i < 59; i++)
		(void) fputc((int) ' ', AraFile());
	    (void) fputc((int) '\n', AraFile());
	}
    (void) fprintf(AraFile(),
		   "HISTORY |created by %s|\nEND_HEADER\nSYMBOLS\n0 i\n",
		   myName);
}

/***************************************************************************
*
* Function:  LTSEnd
* Purpose:   This routine finishes the creation of an ARA LTS.
* Returns:
* See also:  PtAraOutput, LTSBegin, TrnFire (ptnet.c), InsOutput (encode.c),
*            AraStep (inst.c)
*
****************************************************************************
*/
FE0(void, LTSEnd)
{
    char *s;
    unsigned n;
    unsigned class;
    BOOL rdStat;

    if (AraArcFile() != AraFile())
	{
#ifndef DEBUG
	    DelAraNames(TRUE);
#endif
	    trNumber = BalTrGElCount(AraTree()) + 1L;
	    BalTrDestroy(AraTree(), TRUE);
	    (void) fprintf(AraFile(), "END_SYMBOLS\nARCS\n");
	    (void) fseek(AraArcFile(), 0L, 0);
	    n = (unsigned) BUFSIZ;
	    s = (char *) Allocate(n);
	    while (TRUE)
		{
		    rdStat = BruteReadln(&s, &n, AraArcFile());
		    if ((!rdStat) && !*s)
			break;
		    (void) fputs(s, AraFile());
		    if (!rdStat)
			break;
		}
	    Deallocate(s);
	    (void) fclose(AraArcFile());
	    (void) unlink(araArcName);
	    Deallocate(araArcName);
	}
    else
	{
	    trNumber = 1L;
	    for (class = 0; class < IngGNofClasses(); class++)
		trNumber += PtrBlGCount(TblTrn(class));
	}
    (void) fprintf(AraFile(), "END_ARCS\nNODES\nEND_NODES\nEND_LTS\n");
    (void) fseek(AraFile(), 0L, 0);
    (void) fprintf(AraFile(),
		   "LTS |B|\nROOT |%lu|\nNODE_COUNT |%lu|\nARC_COUNT |%lu|\nSYMBOL_COUNT |%lu|",
		   GphOrdinal(theGraph, GphStart(theGraph)),
		   GphNdCount(theGraph), GphGlAwCount(theGraph), trNumber);
    (void) fclose(AraFile());
}

/***************************************************************************
*
* Function:  GetPlc
* Purpose:   This routine checks if a place tree has a place corresponding
*            to a given high level encoding. If not, such place is inserted
*            into the tree.
* Arguments: key - the encoding
* Returns:   the place corresponding to the encoding
* See also:  GetTrn
*
****************************************************************************
*/
FE1(PLC *, GetPlc,
    DATATYPE *, key)
{
    PLC *plc;

    if ((plc = (PLC *) BalTrElement(PlcBalTree(), (char *) key)))
	return(plc);
    plc = PlcCons();
    PlcSKey(plc, DataCopy(key));
    (void) BalTrInsert(PlcBalTree(), (char *) plc, (char *) key, TRUE);
    return(plc);
}

/***************************************************************************
*
* Function:  GetTrn
* Purpose:   This routine checks if a transition tree has a transition
*            corresponding to a given high level encoding. If not, such
*            transition is inserted into the tree.
* Arguments: class - the precedence class determining the tree
*            key - the encoding
*            new - at return indicated whether or not the encoding was
*                  new in the tree
* Returns:   the transition corresponding to the encoding
* See also:  GetPlc
*
****************************************************************************
*/
FE3(TRN *, GetTrn,
    unsigned, class,
    DATATYPE *, key,
    BOOL *, new)
{
    TRN *trn;

    if ((trn = (TRN *) BalTrElement(TrnBalTree(class), (char *) key)))
	{
	    *new = FALSE;
	    return(trn);
	}
    trn = TrnCons();
    TrnSKey(trn, DataCopy(key));
    (void) BalTrInsert(TrnBalTree(class), (char *) trn, (char *) key, TRUE);
    *new = TRUE;
    return(trn);
}

static unsigned long ptI;
static unsigned long ptEqNdAddr; /* a node in the reachability graph
                                  * whose marking is equivalent to
                                  * a given marking with respect to place
                                  * symmetries */
static BOOL ptFaithful; /* indicates whether the marking of the target
                         * node of an arrow is the actual marking
                         * obtained by firing the transition of the arrow
                         * (this flag is needed in the symmetry method) */
static unsigned long *b_a_c_k;

/***************************************************************************
*
* Function:  FndEquiv
* Purpose:   This routine checks if a place symmetry maps a marking
*            to a marking that occurs in the reachability graph of the net.
* Arguments: symm - the place symmetry
* Returns:
* See also:
*
****************************************************************************
*/
FS1(BOOL, FndEquiv,
    PTRBL *, symm)
{
    ptI = PtrBlGCount(TblPlc());
    while (ptI--)
	PlcSCount((PLC *)
		  PtrBlGet(DupTblPlc(), *((unsigned long *)
					  PtrBlGet(symm, ptI))),
		  PlcGCount((PLC *) PtrBlGet(TblPlc(), ptI)));
    (void) PlcEncode(DupTblPlc(), EngABuffer(), EngABufSize());
    ptEqNdAddr = GphFndNode(theGraph, EngBuffer(), b_a_c_k);
    return(ptEqNdAddr != (unsigned long) -1L);
}

/***************************************************************************
*
* Function:  TrnStore
* Purpose:   This routine stores the "val" information of a transition.
* Arguments: trn - the transition
* Returns:
* See also:  TrnRestore
*
****************************************************************************
*/
FS1(BOOL, TrnStore,
    TRN *, trn)
{
    TrnFlagStore(trn);
    if (PtStub() && !PtDelAlg())
	TrnSDupVal(trn, TrnGVal(trn));
    return(FALSE);
}

/***************************************************************************
*
* Function:  TrnRestore
* Purpose:   This routine restores the "val" information of a transition.
* Arguments: trn - the transition
* Returns:
* See also:  TrnStore
*
****************************************************************************
*/
FS1(BOOL, TrnRestore,
    TRN *, trn)
{
    TrnFlagRestore(trn);
    if (PtStub() && !PtDelAlg())
	TrnSVal(trn, TrnGDupVal(trn));
    return(FALSE);
}

/***************************************************************************
*
* Function:  TrnHowBest
* Purpose:   This routine simulates TrnBest in order to know if more than
*            transition is to be fired.
* Arguments: trn - a transition
* Returns:
* See also:  TrnBest
*
****************************************************************************
*/
FS1(BOOL, TrnHowBest,
    TRN *, trn)
{
    if (TrnEnabled(trn) && ((TrnGVal(trn) == PtBest()) ||
			    (TrnGVal(trn) == PtrBlGCount(TblTrn(0)) + 1L)))
	{
	    if (ptFirst)
		{
		    ptBranching = TRUE;
		    return(TRUE);
		}
	    else
		ptFirst = trn;
	}
    return(FALSE);
}

/***************************************************************************
*
* Function:  TrnHowDelCheck
* Purpose:   This routine simulates TrnDelCheck in order to know if
*            more than one transition is to be fired.
* Arguments: trn - a transition
* Returns:
* See also:  TrnDelCheck
*
****************************************************************************
*/
FS1(BOOL, TrnHowDelCheck,
    TRN *, trn)
{
    if (TrnEnabled(trn) && !TrnSatisfied(trn))
	{
	    if (ptFirst)
		{
		    ptBranching = TRUE;
		    return(TRUE);
		}
	    else
		ptFirst = trn;
	}
    return(FALSE);
}

/***************************************************************************
*
* Function:  TrnFire
* Purpose:   This routine fires a transition.
* Arguments: trn - the transition
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, TrnFire,
    TRN *, trn)
{
    register DATATYPE *pointer;
    register unsigned long a;
    register unsigned len;
    unsigned awLen;
    unsigned long awAddr = 0L;
    TRARC *trArc;
    TRDRC *trDrc;
    unsigned long ndAddr;
    PLC *plc = NULL;
    GRAPH *graph;
    unsigned long best;
    LIST *speculation;
    VOID *cursor;
    BALTREE *tree = NULL;
    unsigned long end = 0L;
    PLARC *plArc;
    BOOL recurs;
    BOOL divNow;
    BOOL icold;
    
    len = 0;
    if (PtSymm())
	a = (TrnGNumber(trn) << 1) | 1L;
    else
	a = TrnGNumber(trn);
    IncLen(a, len);
    if (len >= EngBufSize())
	{
	    EngSBufSize((len >> 1) + len + 1);
	    Deallocate((char *) EngBuffer());
	    EngSBuffer((DATATYPE *) AllocArray(EngBufSize(),
					       sizeof(DATATYPE)));
	}
    awLen = len;
    if (TrnGPurIn(trn))
	LSTFOR (TrnGPurIn(trn), trArc, TRARC)
	    PlcDCount(TrArcGPlc(trArc), TrArcGWeight(trArc));
    if (TrnGPurOut(trn))
	LSTFOR (TrnGPurOut(trn), trArc, TRARC)
	    PlcICount(TrArcGPlc(trArc), TrArcGWeight(trArc));
    if (TrnGMosIn(trn))
	LSTFOR (TrnGMosIn(trn), trDrc, TRDRC)
	    PlcDCount(TrDrcGPlc(trDrc),
		      TrDrcGIn(trDrc) - TrDrcGOut(trDrc));
    if (TrnGMosOut(trn))
	LSTFOR (TrnGMosOut(trn), trDrc, TRDRC)
	    PlcICount(TrDrcGPlc(trDrc),
		      TrDrcGOut(trDrc) - TrDrcGIn(trDrc));
    if (Buchi())
	{
	    if (PtStub())
		graph = the2Graph;
	    else
		{
		    graph = theGraph;
		    PlcSCount(buPlc, (unsigned long) 0L);
		    plc = (PLC *) LstBegin(PlcGNext(buPlc));
		}
	}
    else
	graph = theGraph;
 START: ;
    if (Buchi() && (graph == theGraph))
	{
	    PlcSCount(plc, (unsigned long) 1L);
	    theEvalState = PtState();
	    if (!((*_buchiOK)()))
		{
		    StaDiscard(theEvalState);
		    goto NEXT;
		}
	    StaDiscard(theEvalState);
	}
    if (PtSymm())
	{
	    theBack = (unsigned long) -1L;
	    (void) PlcEncode(TblPlc(), EngABuffer(), EngABufSize());
	    ndAddr = GphFndNode(graph, EngBuffer(), &theBack);
	    if (ndAddr == (unsigned long) -1L)
		{
		    b_a_c_k = &theBack;
		    if (BalTrApply(PlcSymmetries(), FndEquiv))
			{
			    ndAddr = ptEqNdAddr;
			    ptFaithful = FALSE;
			}
		    else
			{
			    len = PlcEncode(TblPlc(), EngABuffer(),
					    EngABufSize());
			    ndAddr = GphPutNode(graph, EngBuffer(),
						len, &theBack);
			    ptFaithful = TRUE;
			}
		}
	    else
		ptFaithful = TRUE;
	}
    else
	{
	    len = PlcEncode(TblPlc(), EngABuffer(), EngABufSize());
	    ndAddr = GphPutNode(graph, EngBuffer(), len, &theBack);
	}
    pointer = EngBuffer();
    if (PtSymm())
	a = (TrnGNumber(trn) << 1) | (ptFaithful ? 0L : 1L);
    else
	a = TrnGNumber(trn);
    PutValue(a, pointer);
    *pointer = (DATATYPE) 0;
    awAddr = GphPutArrow(graph, EngBuffer(), awLen, theAwCount,
			 ((!PtSleep()) && Buchi() && PtStub())
			 ? PthEnd(thePath) : PthEnd(nowPath),
			 ndAddr, theAwFirst, theBack, !TrnVisible(trn));
    if ((theAwCount == 0L) || FreeExpansion())
	{
	    theAwFirst = awAddr;
	    theNdFirst = ndAddr;
	}
    theAwCount++;
    if ((graph == theGraph) && Ara())
	{
	    (void) fprintf(AraArcFile(), "%lu %lu %lu",
			   GphOrdinal(graph, PthEnd(nowPath)),
			   GphOrdinal(graph, ndAddr),
			   TrnVisible(trn)
			   ? (TrnGNumber(trn) + 1L) : (unsigned long) 0L);
	    if (PtSymm() && !ptFaithful)
		(void) fprintf(AraArcFile(),
			       " -- arc explained by the state equivalence relation");
	    (void) fputc((int) '\n', AraArcFile());
	}
    if (Buchi())
	{
	    if ((!PtStub()) && InfNow() && (ndAddr == PthEnd(the2Path)))
		{
		    PtGphUpdate();
		    PtC4(awAddr, ndAddr);
		}
	}
    else if (OnTheFly())
	{
	    if (((unsigned long) -1L) != theBack)
		{
		    if (InfNow())
			{
			    if (ndAddr == StpGWorld(PthLast(the2Path)))
				{
				    PtGphUpdate();
				    PtC4(awAddr, ndAddr);
				}
			}
		    else if (DivNow() && (!TrnVisible(trn)) &&
			     ((ndAddr == PthEnd(thePath)) ||
			      BalTrElement(theNodeTree,
					   (char *) &ndAddr)))
			{
			    PtGphUpdate();
			    PtC3(awAddr, ndAddr);
			}
		}
	    else if (PtFlyPlc())
		{
		    if (PlcReject(PtFlyPlc()))
			PtC1(awAddr, ndAddr);
		    else if (PlcDeadlock(PtFlyPlc()))
			PtC2(awAddr, ndAddr);
		}
	}
 NEXT: ;
    if (Buchi() && (graph == theGraph))
	{
	    PlcSCount(plc, (unsigned long) 0L);
	    plc = (PLC *) LstStep(PlcGNext(buPlc));
	    if (plc)
		goto START;
	}
    if ((((unsigned long) -1L) == theBack) && Elimination() &&
	((!PtSleep()) || (BalTrGElCount(Pt4SlpSet()) == 0L)) &&
	((!Buchi()) || (graph != theGraph)))
	{
	    recurs = PtRecurs();
	    best = PtBest();
	    divNow = DivNow() ? TRUE : FALSE;
	    if (PtFlyPlc() && PlcLivelock(PtFlyPlc()))
		SetDivNow();
	    else
		ClDivNow();
	    ptDoIt = FALSE;
	    icold = FALSE;
	    (void) PtrBlApply(TblTrn(0), TrnStore);
	    speculation = LstCreate(&plArcTools);
	    (void) LstInsLast(speculation, (char *) PlArcCons(trn, ndAddr));
	    if (PtCFFD() || DivNow())
		{
		    end = PthEnd(thePath);
		    PthSEnd(thePath, (unsigned long) -1L);
		    tree = theNodeTree;
		    if (TrnVisible(trn))
			theNodeTree = BalTrCreate(&lngBalTools);
		    else
			{
			    theNodeTree = BalTrCopy(tree, FALSE);
			    (void) BalTrInsert(theNodeTree, (char *) &end,
					       (char *) &end, TRUE);
			}
		    (void) BalTrInsert(theNodeTree, (char *)
				       PlArcAWeight((PLARC *)
						    LstLast(speculation)),
				       (char *)
				       PlArcAWeight((PLARC *)
						    LstLast(speculation)),
				       TRUE);
		}
	    while (TRUE)
		{
		    FndEnbl();
		    if (PtGECnt() == 0L)
			goto TERMINAL;
		    if (PtGInvisCnt() == 0L)
			goto OBSTACLE;
		    if (PtGECnt() > 1L)
			{
			    if (!PtStub())
				goto OBSTACLE;
			    ptBranching = FALSE;
			    if (PtDelAlg())
				PtDeletionAlg();
			    else
				PtIncrementalAlg();
			    if (ptBranching)
				goto OBSTACLE;
			    ptFirst = NULL;
			    if (PtDelAlg())
				(void) PtrBlApply(TblTrn(0), TrnHowDelCheck);
			    else
				(void) PtrBlApply(TblTrn(0), TrnHowBest);
			    if (ptBranching)
				goto OBSTACLE;
			}
		    if (TrnVisible(ptFirst))
			goto OBSTACLE;
		    if (PtCFFD() && (!DivNow()) && !PtRecurs())
			{
			    BalTrDestroy(theNodeTree, FALSE);
			    theNodeTree = BalTrCreate(&lngBalTools);
			    if (!TrnVisible(trn))
				icold = TRUE;
			}
		    if (TrnGPurIn(ptFirst))
			LSTFOR (TrnGPurIn(ptFirst), trArc, TRARC)
			    PlcDCount(TrArcGPlc(trArc), TrArcGWeight(trArc));
		    if (TrnGPurOut(ptFirst))
			LSTFOR (TrnGPurOut(ptFirst), trArc, TRARC)
			    PlcICount(TrArcGPlc(trArc), TrArcGWeight(trArc));
		    if (TrnGMosIn(ptFirst))
			LSTFOR (TrnGMosIn(ptFirst), trDrc, TRDRC)
			    PlcDCount(TrDrcGPlc(trDrc),
				      TrDrcGIn(trDrc) - TrDrcGOut(trDrc));
		    if (TrnGMosOut(ptFirst))
			LSTFOR (TrnGMosOut(ptFirst), trDrc, TRDRC)
			    PlcICount(TrDrcGPlc(trDrc),
				      TrDrcGOut(trDrc) - TrDrcGIn(trDrc));
		    if (PtSymm())
			{
			    theBack = (unsigned long) -1L;
			    (void) PlcEncode(TblPlc(), EngABuffer(),
					     EngABufSize());
			    ndAddr = GphFndNode(graph, EngBuffer(), &theBack);
			    if (ndAddr == (unsigned long) -1L)
				{
				    b_a_c_k = &theBack;
				    if (BalTrApply(PlcSymmetries(), FndEquiv))
					{
					    ndAddr = ptEqNdAddr;
					    ptFaithful = FALSE;
					}
				    else
					{
					    len = PlcEncode(TblPlc(),
							    EngABuffer(),
							    EngABufSize());
					    ndAddr = GphPutNode(graph,
								EngBuffer(),
								len, &theBack);
					    ptFaithful = TRUE;
					}
				}
			    else
				ptFaithful = TRUE;
			}
		    else
			{
			    len = PlcEncode(TblPlc(), EngABuffer(),
					    EngABufSize());
			    ndAddr = GphPutNode(graph, EngBuffer(),
						len, &theBack);
			}
		    (void) LstInsLast(speculation, (char *)
				      PlArcCons(ptFirst, ndAddr));
		    if (((unsigned long) -1L) != theBack)
			{
			    if (ndAddr >=
				PlArcGWeight((PLARC *) LstFirst(speculation)))
				goto LATE_LOOP;
			    goto OLD;
			}
		    if (PtCFFD() || DivNow())
			(void) BalTrInsert(theNodeTree, (char *)
					   PlArcAWeight((PLARC *)
							LstLast(speculation)),
					   (char *)
					   PlArcAWeight((PLARC *)
							LstLast(speculation)),
					   TRUE);
		}
	OBSTACLE: ;
	    if (PtCFFD() || DivNow())
		PthSEnd(thePath, end);
	    if (LstLast(speculation) == LstFirst(speculation))
		{
		    if (PtSleep())
			GphPush(graph, ndAddr, Pt4SlpSet());
		    goto RESUME;
		}
	    GphErase(graph, PlArcGWeight((PLARC *) LstFirst(speculation)),
		     EngBuffer());
	    len = PlcEncode(TblPlc(), EngABuffer(), EngABufSize());
	    ndAddr = GphPutNode(graph, EngBuffer(), len, &theBack);
	    if (((unsigned long) -1L) != theBack)
		{
		    (void) fprintf(_out, "\n");
		    (void) fflush(_out);
		    (void) fprintf(_err, "\nInternal error in TrnFire\n");
		    (void) fflush(_err);
		    abort();
		}
	    len = 0;
	    LSTFOR (speculation, plArc, PLARC)
		{
		    if (PtSymm())
			a = TrnGNumber(PlArcGTrn(plArc)) << 1;
		    else
			a = TrnGNumber(PlArcGTrn(plArc));
		    IncLen(a, len);
		}
	    if (len >= EngBufSize())
		{
		    EngSBufSize((len >> 1) + len + 1);
		    Deallocate((char *) EngBuffer());
		    EngSBuffer((DATATYPE *) AllocArray(EngBufSize(),
						       sizeof(DATATYPE)));
		}
	    pointer = EngBuffer();
	    LSTFOR (speculation, plArc, PLARC)
		{
		    if (PtSymm())
			a = TrnGNumber(PlArcGTrn(plArc)) << 1;
		    else
			a = TrnGNumber(PlArcGTrn(plArc));
		    PutValue(a, pointer);
		}
	    *pointer = (DATATYPE) 0;
	    GphChLastArrow(graph, EngBuffer(), len, ndAddr,
			   (unsigned long *) NULL, icold);
	    if (PlArcGWeight((PLARC *) LstFirst(speculation)) == theNdFirst)
		theNdFirst = ndAddr;
	    if (PtSleep())
		GphPush(graph, ndAddr, Pt4SlpSet());
	    goto RESUME;
	TERMINAL: ;
	    if (PtCFFD() || DivNow())
		PthSEnd(thePath, end);
	    if (LstLast(speculation) != LstFirst(speculation))
		{
		    GphErase(graph, PlArcGWeight((PLARC *)
						 LstFirst(speculation)),
			     EngBuffer());
		    len = PlcEncode(TblPlc(), EngABuffer(), EngABufSize());
		    ndAddr = GphPutNode(graph, EngBuffer(), len, &theBack);
		    if (((unsigned long) -1L) != theBack)
			{
			    (void) fprintf(_out, "\n");
			    (void) fflush(_out);
			    (void) fprintf(_err,
					   "\nInternal error in TrnFire\n");
			    (void) fflush(_err);
			    abort();
			}
		    len = 0;
		    LSTFOR (speculation, plArc, PLARC)
			{
			    if (PtSymm())
				a = TrnGNumber(PlArcGTrn(plArc)) << 1;
			    else
				a = TrnGNumber(PlArcGTrn(plArc));
			    IncLen(a, len);
			}
		    if (len >= EngBufSize())
			{
			    EngSBufSize((len >> 1) + len + 1);
			    Deallocate((char *) EngBuffer());
			    EngSBuffer((DATATYPE *)
				       AllocArray(EngBufSize(),
						  sizeof(DATATYPE)));
			}
		    pointer = EngBuffer();
		    LSTFOR (speculation, plArc, PLARC)
			{
			    if (PtSymm())
				a = TrnGNumber(PlArcGTrn(plArc)) << 1;
			    else
				a = TrnGNumber(PlArcGTrn(plArc));
			    PutValue(a, pointer);
			}
		    *pointer = (DATATYPE) 0;
		    GphChLastArrow(graph, EngBuffer(), len, ndAddr,
				   (unsigned long *) NULL, icold);
		    if (PlArcGWeight((PLARC *) LstFirst(speculation))
			== theNdFirst)
			theNdFirst = ndAddr;
		}
	    GphAddTermNode(graph, ndAddr);
	    if (!PtSleep())
		GphIncGenCount(graph);
	    if (theDots && (graph == theGraph))
		DotOutput();
	    if (PtFlyPlc() && PlcDeadlock(PtFlyPlc()))
		{
		    LstDestroy(speculation, TRUE);
		    BalTrDestroy(theNodeTree, FALSE);
		    BalTrDestroy(tree, FALSE);
		    PtGphUpdate();
		    (void) fprintf(_out, "\nDeadlock reached\n");
		    GphSetSeed(graph, PtTerm());
		    ExOnTheFly(graph, PthCreate(ndAddr), TRUE);
		}
	    goto RESUME;
	LATE_LOOP: ;
	    if (PtCFFD() || DivNow())
		PthSEnd(thePath, end);
	    if (ndAddr == PlArcGWeight((PLARC *) LstBegin(speculation)))
		{
		    if (LstNext(speculation) != LstLast(speculation))
			GphErase(graph, PlArcGWeight((PLARC *)
						     LstNext(speculation)),
				 EngBuffer());
		}
	    else
		{
		    GphErase(graph, PlArcGWeight((PLARC *)
						 LstFirst(speculation)),
			     EngBuffer());
		    len = PlcEncode((PtSymm() && !ptFaithful)
				    ? DupTblPlc() : TblPlc(),
				    EngABuffer(), EngABufSize());
		    ndAddr = GphPutNode(graph, EngBuffer(), len, &theBack);
		    if (((unsigned long) -1L) != theBack)
			{
			    (void) fprintf(_out, "\n");
			    (void) fflush(_out);
			    (void) fprintf(_err,
					   "\nInternal error in TrnFire\n");
			    (void) fflush(_err);
			    abort();
			}
		    len = 0;
		    LSTFOR (speculation, plArc, PLARC)
			{
			    if (PtSymm())
				a = TrnGNumber(PlArcGTrn(plArc)) << 1;
			    else
				a = TrnGNumber(PlArcGTrn(plArc));
			    IncLen(a, len);
			    if (PlArcGWeight(plArc) ==
				PlArcGWeight((PLARC *) LstLast(speculation)))
				break;
			}
		    if (len >= EngBufSize())
			{
			    EngSBufSize((len >> 1) + len + 1);
			    Deallocate((char *) EngBuffer());
			    EngSBuffer((DATATYPE *)
				       AllocArray(EngBufSize(),
						  sizeof(DATATYPE)));
			}
		    pointer = EngBuffer();
		    LSTFOR (speculation, plArc, PLARC)
			{
			    if (PtSymm())
				a = TrnGNumber(PlArcGTrn(plArc)) << 1;
			    else
				a = TrnGNumber(PlArcGTrn(plArc));
			    PutValue(a, pointer);
			    if (PlArcGWeight(plArc) ==
				PlArcGWeight((PLARC *) LstLast(speculation)))
				break;
			}
		    *pointer = (DATATYPE) 0;
		    GphChLastArrow(graph, EngBuffer(), len, ndAddr,
				   &theBack, icold);
		    if (PlArcGWeight((PLARC *) LstFirst(speculation))
			== theNdFirst)
			theNdFirst = ndAddr;
		}
	    len = 0;
	    for (plArc = (PLARC *) LstStep(speculation),
		 cursor = LstGet(speculation); plArc;
		 plArc = (PLARC *) LstStep(speculation))
		{
		    if (PtSymm())
			a = (TrnGNumber(PlArcGTrn(plArc)) << 1) |
			    ((LstNext(speculation) || ptFaithful) ? 0L : 1L);
		    else
			a = TrnGNumber(PlArcGTrn(plArc));
		    IncLen(a, len);
		}
	    if (len >= EngBufSize())
		{
		    EngSBufSize((len >> 1) + len + 1);
		    Deallocate((char *) EngBuffer());
		    EngSBuffer((DATATYPE *) AllocArray(EngBufSize(),
						       sizeof(DATATYPE)));
		}
	    pointer = EngBuffer();
	    for (LstSet(speculation, cursor),
		 plArc = (PLARC *) LstThis(speculation); plArc;
		 plArc = (PLARC *) LstStep(speculation))
		{
		    if (PtSymm())
			a = (TrnGNumber(PlArcGTrn(plArc)) << 1) |
			    ((LstNext(speculation) || ptFaithful) ? 0L : 1L);
		    else
			a = TrnGNumber(PlArcGTrn(plArc));
		    PutValue(a, pointer);
		}
	    *pointer = (DATATYPE) 0;
	    awAddr = GphPutArrow(graph, EngBuffer(), len, (unsigned long) 0L,
				 ndAddr, ndAddr, (unsigned long) -1L,
				 theBack, TRUE);
	    GphUpdate(graph, ndAddr, (unsigned long) 1L, awAddr, Pt4SlpSet());
	    if (!PtSleep())
		GphIncGenCount(graph);
	    if (theDots && (graph == theGraph))
		DotOutput();
	    if (DivNow() &&
		BalTrElement(theNodeTree, (char *)
			     PlArcAWeight((PLARC *) LstLast(speculation))))
		{
		    LstDestroy(speculation, TRUE);
		    BalTrDestroy(theNodeTree, FALSE);
		    BalTrDestroy(tree, FALSE);
		    PtGphUpdate();
		    PthDiscard(thePath);
		    thePath = PthCreate(ndAddr);
		    PthInsLast(thePath, awAddr, ndAddr);
		    (void) fprintf(_out, "\nLivelock reached\n");
		    GphSetSeed(graph, PtTerm());
		    ExOnTheFly(graph, thePath, TRUE);
		}
	    goto RESUME;
	OLD: ;
	    if (PtCFFD() || DivNow())
		PthSEnd(thePath, end);
	    GphErase(graph, PlArcGWeight((PLARC *) LstFirst(speculation)),
		     EngBuffer());
	    len = 0;
	    LSTFOR (speculation, plArc, PLARC)
		{
		    if (PtSymm())
			a = (TrnGNumber(PlArcGTrn(plArc)) << 1) |
			    ((LstNext(speculation) || ptFaithful) ? 0L : 1L);
		    else
			a = TrnGNumber(PlArcGTrn(plArc));
		    IncLen(a, len);
		}
	    if (len >= EngBufSize())
		{
		    EngSBufSize((len >> 1) + len + 1);
		    Deallocate((char *) EngBuffer());
		    EngSBuffer((DATATYPE *) AllocArray(EngBufSize(),
						       sizeof(DATATYPE)));
		}
	    pointer = EngBuffer();
	    LSTFOR (speculation, plArc, PLARC)
		{
		    if (PtSymm())
			a = (TrnGNumber(PlArcGTrn(plArc)) << 1) |
			    ((LstNext(speculation) || ptFaithful) ? 0L : 1L);
		    else
			a = TrnGNumber(PlArcGTrn(plArc));
		    PutValue(a, pointer);
		}
	    *pointer = (DATATYPE) 0;
	    GphChLastArrow(graph, EngBuffer(), len, ndAddr,
			   (unsigned long *) NULL, icold);
	    if (PlArcGWeight((PLARC *) LstFirst(speculation)) == theNdFirst)
		theNdFirst = ndAddr;
	    if (PtSleep())
		GphPush(graph, ndAddr, Pt4SlpSet());
	    if (DivNow() &&
		BalTrElement(theNodeTree, (char *)
			     PlArcAWeight((PLARC *) LstLast(speculation))))
		{
		    LstDestroy(speculation, TRUE);
		    BalTrDestroy(theNodeTree, FALSE);
		    BalTrDestroy(tree, FALSE);
		    PtGphUpdate();
		    PthInsLast(thePath, awAddr, ndAddr);
		    (void) fprintf(_out, "\nLivelock reached\n");
		    GphSetSeed(graph, PtTerm());
		    ExOnTheFly(graph, thePath, TRUE);
		}
	RESUME: ;
	    if (PtCFFD() || DivNow())
		{
		    BalTrDestroy(theNodeTree, FALSE);
		    theNodeTree = tree;
		}
	    LSTFOR (speculation, plArc, PLARC)
		{
		    if (plArc == (PLARC *) LstFirst(speculation))
			continue;
		    if (TrnGPurIn(PlArcGTrn(plArc)))
			LSTFOR (TrnGPurIn(PlArcGTrn(plArc)), trArc, TRARC)
			    PlcICount(TrArcGPlc(trArc),
				      TrArcGWeight(trArc));
		    if (TrnGPurOut(PlArcGTrn(plArc)))
			LSTFOR (TrnGPurOut(PlArcGTrn(plArc)), trArc, TRARC)
			    PlcDCount(TrArcGPlc(trArc),
				      TrArcGWeight(trArc));
		    if (TrnGMosIn(PlArcGTrn(plArc)))
			LSTFOR (TrnGMosIn(PlArcGTrn(plArc)), trDrc, TRDRC)
			    PlcICount(TrDrcGPlc(trDrc),
				      TrDrcGIn(trDrc) - TrDrcGOut(trDrc));
		    if (TrnGMosOut(PlArcGTrn(plArc)))
			LSTFOR (TrnGMosOut(PlArcGTrn(plArc)), trDrc, TRDRC)
			    PlcDCount(TrDrcGPlc(trDrc),
				      TrDrcGOut(trDrc) - TrDrcGIn(trDrc));
		}
	    LstDestroy(speculation, TRUE);
	    (void) PtrBlApply(TblTrn(0), TrnRestore);
	    if (recurs)
		SetPtRecurs();
	    else
		ClPtRecurs();
	    PtSetBest(best);
	    if (divNow)
		SetDivNow();
	    else
		ClDivNow();
	    if (!PtSleep())
		ptDoIt = TRUE;
	}
    else if (PtSleep())
	GphPush(graph, ndAddr, Pt4SlpSet());
    if (TrnGPurIn(trn))
	LSTFOR (TrnGPurIn(trn), trArc, TRARC)
	    PlcICount(TrArcGPlc(trArc), TrArcGWeight(trArc));
    if (TrnGPurOut(trn))
	LSTFOR (TrnGPurOut(trn), trArc, TRARC)
	    PlcDCount(TrArcGPlc(trArc), TrArcGWeight(trArc));
    if (TrnGMosIn(trn))
	LSTFOR (TrnGMosIn(trn), trDrc, TRDRC)
	    PlcICount(TrDrcGPlc(trDrc),
		      TrDrcGIn(trDrc) - TrDrcGOut(trDrc));
    if (TrnGMosOut(trn))
	LSTFOR (TrnGMosOut(trn), trDrc, TRDRC)
	    PlcDCount(TrDrcGPlc(trDrc),
		      TrDrcGOut(trDrc) - TrDrcGIn(trDrc));
}

/***************************************************************************
*
* Function:  TrnLookStack
* Purpose:   In the CFFD-preserving version of the stubborn set method,
*            this routine checks if a possible firing of a transition
*            closes an invisible loop.
* Arguments: trn - the transition
* Returns:
* See also:  TrnRedInc, TrnRedDel, PtRedInc, PtRedDel
*
****************************************************************************
*/
FE1(void, TrnLookStack,
    TRN *, trn)
{
    TRARC *trArc;
    TRDRC *trDrc;
    unsigned long ndAddr;

    if (TrnGPurIn(trn))
	{
	    LSTFOR (TrnGPurIn(trn), trArc, TRARC)
		PlcDCount(TrArcGPlc(trArc), TrArcGWeight(trArc));
	}
    if (TrnGPurOut(trn))
	{
	    LSTFOR (TrnGPurOut(trn), trArc, TRARC)
		PlcICount(TrArcGPlc(trArc), TrArcGWeight(trArc));
	}
    if (TrnGMosIn(trn))
	{
	    LSTFOR (TrnGMosIn(trn), trDrc, TRDRC)
		PlcDCount(TrDrcGPlc(trDrc),
			  TrDrcGIn(trDrc) - TrDrcGOut(trDrc));
	}
    if (TrnGMosOut(trn))
	{
	    LSTFOR (TrnGMosOut(trn), trDrc, TRDRC)
		PlcICount(TrDrcGPlc(trDrc),
			  TrDrcGOut(trDrc) - TrDrcGIn(trDrc));
	}
    (void) PlcEncode(TblPlc(), EngABuffer(), EngABufSize());
    ndAddr = GphFndNode(Buchi() ? the2Graph : theGraph,
			EngBuffer(), (unsigned long *) NULL);
    if (PtSymm() && (ndAddr == (unsigned long) -1L))
	{
	    b_a_c_k = (unsigned long *) NULL;
	    if (BalTrApply(PlcSymmetries(), FndEquiv))
		ndAddr = ptEqNdAddr;
	}
    if ((ndAddr != (unsigned long) -1L) &&
	((ndAddr == PthEnd(thePath)) ||
	 BalTrElement(theNodeTree, (char *) &ndAddr)))
	ClPtRecurs();
    if (TrnGPurIn(trn))
	{
	    LSTFOR (TrnGPurIn(trn), trArc, TRARC)
		PlcICount(TrArcGPlc(trArc), TrArcGWeight(trArc));
	}
    if (TrnGPurOut(trn))
	{
	    LSTFOR (TrnGPurOut(trn), trArc, TRARC)
		PlcDCount(TrArcGPlc(trArc), TrArcGWeight(trArc));
	}
    if (TrnGMosIn(trn))
	{
	    LSTFOR (TrnGMosIn(trn), trDrc, TRDRC)
		PlcICount(TrDrcGPlc(trDrc),
			  TrDrcGIn(trDrc) - TrDrcGOut(trDrc));
	}
    if (TrnGMosOut(trn))
	{
	    LSTFOR (TrnGMosOut(trn), trDrc, TRDRC)
		PlcDCount(TrDrcGPlc(trDrc),
			  TrDrcGOut(trDrc) - TrDrcGIn(trDrc));
	}
}
