/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  ptelem2.c
*
* Part of: PROD's programs araprod and "gphgen"
*
* Author:  Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     This is an auxiliary module of ptnet.c.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ptrbl.h"
#include "baltree.h"
#include "bdd.h"
#include "gphutil.h"
#include "graph.h"
#include "mark.h"
#include "encode.h"
#include "ptnet.h"
#include "ptsymm.h"
#include "error.h"
#include "cmdnm.h"

/* All these Cons, Copy, Discard, Dump, KeyCmp, Compare, Tools and
 * balTools are trivial and similar to those explained in detail
 * in modules such as gphutil.c. */

FE0(PLC *, PlcCons)
{
    PLC *plc;

    plc = PlcAlloc();
    PlcSCount(plc, (unsigned long) 0L);
    PlcCKey(plc);
    PlcCPurIn(plc);
    PlcCPurOut(plc);
    PlcCNeutral(plc);
    PlcCMosIn(plc);
    PlcCMosOut(plc);
    if (PtDelAlg())
#ifdef SMODELS
	{
	    if (PtSmodels())
		{
		    PlcCOutMax(plc);
		    PlcCGamma(plc);
		    PlcCIneqs(plc);
		}
	    else
#endif
		PlcCOrPredAnd(plc);
#ifdef SMODELS
	}
#endif
    PlcCFlag(plc);
    return(plc);
}

FE1(PLC *, PlcCopy,
    PLC *, plc)
{
    PLC *copy;

    copy = PlcAlloc();
    PlcSCount(copy, PlcGCount(plc));
    PlcSFlag(copy, PlcGFlag(plc));
    if (PlcGKey(plc))
	PlcSKey(copy, DataCopy(PlcGKey(plc)));
    else
	PlcCKey(copy);
    if (PlcBuchi(plc))
	PlcZNext(copy, LstCopy(PlcGNext(plc), FALSE));
    else
	{
	    if (PlcGPurIn(plc))
		PlcZPurIn(copy, LstCopy(PlcGPurIn(plc), TRUE));
	    else
		PlcCPurIn(copy);
	    if (PlcGPurOut(plc))
		PlcZPurOut(copy, LstCopy(PlcGPurOut(plc), TRUE));
	    else
		PlcCPurOut(copy);
	    if (PlcGNeutral(plc))
		PlcZNeutral(copy, LstCopy(PlcGNeutral(plc), TRUE));
	    else
		PlcCNeutral(copy);
	    if (PlcGMosIn(plc))
		PlcZMosIn(copy, LstCopy(PlcGMosIn(plc), TRUE));
	    else
		PlcCMosIn(copy);
	    if (PlcGMosOut(plc))
		PlcZMosOut(copy, LstCopy(PlcGMosOut(plc), TRUE));
	    else
		PlcCMosOut(copy);
	}
    if (PtDelAlg())
	{
#ifdef SMODELS
	    if (PtSmodels())
		{
		    PlcSOutMax(copy, PlcGOutMax(plc));
		    PlcCGamma(copy);
		    PlcCIneqs(copy);
		}
	    else
		{
#endif
		    if (PlcGOrPredAnd(plc))
			PlcZOrPredAnd(copy, LstCopy(PlcGOrPredAnd(plc),
						    FALSE));
		    else
			PlcCOrPredAnd(copy);
#ifdef SMODELS
		}
#endif
	}
    else if (PtSymm())
	PlcSInd(copy, PlcGInd(plc));
    return(copy);
}

FE1(void, PlcDiscard,
    PLC *, plc)
{
    if (PlcGKey(plc))
	PlcDKey(plc);
    if (PlcBuchi(plc))
	LstDestroy(PlcGNext(plc), FALSE);
    else
	{
	    if (PlcGPurIn(plc))
		LstDestroy(PlcGPurIn(plc), TRUE);
	    if (PlcGPurOut(plc))
		LstDestroy(PlcGPurOut(plc), TRUE);
	    if (PlcGNeutral(plc))
		LstDestroy(PlcGNeutral(plc), TRUE);
	    if (PlcGMosIn(plc))
		LstDestroy(PlcGMosIn(plc), TRUE);
	    if (PlcGMosOut(plc))
		LstDestroy(PlcGMosOut(plc), TRUE);
	}
    if (PtDelAlg() &&
#ifdef SMODELS
	(!PtSmodels()) &&
#endif
	PlcGOrPredAnd(plc))
	LstDestroy(PlcGOrPredAnd(plc), FALSE);
#ifdef SMODELS
    if (PtSmodels() && PlcGIneqs(plc))
	BalTrDestroy(PlcGIneqs(plc), TRUE);
#endif
    Deallocate((char *) plc);
}

#ifdef DEBUG
FE2(void, PlcDump,
    PLC *, plc,
    unsigned, depth)
{
    TRN *trn;
    unsigned p = 0;
    TUPLE t_u_p_l_e;
    TUPLE *tuple;
    PLC *pl2c;

    DumpIndent(depth);
    if (PtSymm() && !PtDelAlg())
	(void) fprintf(_err, "PLC 0x%lx count %lu flag %u ind %lu\n",
		       (unsigned long) plc, PlcGCount(plc), PlcGFlag(plc),
		       PlcGInd(plc));
    else
	(void) fprintf(_err, "PLC 0x%lx count %lu flag %u\n",
		       (unsigned long) plc, PlcGCount(plc), PlcGFlag(plc));
    DumpIndent(depth + 1);
    if (PlcGKey(plc))
	{
	    tuple = &t_u_p_l_e;
	    TupSArity(tuple, (unsigned) 0);
	    TupSArray(tuple, (DOMAIN *) NULL);
	    TupDecode(&p, PlcGKey(plc), tuple);
	    (void) fprintf(_err, "place %u\n", p);
	    TupDump(tuple, depth + 1);
	    if (TupGArity(tuple))
		Deallocate((char *) TupGArray(tuple));
	}
    DumpIndent(depth + 1);
    if (PlcBuchi(plc))
	{
	    (void) fprintf(_err, "next\n");
	    LSTFOR (PlcGNext(plc), pl2c, PLC)
		{
		    DumpIndent(depth + 2);
		    (void) fprintf(_err, "PLC 0x%lx\n", (unsigned long) pl2c);
		}
	}
    else
	{
	    (void) fprintf(_err, "pureIn\n");
	    if (PlcGPurIn(plc))
		LstDump(PlcGPurIn(plc), depth + 2);
	    DumpIndent(depth + 1);
	    (void) fprintf(_err, "pureOut\n");
	    if (PlcGPurOut(plc))
		LstDump(PlcGPurOut(plc), depth + 2);
	    DumpIndent(depth + 1);
	    (void) fprintf(_err, "neutral\n");
	    if (PlcGNeutral(plc))
		LstDump(PlcGNeutral(plc), depth + 2);
	    DumpIndent(depth + 1);
	    (void) fprintf(_err, "mostIn\n");
	    if (PlcGMosIn(plc))
		LstDump(PlcGMosIn(plc), depth + 2);
	    DumpIndent(depth + 1);
	    (void) fprintf(_err, "mostOut\n");
	    if (PlcGMosOut(plc))
		LstDump(PlcGMosOut(plc), depth + 2);
	}
    if (PtDelAlg()
#ifdef SMODELS
	&& !PtSmodels()
#endif
	)
	{
	    DumpIndent(depth + 1);
	    (void) fprintf(_err, "orPredAnd\n");
	    if (PlcGOrPredAnd(plc))
		{
		    LSTFOR (PlcGOrPredAnd(plc), trn, TRN)
			{
			    DumpIndent(depth + 2);
			    (void) fprintf(_err, "TRN 0x%lx\n",
					   (unsigned long) trn);
			}
		}
	}
    DumpIndent(depth);
    (void) fprintf(_err, "End of PLC\n");
    (void) fflush(_err);
}
#endif

FE2(int, PlcKeyCmp,
    PLC *, plc,
    DATATYPE *, key)
{
    return(BTupComp(PlcGKey(plc), key));
}

TOOLS plcTools =
{ (CFUNC) PlcCopy, PlcDiscard
#ifdef DEBUG
, PlcDump
#endif
 };

BALTOOLS plcBalTools =
{ (CFUNC) PlcCopy, PlcDiscard
#ifdef DEBUG
, PlcDump
#endif
, PlcKeyCmp, (FOOL) NULL };

FE0(TRN *, TrnCons)
{
    TRN *trn;

    trn = TrnAlloc();
    TrnCKey(trn);
    TrnCNumber(trn);
    TrnCFlag(trn);
    TrnSWhere(trn, (unsigned) 0);
    TrnCCursor(trn);
    TrnCDown(trn);
    TrnCBlocks(trn);
    TrnCPurIn(trn);
    TrnCPurOut(trn);
    TrnCNeutral(trn);
    TrnCMosIn(trn);
    TrnCMosOut(trn);
    if (PtDelAlg())
	{
#ifdef SMODELS
	    if (PtSmodels())
		TrnCAtoms(trn);
	    else
		{
#endif
		    TrnCPlcPredAO(trn);
		    TrnCOrStrongPredAO(trn);
		    TrnCOrWeakPredAO(trn);
		    TrnCNOrSucc(trn);
		    TrnCDupNOrSucc(trn);
		    TrnCBinOr(trn);
#ifdef SMODELS
		}
#endif
	}
    else if (PtStub())
	{
	    TrnSVal(trn, (unsigned long) 0L);
	    TrnSDupVal(trn, (unsigned long) 0L);
            TrnSMin(trn, (unsigned long) 0L);
	}
    else
	TrnCHigher(trn);
    return(trn);
}

FE1(TRN *, TrnCopy,
    TRN *, trn)
{
    TRN *copy;
    unsigned i;

    copy = TrnAlloc();
    if (TrnGKey(trn))
	TrnSKey(copy, DataCopy(TrnGKey(trn)));
    else
	TrnCKey(copy);
    TrnSNumber(copy, TrnGNumber(trn));
    TrnSFlag(copy, TrnGFlag(trn));
    TrnSWhere(trn, TrnGWhere(trn));
    TrnCCursor(trn);
    TrnCDown(trn);
    if (TrnBlocks(trn))
        {
            for (i = 0; TrnGBlock(trn, i) != (unsigned) -1; i++)
                ;
            TrnABlocks(copy, i + 1);
            while (TRUE)
                {
                    TrnSBlock(copy, i, TrnGBlock(trn, i));
                    if (!i)
                        break;
                    i--;
                }
        }
    else
        TrnCBlocks(trn);
    if (TrnGPurIn(trn))
	TrnZPurIn(copy, LstCopy(TrnGPurIn(trn), TRUE));
    else
	TrnCPurIn(copy);
    if (TrnGPurOut(trn))
	TrnZPurOut(copy, LstCopy(TrnGPurOut(trn), TRUE));
    else
	TrnCPurOut(copy);
    if (TrnGNeutral(trn))
	TrnZNeutral(copy, LstCopy(TrnGNeutral(trn), TRUE));
    else
	TrnCNeutral(copy);
    if (TrnGMosIn(trn))
	TrnZMosIn(copy, LstCopy(TrnGMosIn(trn), TRUE));
    else
	TrnCMosIn(copy);
    if (TrnGMosOut(trn))
	TrnZMosOut(copy, LstCopy(TrnGMosOut(trn), TRUE));
    else
	TrnCMosOut(copy);
    if (PtDelAlg())
	{
#ifdef SMODELS
	    if (PtSmodels())
		TrnCAtoms(copy);
	    else
		{
#endif
		    if (TrnGPlcPredAO(trn))
			TrnZPlcPredAO(copy, LstCopy(TrnGPlcPredAO(trn),
						    FALSE));
		    else
			TrnCPlcPredAO(copy);
		    if (TrnGOrStrongPredAO(trn))
			TrnZOrStrongPredAO(copy,
					   LstCopy(TrnGOrStrongPredAO(trn),
						   FALSE));
		    else
			TrnCOrStrongPredAO(copy);
		    if (TrnGOrWeakPredAO(trn))
			TrnZOrWeakPredAO(copy, LstCopy(TrnGOrWeakPredAO(trn),
						       FALSE));
		    else
			TrnCOrWeakPredAO(copy);
		    TrnSNOrSucc(copy, TrnGNOrSucc(trn));
		    TrnSDupNOrSucc(copy, TrnGDupNOrSucc(trn));
		    TrnSBinOr(copy, TrnGBinOr(trn));
#ifdef SMODELS
		}
#endif
	}
    else if (PtStub())
	{
	    TrnSVal(copy, TrnGVal(trn));
            TrnSDupVal(copy, TrnGDupVal(trn));
            TrnSMin(copy, TrnGMin(trn));
	}
    else
	{
	    if (TrnGHigher(trn))
		TrnZHigher(copy, LstCopy(TrnGHigher(trn), FALSE));
	    else
		TrnCHigher(copy);
	}
    return(copy);
}

FE1(void, TrnDiscard,
    TRN *, trn)
{
    if (TrnGKey(trn))
	TrnDKey(trn);
    if (TrnBlocks(trn))
        TrnDBlocks(trn);
    if (TrnGPurIn(trn))
	LstDestroy(TrnGPurIn(trn), TRUE);
    if (TrnGPurOut(trn))
	LstDestroy(TrnGPurOut(trn), TRUE);
    if (TrnGNeutral(trn))
	LstDestroy(TrnGNeutral(trn), TRUE);
    if (TrnGMosIn(trn))
	LstDestroy(TrnGMosIn(trn), TRUE);
    if (TrnGMosOut(trn))
	LstDestroy(TrnGMosOut(trn), TRUE);
    if (PtDelAlg())
	{
#ifdef SMODELS
	    if (!PtSmodels())
		{
#endif
		    if (TrnGPlcPredAO(trn))
			LstDestroy(TrnGPlcPredAO(trn), FALSE);
		    if (TrnGOrStrongPredAO(trn))
			LstDestroy(TrnGOrStrongPredAO(trn), FALSE);
		    if (TrnGOrWeakPredAO(trn))
			LstDestroy(TrnGOrWeakPredAO(trn), FALSE);
#ifdef SMODELS
		}
#endif
	}
    else if ((!PtStub()) && TrnGHigher(trn))
	LstDestroy(TrnGHigher(trn), FALSE);
    Deallocate((char *) trn);
}

#ifdef DEBUG
FE2(void, TrnDump,
    TRN *, trn,
    unsigned, depth)
{
    TRN *htrn;
    PLC *plc;
    BINOR *binOr;
    DOMAIN *instance;
    unsigned t = 0;
    unsigned p = 0;
    unsigned i;

    DumpIndent(depth);
    (void) fprintf(_err, "TRN 0x%lx number %lu flag %u", (unsigned long) trn,
		   TrnGNumber(trn), TrnGFlag(trn));
    if (PtDelAlg())
	{
#ifdef SMODELS
	    if (!PtSmodels())
#endif
		(void) fprintf(_err, " NOrSucc %lu DupNOrSucc %lu",
			       TrnGNOrSucc(trn), TrnGDupNOrSucc(trn));
	}
    else if (PtStub())
        (void) fprintf(_err, " val %lu dupVal %lu min %lu",
                       TrnGVal(trn), TrnGDupVal(trn), TrnGMin(trn));
    (void) fprintf(_err, "\n");
    if (TrnGKey(trn))
	{
	    DumpIndent(depth + 1);
	    instance = InsDecode(&t, &p, TrnGKey(trn));
	    (void) fprintf(_err, "transition %u priority %u\n", t, p);
	    DumpIndent(depth + 1);
	    (void) fprintf(_err, "instance");
	    for (i = 0; i < IngGProfile(t); i++)
		{
		    (void) fprintf(_err, " ");
		    (void) fprintf(_err, DOMFORMAT, instance[i]);
		}
	    (void) fprintf(_err, "\n");
	    Deallocate((char *) instance);
	}
    if (TrnBlocks(trn))
        {
            DumpIndent(depth + 1);
            (void) fprintf(_err, "blocks");
            for (i = 0; TrnGBlock(trn, i) != (unsigned) -1; i++)
                (void) fprintf(_err, " %u", TrnGBlock(trn, i));
	    (void) fprintf(_err, "\n");
        }
    DumpIndent(depth + 1);
    (void) fprintf(_err, "pureIn\n");
    if (TrnGPurIn(trn))
	LstDump(TrnGPurIn(trn), depth + 2);
    DumpIndent(depth + 1);
    (void) fprintf(_err, "pureOut\n");
    if (TrnGPurOut(trn))
	LstDump(TrnGPurOut(trn), depth + 2);
    DumpIndent(depth + 1);
    (void) fprintf(_err, "neutral\n");
    if (TrnGNeutral(trn))
	LstDump(TrnGNeutral(trn), depth + 2);
    DumpIndent(depth + 1);
    (void) fprintf(_err, "mostIn\n");
    if (TrnGMosIn(trn))
	LstDump(TrnGMosIn(trn), depth + 2);
    DumpIndent(depth + 1);
    (void) fprintf(_err, "mostOut\n");
    if (TrnGMosOut(trn))
	LstDump(TrnGMosOut(trn), depth + 2);
    DumpIndent(depth + 1);
    if (PtDelAlg())
	{
#ifdef SMODELS
	    if (!PtSmodels())
		{
#endif
		    (void) fprintf(_err, "plcPredAO\n");
		    if (TrnGPlcPredAO(trn))
			{
			    LSTFOR (TrnGPlcPredAO(trn), plc, PLC)
				{
				    DumpIndent(depth + 2);
				    (void) fprintf(_err, "PLC 0x%lx\n",
						   (unsigned long) plc);
				}
			}
		    DumpIndent(depth + 1);
		    (void) fprintf(_err, "orStrongPredAO\n");
		    if (TrnGOrStrongPredAO(trn))
			{
			    LSTFOR (TrnGOrStrongPredAO(trn), binOr, BINOR)
				{
				    DumpIndent(depth + 2);
				    (void) fprintf(_err,
						   "BINOR 0x%lx\n",
						   (unsigned long) binOr);
				}
			}
		    DumpIndent(depth + 1);
		    (void) fprintf(_err, "orWeakPredAO\n");
		    if (TrnGOrWeakPredAO(trn))
			{
			    LSTFOR (TrnGOrWeakPredAO(trn), binOr, BINOR)
				{
				    DumpIndent(depth + 2);
				    (void) fprintf(_err,
						   "BINOR 0x%lx\n",
						   (unsigned long) binOr);
				}
			}
#ifdef SMODELS
		}
#endif
	}
    else if (!PtStub())
	{
	    (void) fprintf(_err, "higher\n");
	    if (TrnGHigher(trn))
		{
		    LSTFOR (TrnGHigher(trn), htrn, TRN)
			{
			    DumpIndent(depth + 2);
			    (void) fprintf(_err, "TRN 0x%lx\n",
					   (unsigned long) htrn);
			}
		}
	}
    DumpIndent(depth);
    (void) fprintf(_err, "End of TRN\n");
    (void) fflush(_err);
}
#endif

FE2(int, TrnKeyCmp,
    TRN *, trn,
    DATATYPE *, key)
{
    return(InsComp(TrnGKey(trn), key));
}

TOOLS trnTools =
{ (CFUNC) TrnCopy, TrnDiscard
#ifdef DEBUG
, TrnDump
#endif
 };

BALTOOLS trnBalTools =
{ (CFUNC) TrnCopy, TrnDiscard
#ifdef DEBUG
, TrnDump
#endif
, TrnKeyCmp, (FOOL) NULL };

FE2(PLARC *, PlArcCons,
    TRN *, trn,
    unsigned long, weight)
{
    PLARC *plArc;

    plArc = PlArcAlloc();
    PlArcSTrn(plArc, trn);
    PlArcSWeight(plArc, weight);
    return(plArc);
}

FE1(PLARC *, PlArcCopy,
    PLARC *, plArc)
{
    PLARC *copy;

    copy = PlArcAlloc();
    PlArcSTrn(copy, PlArcGTrn(plArc));
    PlArcSWeight(copy, PlArcGWeight(plArc));
    return(copy);
}

#ifdef DEBUG
FE2(void, PlArcDump,
    PLARC *, plArc,
    unsigned, depth)
{
    DumpIndent(depth);
    (void) fprintf(_err, "PLARC weight %lu trn 0x%lx\n",
		   PlArcGWeight(plArc), (unsigned long) PlArcGTrn(plArc));
    (void) fflush(_err);
}
#endif

FE2(int, PlArcCompare,
    PLARC *, plArc1,
    PLARC *, plArc2)
{
    if (PlArcGWeight(plArc1) < PlArcGWeight(plArc2))
	return(-1);
    if (PlArcGWeight(plArc1) > PlArcGWeight(plArc2))
	return(1);
    return(0);
}

TOOLS plArcTools =
{ (CFUNC) PlArcCopy, Deallocate
#ifdef DEBUG
, PlArcDump
#endif
 };

BALTOOLS plArcBalTools =
{ (CFUNC) PlArcCopy, Deallocate
#ifdef DEBUG
, PlArcDump
#endif
, PlArcCompare, (FOOL) NULL };

FE3(PLDRC *, PlDrcCons,
    TRN *, trn,
    unsigned long, in,
    unsigned long, out)
{
    PLDRC *plDrc;

    plDrc = PlDrcAlloc();
    PlDrcSTrn(plDrc, trn);
    PlDrcSIn(plDrc, in);
    PlDrcSOut(plDrc, out);
    return(plDrc);
}

FE1(PLDRC *, PlDrcCopy,
    PLDRC *, plDrc)
{
    PLDRC *copy;

    copy = PlDrcAlloc();
    PlDrcSTrn(copy, PlDrcGTrn(plDrc));
    PlDrcSIn(copy, PlDrcGIn(plDrc));
    PlDrcSOut(copy, PlDrcGOut(plDrc));
    return(copy);
}

#ifdef DEBUG
FE2(void, PlDrcDump,
    PLDRC *, plDrc,
    unsigned, depth)
{
    DumpIndent(depth);
    (void) fprintf(_err, "PLDRC in %lu out %lu trn 0x%lx\n",
		   PlDrcGIn(plDrc), PlDrcGOut(plDrc),
		   (unsigned long) PlDrcGTrn(plDrc));
    (void) fflush(_err);
}
#endif

FE2(int, PlDrcCompare,
    PLDRC *, plDrc1,
    PLDRC *, plDrc2)
{
    if (PlDrcGIn(plDrc1) < PlDrcGIn(plDrc2))
	return(-1);
    if (PlDrcGIn(plDrc1) > PlDrcGIn(plDrc2))
	return(1);
    if (PlDrcGOut(plDrc1) < PlDrcGOut(plDrc2))
	return(-1);
    if (PlDrcGOut(plDrc1) > PlDrcGOut(plDrc2))
	return(1);
    return(0);
}

TOOLS plDrcTools =
{ (CFUNC) PlDrcCopy, Deallocate
#ifdef DEBUG
, PlDrcDump
#endif
 };

BALTOOLS plDrcBalTools =
{ (CFUNC) PlDrcCopy, Deallocate
#ifdef DEBUG
, PlDrcDump
#endif
, PlDrcCompare, (FOOL) NULL };

FE2(TRARC *, TrArcCons,
    PLC *, plc,
    unsigned long, weight)
{
    TRARC *trArc;

    trArc = TrArcAlloc();
    TrArcSPlc(trArc, plc);
    TrArcSWeight(trArc, weight);
#ifdef SMODELS
    TrArcCGamma(trArc);
#endif    
    return(trArc);
}

FE1(TRARC *, TrArcCopy,
    TRARC *, trArc)
{
    TRARC *copy;

    copy = TrArcAlloc();
    TrArcSPlc(copy, TrArcGPlc(trArc));
    TrArcSWeight(copy, TrArcGWeight(trArc));
#ifdef SMODELS
    TrArcSGamma(copy, TrArcGGamma(trArc));
#endif
    return(copy);
}

#ifdef DEBUG
FE2(void, TrArcDump,
    TRARC *, trArc,
    unsigned, depth)
{
    DumpIndent(depth);
    (void) fprintf(_err, "TRARC weight %lu trn 0x%lx\n",
		   TrArcGWeight(trArc), (unsigned long) TrArcGPlc(trArc));
    (void) fflush(_err);
}
#endif

FE2(int, TrArcCompare,
    TRARC *, trArc1,
    TRARC *, trArc2)
{
    if (TrArcGWeight(trArc1) < TrArcGWeight(trArc2))
	return(-1);
    if (TrArcGWeight(trArc1) > TrArcGWeight(trArc2))
	return(1);
    return(0);
}

TOOLS trArcTools =
{ (CFUNC) TrArcCopy, Deallocate
#ifdef DEBUG
, TrArcDump
#endif
 };

BALTOOLS trArcBalTools =
{ (CFUNC) TrArcCopy, Deallocate
#ifdef DEBUG
, TrArcDump
#endif
, TrArcCompare, (FOOL) NULL };

FE3(TRDRC *, TrDrcCons,
    PLC *, plc,
    unsigned long, in,
    unsigned long, out)
{
    TRDRC *trDrc;

    trDrc = TrDrcAlloc();
    TrDrcSPlc(trDrc, plc);
    TrDrcSIn(trDrc, in);
    TrDrcSOut(trDrc, out);
#ifdef SMODELS
    TrDrcCGamma(trDrc);
#endif    
    return(trDrc);
}

FE1(TRDRC *, TrDrcCopy,
    TRDRC *, trDrc)
{
    TRDRC *copy;

    copy = TrDrcAlloc();
    TrDrcSPlc(copy, TrDrcGPlc(trDrc));
    TrDrcSIn(copy, TrDrcGIn(trDrc));
    TrDrcSOut(copy, TrDrcGOut(trDrc));
#ifdef SMODELS
    TrDrcSGamma(copy, TrDrcGGamma(trDrc));
#endif
    return(copy);
}

#ifdef DEBUG
FE2(void, TrDrcDump,
    TRDRC *, trDrc,
    unsigned, depth)
{
    DumpIndent(depth);
    (void) fprintf(_err, "TRDRC in %lu out %lu trn 0x%lx\n",
		   TrDrcGIn(trDrc), TrDrcGOut(trDrc),
		   (unsigned long) TrDrcGPlc(trDrc));
    (void) fflush(_err);
}
#endif

FE2(int, TrDrcCompare,
    TRDRC *, trDrc1,
    TRDRC *, trDrc2)
{
    if (TrDrcGIn(trDrc1) < TrDrcGIn(trDrc2))
	return(-1);
    if (TrDrcGIn(trDrc1) > TrDrcGIn(trDrc2))
	return(1);
    if (TrDrcGOut(trDrc1) < TrDrcGOut(trDrc2))
	return(-1);
    if (TrDrcGOut(trDrc1) > TrDrcGOut(trDrc2))
	return(1);
    return(0);
}

TOOLS trDrcTools =
{ (CFUNC) TrDrcCopy, Deallocate
#ifdef DEBUG
, TrDrcDump
#endif
 };

BALTOOLS trDrcBalTools =
{ (CFUNC) TrDrcCopy, Deallocate
#ifdef DEBUG
, TrDrcDump
#endif
, TrDrcCompare, (FOOL) NULL };

FE1(BINOR *, BinOrCons,
    TRN *, trn)
{
    BINOR *binOr;

    binOr = BinOrAlloc();
    BinOrCFlag(binOr);
    BinOrSStrong(binOr);
    BinOrSWeak(binOr);
    BinOrSTrn(binOr, trn);
    return(binOr);
}

FE1(BINOR *, BinOrCopy,
    BINOR *, binOr)
{
    BINOR *copy;

    copy = BinOrAlloc();
    BinOrSFlag(copy, BinOrGFlag(binOr));
    BinOrSTrn(copy, BinOrGTrn(binOr));
    return(copy);
}

#ifdef DEBUG
FE2(void, BinOrDump,
    BINOR *, binOr,
    unsigned, depth)
{
    DumpIndent(depth);
    (void) fprintf(_err, "BINOR 0x%lx flag %u trn 0x%lx\n",
		   (unsigned long) binOr, BinOrGFlag(binOr),
		   (unsigned long) BinOrGTrn(binOr));
    (void) fflush(_err);
}
#endif

TOOLS binOrTools =
{ (CFUNC) BinOrCopy, Deallocate
#ifdef DEBUG
, BinOrDump
#endif
 };

static STATE *ptState;
static TUPLE *ptTuple;
static unsigned ptPlInd;

/***************************************************************************
*
* Function:  PlcState
* Purpose:   This is an auxiliary routine of PtState
* Arguments: plc - a place
* Returns:
* See also:  PtState
*
****************************************************************************
*/
FS1(BOOL, PlcState,
    PLC *, plc)
{
    if (PlcGCount(plc) != 0L)
	{
	    ptTuple = TupAlloc();
	    TupDecode(&ptPlInd, PlcGKey(plc), ptTuple);
	    MarInsLast(StaGMarking(ptState, ptPlInd),
		       MultupCons(PlcGCount(plc), ptTuple));
	}
    return(FALSE);
}

/***************************************************************************
*
* Function:  PtState
* Purpose:   This routine constructs a high level marking of a net
*            from a low level marking.
* Arguments:
* Returns:   the high level marking
* See also:
*
****************************************************************************
*/
FE0(STATE *, PtState)
{
    ptState = StaAlloc();
    for (ptPlInd = 0; ptPlInd < MagGNofPlaces(); ptPlInd++)
	StaIMarking(ptState, ptPlInd, MarCreate());
    (void) PtrBlApply(TblPlc(), PlcState);
    return(ptState);
}
