/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  ptnet2.c
*
* Part of: PROD's programs araprod and "gphgen"
*
* Author:  Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     See ptnet.c.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ptrbl.h"
#include "baltree.h"
#include "bdd.h"
#include "gphutil.h"
#include "graph.h"
#include "mark.h"
#include "encode.h"
#include "ptnet.h"
#include "ptsymm.h"

/***************************************************************************
*
* Function:  TrnYrd
* Purpose:   This is an auxiliary routine of TrnDry.
* Arguments: trn - a transition
* Returns:
* See also:  TrnDry
*
****************************************************************************
*/
FE1(void, TrnYrd,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        case 3:
            TrnSWhere(trn, (unsigned) 0);
            goto L3;
        }
    TrnSalt(trn);
    if (TrnGPlcPredAO(trn))
	LSTFOR (TrnGPlcPredAO(trn), ptPlc, PLC)
	    if ((!PlcSatisfied(ptPlc)) && (!PlcThirsty(ptPlc)))
		{
		    PlcSalt(ptPlc);
		    LSTFOR (PlcGOrPredAnd(ptPlc), ptTrn, TRN)
			if ((!TrnSatisfied(ptTrn)) && !TrnThirsty(ptTrn))
			    {
				TrnDNOrSucc(ptTrn);
				if (TrnGNOrSucc(ptTrn) == 0L)
				    {
                                        TrnSCursor(trn,
                                            LstGet(PlcGOrPredAnd(ptPlc)));
                                        TrnSWhere(trn, (unsigned) 1);
                                        return;
L1: ;
					ptPlc = (PLC *)
					    LstThis(TrnGPlcPredAO(trn));
                                        LstSet(PlcGOrPredAnd(ptPlc),
                                               TrnGCursor(trn));
				    }
			    }
		}
    if (TrnGOrStrongPredAO(trn))
	LSTFOR (TrnGOrStrongPredAO(trn), ptBinOr, BINOR)
	    if (BinOrGStrong(ptBinOr) && !BinOrHStrongNot(ptBinOr))
		{
		    BinOrTStrongNot(ptBinOr);
		    ptTrn = BinOrGTrn(ptBinOr);
		    if ((!TrnSatisfied(ptTrn)) && (!TrnThirsty(ptTrn)) &&
			(BinOrHWeakNot(ptBinOr) || !BinOrGWeak(ptBinOr)))
                        {
                            TrnSWhere(trn, (unsigned) 2);
                            return;
L2: ;
                        }
		}
    if (TrnGOrWeakPredAO(trn))
	LSTFOR (TrnGOrWeakPredAO(trn), ptBinOr, BINOR)
	    if (BinOrGWeak(ptBinOr) && !BinOrHWeakNot(ptBinOr))
		{
		    BinOrTWeakNot(ptBinOr);
		    ptTrn = BinOrGTrn(ptBinOr);
		    if ((!TrnSatisfied(ptTrn)) && (!TrnThirsty(ptTrn)) &&
			(BinOrHStrongNot(ptBinOr) || !BinOrGStrong(ptBinOr)))
                        {
                            TrnSWhere(trn, (unsigned) 3);
                            return;
L3: ;
                        }
		}
}

/***************************************************************************
*
* Function:  TrnDry
* Purpose:   This routine "simulates" the deletion of a transition
*            in the deletion algorithm of the stubborn set method.
* Arguments: trn - the transition
* Returns:
* See also:  TrnTryDel, TrnCancel, TrnSatWet
*
****************************************************************************
*/
FE1(void, TrnDry,
    TRN *, trn)
{
    while (TRUE)
        {
            TrnYrd(trn);
            if (TrnGWhere(trn))
                {
                    TrnSDown(ptTrn, trn);
                    trn = ptTrn;
                }
            else if (TrnGDown(trn))
                {
                    ptTrn = TrnGDown(trn);
                    TrnCDown(trn);
                    trn = ptTrn;
                }
            else
                return;
        }
}

/***************************************************************************
*
* Function:  TrnTryDel
* Purpose:   In the deletion algorithm of the stubborn set method,
*            this routine chooses a candidate to be deleted and
*            then "simulates" its deletion.
* Arguments: trn - a transition
* Returns:   TRUE if and only if trn is the intended candidate
* See also:  TrnDry, TrnCancel, TrnSatWet
*
****************************************************************************
*/
FE1(BOOL, TrnTryDel,
    TRN *, trn)
{
    if (!trn)
	return(TRUE);
    if (TrnSatisfied(trn) || TrnThirsty(trn) || TrnAsleep(trn))
        return(FALSE);
    if (ptM-- != 0L)
	return(FALSE);
    TrnPutToSleep(trn);
    TrnDry(trn);
    return(TRUE);
}

/***************************************************************************
*
* Function:  TrnLecnac
* Purpose:   This is an auxiliary routine of TrnCancel.
* Arguments: trn - a transition
* Returns:
* See also:  TrnCancel
*
****************************************************************************
*/
FE1(void, TrnLecnac,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        case 3:
            TrnSWhere(trn, (unsigned) 0);
            goto L3;
        }
    TrnWater(trn);
    if (TrnGPlcPredAO(trn))
	LSTFOR (TrnGPlcPredAO(trn), ptPlc, PLC)
	    if (PlcThirsty(ptPlc))
		{
		    PlcWater(ptPlc);
		    LSTFOR (PlcGOrPredAnd(ptPlc), ptTrn, TRN)
			if (!TrnSatisfied(trn))
			    {
				TrnINOrSucc(ptTrn);
				if (TrnThirsty(ptTrn))
				    {
                                        TrnSCursor(trn,
                                            LstGet(PlcGOrPredAnd(ptPlc)));
                                        TrnSWhere(trn, (unsigned) 1);
                                        return;
L1: ;
					ptPlc = (PLC *)
					    LstThis(TrnGPlcPredAO(trn));
                                        LstSet(PlcGOrPredAnd(ptPlc),
                                               TrnGCursor(trn));
				    }
			    }
		}
    if (TrnGOrStrongPredAO(trn))
	LSTFOR (TrnGOrStrongPredAO(trn), ptBinOr, BINOR)
	    if (BinOrHStrongNot(ptBinOr))
		{
		    BinOrDStrongNot(ptBinOr);
		    ptTrn = BinOrGTrn(ptBinOr);
		    if (TrnThirsty(ptTrn))
                        {
                            TrnSWhere(trn, (unsigned) 2);
                            return;
L2: ;
                        }
		}
    if (TrnGOrWeakPredAO(trn))
	LSTFOR (TrnGOrWeakPredAO(trn), ptBinOr, BINOR)
	    if (BinOrHWeakNot(ptBinOr))
		{
		    BinOrDWeakNot(ptBinOr);
		    ptTrn = BinOrGTrn(ptBinOr);
		    if (TrnThirsty(ptTrn))
                        {
                            TrnSWhere(trn, (unsigned) 3);
                            return;
L3: ;
                        }
		}
}

/***************************************************************************
*
* Function:  TrnCancel
* Purpose:   This routine restores and marks "tried" a transition that
*            was attempted to be deleted in the deletion algorithm of
*            the stubborn set method in the case that the attempt failed.
* Arguments: trn - the transition
* Returns:
* See also:  TrnDry, TrnTryDel, TrnSatWet
*
****************************************************************************
*/
FE1(void, TrnCancel,
    TRN *, trn)
{
    while (TRUE)
        {
            TrnLecnac(trn);
            if (TrnGWhere(trn))
                {
                    TrnSDown(ptTrn, trn);
                    trn = ptTrn;
                }
            else if (TrnGDown(trn))
                {
                    ptTrn = TrnGDown(trn);
                    TrnCDown(trn);
                    trn = ptTrn;
                }
            else
                return;
        }
}

/***************************************************************************
*
* Function:  TrnTewTas
* Purpose:   This is an auxiliary routine of TrnSatWet
* Arguments: trn - a transition
* Returns:
* See also:  TrnSatWet
*
****************************************************************************
*/
FE1(void, TrnTewTas,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        case 3:
            TrnSWhere(trn, (unsigned) 0);
            goto L3;
        }
    TrnFeed(trn);
    TrnWater(trn);
    if (TrnGPlcPredAO(trn))
	LSTFOR (TrnGPlcPredAO(trn), ptPlc, PLC)
	    if (PlcThirsty(ptPlc))
		{
		    PlcWater(ptPlc);
		    PlcFeed(ptPlc);
		    LSTFOR (PlcGOrPredAnd(ptPlc), ptTrn, TRN)
			if (TrnThirsty(ptTrn) && !TrnSatisfied(ptTrn))
			    {
                                TrnSCursor(trn,
                                    LstGet(PlcGOrPredAnd(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return;
L1: ;
				ptPlc = (PLC *) LstThis(TrnGPlcPredAO(trn));
                                LstSet(PlcGOrPredAnd(ptPlc),
                                       TrnGCursor(trn));
			    }
		}
    if (TrnGOrStrongPredAO(trn))
	LSTFOR (TrnGOrStrongPredAO(trn), ptBinOr, BINOR)
	    if (BinOrHStrongNot(ptBinOr))
		{
		    BinOrDStrongNot(ptBinOr);
		    BinOrCStrong(ptBinOr);
		    ptTrn = BinOrGTrn(ptBinOr);
		    if (TrnThirsty(ptTrn))
                        {
                            TrnSWhere(trn, (unsigned) 2);
                            return;
L2: ;
                        }
		}
    if (TrnGOrWeakPredAO(trn))
	LSTFOR (TrnGOrWeakPredAO(trn), ptBinOr, BINOR)
	    if (BinOrHWeakNot(ptBinOr))
		{
		    BinOrDWeakNot(ptBinOr);
		    BinOrCWeak(ptBinOr);
		    ptTrn = BinOrGTrn(ptBinOr);
		    if (TrnThirsty(ptTrn))
                        {
                            TrnSWhere(trn, (unsigned) 3);
                            return;
L3: ;
                        }
		}
}

/***************************************************************************
*
* Function:  TrnSatWet
* Purpose:   This routine deletes a transition in the deletion algorithm
*            of the stubborn set method.
* Arguments: trn - the transition
* Returns:
* See also:  TrnDry, TrnTryDel, TrnCancel
*
****************************************************************************
*/
FE1(void, TrnSatWet,
    TRN *, trn)
{
    while (TRUE)
        {
            TrnTewTas(trn);
            if (TrnGWhere(trn))
                {
                    TrnSDown(ptTrn, trn);
                    trn = ptTrn;
                }
            else if (TrnGDown(trn))
                {
                    ptTrn = TrnGDown(trn);
                    TrnCDown(trn);
                    trn = ptTrn;
                }
            else
                return;
        }
}

/***************************************************************************
*
* Function:  TrnPreAO
* Purpose:   This routine cleans the flag of a transition so that the
*            construction of an and/or-graph can be started in the
*            deletion algorithm of the stubborn set method.
* Arguments: trn - the transition
* Returns:
* See also:  PlcIniFlag (ptelem3.c)
*
****************************************************************************
*/
FE1(BOOL, TrnPreAO,
    TRN *, trn)
{
#ifdef SMODELS
    if (PtSmodels())
        TrnCAtoms(trn);
    else
	{
#endif
	    TrnImpoverish(trn);
	    TrnWake(trn);
#ifdef SMODELS
	}
#endif
    TrnWater(trn);
    TrnLeave(trn);
    return(FALSE);
}

/***************************************************************************
*
* Function:  TrnAOClean
* Purpose:   This routine is used for removing an and/or-graph from
*            the memory in the deletion algorithm of the stubborn set
*            method.
* Arguments: trn - a transition
* Returns:
* See also:  PlcAOClean (ptelem.c)
*
****************************************************************************
*/
FE1(BOOL, TrnAOClean,
    TRN *, trn)
{
    if (TrnGPlcPredAO(trn))
	TrnDPlcPredAO(trn);
    if (TrnGOrStrongPredAO(trn))
	TrnDOrStrongPredAO(trn);
    if (TrnGOrWeakPredAO(trn))
	TrnDOrWeakPredAO(trn);
    return(FALSE);
}

/***************************************************************************
*
* Function:  TrnDelCheck
* Purpose:   This routine fires a transition if it is enabled in the
*            stubborn set computed by the deletion algorithm.
* Arguments: trn - the transition
* Returns:
* See also:  TrnFire, TrnBest, TrnPrecCheck, TrnReCheck, PtOldSleep,
*            TrnChkRe
*
****************************************************************************
*/
FE1(BOOL, TrnDelCheck,
    TRN *, trn)
{
    if (TrnEnabled(trn) && !TrnSatisfied(trn))
	TrnFire(trn);
    return(FALSE);
}

/* The following routines before DelThem are auxiliary routines of DelThem. */

FE1(BOOL, TrnDupOr,
    TRN *, trn)
{
    TrnSDupNOrSucc(trn, TrnGNOrSucc(trn));
    return(FALSE);
}

FE1(BOOL, TrnBackup,
    TRN *, trn)
{
    if (!trn)
	return(TRUE);
    if (TrnEnabled(trn) && !TrnSatisfied(trn))
	TrnEnter(trn);
    else
	TrnLeave(trn);
    return(FALSE);
}

FE1(BOOL, TrnRehab,
    TRN *, trn)
{
    TrnImpoverish(trn);
    TrnWater(trn);
    TrnWake(trn);
    TrnSNOrSucc(trn, TrnGDupNOrSucc(trn));
    return(FALSE);
}

FE1(BOOL, TrnRemember,
    TRN *, trn)
{
    if (TrnOld(trn))
	TrnImpoverish(trn);
    else
	TrnFeed(trn);
    return(FALSE);
}

static TRN *ptDone;       /* a candidate to be deleted in
			   * the deletion algorithm of the stubborn
			   * set method */
static PTRBL *candidates = NULL;
static PTRBL *strongCands = NULL;
static PTRBL *visCands = NULL;
static PTRBL *cndTbl;
static unsigned long *candInd = NULL;
static unsigned maxExhaustive;
static BOOL pt_visible;

FE1(BOOL, TrnCollect,
    TRN *, trn)
{
    if ((!TrnEnabled(trn)) || TrnSatisfied(trn))
        return(FALSE);
    if (TrnVisible(trn))
	{
            if ((!pt_visible) && (PtCFFD() || OnTheFly() || Blocked()))
		return(FALSE);
	}
    else if (pt_visible && (PtCFFD() || Blocked()))
	return(FALSE);
    PtrBlPut(cndTbl, (char *) trn, ptM);
    ptM++;
    return(FALSE);
}

FE0(void, Collect)
{
    if (!candidates)
	{
	    candidates = PtrBlCreate(PtrBlGCount(TblTrn(0)) + 1L, &trnTools);
	    maxExhaustive = 5;
	    candInd = (unsigned long *)
		AllocArray(maxExhaustive - 2, sizeof(unsigned long));
	}
    cndTbl = candidates;
    ptM = 0L;
    (void) PtrBlApply(TblTrn(0), TrnCollect);
    PtrBlPut(cndTbl, (char *) NULL, ptM);
}

FE1(BOOL, TrnSCollect,
    TRN *, trn)
{
    if ((!TrnEnabled(trn)) || TrnSatisfied(trn) || TrnThirsty(trn) ||
	(TrnVisible(trn) && (PtCFFD() || (OnTheFly() && PtFlyPlc() &&
					  PlcLivelock(PtFlyPlc())))))
	return(FALSE);
    PtrBlPut(cndTbl, (char *) trn, ptM);
    ptM++;
    return(FALSE);
}

FE0(void, SCollect)
{
    if (!strongCands)
	strongCands = PtrBlCreate(PtrBlGCount(TblTrn(0)) + 1L, &trnTools);
    cndTbl = strongCands;
    ptM = 0L;
    (void) PtrBlApply(TblTrn(0), TrnSCollect);
    PtrBlPut(cndTbl, (char *) NULL, ptM);
}

FE1(BOOL, TrnCntV,
    TRN *, trn)
{
    if (TrnVisible(trn))
	ptM++;
    return(FALSE);
}

FE1(BOOL, TrnVCollect,
    TRN *, trn)
{
    if (TrnVisible(trn))
	{
	    PtrBlPut(cndTbl, (char *) trn, ptM);
	    ptM++;
	}
    return(FALSE);
}

FE0(void, VCollect)
{
    ptM = 0L;
    (void) PtrBlApply(TblTrn(0), TrnCntV);
    visibles = PtrBlCreate(ptM + 1L, &trnTools);
    cndTbl = visibles;
    ptM = 0L;
    (void) PtrBlApply(TblTrn(0), TrnVCollect);
    PtrBlPut(cndTbl, (char *) NULL, ptM);
}

FE0(void, WCollect)
{
    if (!visCands)
	{
	    ptM = 0L;
	    (void) PtrBlApply(TblTrn(0), TrnCntV);
	    visCands = PtrBlCreate(ptM + 1L, &trnTools);
	}
    cndTbl = visCands;
    pt_visible = TRUE;
    ptM = 0L;
    (void) PtrBlApply(TblTrn(0), TrnCollect);
    PtrBlPut(cndTbl, (char *) NULL, ptM);
}

/***************************************************************************
*
* Function:  DelThem
* Purpose:   This routine is the heart of the deletion algorithm of the
*            stubborn set method.
* Arguments: nInitial - the size of the set of candidate transitions
*                       from which some transitions are tried to be
*                       deleted
*            limCand - the least acceptable number of candidates
*                      remaining after all deletions
*            vremCheck - if TRUE, then visible transitions are not
*                        allowed to become deleted
*            fromScratch - if FALSE, then all visible transitions
*                          have been deleted beforehand
*            tp_visible - an auxiliary flag
*            sCollected - if TRUE, then key transition candidates
*                         have been collected beforehand
* Returns:
* See also:  PtDeletionAlg
*
****************************************************************************
*/
FE6(void, DelThem,
    unsigned long, nInitial,
    unsigned long, limCand,
    BOOL, vremCheck,
    BOOL, fromScratch,
    BOOL, tp_visible,
    BOOL, sCollected)
{
    unsigned long nCand;
    unsigned long nDeCand;
    unsigned many;
    unsigned ceiling;
    unsigned i;

    if (fromScratch)
	{
	    (void) PtrBlApply(TblPlc(), PlcIniFlag);
	    BinOrSList();
	    (void) PtrBlApply(TblTrn(0), TrnAndOr);
	    if (PtTryAgain())
		(void) PtrBlApply(TblTrn(0), TrnDupOr);
	}
    pt_visible = tp_visible;
    Collect();
    if (!sCollected)
      SCollect();
    if (vremCheck && !visibles)
      VCollect();
    nDeCand = nCand = nInitial;
    while ((nDeCand != 0L) && (nCand != limCand))
	{
	    if (TrRnd())
		ptM = (unsigned long) PtRandom((UNS32) nDeCand);
	    else
		ptM = 0L;
	    ptDone = *((TRN **) PtrBlApply(candidates, TrnTryDel));
	    if (((!vremCheck) || !*PtrBlApply(visibles, TrnVRemoved)) &&
		*PtrBlApply(strongCands, TrnFndStrong))
		{
		    ptM = 0L;
		    (void) PtrBlApply(candidates,
				      vremCheck ? TrnCntICand : TrnCntCand);
		    nDeCand -= nCand - ptM;
		    nCand = ptM;
		    TrnSatWet(ptDone);
		}
	    else
		{
		    nDeCand--;
		    TrnCancel(ptDone);
		}
	}
    if (PtTryAgain() && (nCand > 1L) && (nCand < nInitial))
	{
	    if (nInitial > (unsigned long) maxExhaustive)
		ceiling = 1;
	    else
		ceiling = ((unsigned) nCand) - 1;
	    (void) PtrBlApply(TblTrn(0), TrnBackup);
	    many = 1;
	    for (candInd[0] = 0L;
		 TrnOld((TRN *) PtrBlGet(candidates, candInd[0]));
		 candInd[0]++)
	      ;
	    do
		{
		    (void) PtrBlApply(TblTrn(0), TrnRehab);
		    (void) PtrBlApply(TblPlc(), PlcIniFlag);
		    LSTFOR (BinOrList(), ptBinOr, BINOR)
			{
			    BinOrCFlag(ptBinOr);
			    BinOrSStrong(ptBinOr);
			    BinOrSWeak(ptBinOr);
			}
		    if (!fromScratch)
		      for (ptM = 0L;
			   (ptDone = *((TRN **)
				       PtrBlApply(visCands, TrnTryDel)));
			   TrnSatWet(ptDone), ptM = 0L)
			;
		    for (i = many; i--; )
			TrnPutToSleep((TRN *) PtrBlGet(candidates,
						       candInd[i]));
		    for (ptM = 0L;
			 (ptDone = *((TRN **)
				     PtrBlApply(candidates, TrnTryDel)));
			 ptM = 0L)
			{
			    for (i = many;
				 i && !TrnThirsty((TRN *)
						  PtrBlGet(candidates,
							   candInd[i - 1]));
				 i--)
				;
			    if ((!i) &&
				((!vremCheck) ||
				 !*PtrBlApply(visibles, TrnVRemoved)) &&
				*PtrBlApply(strongCands, TrnFndStrong))
			      TrnSatWet(ptDone);
			    else
			      TrnCancel(ptDone);
			}
		    ptM = 0L;
		    (void) PtrBlApply(TblTrn(0),
				      vremCheck ? TrnCntICand : TrnCntCand);
		    nCand = ptM;
		    if (nCand > (unsigned long) many)
			{
			    if (nCand <= (unsigned long) ceiling)
				{
				    ceiling = ((unsigned) nCand) - 1;
				    (void) PtrBlApply(candidates, TrnBackup);
				}
			    do
				{
				    if (candInd[many - 1] < nInitial - 1L)
					candInd[many - 1]++;
				    else if (candInd[0] <
					     nInitial - (unsigned long) many)
					{
					    for (i = many - 2;
						 candInd[i + 1] ==
						 candInd[i] + 1L;
						 i--)
						;
					    candInd[i]++;
					    for (; i < many - 1; i++)
						candInd[i + 1] =
						    candInd[i] + 1L;
					}
				    else
					{
					    many++;
					    if (many <= ceiling)
						for (i = many; i--; )
						    candInd[i] =
							(unsigned long) i;
					    else
						(void) PtrBlApply(TblTrn(0),
								  TrnRemember);
					}
				    if (many <= ceiling)
					for (i = many;
					     i &&
					     TrnOld((TRN *)
						    PtrBlGet(candidates,
							     candInd[i - 1]));
					     i--)
					    ;
				} while ((many <= ceiling) && i);
			}
		} while ((many <= ceiling) &&
			 (nCand > (unsigned long) many));
	}
    (void) PtrBlApply(TblPlc(), PlcAOClean);
    (void) PtrBlApply(TblTrn(0), TrnAOClean);
    BinOrDList();
}

FE0(void, Upsilon)
{
    unsigned long nCand;
    unsigned long nDeCand;
    unsigned i;
    unsigned k;
    TRN *trn;
    BOOL found;

    (void) PtrBlApply(TblPlc(), PlcIniFlag);
    BinOrSList();
    (void) PtrBlApply(TblTrn(0), TrnAndOr);
    pt_visible = TRUE;
    Collect();
    for (i = 0; Block(i); i++)
        SetYellow(i);
    found = FALSE;
    while (TRUE)
        {
            (void) PtrBlApply(TblTrn(0), TrnRehab);
            (void) PtrBlApply(TblPlc(), PlcIniFlag);
            LSTFOR (BinOrList(), ptBinOr, BINOR)
                {
                    BinOrCFlag(ptBinOr);
                    BinOrSStrong(ptBinOr);
                    BinOrSWeak(ptBinOr);
                }
            for (i = 0; Block(i); i++)
                {
                    if (!Yellow(i))
                        continue;
                    LSTFOR (Block(i), trn, TRN)
                        if (TrnEnabled(trn))
                            break;
                    if (trn)
                        break;
                }
            if (!Block(i))
                break;
            LSTFOR (Block(i), trn, TRN)
                {
                    if (!TrnEnabled(trn))
                        continue;
                    for (k = 0; TrnGBlock(trn, k) != (unsigned) -1; k++)
                        SetOrange(TrnGBlock(trn, k));
                }
            for (i = 0; Block(i); i++)
                {
                    if (!Orange(i))
                        continue;
                    LSTFOR (Block(i), trn, TRN)
                        TrnPutToSleep(trn);
                }
            for (ptM = 0L;
                 (ptDone = *((TRN **)
                      PtrBlApply(candidates, TrnTryDel)));
                 ptM = 0L)
                {
                    for (i = 0; Block(i); i++)
                        {
                            if (!Orange(i))
                                continue;
                            LSTFOR (Block(i), trn, TRN)
                                if (TrnThirsty(trn))
                                    break;
                            if (trn)
                                break;
                        }
                    if ((!Block(i)) &&
                        *PtrBlApply(strongCands, TrnFndStrong))
                        TrnSatWet(ptDone);
                    else
                        break;
                }
	    if (!ptDone)
		{
                    found = TRUE;
                    break;
		}
            for (i = 0; Block(i); i++)
                {
                    if (!Orange(i))
                        continue;
                    SetBlue(i);
                }
        }
    pt_visible = FALSE;
    Collect();
    nDeCand = nCand = PtGInvisCnt();
    while ((nDeCand != 0L) && (nCand != 1L))
	{
	    if (TrRnd())
		ptM = (unsigned long) PtRandom((UNS32) nDeCand);
	    else
		ptM = 0L;
	    ptDone = *((TRN **) PtrBlApply(candidates, TrnTryDel));
            for (i = 0; Block(i); i++)
                {
                    if ((found && !Orange(i)) || ((!found) && !Blue(i)))
                        continue;
                    LSTFOR (Block(i), trn, TRN)
                        if (TrnThirsty(trn))
                            break;
                    if (trn)
                        break;
                }
            if ((!Block(i)) &&
                *PtrBlApply(strongCands, TrnFndStrong))
		{
		    ptM = 0L;
                    (void) PtrBlApply(candidates, TrnCntICand);
		    nDeCand -= nCand - ptM;
		    nCand = ptM;
		    TrnSatWet(ptDone);
		}
	    else
		{
		    nDeCand--;
		    TrnCancel(ptDone);
		}
	}
    for (i = 0; Block(i) && ((found && Orange(i)) || ((!found) && Blue(i)));
         i++)
        ;
    if (Block(i))
        {
            SetPtRecurs();
            (void) PtrBlApply(TblTrn(0), TrnRedDel);
            if (!PtRecurs())
                {
		    (void) PtrBlApply(TblTrn(0), TrnPreAO);
                    if (PtGInvisCnt() > 1L)
                        {
                            DelThem(PtGInvisCnt(), 1L, TRUE, TRUE,
                                    FALSE, TRUE);
                            return;
                        }
                }
	}
    (void) PtrBlApply(TblPlc(), PlcAOClean);
    (void) PtrBlApply(TblTrn(0), TrnAOClean);
    BinOrDList();
}

/***************************************************************************
*
* Function:  PtRedDel
* Purpose:   In the CFFD-preserving version of the deletion algorithm
*            of the stubborn set method, this routine computes a new
*            stubborn set having all visible transitions if the preliminary
*            stubborn set is not known to contain them and firing the
*            enabled transitions of the preliminary set would close
*            an invisible loop. The new stubborn set is then final.
* Arguments: sCollected - 
* Returns:
* See also:  TrnLookStack, TrnRedInc, TrnRedDel, PtRedInc, DelThem
*
****************************************************************************
*/
FE1(void, PtRedDel,
    BOOL, sCollected)
{
    SetPtRecurs();
    (void) PtrBlApply(TblTrn(0), TrnRedDel);
    if (!PtRecurs())
	{
	    (void) PtrBlApply(TblTrn(0), TrnPreAO);
	    if (PtGInvisCnt() > 1L)
                DelThem(PtGInvisCnt(), 1L, TRUE, TRUE, FALSE, sCollected);
	}
}

/***************************************************************************
*
* Function:  PtDeletionAlg
* Purpose:   This routine is the surface of the deletion algorithm of the
*            stubborn set method.
* Arguments:
* Returns:
* See also:  PtIncrementalAlg, DelThem
*
****************************************************************************
*/
FE0(void, PtDeletionAlg)
{
#ifdef SMODELS
    if (PtSmodels())
	{
	    PtDelegate();
	    goto END_DEL;
	}
#endif
    (void) PtrBlApply(TblTrn(0), TrnPreAO);
    if (OnTheFly() && (!PtCFFD()) && !Blocked())
	{
	    if ((!PtFlyPlc()) || !PlcLivelock(PtFlyPlc()))
		{
		    if (PtGInvisCnt() != 0L)
			DelThem(PtGInvisCnt(),
				(PtGInvisCnt() == PtGECnt())
				? ((unsigned long) 1L)
				: (unsigned long) 0L,
                                TRUE, TRUE, FALSE, FALSE);
		}
	    else if (PtGInvisCnt() != 0L)
                DelThem(PtGECnt(), 1L, FALSE, TRUE, TRUE, FALSE);
	}
    else if (((!PtCFFD()) && !Blocked()) || (PtGInvisCnt() == PtGECnt()))
	{           
            DelThem(PtGECnt(), 1L, FALSE, TRUE, FALSE, FALSE);
	    ptM = 0L;
	    (void) PtrBlApply(TblTrn(0), TrnCntCand);
	    if (ptM == PtGECnt())
		ptBranching = TRUE;
	    else if (PtCFFD() && ((!OnTheFly()) ||
				  (!PtFlyPlc()) || !PlcLivelock(PtFlyPlc())))
		{
		    if (!visibles)
			VCollect();
		    if (*PtrBlApply(visibles, TrnVRemoved))
			PtRedDel(FALSE);
		}
	}
    else
	{
	    (void) PtrBlApply(TblPlc(), PlcIniFlag);
	    BinOrSList();
	    (void) PtrBlApply(TblTrn(0), TrnAndOr);
	    if (PtTryAgain())
		(void) PtrBlApply(TblTrn(0), TrnDupOr);
	    SCollect();
	    WCollect();
	    for (ptM = 0L;
		 (ptDone = *((TRN **) PtrBlApply(visCands, TrnTryDel))) &&
		  *PtrBlApply(strongCands, TrnFndStrong);
		 TrnSatWet(ptDone), ptM = 0L)
	      ;
	    if (!ptDone)
		{
		    ptM = 0L;
		    (void) PtrBlApply(TblTrn(0), TrnCntCand);
                    DelThem(ptM, 1L, FALSE, FALSE, FALSE, TRUE);
		    if ((!OnTheFly()) ||
			(!PtFlyPlc()) || !PlcLivelock(PtFlyPlc()))
			PtRedDel(TRUE);
		}
	    else
		{
		    (void) PtrBlApply(TblPlc(), PlcAOClean);
		    (void) PtrBlApply(TblTrn(0), TrnAOClean);
		    BinOrDList();
		    (void) PtrBlApply(TblTrn(0), TrnPreAO);
                    if (Blocked())
                        Upsilon();
                    else if (PtGInvisCnt() > 1L)
                        DelThem(PtGInvisCnt(), 1L, TRUE, TRUE, FALSE, TRUE);
		}
	}
#ifdef SMODELS
  END_DEL: ;
#endif
    if (ptDoIt)
	{
	    (void) PtrBlApply(TblTrn(0), TrnDelCheck);
	    PtGphUpdate();
	}
}
