/***************************************************************************
*
* Copyright (c) 1999 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  ptnet4.c
*
* Part of: PROD's programs araprod and "gphgen"
*
* Author:  Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     See ptnet.c.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#ifdef SMODELS

#include <stdio.h>
#ifdef DEBUG
#include <string.h>
#endif
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ptrbl.h"
#include "baltree.h"
#include "bdd.h"
#include "gphutil.h"
#include "graph.h"
#include "mark.h"
#include "encode.h"
#include "ptnet.h"
#include "ptstable.h"

FE1(INEQ *, IneqCons,
    unsigned long, card)
{
    INEQ *ineq;

    ineq = IneqAlloc();
    IneqSCard(ineq, card);
    IneqCGe(ineq);
    IneqCLt(ineq);
    return(ineq);
}

FE1(INEQ *, IneqCopy,
    INEQ *, ineq)
{
    INEQ *copy;

    copy = IneqAlloc();
    IneqSCard(copy, IneqGCard(ineq));
    IneqSGe(copy, IneqGGe(ineq));
    IneqSLt(copy, IneqGLt(ineq));
    return(copy);
}

#ifdef DEBUG
FE2(void, IneqDump,
    INEQ *, ineq,
    unsigned, depth)
{
    DumpIndent(depth);
    (void) fprintf(_err, "INEQ card %lu ge 0x%lx lt 0x%lx\n",
		   IneqGCard(ineq), (unsigned long) IneqGGe(ineq),
		   (unsigned long) IneqGLt(ineq));
    (void) fflush(_err);
}
#endif

FE2(int, IneqKeyCmp,
    INEQ *, ineq,
    unsigned long *, key)
{
    if (IneqGCard(ineq) < *key)
	return(-1);
    if (IneqGCard(ineq) > *key)
	return(1);
    return(0);
}

TOOLS ineqTools =
{ (CFUNC) IneqCopy, Deallocate
#ifdef DEBUG
, IneqDump
#endif
 };

BALTOOLS ineqBalTools =
{ (CFUNC) IneqCopy, Deallocate
#ifdef DEBUG
, IneqDump
#endif
, IneqKeyCmp, (FOOL) NULL };

FS1(BOOL, TrnKeyRules,
    TRN *, trn)
{
    if (!TrnGKappa(trn))
	return(FALSE);
    if (TrnGPurIn(trn))
	LSTFOR (TrnGPurIn(trn), ptTrArc, TRARC)
	    {
		ptPlc = TrArcGPlc(ptTrArc);
		if (PlcGPurOut(ptPlc))
		    LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
			{
			    ptTrn = PlArcGTrn(ptPlArc);
			    TrnImpoverish(ptTrn);
			}
		if (PlcGMosOut(ptPlc))
		    LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
			{
			    ptTrn = PlDrcGTrn(ptPlDrc);
			    TrnImpoverish(ptTrn);
			}
	    }
    if (TrnGNeutral(trn))
	LSTFOR (TrnGNeutral(trn), ptTrArc, TRARC)
	    {
		ptPlc = TrArcGPlc(ptTrArc);
		if (PlcGPurOut(ptPlc))
		    LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
			{
			    ptTrn = PlArcGTrn(ptPlArc);
			    TrnImpoverish(ptTrn);
			}
		if (PlcGMosOut(ptPlc))
		    LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
			{
			    ptTrn = PlDrcGTrn(ptPlDrc);
			    TrnImpoverish(ptTrn);
			}
	    }
    if (TrnGMosIn(trn))
	LSTFOR (TrnGMosIn(trn), ptTrDrc, TRDRC)
	    {
		ptPlc = TrDrcGPlc(ptTrDrc);
		if (PlcGPurOut(ptPlc))
		    LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
			{
			    ptTrn = PlArcGTrn(ptPlArc);
			    TrnImpoverish(ptTrn);
			}
		if (PlcGMosOut(ptPlc))
		    LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
			{
			    ptTrn = PlDrcGTrn(ptPlDrc);
			    TrnImpoverish(ptTrn);
			}
	    }
    if (TrnGMosOut(trn))
	LSTFOR (TrnGMosOut(trn), ptTrDrc, TRDRC)
	    {
		ptPlc = TrDrcGPlc(ptTrDrc);
		if (PlcGPurOut(ptPlc))
		    LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
			{
			    ptTrn = PlArcGTrn(ptPlArc);
			    TrnImpoverish(ptTrn);
			}
		if (PlcGMosOut(ptPlc))
		    LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
			{
			    ptTrn = PlDrcGTrn(ptPlDrc);
			    TrnImpoverish(ptTrn);
			}
	    }
    TrnFeed(trn);
    if (PtLongPlay() && TrnGActive(trn))
	{
	    StartRule(TrnGKappa(trn));
	    NegContRule(TrnGActive(trn));
	    EndRule();
	}
    StartRule(TrnGKappa(trn));
    if (PtLongPlay() && TrnGActive(trn))
	ContRule(TrnGActive(trn));
    ContRule(TrnGGamma(trn));
    EndRule();
    if (TrnGPurIn(trn))
	LSTFOR (TrnGPurIn(trn), ptTrArc, TRARC)
	    {
		ptPlc = TrArcGPlc(ptTrArc);
		if (PlcGPurOut(ptPlc))
		    LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
			{
			    ptTrn = PlArcGTrn(ptPlArc);
			    if (!TrnSatisfied(ptTrn))
				{
				    TrnFeed(ptTrn);
				    StartRule(TrnGKappa(trn));
				    if (PtLongPlay() && TrnGActive(trn))
					ContRule(TrnGActive(trn));
				    ContRule(TrnGGamma(ptTrn));
				    EndRule();
				}
			}
		if (PlcGMosOut(ptPlc))
		    LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
			{
			    ptTrn = PlDrcGTrn(ptPlDrc);
			    if (!TrnSatisfied(ptTrn))
				{
				    TrnFeed(ptTrn);
				    StartRule(TrnGKappa(trn));
				    if (PtLongPlay() && TrnGActive(trn))
					ContRule(TrnGActive(trn));
				    ContRule(TrnGGamma(ptTrn));
				    EndRule();
				}
			}
	    }
    if (TrnGNeutral(trn))
	LSTFOR (TrnGNeutral(trn), ptTrArc, TRARC)
	    {
		ptPlc = TrArcGPlc(ptTrArc);
		if (PlcGPurOut(ptPlc))
		    LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
			{
			    ptTrn = PlArcGTrn(ptPlArc);
			    if (!TrnSatisfied(ptTrn))
				{
				    TrnFeed(ptTrn);
				    StartRule(TrnGKappa(trn));
				    if (PtLongPlay() && TrnGActive(trn))
					ContRule(TrnGActive(trn));
				    ContRule(TrnGGamma(ptTrn));
				    EndRule();
				}
			}
		if (PlcGMosOut(ptPlc))
		    LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
			{
			    ptTrn = PlDrcGTrn(ptPlDrc);
			    if (!TrnSatisfied(ptTrn))
				{
				    TrnFeed(ptTrn);
				    StartRule(TrnGKappa(trn));
				    if (PtLongPlay() && TrnGActive(trn))
					ContRule(TrnGActive(trn));
				    ContRule(TrnGGamma(ptTrn));
				    EndRule();
				}
			}
	    }
    if (TrnGMosIn(trn))
	LSTFOR (TrnGMosIn(trn), ptTrDrc, TRDRC)
	    {
		ptPlc = TrDrcGPlc(ptTrDrc);
		if (PlcGPurOut(ptPlc))
		    LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
			{
			    ptTrn = PlArcGTrn(ptPlArc);
			    if (!TrnSatisfied(ptTrn))
				{
				    TrnFeed(ptTrn);
				    StartRule(TrnGKappa(trn));
				    if (PtLongPlay() && TrnGActive(trn))
					ContRule(TrnGActive(trn));
				    ContRule(TrnGGamma(ptTrn));
				    EndRule();
				}
			}
		if (PlcGMosOut(ptPlc))
		    LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
			{
			    ptTrn = PlDrcGTrn(ptPlDrc);
			    if (!TrnSatisfied(ptTrn))
				{
				    TrnFeed(ptTrn);
				    StartRule(TrnGKappa(trn));
				    if (PtLongPlay() && TrnGActive(trn))
					ContRule(TrnGActive(trn));
				    ContRule(TrnGGamma(ptTrn));
				    EndRule();
				}
			}
	    }
    if (TrnGMosOut(trn))
	LSTFOR (TrnGMosOut(trn), ptTrDrc, TRDRC)
	    {
		ptPlc = TrDrcGPlc(ptTrDrc);
		if (PlcGPurOut(ptPlc))
		    LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
			{
			    ptTrn = PlArcGTrn(ptPlArc);
			    if (!TrnSatisfied(ptTrn))
				{
				    TrnFeed(ptTrn);
				    StartRule(TrnGKappa(trn));
				    if (PtLongPlay() && TrnGActive(trn))
					ContRule(TrnGActive(trn));
				    ContRule(TrnGGamma(ptTrn));
				    EndRule();
				}
			}
		if (PlcGMosOut(ptPlc))
		    LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
			{
			    ptTrn = PlDrcGTrn(ptPlDrc);
			    if (!TrnSatisfied(ptTrn))
				{
				    TrnFeed(ptTrn);
				    StartRule(TrnGKappa(trn));
				    if (PtLongPlay() && TrnGActive(trn))
					ContRule(TrnGActive(trn));
				    ContRule(TrnGGamma(ptTrn));
				    EndRule();
				}
			}
	    }
    return(FALSE);
}

FS1(BOOL, TrnRootRule,
    TRN *, trn)
{
    if (TrnGKappa(trn))
	ContRule(TrnGKappa(trn));
    return(FALSE);
}

FS1(BOOL, TrnMinRule,
    TRN *, trn)
{
    if (TrnGPlain(trn))
	ContMinRule(TrnGPlain(trn));
    return(FALSE);
}

FS1(BOOL, TrnVisInner,
    TRN *, trn)
{
    if (!trn)
	return(TRUE);
    if (TrnGGamma(trn) && (trn != ptTrn))
	{
	    StartRule(TrnGGamma(ptTrn));
	    if (PtLongPlay() && TrnGActive(ptTrn))
		ContRule(TrnGActive(ptTrn));
	    ContRule(TrnGGamma(trn));
	    EndRule();
	}
    return(FALSE);
}

FS1(BOOL, TrnVisRules,
    TRN *, trn)
{
    if (!trn)
	return(TRUE);
    if ((!TrnGGamma(trn)) ||
	((!PtLongPlay()) && (TrnThirsty(trn) || !TrnEnabled(trn))))
	return(FALSE);
    ptTrn = trn;
    (void) PtrBlApply(visibles, TrnVisInner);
    return(FALSE);
}

FS1(BOOL, TrnVisLong,
    TRN *, trn)
{
    if (!trn)
	return(TRUE);
    if (TrnGGamma(trn) && (trn != ptTrn) &&
	((!TrnVisible(ptTrn)) || !TrnSatisfied(trn)))
	{
	    StartRule(TrnGGamma(ptTrn));
	    if (TrnGActive(ptTrn))
		ContRule(TrnGActive(ptTrn));
	    if (TrnGToStack(ptTrn))
		ContRule(TrnGToStack(ptTrn));
	    ContRule(TrnGGamma(trn));
	    EndRule();
	}
    return(FALSE);
}

FS1(BOOL, TrnProRules,
    TRN *, trn)
{
    if ((!TrnGGamma(trn)) ||
	(PtLongPlay() ? TrnVisible(trn) : !TrnThirsty(trn)))
	return(FALSE);
    ptTrn = trn;
    if (PtLongPlay())
        (void) PtrBlApply(visibles, TrnVisLong);
    else
	(void) PtrBlApply(visibles, TrnVisInner);
    return(FALSE);
}

FS1(BOOL, TrnMakePoor,
    TRN *, trn)
{
    if (!trn)
	return(TRUE);
    if (TrnGGamma(trn))
	TrnImpoverish(trn);
    return(FALSE);
}

static unsigned blkInd;

FS1(BOOL, TrnBlockRules,
    TRN *, trn)
{
    if (!trn)
	return(TRUE);
    if ((!TrnGGamma(trn)) ||
	((!PtLongPlay()) && (TrnThirsty(trn) || !TrnEnabled(trn))))
	return(FALSE);
    if (PtLongPlay())
	(void) PtrBlApply(visibles, TrnMakePoor);
    else
	{
	    for (blkInd = 0; TrnGBlock(trn, blkInd) != (unsigned) -1; blkInd++)
		LSTFOR (Block(TrnGBlock(trn, blkInd)), ptTrn, TRN)
		    {
			if (TrnGGamma(ptTrn))
			    TrnImpoverish(ptTrn);
		    }
	}
    TrnFeed(trn);
    for (blkInd = 0; TrnGBlock(trn, blkInd) != (unsigned) -1; blkInd++)
	LSTFOR (Block(TrnGBlock(trn, blkInd)), ptTrn, TRN)
	    {
		if (TrnGGamma(ptTrn) && !TrnSatisfied(ptTrn))
		    {
			TrnFeed(ptTrn);
			StartRule(TrnGGamma(trn));
			if (PtLongPlay() && TrnGActive(trn))
			    ContRule(TrnGActive(trn));
			ContRule(TrnGGamma(ptTrn));
			EndRule();
		    }
	    }
    if (PtLongPlay())
	{
	    ptTrn = trn;
	    (void) PtrBlApply(visibles, TrnVisLong);
	}
    return(FALSE);
}

FS1(BOOL, TrnStable,
    TRN *, trn)
{
    if (TrnEnabled(trn))
	{
	    if ((!TrnGPlain(trn)) || Stable(TrnGPlain(trn)))
	        {
		    TrnImpoverish(trn);
		    ptM++;
		}
	    else
		TrnFeed(trn);
	}
    return(FALSE);
}

#ifdef DEBUG
static char *atomName = NULL;
#endif

FS0(void, MkABK)
{
    VOID *alpha;
    VOID *kappa;

#ifdef DEBUG
    (void) strcpy(atomName, "alpha");
    alpha = NmAtom(atomName);
    (void) strcpy(atomName, "kappa");
    kappa = NmAtom(atomName);
#else
    alpha = FreshAtom();
    kappa = FreshAtom();
#endif
    StartRule(alpha);
    ContRule(kappa);
    NegContRule(alpha);
    EndRule();
    StartRule(kappa);
}

FS1(BOOL, IneqExtra,
    INEQ *, ineq)
{
    ReverseRules(IneqGGe(ineq), IneqGLt(ineq));
    if (ptIneq)
	{
	    SimpleRule(IneqGGe(ptIneq), IneqGGe(ineq));
	    SimpleRule(IneqGLt(ineq), IneqGLt(ptIneq));
	}
    ptIneq = ineq;
    return(FALSE);
}

FS1(BOOL, PlcIneqExtra,
    PLC *, plc)
{
    if (PlcGIneqs(plc))
        {
	    ptIneq = NULL;
	    (void) BalTrApply(PlcGIneqs(plc), IneqExtra);
	}
    return(FALSE);
}

FS1(BOOL, PlcSmoSet,
    PLC *, plc)
{
    if (PlcGIneqs(plc))
        {
	    if ((ptIneq = (INEQ *)
                 BalTrLtEqPrefix(PlcGIneqs(plc),
                                 (char *) PlcACount(plc), FALSE)))
	        {
	            SetPosCompute(IneqGGe(ptIneq));
	            if ((ptIneq = (INEQ *)
		         BalTrNext(PlcGIneqs(plc), (char *) ptIneq,
			           (char *) IneqACard(ptIneq), (char *) NULL)))
		        SetNegCompute(IneqGGe(ptIneq));
		}
            else if ((ptIneq = (INEQ *) BalTrFirst(PlcGIneqs(plc))))
	        SetNegCompute(IneqGGe(ptIneq));
            else
	        {
		    (void) fprintf(_out, "\n");
		    (void) fflush(_out);
		    (void) fprintf(_err,
				   "\nInternal error in PlcSmoSet\n");
		    (void) fflush(_err);
		    abort();
	        }
	}
    return(FALSE);
}

FS1(BOOL, PlcSmoRes,
    PLC *, plc)
{
    if (PlcGIneqs(plc))
        {
	    if ((ptIneq = (INEQ *)
                 BalTrLtEqPrefix(PlcGIneqs(plc),
                                 (char *) PlcACount(plc), FALSE)))
	        {
	            ResPosCompute(IneqGGe(ptIneq));
	            if ((ptIneq = (INEQ *)
		         BalTrNext(PlcGIneqs(plc), (char *) ptIneq,
			           (char *) IneqACard(ptIneq), (char *) NULL)))
		        ResNegCompute(IneqGGe(ptIneq));
		}
            else if ((ptIneq = (INEQ *) BalTrFirst(PlcGIneqs(plc))))
	        ResNegCompute(IneqGGe(ptIneq));
            else
	        {
		    (void) fprintf(_out, "\n");
		    (void) fflush(_out);
		    (void) fprintf(_err,
				   "\nInternal error in PlcSmoRes\n");
		    (void) fflush(_err);
		    abort();
	        }
	}
    return(FALSE);
}

FS1(BOOL, TrnSmoSet,
    TRN *, trn)
{
    if (TrnGToStack(trn) && TrnEnabled(trn))
	{
	    TrnWater(trn);
	    SetPtRecurs();
	    TrnLookStack(trn);
	    if (PtRecurs())
		SetNegCompute(TrnGToStack(trn));
	    else
		{
		    TrnSalt(trn);
		    SetPosCompute(TrnGToStack(trn));
		}
	}
    return(FALSE);
}

FS1(BOOL, TrnSmoRes,
    TRN *, trn)
{
    if (TrnGToStack(trn) && TrnEnabled(trn))
	{
	    if (TrnThirsty(trn))
		ResPosCompute(TrnGToStack(trn));
	    else
		ResNegCompute(TrnGToStack(trn));
	}
    return(FALSE);
}

static BOOL constructed = FALSE;

FE0(void, PtDelegate)
{
#ifdef DEBUG
    if (!atomName)
	atomName = (char *) Allocate(8192);
#endif
    if ((!PtLongPlay()) || !constructed)
	{
	    (void) PtrBlApply(TblTrn(0), TrnPreAO);
	    (void) PtrBlApply(TblPlc(), PlcIniFlag);
	    MkApi();
	    (void) PtrBlApply(TblTrn(0), TrnAndOr);
	    MkABK();
	    (void) PtrBlApply(TblTrn(0), TrnRootRule);
	    EndRule();
	    (void) PtrBlApply(TblTrn(0), TrnKeyRules);
	    if (OnTheFly() || PtCFFD())
		{
		    if (!visibles)
			VCollect();
		    if (Blocked())
			(void) PtrBlApply(visibles, TrnBlockRules);
		    else
			(void) PtrBlApply(visibles, TrnVisRules);
		    (void) PtrBlApply(TblTrn(0), TrnProRules);
		}
	    if (PtLongPlay())
		(void) PtrBlApply(TblPlc(), PlcIneqExtra);
	    StartMinRule();
	    (void) PtrBlApply(TblTrn(0), TrnMinRule);
	    EndRule();
	}
    if (PtLongPlay())
	{
	    (void) PtrBlApply(TblPlc(), PlcSmoSet);
	    if (OnTheFly() || PtCFFD())
		(void) PtrBlApply(TblTrn(0), TrnSmoSet);
	}
    if ((!PtLongPlay()) || !constructed)
	{
	    DoneApi();
	    constructed = TRUE;
	    SmoCalcInit();
	}
    if (!MkModel())
	{
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err, "\nInternal MkModel error in PtDelegate\n");
	    (void) fflush(_err);
	    abort();
	}
    do
	{
#ifdef DEBUG
	    ShowModel();
#endif
	    ptM = (unsigned long) 0L;
	    (void) PtrBlApply(TblTrn(0), TrnStable);
	    if (((unsigned long) 0L) == ptM)
		{
		    (void) fprintf(_out, "\n");
		    (void) fflush(_out);
		    (void)
			fprintf(_err,
				"\nInternal TrnStable error in PtDelegate\n");
		    (void) fflush(_err);
		    abort();
		}
	} while ((((unsigned long) 1L) != ptM) && MkModel());
    if (PtLongPlay())
	{
	    SmoRevert();
	    (void) PtrBlApply(TblPlc(), PlcSmoRes);
	    if (OnTheFly() || PtCFFD())
		(void) PtrBlApply(TblTrn(0), TrnSmoRes);
	}
    else
	DelApi();
#ifdef DEBUG
    (void) fprintf(_out, "---------------\n");
    (void) fflush(_out);
#endif
}

FE1(void, MkPlcGamma,
    PLC *, plc)
{
#ifdef DEBUG
    (void) strcpy(atomName, "gamma__");
    PlKeySp(PlcGKey(plc), atomName);
    PlcSGamma(plc, NmAtom(atomName));
#else
    PlcSGamma(plc, FreshAtom());
#endif
}

FE2(INEQ *, MkPlcIneq,
    PLC *, plc,
    unsigned long, card)
{
    INEQ *ineq;
#ifdef DEBUG
    char *s;
#endif
    
    if (((unsigned long) 0L) == card)
	{
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err, "\nInternal error in MkPlcIneq\n");
	    (void) fflush(_err);
	    abort();
	}
    if (PlcGIneqs(plc))
	{
	    if ((ineq = (INEQ *) BalTrElement(PlcGIneqs(plc), (char *) &card)))
		return(ineq);
	}
    else
	PlcSIneqs(plc);
    ineq = IneqCons(card);
    (void) BalTrInsert(PlcGIneqs(plc), (char *) ineq, (char *) &card, TRUE);
#ifdef DEBUG
    (void) strcpy(atomName, "mk__");
    PlKeySp(PlcGKey(plc), atomName);
    (void) strcat(atomName, "__");
    s = atomName + strlen(atomName);
    (void) sprintf(s, "ge__%lu", card);
    IneqSGe(ineq, NmAtom(atomName));
    s[0] = 'l';
    s[1] = 't';
    IneqSLt(ineq, NmAtom(atomName));
#else
    IneqSGe(ineq, FreshAtom());
    IneqSLt(ineq, FreshAtom());
#endif
    return(ineq);
}

FE1(void, MkTrnGamma,
    TRN *, trn)
{
#ifdef DEBUG
    (void) strcpy(atomName, "gamma__");
    InsSp(TrnGKey(trn), atomName);
    TrnSGamma(trn, NmAtom(atomName));
#else
    TrnSGamma(trn, FreshAtom());
#endif
}

FE1(void, MkTrnPlain,
    TRN *, trn)
{
#ifdef DEBUG
    (void) strcpy(atomName, "plain__");
    InsSp(TrnGKey(trn), atomName);
    TrnSPlain(trn, NmAtom(atomName));
#else
    TrnSPlain(trn, FreshAtom());
#endif
}

FE1(void, MkTrnKappa,
    TRN *, trn)
{
#ifdef DEBUG
    (void) strcpy(atomName, "kappa__");
    InsSp(TrnGKey(trn), atomName);
    TrnSKappa(trn, NmAtom(atomName));
#else
    TrnSKappa(trn, FreshAtom());
#endif
}

FE1(void, MkTrnActive,
    TRN *, trn)
{
#ifdef DEBUG
    (void) strcpy(atomName, "active__");
    InsSp(TrnGKey(trn), atomName);
    TrnSActive(trn, NmAtom(atomName));
#else
    TrnSActive(trn, FreshAtom());
#endif
}

FE1(void, MkTrnToStack,
    TRN *, trn)
{
    VOID *offStack;

#ifdef DEBUG
    (void) strcpy(atomName, "tostack__");
    InsSp(TrnGKey(trn), atomName);
    TrnSToStack(trn, NmAtom(atomName));
    (void) strcpy(atomName, "offstack__");
    InsSp(TrnGKey(trn), atomName);
    offStack = NmAtom(atomName);
#else
    TrnSToStack(trn, FreshAtom());
    offStack = FreshAtom();
#endif
    if (TrnGActive(trn))
	{
	    StartRule(TrnGToStack(trn));
	    ContRule(TrnGActive(trn));
	    NegContRule(offStack);
	    EndRule();
	    StartRule(offStack);
	    ContRule(TrnGActive(trn));
	    NegContRule(TrnGToStack(trn));
	    EndRule();
	}
    else
	ReverseRules(TrnGToStack(trn), offStack);
}

#ifdef DEBUG
FE2(void, MkTrArcGamma,
    TRN *, trn,
    TRARC *, trArc)
#else
FE1(void, MkTrArcGamma,
    TRARC *, trArc)
#endif
{
#ifdef DEBUG
    (void) strcpy(atomName, "gamma__");
    InsSp(TrnGKey(trn), atomName);
    (void) strcat(atomName, "____");
    PlKeySp(PlcGKey(TrArcGPlc(trArc)), atomName);
    TrArcSGamma(trArc, NmAtom(atomName));
#else
    TrArcSGamma(trArc, FreshAtom());
#endif
}

#ifdef DEBUG
FE2(void, MkTrDrcGamma,
    TRN *, trn,
    TRDRC *, trDrc)
#else
FE1(void, MkTrDrcGamma,
    TRDRC *, trDrc)
#endif
{
#ifdef DEBUG
    (void) strcpy(atomName, "gamma__");
    InsSp(TrnGKey(trn), atomName);
    (void) strcat(atomName, "____");
    PlKeySp(PlcGKey(TrDrcGPlc(trDrc)), atomName);
    TrDrcSGamma(trDrc, NmAtom(atomName));
#else
    TrDrcSGamma(trDrc, FreshAtom());
#endif
}

#ifdef DEBUG
FE2(void, MkTrPl,
    TRN *, trn,
    PLC *, plc)
#else
FE1(void, MkTrPl,
    TRN *, trn)
#endif
{
    VOID *oder;

#ifdef DEBUG
    (void) strcpy(atomName, "oder__");
    InsSp(TrnGKey(trn), atomName);
    (void) strcat(atomName, "____");
    PlKeySp(PlcGKey(plc), atomName);
    oder = NmAtom(atomName);
    (void) strcpy(atomName, "strong__");
    InsSp(TrnGKey(trn), atomName);
    (void) strcat(atomName, "____");
    PlKeySp(PlcGKey(plc), atomName);
    TrnSStrong(trn, NmAtom(atomName));
    (void) strcpy(atomName, "weak__");
    InsSp(TrnGKey(trn), atomName);
    (void) strcat(atomName, "____");
    PlKeySp(PlcGKey(plc), atomName);
    TrnSWeak(trn, NmAtom(atomName));
#else
    oder = FreshAtom();
    TrnSStrong(trn, FreshAtom());
    TrnSWeak(trn, FreshAtom());
#endif
    StartRule(TrnGGamma(trn));
    if (PtLongPlay())
	ContRule(TrnGActive(trn));
    ContRule(oder);
    EndRule();
    StartRule(oder);
    if (PtLongPlay())
	ContRule(TrnGActive(trn));
    ContRule(TrnGStrong(trn));
    ContRule(TrnGWeak(trn));
    EndRule();
}

FE2(void, SimpleRule,
    VOID *, head,
    VOID *, tail)
{
    StartRule(head);
    ContRule(tail);
    EndRule();
}

FE2(void, ReverseRules,
    VOID *, atom1,
    VOID *, atom2)
{
    StartRule(atom1);
    NegContRule(atom2);
    EndRule();
    StartRule(atom2);
    NegContRule(atom1);
    EndRule();
}

#endif
