/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  ptsymm2.c
*
* Part of: PROD's programs araprod and "gphgen"
*
* Author:  Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     See ptsymm.c.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ptrbl.h"
#include "baltree.h"
#include "bdd.h"
#include "gphutil.h"
#include "graph.h"
#include "mark.h"
#include "encode.h"
#include "ptnet.h"
#include "ptsymm.h"

FE1(BOOL, MltTrnPre,
    TRN *, trn)
{
    TRARC *trArc;
    TRDRC *trDrc;
    unsigned long *lng;

    if (TrnGPurIn(trn))
	LSTFOR (TrnGPurIn(trn), trArc, TRARC)
	    {
		lng = TrArcAWeight(trArc);
		(void) BalTrInsert(ptrTree, (char *) lng, (char *) lng, TRUE);
	    }
    if (TrnGNeutral(trn))
	LSTFOR (TrnGNeutral(trn), trArc, TRARC)
	    {
		lng = TrArcAWeight(trArc);
		(void) BalTrInsert(ptrTree, (char *) lng, (char *) lng, TRUE);
	    }
    if (TrnGMosIn(trn))
	LSTFOR (TrnGMosIn(trn), trDrc, TRDRC)
	    {
		lng = TrDrcAIn(trDrc);
		(void) BalTrInsert(ptrTree, (char *) lng, (char *) lng, TRUE);
	    }
    if (TrnGMosOut(trn))
	LSTFOR (TrnGMosOut(trn), trDrc, TRDRC)
	    {
		lng = TrDrcAIn(trDrc);
		(void) BalTrInsert(ptrTree, (char *) lng, (char *) lng, TRUE);
	    }
    return(FALSE);
}

FE1(BOOL, MltTrnPost,
    TRN *, trn)
{
    TRARC *trArc;
    TRDRC *trDrc;
    unsigned long *lng;

    if (TrnGPurOut(trn))
	LSTFOR (TrnGPurOut(trn), trArc, TRARC)
	    {
		lng = TrArcAWeight(trArc);
		(void) BalTrInsert(ptrTree, (char *) lng, (char *) lng, TRUE);
	    }
    if (TrnGNeutral(trn))
	LSTFOR (TrnGNeutral(trn), trArc, TRARC)
	    {
		lng = TrArcAWeight(trArc);
		(void) BalTrInsert(ptrTree, (char *) lng, (char *) lng, TRUE);
	    }
    if (TrnGMosIn(trn))
	LSTFOR (TrnGMosIn(trn), trDrc, TRDRC)
	    {
		lng = TrDrcAOut(trDrc);
		(void) BalTrInsert(ptrTree, (char *) lng, (char *) lng, TRUE);
	    }
    if (TrnGMosOut(trn))
	LSTFOR (TrnGMosOut(trn), trDrc, TRDRC)
	    {
		lng = TrDrcAOut(trDrc);
		(void) BalTrInsert(ptrTree, (char *) lng, (char *) lng, TRUE);
	    }
    return(FALSE);
}

FE2(BALTREE *, MltPrePlcTr,
    BALTREE *, plcTree,
    BALTREE *, lngTree)
{
    ptrTree = lngTree;
    (void) BalTrApply(plcTree, MltPlcPre);
    return(ptrTree);
}

FE2(BALTREE *, MltPostPlcTr,
    BALTREE *, plcTree,
    BALTREE *, lngTree)
{
    ptrTree = lngTree;
    (void) BalTrApply(plcTree, MltPlcPost);
    return(ptrTree);
}

FE2(BALTREE *, MltPreTrnTr,
    BALTREE *, trnTree,
    BALTREE *, lngTree)
{
    ptrTree = lngTree;
    (void) BalTrApply(trnTree, MltTrnPre);
    return(ptrTree);
}

FE2(BALTREE *, MltPostTrnTr,
    BALTREE *, trnTree,
    BALTREE *, lngTree)
{
    ptrTree = lngTree;
    (void) BalTrApply(trnTree, MltTrnPost);
    return(ptrTree);
}

FE2(PP *, PpCons,
    BALTREE *, domain,
    BALTREE *, range)
{
    PP *pp;

    pp = PpAlloc();
    PpSDomain(pp, domain);
    PpSRange(pp, range);
    return(pp);
}

FE1(PP *, PpCopy,
    PP *, pp)
{
    PP *copy;

    copy = PpAlloc();
    PpSDomain(copy, BalTrCopy(PpGDomain(pp), FALSE));
    PpSRange(copy, BalTrCopy(PpGRange(pp), FALSE));
    return(copy);
}

FE1(void, PpDiscard,
    PP *, pp)
{
    BalTrDestroy(PpGDomain(pp), FALSE);
    BalTrDestroy(PpGRange(pp), FALSE);
    Deallocate((char *) pp);
}

#ifdef DEBUG
FE2(void, PpDump,
    PP *, pp,
    unsigned, depth)
{
    DumpIndent(depth);
    (void) fprintf(_err, "PP domain\n");
    BalTrDump(PpGDomain(pp), depth + 1);
    DumpIndent(depth);
    (void) fprintf(_err, "PP range\n");
    BalTrDump(PpGRange(pp), depth + 1);
}
#endif

TOOLS ppTools =
{ (CFUNC) PpCopy, PpDiscard
#ifdef DEBUG
, PpDump
#endif
 };

FE2(CNSTR *, CnstrCons,
    LIST *, pPairs,
    LIST *, tPairs)
{
    CNSTR *cnstr;

    cnstr = CnstrAlloc();
    CnstrSPPairs(cnstr, pPairs);
    CnstrSTPairs(cnstr, tPairs);
    return(cnstr);
}

FE1(CNSTR *, CnstrCopy,
    CNSTR *, cnstr)
{
    CNSTR *copy;

    copy = CnstrAlloc();
    CnstrSPPairs(copy, LstCopy(CnstrGPPairs(cnstr), TRUE));
    CnstrSTPairs(copy, LstCopy(CnstrGTPairs(cnstr), TRUE));
    return(copy);
}

FE1(void, CnstrDiscard,
    CNSTR *, cnstr)
{
    LstDestroy(CnstrGPPairs(cnstr), TRUE);
    LstDestroy(CnstrGTPairs(cnstr), TRUE);
    Deallocate((char *) cnstr);
}

#ifdef DEBUG
FE2(void, CnstrDump,
    CNSTR *, cnstr,
    unsigned, depth)
{
    DumpIndent(depth);
    (void) fprintf(_err, "CNSTR pPairs\n");
    LstDump(CnstrGPPairs(cnstr), depth + 1);
    DumpIndent(depth);
    (void) fprintf(_err, "CNSTR tPairs\n");
    LstDump(CnstrGTPairs(cnstr), depth + 1);
}

FE1(void, CnstrChk,
    CNSTR *, cnstr)
{
    unsigned long i;
    PP *pp;

    i = 0L;
    LSTFOR (CnstrGPPairs(cnstr), pp, PP)
	{
	    BalTrChkBal(PpGDomain(pp));
	    BalTrChkBal(PpGRange(pp));
	    i += BalTrGElCount(PpGDomain(pp));
	    if ((BalTrGElCount(PpGDomain(pp)) != 0L) &&
		BalTrGElCount(PpGDomain(pp)) == BalTrGElCount(PpGRange(pp)))
		continue;
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err,
			   "\nCnstrChk: domain %lu, range %lu\n",
			   BalTrGElCount(PpGDomain(pp)),
			   BalTrGElCount(PpGRange(pp)));
	    (void) fflush(_err);
	    abort();
	}
    if (i != PtrBlGCount(TblPlc()))
	{
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err,
			   "\nCnstrChk: domain sum %lu, PtrBlGCount(TblPLc()) %lu\n",
			   i, PtrBlGCount(TblPlc()));
	    (void) fflush(_err);
	    abort();
	}
    i = 0L;
    LSTFOR (CnstrGTPairs(cnstr), pp, PP)
	{
	    BalTrChkBal(PpGDomain(pp));
	    BalTrChkBal(PpGRange(pp));
	    i += BalTrGElCount(PpGDomain(pp));
	    if ((BalTrGElCount(PpGDomain(pp)) != 0L) &&
		BalTrGElCount(PpGDomain(pp)) == BalTrGElCount(PpGRange(pp)))
		continue;
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err,
			   "\nCnstrChk: domain %lu, range %lu\n",
			   BalTrGElCount(PpGDomain(pp)),
			   BalTrGElCount(PpGRange(pp)));
	    (void) fflush(_err);
	    abort();
	}
    if (i != PtrBlGCount(TblTrn(0)))
	{
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err,
			   "\nCnstrChk: domain sum %lu, PtrBlGCount(TblTrn(0)) %lu\n",
			   i, PtrBlGCount(TblTrn(0)));
	    (void) fflush(_err);
	    abort();
	}
}
#endif

FE1(PTRBL *, MrepCopy,
    PTRBL *, mrep)
{
    return(PtrBlCopy(mrep, FALSE));
}

FE1(void, MrepDiscard,
    PTRBL *, mrep)
{
    PtrBlDestroy(mrep, FALSE);
}

FE2(int, MrepCompare,
    PTRBL *, mrep1,
    PTRBL *, mrep2)
{
    unsigned long i, u, v;

    if (mrep1 == mrep2)
	return(0);
    for (i = 0L; i < PtrBlGCount(TblPlc()); i++)
	{
	    u = *((unsigned long *) PtrBlGet(mrep1, i));
	    v = *((unsigned long *) PtrBlGet(mrep2, i));
	    if (u < v)
		return(-1);
	    else if (u > v)
		return(1);
	}
    return(0);
}

TOOLS mrepTools =
{ (CFUNC) MrepCopy, MrepDiscard
#ifdef DEBUG
, PtrBlDump
#endif
 };

BALTOOLS mrepBalTools =
{ (CFUNC) MrepCopy, MrepDiscard
#ifdef DEBUG
, PtrBlDump
#endif
, MrepCompare, (FOOL) NULL };

static CNSTR *ptCnstr;
static PP *ptPpOld, *ptTtOld, *ptPp1, *ptPp2, *ptTt1, *ptTt2;
static BALTREE *ptF, *ptA1, *ptA2, *ptB1, *ptB2;
static BOOL ptRefOK;

FE1(BOOL, RefiPrePost,
    unsigned long *, weight)
{
    ptF = PreTrnTr(PpGDomain(ptTtOld), *weight);
    ptA1 = PtrTrCut(PpGDomain(ptPpOld), ptF);
    ptA2 = PtrTrMinus(PpGDomain(ptPpOld), ptF);
    BalTrDestroy(ptF, FALSE);
    ptF = PreTrnTr(PpGRange(ptTtOld), *weight);
    ptB1 = PtrTrCut(PpGRange(ptPpOld), ptF);
    ptB2 = PtrTrMinus(PpGRange(ptPpOld), ptF);
    BalTrDestroy(ptF, FALSE);
    if ((BalTrGElCount(ptA1) != BalTrGElCount(ptB1)) ||
	(BalTrGElCount(ptA2) != BalTrGElCount(ptB2)))
	ptRefOK = FALSE;
    if ((BalTrGElCount(ptA1) == 0L) || (BalTrGElCount(ptA2) == 0L)
	|| !ptRefOK)
	{
	    BalTrDestroy(ptA1, FALSE);
	    BalTrDestroy(ptA2, FALSE);
	    BalTrDestroy(ptB1, FALSE);
	    BalTrDestroy(ptB2, FALSE);
	    if (!ptRefOK)
		return(TRUE);
	}
    else
	{
	    ptPp1 = PpCons(ptA1, ptB1);
	    ptPp2 = PpCons(ptA2, ptB2);
	}
    ptF = PostPlcTr(PpGDomain(ptPpOld), *weight);
    ptA1 = PtrTrCut(PpGDomain(ptTtOld), ptF);
    ptA2 = PtrTrMinus(PpGDomain(ptTtOld), ptF);
    BalTrDestroy(ptF, FALSE);
    ptF = PostPlcTr(PpGRange(ptPpOld), *weight);
    ptB1 = PtrTrCut(PpGRange(ptTtOld), ptF);
    ptB2 = PtrTrMinus(PpGRange(ptTtOld), ptF); 
    BalTrDestroy(ptF, FALSE);
    if ((BalTrGElCount(ptA1) != BalTrGElCount(ptB1)) ||
	(BalTrGElCount(ptA2) != BalTrGElCount(ptB2)))
	ptRefOK = FALSE;
    if ((BalTrGElCount(ptA1) == 0L) || (BalTrGElCount(ptA2) == 0L)
	|| !ptRefOK)
	{
	    BalTrDestroy(ptA1, FALSE);
	    BalTrDestroy(ptA2, FALSE);
	    BalTrDestroy(ptB1, FALSE);
	    BalTrDestroy(ptB2, FALSE);
	}
    else
	{
	    ptTt1 = PpCons(ptA1, ptB1);
	    ptTt2 = PpCons(ptA2, ptB2);
	}
    return(ptPp1 || ptTt1 || !ptRefOK);
}

FE1(BOOL, RefiPostPre,
    unsigned long *, weight)
{
    ptF = PostTrnTr(PpGDomain(ptTtOld), *weight);
    ptA1 = PtrTrCut(PpGDomain(ptPpOld), ptF);
    ptA2 = PtrTrMinus(PpGDomain(ptPpOld), ptF);
    BalTrDestroy(ptF, FALSE);
    ptF = PostTrnTr(PpGRange(ptTtOld), *weight);
    ptB1 = PtrTrCut(PpGRange(ptPpOld), ptF);
    ptB2 = PtrTrMinus(PpGRange(ptPpOld), ptF);
    if ((BalTrGElCount(ptA1) != BalTrGElCount(ptB1)) ||
	(BalTrGElCount(ptA2) != BalTrGElCount(ptB2)))
	ptRefOK = FALSE;
    if ((BalTrGElCount(ptA1) == 0L) || (BalTrGElCount(ptA2) == 0L)
	|| !ptRefOK)
	{
	    BalTrDestroy(ptA1, FALSE);
	    BalTrDestroy(ptA2, FALSE);
	    BalTrDestroy(ptB1, FALSE);
	    BalTrDestroy(ptB2, FALSE);
	    if (!ptRefOK)
		return(TRUE);
	}
    else
	{
	    ptPp1 = PpCons(ptA1, ptB1);
	    ptPp2 = PpCons(ptA2, ptB2);
	}
    BalTrDestroy(ptF, FALSE);
    ptF = PrePlcTr(PpGDomain(ptPpOld), *weight);
    ptA1 = PtrTrCut(PpGDomain(ptTtOld), ptF);
    ptA2 = PtrTrMinus(PpGDomain(ptTtOld), ptF);
    BalTrDestroy(ptF, FALSE);
    ptF = PrePlcTr(PpGRange(ptPpOld), *weight);
    ptB1 = PtrTrCut(PpGRange(ptTtOld), ptF);
    ptB2 = PtrTrMinus(PpGRange(ptTtOld), ptF);
    BalTrDestroy(ptF, FALSE);
    if ((BalTrGElCount(ptA1) != BalTrGElCount(ptB1)) ||
	(BalTrGElCount(ptA2) != BalTrGElCount(ptB2)))
	ptRefOK = FALSE;
    if ((BalTrGElCount(ptA1) == 0L) || (BalTrGElCount(ptA2) == 0L)
	|| !ptRefOK)
	{
	    BalTrDestroy(ptA1, FALSE);
	    BalTrDestroy(ptA2, FALSE);
	    BalTrDestroy(ptB1, FALSE);
	    BalTrDestroy(ptB2, FALSE);
	}
    else
	{
	    ptTt1 = PpCons(ptA1, ptB1);
	    ptTt2 = PpCons(ptA2, ptB2);
	}
    return(ptPp1 || ptTt1 || !ptRefOK);
}

FE0(BOOL, Refine)
{
    static BALTREE *preTree, *pstTree, *tree;
    static VOID *ppCrsr, *ttCrsr;
    static unsigned long elCount;

    ptPp1 = NULL;
    ptTt1 = NULL;
    ppCrsr = NULL;
    ttCrsr = NULL;
    LSTFOR (CnstrGPPairs(ptCnstr), ptPpOld, PP)
	{
	    preTree = MltPrePlcTr(PpGDomain(ptPpOld),
				  BalTrCreate(&lngBalTools));
	    elCount = BalTrGElCount(preTree);
	    preTree = MltPrePlcTr(PpGRange(ptPpOld), preTree);
	    if (BalTrGElCount(preTree) != elCount)
		{
		    ptRefOK = FALSE;
		    BalTrDestroy(preTree, FALSE);
		    return(FALSE);
		}
	    pstTree = MltPostPlcTr(PpGDomain(ptPpOld),
				    BalTrCreate(&lngBalTools));
	    elCount = BalTrGElCount(pstTree);
	    pstTree = MltPostPlcTr(PpGRange(ptPpOld), pstTree);
	    if (BalTrGElCount(pstTree) != elCount)
		{
		    ptRefOK = FALSE;
		    BalTrDestroy(preTree, FALSE);
		    BalTrDestroy(pstTree, FALSE);
		    return(FALSE);
		}
	    ttCrsr = NULL;
	    LSTFOR (CnstrGTPairs(ptCnstr), ptTtOld, PP)
		{
		    tree = MltPreTrnTr(PpGDomain(ptTtOld),
				       BalTrCopy(pstTree, FALSE));
		    elCount = BalTrGElCount(tree);
		    tree = MltPreTrnTr(PpGRange(ptTtOld), tree);
		    if (BalTrGElCount(tree) != elCount)
			{
			    ptRefOK = FALSE;
			    BalTrDestroy(preTree, FALSE);
			    BalTrDestroy(pstTree, FALSE);
			    BalTrDestroy(tree, FALSE);
			    return(FALSE);
			}
		    if (BalTrApply(tree, RefiPrePost))
			{
			    BalTrDestroy(tree, FALSE);
			    break;
			}
		    BalTrDestroy(tree, FALSE);
		    tree = MltPostTrnTr(PpGDomain(ptTtOld),
					BalTrCopy(preTree, FALSE));
		    elCount = BalTrGElCount(tree);
		    tree = MltPostTrnTr(PpGRange(ptTtOld), tree);
		    if (BalTrGElCount(tree) != elCount)
			{
			    ptRefOK = FALSE;
			    BalTrDestroy(preTree, FALSE);
			    BalTrDestroy(pstTree, FALSE);
			    BalTrDestroy(tree, FALSE);
			    return(FALSE);
			}
		    if (BalTrApply(tree, RefiPostPre))
			{
			    BalTrDestroy(tree, FALSE);
			    break;
			}
		    BalTrDestroy(tree, FALSE);
		    ttCrsr = LstGet(CnstrGTPairs(ptCnstr));
		}
	    BalTrDestroy(preTree, FALSE);
	    BalTrDestroy(pstTree, FALSE);
	    if (!ptRefOK)
		return(FALSE);
	    if (ptPp1 || ptTt1)
		break;
	    ppCrsr = LstGet(CnstrGPPairs(ptCnstr));
	}
    if (ptPp1)
	{
	    LstSet(CnstrGPPairs(ptCnstr), ppCrsr);
	    (void) LstDelNext(CnstrGPPairs(ptCnstr), TRUE);
	    (void) LstInsFirst(CnstrGPPairs(ptCnstr), (char *) ptPp1);
	    (void) LstInsFirst(CnstrGPPairs(ptCnstr), (char *) ptPp2);
	}
    if (ptTt1)
	{
	    LstSet(CnstrGTPairs(ptCnstr), ttCrsr);
	    (void) LstDelNext(CnstrGTPairs(ptCnstr), TRUE);
	    (void) LstInsFirst(CnstrGTPairs(ptCnstr), (char *) ptTt1);
	    (void) LstInsFirst(CnstrGTPairs(ptCnstr), (char *) ptTt2);
	}
    return((ptPp1 != NULL) || (ptTt1 != NULL));
}

FE0(void, ExhaustRefine)
{
    ptRefOK = TRUE;
    while (Refine())
	;
    if (!ptRefOK)
	{
	    CnstrDiscard(ptCnstr);
	    ptCnstr = NULL;
	}
}

static BALTREE *front, *newFront;
static CNSTR *ptConstraint;
static unsigned long ptI;
static PP *ptKL;
static char *ptX;
static VOID *ptCursor;
static PTRBL *plcSymm, *plcSmm2;
static PLC *plcPrev;
static TRN *trnPrev;

FE1(BOOL, PartialSymm,
    PTRBL *, symm)
{
    LSTFOR (CnstrGPPairs(ptCnstr), ptKL, PP)
	{
	    if (BalTrGElCount(PpGDomain(ptKL)) != 1L)
		continue;
	    ptI = PlcGInd((PLC *) BalTrFirst(PpGDomain(ptKL)));
	    if (PlcGInd((PLC *)	BalTrFirst(PpGRange(ptKL))) !=
		*((unsigned long *) PtrBlGet(symm, ptI)))
		return(FALSE);
	}
    return(TRUE);
}

static BOOL ptStarted;

FE1(BOOL, FirstExpand,
    PTRBL *, symm)
{
    if (!ptStarted)
	{
	    ptStarted = TRUE;
	    return(FALSE);
	}
    plcSmm2 = PtrBlCreate(PtrBlGCount(TblPlc()), &ptrTools);
    ptI = PtrBlGCount(TblPlc());
    while(ptI--)
	PtrBlPut(plcSmm2, (char *)
		 PtrBlGet(plcSymm,
			  *((unsigned long *)
			    PtrBlGet(symm, ptI))),
		 ptI);
    if (BalTrElement(PlcSymmetries(), (char *) plcSmm2))
	{
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err,
			   "\nFirstExpand: if y and z are old and x * y = z then x = z * y^(-1)\nis no new generator\n");
	    abort();
	}
    if (BalTrInsert(front, (char *) plcSmm2, (char *) plcSmm2, TRUE))
	{
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err,
			   "\nFirstExpand: no group: x * y == x * z though y != z\n");
	    (void) fflush(_err);
	    abort();
	}
    return(FALSE);
}

FE1(BOOL, SymmExpand,
    PTRBL *, symm)
{
    LSTFOR (PlcSymmGenerators(), plcSymm, PTRBL)
	{
	    plcSmm2 = PtrBlCreate(PtrBlGCount(TblPlc()), &ptrTools);
	    ptI = PtrBlGCount(TblPlc());
	    while(ptI--)
		PtrBlPut(plcSmm2, (char *)
			 PtrBlGet(plcSymm,
				  *((unsigned long *)
				    PtrBlGet(symm, ptI))),
			 ptI);
	    if (BalTrElement(PlcSymmetries(), (char *) plcSmm2))
		{
		    MrepDiscard(plcSmm2);
		    continue;
		}
	    if (!newFront)
		{
		    newFront = BalTrCreate(&mrepBalTools);
		    (void) BalTrInsert(newFront, (char *) plcSmm2,
				       (char *) plcSmm2, TRUE);
		    continue;
		}
	    if (BalTrInsert(newFront, (char *) plcSmm2,
			    (char *) plcSmm2, TRUE))
		MrepDiscard(plcSmm2);
	}
    return(FALSE);
}

FE1(BOOL, SymmRealize,
    PTRBL *, symm)
{
    (void) BalTrInsert(PlcSymmetries(), (char *) symm, (char *) symm, TRUE);
    return(FALSE);
}

FE1(BOOL, DeSyP,
    PLC *, plc)
{
    ptCnstr = CnstrCopy(ptConstraint);
    (void) BalTrDelete(PpGRange((PP *) LstFirst(CnstrGPPairs(ptCnstr))),
		       (char *) NULL, (char *) plc, FALSE);
    (void) BalTrInsert(PpGRange((PP *) LstLast(CnstrGPPairs(ptCnstr))),
		       (char *) plc, (char *) plc, TRUE);
    ExhaustRefine();
    if (!ptCnstr)
	return(FALSE);
    if (BalTrApply(PlcSymmetries(), PartialSymm))
	{
	    CnstrDiscard(ptCnstr);
	    return(FALSE);
	}
    ptCursor = NULL;
    LSTFOR (CnstrGPPairs(ptCnstr), ptKL, PP)
	{
	    if (BalTrGElCount(PpGDomain(ptKL)) != 1L)
		break;
	    ptCursor = LstGet(CnstrGPPairs(ptCnstr));
	}
    if (ptKL)
	{
	    DefineSym(ptConstraint);
	    return(FALSE);
	}
    plcSymm = PtrBlCreate(PtrBlGCount(TblPlc()), &ptrTools);
    LSTFOR (CnstrGPPairs(ptCnstr), ptKL, PP)
	PtrBlPut(plcSymm, (char *)
		 PlcAInd((PLC *) BalTrFirst(PpGRange(ptKL))),
		 PlcGInd((PLC *) BalTrFirst(PpGDomain(ptKL))));
    CnstrDiscard(ptCnstr);
    if (BalTrGElCount(PlcSymmetries()) == 0L)
	{
	    ptI = PtrBlGCount(TblPlc());
	    while (ptI--)
		{
		    if (ptI == *((unsigned long *) PtrBlGet(plcSymm, ptI)))
			continue;
		    (void) fprintf(_out, "\n");
		    (void) fflush(_out);
		    (void) fprintf(_err,
				   "\nDeSyP: the first mapping found is not the identity mapping\n");
		    (void) fflush(_err);
		    abort();
		}
	    (void) BalTrInsert(PlcSymmetries(), (char *) plcSymm,
			       (char *) plcSymm, TRUE);
	    return(FALSE);
	}
    front = BalTrCreate(&mrepBalTools);
    ptStarted = FALSE;
    (void) BalTrApply(PlcSymmetries(), FirstExpand);
    if (BalTrInsert(PlcSymmetries(), (char *) plcSymm, (char *) plcSymm, TRUE))
	{
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err,
			   "\nDeSyP: old mapping instead of new generator\n");
	    (void) fflush(_err);
	    abort();
	}
    (void) BalTrApply(front, SymmRealize);
    if (BalTrInsert(front, (char *) plcSymm, (char *) plcSymm, TRUE))
	{
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err,
			   "\nDeSyP: no group: x * y == x * z though y != z\n");
	    (void) fflush(_err);
	    abort();
	}
    (void) LstInsLast(PlcSymmGenerators(), (char *) plcSymm);
    do
	{
	    newFront = NULL;
	    (void) BalTrApply(front, SymmExpand);
	    BalTrDestroy(front, FALSE);
	    if (newFront)
		{
		    front = newFront;
		    (void) BalTrApply(front, SymmRealize);
		}
	} while (newFront);
    return(FALSE);
}

FE1(void, DefineSym,
    CNSTR *, cnstr)
{
    LstSet(CnstrGPPairs(ptCnstr), ptCursor);
    (void) LstDelNext(CnstrGPPairs(ptCnstr), FALSE);
    ptX = (char *) BalTrFirst(PpGDomain(ptKL));
    (void) BalTrDelete(PpGDomain(ptKL), (char *) NULL, ptX, FALSE);
    (void) LstInsFirst(CnstrGPPairs(ptCnstr), (char *) ptKL);
    (void) LstInsLast(CnstrGPPairs(ptCnstr), (char *)
		      PpCons(BalTrCreate(&ptrBalTools),
			     BalTrCreate(&ptrBalTools)));
    (void) BalTrInsert(PpGDomain((PP *)
				 LstLast(CnstrGPPairs(ptCnstr))),
		       ptX, ptX, TRUE);
    ptConstraint = ptCnstr;
    (void) BalTrApply(PpGRange(ptKL), DeSyP);
    CnstrDiscard(ptConstraint);
    ptConstraint = cnstr;
}

FE1(BOOL, PlcGroup,
    PLC *, plc)
{
    if (plcPrev && PlcDegCompare(plc, plcPrev))
	{
	    (void) LstInsLast(CnstrGPPairs(ptCnstr), (char *)
			      PpCons(ptrTree2,
				     BalTrCopy(ptrTree2, FALSE)));
	    ptrTree2 = BalTrCreate(&ptrBalTools);
	}
    (void) BalTrInsert(ptrTree2, (char *) plc, (char *) plc, TRUE);
    plcPrev = plc;
    return(FALSE);
}

FE1(BOOL, TrnGroup,
    TRN *, trn)
{
    if (trnPrev && TrnDegCompare(trn, trnPrev))
	{
	    (void) LstInsLast(CnstrGTPairs(ptCnstr), (char *)
			      PpCons(ptrTree2,
				     BalTrCopy(ptrTree2, FALSE)));
	    ptrTree2 = BalTrCreate(&ptrBalTools);
	}
    (void) BalTrInsert(ptrTree2, (char *) trn, (char *) trn, TRUE);
    trnPrev = trn;
    return(FALSE);
}

static BOOL delAlg;
static BOOL sySkip;
static BOOL syCheck;
static unsigned long ptU;

FE1(BOOL, PlcNorm,
    PLC *, plc)
{
#ifndef DEBUG
    if (PtVerbose())
	PlcDKey(plc);
#endif
    if (delAlg)
	PlcCOrPredAnd(plc);
    return(FALSE);
}

FE1(BOOL, SymmOut,
    PTRBL *, symm)
{
    if (sySkip)
	{
	    sySkip = FALSE;
	    return(FALSE);
	}
    if (syCheck && LstWhich(PlcSymmGenerators(), (char *) symm))
	return(FALSE);
    (void) fprintf(_out, "%s %lu:\n", syCheck ? "Symmetry" : "Generator", ptU);
    for (ptI = 0L; ptI < PtrBlGCount(TblPlc()); ptI++)
	{
	    PlKeyOut(PlcGKey((PLC *) PtrBlGet(TblPlc(), ptI)),
		     _out, FALSE);
	    (void) fprintf(_out, " -> ");
	    PlKeyOut(PlcGKey((PLC *)
			     PtrBlGet(TblPlc(),
				      *((unsigned long *)
					PtrBlGet(symm, ptI)))),
		     _out, FALSE);
	    (void) fprintf(_out, "\n");
	}
    ptU++;
    return(FALSE);
}

FE1(void, CompSymm,
    BOOL, delIdMap)
{
    delAlg = PtDelAlg();
    ClPtDelAlg();
    _plcSyGenerators = LstCreate(&mrepTools);
    _plcSymmetries = BalTrCreate(&mrepBalTools);
    ptCnstr = CnstrCons(LstCreate(&ppTools), LstCreate(&ppTools));
    ptrTree = BalTrCreate(&clpBalTools);
    (void) PtrBlApply(TblPlc(), PtrAdd);
    ptrTree2 = BalTrCreate(&ptrBalTools);
    plcPrev = NULL;
    (void) BalTrApply(ptrTree, PlcGroup);
    (void) LstInsLast(CnstrGPPairs(ptCnstr), (char *)
		      PpCons(ptrTree2, BalTrCopy(ptrTree2, FALSE)));
    BalTrDestroy(ptrTree, FALSE);
    LSTFOR (CnstrGPPairs(ptCnstr), ptKL, PP)
	if (BalTrGElCount(PpGDomain(ptKL)) != 1L)
	    break;
    if (!ptKL)
	{
	    (void) fprintf(_out, "\nNo non-identity place mapping in any symmetry\n");
	    (void) fflush(_out);
	    if (delIdMap)
		{
		    CnstrDiscard(ptCnstr);
		    return;
		}
	    plcSymm = PtrBlCreate(PtrBlGCount(TblPlc()), &ptrTools);
	    LSTFOR (CnstrGPPairs(ptCnstr), ptKL, PP)
		PtrBlPut(plcSymm, (char *)
			 PlcAInd((PLC *) BalTrFirst(PpGRange(ptKL))),
			 PlcGInd((PLC *) BalTrFirst(PpGDomain(ptKL))));
	    CnstrDiscard(ptCnstr);
	    (void) BalTrInsert(PlcSymmetries(), (char *) plcSymm,
			       (char *) plcSymm, TRUE);
	    if (delAlg)
		SetPtDelAlg();
	    if (delAlg || PtVerbose())
		(void) PtrBlApply(TblPlc(), PlcNorm);
#ifndef DEBUG
	    if (PtVerbose())
		DelPlaNames();
#endif
	    return;
	}
    ptrTree = BalTrCreate(&nrtBalTools);
    (void) PtrBlApply(TblTrn(0), PtrAdd);
    ptrTree2 = BalTrCreate(&ptrBalTools);
    trnPrev = NULL;
    (void) BalTrApply(ptrTree, TrnGroup);
    (void) LstInsLast(CnstrGTPairs(ptCnstr), (char *)
		      PpCons(ptrTree2,
			     BalTrCopy(ptrTree2, FALSE)));
    BalTrDestroy(ptrTree, FALSE);
    ExhaustRefine();
    if (!ptCnstr)
	{
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err,
			   "\nCompSymm: initial refinement failed\n");
	    (void) fflush(_err);
	    abort();
	}
    ptCursor = NULL;
    LSTFOR (CnstrGPPairs(ptCnstr), ptKL, PP)
	{
	    if (BalTrGElCount(PpGDomain(ptKL)) != 1L)
		break;
	    ptCursor = LstGet(CnstrGPPairs(ptCnstr));
	}
    if (!ptKL)
	{
	    (void) fprintf(_out, "\nNo non-identity place mapping in any symmetry\n");
	    (void) fflush(_out);
	    if (delIdMap)
		{
		    CnstrDiscard(ptCnstr);
		    return;
		}
	    plcSymm = PtrBlCreate(PtrBlGCount(TblPlc()), &ptrTools);
	    LSTFOR (CnstrGPPairs(ptCnstr), ptKL, PP)
		PtrBlPut(plcSymm, (char *)
			 PlcAInd((PLC *) BalTrFirst(PpGRange(ptKL))),
			 PlcGInd((PLC *) BalTrFirst(PpGDomain(ptKL))));
	    CnstrDiscard(ptCnstr);
	    (void) BalTrInsert(PlcSymmetries(), (char *) plcSymm,
			       (char *) plcSymm, TRUE);
	    if (delAlg)
		SetPtDelAlg();
	    if (delAlg || PtVerbose())
		(void) PtrBlApply(TblPlc(), PlcNorm);
#ifndef DEBUG
	    if (PtVerbose())
		DelPlaNames();
#endif
	    return;
	}
    DefineSym((CNSTR *) NULL);
    plcSymm = (PTRBL *) BalTrFirst(PlcSymmetries());
    ptI = PtrBlGCount(TblPlc());
    while (ptI--)
	{
	    if (ptI == *((unsigned long *) PtrBlGet(plcSymm, ptI)))
		continue;
	    (void) fprintf(_out, "\n");
	    (void) fflush(_out);
	    (void) fprintf(_err,
			   "\nCompSymm: the least mapping is not the identity mapping\n");
	    (void) fflush(_err);
	    abort();
	}
    (void) fprintf(_out, "\n%lu non-identity place mappings in symmetries\n%u non-identity place mappings generate the group\n",
		  BalTrGElCount(PlcSymmetries()) - 1L,
		  LstSize(PlcSymmGenerators()));
    if (PtVerbose())
	{
	    sySkip = FALSE;
	    syCheck = FALSE;
	    ptU = 1L;
	    LSTFOR (PlcSymmGenerators(), plcSymm, PTRBL)
		(void) SymmOut(plcSymm);
	    if (PtSuperVerbose())
		{
		    sySkip = TRUE;
		    syCheck = TRUE;
		    (void) BalTrApply(PlcSymmetries(), SymmOut);
		}
	    (void) fflush(_out);
	}
    if (delIdMap)
	(void) BalTrDelete(PlcSymmetries(), (char *) NULL,
			   (char *) BalTrFirst(PlcSymmetries()), TRUE);
    if (delAlg)
	SetPtDelAlg();
    if (delAlg || PtVerbose())
	(void) PtrBlApply(TblPlc(), PlcNorm);
#ifndef DEBUG
    if (PtVerbose())
	DelPlaNames();
#endif
}
