/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  ptvisit.c
*
* Part of: PROD's programs araprod and "gphgen"
*
* Author:  Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     This module contains routines for the incremental algorithm of
*     the stubborn set method.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ptrbl.h"
#include "baltree.h"
#include "bdd.h"
#include "gphutil.h"
#include "graph.h"
#include "mark.h"
#include "encode.h"
#include "ptnet.h"

TRN *ptVisible;
TRN *ptInvisible;
unsigned long ptSz;
unsigned long ptV;
unsigned long ptLimit;

/***************************************************************************
*
* Function:  TrnAntiVisit
* Purpose:   This routine marks a transition unvisited.
* Arguments: trn - the transition
* Returns:
* See also:
*
****************************************************************************
*/
FE1(BOOL, TrnAntiVisit,
    TRN *, trn)
{
    TrnSVal(trn, 0L);
    TrnImpoverish(trn);
    return(FALSE);
}

/***************************************************************************
*
* Function:  TrnDsPrIn
* Purpose:   This is an auxiliary routine for TrnStub
* Arguments: trn - a transition
* Returns:
* See also:  TrnStub
*
****************************************************************************
*/
FE1(BOOL, TrnDsPrIn,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        }
    LSTFOR (TrnGPurIn(trn), ptTrArc, TRARC)
	{
	    ptPlc = TrArcGPlc(ptTrArc);
	    if (PlcGCount(ptPlc) >= TrArcGWeight(ptTrArc))
		continue;
	    if (ptV-- != 0L)
		continue;
	    if (PlcGPurIn(ptPlc))
		LSTFOR (PlcGPurIn(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGPurIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return(TRUE);
L1: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return(TRUE);
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
                                LstSet(PlcGPurIn(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGPurIn(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	    ptLimit = PlcGCount(ptPlc);
	    if (PlcGMosIn(ptPlc))
		LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
		    {
			if (PlDrcGOut(ptPlDrc) > ptLimit)
			    continue;
			ptTrn = PlDrcGTrn(ptPlDrc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return(TRUE);
L2: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return(TRUE);
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
				ptLimit = PlcGCount(ptPlc);
                                LstSet(PlcGMosIn(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosIn(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	    return(TRUE);
	}
    return(FALSE);
}

/***************************************************************************
*
* Function:  TrnDsNeutral
* Purpose:   This is an auxiliary routine for TrnStub
* Arguments: trn - a transition
* Returns:
* See also:  TrnStub
*
****************************************************************************
*/
FE1(BOOL, TrnDsNeutral,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        }
    LSTFOR (TrnGNeutral(trn), ptTrArc, TRARC)
	{
	    ptPlc = TrArcGPlc(ptTrArc);
	    if (PlcGCount(ptPlc) >= TrArcGWeight(ptTrArc))
		continue;
	    if (ptV-- != 0L)
		continue;
	    if (PlcGPurIn(ptPlc))
		LSTFOR (PlcGPurIn(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGPurIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return(TRUE);
L1: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return(TRUE);
				ptTrArc = (TRARC *)
				    LstThis(TrnGNeutral(trn));
				ptPlc = TrArcGPlc(ptTrArc);
                                LstSet(PlcGPurIn(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGPurIn(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	    ptLimit = PlcGCount(ptPlc);
	    if (PlcGMosIn(ptPlc))
		LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
		    {
			if (PlDrcGOut(ptPlDrc) > ptLimit)
			    continue;
			ptTrn = PlDrcGTrn(ptPlDrc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return(TRUE);
L2: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return(TRUE);
				ptTrArc = (TRARC *)
				    LstThis(TrnGNeutral(trn));
				ptPlc = TrArcGPlc(ptTrArc);
				ptLimit = PlcGCount(ptPlc);
                                LstSet(PlcGMosIn(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosIn(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	    return(TRUE);
	}
    return(FALSE);
}

/***************************************************************************
*
* Function:  TrnDsMsIn
* Purpose:   This is an auxiliary routine for TrnStub
* Arguments: trn - a transition
* Returns:
* See also:  TrnStub
*
****************************************************************************
*/
FE1(BOOL, TrnDsMsIn,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        }
    LSTFOR (TrnGMosIn(trn), ptTrDrc, TRDRC)
	{
	    ptPlc = TrDrcGPlc(ptTrDrc);
	    if (PlcGCount(ptPlc) >= TrDrcGIn(ptTrDrc))
		continue;
	    if (ptV-- != 0L)
		continue;
	    if (PlcGPurIn(ptPlc))
		LSTFOR (PlcGPurIn(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGPurIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return(TRUE);
L1: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return(TRUE);
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosIn(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
                                LstSet(PlcGPurIn(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGPurIn(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	    ptLimit = PlcGCount(ptPlc);
	    if (PlcGMosIn(ptPlc))
		LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
		    {
			if (PlDrcGOut(ptPlDrc) > ptLimit)
			    continue;
			ptTrn = PlDrcGTrn(ptPlDrc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return(TRUE);
L2: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return(TRUE);
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosIn(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
				ptLimit = PlcGCount(ptPlc);
                                LstSet(PlcGMosIn(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosIn(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	    return(TRUE);
	}
    return(FALSE);
}

/***************************************************************************
*
* Function:  TrnDsMsOut
* Purpose:   This is an auxiliary routine for TrnStub
* Arguments: trn - a transition
* Returns:
* See also:  TrnStub
*
****************************************************************************
*/
FE1(BOOL, TrnDsMsOut,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        }
    LSTFOR (TrnGMosOut(trn), ptTrDrc, TRDRC)
	{
	    ptPlc = TrDrcGPlc(ptTrDrc);
	    if (PlcGCount(ptPlc) >= TrDrcGIn(ptTrDrc))
		continue;
	    if (ptV-- != 0L)
		continue;
	    if (PlcGPurIn(ptPlc))
		LSTFOR (PlcGPurIn(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGPurIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return(TRUE);
L1: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return(TRUE);
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosOut(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
                                LstSet(PlcGPurIn(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGPurIn(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	    ptLimit = PlcGCount(ptPlc);
	    if (PlcGMosIn(ptPlc))
		LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
		    {
			if (PlDrcGOut(ptPlDrc) > ptLimit)
			    continue;
			ptTrn = PlDrcGTrn(ptPlDrc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return(TRUE);
L2: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return(TRUE);
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosOut(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
				ptLimit = PlcGCount(ptPlc);
                                LstSet(PlcGMosIn(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosIn(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	    return(TRUE);
	}
    return(FALSE);
}

/***************************************************************************
*
* Function:  TrnEnNeutral
* Purpose:   This is an auxiliary routine for TrnStub
* Arguments: trn - a transition
* Returns:
* See also:  TrnStub
*
****************************************************************************
*/
FE1(void, TrnEnNeutral,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        }
    LSTFOR (TrnGNeutral(trn), ptTrArc, TRARC)
	{
	    ptPlc = TrArcGPlc(ptTrArc);
	    if (PlcGPurOut(ptPlc))
		LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGPurOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return;
L1: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return;
				ptTrArc = (TRARC *)
				    LstThis(TrnGNeutral(trn));
				ptPlc = TrArcGPlc(ptTrArc);
                                LstSet(PlcGPurOut(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGPurOut(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	    if (PlcGMosOut(ptPlc))
		LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
		    {
			ptTrn = PlDrcGTrn(ptPlDrc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return;
L2: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return;
				ptTrArc = (TRARC *)
				    LstThis(TrnGNeutral(trn));
				ptPlc = TrArcGPlc(ptTrArc);
                                LstSet(PlcGMosOut(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosOut(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	}
}

/***************************************************************************
*
* Function:  TrnEnMsOut
* Purpose:   This is an auxiliary routine for TrnStub
* Arguments: trn - a transition
* Returns:
* See also:  TrnStub
*
****************************************************************************
*/
FE1(void, TrnEnMsOut,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        }
    LSTFOR (TrnGMosOut(trn), ptTrDrc, TRDRC)
	{
	    ptPlc = TrDrcGPlc(ptTrDrc);
	    if (PlcGPurOut(ptPlc))
		LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGPurOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return;
L1: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosOut(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
                                LstSet(PlcGPurOut(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGPurOut(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	    if (PlcGMosOut(ptPlc))
		LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
		    {
			ptTrn = PlDrcGTrn(ptPlDrc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return;
L2: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosOut(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
                                LstSet(PlcGMosOut(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosOut(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	}
}

/***************************************************************************
*
* Function:  TrnEnPrIn
* Purpose:   This is an auxiliary routine for TrnStub
* Arguments: trn - a transition
* Returns:
* See also:  TrnStub
*
****************************************************************************
*/
FE1(void, TrnEnPrIn,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        case 3:
            TrnSWhere(trn, (unsigned) 0);
            goto L3;
        case 4:
            TrnSWhere(trn, (unsigned) 0);
            goto L4;
        }
    LSTFOR (TrnGPurIn(trn), ptTrArc, TRARC)
	{
	    ptPlc = TrArcGPlc(ptTrArc);
	    if (PlcGPurOut(ptPlc))
		LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGPurOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return;
L1: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return;
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
                                LstSet(PlcGPurOut(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGPurOut(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	    if (PlcGMosOut(ptPlc))
		LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
		    {
			ptTrn = PlDrcGTrn(ptPlDrc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return;
L2: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return;
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
                                LstSet(PlcGMosOut(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosOut(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	    ptLimit = PlcGCount(ptPlc) - TrArcGWeight(ptTrArc);
	    if (PlcGNeutral(ptPlc))
		LSTFOR (PlcGNeutral(ptPlc), ptPlArc, PLARC)
		    {
			if (PlArcGWeight(ptPlArc) <= ptLimit)
			    continue;
			ptTrn = PlArcGTrn(ptPlArc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGNeutral(ptPlc)));
                                TrnSWhere(trn, (unsigned) 3);
                                return;
L3: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return;
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
				ptLimit = PlcGCount(ptPlc)
				    - TrArcGWeight(ptTrArc);
                                LstSet(PlcGNeutral(ptPlc), TrnGCursor(trn));
				ptPlArc = (PLARC *)
				    LstThis(PlcGNeutral(ptPlc));
				ptTrn = PlArcGTrn(ptPlArc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	    if (PlcGMosIn(ptPlc))
		LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
		    {
			if (PlDrcGOut(ptPlDrc) <= ptLimit)
			    continue;
			ptTrn = PlDrcGTrn(ptPlDrc);
			ptV = TrnGVal(ptTrn);
			if (ptV == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 4);
                                return;
L4: ;
				if ((PtSzBest() == 1L) ||
				    (PtLazy() && (PtBest() != 0L)))
				    return;
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
				ptLimit = PlcGCount(ptPlc)
				    - TrArcGWeight(ptTrArc);
                                LstSet(PlcGMosIn(ptPlc), TrnGCursor(trn));
				ptPlDrc = (PLDRC *)
				    LstThis(PlcGMosIn(ptPlc));
				ptTrn = PlDrcGTrn(ptPlDrc);
			    }
                        if (ptV < TrnGMin(trn))
                            TrnSMin(trn, ptV);
			if (TrnSatisfied(ptTrn))
			    TrnFeed(trn);
		    }
	}
}
