/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  ptvisit3.c
*
* Part of: PROD's programs araprod and "gphgen"
*
* Author:  Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     See ptvisit.c.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ptrbl.h"
#include "baltree.h"
#include "bdd.h"
#include "gphutil.h"
#include "graph.h"
#include "mark.h"
#include "encode.h"
#include "ptnet.h"

/***************************************************************************
*
* Function:  TrnDvMsIn
* Purpose:   This is an auxiliary routine of TrnSpan.
* Arguments: trn - a transition
* Returns:
* See also:  TrnSpan
*
****************************************************************************
*/
FE1(BOOL, TrnDvMsIn,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        }
    LSTFOR (TrnGMosIn(trn), ptTrDrc, TRDRC)
	{
	    ptPlc = TrDrcGPlc(ptTrDrc);
	    if (PlcGCount(ptPlc) >= TrDrcGIn(ptTrDrc))
		continue;
	    if (ptV-- != 0L)
		continue;
	    if (PlcGPurIn(ptPlc))
		LSTFOR (PlcGPurIn(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGPurIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return(TRUE);
L1: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosIn(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
                                LstSet(PlcGPurIn(ptPlc), TrnGCursor(trn));
			    }
		    }
	    ptLimit = PlcGCount(ptPlc);
	    if (PlcGMosIn(ptPlc))
		LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
		    {
			if (PlDrcGOut(ptPlDrc) > ptLimit)
			    continue;
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return(TRUE);
L2: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosIn(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
				ptLimit = PlcGCount(ptPlc);
                                LstSet(PlcGMosIn(ptPlc), TrnGCursor(trn));
			    }
		    }
	    return(TRUE);
	}
    return(FALSE);
}

/***************************************************************************
*
* Function:  TrnDvMsOut
* Purpose:   This is an auxiliary routine of TrnSpan.
* Arguments: trn - a transition
* Returns:
* See also:  TrnSpan
*
****************************************************************************
*/
FE1(BOOL, TrnDvMsOut,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        }
    LSTFOR (TrnGMosOut(trn), ptTrDrc, TRDRC)
	{
	    ptPlc = TrDrcGPlc(ptTrDrc);
	    if (PlcGCount(ptPlc) >= TrDrcGIn(ptTrDrc))
		continue;
	    if (ptV-- != 0L)
		continue;
	    if (PlcGPurIn(ptPlc))
		LSTFOR (PlcGPurIn(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGPurIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return(TRUE);
L1: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosOut(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
                                LstSet(PlcGPurIn(ptPlc), TrnGCursor(trn));
			    }
		    }
	    ptLimit = PlcGCount(ptPlc);
	    if (PlcGMosIn(ptPlc))
		LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
		    {
			if (PlDrcGOut(ptPlDrc) > ptLimit)
			    continue;
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return(TRUE);
L2: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosOut(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
				ptLimit = PlcGCount(ptPlc);
                                LstSet(PlcGMosIn(ptPlc), TrnGCursor(trn));
			    }
		    }
	    return(TRUE);
	}
    return(FALSE);
}

/***************************************************************************
*
* Function:  TrnEvNeutral
* Purpose:   This is an auxiliary routine of TrnSpan.
* Arguments: trn - a transition
* Returns:
* See also:  TrnSpan
*
****************************************************************************
*/
FE1(void, TrnEvNeutral,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        }
    LSTFOR (TrnGNeutral(trn), ptTrArc, TRARC)
	{
	    ptPlc = TrArcGPlc(ptTrArc);
	    if (PlcGPurOut(ptPlc))
		LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGPurOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return;
L1: ;
				ptTrArc = (TRARC *)
				    LstThis(TrnGNeutral(trn));
				ptPlc = TrArcGPlc(ptTrArc);
                                LstSet(PlcGPurOut(ptPlc), TrnGCursor(trn));
			    }
		    }
	    if (PlcGMosOut(ptPlc))
		LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
		    {
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return;
L2: ;
				ptTrArc = (TRARC *)
				    LstThis(TrnGNeutral(trn));
				ptPlc = TrArcGPlc(ptTrArc);
                                LstSet(PlcGMosOut(ptPlc), TrnGCursor(trn));
			    }
		    }
	}
}

/***************************************************************************
*
* Function:  TrnEvMsOut
* Purpose:   This is an auxiliary routine of TrnSpan.
* Arguments: trn - a transition
* Returns:
* See also:  TrnSpan
*
****************************************************************************
*/
FE1(void, TrnEvMsOut,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        }
    LSTFOR (TrnGMosOut(trn), ptTrDrc, TRDRC)
	{
	    ptPlc = TrDrcGPlc(ptTrDrc);
	    if (PlcGPurOut(ptPlc))
		LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGPurOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return;
L1: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosOut(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
                                LstSet(PlcGPurOut(ptPlc), TrnGCursor(trn));
			    }
		    }
	    if (PlcGMosOut(ptPlc))
		LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
		    {
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return;
L2: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosOut(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
                                LstSet(PlcGMosOut(ptPlc), TrnGCursor(trn));
			    }
		    }
	}
}

/***************************************************************************
*
* Function:  TrnEvPrIn
* Purpose:   This is an auxiliary routine of TrnSpan.
* Arguments: trn - a transition
* Returns:
* See also:  TrnSpan
*
****************************************************************************
*/
FE1(void, TrnEvPrIn,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        case 3:
            TrnSWhere(trn, (unsigned) 0);
            goto L3;
        case 4:
            TrnSWhere(trn, (unsigned) 0);
            goto L4;
        }
    LSTFOR (TrnGPurIn(trn), ptTrArc, TRARC)
	{
	    ptPlc = TrArcGPlc(ptTrArc);
	    if (PlcGPurOut(ptPlc))
		LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGPurOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return;
L1: ;
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
                                LstSet(PlcGPurOut(ptPlc), TrnGCursor(trn));
			    }
		    }
	    if (PlcGMosOut(ptPlc))
		LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
		    {
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return;
L2: ;
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
                                LstSet(PlcGMosOut(ptPlc), TrnGCursor(trn));
			    }
		    }
	    ptLimit = PlcGCount(ptPlc) - TrArcGWeight(ptTrArc);
	    if (PlcGNeutral(ptPlc))
		LSTFOR (PlcGNeutral(ptPlc), ptPlArc, PLARC)
		    {
			if (PlArcGWeight(ptPlArc) <= ptLimit)
			    continue;
			ptTrn = PlArcGTrn(ptPlArc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGNeutral(ptPlc)));
                                TrnSWhere(trn, (unsigned) 3);
                                return;
L3: ;
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
				ptLimit = PlcGCount(ptPlc)
				    - TrArcGWeight(ptTrArc);
                                LstSet(PlcGNeutral(ptPlc), TrnGCursor(trn));
			    }
		    }
	    if (PlcGMosIn(ptPlc))
		LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
		    {
			if (PlDrcGOut(ptPlDrc) <= ptLimit)
			    continue;
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 4);
                                return;
L4: ;
				ptTrArc = (TRARC *)
				    LstThis(TrnGPurIn(trn));
				ptPlc = TrArcGPlc(ptTrArc);
				ptLimit = PlcGCount(ptPlc)
				    - TrArcGWeight(ptTrArc);
                                LstSet(PlcGMosIn(ptPlc), TrnGCursor(trn));
			    }
		    }
	}
}

/***************************************************************************
*
* Function:  TrnEvMsIn
* Purpose:   This is an auxiliary routine of TrnSpan.
* Arguments: trn - a transition
* Returns:
* See also:  TrnSpan
*
****************************************************************************
*/
FE1(void, TrnEvMsIn,
    TRN *, trn)
{
    switch (TrnGWhere(trn))
        {
        case 1:
            TrnSWhere(trn, (unsigned) 0);
            goto L1;
        case 2:
            TrnSWhere(trn, (unsigned) 0);
            goto L2;
        case 3:
            TrnSWhere(trn, (unsigned) 0);
            goto L3;
        case 4:
            TrnSWhere(trn, (unsigned) 0);
            goto L4;
        }
    LSTFOR (TrnGMosIn(trn), ptTrDrc, TRDRC)
	{
	    ptPlc = TrDrcGPlc(ptTrDrc);
	    if (PlcGPurOut(ptPlc))
		LSTFOR (PlcGPurOut(ptPlc), ptPlArc, PLARC)
		    {
			ptTrn = PlArcGTrn(ptPlArc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGPurOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 1);
                                return;
L1: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosIn(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
                                LstSet(PlcGPurOut(ptPlc), TrnGCursor(trn));
			    }
		    }
	    if (PlcGMosOut(ptPlc))
		LSTFOR (PlcGMosOut(ptPlc), ptPlDrc, PLDRC)
		    {
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosOut(ptPlc)));
                                TrnSWhere(trn, (unsigned) 2);
                                return;
L2: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosIn(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
                                LstSet(PlcGMosOut(ptPlc), TrnGCursor(trn));
			    }
		    }
	    ptLimit = PlcGCount(ptPlc)
		- TrDrcGIn(ptTrDrc) + TrDrcGOut(ptTrDrc);
	    if (PlcGNeutral(ptPlc))
		LSTFOR (PlcGNeutral(ptPlc), ptPlArc, PLARC)
		    {
			if (PlArcGWeight(ptPlArc) <= ptLimit)
			    continue;
			ptTrn = PlArcGTrn(ptPlArc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGNeutral(ptPlc)));
                                TrnSWhere(trn, (unsigned) 3);
                                return;
L3: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosIn(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
				ptLimit = PlcGCount(ptPlc)
				    - TrDrcGIn(ptTrDrc)
					+ TrDrcGOut(ptTrDrc);
                                LstSet(PlcGNeutral(ptPlc), TrnGCursor(trn));
			    }
		    }
	    if (PlcGMosIn(ptPlc))
		LSTFOR (PlcGMosIn(ptPlc), ptPlDrc, PLDRC)
		    {
			if (PlDrcGOut(ptPlDrc) <= ptLimit)
			    continue;
			ptTrn = PlDrcGTrn(ptPlDrc);
			if (TrnGVal(ptTrn) == 0L)
			    {
                                TrnSCursor(trn, LstGet(PlcGMosIn(ptPlc)));
                                TrnSWhere(trn, (unsigned) 4);
                                return;
L4: ;
				ptTrDrc = (TRDRC *)
				    LstThis(TrnGMosIn(trn));
				ptPlc = TrDrcGPlc(ptTrDrc);
				ptLimit = PlcGCount(ptPlc)
				    - TrDrcGIn(ptTrDrc)
					+ TrDrcGOut(ptTrDrc);
                                LstSet(PlcGMosIn(ptPlc), TrnGCursor(trn));
			    }
		    }
	}
}

/***************************************************************************
*
* Function:  TrnNaps
* Purpose:   This is an auxiliary routine of TrnSpan.
* Arguments: trn - a transition
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, TrnNaps,
    TRN *, trn)
{
    switch (TrnGWhere(trn) >> 3)
        {
        case 1:
                TrnSWhere(trn, TrnGWhere(trn) & 7);
                goto L1;
        case 2:
                TrnSWhere(trn, TrnGWhere(trn) & 7);
                goto L2;
        case 3:
                TrnSWhere(trn, TrnGWhere(trn) & 7);
                goto L3;
        case 4:
                TrnSWhere(trn, TrnGWhere(trn) & 7);
                goto L4;
        case 5:
                TrnSWhere(trn, TrnGWhere(trn) & 7);
                goto L5;
        case 6:
                TrnSWhere(trn, TrnGWhere(trn) & 7);
                goto L6;
        case 7:
                TrnSWhere(trn, TrnGWhere(trn) & 7);
                goto L7;
        case 8:
                TrnSWhere(trn, TrnGWhere(trn) & 7);
                goto L8;
        }
    TrnSVal(trn, PtrBlGCount(TblTrn(0)) + 1L);
    if (!TrnEnabled(trn))
	{
	    ptV = 0L;
	    if (PlRnd())
		{
		    if (TrnGPurIn(trn))
			LSTFOR (TrnGPurIn(trn), ptTrArc, TRARC)
			    if (PlcGCount(TrArcGPlc(ptTrArc)) <
				TrArcGWeight(ptTrArc))
				ptV++;
		    if (TrnGNeutral(trn))
			LSTFOR (TrnGNeutral(trn), ptTrArc, TRARC)
			    if (PlcGCount(TrArcGPlc(ptTrArc)) <
				TrArcGWeight(ptTrArc))
				ptV++;
		    if (TrnGMosIn(trn))
			LSTFOR (TrnGMosIn(trn), ptTrDrc, TRDRC)
			    if (PlcGCount(TrDrcGPlc(ptTrDrc)) <
				TrDrcGIn(ptTrDrc))
				ptV++;
		    if (TrnGMosOut(trn))
			LSTFOR (TrnGMosOut(trn), ptTrDrc, TRDRC)
			    if (PlcGCount(TrDrcGPlc(ptTrDrc)) <
				TrDrcGIn(ptTrDrc))
				ptV++;
		    ptV = (unsigned long) PtRandom((UNS32) ptV);
		}
	    if (TrnGPurIn(trn) && !LstEmpty(TrnGPurIn(trn)))
		{
L1: ;
		    if (TrnDvPrIn(trn))
                        {
                            if (TrnGWhere(trn))
                                TrnSWhere(trn, TrnGWhere(trn) | (1 << 3));
                            return;
                        }
		}
	    if (TrnGNeutral(trn) && !LstEmpty(TrnGNeutral(trn)))
		{
L2: ;
		    if (TrnDvNeutral(trn))
                        {
                            if (TrnGWhere(trn))
                                TrnSWhere(trn, TrnGWhere(trn) | (2 << 3));
                            return;
                        }
		}
	    if (TrnGMosIn(trn) && !LstEmpty(TrnGMosIn(trn)))
		{
L3: ;
		    if (TrnDvMsIn(trn))
                        {
                            if (TrnGWhere(trn))
                                TrnSWhere(trn, TrnGWhere(trn) | (3 << 3));
                            return;
                        }
		}
	    if (TrnGMosOut(trn) && !LstEmpty(TrnGMosOut(trn)))
		{
L4: ;
		    if (TrnDvMsOut(trn))
                        {
                            if (TrnGWhere(trn))
                                TrnSWhere(trn, TrnGWhere(trn) | (4 << 3));
                            return;
                        }
		}
	}
    else
	{
	    if (!TrnVisible(trn))
		PtSKingTrn(trn);
	    if (TrnGNeutral(trn) && !LstEmpty(TrnGNeutral(trn)))
                {
L5: ;
                    TrnEvNeutral(trn);
                    if (TrnGWhere(trn))
                        {
                            TrnSWhere(trn, TrnGWhere(trn) | (5 << 3));
                            return;
                        }
                }
	    if (TrnGMosOut(trn) && !LstEmpty(TrnGMosOut(trn)))
                {
L6: ;
                    TrnEvMsOut(trn);
                    if (TrnGWhere(trn))
                        {
                            TrnSWhere(trn, TrnGWhere(trn) | (6 << 3));
                            return;
                        }
                }
	    if (TrnGPurIn(trn) && !LstEmpty(TrnGPurIn(trn)))
                {
L7: ;
                    TrnEvPrIn(trn);
                    if (TrnGWhere(trn))
                        {
                            TrnSWhere(trn, TrnGWhere(trn) | (7 << 3));
                            return;
                        }
                }
	    if (TrnGMosIn(trn) && !LstEmpty(TrnGMosIn(trn)))
                {
L8: ;
                    TrnEvMsIn(trn);
                    if (TrnGWhere(trn))
                        {
                            TrnSWhere(trn, TrnGWhere(trn) | (8 << 3));
                            return;
                        }
                }
	}
}

/***************************************************************************
*
* Function:  TrnSpan
* Purpose:   This routine marks all unvisited transitions that a reachable
*            from trn in a transition dependency graph of the incremental
*            algorithm of the stubbborn set method.
* Arguments: trn - as explained above
* Returns:
* See also:
*
****************************************************************************
*/
FE1(void, TrnSpan,
    TRN *, trn)
{
    while (TRUE)
        {
            TrnNaps(trn);
            if (TrnGWhere(trn))
                {
                    TrnSDown(ptTrn, trn);
                    trn = ptTrn;
                }
            else if (TrnGDown(trn))
                {
                    ptTrn = TrnGDown(trn);
                    TrnCDown(trn);
                    trn = ptTrn;
                }
            else
                return;
        }
}

/***************************************************************************
*
* Function:  TrnVisStub
* Purpose:   This routine is needed for constructing a stubborn set
*            that contains all visible transitions. Such set is sometimes
*            needed to guarantee CFFD-preservation.
* Arguments: trn - a transition
* Returns:
* See also:  TrnStub
*
****************************************************************************
*/
FE1(BOOL, TrnVisStub,
    TRN *, trn)
{
    if ((TrnGVal(trn) == 0L) && TrnVisible(trn))
	TrnSpan(trn);
    return(FALSE);
}
