/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  stackpro.h
*
* Part of: PROD's program probe
*
* Authors: Keijo Heljanko
*
* Current version:
*     $Id$
*
* Description:
*     See stackpro.c.
*
* Version history:
*     $Log$
***************************************************************************/

typedef unsigned long STACK_ITEM;

#define STACK_BLOCK_SIZE(stack) 8192

typedef struct _stack_block
{
    STACK_ITEM *block;               /* Pointer to a block of stack items. */
    struct _stack_block *prev;       /* Previous stack block, if NULL, the
				        block isn't in memory, but on disk,
				        or next block doesn't exist. */
} STACK_BLOCK;


typedef struct _stack
{
    unsigned long curr_numitems;     /* Number of items used in the current
					block. */
    STACK_ITEM *curr_item;           /* Pointer to the current location on the
					top of the stack. */

    STACK_BLOCK *blocks;             /* Linked list of blocks in memory. */
    STACK_BLOCK *last;               /* End of the list for fast access. */
  
    unsigned long numblocks;         /* Total number of blocks in stack. */
    unsigned long blocks_in_memory;  /* Use this number of blocks in memory,
					the rest will reside on disk. */
    char *filename;                  /* Name of the disk paging file,
				        currently not used. */

} STACK;


/* Some macros to inline parts of the stack handling.
   More difficult parts are handled by the RealxxxxStack
   functions. */

#define PushStack(stack, item) \
\
    if((stack)->curr_numitems < STACK_BLOCK_SIZE(stack)) { \
        (stack)->curr_numitems++; \
	*((stack)->curr_item) = (item); \
	(stack)->curr_item++; \
    } else { \
        RealPushStack((stack), (STACK_ITEM) (item)); \
    }


#define PopStack(stack) \
    ((stack)->curr_numitems > 0 ? \
\
     (stack)->curr_numitems--, (stack)->curr_item--, *(stack)->curr_item : \
\
     RealPopStack(stack))
   

#ifdef DEBUG
#define PeekStack(stack) \
\
    ((stack)->curr_numitems > 0 ? \
\
     (*(((stack)->curr_item)-1)): \
\
     RealPeekStack(stack))
#endif

#define EmptyStack(stack) \
\
    (((stack)->curr_numitems > 0 || (stack)->numblocks > 1) ? \
\
     (BOOL)(FALSE) : \
\
     (BOOL)(TRUE))

DE0(STACK *, CreateStack)
DE1(void, AllocBlock,
    STACK *, stack)
DE1(void, DeleteStack,
    STACK *, stack)
DE1(void, ClearStack,
    STACK *, stack)
#ifdef DEBUG
DE2(void, DumpStack,
    STACK *, stack,
    unsigned, depth)
#endif
DE2(void, RealPushStack,
    STACK *, stack,
    STACK_ITEM, item)
DE1(STACK_ITEM, RealPopStack,
    STACK *, stack)
#ifdef DEBUG
DE1(STACK_ITEM, RealPeekStack,
    STACK *, stack)
DE1(unsigned long, StackSize,
    STACK *, stack)
#endif
