/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  strong.c
*
* Part of: PROD's program strong
*
* Authors:  Lasse Anderson, Robert Jurgens and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     The "strong" program computes the strongly connected components
*     of a reachability graph of a net.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include <string.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "error.h"
#include "dynam.h"
#include "ptrbl.h"
#include "baltree.h"
#include "bdd.h"
#include "gphutil.h"
#include "graph.h"
#include "sgsig.h"
#include "cmdnm.h"

/***************************************************************************
*
* Function:  Misusage
* Purpose:   This routine handles most of the cases where the command line
*            of the program is incorrect.
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FS0(void, Misusage)
{
    MsgExit("[-G] [-t]name", (int) 1);
}

/***************************************************************************
*
* Function:  main
* Purpose:   See the "Description:" in the beginning of this file.
* Arguments: argc - the number of command line arguments given to the
*                   program
*            argv - an array containing the arguments
* Returns:   0 if everything succeeds, and something else otherwise
*            (PROD has no systematic convention for non-zero exits.)
* See also:
*
****************************************************************************
*/
MAIN2()
{
    int argc = 0;
    char **argv = NULL;
    GRAPH *graph;
    char *argument, *fileName;
    int i;
    BOOL mapToMem;

#ifndef UNIX
    ClockOn();
#endif
    SetIO();
    ClearErrno();
    baseName = StrCopy("strong");
#ifdef ARGV0
    myName = CmdNm(_argv_[0]);
#else
    myName = baseName;
#endif
#ifndef UNIX
    StoreCmd(_argc_, myName, (char **) _argv_);
#endif
    ExpArgs(_argc_, (char **) _argv_, &argc, &argv);
    IgnoreSignals();
    DynamInit();
    fileName = NULL;
    mapToMem = FALSE;
    for (i = 1; i < argc; i++)
	{
	    argument = argv[i];
	    if (*argument == '-')
		{
		    argument++;
		    if ((*argument == '\0') &&
			(i + 1 == argc))
			Misusage();
		    if (*argument == '\0')
			{
			    argument = argv[i + 1];
			    i++;
			}
		    while (*argument != '\0')
			{
			    if (*argument == 'G')
				mapToMem = TRUE;
			    else if (*argument == 't')
				{
				    argument++;
				    if ((fileName != NULL) ||
					((*argument == '\0') && 
					 (i + 1 == argc)))
					Misusage();
				    if (*argument == '\0')
					{
					    argument = argv[i + 1];
					    i++;
					}
				    fileName = argument;
				    break;
				}
			    else
				Misusage();
			    argument++;
			}
		}
	    else if (fileName == NULL)
		fileName = argument;
	    else
		Misusage();
	}
    if (fileName == NULL)
	Misusage();
    i = (int) strlen(fileName);
    if ((i > 4) && (strcmp(fileName + i - 4, ".gph") == 0))
	fileName[i - 4] = '\0';
    graph = GphOpen(fileName, GPH_COMPS, FALSE);
    if (graph == NULL)
	Exit(1);
    if (mapToMem)
	GphMapToMem(graph, (unsigned long) 0L);
    GphSgSearch(graph);
    GphClose(graph);
    Exit(0);
    return(0);
}
