/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  trans.c
*
* Part of: PROD's program prpp
*
* Authors: Johannes Helander and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     This module handles transition declarations.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "dynam.h"
#include "ppio.h"
#include "token.h"
#include "place.h"
#include "prpp.h"
#include "lexer.h"
#include "macro.h"
#include "output.h"
#include "parse.h"
#include "util.h"
#include "ptrbl.h"
#include "baltree.h"
#include "gphutil.h"
#include "mark.h"
#include "encode.h"
#include "expr.h"
#include "net.h"
#include "pplst.h"
#include "cmdnm.h"
#include "prpexpr.h"
#include "trans.h"

/***************************************************************************
 *
 * Function:  SymVars
 * Purpose:
 * Arguments: symbol - 
 * Returns:
 * See also:
 *
 ****************************************************************************
 */
FE1(BOOL, SymVars,
    SYMBOL *, symbol)
{
    SYMBOL *symbol2;
    VARIABLE *var;

    if (SymIsVariable(symbol))
	{
	    var = SymGVariable(symbol);
	    if ((!VarUnified(var)) && (!VarHard(var)) && (!VarBad(var)))
		{
		    VarSBad(var);
		    (void) sprintf(errStr,
				   "Input variable %s is not unifiable",
				   VarGName(var));
		    error(errStr);
		    return(FALSE);
		}
	}
    else if (!SymIsConstant(symbol))
        LSTFOR (symbol->_uy._operation, symbol2, SYMBOL)
	    if (!SymVars(symbol2))
		return(FALSE);
    return(TRUE);
}

/***************************************************************************
*
* Function:  parse_trans_line
* Purpose:
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FE0(TRANSITION *, parse_trans_line)
{
    PP_TOKEN *token;
    TRANSITION *tran = NULL;
    BOOL fdRead = FALSE;
    VARIABLE *var;

    token = gTSkSpace();
    TOKEN_MATCH(token, NAME, "name after \"#trans\"");
    LSTFOR (TransLst, tran, TRANSITION)
	if (!strcmp(token->token, TraGName(tran)))
	    break;
    if (tran)
	{
	    (void) sprintf(errStr, "Transition %s redefined", token->token);
	    error(errStr);
	    goto recover;
	}
    if (verify && ((!strcmp(token->token, "i_n_i_t_i_a_l_i_z_e")) ||
		   !strcmp(token->token, "d_u_m_m_y")))
	{
	    (void) sprintf(errStr,
			   "Transition %s is possible\nonly when there is no #verify",
			   token->token);
	    error(errStr);
	    goto recover;
	}
    tran = TraCons(StrCopy(token->token));
    for (;;)
	{
	    PpTokDiscard(token);
	    token = gTSkSpace();
	    TOKEN_CHECK(token);
	    if (strcmp(token->token, "fd"))
		TOKEN_MATCH(token, NEVERMATCH,
			    ((void) sprintf(errStr, "fd in transition %s",
					    TraGName(tran)),
			     errStr));
	    if (fdRead)
		{
		    (void) sprintf(errStr,
				   "Transition %s: fd allowed only once",
				   TraGName(tran));
		    error(errStr);
		    goto recover;
		}
	    PpTokDiscard(token);
	    fdRead = TRUE;
	    token = gTSkSpace();
	    TOKEN_MATCH(token, OPENING_PARENTHESIS,
			((void) sprintf(errStr,
					"\"(\" after \"fd\" in transition %s",
					TraGName(tran)), errStr));
            for (;;)
		{
		    PpTokDiscard(token);
		    token = gTSkSpace();
		    TOKEN_MATCH(token, NAME,
				((void) sprintf(errStr,
						"variable name in fd list of transition %s",
						TraGName(tran)),
				 errStr));
		    if (TrVarLookup(token->token, tran))
			{
			    (void) sprintf(errStr,
					   "transition %s: duplicate fd variable %s",
					   TraGName(tran), token->token);
			    error(errStr);
			}
		    else
			{
			    var = VarCons(StrCopy(token->token));
			    VarSIsFold(var);
			    (void) LstInsLast(TraGVariables(tran),
					      (char *) var);
			    TraIFdWidth(tran);
			}
		    PpTokDiscard(token);
		    token = gTSkSpace();
		    TOKEN_CHECK(token);
		    switch(token->type)
			{
			case COMMA:
			    continue;
			case CLOSING_PARENTHESIS:
			    break;
			default:
			    TOKEN_MATCH(token, NEVERMATCH,
					((void) sprintf(errStr,
							"\",\" or \")\" in fd list of transition %s",
							TraGName(tran)),
					 errStr));
			}
		    break;
		}
	}
  recover: ;
    if (token)
	PpTokDiscard(token);
    if (tran)
	TraDiscard(tran);
    return((TRANSITION *) NULL);
  completed: ;
    if (token)
	PpTokDiscard(token);
    return(tran);
}

/***************************************************************************
*
* Function:  trans_let
* Purpose:
* Arguments: tran - 
* Returns:
* See also:
*
****************************************************************************
*/
FE1(BOOL, trans_let,
    TRANSITION *, tran)
{
    LIST *letVars;
    LIST *list;
    PP_TOKEN *dToken;
    PP_TOKEN *token;
    VARIABLE *var;
    MARKING *marking;
    MULTUP *multup;
    DOMAIN *arr;
    unsigned i;
    DATATYPE *buffer;
    register unsigned len;
    register UDOMAIN value;
    register DATATYPE *pointer;

    marking = NULL;
    letVars = NULL;
    list = LstCreate(&ppTokTools);
    (void) LstInsFirst(ppLsLs, (char *) list);
    dToken = app_parse(LESS_THAN_OR_EQUAL, FALSE);
    (void) LstDelFirst(ppLsLs, FALSE);
    TOKEN_MATCH(dToken, LESS_THAN_OR_EQUAL,
                ((void) sprintf(errStr,
                                "\"<=\" in \"let\" of transition %s",
                                TraGName(tran)),
                 errStr));
    PpTokDiscard(dToken);
    token = (PP_TOKEN *) LstBegin(list);
    while (token && ((token->type == SPACE) || (token->type == NEWLINE) ||
                     (token->type == COMMENT)))
        token = (PP_TOKEN *) LstStep(list);
    TOKEN_MATCH(token, TUPLE_LEFT,
                ((void) sprintf(errStr,
                                "\"<.\" after \"let\" in transition %s",
                                TraGName(tran)),
                 errStr));
    letVars = LstCreate(&varTools);
    while (TRUE)
        {
            token = (PP_TOKEN *) LstStep(list);
            while (token && ((token->type == SPACE) ||
                             (token->type == NEWLINE) ||
                             (token->type == COMMENT)))
                token = (PP_TOKEN *) LstStep(list);
            TOKEN_MATCH(token, NAME,
                        ((void) sprintf(errStr,
                                        "variable name in \"let\" of transition %s",
                                        TraGName(tran)),
                        errStr));
            if ((var = TrVarLookup(token->token, tran)) && VarIsLet(var))
                {
                    (void) sprintf(errStr,
                                   "transition %s: duplicate \"let\" variable %s",
                                   TraGName(tran), token->token);
                    error(errStr);
                    goto recover;
                }
            else
                {
                    if (!var)
                        {
                            var = VarCons(StrCopy(token->token));
                            (void) LstInsLast(TraGVariables(tran),
                                              (char *) var);
			}
                    VarSIsLet(var);
                    VarSLow(var, (DOMAIN) MAXDOMAIN);
                    VarSHigh(var, (DOMAIN) MINDOMAIN);
                    (void) LstInsLast(letVars, (char *) var);
                }
            token = (PP_TOKEN *) LstStep(list);
            while (token && ((token->type == SPACE) ||
                             (token->type == NEWLINE) ||
                             (token->type == COMMENT)))
                token = (PP_TOKEN *) LstStep(list);
            if (token && (token->type == TUPLE_RIGHT))
                break;
            TOKEN_MATCH(token, COMMA,
                        ((void) sprintf(errStr,
                                        "\",\" or \".>\" after \"%s\" in \"let\" of transition %s",
                                        VarGName(var), TraGName(tran)),
                        errStr));
        }
    token = (PP_TOKEN *) LstStep(list);
    while (token && ((token->type == SPACE) || (token->type == NEWLINE) ||
                     (token->type == COMMENT)))
        token = (PP_TOKEN *) LstStep(list);
    if (token)
        TOKEN_MATCH(token, NEVERMATCH,
                    ((void) sprintf(errStr,
                                    "\"<=\" after the first \".>\" in \"let\" of transition %s",
                                    TraGName(tran)),
                     errStr));
    LstDestroy(list, TRUE);
    list = LstCreate(&ppTokTools);
    (void) LstInsFirst(ppLsLs, (char *) list);
    dToken = app_parse(SEMICOLON, FALSE);
    (void) LstDelFirst(ppLsLs, FALSE);
    TOKEN_MATCH(dToken, SEMICOLON,
                ((void) sprintf(errStr,
                                "\";\" in \"let\" of transition %s",
                                TraGName(tran)),
                 errStr));
    PpTokDiscard(dToken);
    LSTFOR (list, token, PP_TOKEN)
        if ((token->type == NAME) && strcmp(token->token, "card") &&
	    strcmp(token->token, "empty") && strcmp(token->token, "field"))
            {
                (void) sprintf(errStr,
                               "Inappropriate \"%s\" after the first \"<=\"\nin \"let\" of transition %s",
                               token->token, TraGName(tran));
                error(errStr);
                goto recover;
            }
    PrpExpr(&marking, (char *) NULL, (unsigned) 0,
            list, (DOMAIN *) NULL);
    list = NULL;
    if (errorFlag != NO_ERROR)
        goto recover;
    if (MarEmpty(marking))
        {
            (void) sprintf(errStr,
                           "empty marking after the first \"<=\" in \"let\" of transition %s",
                           TraGName(tran));
            error(errStr);
            goto recover;
        }
    i = LstSize(letVars);
    MARFOR (marking, multup)
        if (MultupGArity(multup) != i)
            {
                (void) sprintf(errStr,
                               "\"let\" of transition %s: the tuple before the first \"<=\" is of arity %u\nwhereas some later tuple is of arity %u",
                               TraGName(tran), i, MultupGArity(multup));
                error(errStr);
                goto recover;
            }
    MARFOR (marking, multup)
        {
            arr = (DOMAIN *) AllocArray(MultupGArity(multup),
                                        sizeof(DOMAIN));
            i = 0;
            LSTFOR (TraGVariables(tran), var, VARIABLE)
                {
                    if (!VarIsLet(var))
                        continue;
                    arr[i] =
                        MultupGField(multup,
                                     LstWhich(letVars, (char *) var) - 1);
                    if (VarGLow(var) > arr[i])
                        VarSLow(var, arr[i]);
                    if (VarGHigh(var) < arr[i])
                        VarSHigh(var, arr[i]);
                    i++;
                }
            len = 0;
            for (i = 0; i < MultupGArity(multup); i++)
                {
                    value = arr[i];
                    value = Encode((DOMAIN) value);
                    IncLen(value, len);
                }
            buffer = (DATATYPE *) AllocArray(len + 1, sizeof(DATATYPE));
            pointer = buffer;
            for (i = 0; i < MultupGArity(multup); i++)
                {
                    value = arr[i];
                    value = Encode((DOMAIN) value);
                    PutValue(value, pointer);
                }
            *pointer = (DATATYPE) 0;
            Deallocate((char *) arr);
            (void) LstInsLast(TraGRange(tran), (char *) buffer);
        }
    MarDiscard(marking);
    LstDestroy(letVars, FALSE);
    return(TRUE);
recover: ;
    if (dToken)
	PpTokDiscard(dToken);
    error_flag = 1;
    if (marking)
        MarDiscard(marking);
    if (list)
        LstDestroy(list, TRUE);
    if (letVars)
        LstDestroy(letVars, FALSE);
    return(FALSE);
}

/***************************************************************************
*
* Function:  trans_expression
* Purpose:
* Arguments: tran - 
*            isin - 
*            place - 
* Returns:
* See also:
*
****************************************************************************
*/
FE3(SYMMULTUP *, trans_expression,
    TRANSITION *, tran,
    BOOL, isin,
    PLACE *, place)
{
    PP_TOKEN *token;
    PP_TOKEN *nekot;
    PP_TOKEN *dToken;
    LIST *list;
    SYMMULTUP *symmultup;
    unsigned count;
    LIST *symlist;
    LIST *spclist;
    VARIABLE *var;
    SYMBOL *symbol;
    SYMBOL *varsym;
    char *str;
    LIST *aux;
    LIST *stack;
    VOID *cursor;

    list = LstCreate(&ppTokTools);
    (void) LstInsFirst(ppLsLs, (char *) list);
    dToken = app_parse(TUPLE_RIGHT, FALSE);
    (void) LstDelFirst(ppLsLs, FALSE);
    TOKEN_MATCH(dToken, TUPLE_RIGHT,
		((void) sprintf(errStr,
				"\".>\" in \"%s block\" of transition %s",
				isin ? "in" : "out", TraGName(tran)),
		 errStr));
    if (!place)
	goto recover;
    PpTokDiscard(dToken);
    aux = LstCreate(&symTools);
    symlist = LstCreate(&symTools);
    spclist = LstCreate(&strTools);
    stack = LstCreate(&strTools);
    count = 0;
    LSTFOR (list, nekot, PP_TOKEN)
	{
	    switch(nekot->type)
		{
		case NAME:
		    if (count == 0)
			count = 1;
		    cursor = LstGet(list);
		    token = (PP_TOKEN *) LstStep(list);
		    while(token && ((token->type == SPACE) ||
				    (token->type == NEWLINE) ||
				    (token->type == COMMENT)))
			token = (PP_TOKEN *) LstStep(list);
		    LstSet(list, cursor);
		    if (token && ((token->type == OPENING_PARENTHESIS) ||
				  (token->type == OPENING_BRACKET) ||
				  (token->type == OPENING_SQUARE_BRACKET) ||
				  (token->type == TUPLE_LEFT)))
			{
			    (void) LstInsLast(symlist, (char *)
					     SymCons(SK_CONSTANT,
						     StrCopy(nekot->token)));
			    (void) LstInsLast(spclist, (char *) "N");
			    break;
			}
		    else
			{
			    var = TrVarLookup(nekot->token, tran);
			    if (!var)
				{
				    var = VarCons(StrCopy(nekot->token));
				    if (isin)
					VarSIsIn(var);
				    else
					VarSIsOut(var);
				    (void) LstInsLast(TraGVariables(tran),
						      (char *) var);
				}
			    else if (isin)
				VarSIsIn(var);
			    else if (!VarIsIn(var))
				VarSIsOut(var);
			    (void) LstInsLast(symlist, (char *)
					     SymCons(SK_VARIABLE,
						     (char *) var));
			    (void) LstInsLast(spclist, (char *) "N");
			}
		    break;
		case OPENING_PARENTHESIS:
		case OPENING_BRACKET:
		case OPENING_SQUARE_BRACKET:
		case TUPLE_LEFT:
		    if (count == 0)
			count = 1;
		    (void) LstInsFirst(stack, (char *) 
				       (closing_bracket_pair + nekot->type));
		    (void) LstInsLast(symlist, (char *)
				      SymCons(SK_CONSTANT,
					      StrCopy(nekot->token)));
		    (void) LstInsLast(spclist, (char *) "N");
		    break;
		case CLOSING_PARENTHESIS:
		case CLOSING_BRACKET:
		case CLOSING_SQUARE_BRACKET:
		case TUPLE_RIGHT:
		    if (count == 0)
			count = 1;
		    if ((!LstEmpty(stack)) &&
			(*((PP_TOKEN_TYPE *) LstFirst(stack)) == nekot->type))
			(void) LstDelFirst(stack, FALSE);
		    (void) LstInsLast(symlist, (char *)
				      SymCons(SK_CONSTANT,
					      StrCopy(nekot->token)));
		    (void) LstInsLast(spclist, (char *) "N");
		    break;
		case COMMA:
		    if (count == 0)
			count = 1;
		    if (!LstEmpty(stack))
			{
			    (void) LstInsLast(symlist, (char *)
					      SymCons(SK_CONSTANT,
						      StrCopy(nekot->token)));
			    (void) LstInsLast(spclist, (char *) "N");
			    break;
			}
		    LSTFOR (spclist, str, char)
			if (*str == 'N')
			    break;
		    if (!str)
			{
			    (void) sprintf(errStr,
					   "%s: an empty field in a nonempty tuple\n%s %s",
					   TraGName(tran), isin ? "from" : "to",
					   PlaGName(place));
			    error(errStr);
			    LstDestroy(spclist, FALSE);
			    LstDestroy(symlist, TRUE);
			    LstDestroy(aux, TRUE);
			    LstDestroy(stack, FALSE);
			    goto recover;
			}
		    varsym = NULL;
		    str = (char *) LstBegin(spclist);
		    LSTFOR (symlist, symbol, SYMBOL)
			{
			    if ((!SymIsConstant(symbol)) || (*str == 'N'))
				{
				    if (varsym || !SymIsVariable(symbol))
					{
					    varsym = NULL;
					    break;
					}
				    varsym = symbol;
				}
			    str = (char *) LstStep(spclist);
			}
		    if (varsym)
			{
			    (void) LstDelete(symlist, (char *) varsym,
					     FALSE);
			    LstDestroy(symlist, TRUE);
			    (void) LstInsLast(aux, (char *) varsym);
			}
		    else
			(void) LstInsLast(aux, (char *)
					 SymCons(SK_OPERATION,
						 (char *) symlist));
		    count++;
		    LstDestroy(spclist, FALSE);
		    spclist = LstCreate(&strTools);
		    symlist = LstCreate(&symTools);
		    break;
		case SPACE:
		case NEWLINE:
		case COMMENT:
		    if (count != 0)
			{
			    (void) LstInsLast(symlist, (char *)
					     SymCons(SK_CONSTANT,
						     StrCopy(nekot->token)));
			    (void) LstInsLast(spclist, (char *) "Y");
			}
		    break;
		case TUPLE_RANGE:
		    (void) sprintf(errStr,
				   "%s: \"..\" on an arc %s %s\nis completely meaningless",
				   TraGName(tran), isin ? "from" : "to",
				   PlaGName(place));
		    error(errStr);
		    LstDestroy(spclist, FALSE);
		    LstDestroy(symlist, TRUE);
		    LstDestroy(aux, TRUE);
		    LstDestroy(stack, FALSE);
		    goto recover;
		default:
		    if (count == 0)
			count = 1;
		    (void) LstInsLast(symlist, (char *)
				      SymCons(SK_CONSTANT,
					      StrCopy(nekot->token)));
		    (void) LstInsLast(spclist, (char *) "N");
		}
	}
    LstDestroy(stack, FALSE);
    if (count == 0)
	LstDestroy(symlist, TRUE);
    else
	{
	    LSTFOR (spclist, str, char)
		if (*str == 'N')
		    break;
	    if (!str)
		{
		    (void) sprintf(errStr,
				   "%s: an empty field in a nonempty tuple\n%s %s",
				   TraGName(tran), isin ? "from" : "to",
				   PlaGName(place));
		    error(errStr);
		    LstDestroy(spclist, FALSE);
		    LstDestroy(symlist, TRUE);
		    LstDestroy(aux, TRUE);
		    goto recover;
		}
	    varsym = NULL;
	    str = (char *) LstBegin(spclist);
	    LSTFOR (symlist, symbol, SYMBOL)
		{
		    if ((!SymIsConstant(symbol)) || (*str == 'N'))
			{
			    if (varsym || !SymIsVariable(symbol))
				{
				    varsym = NULL;
				    break;
				}
			    varsym = symbol;
			}
		    str = (char *) LstStep(spclist);
		}
	    if (varsym)
		{
		    (void) LstDelete(symlist, (char *) varsym,
				     FALSE);
		    LstDestroy(symlist, TRUE);
		    (void) LstInsLast(aux, (char *) varsym);
		}
	    else
		(void) LstInsLast(aux, (char *)
				  SymCons(SK_OPERATION,
					  (char *) symlist));
	}
    LstDestroy(spclist, FALSE);
    LstDestroy(list, TRUE);
    if (PlaGArity(place) == (unsigned) -1)
	PlaSArity(place, count);
    else if (arity_complaints && (count != PlaGArity(place)))
	{
	    (void) sprintf(errStr,
			   "Varying arity in %s (%u and %u)\nIf you want several arities in a place, run %s without the -a option",
			   PlaGName(place), PlaGArity(place), count, myName);
	    error(errStr);
	    LstDestroy(aux, TRUE);
	    return NULL;
	}
    symmultup = SytCons(count);
    count = 0;
    LSTFOR (aux, symbol, SYMBOL)
	SytRSymbol(symmultup, count++, symbol);
    LstDestroy(aux, FALSE);
    return symmultup;
 recover: ;
    if (dToken)
	PpTokDiscard(dToken);
    LstDestroy(list, TRUE);
    return NULL;
}
