/***************************************************************************
*
* Copyright (c) 1989 -- 2000
* by Helsinki University of Technology, Digital Systems Laboratory,
* a.k.a. Laboratory for Theoretical Computer Science.
* All rights reserved.
*
* This software is provided "as is" without express or implied warranty.
*
* Module:  util.c
*
* Part of: PROD's program prpp
*
* Authors: Johannes Helander and Kimmo Varpaaniemi
*
* Current version:
*     $Id$
*
* Description:
*     This module contains routines for the handling of errors and such.
*
* Version history:
*     $Log$
***************************************************************************/

/***************************************************************************
*
* Static variable rcsid is not used anywhere in the code. The only reason
* for rcsid is the Revision Control System (RCS). RCS fills the template
* string with the latest revision information. C compiler puts the
* resulting literal string into the object file, and linker puts it into
* the executable file. It is then possible to check the revision
* information of the object or executable file. (There is a little
* problem. An optimizing compiler or linker may eliminate the unused
* static variable.)
*
****************************************************************************
*/
#ifdef RCS
static char rcsid[] = "$Id$";
#endif

#include <stdio.h>
#include <string.h>
#include "absolute.h"
#include "prod.h"
#include "alloc.h"
#include "error.h"
#include "dynam.h"
#include "ppio.h"
#include "prpp.h"
#include "util.h"

int error_flag;  /* If the program is run without a premature exit,
                  * the value of error_flag becomes the exit status. */
char *errStr;    /* An error message can be put here if wanted. */

static int c_e_t = 0;

/***************************************************************************
*
* Function:  common_error_texts
* Purpose:   This routine shows the name of the file and the line
*            where an error has been observed.
* Arguments:
* Returns:
* See also:
*
****************************************************************************
*/
FS0(void, common_error_texts)
{
    if (!c_e_t)
	{
	    (void) fputc((int) '\n', _err);
	    c_e_t = 1;
	}
    if (curFile)
	{
            (void) fprintf(_err, "%s: %lu: ",
			   curFile->name ? curFile->name
			   : "<standard input>",
			   curFile->line + 1
			   - curFile->newlines_pushed);
	}
}

#define FLUSH_FILES (void) fflush(_out); (void) fflush(_err)

/***************************************************************************
*
* Function:  panic
* Purpose:   This routine handles errors that cannot be handled without
*            an immediate exit. An error message is displayed.
* Arguments: s - the error message (if begins with a dollar, a system
*                level error message is displayed, too)
* Returns:
* See also:  error, warning
*
****************************************************************************
*/
FE1(void, panic,
    char *, s)
{
    FLUSH_FILES;
    common_error_texts();
    if (*s == '$')
	{
	    (void) fprintf(_err, s + 1);
	    (void) fputc((int) '\n', _err);
	    ErrnoOutput(_err);
	}
    else
	{
	    (void) fprintf(_err, s);
	    (void) fputc((int) '\n', _err);
	    (void) fflush(_err);
	}
    Exit(1);
}

/***************************************************************************
*
* Function:  error
* Purpose:   This routine handles errors that can be handled without
*            an immediate exit. An error message is displayed.
* Arguments: s - the error message (if begins with a dollar, a system
*                level error message is displayed, too)
* Returns:
* See also:  panic, warning
*
****************************************************************************
*/
FE1(void, error,
    char *, s)
{
    error_flag = 1;
    FLUSH_FILES;
    common_error_texts();
    if (*s == '$')
	{
	    (void) fprintf(_err, s + 1);
	    (void) fputc((int) '\n', _err);
	    ErrnoOutput(_err);
	}
    else
	{
	    (void) fprintf(_err, s);
	    (void) fputc((int) '\n', _err);
	    (void) fflush(_err);
	}
}

/***************************************************************************
*
* Function:  warning
* Purpose:   This routine displays a warning.
* Arguments: s - the warning
* Returns:
* See also:  panic, error
*
****************************************************************************
*/
FE1(void, warning,
    char *, s)
{
    FLUSH_FILES;
    common_error_texts();
    (void) fputs("Warning: ", _err);
    (void) fprintf(_err, s);
    (void) fputc((int) '\n', _err);
    (void) fflush(_err);
}

