/**
 * (C) University Pierre & Marie Curie - UMR CNRS 7606 (LIP6/MoVe)
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *  
 *  Contributors: 
 *    Lom HILLAH (LIP6) - Initial models and implementation
 *    Rachid Alahyane (UPMC) - Infrastructure and continuous integration
 *    Bastien Bouzerau (UPMC) - Architecture
 *    Guillaume Giffo (UPMC)  - Code generation refactoring, high-level API
 *
 * $Id ggiffo, Wed Sep 10 13:41:23 CEST 2008$
 */
package fr.lip6.move.pnml.hlpn.hlcorestructure.hlapi;

import org.apache.axiom.om.OMElement;
import org.eclipse.emf.common.util.DiagnosticChain;

import fr.lip6.move.pnml.framework.hlapi.HLAPIClass;
import fr.lip6.move.pnml.framework.utils.IdRefLinker;
import fr.lip6.move.pnml.framework.utils.exception.InnerBuildException;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.hlpn.hlcorestructure.AnnotationGraphics;
import fr.lip6.move.pnml.hlpn.hlcorestructure.HlcorestructureFactory;
import fr.lip6.move.pnml.hlpn.hlcorestructure.Offset;
import fr.lip6.move.pnml.hlpn.hlcorestructure.impl.HlcorestructureFactoryImpl;

public class OffsetHLAPI implements HLAPIClass, CoordinateHLAPI {

    /**
     * The contained LLAPI element.
     */
    private Offset item;

    /**
     * this constructor allows you to set all 'settable' values excepted
     * container.
     */

    public OffsetHLAPI(java.lang.Integer x

    , java.lang.Integer y) {// BEGIN CONSTRUCTOR BODY
        HlcorestructureFactory fact = new HlcorestructureFactoryImpl();
        item = fact.createOffset();

        if (x != null) {

            item.setX(x);
        }

        if (y != null) {

            item.setY(y);
        }

    }

    /**
     * this constructor allows you to set all 'settable' values, including
     * container if any.
     */

    public OffsetHLAPI(java.lang.Integer x

    , java.lang.Integer y

    , AnnotationGraphicsHLAPI containerAnnotationGraphics) {// BEGIN CONSTRUCTOR
                                                            // BODY
        HlcorestructureFactory fact = new HlcorestructureFactoryImpl();
        item = fact.createOffset();

        if (x != null) {

            item.setX(x);
        }

        if (y != null) {

            item.setY(y);
        }

        if (containerAnnotationGraphics != null)
            item
                    .setContainerAnnotationGraphics((AnnotationGraphics) containerAnnotationGraphics
                            .getContainedItem());

    }

    /**
     * This constructor encapsulate a low level API object in HLAPI.
     */
    public OffsetHLAPI(Offset lowLevelAPI) {
        item = lowLevelAPI;
    }

    // access to low level API
    /**
     * Return encapsulated object
     */
    public Offset getContainedItem() {
        return item;
    }

    // getters giving LLAPI object

    /**
     * Return the encapsulate Low Level API object.
     */
    public Integer getX() {
        return item.getX();
    }

    /**
     * Return the encapsulate Low Level API object.
     */
    public Integer getY() {
        return item.getY();
    }

    /**
     * Return the encapsulate Low Level API object.
     */
    public AnnotationGraphics getContainerAnnotationGraphics() {
        return item.getContainerAnnotationGraphics();
    }

    // getters giving HLAPI object

    /**
     * This accessor automatically encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     * 
     * @return : null if the element is null
     */

    public AnnotationGraphicsHLAPI getContainerAnnotationGraphicsHLAPI() {
        if (item.getContainerAnnotationGraphics() == null)
            return null;
        return new AnnotationGraphicsHLAPI(item
                .getContainerAnnotationGraphics());
    }

    // Special getter for list of generics object, return only one object type.

    // setters (including container setter if aviable)

    /**
     * set X
     */
    public void setXHLAPI(

    java.lang.Integer elem) {

        if (elem != null) {

            item.setX(elem);
        }

    }

    /**
     * set Y
     */
    public void setYHLAPI(

    java.lang.Integer elem) {

        if (elem != null) {

            item.setY(elem);
        }

    }

    /**
     * set ContainerAnnotationGraphics
     */
    public void setContainerAnnotationGraphicsHLAPI(

    AnnotationGraphicsHLAPI elem) {

        if (elem != null)
            item.setContainerAnnotationGraphics((AnnotationGraphics) elem
                    .getContainedItem());

    }

    // setters/remover for lists.

    // equals method
    public boolean equals(OffsetHLAPI item) {
        return item.getContainedItem().equals(getContainedItem());
    }

    // PNML

    /**
     * return the PNML xml tree for this object.
     */
    public String toPNML() {
        return item.toPNML();
    }

    /**
     * creates an object from the xml nodes.(symetric work of toPNML)
     */
    public void fromPNML(OMElement subRoot, IdRefLinker idr)
            throws InnerBuildException, InvalidIDException,
            VoidRepositoryException {
        item.fromPNML(subRoot, idr);
    }

    public boolean validateOCL(DiagnosticChain diagnostics) {
        return item.validateOCL(diagnostics);
    }

}