/**
 * (C) University Pierre & Marie Curie - UMR CNRS 7606 (LIP6/MoVe)
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *  
 *  Contributors: 
 *    Lom HILLAH (LIP6) - Initial models and implementation
 *    Rachid Alahyane (UPMC) - Infrastructure and continuous integration
 *    Bastien Bouzerau (UPMC) - Architecture
 *    Guillaume Giffo (UPMC)  - Code generation refactoring, high-level API
 *
 * $Id ggiffo, Wed Sep 10 13:41:23 CEST 2008$
 */
package fr.lip6.move.pnml.hlpn.hlcorestructure.hlapi;

import java.util.ArrayList;
import java.util.List;

import org.apache.axiom.om.OMElement;
import org.eclipse.emf.common.util.DiagnosticChain;

import fr.lip6.move.pnml.framework.hlapi.HLAPIClass;
import fr.lip6.move.pnml.framework.utils.IdRefLinker;
import fr.lip6.move.pnml.framework.utils.ModelRepository;
import fr.lip6.move.pnml.framework.utils.exception.InnerBuildException;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.OtherException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.hlpn.hlcorestructure.Arc;
import fr.lip6.move.pnml.hlpn.hlcorestructure.HlcorestructureFactory;
import fr.lip6.move.pnml.hlpn.hlcorestructure.Name;
import fr.lip6.move.pnml.hlpn.hlcorestructure.NodeGraphics;
import fr.lip6.move.pnml.hlpn.hlcorestructure.Page;
import fr.lip6.move.pnml.hlpn.hlcorestructure.PlaceNode;
import fr.lip6.move.pnml.hlpn.hlcorestructure.RefPlace;
import fr.lip6.move.pnml.hlpn.hlcorestructure.ToolInfo;
import fr.lip6.move.pnml.hlpn.hlcorestructure.impl.HlcorestructureFactoryImpl;

public class RefPlaceHLAPI implements HLAPIClass, PnObjectHLAPI, NodeHLAPI,
        PlaceNodeHLAPI {

    /**
     * The contained LLAPI element.
     */
    private RefPlace item;

    /**
     * this constructor allows you to set all 'settable' values excepted
     * container.
     */

    public RefPlaceHLAPI(java.lang.String id

    , NameHLAPI name

    , NodeGraphicsHLAPI nodegraphics

    , PlaceNodeHLAPI ref) throws InvalidIDException, VoidRepositoryException {// BEGIN
                                                                                // CONSTRUCTOR
                                                                                // BODY
        HlcorestructureFactory fact = new HlcorestructureFactoryImpl();
        item = fact.createRefPlace();

        if (id != null) {

            item.setId(ModelRepository.getInstance().getCurrentIdRepository()
                    .checkId(id, this));
        }

        if (name != null)
            item.setName((Name) name.getContainedItem());

        if (nodegraphics != null)
            item
                    .setNodegraphics((NodeGraphics) nodegraphics
                            .getContainedItem());

        if (ref != null)
            item.setRef((PlaceNode) ref.getContainedItem());

    }

    /**
     * this constructor allows you to set all 'settable' values, including
     * container if any.
     */

    public RefPlaceHLAPI(java.lang.String id

    , NameHLAPI name

    , NodeGraphicsHLAPI nodegraphics

    , PlaceNodeHLAPI ref

    , PageHLAPI containerPage) throws InvalidIDException,
            VoidRepositoryException {// BEGIN CONSTRUCTOR BODY
        HlcorestructureFactory fact = new HlcorestructureFactoryImpl();
        item = fact.createRefPlace();

        if (id != null) {

            item.setId(ModelRepository.getInstance().getCurrentIdRepository()
                    .checkId(id, this));
        }

        if (name != null)
            item.setName((Name) name.getContainedItem());

        if (nodegraphics != null)
            item
                    .setNodegraphics((NodeGraphics) nodegraphics
                            .getContainedItem());

        if (ref != null)
            item.setRef((PlaceNode) ref.getContainedItem());

        if (containerPage != null)
            item.setContainerPage((Page) containerPage.getContainedItem());

    }

    /**
     * This constructor give access to required stuff only (not container if
     * any)
     */
    public RefPlaceHLAPI(java.lang.String id

    , PlaceNodeHLAPI ref) throws InvalidIDException, VoidRepositoryException {// BEGIN
                                                                                // CONSTRUCTOR
                                                                                // BODY
        HlcorestructureFactory fact = new HlcorestructureFactoryImpl();
        item = fact.createRefPlace();

        if (id != null) {

            item.setId(ModelRepository.getInstance().getCurrentIdRepository()
                    .checkId(id, this));
        }

        if (ref != null)
            item.setRef((PlaceNode) ref.getContainedItem());

    }

    /**
     * This constructor give access to required stuff only (and container)
     */
    public RefPlaceHLAPI(java.lang.String id

    , PlaceNodeHLAPI ref

    , PageHLAPI containerPage) throws InvalidIDException,
            VoidRepositoryException {// BEGIN CONSTRUCTOR BODY
        HlcorestructureFactory fact = new HlcorestructureFactoryImpl();
        item = fact.createRefPlace();

        if (id != null) {

            item.setId(ModelRepository.getInstance().getCurrentIdRepository()
                    .checkId(id, this));
        }

        if (ref != null)
            item.setRef((PlaceNode) ref.getContainedItem());

        if (containerPage != null)
            item.setContainerPage((Page) containerPage.getContainedItem());

    }

    /**
     * This constructor encapsulate a low level API object in HLAPI.
     */
    public RefPlaceHLAPI(RefPlace lowLevelAPI) {
        item = lowLevelAPI;
    }

    // access to low level API
    /**
     * Return encapsulated object
     */
    public RefPlace getContainedItem() {
        return item;
    }

    // getters giving LLAPI object

    /**
     * Return the encapsulate Low Level API object.
     */
    public String getId() {
        return item.getId();
    }

    /**
     * Return the encapsulate Low Level API object.
     */
    public Name getName() {
        return item.getName();
    }

    /**
     * Return the encapsulate Low Level API object.
     */
    public List<ToolInfo> getToolspecifics() {
        return item.getToolspecifics();
    }

    /**
     * Return the encapsulate Low Level API object.
     */
    public Page getContainerPage() {
        return item.getContainerPage();
    }

    /**
     * Return the encapsulate Low Level API object.
     */
    public List<Arc> getInArcs() {
        return item.getInArcs();
    }

    /**
     * Return the encapsulate Low Level API object.
     */
    public List<Arc> getOutArcs() {
        return item.getOutArcs();
    }

    /**
     * Return the encapsulate Low Level API object.
     */
    public NodeGraphics getNodegraphics() {
        return item.getNodegraphics();
    }

    /**
     * Return the encapsulate Low Level API object.
     */
    public List<RefPlace> getReferencingPlaces() {
        return item.getReferencingPlaces();
    }

    /**
     * Return the encapsulate Low Level API object.
     */
    public PlaceNode getRef() {
        return item.getRef();
    }

    // getters giving HLAPI object

    /**
     * This accessor automatically encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     * 
     * @return : null if the element is null
     */

    public NameHLAPI getNameHLAPI() {
        if (item.getName() == null)
            return null;
        return new NameHLAPI(item.getName());
    }

    /**
     * This accessor automatically encapsulate all elements of the selected
     * sublist. WARNING : this can creates a lot of new object in memory.
     */

    public List<ToolInfoHLAPI> getToolspecificsHLAPI() {
        List<ToolInfoHLAPI> retour = new ArrayList<ToolInfoHLAPI>();
        for (ToolInfo elemnt : getToolspecifics()) {
            retour.add(new ToolInfoHLAPI(elemnt));
        }
        return retour;
    }

    /**
     * This accessor automatically encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     * 
     * @return : null if the element is null
     */

    public PageHLAPI getContainerPageHLAPI() {
        if (item.getContainerPage() == null)
            return null;
        return new PageHLAPI(item.getContainerPage());
    }

    /**
     * This accessor automatically encapsulate all elements of the selected
     * sublist. WARNING : this can creates a lot of new object in memory.
     */

    public List<ArcHLAPI> getInArcsHLAPI() {
        List<ArcHLAPI> retour = new ArrayList<ArcHLAPI>();
        for (Arc elemnt : getInArcs()) {
            retour.add(new ArcHLAPI(elemnt));
        }
        return retour;
    }

    /**
     * This accessor automatically encapsulate all elements of the selected
     * sublist. WARNING : this can creates a lot of new object in memory.
     */

    public List<ArcHLAPI> getOutArcsHLAPI() {
        List<ArcHLAPI> retour = new ArrayList<ArcHLAPI>();
        for (Arc elemnt : getOutArcs()) {
            retour.add(new ArcHLAPI(elemnt));
        }
        return retour;
    }

    /**
     * This accessor automatically encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     * 
     * @return : null if the element is null
     */

    public NodeGraphicsHLAPI getNodegraphicsHLAPI() {
        if (item.getNodegraphics() == null)
            return null;
        return new NodeGraphicsHLAPI(item.getNodegraphics());
    }

    /**
     * This accessor automatically encapsulate all elements of the selected
     * sublist. WARNING : this can creates a lot of new object in memory.
     */

    public List<RefPlaceHLAPI> getReferencingPlacesHLAPI() {
        List<RefPlaceHLAPI> retour = new ArrayList<RefPlaceHLAPI>();
        for (RefPlace elemnt : getReferencingPlaces()) {
            retour.add(new RefPlaceHLAPI(elemnt));
        }
        return retour;
    }

    /**
     * This accessor automatically encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     * 
     * @return : null if the element is null
     */

    public PlaceNodeHLAPI getRefHLAPI() {
        if (item.getRef() == null)
            return null;
        PlaceNode object = item.getRef();

        if (object.getClass().equals(
                fr.lip6.move.pnml.hlpn.hlcorestructure.impl.PlaceImpl.class)) {
            return new fr.lip6.move.pnml.hlpn.hlcorestructure.hlapi.PlaceHLAPI(
                    (fr.lip6.move.pnml.hlpn.hlcorestructure.Place) object);
        }

        if (object.getClass().equals(
                fr.lip6.move.pnml.hlpn.hlcorestructure.impl.RefPlaceImpl.class)) {
            return new fr.lip6.move.pnml.hlpn.hlcorestructure.hlapi.RefPlaceHLAPI(
                    (fr.lip6.move.pnml.hlpn.hlcorestructure.RefPlace) object);
        }

        return null;
    }

    // Special getter for list of generics object, return only one object type.

    // setters (including container setter if aviable)

    /**
     * set Id
     */
    public void setIdHLAPI(

    java.lang.String elem) throws InvalidIDException, VoidRepositoryException {

        if (elem != null) {

            try {
                item.setId(ModelRepository.getInstance()
                        .getCurrentIdRepository().changeId(this, elem));
            } catch (OtherException e) {
                ModelRepository.getInstance().getCurrentIdRepository().checkId(
                        elem, this);
            }
        }

    }

    /**
     * set Name
     */
    public void setNameHLAPI(

    NameHLAPI elem) {

        if (elem != null)
            item.setName((Name) elem.getContainedItem());

    }

    /**
     * set Nodegraphics
     */
    public void setNodegraphicsHLAPI(

    NodeGraphicsHLAPI elem) {

        if (elem != null)
            item.setNodegraphics((NodeGraphics) elem.getContainedItem());

    }

    /**
     * set Ref
     */
    public void setRefHLAPI(

    PlaceNodeHLAPI elem) {

        if (elem != null)
            item.setRef((PlaceNode) elem.getContainedItem());

    }

    /**
     * set ContainerPage
     */
    public void setContainerPageHLAPI(

    PageHLAPI elem) {

        if (elem != null)
            item.setContainerPage((Page) elem.getContainedItem());

    }

    // setters/remover for lists.

    public void addToolspecificsHLAPI(ToolInfoHLAPI unit) {
        item.getToolspecifics().add((ToolInfo) unit.getContainedItem());
    }

    public void removeToolspecificsHLAPI(ToolInfoHLAPI unit) {
        item.getToolspecifics().remove((ToolInfo) unit.getContainedItem());
    }

    // equals method
    public boolean equals(RefPlaceHLAPI item) {
        return item.getContainedItem().equals(getContainedItem());
    }

    // PNML

    /**
     * return the PNML xml tree for this object.
     */
    public String toPNML() {
        return item.toPNML();
    }

    /**
     * creates an object from the xml nodes.(symetric work of toPNML)
     */
    public void fromPNML(OMElement subRoot, IdRefLinker idr)
            throws InnerBuildException, InvalidIDException,
            VoidRepositoryException {
        item.fromPNML(subRoot, idr);
    }

    public boolean validateOCL(DiagnosticChain diagnostics) {
        return item.validateOCL(diagnostics);
    }

}