/**
 * (C) University Pierre & Marie Curie - UMR CNRS 7606 (LIP6/MoVe)
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *  
 *  Contributors: 
 *    Lom HILLAH (LIP6) - Initial models and implementation
 *    Rachid Alahyane (UPMC) - Infrastructure and continuous integration
 *    Bastien Bouzerau (UPMC) - Architecture
 *    Guillaume Giffo (UPMC)  - Code generation refactoring, high-level API
 *
 * $Id$
 */
package fr.lip6.move.pnml.hlpn.partitions.impl;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.DiagnosticChain;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.util.EObjectContainmentWithInverseEList;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.lip6.move.pnml.framework.utils.IdRefLinker;
import fr.lip6.move.pnml.framework.utils.ModelRepository;
import fr.lip6.move.pnml.framework.utils.PrettyPrintData;
import fr.lip6.move.pnml.framework.utils.exception.InnerBuildException;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.hlpn.arbitrarydeclarations.AnySort;
import fr.lip6.move.pnml.hlpn.arbitrarydeclarations.impl.ArbitrarydeclarationsFactoryImpl;
import fr.lip6.move.pnml.hlpn.booleans.Bool;
import fr.lip6.move.pnml.hlpn.booleans.impl.BooleansFactoryImpl;
import fr.lip6.move.pnml.hlpn.cyclicEnumerations.CyclicEnumeration;
import fr.lip6.move.pnml.hlpn.cyclicEnumerations.impl.CyclicEnumerationsFactoryImpl;
import fr.lip6.move.pnml.hlpn.dots.Dot;
import fr.lip6.move.pnml.hlpn.dots.impl.DotsFactoryImpl;
import fr.lip6.move.pnml.hlpn.finiteEnumerations.FiniteEnumeration;
import fr.lip6.move.pnml.hlpn.finiteEnumerations.impl.FiniteEnumerationsFactoryImpl;
import fr.lip6.move.pnml.hlpn.finiteIntRanges.FiniteIntRange;
import fr.lip6.move.pnml.hlpn.finiteIntRanges.impl.FiniteIntRangesFactoryImpl;
import fr.lip6.move.pnml.hlpn.integers.HLInteger;
import fr.lip6.move.pnml.hlpn.integers.Natural;
import fr.lip6.move.pnml.hlpn.integers.Positive;
import fr.lip6.move.pnml.hlpn.integers.impl.IntegersFactoryImpl;
import fr.lip6.move.pnml.hlpn.lists.HLPNList;
import fr.lip6.move.pnml.hlpn.lists.impl.ListsFactoryImpl;
import fr.lip6.move.pnml.hlpn.partitions.Partition;
import fr.lip6.move.pnml.hlpn.partitions.PartitionElement;
import fr.lip6.move.pnml.hlpn.partitions.PartitionsFactory;
import fr.lip6.move.pnml.hlpn.partitions.PartitionsPackage;
import fr.lip6.move.pnml.hlpn.terms.MultisetSort;
import fr.lip6.move.pnml.hlpn.terms.ProductSort;
import fr.lip6.move.pnml.hlpn.terms.Sort;
import fr.lip6.move.pnml.hlpn.terms.TermsPackage;
import fr.lip6.move.pnml.hlpn.terms.UserSort;
import fr.lip6.move.pnml.hlpn.terms.impl.SortDeclImpl;
import fr.lip6.move.pnml.hlpn.terms.impl.TermsFactoryImpl;
import fr.lip6.move.pnml.hlpn.terms.util.TermsValidator;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Partition</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.lip6.move.pnml.hlpn.partitions.impl.PartitionImpl#getDef <em>Def</em>}</li>
 *   <li>{@link fr.lip6.move.pnml.hlpn.partitions.impl.PartitionImpl#getPartitionelements <em>Partitionelements</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class PartitionImpl extends SortDeclImpl implements Partition {
	/**
	 * The cached value of the '{@link #getDef() <em>Def</em>}' containment reference.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @see #getDef()
	 * @generated
	 * @ordered
	 */
	protected Sort def;

	/**
	 * The cached value of the '{@link #getPartitionelements() <em>Partitionelements</em>}' containment reference list.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @see #getPartitionelements()
	 * @generated
	 * @ordered
	 */
	protected EList<PartitionElement> partitionelements;

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	protected PartitionImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return PartitionsPackage.Literals.PARTITION;
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	public Sort getDef() {
		return def;
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	public NotificationChain basicSetDef(Sort newDef, NotificationChain msgs) {
		Sort oldDef = def;
		def = newDef;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this,
					Notification.SET, PartitionsPackage.PARTITION__DEF, oldDef,
					newDef);
			if (msgs == null)
				msgs = notification;
			else
				msgs.add(notification);
		}
		return msgs;
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	public void setDef(Sort newDef) {
		if (newDef != def) {
			NotificationChain msgs = null;
			if (def != null)
				msgs = ((InternalEObject) def).eInverseRemove(this,
						TermsPackage.SORT__CONTAINER_PARTITION, Sort.class,
						msgs);
			if (newDef != null)
				msgs = ((InternalEObject) newDef).eInverseAdd(this,
						TermsPackage.SORT__CONTAINER_PARTITION, Sort.class,
						msgs);
			msgs = basicSetDef(newDef, msgs);
			if (msgs != null)
				msgs.dispatch();
		} else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET,
					PartitionsPackage.PARTITION__DEF, newDef, newDef));
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	public List<PartitionElement> getPartitionelements() {
		if (partitionelements == null) {
			partitionelements = new EObjectContainmentWithInverseEList<PartitionElement>(
					PartitionElement.class, this,
					PartitionsPackage.PARTITION__PARTITIONELEMENTS,
					PartitionsPackage.PARTITION_ELEMENT__REFPARTITION);
		}
		return partitionelements;
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd,
			int featureID, NotificationChain msgs) {
		switch (featureID) {
		case PartitionsPackage.PARTITION__DEF:
			if (def != null)
				msgs = ((InternalEObject) def).eInverseRemove(this,
						EOPPOSITE_FEATURE_BASE
								- PartitionsPackage.PARTITION__DEF, null, msgs);
			return basicSetDef((Sort) otherEnd, msgs);
		case PartitionsPackage.PARTITION__PARTITIONELEMENTS:
			return ((InternalEList<InternalEObject>) (InternalEList<?>) getPartitionelements())
					.basicAdd(otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd,
			int featureID, NotificationChain msgs) {
		switch (featureID) {
		case PartitionsPackage.PARTITION__DEF:
			return basicSetDef(null, msgs);
		case PartitionsPackage.PARTITION__PARTITIONELEMENTS:
			return ((InternalEList<?>) getPartitionelements()).basicRemove(
					otherEnd, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
		case PartitionsPackage.PARTITION__DEF:
			return getDef();
		case PartitionsPackage.PARTITION__PARTITIONELEMENTS:
			return getPartitionelements();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
		case PartitionsPackage.PARTITION__DEF:
			setDef((Sort) newValue);
			return;
		case PartitionsPackage.PARTITION__PARTITIONELEMENTS:
			getPartitionelements().clear();
			getPartitionelements().addAll(
					(Collection<? extends PartitionElement>) newValue);
			return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
		case PartitionsPackage.PARTITION__DEF:
			setDef((Sort) null);
			return;
		case PartitionsPackage.PARTITION__PARTITIONELEMENTS:
			getPartitionelements().clear();
			return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
		case PartitionsPackage.PARTITION__DEF:
			return def != null;
		case PartitionsPackage.PARTITION__PARTITIONELEMENTS:
			return partitionelements != null && !partitionelements.isEmpty();
		}
		return super.eIsSet(featureID);
	}

	/**
	 * Return the string containing the pnml output
	 */
	public String toPNML() {
		// id 1
		// idref 0
		// attributes 1
		// sons 2

		Boolean prettyPrintStatus = ModelRepository.getInstance()
				.isPrettyPrintActive();
		String retline = "";
		String headline = "";
		PrettyPrintData prpd = null;
		if (prettyPrintStatus) {
			retline = "\n";
			prpd = ModelRepository.getInstance().getPrettyPrintData();
			headline = prpd.getCurrentLineHeader();
		}

		StringBuffer sb = new StringBuffer();
		sb.append(headline);
		sb.append("<partition");
		if (prettyPrintStatus) {
			headline = prpd.increaseLineHeaderLevel();
		}
		// begin attributes, id and id ref processing

		if (getId() != null) {
			sb.append(" id");
			sb.append("=\"");
			sb.append(getId());
			sb.append("\"");
		}

		if (getName() != null) {
			sb.append(" name");
			sb.append("=\"");
			sb.append(getName());
			sb.append("\"");
		}

		boolean haveSons = false;
		sb.append(">");
		sb.append(retline);

		// sons, follow processing

		if (getDef() != null) {

			String top = getDef().toPNML();
			if (!top.equals("")) {

				sb.append(top);

				haveSons = true;
			}

		}

		if (getPartitionelements() != null) {

			java.util.List<fr.lip6.move.pnml.hlpn.partitions.PartitionElement> items = getPartitionelements();
			for (Iterator<PartitionElement> iterator = items.iterator(); iterator
					.hasNext();) {
				PartitionElement item = (PartitionElement) iterator.next();

				sb.append(item.toPNML());

				haveSons = true;
			}

		}

		/** * */

		if (prettyPrintStatus) {
			headline = prpd.decreaseLineHeaderLevel();
		}

		if (haveSons) {
			sb.append(headline);
			sb.append("</partition>");
			sb.append(retline);
		} else {
			sb.delete(sb.lastIndexOf(">"), sb.length());
			sb.append("/>");
		}

		return sb.toString();
	}

	@SuppressWarnings("unchecked")
	public void fromPNML(OMElement locRoot, IdRefLinker idr)
			throws InnerBuildException, InvalidIDException,
			VoidRepositoryException {
		// 1
		// 0
		// 1
		// 2
		@SuppressWarnings("unused")
		PartitionsFactory fact = new PartitionsFactoryImpl();

		// processing id

		if (locRoot.getAttributeValue(new QName("id")) != null) {
			this.setId(new java.lang.String(locRoot
					.getAttributeValue(new QName("id"))));
			ModelRepository.getInstance().getCurrentIdRepository().checkId(
					new java.lang.String(locRoot.getAttributeValue(new QName(
							"id"))).toString(), (Object) this);
		}

		// processing idref

		// processing attributes

		if (locRoot.getAttributeValue(new QName("name")) != null) {
			try {
				this.setName(new String(locRoot.getAttributeValue(new QName(
						"name"))));
			} catch (Exception e) {
				throw new fr.lip6.move.pnml.framework.utils.exception.InnerBuildException(
						e);
			}
		}

		// processing sons

		for (Iterator iterator = locRoot.getChildElements(); iterator.hasNext();) {
			@SuppressWarnings("unused")
			OMElement type = (OMElement) iterator.next();

			if (type.getLocalName().equals("anysort")) {
				AnySort item;
				item = new ArbitrarydeclarationsFactoryImpl().createAnySort();
				item.fromPNML(type, idr);

				item.setContainerPartition(this);

				continue;
			}// end if

			if (type.getLocalName().equals("null")) {
				Bool item;
				item = new BooleansFactoryImpl().createBool();
				item.fromPNML(type, idr);

				item.setContainerPartition(this);

				continue;
			}// end if

			if (type.getLocalName().equals("cyclicenumeration")) {
				CyclicEnumeration item;
				item = new CyclicEnumerationsFactoryImpl()
						.createCyclicEnumeration();
				item.fromPNML(type, idr);

				item.setContainerPartition(this);

				continue;
			}// end if

			if (type.getLocalName().equals("dot")) {
				Dot item;
				item = new DotsFactoryImpl().createDot();
				item.fromPNML(type, idr);

				item.setContainerPartition(this);

				continue;
			}// end if

			if (type.getLocalName().equals("finiteenumeration")) {
				FiniteEnumeration item;
				item = new FiniteEnumerationsFactoryImpl()
						.createFiniteEnumeration();
				item.fromPNML(type, idr);

				item.setContainerPartition(this);

				continue;
			}// end if

			if (type.getLocalName().equals("finiteintrange")) {
				FiniteIntRange item;
				item = new FiniteIntRangesFactoryImpl().createFiniteIntRange();
				item.fromPNML(type, idr);

				item.setContainerPartition(this);

				continue;
			}// end if

			if (type.getLocalName().equals("natural")) {
				Natural item;
				item = new IntegersFactoryImpl().createNatural();
				item.fromPNML(type, idr);

				item.setContainerPartition(this);

				continue;
			}// end if

			if (type.getLocalName().equals("positive")) {
				Positive item;
				item = new IntegersFactoryImpl().createPositive();
				item.fromPNML(type, idr);

				item.setContainerPartition(this);

				continue;
			}// end if

			if (type.getLocalName().equals("integer")) {
				HLInteger item;
				item = new IntegersFactoryImpl().createHLInteger();
				item.fromPNML(type, idr);

				item.setContainerPartition(this);

				continue;
			}// end if

			if (type.getLocalName().equals("list")) {
				HLPNList item;
				item = new ListsFactoryImpl().createHLPNList();
				item.fromPNML(type, idr);

				item.setContainerPartition(this);

				continue;
			}// end if

			if (type.getLocalName().equals("multisetsort")) {
				MultisetSort item;
				item = new TermsFactoryImpl().createMultisetSort();
				item.fromPNML(type, idr);

				item.setContainerPartition(this);

				continue;
			}// end if

			if (type.getLocalName().equals("productsort")) {
				ProductSort item;
				item = new TermsFactoryImpl().createProductSort();
				item.fromPNML(type, idr);

				item.setContainerPartition(this);

				continue;
			}// end if

			if (type.getLocalName().equals("usersort")) {
				UserSort item;
				item = new TermsFactoryImpl().createUserSort();
				item.fromPNML(type, idr);

				item.setContainerPartition(this);

				continue;
			}// end if

			if (type.getLocalName().equals("partitionelement")) {
				PartitionElement item;
				item = new PartitionsFactoryImpl().createPartitionElement();
				item.fromPNML(type, idr);

				item.setRefpartition(this);

				continue;
			}// end if

		}

	}

	/**
	 * -
	 */
	public boolean validateOCL(DiagnosticChain diagnostics) {

		TermsValidator val = new TermsValidator();
		boolean retour = true;

		if (getDef() != null) {
			retour &= getDef().validateOCL(diagnostics);
		}

		if (getPartitionelements() != null) {
			java.util.List<fr.lip6.move.pnml.hlpn.partitions.PartitionElement> items = getPartitionelements();
			for (Iterator<PartitionElement> iterator = items.iterator(); iterator
					.hasNext();) {
				PartitionElement item = (PartitionElement) iterator.next();
				retour &= item.validateOCL(diagnostics);
			}
		}

		retour &= val.validate(this, diagnostics, null);
		return retour;

	}
} // PartitionImpl
