/**
 * (C) University Pierre & Marie Curie - UMR CNRS 7606 (LIP6/MoVe)
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *  
 *  Contributors: 
 *    Lom HILLAH (LIP6) - Initial models and implementation
 *    Rachid Alahyane (UPMC) - Infrastructure and continuous integration
 *    Bastien Bouzerau (UPMC) - Architecture
 *    Guillaume Giffo (UPMC)  - Code generation refactoring, high-level API
 *
 * $Id ggiffo, Wed Sep 10 13:41:33 CEST 2008$
 */
package fr.lip6.move.pnml.hlpn.terms.hlapi;

import java.util.List;

import fr.lip6.move.pnml.framework.hlapi.HLAPIClass;
import fr.lip6.move.pnml.hlpn.hlcorestructure.Condition;
import fr.lip6.move.pnml.hlpn.hlcorestructure.HLAnnotation;
import fr.lip6.move.pnml.hlpn.hlcorestructure.HLMarking;
import fr.lip6.move.pnml.hlpn.hlcorestructure.hlapi.ConditionHLAPI;
import fr.lip6.move.pnml.hlpn.hlcorestructure.hlapi.HLAnnotationHLAPI;
import fr.lip6.move.pnml.hlpn.hlcorestructure.hlapi.HLMarkingHLAPI;
import fr.lip6.move.pnml.hlpn.partitions.PartitionElement;
import fr.lip6.move.pnml.hlpn.partitions.hlapi.PartitionElementHLAPI;
import fr.lip6.move.pnml.hlpn.terms.NamedOperator;
import fr.lip6.move.pnml.hlpn.terms.Operator;
import fr.lip6.move.pnml.hlpn.terms.Sort;
import fr.lip6.move.pnml.hlpn.terms.Term;

public interface OperatorHLAPI extends HLAPIClass, TermHLAPI {

    // getters giving LLAPI object

    /**
     * 
     */
    public Sort getSort();

    /**
     * 
     */
    public Operator getContainerOperator();

    /**
     * 
     */
    public NamedOperator getContainerNamedOperator();

    /**
     * 
     */
    public HLMarking getContainerHLMarking();

    /**
     * 
     */
    public Condition getContainerCondition();

    /**
     * 
     */
    public HLAnnotation getContainerHLAnnotation();

    /**
     * 
     */
    public PartitionElement getContainerPartitionElement();

    /**
     * 
     */
    public List<Term> getSubterm();

    /**
     * 
     */
    public Sort getOutput();

    /**
     * 
     */
    public List<Sort> getInput();

    // getters giving HLAPI object

    /**
     * This accessor automaticaly encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     */

    public SortHLAPI getSortHLAPI();

    /**
     * This accessor automaticaly encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     */

    public OperatorHLAPI getContainerOperatorHLAPI();

    /**
     * This accessor automaticaly encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     */

    public NamedOperatorHLAPI getContainerNamedOperatorHLAPI();

    /**
     * This accessor automaticaly encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     */

    public HLMarkingHLAPI getContainerHLMarkingHLAPI();

    /**
     * This accessor automaticaly encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     */

    public ConditionHLAPI getContainerConditionHLAPI();

    /**
     * This accessor automaticaly encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     */

    public HLAnnotationHLAPI getContainerHLAnnotationHLAPI();

    /**
     * This accessor automaticaly encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     */

    public PartitionElementHLAPI getContainerPartitionElementHLAPI();

    /**
     * This accessor automaticaly encapsulate all elements of the selected
     * sublist. WARNING : this can creates a lot of new object in memory.
     */

    public List<TermHLAPI> getSubtermHLAPI();

    /**
     * This accessor automaticaly encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     */

    public SortHLAPI getOutputHLAPI();

    /**
     * This accessor automaticaly encapsulate all elements of the selected
     * sublist. WARNING : this can creates a lot of new object in memory.
     */

    public List<SortHLAPI> getInputHLAPI();

    // setters (including container setter if aviable)

    /**
     * set Sort
     */
    public void setSortHLAPI(SortHLAPI elem);

    /**
     * set Output
     */
    public void setOutputHLAPI(SortHLAPI elem);

    /**
     * set ContainerOperator
     */
    public void setContainerOperatorHLAPI(OperatorHLAPI elem);

    /**
     * set ContainerNamedOperator
     */
    public void setContainerNamedOperatorHLAPI(NamedOperatorHLAPI elem);

    /**
     * set ContainerHLMarking
     */
    public void setContainerHLMarkingHLAPI(HLMarkingHLAPI elem);

    /**
     * set ContainerCondition
     */
    public void setContainerConditionHLAPI(ConditionHLAPI elem);

    /**
     * set ContainerHLAnnotation
     */
    public void setContainerHLAnnotationHLAPI(HLAnnotationHLAPI elem);

    /**
     * set ContainerPartitionElement
     */
    public void setContainerPartitionElementHLAPI(PartitionElementHLAPI elem);

    /**
     * This accessor return a list of encapsulated subelement, only of
     * VariableHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.terms.hlapi.VariableHLAPI> getSubterm_terms_VariableHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * TupleHLAPI kind. WARNING : this method can creates a lot of new object in
     * memory.
     */
    public List<fr.lip6.move.pnml.hlpn.terms.hlapi.TupleHLAPI> getSubterm_terms_TupleHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * UserOperatorHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.terms.hlapi.UserOperatorHLAPI> getSubterm_terms_UserOperatorHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * EqualityHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.booleans.hlapi.EqualityHLAPI> getSubterm_booleans_EqualityHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * InequalityHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.booleans.hlapi.InequalityHLAPI> getSubterm_booleans_InequalityHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * BooleanConstantHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.booleans.hlapi.BooleanConstantHLAPI> getSubterm_booleans_BooleanConstantHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of OrHLAPI
     * kind. WARNING : this method can creates a lot of new object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.booleans.hlapi.OrHLAPI> getSubterm_booleans_OrHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of AndHLAPI
     * kind. WARNING : this method can creates a lot of new object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.booleans.hlapi.AndHLAPI> getSubterm_booleans_AndHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * ImplyHLAPI kind. WARNING : this method can creates a lot of new object in
     * memory.
     */
    public List<fr.lip6.move.pnml.hlpn.booleans.hlapi.ImplyHLAPI> getSubterm_booleans_ImplyHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of NotHLAPI
     * kind. WARNING : this method can creates a lot of new object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.booleans.hlapi.NotHLAPI> getSubterm_booleans_NotHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * SuccessorHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.cyclicEnumerations.hlapi.SuccessorHLAPI> getSubterm_cyclicEnumerations_SuccessorHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * PredecessorHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.cyclicEnumerations.hlapi.PredecessorHLAPI> getSubterm_cyclicEnumerations_PredecessorHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * DotConstantHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.dots.hlapi.DotConstantHLAPI> getSubterm_dots_DotConstantHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * FiniteIntRangeConstantHLAPI kind. WARNING : this method can creates a lot
     * of new object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.finiteIntRanges.hlapi.FiniteIntRangeConstantHLAPI> getSubterm_finiteIntRanges_FiniteIntRangeConstantHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * LessThanHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.finiteIntRanges.hlapi.LessThanHLAPI> getSubterm_finiteIntRanges_LessThanHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * GreaterThanHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.finiteIntRanges.hlapi.GreaterThanHLAPI> getSubterm_finiteIntRanges_GreaterThanHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * LessThanOrEqualHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.finiteIntRanges.hlapi.LessThanOrEqualHLAPI> getSubterm_finiteIntRanges_LessThanOrEqualHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * GreaterThanOrEqualHLAPI kind. WARNING : this method can creates a lot of
     * new object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.finiteIntRanges.hlapi.GreaterThanOrEqualHLAPI> getSubterm_finiteIntRanges_GreaterThanOrEqualHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * NumberConstantHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.integers.hlapi.NumberConstantHLAPI> getSubterm_integers_NumberConstantHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * AdditionHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.integers.hlapi.AdditionHLAPI> getSubterm_integers_AdditionHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * SubtractionHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.integers.hlapi.SubtractionHLAPI> getSubterm_integers_SubtractionHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * MultiplicationHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.integers.hlapi.MultiplicationHLAPI> getSubterm_integers_MultiplicationHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * DivisionHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.integers.hlapi.DivisionHLAPI> getSubterm_integers_DivisionHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * ModuloHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.integers.hlapi.ModuloHLAPI> getSubterm_integers_ModuloHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * GreaterThanHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.integers.hlapi.GreaterThanHLAPI> getSubterm_integers_GreaterThanHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * GreaterThanOrEqualHLAPI kind. WARNING : this method can creates a lot of
     * new object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.integers.hlapi.GreaterThanOrEqualHLAPI> getSubterm_integers_GreaterThanOrEqualHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * LessThanHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.integers.hlapi.LessThanHLAPI> getSubterm_integers_LessThanHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * LessThanOrEqualHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.integers.hlapi.LessThanOrEqualHLAPI> getSubterm_integers_LessThanOrEqualHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * EmptyListHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.lists.hlapi.EmptyListHLAPI> getSubterm_lists_EmptyListHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * AppendHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.lists.hlapi.AppendHLAPI> getSubterm_lists_AppendHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * ConcatenationHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.lists.hlapi.ConcatenationHLAPI> getSubterm_lists_ConcatenationHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * LengthHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.lists.hlapi.LengthHLAPI> getSubterm_lists_LengthHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * MakeListHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.lists.hlapi.MakeListHLAPI> getSubterm_lists_MakeListHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * MemberAtIndexHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.lists.hlapi.MemberAtIndexHLAPI> getSubterm_lists_MemberAtIndexHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * SublistHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.lists.hlapi.SublistHLAPI> getSubterm_lists_SublistHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * CardinalityHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.multisets.hlapi.CardinalityHLAPI> getSubterm_multisets_CardinalityHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * ContainsHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.multisets.hlapi.ContainsHLAPI> getSubterm_multisets_ContainsHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * CardinalityOfHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.multisets.hlapi.CardinalityOfHLAPI> getSubterm_multisets_CardinalityOfHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of AddHLAPI
     * kind. WARNING : this method can creates a lot of new object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.multisets.hlapi.AddHLAPI> getSubterm_multisets_AddHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of AllHLAPI
     * kind. WARNING : this method can creates a lot of new object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.multisets.hlapi.AllHLAPI> getSubterm_multisets_AllHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * EmptyHLAPI kind. WARNING : this method can creates a lot of new object in
     * memory.
     */
    public List<fr.lip6.move.pnml.hlpn.multisets.hlapi.EmptyHLAPI> getSubterm_multisets_EmptyHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * NumberOfHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.multisets.hlapi.NumberOfHLAPI> getSubterm_multisets_NumberOfHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * SubtractHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.multisets.hlapi.SubtractHLAPI> getSubterm_multisets_SubtractHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * ScalarProductHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.multisets.hlapi.ScalarProductHLAPI> getSubterm_multisets_ScalarProductHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * GreaterThanHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.partitions.hlapi.GreaterThanHLAPI> getSubterm_partitions_GreaterThanHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * PartitionElementOfHLAPI kind. WARNING : this method can creates a lot of
     * new object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.partitions.hlapi.PartitionElementOfHLAPI> getSubterm_partitions_PartitionElementOfHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * LessThanHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.partitions.hlapi.LessThanHLAPI> getSubterm_partitions_LessThanHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * StringConstantHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.strings.hlapi.StringConstantHLAPI> getSubterm_strings_StringConstantHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * AppendHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.strings.hlapi.AppendHLAPI> getSubterm_strings_AppendHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * ConcatenationHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.strings.hlapi.ConcatenationHLAPI> getSubterm_strings_ConcatenationHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * GreaterThanHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.strings.hlapi.GreaterThanHLAPI> getSubterm_strings_GreaterThanHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * GreaterThanOrEqualHLAPI kind. WARNING : this method can creates a lot of
     * new object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.strings.hlapi.GreaterThanOrEqualHLAPI> getSubterm_strings_GreaterThanOrEqualHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * LessThanHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.strings.hlapi.LessThanHLAPI> getSubterm_strings_LessThanHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * LessThanOrEqualHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.strings.hlapi.LessThanOrEqualHLAPI> getSubterm_strings_LessThanOrEqualHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * LengthHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.strings.hlapi.LengthHLAPI> getSubterm_strings_LengthHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * SubstringHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.strings.hlapi.SubstringHLAPI> getSubterm_strings_SubstringHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * MultisetSortHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.terms.hlapi.MultisetSortHLAPI> getInput_terms_MultisetSortHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * ProductSortHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.terms.hlapi.ProductSortHLAPI> getInput_terms_ProductSortHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * UserSortHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.terms.hlapi.UserSortHLAPI> getInput_terms_UserSortHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * AnySortHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.arbitrarydeclarations.hlapi.AnySortHLAPI> getInput_arbitrarydeclarations_AnySortHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of BoolHLAPI
     * kind. WARNING : this method can creates a lot of new object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.booleans.hlapi.BoolHLAPI> getInput_booleans_BoolHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * CyclicEnumerationHLAPI kind. WARNING : this method can creates a lot of
     * new object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.cyclicEnumerations.hlapi.CyclicEnumerationHLAPI> getInput_cyclicEnumerations_CyclicEnumerationHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of DotHLAPI
     * kind. WARNING : this method can creates a lot of new object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.dots.hlapi.DotHLAPI> getInput_dots_DotHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * FiniteEnumerationHLAPI kind. WARNING : this method can creates a lot of
     * new object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.finiteEnumerations.hlapi.FiniteEnumerationHLAPI> getInput_finiteEnumerations_FiniteEnumerationHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * FiniteIntRangeHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.finiteIntRanges.hlapi.FiniteIntRangeHLAPI> getInput_finiteIntRanges_FiniteIntRangeHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * NaturalHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.integers.hlapi.NaturalHLAPI> getInput_integers_NaturalHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * PositiveHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.integers.hlapi.PositiveHLAPI> getInput_integers_PositiveHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * HLIntegerHLAPI kind. WARNING : this method can creates a lot of new
     * object in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.integers.hlapi.HLIntegerHLAPI> getInput_integers_HLIntegerHLAPI();

    /**
     * This accessor return a list of encapsulated subelement, only of
     * HLPNListHLAPI kind. WARNING : this method can creates a lot of new object
     * in memory.
     */
    public List<fr.lip6.move.pnml.hlpn.lists.hlapi.HLPNListHLAPI> getInput_lists_HLPNListHLAPI();

    // setters/remover for lists.

    public void addSubtermHLAPI(TermHLAPI unit);

    public void removeSubtermHLAPI(TermHLAPI unit);

    // equals method
    public boolean equals(Object item);

}