/**
 * (C) University Pierre & Marie Curie - UMR CNRS 7606 (LIP6/MoVe)
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *  
 *  Contributors: 
 *    Lom HILLAH (LIP6) - Initial models and implementation
 *    Rachid Alahyane (UPMC) - Infrastructure and continuous integration
 *    Bastien Bouzerau (UPMC) - Architecture 
 *    Guillaume Giffo (UPMC) - Code generation refactoring, High-level API
 *
 * $Id$
 */
package fr.lip6.move.pnml.pnmlcoremodel;

import java.util.List;

import org.apache.axiom.om.OMElement;
import org.eclipse.emf.common.util.DiagnosticChain;

import fr.lip6.move.pnml.framework.utils.IdRefLinker;
import fr.lip6.move.pnml.framework.utils.exception.InnerBuildException;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;

/**
 * <!-- begin-user-doc --> A representation of the model object '<em><b>Node</b></em>'.
 * <!-- end-user-doc -->
 * 
 * <p>
 * The following features are supported:
 * <ul>
 * <li>{@link fr.lip6.move.pnml.pnmlcoremodel.Node#getInArcs <em>In Arcs</em>}</li>
 * <li>{@link fr.lip6.move.pnml.pnmlcoremodel.Node#getOutArcs <em>Out Arcs</em>}</li>
 * <li>{@link fr.lip6.move.pnml.pnmlcoremodel.Node#getNodegraphics <em>Nodegraphics</em>}</li>
 * </ul>
 * </p>
 * 
 * @see fr.lip6.move.pnml.pnmlcoremodel.PnmlcoremodelPackage#getNode()
 * @model abstract="true"
 * @generated
 */
public interface Node extends PnObject {
    /**
     * Returns the value of the '<em><b>In Arcs</b></em>' reference list.
     * The list contents are of type {@link fr.lip6.move.pnml.pnmlcoremodel.Arc}.
     * It is bidirectional and its opposite is '{@link fr.lip6.move.pnml.pnmlcoremodel.Arc#getTarget <em>Target</em>}'.
     * <!-- begin-user-doc -->
     * <p>
     * If the meaning of the '<em>In Arcs</em>' reference list isn't clear,
     * there really should be more of a description here...
     * </p>
     * <!-- end-user-doc -->
     * 
     * @return the value of the '<em>In Arcs</em>' reference list.
     * @see fr.lip6.move.pnml.pnmlcoremodel.PnmlcoremodelPackage#getNode_InArcs()
     * @see fr.lip6.move.pnml.pnmlcoremodel.Arc#getTarget
     * @model opposite="target" changeable="false" ordered="false"
     * @generated
     */
    List<Arc> getInArcs();

    /**
     * Returns the value of the '<em><b>Out Arcs</b></em>' reference list.
     * The list contents are of type {@link fr.lip6.move.pnml.pnmlcoremodel.Arc}.
     * It is bidirectional and its opposite is '{@link fr.lip6.move.pnml.pnmlcoremodel.Arc#getSource <em>Source</em>}'.
     * <!-- begin-user-doc -->
     * <p>
     * If the meaning of the '<em>Out Arcs</em>' reference list isn't clear,
     * there really should be more of a description here...
     * </p>
     * <!-- end-user-doc -->
     * 
     * @return the value of the '<em>Out Arcs</em>' reference list.
     * @see fr.lip6.move.pnml.pnmlcoremodel.PnmlcoremodelPackage#getNode_OutArcs()
     * @see fr.lip6.move.pnml.pnmlcoremodel.Arc#getSource
     * @model opposite="source" changeable="false" ordered="false"
     * @generated
     */
    List<Arc> getOutArcs();

    /**
     * Returns the value of the '<em><b>Nodegraphics</b></em>' containment
     * reference. It is bidirectional and its opposite is '{@link fr.lip6.move.pnml.pnmlcoremodel.NodeGraphics#getContainerNode <em>Container Node</em>}'.
     * <!-- begin-user-doc -->
     * <p>
     * If the meaning of the '<em>Nodegraphics</em>' containment reference
     * isn't clear, there really should be more of a description here...
     * </p>
     * <!-- end-user-doc -->
     * 
     * @return the value of the '<em>Nodegraphics</em>' containment
     *         reference.
     * @see #setNodegraphics(NodeGraphics)
     * @see fr.lip6.move.pnml.pnmlcoremodel.PnmlcoremodelPackage#getNode_Nodegraphics()
     * @see fr.lip6.move.pnml.pnmlcoremodel.NodeGraphics#getContainerNode
     * @model opposite="containerNode" containment="true"
     *        annotation="http://www.pnml.org/models/ToPNML kind='follow'"
     * @generated
     */
    NodeGraphics getNodegraphics();

    /**
     * Sets the value of the '{@link fr.lip6.move.pnml.pnmlcoremodel.Node#getNodegraphics <em>Nodegraphics</em>}'
     * containment reference. <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param value
     *            the new value of the '<em>Nodegraphics</em>' containment
     *            reference.
     * @see #getNodegraphics()
     * @generated
     */
    void setNodegraphics(NodeGraphics value);

    // public abstract void idRefHang(java.util.ArrayList<Object> oList);
    public abstract String toPNML();

    public abstract void fromPNML(OMElement locRoot, IdRefLinker idr)
            throws InnerBuildException, InvalidIDException,
            VoidRepositoryException;

    public abstract boolean validateOCL(DiagnosticChain diagnostics);
} // Node
