/**
 * (C) University Pierre & Marie Curie - UMR CNRS 7606 (LIP6/MoVe)
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *  
 *  Contributors: 
 *    Lom HILLAH (LIP6) - Initial models and implementation
 *    Rachid Alahyane (UPMC) - Infrastructure and continuous integration
 *    Bastien Bouzerau (UPMC) - Architecture 
 *    Guillaume Giffo (UPMC) - Code generation refactoring, High-level API
 *
 * $Id$
 */
package fr.lip6.move.pnml.pnmlcoremodel.impl;

import java.util.Iterator;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.eclipse.emf.common.util.DiagnosticChain;
import org.eclipse.emf.ecore.EClass;

import fr.lip6.move.pnml.framework.utils.IdRefLinker;
import fr.lip6.move.pnml.framework.utils.ModelRepository;
import fr.lip6.move.pnml.framework.utils.PrettyPrintData;
import fr.lip6.move.pnml.framework.utils.exception.InnerBuildException;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.pnmlcoremodel.Name;
import fr.lip6.move.pnml.pnmlcoremodel.NodeGraphics;
import fr.lip6.move.pnml.pnmlcoremodel.Place;
import fr.lip6.move.pnml.pnmlcoremodel.PnmlcoremodelFactory;
import fr.lip6.move.pnml.pnmlcoremodel.PnmlcoremodelPackage;
import fr.lip6.move.pnml.pnmlcoremodel.ToolInfo;
import fr.lip6.move.pnml.pnmlcoremodel.util.PnmlcoremodelValidator;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Place</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 * 
 * @generated
 */
public class PlaceImpl extends PlaceNodeImpl implements Place {
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected PlaceImpl() {
        super();
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    @Override
    protected EClass eStaticClass() {
        return PnmlcoremodelPackage.Literals.PLACE;
    }

    // TODO this element (InArcs) seems not to have any ToPNML associated tag.
    // This is maybe a mistake ?

    // TODO this element (OutArcs) seems not to have any ToPNML associated tag.
    // This is maybe a mistake ?

    // TODO this element (referencingPlaces) seems not to have any ToPNML
    // associated tag.
    // This is maybe a mistake ?

    /**
     * Return the string containing the pnml output
     */
    public String toPNML() {
        // id 1
        // idref 0
        // attributes 0
        // sons 3

        Boolean prettyPrintStatus = ModelRepository.getInstance()
                .isPrettyPrintActive();
        String retline = "";
        String headline = "";
        PrettyPrintData prpd = null;
        if (prettyPrintStatus) {
            retline = "\n";
            prpd = ModelRepository.getInstance().getPrettyPrintData();
            headline = prpd.getCurrentLineHeader();
        }

        StringBuffer sb = new StringBuffer();
        sb.append(headline);
        sb.append("<place");
        if (prettyPrintStatus) {
            headline = prpd.increaseLineHeaderLevel();
        }
        // begin attributes, id and id ref processing

        if (getId() != null) {
            sb.append(" id");
            sb.append("=\"");
            sb.append(getId());
            sb.append("\"");
        }

        boolean haveSons = false;
        sb.append(">");
        sb.append(retline);

        // sons, follow processing

        if (getName() != null) {

            String top = getName().toPNML();
            if (!top.equals("")) {

                sb.append(top);

                haveSons = true;
            }

        }

        if (getToolspecifics() != null) {

            java.util.List<fr.lip6.move.pnml.pnmlcoremodel.ToolInfo> items = getToolspecifics();
            for (Iterator<ToolInfo> iterator = items.iterator(); iterator
                    .hasNext();) {
                ToolInfo item = (ToolInfo) iterator.next();

                sb.append(item.toPNML());

                haveSons = true;
            }

        }

        if (getNodegraphics() != null) {

            String top = getNodegraphics().toPNML();
            if (!top.equals("")) {

                sb.append(top);

                haveSons = true;
            }

        }

        /** * */

        if (prettyPrintStatus) {
            headline = prpd.decreaseLineHeaderLevel();
        }

        if (haveSons) {
            sb.append(headline);
            sb.append("</place>");
            sb.append(retline);
        } else {
            sb.delete(sb.lastIndexOf(">"), sb.length());
            sb.append("/>");
        }

        return sb.toString();
    }

    @SuppressWarnings("unchecked")
    public void fromPNML(OMElement locRoot, IdRefLinker idr)
            throws InnerBuildException, InvalidIDException,
            VoidRepositoryException {
        // 1
        // 0
        // 0
        // 3
        @SuppressWarnings("unused")
        PnmlcoremodelFactory fact = new PnmlcoremodelFactoryImpl();

        // processing id

        if (locRoot.getAttributeValue(new QName("id")) != null) {
            this.setId(new java.lang.String(locRoot
                    .getAttributeValue(new QName("id"))));
            ModelRepository.getInstance().getCurrentIdRepository().checkId(
                    new java.lang.String(locRoot.getAttributeValue(new QName(
                            "id"))).toString(), (Object) this);
        }

        // processing idref

        // processing attributes

        // processing sons

        for (Iterator iterator = locRoot.getChildElements(); iterator.hasNext();) {
            @SuppressWarnings("unused")
            OMElement type = (OMElement) iterator.next();

            if (type.getLocalName().equals("name")) {
                Name item;
                item = new PnmlcoremodelFactoryImpl().createName();
                item.fromPNML(type, idr);

                item.setContainerNamePnObject(this);

                continue;
            }// end if

            if (type.getLocalName().equals("toolspecific")) {
                ToolInfo item;
                item = new PnmlcoremodelFactoryImpl().createToolInfo();
                item.fromPNML(type, idr);

                item.setContainerPnObject(this);

                continue;
            }// end if

            if (type.getLocalName().equals("graphics")) {
                NodeGraphics item;
                item = new PnmlcoremodelFactoryImpl().createNodeGraphics();
                item.fromPNML(type, idr);

                item.setContainerNode(this);

                continue;
            }// end if

        }

    }

    /**
     * -
     */
    public boolean validateOCL(DiagnosticChain diagnostics) {

        PnmlcoremodelValidator val = new PnmlcoremodelValidator();
        boolean retour = true;

        if (getName() != null) {
            retour &= getName().validateOCL(diagnostics);
        }

        if (getToolspecifics() != null) {
            java.util.List<fr.lip6.move.pnml.pnmlcoremodel.ToolInfo> items = getToolspecifics();
            for (Iterator<ToolInfo> iterator = items.iterator(); iterator
                    .hasNext();) {
                ToolInfo item = (ToolInfo) iterator.next();
                retour &= item.validateOCL(diagnostics);
            }
        }

        if (getNodegraphics() != null) {
            retour &= getNodegraphics().validateOCL(diagnostics);
        }

        retour &= val.validate(this, diagnostics, null);
        return retour;

    }
} // PlaceImpl
