/**
 * (C) University Pierre & Marie Curie - UMR CNRS 7606 (LIP6/MoVe)
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *  
 *  Contributors: 
 *    Lom HILLAH (LIP6) - Initial models and implementation
 *    Rachid Alahyane (UPMC) - Infrastructure and continuous integration
 *    Bastien Bouzerau (UPMC) - Architecture
 *    Guillaume Giffo (UPMC)  - Code generation refactoring, high-level API
 *
 * $Id ggiffo, Mon Sep 08 14:31:27 CEST 2008$
 */
package fr.lip6.move.pnml.pthlpng.booleans.hlapi;

import org.apache.axiom.om.OMElement;
import org.eclipse.emf.common.util.DiagnosticChain;

import fr.lip6.move.pnml.framework.hlapi.HLAPIClass;
import fr.lip6.move.pnml.framework.utils.IdRefLinker;
import fr.lip6.move.pnml.framework.utils.exception.InnerBuildException;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.pthlpng.booleans.Bool;
import fr.lip6.move.pnml.pthlpng.booleans.BooleansFactory;
import fr.lip6.move.pnml.pthlpng.booleans.impl.BooleansFactoryImpl;
import fr.lip6.move.pnml.pthlpng.hlcorestructure.Type;
import fr.lip6.move.pnml.pthlpng.hlcorestructure.hlapi.TypeHLAPI;
import fr.lip6.move.pnml.pthlpng.multisets.All;
import fr.lip6.move.pnml.pthlpng.multisets.Empty;
import fr.lip6.move.pnml.pthlpng.multisets.hlapi.AllHLAPI;
import fr.lip6.move.pnml.pthlpng.multisets.hlapi.EmptyHLAPI;
import fr.lip6.move.pnml.pthlpng.partitions.Partition;
import fr.lip6.move.pnml.pthlpng.partitions.hlapi.PartitionHLAPI;
import fr.lip6.move.pnml.pthlpng.terms.MultisetSort;
import fr.lip6.move.pnml.pthlpng.terms.NamedSort;
import fr.lip6.move.pnml.pthlpng.terms.ProductSort;
import fr.lip6.move.pnml.pthlpng.terms.VariableDecl;
import fr.lip6.move.pnml.pthlpng.terms.hlapi.MultisetSortHLAPI;
import fr.lip6.move.pnml.pthlpng.terms.hlapi.NamedSortHLAPI;
import fr.lip6.move.pnml.pthlpng.terms.hlapi.ProductSortHLAPI;
import fr.lip6.move.pnml.pthlpng.terms.hlapi.SortHLAPI;
import fr.lip6.move.pnml.pthlpng.terms.hlapi.VariableDeclHLAPI;

public class BoolHLAPI implements HLAPIClass, SortHLAPI {

	/**
	 * The contained LLAPI element.
	 */
	private Bool item;

	/**
	 * this constructor allows you to set all 'settable' values excepted
	 * container.
	 */

	public BoolHLAPI() {// BEGIN CONSTRUCTOR BODY
		BooleansFactory fact = new BooleansFactoryImpl();
		item = fact.createBool();

	}

	/**
	 * this constructor allows you to set all 'settable' values, including
	 * container if any.
	 */

	public BoolHLAPI(MultisetSortHLAPI multi) {// BEGIN CONSTRUCTOR BODY
		BooleansFactory fact = new BooleansFactoryImpl();
		item = fact.createBool();

		if (multi != null)
			item.setMulti((MultisetSort) multi.getContainedItem());

	}

	/**
	 * this constructor allows you to set all 'settable' values, including
	 * container if any.
	 */

	public BoolHLAPI(NamedSortHLAPI containerNamedSort) {// BEGIN CONSTRUCTOR
		// BODY
		BooleansFactory fact = new BooleansFactoryImpl();
		item = fact.createBool();

		if (containerNamedSort != null)
			item.setContainerNamedSort((NamedSort) containerNamedSort
					.getContainedItem());

	}

	/**
	 * this constructor allows you to set all 'settable' values, including
	 * container if any.
	 */

	public BoolHLAPI(VariableDeclHLAPI containerVariableDecl) {// BEGIN
		// CONSTRUCTOR
		// BODY
		BooleansFactory fact = new BooleansFactoryImpl();
		item = fact.createBool();

		if (containerVariableDecl != null)
			item.setContainerVariableDecl((VariableDecl) containerVariableDecl
					.getContainedItem());

	}

	/**
	 * this constructor allows you to set all 'settable' values, including
	 * container if any.
	 */

	public BoolHLAPI(ProductSortHLAPI containerProductSort) {// BEGIN
		// CONSTRUCTOR
		// BODY
		BooleansFactory fact = new BooleansFactoryImpl();
		item = fact.createBool();

		if (containerProductSort != null)
			item.setContainerProductSort((ProductSort) containerProductSort
					.getContainedItem());

	}

	/**
	 * this constructor allows you to set all 'settable' values, including
	 * container if any.
	 */

	public BoolHLAPI(TypeHLAPI containerType) {// BEGIN CONSTRUCTOR BODY
		BooleansFactory fact = new BooleansFactoryImpl();
		item = fact.createBool();

		if (containerType != null)
			item.setContainerType((Type) containerType.getContainedItem());

	}

	/**
	 * this constructor allows you to set all 'settable' values, including
	 * container if any.
	 */

	public BoolHLAPI(AllHLAPI containerAll) {// BEGIN CONSTRUCTOR BODY
		BooleansFactory fact = new BooleansFactoryImpl();
		item = fact.createBool();

		if (containerAll != null)
			item.setContainerAll((All) containerAll.getContainedItem());

	}

	/**
	 * this constructor allows you to set all 'settable' values, including
	 * container if any.
	 */

	public BoolHLAPI(EmptyHLAPI containerEmpty) {// BEGIN CONSTRUCTOR BODY
		BooleansFactory fact = new BooleansFactoryImpl();
		item = fact.createBool();

		if (containerEmpty != null)
			item.setContainerEmpty((Empty) containerEmpty.getContainedItem());

	}

	/**
	 * this constructor allows you to set all 'settable' values, including
	 * container if any.
	 */

	public BoolHLAPI(PartitionHLAPI containerPartition) {// BEGIN CONSTRUCTOR
		// BODY
		BooleansFactory fact = new BooleansFactoryImpl();
		item = fact.createBool();

		if (containerPartition != null)
			item.setContainerPartition((Partition) containerPartition
					.getContainedItem());

	}

	/**
	 * This constructor encapsulate a low level API object in HLAPI.
	 */
	public BoolHLAPI(Bool lowLevelAPI) {
		item = lowLevelAPI;
	}

	// access to low level API
	/**
	 * Return encapsulated object
	 */
	public Bool getContainedItem() {
		return item;
	}

	// getters giving LLAPI object

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public MultisetSort getMulti() {
		return item.getMulti();
	}

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public NamedSort getContainerNamedSort() {
		return item.getContainerNamedSort();
	}

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public VariableDecl getContainerVariableDecl() {
		return item.getContainerVariableDecl();
	}

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public ProductSort getContainerProductSort() {
		return item.getContainerProductSort();
	}

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public Type getContainerType() {
		return item.getContainerType();
	}

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public All getContainerAll() {
		return item.getContainerAll();
	}

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public Empty getContainerEmpty() {
		return item.getContainerEmpty();
	}

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public Partition getContainerPartition() {
		return item.getContainerPartition();
	}

	// getters giving HLAPI object

	/**
	 * This accessor automatically encapsulate an element of the current object.
	 * WARNING : this creates a new object in memory.
	 * 
	 * @return : null if the element is null
	 */

	public MultisetSortHLAPI getMultiHLAPI() {
		if (item.getMulti() == null)
			return null;
		return new MultisetSortHLAPI(item.getMulti());
	}

	/**
	 * This accessor automatically encapsulate an element of the current object.
	 * WARNING : this creates a new object in memory.
	 * 
	 * @return : null if the element is null
	 */

	public NamedSortHLAPI getContainerNamedSortHLAPI() {
		if (item.getContainerNamedSort() == null)
			return null;
		return new NamedSortHLAPI(item.getContainerNamedSort());
	}

	/**
	 * This accessor automatically encapsulate an element of the current object.
	 * WARNING : this creates a new object in memory.
	 * 
	 * @return : null if the element is null
	 */

	public VariableDeclHLAPI getContainerVariableDeclHLAPI() {
		if (item.getContainerVariableDecl() == null)
			return null;
		return new VariableDeclHLAPI(item.getContainerVariableDecl());
	}

	/**
	 * This accessor automatically encapsulate an element of the current object.
	 * WARNING : this creates a new object in memory.
	 * 
	 * @return : null if the element is null
	 */

	public ProductSortHLAPI getContainerProductSortHLAPI() {
		if (item.getContainerProductSort() == null)
			return null;
		return new ProductSortHLAPI(item.getContainerProductSort());
	}

	/**
	 * This accessor automatically encapsulate an element of the current object.
	 * WARNING : this creates a new object in memory.
	 * 
	 * @return : null if the element is null
	 */

	public TypeHLAPI getContainerTypeHLAPI() {
		if (item.getContainerType() == null)
			return null;
		return new TypeHLAPI(item.getContainerType());
	}

	/**
	 * This accessor automatically encapsulate an element of the current object.
	 * WARNING : this creates a new object in memory.
	 * 
	 * @return : null if the element is null
	 */

	public AllHLAPI getContainerAllHLAPI() {
		if (item.getContainerAll() == null)
			return null;
		return new AllHLAPI(item.getContainerAll());
	}

	/**
	 * This accessor automatically encapsulate an element of the current object.
	 * WARNING : this creates a new object in memory.
	 * 
	 * @return : null if the element is null
	 */

	public EmptyHLAPI getContainerEmptyHLAPI() {
		if (item.getContainerEmpty() == null)
			return null;
		return new EmptyHLAPI(item.getContainerEmpty());
	}

	/**
	 * This accessor automatically encapsulate an element of the current object.
	 * WARNING : this creates a new object in memory.
	 * 
	 * @return : null if the element is null
	 */

	public PartitionHLAPI getContainerPartitionHLAPI() {
		if (item.getContainerPartition() == null)
			return null;
		return new PartitionHLAPI(item.getContainerPartition());
	}

	// Special getter for list of generics object, return only one object type.

	// setters (including container setter if aviable)

	/**
	 * set Multi
	 */
	public void setMultiHLAPI(

	MultisetSortHLAPI elem) {

		if (elem != null)
			item.setMulti((MultisetSort) elem.getContainedItem());

	}

	/**
	 * set ContainerNamedSort
	 */
	public void setContainerNamedSortHLAPI(

	NamedSortHLAPI elem) {

		if (elem != null)
			item.setContainerNamedSort((NamedSort) elem.getContainedItem());

	}

	/**
	 * set ContainerVariableDecl
	 */
	public void setContainerVariableDeclHLAPI(

	VariableDeclHLAPI elem) {

		if (elem != null)
			item.setContainerVariableDecl((VariableDecl) elem
					.getContainedItem());

	}

	/**
	 * set ContainerProductSort
	 */
	public void setContainerProductSortHLAPI(

	ProductSortHLAPI elem) {

		if (elem != null)
			item.setContainerProductSort((ProductSort) elem.getContainedItem());

	}

	/**
	 * set ContainerType
	 */
	public void setContainerTypeHLAPI(

	TypeHLAPI elem) {

		if (elem != null)
			item.setContainerType((Type) elem.getContainedItem());

	}

	/**
	 * set ContainerAll
	 */
	public void setContainerAllHLAPI(

	AllHLAPI elem) {

		if (elem != null)
			item.setContainerAll((All) elem.getContainedItem());

	}

	/**
	 * set ContainerEmpty
	 */
	public void setContainerEmptyHLAPI(

	EmptyHLAPI elem) {

		if (elem != null)
			item.setContainerEmpty((Empty) elem.getContainedItem());

	}

	/**
	 * set ContainerPartition
	 */
	public void setContainerPartitionHLAPI(

	PartitionHLAPI elem) {

		if (elem != null)
			item.setContainerPartition((Partition) elem.getContainedItem());

	}

	// setters/remover for lists.

	// equals method
	public boolean equals(BoolHLAPI item) {
		return item.getContainedItem().equals(getContainedItem());
	}

	// PNML

	/**
	 * return the PNML xml tree for this object.
	 */
	public String toPNML() {
		return item.toPNML();
	}

	/**
	 * creates an object from the xml nodes.(symetric work of toPNML)
	 */
	public void fromPNML(OMElement subRoot, IdRefLinker idr)
			throws InnerBuildException, InvalidIDException,
			VoidRepositoryException {
		item.fromPNML(subRoot, idr);
	}

	public boolean validateOCL(DiagnosticChain diagnostics) {
		return item.validateOCL(diagnostics);
	}

}