/**
 * (C) University Pierre & Marie Curie - UMR CNRS 7606 (LIP6/MoVe)
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *  
 *  Contributors: 
 *    Lom HILLAH (LIP6) - Initial models and implementation
 *    Rachid Alahyane (UPMC) - Infrastructure and continuous integration
 *    Bastien Bouzerau (UPMC) - Architecture
 *    Guillaume Giffo (UPMC)  - Code generation refactoring, high-level API
 *
 * $Id ggiffo, Mon Sep 08 14:31:27 CEST 2008$
 */
package fr.lip6.move.pnml.pthlpng.terms.hlapi;

import org.apache.axiom.om.OMElement;
import org.eclipse.emf.common.util.DiagnosticChain;

import fr.lip6.move.pnml.framework.hlapi.HLAPIClass;
import fr.lip6.move.pnml.framework.utils.IdRefLinker;
import fr.lip6.move.pnml.framework.utils.ModelRepository;
import fr.lip6.move.pnml.framework.utils.exception.InnerBuildException;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.OtherException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.pthlpng.terms.Declarations;
import fr.lip6.move.pnml.pthlpng.terms.NamedSort;
import fr.lip6.move.pnml.pthlpng.terms.Sort;
import fr.lip6.move.pnml.pthlpng.terms.TermsFactory;
import fr.lip6.move.pnml.pthlpng.terms.impl.TermsFactoryImpl;

public class NamedSortHLAPI implements HLAPIClass, TermsDeclarationHLAPI,
		SortDeclHLAPI {

	/**
	 * The contained LLAPI element.
	 */
	private NamedSort item;

	/**
	 * this constructor allows you to set all 'settable' values excepted
	 * container.
	 */

	public NamedSortHLAPI(java.lang.String id

	, java.lang.String name

	, SortHLAPI sortdef) throws InvalidIDException, VoidRepositoryException {// BEGIN
		// CONSTRUCTOR
		// BODY
		TermsFactory fact = new TermsFactoryImpl();
		item = fact.createNamedSort();

		if (id != null) {

			item.setId(ModelRepository.getInstance().getCurrentIdRepository()
					.checkId(id, this));
		}

		if (name != null) {

			item.setName(name);
		}

		if (sortdef != null)
			item.setSortdef((Sort) sortdef.getContainedItem());

	}

	/**
	 * this constructor allows you to set all 'settable' values, including
	 * container if any.
	 */

	public NamedSortHLAPI(java.lang.String id

	, java.lang.String name

	, SortHLAPI sortdef

	, DeclarationsHLAPI containerDeclarations) throws InvalidIDException,
			VoidRepositoryException {// BEGIN CONSTRUCTOR BODY
		TermsFactory fact = new TermsFactoryImpl();
		item = fact.createNamedSort();

		if (id != null) {

			item.setId(ModelRepository.getInstance().getCurrentIdRepository()
					.checkId(id, this));
		}

		if (name != null) {

			item.setName(name);
		}

		if (sortdef != null)
			item.setSortdef((Sort) sortdef.getContainedItem());

		if (containerDeclarations != null)
			item.setContainerDeclarations((Declarations) containerDeclarations
					.getContainedItem());

	}

	/**
	 * This constructor encapsulate a low level API object in HLAPI.
	 */
	public NamedSortHLAPI(NamedSort lowLevelAPI) {
		item = lowLevelAPI;
	}

	// access to low level API
	/**
	 * Return encapsulated object
	 */
	public NamedSort getContainedItem() {
		return item;
	}

	// getters giving LLAPI object

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public String getId() {
		return item.getId();
	}

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public String getName() {
		return item.getName();
	}

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public Declarations getContainerDeclarations() {
		return item.getContainerDeclarations();
	}

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public Sort getSortdef() {
		return item.getSortdef();
	}

	// getters giving HLAPI object

	/**
	 * This accessor automatically encapsulate an element of the current object.
	 * WARNING : this creates a new object in memory.
	 * 
	 * @return : null if the element is null
	 */

	public DeclarationsHLAPI getContainerDeclarationsHLAPI() {
		if (item.getContainerDeclarations() == null)
			return null;
		return new DeclarationsHLAPI(item.getContainerDeclarations());
	}

	/**
	 * This accessor automatically encapsulate an element of the current object.
	 * WARNING : this creates a new object in memory.
	 * 
	 * @return : null if the element is null
	 */

	public SortHLAPI getSortdefHLAPI() {
		if (item.getSortdef() == null)
			return null;
		Sort object = item.getSortdef();

		if (object.getClass().equals(
				fr.lip6.move.pnml.pthlpng.terms.impl.MultisetSortImpl.class)) {
			return new fr.lip6.move.pnml.pthlpng.terms.hlapi.MultisetSortHLAPI(
					(fr.lip6.move.pnml.pthlpng.terms.MultisetSort) object);
		}

		if (object.getClass().equals(
				fr.lip6.move.pnml.pthlpng.terms.impl.ProductSortImpl.class)) {
			return new fr.lip6.move.pnml.pthlpng.terms.hlapi.ProductSortHLAPI(
					(fr.lip6.move.pnml.pthlpng.terms.ProductSort) object);
		}

		if (object.getClass().equals(
				fr.lip6.move.pnml.pthlpng.terms.impl.UserSortImpl.class)) {
			return new fr.lip6.move.pnml.pthlpng.terms.hlapi.UserSortHLAPI(
					(fr.lip6.move.pnml.pthlpng.terms.UserSort) object);
		}

		if (object.getClass().equals(
				fr.lip6.move.pnml.pthlpng.booleans.impl.BoolImpl.class)) {
			return new fr.lip6.move.pnml.pthlpng.booleans.hlapi.BoolHLAPI(
					(fr.lip6.move.pnml.pthlpng.booleans.Bool) object);
		}

		if (object.getClass().equals(
				fr.lip6.move.pnml.pthlpng.dots.impl.DotImpl.class)) {
			return new fr.lip6.move.pnml.pthlpng.dots.hlapi.DotHLAPI(
					(fr.lip6.move.pnml.pthlpng.dots.Dot) object);
		}

		return null;
	}

	// Special getter for list of generics object, return only one object type.

	// setters (including container setter if aviable)

	/**
	 * set Id
	 */
	public void setIdHLAPI(

	java.lang.String elem) throws InvalidIDException, VoidRepositoryException {

		if (elem != null) {

			try {
				item.setId(ModelRepository.getInstance()
						.getCurrentIdRepository().changeId(this, elem));
			} catch (OtherException e) {
				ModelRepository.getInstance().getCurrentIdRepository().checkId(
						elem, this);
			}
		}

	}

	/**
	 * set Name
	 */
	public void setNameHLAPI(

	java.lang.String elem) {

		if (elem != null) {

			item.setName(elem);
		}

	}

	/**
	 * set Sortdef
	 */
	public void setSortdefHLAPI(

	SortHLAPI elem) {

		if (elem != null)
			item.setSortdef((Sort) elem.getContainedItem());

	}

	/**
	 * set ContainerDeclarations
	 */
	public void setContainerDeclarationsHLAPI(

	DeclarationsHLAPI elem) {

		if (elem != null)
			item.setContainerDeclarations((Declarations) elem
					.getContainedItem());

	}

	// setters/remover for lists.

	// equals method
	public boolean equals(NamedSortHLAPI item) {
		return item.getContainedItem().equals(getContainedItem());
	}

	// PNML

	/**
	 * return the PNML xml tree for this object.
	 */
	public String toPNML() {
		return item.toPNML();
	}

	/**
	 * creates an object from the xml nodes.(symetric work of toPNML)
	 */
	public void fromPNML(OMElement subRoot, IdRefLinker idr)
			throws InnerBuildException, InvalidIDException,
			VoidRepositoryException {
		item.fromPNML(subRoot, idr);
	}

	public boolean validateOCL(DiagnosticChain diagnostics) {
		return item.validateOCL(diagnostics);
	}

}