/**
 * (C) University Pierre & Marie Curie - UMR CNRS 7606 (LIP6/MoVe)
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *  
 *  Contributors: 
 *    Lom HILLAH (LIP6) - Initial models and implementation
 *    Rachid Alahyane (UPMC) - Infrastructure and continuous integration
 *    Bastien Bouzerau (UPMC) - Architecture
 *    Guillaume Giffo (UPMC)  - Code generation refactoring, high-level API
 *
 * $Id ggiffo, Wed Sep 10 13:41:08 CEST 2008$
 */
package fr.lip6.move.pnml.ptnet.hlapi;

import java.util.ArrayList;
import java.util.List;

import org.apache.axiom.om.OMElement;
import org.eclipse.emf.common.util.DiagnosticChain;

import fr.lip6.move.pnml.framework.hlapi.HLAPIClass;
import fr.lip6.move.pnml.framework.utils.IdRefLinker;
import fr.lip6.move.pnml.framework.utils.exception.InnerBuildException;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.ptnet.Arc;
import fr.lip6.move.pnml.ptnet.ArcGraphics;
import fr.lip6.move.pnml.ptnet.Line;
import fr.lip6.move.pnml.ptnet.Position;
import fr.lip6.move.pnml.ptnet.PtnetFactory;
import fr.lip6.move.pnml.ptnet.impl.PtnetFactoryImpl;

public class ArcGraphicsHLAPI implements HLAPIClass, GraphicsHLAPI {

    /**
     * The contained LLAPI element.
     */
    private ArcGraphics item;

    /**
     * this constructor allows you to set all 'settable' values excepted
     * container.
     */

    public ArcGraphicsHLAPI(LineHLAPI line) {// BEGIN CONSTRUCTOR BODY
        PtnetFactory fact = new PtnetFactoryImpl();
        item = fact.createArcGraphics();

        if (line != null)
            item.setLine((Line) line.getContainedItem());

    }

    /**
     * this constructor allows you to set all 'settable' values, including
     * container if any.
     */

    public ArcGraphicsHLAPI(LineHLAPI line

    , ArcHLAPI containerArc) {// BEGIN CONSTRUCTOR BODY
        PtnetFactory fact = new PtnetFactoryImpl();
        item = fact.createArcGraphics();

        if (line != null)
            item.setLine((Line) line.getContainedItem());

        if (containerArc != null)
            item.setContainerArc((Arc) containerArc.getContainedItem());

    }

    /**
     * This constructor give access to required stuff only (and container)
     */
    public ArcGraphicsHLAPI(ArcHLAPI containerArc) {// BEGIN CONSTRUCTOR BODY
        PtnetFactory fact = new PtnetFactoryImpl();
        item = fact.createArcGraphics();

        if (containerArc != null)
            item.setContainerArc((Arc) containerArc.getContainedItem());

    }

    /**
     * This constructor encapsulate a low level API object in HLAPI.
     */
    public ArcGraphicsHLAPI(ArcGraphics lowLevelAPI) {
        item = lowLevelAPI;
    }

    // access to low level API
    /**
     * Return encapsulated object
     */
    public ArcGraphics getContainedItem() {
        return item;
    }

    // getters giving LLAPI object

    /**
     * Return the encapsulate Low Level API object.
     */
    public List<Position> getPositions() {
        return item.getPositions();
    }

    /**
     * Return the encapsulate Low Level API object.
     */
    public Line getLine() {
        return item.getLine();
    }

    /**
     * Return the encapsulate Low Level API object.
     */
    public Arc getContainerArc() {
        return item.getContainerArc();
    }

    // getters giving HLAPI object

    /**
     * This accessor automatically encapsulate all elements of the selected
     * sublist. WARNING : this can creates a lot of new object in memory.
     */

    public List<PositionHLAPI> getPositionsHLAPI() {
        List<PositionHLAPI> retour = new ArrayList<PositionHLAPI>();
        for (Position elemnt : getPositions()) {
            retour.add(new PositionHLAPI(elemnt));
        }
        return retour;
    }

    /**
     * This accessor automatically encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     * 
     * @return : null if the element is null
     */

    public LineHLAPI getLineHLAPI() {
        if (item.getLine() == null)
            return null;
        return new LineHLAPI(item.getLine());
    }

    /**
     * This accessor automatically encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     * 
     * @return : null if the element is null
     */

    public ArcHLAPI getContainerArcHLAPI() {
        if (item.getContainerArc() == null)
            return null;
        return new ArcHLAPI(item.getContainerArc());
    }

    // Special getter for list of generics object, return only one object type.

    // setters (including container setter if aviable)

    /**
     * set Line
     */
    public void setLineHLAPI(

    LineHLAPI elem) {

        if (elem != null)
            item.setLine((Line) elem.getContainedItem());

    }

    /**
     * set ContainerArc
     */
    public void setContainerArcHLAPI(

    ArcHLAPI elem) {

        if (elem != null)
            item.setContainerArc((Arc) elem.getContainedItem());

    }

    // setters/remover for lists.

    public void addPositionsHLAPI(PositionHLAPI unit) {
        item.getPositions().add((Position) unit.getContainedItem());
    }

    public void removePositionsHLAPI(PositionHLAPI unit) {
        item.getPositions().remove((Position) unit.getContainedItem());
    }

    // equals method
    public boolean equals(ArcGraphicsHLAPI item) {
        return item.getContainedItem().equals(getContainedItem());
    }

    // PNML

    /**
     * return the PNML xml tree for this object.
     */
    public String toPNML() {
        return item.toPNML();
    }

    /**
     * creates an object from the xml nodes.(symetric work of toPNML)
     */
    public void fromPNML(OMElement subRoot, IdRefLinker idr)
            throws InnerBuildException, InvalidIDException,
            VoidRepositoryException {
        item.fromPNML(subRoot, idr);
    }

    public boolean validateOCL(DiagnosticChain diagnostics) {
        return item.validateOCL(diagnostics);
    }

}