/**
 * (C) University Pierre & Marie Curie - UMR CNRS 7606 (LIP6/MoVe)
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *  
 *  Contributors: 
 *    Lom HILLAH (LIP6) - Initial models and implementation
 *    Rachid Alahyane (UPMC) - Infrastructure and continuous integration
 *    Bastien Bouzerau (UPMC) - Architecture
 *    Guillaume Giffo (UPMC)  - Code generation refactoring, high-level API
 *
 * $Id ggiffo, Wed Sep 10 13:41:06 CEST 2008$
 */
package fr.lip6.move.pnml.ptnet.hlapi;

import java.util.ArrayList;
import java.util.List;

import org.apache.axiom.om.OMElement;
import org.eclipse.emf.common.util.DiagnosticChain;

import fr.lip6.move.pnml.framework.hlapi.HLAPIClass;
import fr.lip6.move.pnml.framework.utils.IdRefLinker;
import fr.lip6.move.pnml.framework.utils.exception.InnerBuildException;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.ptnet.AnnotationGraphics;
import fr.lip6.move.pnml.ptnet.PTMarking;
import fr.lip6.move.pnml.ptnet.Place;
import fr.lip6.move.pnml.ptnet.PtnetFactory;
import fr.lip6.move.pnml.ptnet.ToolInfo;
import fr.lip6.move.pnml.ptnet.impl.PtnetFactoryImpl;

public class PTMarkingHLAPI implements HLAPIClass, LabelHLAPI, AnnotationHLAPI {

    /**
     * The contained LLAPI element.
     */
    private PTMarking item;

    /**
     * this constructor allows you to set all 'settable' values excepted
     * container.
     */

    public PTMarkingHLAPI(AnnotationGraphicsHLAPI annotationgraphics

    , java.lang.Integer text) {// BEGIN CONSTRUCTOR BODY
        PtnetFactory fact = new PtnetFactoryImpl();
        item = fact.createPTMarking();

        if (annotationgraphics != null)
            item.setAnnotationgraphics((AnnotationGraphics) annotationgraphics
                    .getContainedItem());

        if (text != null) {

            item.setText(text);
        }

    }

    /**
     * this constructor allows you to set all 'settable' values, including
     * container if any.
     */

    public PTMarkingHLAPI(AnnotationGraphicsHLAPI annotationgraphics

    , java.lang.Integer text

    , PlaceHLAPI containerPlace) {// BEGIN CONSTRUCTOR BODY
        PtnetFactory fact = new PtnetFactoryImpl();
        item = fact.createPTMarking();

        if (annotationgraphics != null)
            item.setAnnotationgraphics((AnnotationGraphics) annotationgraphics
                    .getContainedItem());

        if (text != null) {

            item.setText(text);
        }

        if (containerPlace != null)
            item.setContainerPlace((Place) containerPlace.getContainedItem());

    }

    /**
     * This constructor give access to required stuff only (not container if
     * any)
     */
    public PTMarkingHLAPI(java.lang.Integer text) {// BEGIN CONSTRUCTOR BODY
        PtnetFactory fact = new PtnetFactoryImpl();
        item = fact.createPTMarking();

        if (text != null) {

            item.setText(text);
        }

    }

    /**
     * This constructor give access to required stuff only (and container)
     */
    public PTMarkingHLAPI(java.lang.Integer text

    , PlaceHLAPI containerPlace) {// BEGIN CONSTRUCTOR BODY
        PtnetFactory fact = new PtnetFactoryImpl();
        item = fact.createPTMarking();

        if (text != null) {

            item.setText(text);
        }

        if (containerPlace != null)
            item.setContainerPlace((Place) containerPlace.getContainedItem());

    }

    /**
     * This constructor encapsulate a low level API object in HLAPI.
     */
    public PTMarkingHLAPI(PTMarking lowLevelAPI) {
        item = lowLevelAPI;
    }

    // access to low level API
    /**
     * Return encapsulated object
     */
    public PTMarking getContainedItem() {
        return item;
    }

    // getters giving LLAPI object

    /**
     * Return the encapsulate Low Level API object.
     */
    public List<ToolInfo> getToolspecifics() {
        return item.getToolspecifics();
    }

    /**
     * Return the encapsulate Low Level API object.
     */
    public AnnotationGraphics getAnnotationgraphics() {
        return item.getAnnotationgraphics();
    }

    /**
     * Return the encapsulate Low Level API object.
     */
    public Integer getText() {
        return item.getText();
    }

    /**
     * Return the encapsulate Low Level API object.
     */
    public Place getContainerPlace() {
        return item.getContainerPlace();
    }

    // getters giving HLAPI object

    /**
     * This accessor automatically encapsulate all elements of the selected
     * sublist. WARNING : this can creates a lot of new object in memory.
     */

    public List<ToolInfoHLAPI> getToolspecificsHLAPI() {
        List<ToolInfoHLAPI> retour = new ArrayList<ToolInfoHLAPI>();
        for (ToolInfo elemnt : getToolspecifics()) {
            retour.add(new ToolInfoHLAPI(elemnt));
        }
        return retour;
    }

    /**
     * This accessor automatically encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     * 
     * @return : null if the element is null
     */

    public AnnotationGraphicsHLAPI getAnnotationgraphicsHLAPI() {
        if (item.getAnnotationgraphics() == null)
            return null;
        return new AnnotationGraphicsHLAPI(item.getAnnotationgraphics());
    }

    /**
     * This accessor automatically encapsulate an element of the current object.
     * WARNING : this creates a new object in memory.
     * 
     * @return : null if the element is null
     */

    public PlaceHLAPI getContainerPlaceHLAPI() {
        if (item.getContainerPlace() == null)
            return null;
        return new PlaceHLAPI(item.getContainerPlace());
    }

    // Special getter for list of generics object, return only one object type.

    // setters (including container setter if aviable)

    /**
     * set Annotationgraphics
     */
    public void setAnnotationgraphicsHLAPI(

    AnnotationGraphicsHLAPI elem) {

        if (elem != null)
            item.setAnnotationgraphics((AnnotationGraphics) elem
                    .getContainedItem());

    }

    /**
     * set Text
     */
    public void setTextHLAPI(

    java.lang.Integer elem) {

        if (elem != null) {

            item.setText(elem);
        }

    }

    /**
     * set ContainerPlace
     */
    public void setContainerPlaceHLAPI(

    PlaceHLAPI elem) {

        if (elem != null)
            item.setContainerPlace((Place) elem.getContainedItem());

    }

    // setters/remover for lists.

    public void addToolspecificsHLAPI(ToolInfoHLAPI unit) {
        item.getToolspecifics().add((ToolInfo) unit.getContainedItem());
    }

    public void removeToolspecificsHLAPI(ToolInfoHLAPI unit) {
        item.getToolspecifics().remove((ToolInfo) unit.getContainedItem());
    }

    // equals method
    public boolean equals(PTMarkingHLAPI item) {
        return item.getContainedItem().equals(getContainedItem());
    }

    // PNML

    /**
     * return the PNML xml tree for this object.
     */
    public String toPNML() {
        return item.toPNML();
    }

    /**
     * creates an object from the xml nodes.(symetric work of toPNML)
     */
    public void fromPNML(OMElement subRoot, IdRefLinker idr)
            throws InnerBuildException, InvalidIDException,
            VoidRepositoryException {
        item.fromPNML(subRoot, idr);
    }

    public boolean validateOCL(DiagnosticChain diagnostics) {
        return item.validateOCL(diagnostics);
    }

}