/**
 * (C) University Pierre & Marie Curie - UMR CNRS 7606 (LIP6/MoVe)
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *  
 *  Contributors: 
 *    Lom HILLAH (LIP6) - Initial models and implementation
 *    Rachid Alahyane (UPMC) - Infrastructure and continuous integration
 *    Bastien Bouzerau (UPMC) - Architecture
 *    Guillaume Giffo (UPMC)  - Code generation refactoring, high-level API
 *
 * $Id ggiffo, Wed Sep 10 13:41:15 CEST 2008$
 */
package fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi;

import java.util.ArrayList;
import java.util.List;

import org.apache.axiom.om.OMElement;
import org.eclipse.emf.common.util.DiagnosticChain;

import fr.lip6.move.pnml.framework.hlapi.HLAPIClass;
import fr.lip6.move.pnml.framework.utils.IdRefLinker;
import fr.lip6.move.pnml.framework.utils.exception.InnerBuildException;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.AnnotationGraphics;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.HlcorestructureFactory;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.Place;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.ToolInfo;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.Type;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.impl.HlcorestructureFactoryImpl;
import fr.lip6.move.pnml.symmetricnet.terms.Sort;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.SortHLAPI;

public class TypeHLAPI implements HLAPIClass, LabelHLAPI, AnnotationHLAPI {

	/**
	 * The contained LLAPI element.
	 */
	private Type item;

	/**
	 * this constructor allows you to set all 'settable' values excepted
	 * container.
	 */

	public TypeHLAPI(AnnotationGraphicsHLAPI annotationgraphics

	, java.lang.String text

	, SortHLAPI structure) {// BEGIN CONSTRUCTOR BODY
		HlcorestructureFactory fact = new HlcorestructureFactoryImpl();
		item = fact.createType();

		if (annotationgraphics != null)
			item.setAnnotationgraphics((AnnotationGraphics) annotationgraphics
					.getContainedItem());

		if (text != null) {

			item.setText(text);
		}

		if (structure != null)
			item.setStructure((Sort) structure.getContainedItem());

	}

	/**
	 * this constructor allows you to set all 'settable' values, including
	 * container if any.
	 */

	public TypeHLAPI(AnnotationGraphicsHLAPI annotationgraphics

	, java.lang.String text

	, SortHLAPI structure

	, PlaceHLAPI containerPlace) {// BEGIN CONSTRUCTOR BODY
		HlcorestructureFactory fact = new HlcorestructureFactoryImpl();
		item = fact.createType();

		if (annotationgraphics != null)
			item.setAnnotationgraphics((AnnotationGraphics) annotationgraphics
					.getContainedItem());

		if (text != null) {

			item.setText(text);
		}

		if (structure != null)
			item.setStructure((Sort) structure.getContainedItem());

		if (containerPlace != null)
			item.setContainerPlace((Place) containerPlace.getContainedItem());

	}

	/**
	 * This constructor give access to required stuff only (and container)
	 */
	public TypeHLAPI(PlaceHLAPI containerPlace) {// BEGIN CONSTRUCTOR BODY
		HlcorestructureFactory fact = new HlcorestructureFactoryImpl();
		item = fact.createType();

		if (containerPlace != null)
			item.setContainerPlace((Place) containerPlace.getContainedItem());

	}

	/**
	 * This constructor encapsulate a low level API object in HLAPI.
	 */
	public TypeHLAPI(Type lowLevelAPI) {
		item = lowLevelAPI;
	}

	// access to low level API
	/**
	 * Return encapsulated object
	 */
	public Type getContainedItem() {
		return item;
	}

	// getters giving LLAPI object

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public List<ToolInfo> getToolspecifics() {
		return item.getToolspecifics();
	}

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public AnnotationGraphics getAnnotationgraphics() {
		return item.getAnnotationgraphics();
	}

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public String getText() {
		return item.getText();
	}

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public Sort getStructure() {
		return item.getStructure();
	}

	/**
	 * Return the encapsulate Low Level API object.
	 */
	public Place getContainerPlace() {
		return item.getContainerPlace();
	}

	// getters giving HLAPI object

	/**
	 * This accessor automatically encapsulate all elements of the selected
	 * sublist. WARNING : this can creates a lot of new object in memory.
	 */

	public List<ToolInfoHLAPI> getToolspecificsHLAPI() {
		List<ToolInfoHLAPI> retour = new ArrayList<ToolInfoHLAPI>();
		for (ToolInfo elemnt : getToolspecifics()) {
			retour.add(new ToolInfoHLAPI(elemnt));
		}
		return retour;
	}

	/**
	 * This accessor automatically encapsulate an element of the current object.
	 * WARNING : this creates a new object in memory.
	 * 
	 * @return : null if the element is null
	 */

	public AnnotationGraphicsHLAPI getAnnotationgraphicsHLAPI() {
		if (item.getAnnotationgraphics() == null)
			return null;
		return new AnnotationGraphicsHLAPI(item.getAnnotationgraphics());
	}

	/**
	 * This accessor automatically encapsulate an element of the current object.
	 * WARNING : this creates a new object in memory.
	 * 
	 * @return : null if the element is null
	 */

	public SortHLAPI getStructureHLAPI() {
		if (item.getStructure() == null)
			return null;
		Sort object = item.getStructure();

		if (object.getClass().equals(
				fr.lip6.move.pnml.symmetricnet.booleans.impl.BoolImpl.class)) {
			return new fr.lip6.move.pnml.symmetricnet.booleans.hlapi.BoolHLAPI(
					(fr.lip6.move.pnml.symmetricnet.booleans.Bool) object);
		}

		if (object
				.getClass()
				.equals(
						fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.impl.CyclicEnumerationImpl.class)) {
			return new fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.hlapi.CyclicEnumerationHLAPI(
					(fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.CyclicEnumeration) object);
		}

		if (object.getClass().equals(
				fr.lip6.move.pnml.symmetricnet.dots.impl.DotImpl.class)) {
			return new fr.lip6.move.pnml.symmetricnet.dots.hlapi.DotHLAPI(
					(fr.lip6.move.pnml.symmetricnet.dots.Dot) object);
		}

		if (object
				.getClass()
				.equals(
						fr.lip6.move.pnml.symmetricnet.finiteEnumerations.impl.FiniteEnumerationImpl.class)) {
			return new fr.lip6.move.pnml.symmetricnet.finiteEnumerations.hlapi.FiniteEnumerationHLAPI(
					(fr.lip6.move.pnml.symmetricnet.finiteEnumerations.FiniteEnumeration) object);
		}

		if (object
				.getClass()
				.equals(
						fr.lip6.move.pnml.symmetricnet.finiteIntRanges.impl.FiniteIntRangeImpl.class)) {
			return new fr.lip6.move.pnml.symmetricnet.finiteIntRanges.hlapi.FiniteIntRangeHLAPI(
					(fr.lip6.move.pnml.symmetricnet.finiteIntRanges.FiniteIntRange) object);
		}

		if (object.getClass().equals(
				fr.lip6.move.pnml.symmetricnet.integers.impl.NaturalImpl.class)) {
			return new fr.lip6.move.pnml.symmetricnet.integers.hlapi.NaturalHLAPI(
					(fr.lip6.move.pnml.symmetricnet.integers.Natural) object);
		}

		if (object
				.getClass()
				.equals(
						fr.lip6.move.pnml.symmetricnet.integers.impl.PositiveImpl.class)) {
			return new fr.lip6.move.pnml.symmetricnet.integers.hlapi.PositiveHLAPI(
					(fr.lip6.move.pnml.symmetricnet.integers.Positive) object);
		}

		if (object
				.getClass()
				.equals(
						fr.lip6.move.pnml.symmetricnet.integers.impl.HLIntegerImpl.class)) {
			return new fr.lip6.move.pnml.symmetricnet.integers.hlapi.HLIntegerHLAPI(
					(fr.lip6.move.pnml.symmetricnet.integers.HLInteger) object);
		}

		if (object
				.getClass()
				.equals(
						fr.lip6.move.pnml.symmetricnet.terms.impl.MultisetSortImpl.class)) {
			return new fr.lip6.move.pnml.symmetricnet.terms.hlapi.MultisetSortHLAPI(
					(fr.lip6.move.pnml.symmetricnet.terms.MultisetSort) object);
		}

		if (object
				.getClass()
				.equals(
						fr.lip6.move.pnml.symmetricnet.terms.impl.ProductSortImpl.class)) {
			return new fr.lip6.move.pnml.symmetricnet.terms.hlapi.ProductSortHLAPI(
					(fr.lip6.move.pnml.symmetricnet.terms.ProductSort) object);
		}

		if (object.getClass().equals(
				fr.lip6.move.pnml.symmetricnet.terms.impl.UserSortImpl.class)) {
			return new fr.lip6.move.pnml.symmetricnet.terms.hlapi.UserSortHLAPI(
					(fr.lip6.move.pnml.symmetricnet.terms.UserSort) object);
		}

		return null;
	}

	/**
	 * This accessor automatically encapsulate an element of the current object.
	 * WARNING : this creates a new object in memory.
	 * 
	 * @return : null if the element is null
	 */

	public PlaceHLAPI getContainerPlaceHLAPI() {
		if (item.getContainerPlace() == null)
			return null;
		return new PlaceHLAPI(item.getContainerPlace());
	}

	// Special getter for list of generics object, return only one object type.

	// setters (including container setter if aviable)

	/**
	 * set Annotationgraphics
	 */
	public void setAnnotationgraphicsHLAPI(

	AnnotationGraphicsHLAPI elem) {

		if (elem != null)
			item.setAnnotationgraphics((AnnotationGraphics) elem
					.getContainedItem());

	}

	/**
	 * set Text
	 */
	public void setTextHLAPI(

	java.lang.String elem) {

		if (elem != null) {

			item.setText(elem);
		}

	}

	/**
	 * set Structure
	 */
	public void setStructureHLAPI(

	SortHLAPI elem) {

		if (elem != null)
			item.setStructure((Sort) elem.getContainedItem());

	}

	/**
	 * set ContainerPlace
	 */
	public void setContainerPlaceHLAPI(

	PlaceHLAPI elem) {

		if (elem != null)
			item.setContainerPlace((Place) elem.getContainedItem());

	}

	// setters/remover for lists.

	public void addToolspecificsHLAPI(ToolInfoHLAPI unit) {
		item.getToolspecifics().add((ToolInfo) unit.getContainedItem());
	}

	public void removeToolspecificsHLAPI(ToolInfoHLAPI unit) {
		item.getToolspecifics().remove((ToolInfo) unit.getContainedItem());
	}

	// equals method
	public boolean equals(TypeHLAPI item) {
		return item.getContainedItem().equals(getContainedItem());
	}

	// PNML

	/**
	 * return the PNML xml tree for this object.
	 */
	public String toPNML() {
		return item.toPNML();
	}

	/**
	 * creates an object from the xml nodes.(symetric work of toPNML)
	 */
	public void fromPNML(OMElement subRoot, IdRefLinker idr)
			throws InnerBuildException, InvalidIDException,
			VoidRepositoryException {
		item.fromPNML(subRoot, idr);
	}

	public boolean validateOCL(DiagnosticChain diagnostics) {
		return item.validateOCL(diagnostics);
	}

}