/**
 * (C) University Pierre & Marie Curie - UMR CNRS 7606 (LIP6/MoVe)
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *  
 *  Contributors: 
 *    Lom HILLAH (LIP6) - Initial models and implementation
 *    Rachid Alahyane (UPMC) - Infrastructure and continuous integration
 *    Bastien Bouzerau (UPMC) - Architecture
 *    Guillaume Giffo (UPMC)  - Code generation refactoring, high-level API
 *
 * $Id$
 */
package fr.lip6.move.pnml.symmetricnet.integers;

import org.apache.axiom.om.OMElement;
import org.eclipse.emf.common.util.DiagnosticChain;

import fr.lip6.move.pnml.framework.utils.IdRefLinker;
import fr.lip6.move.pnml.framework.utils.exception.InnerBuildException;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.symmetricnet.terms.BuiltInConstant;

/**
 * <!-- begin-user-doc --> A representation of the model object '
 * <em><b>Number Constant</b></em>'. <!-- end-user-doc -->
 * 
 * <p>
 * The following features are supported:
 * <ul>
 * <li>{@link fr.lip6.move.pnml.symmetricnet.integers.NumberConstant#getType
 * <em>Type</em>}</li>
 * <li>{@link fr.lip6.move.pnml.symmetricnet.integers.NumberConstant#getValue
 * <em>Value</em>}</li>
 * </ul>
 * </p>
 * 
 * @see fr.lip6.move.pnml.symmetricnet.integers.IntegersPackage#getNumberConstant()
 * @model annotation="http://www.pnml.org/models/OCL
 *        typeType='self.input->size() = 0 and self.type.oclIsTypeOf(Natural)
 *        implies self.value >= 0 and self.type.oclIsTypeOf(Positive) implies
 *        self.value > 0'" annotation="http://www.eclipse.org/emf/2002/Ecore
 *        constraints='typeType'" annotation="http://www.pnml.org/models/ToPNML
 *        tag='numberconstant' kind='son'"
 * @generated
 */
public interface NumberConstant extends BuiltInConstant {
	/**
	 * Returns the value of the '<em><b>Type</b></em>' containment reference. It
	 * is bidirectional and its opposite is '
	 * {@link fr.lip6.move.pnml.symmetricnet.integers.HLPNNumber#getContainerNumberConstant
	 * <em>Container Number Constant</em>}'. <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Type</em>' containment reference isn't clear,
	 * there really should be more of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * 
	 * @return the value of the '<em>Type</em>' containment reference.
	 * @see #setType(HLPNNumber)
	 * @see fr.lip6.move.pnml.symmetricnet.integers.IntegersPackage#getNumberConstant_Type()
	 * @see fr.lip6.move.pnml.symmetricnet.integers.HLPNNumber#getContainerNumberConstant
	 * @model opposite="containerNumberConstant" containment="true"
	 *        required="true" annotation="http://www.pnml.org/models/ToPNML
	 *        kind='follow'"
	 * @generated
	 */
	HLPNNumber getType();

	/**
	 * Sets the value of the '
	 * {@link fr.lip6.move.pnml.symmetricnet.integers.NumberConstant#getType
	 * <em>Type</em>}' containment reference. <!-- begin-user-doc --> <!--
	 * end-user-doc -->
	 * 
	 * @param value
	 *            the new value of the '<em>Type</em>' containment reference.
	 * @see #getType()
	 * @generated
	 */
	void setType(HLPNNumber value);

	/**
	 * Returns the value of the '<em><b>Value</b></em>' attribute. <!--
	 * begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Value</em>' attribute isn't clear, there
	 * really should be more of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * 
	 * @return the value of the '<em>Value</em>' attribute.
	 * @see #setValue(Integer)
	 * @see fr.lip6.move.pnml.symmetricnet.integers.IntegersPackage#getNumberConstant_Value()
	 * @model required="true" annotation="http://www.pnml.org/models/ToPNML
	 *        tag='value' kind='attribute'"
	 * @generated
	 */
	Integer getValue();

	/**
	 * Sets the value of the '
	 * {@link fr.lip6.move.pnml.symmetricnet.integers.NumberConstant#getValue
	 * <em>Value</em>}' attribute. <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @param value
	 *            the new value of the '<em>Value</em>' attribute.
	 * @see #getValue()
	 * @generated
	 */
	void setValue(Integer value);

	/**
	 * Return the string containing the pnml output
	 */
	public String toPNML();

	/**
	 * set values to conform PNML document
	 */
	public void fromPNML(OMElement subRoot, IdRefLinker idr)
			throws InnerBuildException, InvalidIDException,
			VoidRepositoryException;

	public boolean validateOCL(DiagnosticChain diagnostics);
} // NumberConstant
